#! @PERL@
# -*- perl -*-
# @configure_input@

our $MODVERSION = '$Id: calScript.pl,v 1.15 2008/11/12 23:28:59 sgroom Exp $';

##########################################################################
# NASA Exoplanet Science Institute (NExScI) -- http://nexsci.caltech.edu #
#       Copyright (c) 1997-2008 California Institute of Technology       #
#                     U.S. Government Sponsorship                        #
# Under contract with the National Aeronautics and Space Administration  #
##########################################################################

use strict;
use warnings;
use FindBin qw($RealBin);
use lib "$RealBin/../lib/modules";		# location of getCal Perl modules
use gcConfig;                       # sigFigures()
use internalLine;
use colors;
use coordinates;
use miscString;
use miscArgs;                       # handlecommonOpts

handleCommonOpts(0, "", "");        # set up for --debug, --version, etc.

my $cfg = gcConfig::config();       # sigFigures()

# calScript -- produce a calibration script for visibility calibration

## MODIFIED T. Evans   Jun 2005  -- for release 2.8
##   updated to iron out units wrinkles for proper motions and parallaxes:
##     PLX, pmRA and pmDec now will always be in units of arcsec internally

# Take a line that may be blank and return an output string for it
sub outputLine {
    my $line = trimWhite(shift @_);
    my $isTarget = shift @_;
    if (!$line) {
	return "";
    }
    my $outbuf = "";
    
    my $catLine = new internalLine(line => $line);
    ## Determine the line designation
    ## Prefer the designation field, but use fallbacks as needed
    my $name = $catLine->getIdent("designation","catIndex","catIndex2");
    if (!$name)
    {
	print STDERR "Error: no designation available for source\n";
	return "";
    }
    if ($isTarget)
    {
	$outbuf .= sprintf "## $name\n";
    }

    # Compute model K magnitude
    my $spType = trimWhite($catLine->extractField("tagSPECTYP"));
    my $kmag = trimWhite($catLine->extractField("Kmag"));
    my $vmag = trimWhite($catLine->extractField("Vmag"));
    if ($kmag eq "")  ## if it's blank, attempt to fix...
    {
	if ($vmag eq "") {
	    print STDERR "Warning: Vmag and Kmag not defined for $name\n";
            ## TLE - 8/2005  don't set blanks to 0
	    ##$vmag = 0.0;
	    ##$kmag = 0.0; # not meaningful to do the calculation if vmag isn't defined
	}
	else {
	    my $vmk = VminusK($spType);
	    if (defined $vmk)
	    {
		$kmag = $vmag - $vmk;
	    }
	}
    }

    # Compose the out lines -- first the designation (note the CR for the target)
    $outbuf .= $name . ($isTarget ? "\n" : " ");

    # Next the astrometry (J2000!) 
    $outbuf .= sprintf "%s ", RAhms($catLine->extractField("raDeg"),$cfg->sigFigures());
    $outbuf .= sprintf "%s ", DECdms($catLine->extractField("decDeg"),$cfg->sigFigures());

    ## TLE - 6/2005   now internal pm's and PLX all in arcsec
    ##$outbuf .= sprintf "%6.3f ", floatString($catLine->extractField("pmRA")) / 1000.0;
    ##$outbuf .= sprintf "%6.3f ", floatString($catLine->extractField("pmDec")) / 1000.0;
    ##$outbuf .= sprintf "%4.3f ", floatString($catLine->extractField("tagPLX")) / 1000.0;
    $outbuf .= sprintf "%6.3f ", floatString($catLine->extractField("pmRA"));
    $outbuf .= sprintf "%6.3f ", floatString($catLine->extractField("pmDec"));
    $outbuf .= sprintf "%4.3f ", floatString($catLine->extractField("tagPLX"));

    if (!$isTarget) {
	# Add angular diameter estimate and error if defined
	if ((defined $catLine->extractField("tagDIAM")
	     && (defined $catLine->extractField("tagDIAMERR"))))
	{
	    my $diam = floatString($catLine->extractField("tagDIAM"));
	    my $diamErr = floatString($catLine->extractField("tagDIAMERR"));
	    $outbuf .= sprintf "%4.2f %3.1f ", $diam, $diamErr;
	}
	else ## otherwise mark as unknown
	{
	    $outbuf .= "AD?  ADerr?   ";
	}
    }

    ## The rest are comments
    $outbuf .= sprintf "## %-7s ", $spType;
    ## TLE - 8/2005   print ?flag for blanks
    if ($vmag) {
      $outbuf .= sprintf "V=%3.1f ", $vmag;
    }
    else {
      $outbuf .= sprintf "V=Vmag? ", $vmag;
    }
    if ($kmag) {
      $outbuf .= sprintf "K=%3.1f ", $kmag;
    }
    else {
      $outbuf .= sprintf "K=Kmag? ", $kmag;
    }
    $outbuf .= "\n";
    return $outbuf;
}


use Getopt::Long;
Getopt::Long::config("pass_through");
my ($instream, @filenames, $useDesig, $oldCS, $noTarget);
my $result = GetOptions("stdin" => \$instream,
		     "filename=s" => \@filenames,
		     "useDesignation" => \$useDesig,
             "sigFigures=i" => sub { $cfg->sigFigures($_[1]); },
		     "oldCS" => \$oldCS,
		     "noTarget" => \$noTarget);

my @inputStream;
if (($instream)
    || ((! (scalar @filenames)) && (!(scalar @ARGV)))) ## read from stdin
{
    @inputStream = <STDIN>;
}
elsif (scalar @filenames)  ## otherwise read from designated filenames
{
    foreach my $file (@filenames)
    {
	open (READ,$file) or die "Unable to open $file\n";
	push @inputStream , <READ>;
	close (READ);
    } ## end foreach
}
elsif (scalar @ARGV)
{
    foreach my $file (@ARGV)  ## Read all the remaing filenames
    {
	open (READ,$file) or die "Unable to open $file\n";
	push @inputStream, <READ>;
	close (READ);
    } ## end foreach
}

# We assume the first line is the target unless indicated by the -noTarget switch
##  TLE - may want to remove this assumption sometime?? (check ROLE tag?)
if (!$noTarget)
{
    my $line = shift @inputStream;
    print outputLine($line, 1);
}
print "---\n";

my $calTxt = "";
my $calCount = 0;
foreach my $line (@inputStream)
{
    print STDERR "DEBUG: calScript:\$line=\"$line\"\n" if $cfg->debug();

    my $calLine = outputLine($line, 0);
    if ($calLine) {
	$calTxt .= $calLine;
	++$calCount;
    }
}
print "$calCount\n";
print $calTxt;
