#! @PERL@
# -*- perl -*-
# @configure_input@

our $MODVERSION = '$Id: convertPhotometry.pl,v 1.10 2008/11/12 23:26:50 sgroom Exp $';

##########################################################################
# NASA Exoplanet Science Institute (NExScI) -- http://nexsci.caltech.edu #
#       Copyright (c) 1997-2008 California Institute of Technology       #
#                     U.S. Government Sponsorship                        #
# Under contract with the National Aeronautics and Space Administration  #
##########################################################################

our $MODVERSION = '$Id: convertPhotometry.pl,v 1.10 2008/11/12 23:26:50 sgroom Exp $';

## convertPhotometry -- convert photometry into alternative units

## convertPhotometry is a filter that converts disparate
## photometry/flux units into consistent flux units

use strict;  ## invoke strict syntax checking
use warnings;
use FindBin qw($RealBin);
use lib "$RealBin/../lib/modules";		# location of getCal Perl modules
use photometry;  ## getCal photometry module
use miscString;
use miscArgs;

## setup stdout for line buffering (needed for interaction with fbol)
use IO::Handle;
autoflush STDOUT;

my $FLambda = 0;
my $FNu = 0;

my $readStdin;
my @filenames;


## Command-line options

use Getopt::Long;
handleCommonOpts(1);
my $result = GetOptions("stdin" => \$readStdin,
			"filename=s" => \@filenames,
			"flambda" => \$FLambda,
			"fnu" => \$FNu);

## Set the output units
if ($FNu && !($FLambda)) ## fNu units (Jy)
{
    $FNu = 1;
    $FLambda = 0;
}
else  ## Otherwise default to fLambda units (erg / s / cm^2 / um)
{
    $FNu = 0;
    $FLambda = 1;
}

## Slurp the input stream
my @inputStream;
if (!$readStdin)
{
    if (scalar @filenames)  ## otherwise read from designated filenames
    {
	foreach my $file (@filenames)
	{
	    open (READ,$file) or die "Unable to open $file\n";
	    push @inputStream , <READ>;
	    close (READ);
	} ## end foreach
    }
    elsif (scalar @ARGV)
    {
	foreach my $file (@ARGV)  ## Read all the remaing filenames
	{
	    open (READ,$file) or die "Unable to open $file\n";
	    push @inputStream, <READ>;
	    close (READ);
	} ## end foreach
    } ## end pulling files from @ARGV
} ## end !$readStdin



## Line-by-line processing
if ($readStdin)
{
    while (my $line = <STDIN>)
    {
	&processInputLine($line);
    } ## end while

} ## end $readStdin
else
{
    foreach my $line (@inputStream)
    {
	&processInputLine($line);
    } ## end foreach $line
} ## end !$stdin


## Via con Dios -- return all-clear status
exit(0);


#########################################################################
## processInputLine -- processing individual lines
#########################################################################
sub processInputLine
{
## declarations processInputLine  #######################################
    my $line = shift @_;

## begin processInputLine  ##############################################

    ## Look for line format matches

    ## New-style photometry line
    ## M HD_19373--G0V Johnson V 3.25 0.01  ## Example Johnson magnitude
    if (($line =~ m/^M\s+\S+\s+\w+\s+\D+\d*\s+\d+/)
	|| ($line =~ m/^M\s+\S+\s+\w+\s+\D+\d?\s+\-\d+/)
	|| ($line =~ m/^M\s+\S+\s+\w+\s+\D+\d?\s+\.\d+/)
        || ($line =~ m/^M\s+\S+\s+\w+\s+\D+\d?\s+\-\.\d+/))
    { ## Magnitude line spec
	my ($dummy,$name,$system,$band,$mag,$magErr,$rest) = split /\s+/, $line, 7;

	## print "$system $band $mag $magErr\n";
	my ($fLam,$efLam) = photometry::fLambda($system,$band,$mag,$magErr);
	if (defined $fLam)
	{
	    my ($lambda,$deltaLambda,$z1,$z2) = photometry::bandParameters($system,$band);

	    print outputLine($name,$lambda,$deltaLambda,$fLam,$efLam);
	    print " ## $line";


	} ## end defined $fLam
	else
	{
	    print $line;  ## Pass the line unfiltered
	}
	    
    }  ## end New-style magnitude line

    ## Old-style photometry line
    ## M HD195987--G9V 550 89 7.04 0.05  # UBV Johnson V 1965PROE....5..111L
    elsif ($line =~ m/^M\s+\S+\s+\d+\s+\d+\s+\d+/)
    {
	my ($dummy,$name,$lambda,$deltaLambda,$mag,$magErr,$rest) = split /\s+/, $line, 7;

	my ($system,$band) = photometry::matchBand($lambda,$deltaLambda);

	if (defined $system) ## If we got a good band match
	{
	    ## Ready to go...
	    my ($fLam,$efLam) = photometry::fLambda($system,$band,$mag,$magErr);
	    if (defined $fLam)
	    {
		my ($lambda,$deltaLambda,$z1,$z2) = photometry::bandParameters($system,$band);

		print outputLine($name,$lambda,$deltaLambda,$fLam,$efLam);
		print " ## $line";

	    } ## end defined $fLam
	    else
	    {
		print $line;  ## Pass the line unfiltered
	    }

	} ## end good system match
	else ## Otherwise
	{
	    print $line; ## echo the line

	} 

    } ## end old-style magnitude line conversion


    ## Fnu lines
    ## F HD195987--G9V 411 20 2.23707 0.223707  # Stromgren uvby  v=8.346     
    ## Fn HD195987--G9V 411 20 2.23707 0.223707  # Stromgren uvby  v=8.346     
    ## Fn HD_281934--K5V:e... 12000 2000 .50 0.05  # IRAS 12um flux  1988NASAR1190....1B
    elsif (($line =~ m/^F\s+\S+\s+\d+\s+\d+\s+\d+/)
	   || ($line =~ m/^Fn\s+\S+\s+\d+\s+\d+\s+\d+/)
	   || ($line =~ m/^Fn\s+\S+\s+\d+\s+\.?\d+\s+\.?\d+/))
    {

	my ($dummy,$name,$lambda,$deltaLambda,$fNu,$fNuErr,$rest) = split /\s+/, $line, 7;

	if ($FNu)
	{
	    ## Just pass the line
	    print $line;
	} ## end FNu
	else
	{
	    ## This is Fnu in Jy -- convert
	    my ($fLam,$efLam) = photometry::fNuToFLambda($lambda,$fNu,$fNuErr);
	    ## Output
	    print outputLine($name,$lambda,$deltaLambda,$fLam,$efLam);
	    print " ## $line";
	} ## end FLambda

    } ## end Fnu line conversion


    ## FLambda line
    elsif ($line =~ m/^Fl\s+\S+\s+\d+\s+\d+\s+\d+/)
    {
	my ($dummy,$name,$lambda,$deltaLambda,$fLam,$efLam,$rest) = split /\s+/, $line, 7;

	if ($FNu)
	{
	    ## convert to FNu and output
	    print outputLine($name,$lambda,$deltaLambda,$fLam,$efLam);
	    print "## $line";
	}
	else ## Otherwise FLambda -- just pass line
	{
	    ## Just pass the line
	    print $line;
	} 

    } ## end Flambda line conversion


    else  ## Otherwise we didn't recognize the line format
    {
	print $line; ## Pass the line unfiltered
    }

} ## end processInputLine  ##############################################




#########################################################################
## outputLine -- output desired line
#########################################################################
sub outputLine
{
## declarations outputLine  #############################################
    my $name = shift @_;
    my $lambda = shift @_;
    my $deltaLambda = shift @_;
    my $fLambda = shift @_;
    my $efLambda = shift @_;
    my $l;

## begin outputLine  ####################################################

    if ($FNu)  ## If user wants fNu
    {
	## convert to fNu and output
	my ($fn,$efn) = photometry::fLambdaToFNu($lambda,$fLambda,$efLambda);
	$l = "Fn $name " . formatValues("?", " ", "%.0f", $lambda, "%.0f", $deltaLambda, "%.3g", $fn, "%.2g", $efn);
    } ## end output $fNu
    else ## Otherwise output fLambda
    {
	$l = "Fl $name " . formatValues("?", " ", "%.0f", $lambda, "%.0f", $deltaLambda, "%.3g", $fLambda, "%.2g", $efLambda);
    } ## end output $fLambda
    return $l;
} ## end outputLine  ####################################################
