// -*- c++ -*-
/*************************************************************************
* NASA Exoplanet Science Institute (NExScI) -- http://nexsci.caltech.edu *
*	Copyright (c) 1997-2008 California Institute of Technology       *
*                     U.S. Government Sponsorship                        *
* Under contract with the National Aeronautics and Space Administration  *
**************************************************************************/

/*
   fbol -- bolometric flux/effective temperature/angular diameter
   estimation from broad-band photometry & radiometry -- AB, GvB, and
   BFL.

   Translated to C++ AB 3/02 */

/* 

   fbol notes added by AB 4/02
   
   fbol is a spectral energy distribution (SED) modeling program --
   primarily aimed at estimating the angular diameters of calibration
   sources in the context of optical/IR interferometry in general
   (where diameters knowledge is quite important), and the getCal
   suite of planning tools in particular.  In this incarnation the
   modeling is relatively basic -- the model is a (Lambertian) black
   body radiator -- parameterized simply by an angular size and an
   effective temperature.  Input to fbol is spectral flux and
   photometric data, and ancillary such as initial temperature,
   distance/parallax (for reddening correction) on one or more
   sources.  Output from fbol is a table of model parameters for the
   input sources: effective temperature, bolometric flux, and angular
   diameter.  Optionally fbol (supported by gnuplot) can make SED
   (F_lambda) plots of the input flux data, estimated black body SED
   model, and residuals between the two.

   For a black-body source the fundamental relationship between
   bolometric flux (fbol), angular diameter (theta), and effective
   temperature (T) is:

     fbol = 1/4 theta^2 SIGMA T^4

   where SIGMA is the Stefan-Boltzman constant.  With theta in
   dimensionless units (rad), this form of the relation is the
   prefered form over other pretenders to the throne because it is
   system neutral: it is equally at home in cgs, mks, and English
   units.  The corresponding form of f_lambda is given by

     f_lambda(lambda,theta,T) = pi/4 theta^2 B_lambda(lambda,T)
       = pi/4 theta^2 (2 h c^2 lambda^-5 / (exp(h c / lambda k T) - 1))

   with the standard definitions of h (Plank's constant), c (speed of
   light), and k (Boltzman's constant).  I've never actually seen this
   corresponding normalization for f_lambda published anywhere -- so
   I've included the derivation in doc/fbolSED.tex -- please look
   there.

   fbol works internally in cgs units (almost -- more in a second):
   all the standard physical constants are in the cgs system, the
   computed bolometric flux is in ergs / cm^2 / s, and the effective
   temperature is in K.  The spectral modeling in fbol is done in
   f_lambda space, and (mainly for historical reasons) the units we
   use for this are ergs / cm^2 / s / micron (!).  We could be fully
   cgs compliant, but there doesn't appear to be particularly good
   compliance in the usage of f_lambda in astronomy, and the micron
   has the nice attribute of being on the same order as the
   wavelengths we typically work in.  So you'll see a wavelength
   conversion in the fluxBB routine where we convert to these slightly
   hybrid units.  I hope this doesn't cause anyone any excessive
   grief...

 */

const char *fbolVersion = "$Id: fbol.cc,v 1.20 2008/11/12 23:26:50 sgroom Exp $";

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#if HAVE_STDLIB_H
#include <stdlib.h>
#endif

#if HAVE_STDIO_H
#include <stdio.h>
#endif

#if HAVE_STRING_H
#include <string.h>
#endif

#if HAVE_MATH_H
#include <math.h>
#endif

#if HAVE_UNISTD_H
#include <unistd.h>
#endif

#if HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif

#if HAVE_SYS_STAT_H
#include <sys/stat.h>
#endif

#include "pieces/StarList.h"
#include "pieces/mrqMin.h"
#include "pieces/getopt.h"
#include "pieces/spawnChild.h"


#define MAXLINE 1024
#define MAXPTS  500
#define MAXSTAR 500
#define MAXFILTERS 300
#define TRUE 1
#define FALSE 0

#define T0 1000
#define DELTA_T 25
#define NT 1560
#define DEFAULT_TEMP 5800.0

#define LINE_LENGTH 1024

#define PI 3.141592654    // Pi
#define CC 2.9979250e+10  // Speed of light in cm/s
#define HH 6.62620e-27    // Plank's constant in erg s
#define KK 1.38026e-16    // Boltzman's constant in erg/K
#define NM_TO_CM 1.0e-7   // wavelength conversion
#define NM_TO_UM 1.0e-3   // wavelength conversion
#define CM_TO_UM 1.0e4    // wavelength conversion
#define JY_TO_ERG 1.0e-24 // Energy conversion
#define SIGMA 5.6696e-5   // Stefan-Boltzman constant in units of erg / cm^2 / K^4 */

#define RAD_TO_MAS 2.062648e8

/* cgs units used */

/* structures here */
typedef struct
{
  char name[3];
  double wavelength;
  double bandpass;
  double zeropoint;
} FILTER;




/* Globals here */
/* Filter flux units given in nm, erg/cm2/s/um */

// New Zeropoints from AQ 4th edition table 15.6 (p387) U -- I
//                                    table  7.5 (p150) J -- Q
// Adjustments in these zeropoints relative to the original spec seem
//  to improve the fit quality by ~30%, so this is probably an improvement

FILTER filters[MAXFILTERS] = {  {"U",  360 * NM_TO_CM,  68 * NM_TO_CM,4.22e-5},
                                {"B",  450 * NM_TO_CM,  98 * NM_TO_CM,6.40e-5},
			        {"V",  555 * NM_TO_CM,  89 * NM_TO_CM,3.75e-5},
			        {"R",  668 * NM_TO_CM, 210 * NM_TO_CM,1.74e-5},
			        {"I",  879 * NM_TO_CM, 171 * NM_TO_CM,9.12e-6},
			        {"J", 1215 * NM_TO_CM, 260 * NM_TO_CM,3.31e-6},
			        {"H", 1654 * NM_TO_CM, 290 * NM_TO_CM,1.15e-6},
			        {"K", 2179 * NM_TO_CM, 410 * NM_TO_CM,4.14e-7},
			        {"L", 3500 * NM_TO_CM, 700 * NM_TO_CM,6.59e-8},
				{"M", 4800 * NM_TO_CM, 450 * NM_TO_CM,2.11e-8},
				{"N",10472 * NM_TO_CM,5190 * NM_TO_CM,9.63e-10},
				{"Q",20130 * NM_TO_CM,7800 * NM_TO_CM,7.18e-11}
                               };
int    nfilters=12;

/* global variables for command-line switches */
int doPlots = 0; /* Controls fit plots */
int doReddening = 1; /* Controls reddening correction */
int constrainTemp = 0; /* Constrain to input effective temperature estimate */
int verbose = 0; /* Printout lots of crap */
int hardCopy = 0; /* Do plots to postscript */
int screen = 0;
int eps = 0; 
int ps = 0;
int png = 0;
int readStdin = 0;
int fixedError = 0;
double errorFrac = 0.1;
int doBlackwellCorrections = 0;

int cvPhotFilter = 1; // Use external convertPhotometry filter



/* Local functions here */

void versionMsg()
{
/**********  begin versionMsg  ******************************************/
  (void) fprintf(stdout,"fbol version %s (%s)\n",fbolVersion, PACKAGE_STRING);

  exit(0);

/**********  end versionMsg  ********************************************/
}


/************************************************************************
*  helpMsg -- local to fbol.cc (A.B. 10/98)				*
*									*
*  A.  Purpose: helpMsg does what you think -- it prints a help         *
*        message and then exits.                     			*
*									*
*  B.  Background and use:						*
*									*
*  C.  Called by:							*
*									*
*  D.  Input/output files:						*
*									*
*  E.  Parameter description:						*
*	Input -								*
*	Output -							*
*									*
*  F.  Detail function description:					*
*	      1.  Local Functions:					*
*	      2.  Externals:						*
*	      3.  Globals and Nonlocals used:				*
*	      4.  Global and Nonlocal variables modified:		*
*	      5.  Function flow is as follows:				*
*	      6.  Functions called by					*
*									*
*  G.  Function value: This function returns no useable value.		*
*									*
*************************************************************************/
void helpMsg()
{
  /**********  begin helpMsg  *********************************************/
  versionMsg();
  (void) fprintf(stdout,"Usage: fbol [options] datafile1 [datafile2 etc.]\n\n");
  (void) fprintf(stdout,"Model source SED with Plank blackbody model fit to input photometric\n");
  (void) fprintf(stdout,"and/or flux measurements, estimating Teff, bolometric flux, and apparent\n");
  (void) fprintf(stdout,"(angular) diameter.\n");
  (void) fprintf(stdout,"Options:\n");
  (void) fprintf(stdout," -help: print (this) help message and exit (ovrrides other options)\n");
  (void) fprintf(stdout," -version: print version message and exit (ovrrides other options)\n");
  (void) fprintf(stdout," -stdin: read input from stdin\n");
  (void) fprintf(stdout," -reddening: apply reddening correction to input flux data (default)\n");
  (void) fprintf(stdout," -noReddening: no reddening correction to input flux data\n");
  (void) fprintf(stdout," -constrainTemp: constrain fit temperature to input flux estimate\n");
  (void) fprintf(stdout," -fixedError: apply uniform fractional error to all meas\n");
  (void) fprintf(stdout," -blackwellCorrections: apply mapping to correspond to Blackwell results\n");
  (void) fprintf(stdout," -plots: produce SED fit data/model/residual plot (gnuplot required)\n");
  (void) fprintf(stdout," -hardCopy (hc): PostScript SED plots (gnuplot required)\n");
  (void) fprintf(stdout," -ps: PostScript SED plots (gnuplot required)\n");
  (void) fprintf(stdout," -eps: encapsulated PostScript SED plots (gnuplot required)\n");
  (void) fprintf(stdout," -png: png SED plots (gnuplot required)\n");
  (void) fprintf(stdout," -screen: screen SED plots (gnuplot required)\n");
  (void) fprintf(stdout," -verbose: verbose debugging messages\n");
  (void) fprintf(stdout,(char *) " note:  command-line options may be used in any order, interspersed with non-options\n");
  (void) fprintf(stdout,(char *) "  and abbreviated to uniqueness (e.g. -he, -noR, -cons ).\n\n");
  (void) fprintf(stdout,"The format of the input data is: control_character information \n");
  (void) fprintf(stdout,"Available control characters (and formats) are:\n");
  (void) fprintf(stdout," M StarID Lambda Bandpass Magnitude Error\n");
  (void) fprintf(stdout," F StarID Lambda Bandpass Flux(Jy) Error\n");
  (void) fprintf(stdout," Fn StarID Lambda Bandpass Flux(Jy) Error\n");
  (void) fprintf(stdout," Fl StarID Lambda Bandpass Flux(erg s^-1 cm^-1 um^-1) Error\n");
  (void) fprintf(stdout," D StarID Distance DistanceErr (pc)\n");
  (void) fprintf(stdout," P StarID Parallax ParallaxErr (mas)\n");
  (void) fprintf(stdout," T StarID Temperature (K)\n");
  (void) fprintf(stdout," S StarID Filter(up to 3 characters) Magnitude Error (Not yet implemented).\n"); 
  (void) fprintf(stdout," Z Filter(up to 3 characters) Lambda Bandpass Zeropoint (for specifying a new filter)\n");
  // (void) fprintf(stdout,"N Datafilename BlackbodyFilename (for specifying output filenames)\n");
  (void) fprintf(stdout,"\n"); 
  (void) fprintf(stdout,"The magnitude filter designations that fbol recognizes are\n");
  (void) fprintf(stdout," (desig center FWMH (nm), zeropt (erg/cm2/s/um)):\n");
  (void) fprintf(stdout,"   U   365   68   4.27e-5 (Johnson U)\n");
  (void) fprintf(stdout,"   B   440   98   6.61e-5 (Johnson B)\n");
  (void) fprintf(stdout,"   V   550   89   3.64e-5 (Johnson V)\n");
  (void) fprintf(stdout,"   R   700  220   1.74e-5 (Johnson R)\n");
  (void) fprintf(stdout,"   I   900  240   8.32e-6 (Johnson I)\n");
  (void) fprintf(stdout,"   J  1250  300   3.18e-6 (Johnson J)\n");
  (void) fprintf(stdout,"   H  1600  300   1.18e-6 (Johnson H)\n");
  (void) fprintf(stdout,"   K  2200  400   4.17e-7 (Johnson K)\n");
  (void) fprintf(stdout,"   L  3600 1200   6.23e-8 (Johnson L)\n");
  (void) fprintf(stdout,"   M  4800  800   2.07e-8 (Johnson M)\n");
  (void) fprintf(stdout,"   N 10200 6000   1.23e-9 (Johnson N)\n");
  (void) fprintf(stdout,"   Q 20000 6000   3.27e-10 (Johnson Q)\n\n");
  (void) fprintf(stdout,"Lines that do not begin with a control character are ignored.\n");
  (void) fprintf(stdout,"Note: If the program cannot find a filter for a given magnitude line, it will\n");
  (void) fprintf(stdout,"create a new filter by linearly interpolating a zeropoint and using\n");
  (void) fprintf(stdout,"the wavelength and bandpass specified.\n\n"); 
  (void) fprintf(stdout,(char *) "Find more fbol info at http://nexsci.caltech.edu/software\n");

  (void) exit(0);

/**********  end helpMsg  ***********************************************/
}


/************************************************************************
*  parseCmdLine -- local to fbol.cc (A.B. 10/98)			*
*									*
*  A.  Purpose:	parseCmdLine parses the command line for bb and sets    *
*       option flags and values.  It uses GNU getopt (POSIX-standard    *
*       with GNU extensions) to process the command line.		*
*									*
*  B.  Background and use:						*
*									*
*  C.  Called by:							*
*									*
*  D.  Input/output files:						*
*									*
*  E.  Parameter description:						*
*	Input -								*
*	Output -							*
*									*
*  F.  Detail function description:					*
*	      1.  Local Functions:					*
*	      2.  Externals:						*
*	      3.  Globals and Nonlocals used:				*
*	      4.  Global and Nonlocal variables modified:		*
*	      5.  Function flow is as follows:				*
*	      6.  Functions called by					*
*									*
*  G.  Function value: This function returns no useable value.		*
*									*
*************************************************************************/
int parseCmdLine(int argc, char **argv)
{
/**********  local declarations parseCmdLine  ***************************/
  // Valid command-line options
  int retVal,optIndex,argsProcessed = 0;
  static struct option longOptions[] = 
  {
    {(char *) "help",no_argument,0,0},                 // Option 0
    {(char *) "version",no_argument,0,0},              // Option 1
    {(char *) "plots",no_argument,0,0},                // Option 2
    {(char *) "reddening",no_argument,0,0},            // Option 3
    {(char *) "noReddening",no_argument,0,0},          // Option 4
    {(char *) "constrainTemp",no_argument,0,0},        // Option 5
    {(char *) "verbose",no_argument,0,0},              // Option 6
    {(char *) "hardCopy",no_argument,0,0},             // Option 7
    {(char *) "hc",no_argument,0,0},                   // Option 8
    {(char *) "ps",no_argument,0,0},                   // Option 9
    {(char *) "eps",no_argument,0,0},                  // Option 10
    {(char *) "png",no_argument,0,0},                  // Option 11
    {(char *) "stdin",no_argument,0,0},                // Option 12
    {(char *) "fixedError",optional_argument,0,0},     // Option 13
    {(char *) "blackwellCorrections",no_argument,0,0}, // Option 14

    {0,0,0,0}
  };

  extern char *optarg;
  extern int optind,opterr,optopt;

/**********  begin parseCmdLine  ****************************************/
  opterr = 0; // Set getopt to supress error messages
  do
    {
      if ((retVal = getopt_long_only(argc,argv,"",longOptions,&optIndex)) != EOF)
	{
	  (void) printf("# option %s\n",longOptions[optIndex].name);
	  argsProcessed++;
	  switch (optIndex)
	    {
	    case 0 :
	      {
		(void) helpMsg();
		break;
	      } // end doing helpMessage -- option 0

	    case 1 :
	      {
		(void) versionMsg();
		break;
	      } // end doing versionMsg -- option 1

	    case 2 :
	      {
		doPlots = 1;
		screen = 1; // plots to screen
		break;
	      } // end setting doPlots -- option 2

	    case 3 :
	      {
		doReddening = 1;
		break;
	      } // end setting doReddening on -- option 3

	    case 4 :
	      {
		doReddening = 0;
		break;
	      } // end setting doReddening off -- option 4

	    case 5 :
	      {
		constrainTemp = 1;
		break;
	      } // end setting constrainTemp on -- option 5

	    case 6 :
	      {
		verbose = 1;
		break;
	      } // end setting verbose on -- option 6

	    case 7 :
	      {
		doPlots = 1;
		hardCopy = 1;
		screen = 0;
		ps = 1; // default to PostScript
		break;
	      } // end setting plot hardcopy on -- option 7

	    case 8 :
	      {
		doPlots = 1;
		hardCopy = 1;
		screen = 0;
		ps = 1; // default to PostScript
		break;
	      } // end setting plot hardcopy on -- option 8

	    case 9 :
	      {
		doPlots = 1;
		hardCopy = 1;
		screen = 0;
		ps = 1; // set to PostScript
		break;
	      } // end setting plot PostScipt on -- option 9

	    case 10 :
	      {
		doPlots = 1;
		hardCopy = 1;
		screen = 0;
		eps = 1;
		break;
	      } // end setting plot eps hardcopy on -- option 10

	    case 11 :
	      {
		doPlots = 1;
		hardCopy = 1;
		screen = 0;
		png = 1;
		break;
	      } // end setting plot png hardcopy on -- option 11

	    case 12 :
	      {
		readStdin = 1;
		break;
	      } // end setting readint from stdin -- option 12

	    case 13 :
	      {
		fixedError = 1;
		if (optarg)
		  {
		    errorFrac = atof(optarg);
		  } // end setting errorFrac
		break;
	      } // end setting fixedError -- option 13

	    case 14 :
	      {
		doBlackwellCorrections = 1;
		break;
	      } // end setting doBlackwellCorrections -- option 14


	    } // end switch

	} // end if
    } // end do/while loop
  while (retVal != EOF);

  if (argsProcessed)
    (void) printf("# %d command line arguments processed\n",argsProcessed);

  return optind;

/**********  end parseCmdLine  ******************************************/
}




double inter(double x1, double y1, double x2, double y2, double x)
{

  double deltax = x2-x1;
  double deltay = y2-y1;
  double a = deltay / deltax;
  return y1 + a*(x-x1); 
}


int which_filter(double wavelength, double bandpass, int line)
{
  int j, closest=-1, sec_closest=-1, cf;
  double diff = 1e34;
  double interp =0.0;
  double d1 = 1e31;

  #ifdef FILTERDEBUG
   fprintf(stderr,"which_filter(%8le , %8le ) called.\n", wavelength, bandpass);
  #endif
  for (j=0;j<nfilters;j++)
    {
#ifdef FILTERDEBUG
      fprintf(stderr,"Compared with filter[%i] = L0:%8le dL%8le\n",
	      j, filters[j].wavelength, filters[j].bandpass);
#endif
      if (filters[j].wavelength==wavelength )
	{
	  /* if within 5% */
	  /*  if( fabs(filters[j].wavelength-wavelength) < 0.05*wavelength ){   */ 
#ifdef FILTERDEBUG
	  fprintf(stderr,"Filter[%i] found with L0:%8le dL%8le\n",j, wavelength, bandpass);
#endif
	  return j;
	}
    }
  
  if (verbose)
    printf ("# Line: %i Exact matching filter not found for Wavelength:%7.4g and Bandwidth: %7.4g\n",
	    line, wavelength, bandpass);
  diff = 1e35;
  
  for (j=0;j<nfilters;j++)
    {
      d1 = filters[j].wavelength-wavelength; 
#ifdef INTERDEBUG
      fprintf(stderr,"Comparing with filter[%i] : %s %10.3lg  -> d1 = %10.3lg diff=%11.3lg\n",
	      j, filters[j].name, filters[j].wavelength, d1, diff);
#endif
    if (fabs(d1) < fabs(diff) )
      {
	diff = d1;
	closest = j;
#ifdef INTERDEBUG
	fprintf(stderr,"Setting closest to %i with diff=%11.3lg\n", closest, diff);
#endif
      }
    }
  diff = 1e35;
  
  if (filters[closest].wavelength > wavelength)
    {
      for(j=0;j<nfilters;j++)
	{
	  d1 = filters[j].wavelength-wavelength;    
	  if((fabs(d1)<fabs(diff))&&(j!=closest)&&(d1<0))
	    {
	      diff = d1;
	      sec_closest = j;
	    }
	}
    }
  else
    {
      for(j=0;j<nfilters;j++)
	{
	  d1 = filters[j].wavelength-wavelength;    
	  if ((fabs(d1)<fabs(diff))&&(j!=closest)&&(d1>=0))
	    {
	      diff = d1;
	      sec_closest = j;
	    }
	}
    }
  
  if (closest<0)
    {
      fprintf(stderr,"Error in filter interpolation routine. Exiting.\n");
      exit(0);
    }
  
  if (sec_closest < 0)
    {
      if (filters[closest].wavelength > wavelength)
	{
	  for(j=0;j<nfilters;j++)
	    {
	      d1 = filters[j].wavelength-wavelength;    
	      if((fabs(d1)<fabs(diff))&&(j!=closest))
		{
		  diff = d1;
		  sec_closest = j;
		}
	    }
	}
      else
	{
	  for (j=0;j<nfilters;j++)
	    {
	      d1 = filters[j].wavelength-wavelength;    
	      if((fabs(d1)<fabs(diff))&&(j!=closest))
		{
		  diff = d1;
		  sec_closest = j;
		}
	    }
      
      
	  fprintf(stderr,"# Closest filter: %i Sec_closest:%i\n", closest, sec_closest);
	  /* exit(0); */
    
	}
    }

  if (filters[closest].wavelength > filters[sec_closest].wavelength)
    {
      interp = inter(filters[sec_closest].wavelength, filters[sec_closest].zeropoint,
		     filters[closest].wavelength, filters[closest].zeropoint,
		     wavelength);
    }
  else
    {
      interp = inter(filters[closest].wavelength, filters[closest].zeropoint,
		     filters[sec_closest].wavelength, filters[sec_closest].zeropoint,
		     wavelength);
    }

  if (verbose)
    printf("# Interpolating zeropoint = %7.2g (erg/cm2/s/um)\n", interp);
  filters[nfilters].wavelength = wavelength;
  filters[nfilters].bandpass = bandpass;
  filters[nfilters].zeropoint = interp;
  strcpy(filters[nfilters].name,filters[closest].name) ;
  strcat(filters[nfilters].name,"2");

  #ifdef INTERDEBUG
  fprintf(stderr," Line: %i\nExact matching filter not found for:\n           Wavelength:%10.4g. and Bandwidth: %10.4g\n",
	 line, wavelength, bandpass);
  fprintf(stderr,"Closest Filter       :"); 
  fprintf(stderr,"# %2i : %-3s %8.3g  %8.3g  %8.3g\n",
	  closest, filters[closest].name,  filters[closest].wavelength,
	  filters[closest].bandpass, filters[closest].zeropoint);

  fprintf(stderr,"Second Closest Filter:"); 
  fprintf(stderr,"# %2i : %-3s %8.3g  %8.3g  %8.3g\n",
	 sec_closest,filters[sec_closest].name,  filters[sec_closest].wavelength, 
	 filters[sec_closest].bandpass, filters[sec_closest].zeropoint);

  fprintf(stderr,"New Filter           :");
  fprintf(stderr,"# %2i : %-3s %8.3g  %8.3g  %8.3g\n|n",
	 nfilters,filters[nfilters].name,  filters[nfilters].wavelength,
	 filters[nfilters].bandpass, filters[nfilters].zeropoint);

  #endif
  cf = nfilters;
  nfilters++;
  return cf;

  /* No filter found */
  return -1;
} /* end which_filter */


double flux_from_mag(double mag, int filter)
{
  /* This function presumably computes a flux quantity from a
     magnitude quantity.  It's probably only good for Johnson bands */

  // Return units are erg/s/cm^2/um 

  double flux = pow(10,-0.4*mag + log10(filters[filter].zeropoint));

#ifdef FLUXDEBUG 
  fprintf(stderr,"Filter[%i]: Wavelength:%6.2g  Zeropoint:%6.3g Mag:%6.3lf -> Flux:%7.3g\n",
	  filter, filters[filter].wavelength, filters[filter].zeropoint, mag, flux); 
#endif

  return flux;

} /* end flux_from_mag */






/************************************************************************
*  readDataFile -- local to fbol.c (moved A.B. 10/98)			*
*									*
*  A.  Purpose: readDataFile reads and parses an input file pointer     *
*       for the bb program.						*
*									*
*  B.  Background and use:						*
*									*
*  C.  Called by:							*
*									*
*  D.  Input/output files:						*
*									*
*  E.  Parameter description:						*
*	Input -								*
*	Output -							*
*									*
*  F.  Detail function description:					*
*	      1.  Local Functions:					*
*	      2.  Externals:						*
*	      3.  Globals and Nonlocals used:				*
*	      4.  Global and Nonlocal variables modified:		*
*	      5.  Function flow is as follows:				*
*	      6.  Functions called by					*
*									*
*  G.  Function value: This function returns no useable value.		*
*									*
*************************************************************************/
int readDataFile(char *filename,StarList *starList)
{
/**********  local declarations readDataFile  ***************************/
  char line[MAXLINE],starID[MAXLINE],junk[MAXLINE];
  char work[MAXLINE],filterCommand[MAXLINE];

  char *tmpFile = "cvOut.tmp";

  int i,j,newfilters,found,whichstar,lCount = 0;
  double lambda,bandwidth,flux,fluxerr;

  FILE *fp,*cvPhotIn,*cvPhotOut;


/**********  begin readDataFile  ****************************************/
  // Open the input filename
  // Check to see if we are reading from stdin
  if (strcmp(filename,"stdin") != 0)
    {
      if ((fp = fopen(filename,"r")) == NULL)
	{
	  (void) fprintf(stderr,"# Could not open file %s\n",filename);
	  return 0;
	}
    } // end not reading stdin
  else // Otherwise
    fp = stdin;  // Reading from stdin


  if (cvPhotFilter)  // If we're filtering the input
    {
      // Run input lines through the photometry line converter filter
      // convertPhotometry to facilitate handling many different formats
      char *cvPhot = "convertPhotometry";
      // Setup the command-line arguments to the filter
      char *carg[4];
      carg[0] = cvPhot;
      carg[1] = "-FLambda";
      carg[2] = "-stdin";
      carg[3] = NULL;

      int fd[2]; // for catching the filter pipe file descriptors

      // Spawn the filter process
      if (spawnChild(cvPhot,carg,fd) == 0)
	{
	  fprintf(stderr,"Error spawning child filter process\n");
	  exit(-1);
	}
      
      // Setup the input and output pipe file pointers
      cvPhotIn = fdopen(fd[1],"w");  // child stdin
      cvPhotOut = fdopen(fd[0],"r"); // child stdout
    } // end cvPhotFilter


  /* read in data */
  i = newfilters = 0;
  while (fgets(line,MAXLINE,fp) != NULL)
    {
      found = FALSE;
      i++;

      if (cvPhotFilter) // If we're filtering the input data
	{
	  // Write the line to the filter input (and flush the buffer)...
	  (void) fprintf(cvPhotIn,"%s",line);
	  (void) fflush(cvPhotIn);
	  // Pull the result line from the filter output
	  (void) fgets(line,MAXLINE,cvPhotOut);
	} // end filtering this line


      // Extract line key
      char key[MAXLINE];
      if (sscanf(line,"%s",key) == 1)
	{

	  // Input Flux in Fnu (Jy)
	  if ((strcmp(key,"F") == 0)          
	      || (strcmp(key,"Fn") == 0))
	    {	  
	      // F HD4676--F8V 411 20 21.1096 2.11096  # Stromgren uvby  v=5.909     
	      if (sscanf(line,"%s %s %lf %lf %lf %lf",
			 junk,starID,&lambda,&bandwidth,&flux,&fluxerr) < 6)
		{
		  if (verbose)
		    (void) fprintf(stderr,"# Unable to parse line %i:%s\n", i, line);
		}
	      else
		{
		  lCount++;
		  if (flux > 0) /* If the flux isn't stupid */
		    {
		      // Converting flux from Jy (10^-26 W / m^2 / Hz) to
		      // erg / s / cm^2 / um 
		      fluxerr /= flux;
		      flux *= 3e-9 / ((lambda * NM_TO_UM) * (lambda * NM_TO_UM));
		      fluxerr *= flux;
		      // If running with fixed error fraction
		      if (fixedError)
			fluxerr = flux * errorFrac;

		      // New read/parse logic for linked-list storage
		      char *starName = starID;
		      // Find or create a star node corresponding to this star
		      if (!starList->findStar(starName))
			starList->insert(starName);

		      // wavelength conversions
		      lambda *= NM_TO_CM; // convert to cm
		      bandwidth *= NM_TO_CM;

		      // Create a data node with this flux data contents
		      Fluxdata *fluxNode
			= new Fluxdata(starName,0,
				       lambda,flux,fluxerr,bandwidth);

		      // Insert the data from this line into the linked-list
		      // structure for this star.  Note it is being
		      // insertion-sorted into the list based on wavelength.
		      starList->getStarData()->insert(fluxNode,lambda,INSERT_SORT);
		    } /* end if flux > 0 */
		} // end good line read
	      
	    } // end key = F or Fn/Fnu flux information (Jy)


	  // Input Flux in Flambda (erg cm^-2 s^-1 um^-1)
	  if (strcmp(key,"Fl") == 0)
	    {	  
	      // F HD4676--F8V 411 20 21.1096 2.11096  # Stromgren uvby  v=5.909     
	      if (sscanf(line,"%s %s %lf %lf %lf %lf",
			 junk,starID,&lambda,&bandwidth,&flux,&fluxerr) < 6)
		{
		  if (verbose)
		    (void) fprintf(stderr,"# Unable to parse line %i:%s\n", i, line);
		}
	      else // good line read
		{
		  lCount++;
		  if (flux > 0) /* If the flux isn't stupid */
		    {
		      // New read/parse logic for linked-list storage
		      char *starName = starID;
		      // Find or create a star node corresponding to this star
		      if (!starList->findStar(starName))
			starList->insert(starName);

		      // wavelength conversions
		      lambda *= NM_TO_CM; // convert to cm
		      bandwidth *= NM_TO_CM;
		      // If running with fixed error fraction
		      if (fixedError)
			fluxerr = flux * errorFrac;

		      // Create a data node with this flux data contents
		      Fluxdata *fluxNode
			= new Fluxdata(starName,0,
				       lambda,flux,fluxerr,bandwidth);

		      // Insert the data from this line into the linked-list
		      // structure for this star.  Note it is being
		      // insertion-sorted into the list based on wavelength.
		      starList->getStarData()->insert(fluxNode,lambda,INSERT_SORT);
		    } /* end if flux > 0 */
		} // end good line read
	      
	    } // end key = Fl/Flambda flux information


	  // Flux information in magnitudes
	  if (strcmp(key,"M") == 0)
	  {
	    int filter;
	    double mag;

	    // M HD4676--F8V 550 89 5.07 0.05  # UBV Johnson V 1966ArA.....4..137H
	    if (sscanf(line,"M %s %lf %lf %lf %lf",
		       starID,&lambda,&bandwidth,&mag,&fluxerr) < 5)
	      {
		if (verbose)
		  fprintf(stderr,"# Unable to parse line %i:%s\n", i, line);
		// break;
	      }
	    else // good line read
	      {
		lCount++;
		bandwidth *= NM_TO_CM;  // Convert to cm
		lambda *= NM_TO_CM;
		filter = which_filter(lambda,bandwidth,i);
		if (filter < 0)
		  {
		    if (verbose)
		      fprintf(stderr,"# Unable to find filter used on line %i: %s", i, line);
		  }
		else
		  {
		    flux = flux_from_mag(mag,filter);
		    fluxerr *= flux * 0.92108;		// see Bevington error propagation (p.50 2nd edition)
		    // If running with fixed error fraction
		    if (fixedError)
		      fluxerr = flux * errorFrac;

		    // New read/parse logic for linked-list storage
		    char *starName = starID;
		    // Find or create a star node corresponding to this star
		    if (!starList->findStar(starName))
		      starList->insert(starName);

		    // Create a data node with this flux data contents
		    Fluxdata *fluxNode
		      = new Fluxdata(starName,0,
				     lambda,flux,fluxerr,bandwidth);

		    // Insert the data from this line into the linked-list
		    // structure for this star.  Note it is being
		    // insertion-sorted into the list based on wavelength.
		    starList->getStarData()->insert(fluxNode,lambda,INSERT_SORT);
	
		  } // end good filter
	      } // end good line read
	  } // end key = M/flux given in magnitudes

	      
	// Add a filter
        if (strcmp(key,"Z") == 0)
	  {
	    char filter_name[MAXLINE];
	    double filter_wavelength,filter_bandpass,filter_zeropoint;
       
	    if (sscanf(line,"%s %s %lf %lf %lf",
		       junk,filter_name,&filter_wavelength,
		       &filter_bandpass,&filter_zeropoint) < 5)
	      {
		if (verbose)
		  fprintf(stderr,"# Unable to parse line %i:%s\n", i, line);
	      }
	    else
	      {
		strcpy(filters[nfilters].name,filter_name);
		filters[nfilters].wavelength = filter_wavelength;
		filters[nfilters].bandpass = filter_bandpass;
		filters[nfilters].zeropoint = filter_zeropoint;
		nfilters++;
		newfilters++;
	      } // end good line read
	  } /* end case Z/adding a filter */

	// Provide a distance
        if (strcmp(key,"D") == 0)
	  {
	    double distance,distanceerr;

	    // D HD4676--F8V 23.923 0.429  # PLX 1997A&A...323L..49P
	    if (sscanf(line,"D %s %lf %lf",
		       starID,&distance,&distanceerr) < 3)
	      {
		if (verbose)
		  fprintf(stderr,"# Unable to parse line %i:%s\n", i, line);
	      }
	    else
	      {
		lCount++;

		// New read/parse logic for linked-list storage
		char *starName = starID;
		// Find or create a star node corresponding to this star
		if (!starList->findStar(starName))
		  starList->insert(starName);

		if (starList->get_distance() < 0)
		  {
		    starList->set_distance(distance);
		    starList->set_distanceErr(distanceerr);
		  }
		else // Weighted average with previous
		  {
		    double tmp
		      = starList->get_distance() / pow(starList->get_distanceErr(),2.0)
		      + distance / pow(distanceerr,2.0);
		    double tmpErr
		      = 1.0 /
		      (1.0 / pow(starList->get_distanceErr(),2.0)
		       + 1.0 / pow(distanceerr,2.0));
		    tmp *= tmpErr;
		    tmpErr = sqrt(tmpErr);

		    starList->set_distance(tmp);
		    starList->set_distanceErr(tmpErr);
		  } // end weighted average
	      } // end good line read
	  } // end case D/distance estimate


        // Provide parallax in mas
	if (strcmp(key,"P") == 0)
	  {
	    double parallax,parallaxerr;

	    if (sscanf(line,"P %s %lf %lf",
		       starID,&parallax,&parallaxerr) < 3)
	      {
		if (verbose)
		  fprintf(stderr,"# Unable to parse line %i:%s\n", i, line);
		// break;
	      }
	    lCount++;

	    // New read/parse logic for linked-list storage
	    char *starName = starID;
	    // Find or create a star node corresponding to this star
	    if (!starList->findStar(starName))
	      starList->insert(starName);

	    if (starList->get_distance() < 0)
	      {
		starList->set_distance(1000.0 / parallax);
		starList->set_distanceErr(1000.0 / parallax * parallaxerr / parallax);
	      } // end first estimate
	    else // Weighted average with previous
	      {
		double distance = 1000.0 / parallax;
		double distanceerr = 1000.0 / parallax * parallaxerr / parallax;

		double tmp
		  = starList->get_distance() / pow(starList->get_distanceErr(),2.0)
		    + distance / pow(distanceerr,2.0);
		double tmpErr
		  = 1.0 /
		    (1.0 / pow(starList->get_distanceErr(),2.0)
		     + 1.0 / pow(distanceerr,2.0));
		tmp *= tmpErr;
		tmpErr = sqrt(tmpErr);

		starList->set_distance(tmp);
		starList->set_distanceErr(tmpErr);
	      } // end weighted average

	  } /* end case P/give a parallax estimate */

	
	// Give apparent angular size (ignored)
        if (strcmp(key,"R") == 0)
	  {
	    double radius;

	    if( sscanf(line,"%s %s %lf", junk, starID, &radius) < 3)
	      {
		if (verbose)
		  fprintf(stderr,"# Unable to parse line %i:%s\n", i, line);

	      }

	  } /* end case R/give an apparent angular size estimate */
	
	// case '#':
	// case 'C':
	
        // Provide estimate effective temperature
	if (strcmp(key,"T") == 0)
	  {
	    double dumTemp;

	    // T HD4676--F8V  6137 # Effective Temp <= Spectral Type F8V
	    if (sscanf(line,"T %s %lf",starID,&dumTemp) < 2)
	      {
		if (verbose)
		  fprintf(stderr,"# Unable to parse filenames on line %i.\n", i);
	      }
	    else // good line read
	      {
		lCount++;
		/* Make the effective temp estimate assignment to the star */

		// New read/parse logic for linked-list storage
		char *starName = starID;
		// Find or create a star node corresponding to this star
		if (!starList->findStar(starName))
		  starList->insert(starName);

		if (starList->get_temperature() < 0)  // If first temperature
		  {
		    starList->set_temperature(dumTemp);
		    starList->increment_nTemperatures();
		  }
		else // Avg with previous temperature estimates
		  {
		    int nTemp = starList->get_nTemperatures();
		    
		    double tAvg
		      = nTemp * starList->get_temperature() + dumTemp;

		    nTemp++;
		    tAvg /= (double) nTemp;

		    starList->set_temperature(tAvg);
		    starList->increment_nTemperatures();
		  } // end average
	      } // end good line read

	  } /* end case T/give an effective temperature estimate */


        // Provide estimate color index
	if (strcmp(key,"C") == 0)
	  {
	    double dumColor;

	    // Currently only supporting B-V color index:
	    // C HD_210027 B-V=0.44 ## Simbad B-V color index

	    if (sscanf(line,"C %s B-V=%lf",starID,&dumColor) < 2)
	      {
		if (verbose)
		  fprintf(stderr,"# Unable to parse line %i.\n", i);
	      }
	    else // good line read
	      {
		lCount++;
		// Make the color index assignment to the star 

		// New read/parse logic for linked-list storage
		char *starName = starID;
		// Find or create a star node corresponding to this star
		if (!starList->findStar(starName))
		  starList->insert(starName);

		starList->set_BminusV(dumColor);
	      } // end good line read

	  } // end case C/give a color index


	// Change output filenames
        if (strcmp(key,"N") == 0)
	  {
	    char datafilename[MAXLINE],bbfilename[MAXLINE];

	    if (sscanf(line,"%s %s %s",junk, datafilename , bbfilename) < 3)
	      {
		if (verbose)
		  fprintf(stderr,"# Unable to parse filenames on line %i.\n", i);
	      }

	  } /* end case N/playing filename games */
	      

	} // end if successful key read
      
    } /* end while fgets */


  // Report on results
  (void) fprintf(stdout,"# Read %d data lines from file %s\n",i,filename);

  if (strcmp(filename,"stdin")) // Check to see if we are reading from stdin
    fclose(fp);



  if (cvPhotFilter)
    {
      (void) fclose(cvPhotIn); // Should shut down child
      (void) fclose(cvPhotOut);
    } // end closing up the filter process


  return lCount;

/**********  end readDataFile  ******************************************/
}



/************************************************************************
*  reddenList -- local to fbol.c (A.B. 3/02)				*
*									*
*  A.  Purpose:	reddenList applies wavelength-dependent extinction      *
*       corrections to stellar fluxes.					*
*									*
*  B.  Background and use: Not due to me -- I just put the header on    *
*       it -- due to GvB and Ben.					*
*									*
*  C.  Called by:							*
*									*
*  D.  Input/output files:						*
*									*
*  E.  Parameter description:						*
*	Input -								*
*	Output -							*
*									*
*  F.  Detail function description:					*
*	      1.  Local Functions:					*
*	      2.  Externals:						*
*	      3.  Globals and Nonlocals used:				*
*	      4.  Global and Nonlocal variables modified:		*
*	      5.  Function flow is as follows:				*
*	      6.  Functions called by					*
*									*
*  G.  Function value: This function returns no useable value.		*
*									*
*************************************************************************/
void reddeningList(StarList *starList)
{
/**********  local declarations reddenList  *****************************/
#define Av 1.9 // magnitudes per kiloparsec; see Allen p.263
  int i;
  double Al;
  double Alerr=0.0,temp,temperr;

/**********  begin reddenList  ******************************************/
  dataList<Fluxdata> *fluxList = starList->getStarData();
  if ((fluxList->numberNodes())
      && (starList->get_distance() > 0))
    {
      double distance = starList->get_distance();
      double distanceErr = starList->get_distanceErr();
      fluxList->gotoHead();  // Start at the front

      do
	{
	  // Local wl in microns
	  double wl = fluxList->getCurrent()->get_wavelength() * 1e4;
	  if (wl < 0.554000)  // If shortward of V...
	    Al = pow(10.0,log10(wl) * -0.98786 - 0.23748); // scaling of Al relative to Av
	  else
	    Al = pow(10.0,log10(wl) * -1.72371 - 0.43002); // (double line fit is for 'bend' in data,
	                                                   // fit is good to ~0.03mag from U to K)
	  Al *= Av;					   // scale to Av, see Allen p.264
	  Al *= distance / 1000;			   // scale to distance
	  Alerr = Al * distanceErr / distance;		   // determine error in Al

	  double flux = fluxList->getCurrent()->get_flux();
	  double fluxErr = fluxList->getCurrent()->get_fluxErr();

	  double temp = flux;
	  double temperr = fluxErr / flux / 0.92108;	   // extract old mag err
	  temperr = sqrt(temperr * temperr + Alerr * Alerr);   // adjust for Al error
	  flux *= exp(Al*0.92108);
	  fluxList->getCurrent()->set_flux(flux); 
	  fluxList->getCurrent()->set_fluxErr(flux * 0.92108 * temperr);
	}
      while (fluxList->iterate());
    } // end if check for reddening
/**********  end reddenList  ********************************************/
}


/************************************************************************
*  b_lambda -- local to fbol.cc (A.B.)					*
*									*
*  A.  Purpose:	b_lambda evaluates the Plank blackbody function         *
*       (radiation intensity -- flux per unit wavelength) for the       *
*       input temperature at the input wavelength.      		*
*									*
*  B.  Background and use:						*
*									*
*  C.  Called by:							*
*									*
*  D.  Input/output files:						*
*									*
*  E.  Parameter description:						*
*	Input -								*
*        temp -- the input temperature (K) (double).                    *
*        lambda -- the input wavelength (cm) (double).                  *
*									*
*  F.  Detail function description:					*
*	      1.  Local Functions:					*
*	      2.  Externals:						*
*	      3.  Globals and Nonlocals used:				*
*	      4.  Global and Nonlocal variables modified:		*
*	      5.  Function flow is as follows:				*
*	      6.  Functions called by					*
*									*
*  G.  Function value: This function returns the value of the Plank     *
*       black-body function (B_lambda, radiation intensity per unit     *
*       wavelength -- see AQ 4th Ed, p99) for the input temperature at  *
*       the input wavelength (erg / cm^2 / s /cm) (double).             *
*									*
*************************************************************************/
/**********  function declaration b_lambda  *****************************/
double  b_lambda(double temp, double lambda)
{
/**********  begin b_lambda  ********************************************/
  double result = 2.0 * PI * HH * CC * CC;
  result /= lambda * lambda * lambda * lambda * lambda;  
  // Using expm1 rather than (exp(arg) - 1) -- for accuracy (see man expm1)
  result /= (expm1(HH * CC / (lambda * KK * temp)));
  return result; // return units are erg / cm^2 / s / cm

/**********  end b_lambda  **********************************************/
}


/************************************************************************
*  fluxBB -- local to fbol.c (A.B. 10/98)				*
*									*
*  A.  Purpose: fluxBB performs black-body flux calculations for the    *
*       input black-body model, specifically with NR-style Marquardt-   *
*       Levenberg fitting in mind (i.e. derivatives are computed).      *
*									*
*  B.  Background and use:						*
*									*
*  C.  Called by:							*
*									*
*  D.  Input/output files:						*
*									*
*  E.  Parameter description:						*
*	Input -								*
*        lambda -- input wavelength (cm) (double).                      *
*        model -- source model array (double *):                        *
*                  model[1] -- squared angular diam (rad^2)             *
*                  model[2] -- effective temperature (K)                *
*        np -- vestigial - to follow NR form (int).                     *
*	Output -							*
*        modelFlux -- computed spectral flux per unit wavelength        *
*         (erg / s / cm / micron) (double *)                            *
*        dFluxdModel -- computed derivatives of modelFlux WRT model     *
*         parameters (e.g. dFluxdModel[1] = d modelFlux / d model[1])   *
*         (double *)                                                    *
*									*
*  F.  Detail function description:					*
*	      1.  Local Functions:					*
*	      2.  Externals:						*
*	      3.  Globals and Nonlocals used:				*
*	      4.  Global and Nonlocal variables modified:		*
*	      5.  Function flow is as follows:				*
*	      6.  Functions called by					*
*									*
*  G.  Function value: This function returns no useful value.           *
*									*
*************************************************************************/
/**********  function declaration fluxBB  *******************************/
void fluxBB(double lambda, double *model,
	    double *modelFlux, double *dFluxdModel, int np)
{
/**********  begin fluxBB  **********************************************/
#define dT 30

  // model[0] is the squared angular diameter (rad^2),
  // model[1] is the effective temperature (K)  A.B. 9/02

  double Btlambda = b_lambda(model[1],lambda);  // Evaluate black-body function
  *modelFlux = (1.0 / 4.0) * model[0] * Btlambda; // Units of erg / cm^2 / s / cm
  *modelFlux /= CM_TO_UM; // Units of erg / cm^2 / s / micron
  // Units should now be in erg / cm^2 / s / micron

  /* If we want derivatives... */
  if (dFluxdModel) // Check that we've got a non-zero storage pointer
    {
      // The diameter one is simple
      dFluxdModel[0] = (1.0 / 4.0) * Btlambda / CM_TO_UM;

      /* Evaluate the derivative WRT temperature numerically (yes I'm lazy...) */
      double dBBdT = b_lambda(model[1] + dT,lambda)
	              - b_lambda(model[1] - dT,lambda);
      dBBdT /= 2 * dT;
      dFluxdModel[1] = (1.0 / 4.0) * model[0] * dBBdT / CM_TO_UM;

    } /* end evaluating derivatives -- input pointer is non-trivial */

/**********  begin fluxBB  **********************************************/
}



/************************************************************************
*  fitVectorsList -- local to fbol.c (A.B. 10/98)			*
*									*
*  A.  Purpose:	fitVectors constructs fitting vectors based on the      *
*       agreement of the data with a (possibly) input model, and an     *
*       input agreement tolerance.					*
*									*
*  B.  Background and use:						*
*									*
*  C.  Called by:							*
*									*
*  D.  Input/output files:						*
*									*
*  E.  Parameter description:						*
*	Input -								*
*	Output -							*
*									*
*  F.  Detail function description:					*
*	      1.  Local Functions:					*
*	      2.  Externals:						*
*	      3.  Globals and Nonlocals used:				*
*	      4.  Global and Nonlocal variables modified:		*
*	      5.  Function flow is as follows:				*
*	      6.  Functions called by					*
*									*
*  G.  Function value: This function returns no useable value.		*
*									*
*************************************************************************/
/**********  function declaration fitVectorsList  ***********************/
int fitVectorsList(StarList *starList,double *model,double tol,
		   double *lambdaFit,double *fluxFit,double *fluxErrFit)
{
/**********  local declarations fitVectorsList  *************************/
  int use = 0;

/**********  begin fitVectorsList  **************************************/
  /* Allocate memory */
  dataList<Fluxdata> *fluxList = starList->getStarData();
  int nPts = fluxList->numberNodes();

  /* Loop over each of the input data */
  fluxList->gotoHead();
  do
    {
      double pred;

      /* If there is an input model, evaluate it at this wavelength */
      if (model)
	(void) fluxBB(fluxList->getCurrent()->get_wavelength(),
		      model,&pred,0,2);

      /* If there wasn't an input model, or the agreement of the data
	 and the input model is better than a specified tolerance, add this
	 datum to the fit vectors */
      if ((!model)
	  || (fabs(fluxList->getCurrent()->get_flux() - pred) < tol))
	{
	  lambdaFit[use] = fluxList->getCurrent()->get_wavelength();
	  fluxFit[use] = fluxList->getCurrent()->get_flux();
	  fluxErrFit[use] = fluxList->getCurrent()->get_fluxErr();
	  use++;
	} /* end if */
    } 
  while (fluxList->iterate());

  return use;

/**********  end fitVectorsList  ****************************************/
}


/************************************************************************
*  fitBB -- local to bb.c (A.B. 11/98)					*
*									*
*  A.  Purpose:	The function fitBB fits a black-body model to a set     *
*       of flux measurements using the Numerical Recipes Marquardt      *
*       Levenberg routine.                                              *
*									*
*  B.  Background and use:						*
*									*
*  C.  Called by:							*
*									*
*  D.  Input/output files:						*
*									*
*  E.  Parameter description:						*
*	Input -								*
*	Output -							*
*									*
*  F.  Detail function description:					*
*	      1.  Local Functions:					*
*	      2.  Externals:						*
*	      3.  Globals and Nonlocals used:				*
*	      4.  Global and Nonlocal variables modified:		*
*	      5.  Function flow is as follows:				*
*	      6.  Functions called by					*
*									*
*  G.  Function value: This function returns no useable value.		*
*									*
*************************************************************************/
/**********  function declaration fitBB  ********************************/
double fitBB(double *lambda,double *flux,double *fluxerr,int nPts,
	     double *model, double **cov, int mfit)
{
/**********  local declarations fitBB  **********************************/
#define IMPROVE_TOL 1.0e-2
#define MAX_CHI 0.5
#define MAX_SUCCESSIVE_NOOPS 5
#define MAX_ITERATIONS 100

  int successiveNoOp = 0;
  int iter = 0;

  double oldChi,chi = 1e12;
  double alambda = -1.0;

  // int *lista = ivector(1,2);
  int lista[2];

  double **alpha = new double *[2];
  alpha[0] = new double[2];
  alpha[1] = new double[2];

/**********  begin fitBB  ***********************************************/
  /* 3 cases:
     mfit = 1 -- only fit for geometrical factor, constrain temp
     mfit = 2 -- only fit for temp, constrain geometrical factor
     mfit = 3 -- fit for both
  */
  switch (mfit)
    {
    case 1:
      lista[0] = 0;
      lista[1] = -1;
      mfit = 1;
      break;

    case 2:
      lista[0] = 1;
      lista[1] = 0;
      mfit = 1;
      break;

    case 3:
      lista[0] = 0;
      lista[1] = 1;
      mfit = 2;

    } /* end switch */

  /* Allocate the working space */
  do
    {
      float improve;

      iter++;
      oldChi = chi;
      // Call the marquardt routine
      (void) mrqMin(lambda,flux,fluxerr,nPts,
		    model,2,lista,mfit,
		    cov,alpha,
		    &chi,(&fluxBB),&alambda);

      chi /= nPts - mfit; /* convert to chi/DOF */

      improve = (oldChi - chi) / oldChi;
      if (improve < IMPROVE_TOL)
	successiveNoOp++;
      else
	successiveNoOp = 0;
    }
  while ((chi > MAX_CHI)
	 && (successiveNoOp <= MAX_SUCCESSIVE_NOOPS)
	 && (iter <= MAX_ITERATIONS));

  // Compute the fit covariance matrix
  alambda = 0.0;
  (void) mrqMin(lambda,flux,fluxerr,nPts,
		model,2,lista,mfit,
		cov,alpha,
		&chi,(&fluxBB),&alambda);


  delete []alpha[0];
  delete []alpha[1];
  delete []alpha;

  return chi;

#undef MAX_CHI

/**********  end fitBB  *************************************************/
}




/************************************************************************
*  scoreFit -- local to bb.c (A.B. 11/98)				*
*									*
*  A.  Purpose:	The function scoreFit compares the fit model to data,   *
*       and computes statistics on the agreement.			*
*									*
*  B.  Background and use:						*
*									*
*  C.  Called by:							*
*									*
*  D.  Input/output files:						*
*									*
*  E.  Parameter description:						*
*	Input -								*
*	Output -							*
*									*
*  F.  Detail function description:					*
*	      1.  Local Functions:					*
*	      2.  Externals:						*
*	      3.  Globals and Nonlocals used:				*
*	      4.  Global and Nonlocal variables modified:		*
*	      5.  Function flow is as follows:				*
*	      6.  Functions called by					*
*									*
*  G.  Function value: This function returns no useable value.		*
*									*
*************************************************************************/
/**********  function declaration scoreFit  *****************************/
void scoreFit(double *fitLambda,double *fitFlux,int nUse,
	      double *model,double *rms,double *peak)
{
/**********  local declarations scoreFit  *******************************/
  int i;
  double sum = 0,sum2 = 0,max = 0;

/**********  begin scoreFit  ********************************************/

  for (i = 0; i < nUse; i++)
    {
      double pred,resid;

      (void) fluxBB(fitLambda[i],model,&pred,0,2);
      resid = fitFlux[i] - pred;

      sum += resid;
      sum2 += resid * resid;
      if (fabs(resid) > max)
	max = fabs(resid);

    } // end i loop

  /* Compute/return statistics */
  *rms = sqrt((sum2 - (sum * sum / nUse)) / (nUse - 1));
  *peak = max;

/**********  end scoreFit  **********************************************/
}



char *substituteString(char *oldString, char *target, char *newTarget)
{

  // insert newTarget  wherever we see target  (as in $desig =~ s/$target/$newTarget/g)

  int targetLength = (int) strlen(target);
  int newTargetLength = (int) strlen(newTarget);

  char *newString = new char[MAXLINE]; 
  (void) strcpy(newString,oldString);

  for (int i = 0; i < (int) strlen(newString); i++)
    if (strncmp(&newString[i],target,targetLength) == 0)
      {
	char tmp[MAXLINE];

	strcpy(tmp,&newString[i + targetLength]);
	strncpy(&newString[i],newTarget,newTargetLength);
	i += newTargetLength;
	strcpy(&newString[i],tmp);
      } // end loop

  return newString;

} // end substituteString




/************************************************************************
*  plotResults -- local to fbol.cc (A.B. 3/2002)			*
*									*
*  A.  Purpose: plotResults does the necessary manipulations for        *
*       producing plots of the fit results.     			*
*									*
*  B.  Background and use:						*
*									*
*  C.  Called by:							*
*									*
*  D.  Input/output files:						*
*									*
*  E.  Parameter description:						*
*	Input -								*
*	Output -							*
*									*
*  F.  Detail function description:					*
*	      1.  Local Functions:					*
*	      2.  Externals:						*
*	      3.  Globals and Nonlocals used:				*
*	      4.  Global and Nonlocal variables modified:		*
*	      5.  Function flow is as follows:				*
*	      6.  Functions called by					*
*									*
*  G.  Function value: This function returns no useable value.		*
*									*
*************************************************************************/
/**********  function declaration plotResults  **************************/
void plotResults(double fitTemp,double theta2,
		 double bolFlux,double bolFluxErr,
		 double angDiam,double angDiamErr,
		 double **cov,double chi,
		 StarList *starList,int nStar,
		 dataList<Fluxdata> *fluxList,
		 double *fitLambda,double *fitFlux,double *fitErr,
		 int nUse)
{
/**********  local declarations plotResults  ****************************/
#define WL_MIN 0.1
#define WL_MAX 50
#define MAX_TOL 1.0e-6


/**********  begin plotResults  *****************************************/

  double model[2];
  model[0] = theta2;
  model[1] = fitTemp;

  // The black-body model data
  FILE *pp;
  if ((pp = fopen("bbc.dat","w")) == NULL)
    {
      (void) fprintf(stderr,"Couldn't open plotfile %s for write\n","bbc.dat");
      exit(-1);
    }
  (void) fprintf(pp,"# BB Curve theta = %g mas, Teff = %8.1lf\n",
		 sqrt(theta2) * RAD_TO_MAS,fitTemp);
  (void) fprintf(pp,"# wavelength (um)   F_lambda (erg / cm^2 / s / micron)\n");
  // Compute the maximum in Flambda to guide the plotting range
  double lambdaMax = 0.290 / fitTemp; // Wein displacement law for max F_lambda wl (cm)
  double fluxMax;
  (void) fluxBB(lambdaMax,model,&fluxMax,0,0);
  double tmpLambda = WL_MIN / CM_TO_UM; // wl units are cm
  do
    {
      double result;      // Units of result are erg / cm^2 / s / micron
      (void) fluxBB(tmpLambda,model,&result,0,0);
      if (result > (fluxMax * MAX_TOL))  // control dynamic range of plot
	(void) fprintf(pp,"%g\t%g\n",tmpLambda * CM_TO_UM, result);
      tmpLambda *= 1.05;
    }
  while ((tmpLambda / CM_TO_UM) < WL_MAX);
  (void) fprintf(pp,"\n");    
  (void) fclose(pp);

  // Next the observational data...
  FILE *sdp;
  if ((sdp = fopen("star.dat","w")) == NULL)
    {
      (void) fprintf(stderr,"Couldn't open data file %s for write\n", "star.dat");
      exit(-1);
    }
  (void) fprintf(sdp,"# Star %i: %s\n", nStar, starList->getStarName());
  (void) fprintf(sdp,"# Wavelength(nm)  Flux  Fluxerr (erg/cm^2/s/um)    Resid (O-C)  Norm Resid\n",
		 nStar, starList->getStarName());

  // First write out all the data 
  fluxList->gotoHead();
  do
    {
      // Units of pred claim to be erg / cm^2 / s / micron
      double pred;
      (void) fluxBB(fluxList->getCurrent()->get_wavelength(),
		    model,&pred,0,0);
      double resid = fluxList->getCurrent()->get_flux() - pred;
      (void) fprintf(sdp,"%-11lg %-11lg %-11lg  %-11lg  %-11lg\n",
		     fluxList->getCurrent()->get_wavelength() * CM_TO_UM,
		     fluxList->getCurrent()->get_flux(),
		     fluxList->getCurrent()->get_fluxErr(),
		     resid,resid / pred);
    }
  while (fluxList->iterate());
  (void) fprintf(sdp,"\n\n\n");

  // Next write out only the data we used in the fit 
  for (int kk = 0; kk < nUse; kk++)
    {
      // Units of pred claim to be erg / cm^2 / s / micron
      double pred;
      (void) fluxBB(fitLambda[kk],model,&pred,0,0);
      double resid = fitFlux[kk] - pred;
      (void) fprintf(sdp,"%-11lg %-11lg %-11lg  %-11lg  %-11lg\n", 
		     fitLambda[kk] * CM_TO_UM,fitFlux[kk],
		     fitErr[kk],resid,resid / pred);
    } /* end kk loop */

  (void) fclose(sdp);

  // Do the plots.  Compose the correct gnuplot script
  //  depending on plot options 
  const char *fbolPlot = "fbolPlot";

  // screen plots
  if (screen)
    {
      char work[LINE_LENGTH];

      (void) sprintf(work,"%s -screen -starName %s -temperature %5.0f -tempErr %3.0f -bolometricFlux %.4g -bolFluxErr %.3g -angularDiameter %5.2f -angDiamErr %4.2f",
		     fbolPlot,
		     starList->getStarName(),
		     fitTemp,sqrt(cov[1][1] * chi),
		     bolFlux * 1e8,bolFluxErr * 1e8,
		     angDiam,angDiamErr);
      if (verbose)
	printf("fbol: Executing command \"%s\"\n",work);
      system(work);
    } // end doing screen plot

  // EPS
  if (eps && fbolPlot)
    {
      char work[LINE_LENGTH];

      (void) sprintf(work,"%s -eps -starName %s -temperature %5.0f -tempErr %3.0f -bolometricFlux %.4g -bolFluxErr %.3g -angularDiameter %5.2f -angDiamErr %4.2f",
		     fbolPlot,
		     starList->getStarName(),
		     fitTemp,sqrt(cov[1][1] * chi),
		     bolFlux * 1e8,bolFluxErr * 1e8,
		     angDiam,angDiamErr);
      if (verbose)
	printf("fbol: Executing command \"%s\"\n",work);
      system(work);
    } // end doing eps
	  
  // PostScript
  if (ps && fbolPlot)
    {
      char work[LINE_LENGTH];

      (void) sprintf(work,"%s -postscript -starName %s -temperature %5.0f -tempErr %3.0f -bolometricFlux %.4g -bolFluxErr %.3g -angularDiameter %5.2f -angDiamErr %4.2f",
		     fbolPlot,
		     starList->getStarName(),
		     fitTemp,sqrt(cov[1][1] * chi),
		     bolFlux * 1e8,bolFluxErr * 1e8,
		     angDiam,angDiamErr);
      if (verbose)
	printf("fbol: Executing command \"%s\"\n",work);
      system(work);
    } // end doing PostScript

  // PNG
  if (png && fbolPlot)
    {
      char work[LINE_LENGTH];

      (void) sprintf(work,"%s -png -starName %s -temperature %5.0f -tempErr %3.0f -bolometricFlux %.4g -bolFluxErr %.3g -angularDiameter %5.2f -angDiamErr %4.2f",
		     fbolPlot,
		     starList->getStarName(),
		     fitTemp,sqrt(cov[1][1] * chi),
		     bolFlux * 1e8,bolFluxErr * 1e8,
		     angDiam,angDiamErr);
      if (verbose)
	printf("fbol: Executing command \"%s\"\n",work);
      system(work);
    } // end doing png


/**********  end plotResults  *******************************************/
}



/************************************************************************
*  blackwellCorrections -- local to fbol.cc (AB 1/2004)			*
*									*
*  A.  Purpose: blackwellCorrections computes corrections to the fbol   *
*       modelling parameters to account for the non-black body          *
*       characteristics of stars.  Corrections are derived from         *
*       comparison to the data contained in Blackwell 1994.		*
*									*
*  B.  Background and use:						*
*									*
*  C.  Called by:							*
*									*
*  D.  Input/output files:						*
*									*
*  E.  Parameter description:						*
*	Input -								*
*	Output -							*
*									*
*  F.  Detail function description:					*
*	      1.  Local Functions:					*
*	      2.  Externals:						*
*	      3.  Globals and Nonlocals used:				*
*	      4.  Global and Nonlocal variables modified:		*
*	      5.  Function flow is as follows:				*
*	      6.  Functions called by					*
*									*
*  G.  Function value: This function returns no useable value.		*
*									*
*************************************************************************/
/**********  function declaration blackwellCorrections  *****************/
void blackwellCorrections(double BminusV,double temperature,
			  double bolFlux,double angDiam)
{
/**********  local declarations blackwellCorrections  *******************/

  // The temperature correction function
  double t0 = -0.339679;
  double t1 = 1.27678;
  double t2 = -1.74663;
  double t3 = 1.0806;
  double t4 = -0.240631;

  // The bolometric flux correction function
  double f0 = -0.608232;
  double f1 = 2.18982;
  double f2 = -3.57538;
  double f3 = 2.5783;
  double f4 = -0.700981;

  // The angular diameter correction function
  double a0 = 0.430932;
  double a1 = -1.77175;
  double a2 = 2.29932;
  double a3 = -1.34225;
  double a4 = 0.267781;


#define correction(x,a0,a1,a2,a3,a4) \
(1.0 + (a0 + (a1 * x) + (a2 * x * x) + (a3 * x * x * x) + (a4 * x * x * x * x)))

/**********  begin blackwellCorrections  ********************************/

  // Correct the temperature
  temperature *= correction(BminusV,t0,t1,t2,t3,t4);
  bolFlux *= correction(BminusV,f0,f1,f2,f3,f4);
  angDiam *= correction(BminusV,a0,a1,a2,a3,a4);

  (void) printf("    Blackwell correct:  %5.0lf\t\t\t",temperature);

  (void) printf("   %.4g \t",bolFlux * 1e8);

  (void) printf("  %5.2f \n",angDiam);

/**********  end blackwellCorrections  **********************************/
}




main(int argc, char *argv[])
{
/**********  begin main  ************************************************/
#define MAX_ITER 10
#define MAX_CHI 2.0
#define MAX_PEAK_TO_RMS 3.0

  int argIndx;
  StarList *starList = new StarList(); // Data structure with star flux data


  /* Process command-line options */
  argIndx = parseCmdLine(argc,argv);

  /* for each file AND stdin */
  for (int ifile = argIndx; ifile < argc; ifile++)
    (void) readDataFile(argv[ifile],starList);

  if (readStdin)
    (void) readDataFile("stdin",starList);

#ifdef PRINTFILTERS 
  (void) fprintf(stdout,"# Available Filters:\n");
  (void) fprintf(stdout,"# #: Name  Wavel(cm) Width(cm) Zeropoint(erg/cm2/s/um)\n");
  for (int j = 0; j < nfilters; j++)
    {
      fprintf(stdout,"# %2i : %-3s %11.3g  %11.3g  %11.3g\n",
	      j,filters[j].name,  filters[j].wavelength, filters[j].bandpass, filters[j].zeropoint);
    }
#endif


  if (starList->numberNodes())  // If non-trivial list
    {
      // for each star, fit to temperature
      // Print the all-so informative header
      printf("#                                   ChiSqr        F_bol (10^-8       Ang         \n");
      printf("#   Star               Teff(K)      /DOF   DOF     erg/cm2/s)     Size (mas)    Filters\n");

      starList->gotoHead(); // Pop to the head of the list
      int nStar = 0;
      do  // Loop over stars in input
	{
	  int nUse,nDOF,iter = 0;
	  const int nSize = starList->getStarData()->numberNodes();
	  double rms = 1e10,peak = 1e10;

	  if (nSize)
	    {
	      // Allocate working space for the data fitting
	      double *fitLambda = new double[nSize];
	      double *fitFlux = new double[nSize];
	      double *fitErr = new double[nSize];

	      double model[2];
	      // Allocate the covariance matrix
	      double **cov = new double *[2];
	      cov[0] = new double[2];
	      cov[1] = new double[2];

	      nStar++;
	      if (doReddening && (starList->get_distance() > 0))  // Correct fluxes for reddening
		reddeningList(starList);

	      // Testing the M-L fit
	      nUse = fitVectorsList(starList,0,0,fitLambda,fitFlux,fitErr);

	      // Initialize the model
	      model[0] = 2.5e-17;  // theta^2 -- initialize to 1mas^2 in radians
	      model[1] = starList->get_temperature();
	      if (model[1] < 0)
		model[1] = DEFAULT_TEMP;

	      // Do an initial fit constraining to input effective temperature
	      // to initialize the fit model
	      double chi = fitBB(fitLambda,fitFlux,fitErr,nUse,model,
				 (double **) cov,1);

	      do // Now enter the iteration loop 
		{
		  iter++;

		  // Rebuild the fit vectors, accepting data within tolerance of the model
		  double tol = MAX_PEAK_TO_RMS * rms;
		  nUse = fitVectorsList(starList,model,tol,
					fitLambda,fitFlux,fitErr);

		  /* Do and score the fit */
		  if (constrainTemp) // If we're constraining to input temperature fit only for factor
		    {
		      chi = fitBB(fitLambda,fitFlux,fitErr,nUse,model,
				  (double **) cov,1);
		      nDOF = nUse - 1;
		    }
		  else // Otherwise fit for BOTH factor and temperature
		    {
		      chi = fitBB(fitLambda,fitFlux,fitErr,nUse,model,
				  (double **) cov,3);
		      nDOF = nUse - 2;
		    }


		  (void) scoreFit(fitLambda,fitFlux,nUse,model,&rms,&peak);

		}
	      while ((iter < MAX_ITER)
		     && (chi > MAX_CHI)
		     && ((peak / rms) > MAX_PEAK_TO_RMS));

	      double fitTemp = model[1];  // For convenience of access/notation
	      double theta2 = model[0];

	      // Bolometric flux, units of erg / cm / s
	      // bolFlux = 1/4 theta^2 sigma T^4
	      double bolFlux = (1.0 / 4.0) * theta2 * SIGMA * pow(fitTemp,4.0);

	      double bolFluxErr
		= cov[0][0] / theta2 / theta2
		+ cov[1][1] * 16.0 / fitTemp / fitTemp;
	      + 2.0 * cov[0][1] * 4.0 / fitTemp / theta2; // Error calc with x-term!!!
	      bolFluxErr *= bolFlux * bolFlux; // Common factor removed from first eq
	      bolFluxErr *= chi;   // Weight by chi-squared per DOF 
	      bolFluxErr = sqrt(bolFluxErr); // There must be a sqrt in here!

	      printf("%3d ",nStar);
	      printf("%-15s %5.0lf +/- %3.0lf   %5.2lf %3i  %.4g +/- %.3g ",
		     starList->getStarName(),
		     fitTemp,sqrt(cov[1][1] * chi),
		     chi,nDOF,
		     bolFlux * 1e8,bolFluxErr * 1e8);

	      // In the new fbol scheme the angular diameter (square) is directly estimated
	      double angDiam = sqrt(theta2) * RAD_TO_MAS;
	      double angDiamErr = pow(cov[0][0],0.25) * RAD_TO_MAS;
	      angDiamErr *= sqrt(chi);

	      printf("%5.2f +/- %4.2f ",angDiam,angDiamErr);

	      /* Print a report on the filters present in the data */
	      dataList<Fluxdata> *fluxList = starList->getStarData();
	      fluxList->gotoHead();
	      do
		{
		  double wl = fluxList->getCurrent()->get_wavelength();
		  int found = 0;
		  for (int gg1 = 0; gg1 < nfilters; gg1++)
		    if (wl == filters[gg1].wavelength)
		      {
			printf("X");
			found = 1;
			gg1 = nfilters;
		      }
		  if (!found)
		    printf(".");
		}
	      while (fluxList->iterate());
	      (void) printf("\n");

	      // Compute Blackwell corrections as directed
	      if ((doBlackwellCorrections)
		  && (starList->get_BminusV() > INITIAL_VALUE))
		(void) blackwellCorrections(starList->get_BminusV(),
					    fitTemp,bolFlux,angDiam);

	      // If directed by the command-line switch, do plotting
	      if (doPlots)
		plotResults(fitTemp,theta2,bolFlux,bolFluxErr,
			    angDiam,angDiamErr,(double **) cov,chi,
			    starList,nStar,fluxList,
			    fitLambda,fitFlux,fitErr,nUse);

	      // Release working memory
	      delete []cov[0];  // Model covariance matrix
	      delete []cov[1];
	      delete []cov;

	      delete []fitLambda;  // Flux data vectors
	      delete []fitFlux;
	      delete []fitErr;

	    } // end if photometric data

	} // end star loop
      while (starList->iterate());

    } // end non-trivial list check

/**********  end main  **************************************************/
} // end fbol

