package formatPhotometry;
my $MODVERSION = '$Id: formatPhotometry.pm,v 1.16 2011/03/23 00:24:37 herstein Exp $';

##########################################################################
# NASA Exoplanet Science Institute (NExScI) -- http://nexsci.caltech.edu #
#       Copyright (c) 1997-2008 California Institute of Technology       #
#                     U.S. Government Sponsorship                        #
# Under contract with the National Aeronautics and Space Administration  #
##########################################################################

#
# formatPhotometry - gather and format photometry for the named object(s)
#
# Photometry is retrieved from several sources:
# - SIMBAD (UBV, uvby, Hbeta, Geneva, JP11, IRC)
# - IRAS
# - 2MASS
# - CIO
# - Hipparcos/Tycho
#
# All measurements for an object are then assembled and output using a
# standard format for further processing by "fbol".
#
# This is a rewrite of the "fbolFormat" utility, organized to
# isolate dependencies on SIMBAD, and updated for SIMBAD4 data formats.
# Steve Groom, February 2007

use strict;
use warnings;

require 5.000;

use Exporter();
our @ISA = qw(Exporter);
our @EXPORT = qw(formatPhotometry);

use FindBin qw($RealBin);
use lib "$RealBin/../lib/modules";		# location of getCal Perl modules
use gcConfig;
use gcDebug;
use gcSimbad;		# searchSimbad
use gcIrsa;		# search_irsa_nl
use astroCatalogue;	# normalIdent
use miscArgs;
use effectiveTemperature;
use photometry;
use miscString;

use hipparcosLine;
use hipRecord;

use Carp;

#use astroCatalogue;
#use gcIrsa;
#use miscArgs;
#use hipRecord;

my $cfg = gcConfig::config();

#########################################################################
## formatPhotometry - retrieve and format photometry information for given target
#########################################################################
# options recognized by formatPhotometry:
#    units	one of UNITS_MAG, UNITS_FLAMBDA, UNITS_FNU (default: UNITS_MAG)
#    2Mass	enable retrieval of 2MASS photometry	(default: off)
#    uvby	enable output of Stromgren/uvby photometry returned by SIMBAD (default: off)
#    geneva	enable output of geneva photometry returned by SIMBAD (default: off)
#    longWL|iras   enable output of IRAS photometry returned by SIMBAD (default: off)
#    hipparcos	enable output of photometry from hipparcos catalog (default: off)
#    gezari	enable output of photometry/radiometry from the Gezari catalog
#		(a.k.a. "CIO", Fifth Catalog of Infrared Observations, Gezari et al,
#		see http://ircatalog.gsfc.nasa.gov/. Off by default.
#    noU	suppress output of U-band photometry/radiometry
#    noV	suppress output of V-band photometry/radiometry
#    noB	suppress output of B-band photometry/radiometry
#
sub formatPhotometry
{
## declarations formatPhotometry  ###########################################
    my ($desigsRef, $objsRef, $opts) = @_;

    my @desigs = @{$desigsRef} if defined $desigsRef;	# list of source desigs
    my @objs = @{$objsRef} if defined $objsRef;		# corresponding list of SIMBAD result objects

    my @outLines;
    my $dbgname = "formatPhotometry";

    # set defaults for options that aren't explicitly provided
    if (!defined $opts->{units})	
    {
	$opts->{units} = photometry::UNITS_MAG;
	gcDebug $dbgname,1,"using default MAG units=",$opts->{units},"\n";
    }
    else
    {
	gcDebug $dbgname,1,"using specified units=",$opts->{units},"\n";
    }

## begin formatPhotometry  ##################################################

    dumpopts($dbgname,$opts) if gcConfig::debug(); 

    id: foreach my $id (@desigs)
    {
	my $simbadObj = shift @objs;
	if (defined $simbadObj)
	{
	    gcDebug $dbgname,1,"id=\"$id\": using provided SIMBAD object\n";
	}
	else
	{
	    if ($id eq "POSTARGET")
	    {
		gcDebug $dbgname,1,"skipping id POSTARGET\n";
		next id;
	    }

	    # use SIMBAD to lookup the given ID
	    gcDebug $dbgname,0,"id=\"$id\": simbad lookup\n";
	    $simbadObj = searchSimbad($id);
	    if (!$simbadObj)
	    {
		# maybe we should continue instead of giving up at this point?
		carp "$dbgname: SIMBAD lookup failed for id $id\n";
		next id;
	    }
	    my $desig = $simbadObj->desig();
	    gcDebug $dbgname,1,"id=\"$id\": simbad returned \"$desig\"\n";
	}
	my $simbadName = $simbadObj->desig();
	my $simbadHD = $simbadObj->hd();
	gcDebug $dbgname,1,"id=\"$id\", simbad desig=\"$simbadName\" ($simbadHD)\n";

	# We have SIMBAD information for this object.
	# Extract basic information about the object from the
	# "basic data" returned by SIMBAD

	### Object name and description
	my $nameline = "### $simbadName ($id)";
	my $desc = $simbadObj->description();
	if ($desc)
	{
	    $nameline .= " -- $desc\n";
	}
	push @outLines, $nameline;

	### get the ICRS 2000 position
	my $ra = $simbadObj->raHMS();
	my $dec = $simbadObj->decDMS();
	my $coordLine = "$ra $dec";
	push @outLines, "### ICRS 2000: $coordLine\n";

	### Object name 
	my $name = $simbadName;	# name we'll use for photometry lines
	$name =~ s/\s+/_/g;	# replace space in name with underscore on data lines

	### Spectral type
	my $spType = $simbadObj->spec();
	if ($spType =~ m/\S+/)
	{
	    push @outLines, "### Spectral Type $spType\n";
	    ## Append the spectral type to the name
	    $name .= "--$spType";

	    ## Estimate the effective temperature
	    my $eT = effTemp($spType);
	    my $tempLine;
	    if (!(defined $eT) || ($eT <= 0))  ## check for failure
	    {
		$eT = "undefined";  ## no model effective temperature
		$tempLine = "#T $name $eT # Model Effective Temp <= Spectral Type $spType\n";
	    }
	    else
	    {
		$tempLine = "T $name $eT # Model Effective Temp <= Spectral Type $spType\n";
	    }
	    push @outLines, $tempLine;
	} ## end non-trivial $spType

	gcDebug $dbgname,1,"id=\"$id\": SIMBAD photometry\n";
	my @simbadPhotLines = doSimbadPhot($name,$opts,$simbadObj);
	push @outLines, @simbadPhotLines;


	if ($opts->{twoMass})
	{
	    # retrieve and format photometry from 2MASS
	    my @twoMassPhotLines = do2MassPhot($name,$opts,$simbadObj);
	    push @outLines, @twoMassPhotLines;
	}
	if ($opts->{hipparcos})
	{
	    # retrieve and format photometry from Hipparcos
	    my @hipPhotLines = doHipPhot($name,$opts,$simbadObj);
	    push @outLines, @hipPhotLines;
	}
	if ($opts->{gezari})
	{
	    # retrieve and format photometry from CIO/Gezari
	    my @gezLines = doGezariPhot($name,$opts,$simbadObj);
	    push @outLines, @gezLines;
	}
	push @outLines, "\n\n";

	gcDebug $dbgname,1,"id=\"$id\": done\n";
    }
    return @outLines;
} ## end formatPhotometry  ################################################


#########################################################################
## doSimbadPhot -- extract and report photometry from SIMBAD
#########################################################################
sub doSimbadPhot
{
## declarations doSimbadPhot  ##############################################
    my $name = shift ;		# target name to use
    my $opts = shift ;		# reference to options hash
    my $simbadObj = shift ;	# populated simbad query result object

    my @outLines; # output lines we accumulate as we process various sections
## begin doSimbadPhot  #####################################################

    my $dbgname = "doSimbadPhot";	# name of this routine for debugging
    my $deferr = 0.09;	#XXX this hardcoded value should be a parameter somewhere

    dumpopts($dbgname,$opts) if gcConfig::debug();

    if (!$simbadObj)
    {
	#XXX is this the right error check to do?
	carp "$dbgname: object pointer was null\n";
	return;
    }

    ### V mag
    my $Vmag = $simbadObj->vmag();
    if (&isNumeric($Vmag))
    {
	push @outLines, magval("###", "V", $Vmag) . "\n";	# V comment
	my $out = "";
	$out = "#" if ($opts->{noV});
	$out .= formatPhotRec($name,"Johnson","V",$Vmag,$deferr, magval("Simbad", "V", $Vmag),$opts->{units});
	push @outLines, $out;
    }

    ### B mag
    my $Bmag = $simbadObj->bmag();
    if (&isNumeric($Bmag))
    {
	push @outLines, magval("###", "B", $Bmag) . "\n";	# B comment
	my $out = "";
	$out = "#" if ($opts->{noB});
	$out .= formatPhotRec($name,"Johnson","B",$Bmag,$deferr, magval("Simbad", "B", $Bmag),$opts->{units});
	push @outLines, $out;
    }

    ### B-V mag
    my $BmV = "?";
    if (&isNumeric($Bmag) && &isNumeric($Vmag))
    {
	$BmV = sprintf "%4.2f", $Bmag - $Vmag;
    }
    push @outLines, magval("###", "B-V", $BmV) . "\n";	# B-V comment
    push @outLines, magval("C $name", "B-V", $BmV) . "## Simbad B-V color index\n";

    ### Parallax
    # line is of the form:
    # 65.10 A [0.76] 1997A&A...323L..49P
    # we need the first (plx) and second (plxerr) fields only
    my $plx = $simbadObj->plx();
    my $plxfull = $simbadObj->plxfull();
    my $plxerr = $plxfull;
    if ($plxerr)
    {
	$plxerr =~ s/^.*\[//;	# extracting error from between the brackets
	$plxerr =~ s/\].*$//;	# should have only plxerr left
    }
    gcDebug $dbgname,1,"plxfull= \"",nonNull($plxfull,"?"),"\" plx=",nonNull($plx,"?")," plxerr=",nonNull($plxerr,"?"),"\n";
    if (&isNumeric($plx) && ($plx != 0.0))
    {
	if (!&isNumeric($plxerr))
	{
	    gcDebug $dbgname,1,"parallax error value for name=$name was not numeric, assuming 0.0\n";
	    $plxerr = 0.0;
	}
	my $dist = sprintf "%6.3f", 1000.0 / $plx;
	my $dErr = sprintf "%5.3f", $dist * $plxerr / $plx;

	my $out = "";
	$out .= "#" if ($opts->{noPlx});
	$out .= "D $name $dist $dErr ## Parallax $plxfull\n";
	push @outLines, $out;
    }
    else
    {
	gcDebug $dbgname,1,"parallax value for name=$name is not usable, skipping\n";
    }


    # Get the measurement lines returned by SIMBAD
    # sort the array of lines into a hash-of-arrays with one
    # hash element per measurement system.
    my @measLines = $simbadObj->meas();
    my %meas = sortMeasurements(@measLines);	# group measurements by system
    if (gcConfig::debug())	# debugging: dump the measurements information
    {
	foreach my $m (keys %meas)
	{
	    gcDebug $dbgname,0,"measurement system $m\n";
	    foreach my $line (@{$meas{$m}})
	    {
		gcDebug $dbgname,0,"$m measurement line \"$line\"\n";
	    }
	}
    }

    ### Deal with each measurement system one at a time

    ### process Johnson UBV photometry
    my @UBVLines = &processSimbad_UBV($name, $opts, @{$meas{"UBV"}});
    push @outLines, @UBVLines if (scalar @UBVLines);


    ### process Stromgren uvby photometry
    if ($opts->{uvby})
    {
	# first do the actual Stromgren photometry, saving
	# the output lines and some averages calculated along the way
	my ($avg_BmY, $avg_m1, $avg_c1, $nPts, @uvbyLines) = 
	    &processSimbad_uvby($name, $opts, $Vmag, @{$meas{"uvby1"}});
	push @outLines, @uvbyLines if (scalar @uvbyLines);

	# now use those averages to process any Hbeta photometry
	my @hbetLines = 
	    &processSimbad_Hbeta($name, $opts, $avg_BmY, $avg_m1, $avg_c1, @{$meas{"Hbet1"}});
	push @outLines, @hbetLines if (scalar @hbetLines);
    }

    ### process Geneva photometry
    if ($opts->{geneva})
    {
	my @genevaLines = &processSimbad_Geneva($name, $opts, @{$meas{"GEN"}});
	push @outLines, @genevaLines if (scalar @genevaLines);
    }

    ### process JP11 catalog values
    my @JP11Lines = &processSimbad_JP11($name, $opts, @{$meas{"JP11"}});
    push @outLines, @JP11Lines if (scalar @JP11Lines);

    ### process IRC data
    my @IRCLines = &processSimbad_IRC($name, $opts, @{$meas{"IRC"}});
    push @outLines, @IRCLines if (scalar @IRCLines);

    ### process IRAS data from Simbad
    if ($opts->{longWL})
    {
	my @IRASLines = &processSimbad_IRAS($name, $opts, @{$meas{"IRAS"}});
	push @outLines, @IRASLines if (scalar @IRASLines);
    }

    ### all done
    return @outLines;
}

#########################################################################
## processSimbad_UBV -- Process UBV photometry from Simbad
#########################################################################
sub processSimbad_UBV
{
## declarations processSimbad_UBV  #######################################
    my $name = shift;
    my $opts = shift;
    my @lines = @_;

    my @outLines;	# the output lines we will return
## begin processSimbad_UBV  ##############################################
    my $dbgname = "processSimbad_UBV";
    gcDebug $dbgname, 1, "begin\n";

    # Process and output UBV photometry from measurement lines returned
    # by SIMBAD, respecting options controlling which bands to output.
    #
    # Measurement lines are assumed to have a fixed tabular format, with
    # fields separated by vertical bars. The first line of input
    # is a header line, it is used here only as a quick check to
    # see whether the output format has changed unexpectedly.
    #
    # Section looks like this (first line is header):
    ## UBV  |      V      B-V     U-B  |Q/nbm|  remarks  |     reference     |
    ## UBV  |     5.53   +0.68   +0.20 |3    |           |1953ApJ...117..313J|
    ## UBV  |     5.458  +0.672  +0.245|8    |           |1966PDAUC...1....1G|
    ## UBV  |     5.50   +0.67   +0.20 |3    |           |1966CoLPL...4...99J|
    ## UBV  |     5.52   +0.68   +0.20 |?  1 |           |1971PASP...83...79F|
    ## UBV  |     5.46   +0.61   +0.29 |5    |           |1966ZA.....64..116P|
    ## UBV  |     5.44   +0.67         |10   |           |1957IzKry..17...42N|
    ## UBV  |     5.44   +0.66   +0.22 |6    |           |1966MNRAS.133..475A|
    ## UBV  |(J)  5.48   +0.67   +0.20 |?  1 |           |1964BOTT....3..305J|
    ## UBV  |(J)  5.49   +0.67   +0.20 |?  1 |           |1965CoLPL...3...73J|
    ## UBV  |(J)  5.50   +0.67   +0.20 |3    |           |1966CoLPL...4...99J|
    ## UBV  |(J)  5.48   +0.65   +0.29 |3    |           |1983ApJS...52....7F|
    ## UBV  |(E)  5.46   +0.66   +0.22 |1    |           |1986EgUBV........0M|
    ## UBV  |(*)  5.457  +0.664        |19   |           |1979PASP...91..180M|

    # data header line we expect to see
    my $expectedHeader = "UBV  |      V      B-V     U-B  |Q/nbm|  remarks  |     reference     |";
    my $deferr = 0.05;	#XXX this hardcoded value should be a parameter somewhere
    my $precision = 3;	# computed values will be limited to three places
    my $precisionFmt = "%." . $precision . "f";	# sprintf format string

    # check args
    if (!scalar(@lines))
    {
	gcDebug $dbgname, 1,"no Simbad UBV measurements found for name=$name\n";
	return;	# no UBV measurements
    }

    my $header = 1;	# expecting header line to come first
    line: foreach my $line (@lines)
    {
	chomp($line);
	if ($header) 	# examine header line
	{
	    $header = 0;
	    gcDebug $dbgname, 1, "got header line \"$line\"\n";
	    # complain header doesn't match expected
	    if ($line ne $expectedHeader)
	    {
		print STDERR "$dbgname: WARNING: UBV header line does not match expected format\n";
		gcDebug $dbgname, 1, "expected header line \"$expectedHeader\"\n";
		gcDebug $dbgname, 1, "actual header line   \"$line\"\n";
	    }
	    next line;
	}
	gcDebug $dbgname, 1, "processing line \"$line\"\n";

	# Carve up the line.  Looks like:
	#  |      V      B-V     U-B  |Q/nbm|  remarks  |     reference     |
	my @fields = split /\|/, $line;
	my $mags = trimWhite($fields[1]);
	my $ref = trimWhite($fields[4]);
	$mags =~ s/^\(.\)\s*//g;				# strip leading "(x)" notation if present
	my ($v,$BmV,$UmB,undef) = split /\s+/, $mags;	# separate V, BmV, UmB

	my $vStr = nonNull($v,"?");
	my $BmVStr = nonNull($BmV,"?");
	my $UmBStr = nonNull($UmB,"?");
	gcDebug $dbgname, 1, "read V=$vStr BmV=$BmVStr UmB=$UmBStr\n";

	# V
	if (&isNumeric($v))
	{
	    my $out = "";
	    $out = "#" if ($opts->{noV});
	    $out .= formatPhotRec($name,"Johnson","V",$v,$deferr,"UBV Johnson V=$v $ref",$opts->{units});
	    push @outLines, $out;
	}

	# B
	my $b = undef;
	if (&isNumeric($v) && &isNumeric($BmV))
	{
	    $b = sprintf $precisionFmt, $v + $BmV;	# compute and avoid excess precision
	    my $out = "";
	    $out = "#" if ($opts->{noB});
	    $out .= formatPhotRec($name,"Johnson","B",$b,$deferr,"UBV Johnson B=$b $ref",$opts->{units});
	    push @outLines, $out;
	}

	# U
	if (&isNumeric($b) && &isNumeric($UmB))
	{
	    my $u = sprintf $precisionFmt, $b + $UmB;	# compute and avoid excess precision
	    my $out = "";
	    $out = "#" if ($opts->{noU});
	    $out .= formatPhotRec($name,"Johnson","U",$u,$deferr,"UBV Johnson U=$u $ref",$opts->{units});
	    push @outLines, $out;
	}
    }
    return @outLines;
} ## end processSimbad_UBV  ###############################################

####################################################################
## processSimbad_uvby -- deal with uvby (Stromgren) photometry
####################################################################
sub processSimbad_uvby
{
## declarations procesSimbad_uvby ###################################
    my $name = shift;
    my $opts = shift;
    my $Vmag = shift;
    my @lines = @_;

    my @outLines;	# the output lines we will return
## begin procesSimbad_uvby ##########################################
    my $dbgname = "processSimbad_uvby";
    gcDebug $dbgname, 1, "begin\n";

    # Process and output uvby (Stromgren) photometry from measurement
    # lines returned by SIMBAD, respecting options controlling which
    # bands to output.
    #
    # The uvby photometry values are extracted, and some averages
    # are also calculated and returned. The averages
    # are used elsewhere in conjunction with Hbeta measurements,
    # see processSimbad_Hbeta.
    #
    # Measurement lines are assumed to have a fixed
    # tabular format, with fields separated by vertical bars.
    # The first data line is actually a header line, it is used
    # here only as a quick check to see whether the data
    # format has changed unexpectedly.
    #
    # Section looks like this (first line is header):
    ## uvby1  | b-y    m.e |  m1    m.e |  c1    m.e | nbme|    |     reference     |
    ## uvby1 m| 0.416  0.002| 0.233  0.002| 0.371  0.006|   27|    |1998A&AS..129..431H|
    ## uvby1  | 0.410      | 0.228      | 0.388      |    1|    |1990A&AS...82..531F|
    ## uvby1  | 0.411      | 0.218      | 0.420      |    3|    |1990A&AS...83..467O|
    ## uvby1  | 0.412      | 0.233      | 0.368      |   17|    |1976A&AS...26..155G|
    ## uvby1  | 0.413      | 0.230      | 0.369      |    4|    |1987A&AS...71..421R|
    ## uvby1  | 0.414      | 0.233      | 0.373      |   17|    |1983A&AS...54...55O|
    ## uvby1  | 0.415      | 0.233      | 0.374      |   15|    |1993A&AS..102...89O|
    ## uvby1  | 0.416      | 0.232      | 0.364      |     |    |1970AJ.....75..978C|
    ## uvby1  | 0.416      | 0.232      | 0.364      |     |    |1966AJ.....71..709C|
    ## uvby1  | 0.417      | 0.234      | 0.368      |   30|    |1994A&AS..106..257O|

    # data header line we expect to see:
    my $expectedHeader = "uvby1  | b-y     m.e |  m1     m.e |  c1     m.e | nbme|    |     reference     |";
    my $deferr = 0.08;	#XXX this hardcoded value should be a parameter somewhere
    my $precision = 3;	# computed values will be limited to three places
    my $precisionFmt = "%." . $precision . "f";	# sprintf format string

    # can't continue without valid Vmag
    if (!scalar(@lines))
    {
	gcDebug $dbgname, 1, "no Simbad uvby measurements found for name=$name\n";
	return;	# no uvby measurements
    }
    if (!&isNumeric($Vmag))
    {
	my $vStr = nonNull($Vmag,"undef");
	#gcDebug $dbgname, 1, "can't continue, invalid Vmag=\"$vStr\"\n";
	carp "$dbgname: WARNING: can't continue, invalid Vmag=$vStr\n";	#XXX verbose errors
	return;
    }

    # Calculating average Stromgren quantities
    my $avg_BmY = 0;
    my $avg_m1 = 0;
    my $avg_c1 = 0;
    my $nPts = 0;

    my $header = 1;	# expecting header line to come first
    line: foreach my $line (@lines)
    {
	chomp($line);
	if ($header) 	# examine header line
	{
	    $header = 0;
	    gcDebug $dbgname, 1, "got header line \"$line\"\n";
	    # complain header doesn't match expected
	    if ($line ne $expectedHeader)
	    {
		print STDERR "$dbgname: WARNING: uvby header line does not match expected format\n";
		gcDebug $dbgname, 1, "expected header line \"$expectedHeader\"\n";
		gcDebug $dbgname, 1, "actual header line   \"$line\"\n";
	    }
	    next line;
	}
	gcDebug $dbgname, 1, "processing line \"$line\"\n";

	# Carve up the line.  Looks like:
	## uvby1  | b-y    m.e |  m1    m.e |  c1    m.e | nbme|    |     reference     |
	## uvby1 m| 0.416  0.002| 0.233  0.002| 0.371  0.006|   27|    |1998A&AS..129..431H|
	my @fields = split /\|/, $line;
	my ($BmY,$BmYerr) = split /\s+/, trimWhite($fields[1]);	# separate b-y, m.e.
	my ($m1,$m1err) = split /\s+/, trimWhite($fields[2]);	# separate m1, m.e.
	my ($c1,$c1err) = split /\s+/, trimWhite($fields[3]);	# separate c1, m.e.
	my $rem = trimWhite($fields[5]);
	my $ref = trimWhite($fields[6]);
	$ref = sprintf("%s %s", $rem, $ref) if ($rem);	# combine $rem into $ref

	$BmY = trimWhite($BmY);
	$m1 = trimWhite($m1);
	$c1 = trimWhite($c1);

	my $BmYStr = nonNull($BmY,"?");
	my $m1Str = nonNull($m1,"?");
	my $c1Str = nonNull($c1,"?");

	if (&isNumeric($BmY))
	{
	    if (&isNumeric($m1) && &isNumeric($c1))
	    { # tally sums for means
		$avg_BmY += $BmY;
		$avg_m1 += $m1;
		$avg_c1 += $c1;
		$nPts++;
	    }

	    # derive magnitudes from the Stromgren indices
	    # We know $Vmag and $ BmY are both valid numerics (checked above)
	    my $y = $Vmag;	# treat Stromgren y as equivalent to Johnson V (both 550nm)
	    my $b = sprintf($precisionFmt, $BmY + $y);	# compute b from y and B-Y
	    # now use m1 and c1 to get u and v, where
	    # m1 = (v-b) - (b-y)
	    # c1 = (u-v) - (b-y)
	    my $u = "?";
	    my $v = "?";	# This is Stromgren 'v', not the same as Johnson V
	    $v = sprintf($precisionFmt, $m1 + $b + $b - $y) if &isNumeric($m1);
	    $u = sprintf($precisionFmt, $c1 + $v + $v - $b) if (&isNumeric($c1) && &isNumeric($v));

	    # Compute monochromatic fluxes via the Stromgren indices
	    my $out;
	    # u
	    $out = "";
	    $out = "#" if ($opts->{noU} || !&isNumeric($u));
	    $out .= formatPhotRec($name,"Stromgren","u",$u,$deferr,"Stromgren ubvy u=$u $ref",$opts->{units});
	    push @outLines, $out;

	    # v
	    $out = "";
	    $out = "#" if ($opts->{noV} || !&isNumeric($v));
	    $out .= formatPhotRec($name,"Stromgren","v",$v,$deferr,"Stromgren ubvy v=$v $ref",$opts->{units});
	    push @outLines, $out;

	    # b
	    $out = "";
	    $out = "#" if ($opts->{noB} || !&isNumeric($b));
	    $out .= formatPhotRec($name,"Stromgren","b",$b,$deferr,"Stromgren ubvy b=$b $ref",$opts->{units});
	    push @outLines, $out;

	    # y
	    $out = "";
	    $out = "#" if (!&isNumeric($y));
	    $out .= formatPhotRec($name,"Stromgren","y",$y,$deferr,"Stromgren ubvy y=$y $ref",$opts->{units});
	    push @outLines, $out;
	} # end check $BmY
    } # end loop over lines

    # Compute mean indices
    if ($nPts)
    {
	# take average, use sprintf to avoid excess precision
	$avg_BmY  = sprintf ($precisionFmt, $avg_BmY / $nPts);
	$avg_m1   = sprintf ($precisionFmt, $avg_m1 / $nPts);
	$avg_c1   = sprintf ($precisionFmt, $avg_c1 / $nPts);
    } ## computing mean stromgren indices
    if (gcConfig::debug())
    {
	my $BmYStr = nonNull($avg_BmY,"?");
	my $m1Str = nonNull($avg_m1,"?");
	my $c1Str = nonNull($avg_c1,"?");
	gcDebug $dbgname, 1, "Stromgren averages BmY=$BmYStr m1=$avg_m1 c1=$avg_c1\n";
    }

    return ($avg_BmY, $avg_m1, $avg_c1, $nPts, @outLines);
} ## end procesSimbad_uvby ##########################################

####################################################################
## processSimbad_Hbeta -- process Hbeta measurements to give eff temp
####################################################################
sub processSimbad_Hbeta
{
## declarations procesSimbad_Hbeta ###################################
    my $name = shift;
    my $opts = shift;
    my $BmY= shift;		# Stromgren B-Y
    my $m1 = shift;		# Stromgren m1
    my $c1 = shift;		# Stromgren c1
    my @lines = @_;

    my @outLines;	# the output lines we will return
## begin procesSimbad_Hbeta ##########################################
    my $dbgname = "processSimbad_Hbeta";
    gcDebug $dbgname, 1, "begin $dbgname\n";

    # Estimate the object's effective temperature by combining the Stromgren
    # uvby with Hbeta by techniques described in Balona 1994, MNRAS
    # 268, 119.
    #
    # Section look like this (first line is header):
    ## Hbet1  | Hbet       |nmes |rrrr|     reference     |
    ## Hbet1 m| 2.603  0.00|   12|    |1998A&AS..129..431H|
    ## Hbet1  | 2.603      |    8|    |1988AJ.....95.1558R|
    ## Hbet1  | 2.601      |    1|    |1990A&AS...82..531F|
    ## Hbet1  | 2.603      |    3|    |1966AJ.....71..709C|

    # data header line we expect to see:
    my $expectedHeader = "Hbet1  | Hbet       |nmes |rrrr|     reference     |";
    my $precision = 3;	# computed values will be limited to three places
    my $precisionFmt = "%." . $precision . "f";	# sprintf format string

    # check args
    if (!scalar(@lines))
    {
	gcDebug $dbgname, 1, "no Simbad Hbeta measurements found for name=$name\n";
	return;	# no Hbeta measurements
    }
    if (!&isNumeric($BmY) || !&isNumeric($m1) || !&isNumeric($c1))
    {
	my $BmYStr = nonNull($BmY,"?");
	my $m1Str = nonNull($m1,"?");
	my $c1Str = nonNull($c1,"?");
	#gcDebug $dbgname, 1, "can't continue, invalid Vmag=\"$vStr\"\n";
	#XXX verbose errors for now
	carp "$dbgname: WARNING: can't continue, invalid arg BmY=$BmYStr m1=$m1Str c1=$c1Str\n";
	return;
    }
    gcDebug $dbgname, 1, "args BmY=$BmY m1=$m1 c1=$c1\n";

    # compute an effective temperature estimate
    # first average the hBeta measurement values
    my $hBeta = 0;
    my $nPts = 0;
    my $header = 1;	# expecting header line to come first
    line: foreach my $line (@lines)
    {
	chomp($line);
	if ($header) 	# examine header line
	{
	    $header = 0;
	    gcDebug $dbgname, 1, "got header line \"$line\"\n";
	    # complain header doesn't match expected
	    if ($line ne $expectedHeader)
	    {
		print STDERR "$dbgname: WARNING: header line does not match expected format\n";
		gcDebug $dbgname, 1, "expected header line \"$expectedHeader\"\n";
		gcDebug $dbgname, 1, "actual header line   \"$line\"\n";
	    }
	    next line;
	}
	gcDebug $dbgname, 1, "processing line \"$line\"\n";

	# We want the first value in the second field
	my (undef,$hb,undef) = split /\|/,$line;
	$hb =~ s/^\s+//g;	# remove leading whitespace
	$hb =~ s/\s+.*$//g;	# pick off first word only
	if (&isNumeric($hb))
	{
	    $hBeta += $hb;
	    $nPts++;
	}
    }
    if ($nPts == 0)
    {
	gcDebug $dbgname, 1, "no valid Simbad Hbeta measurements found for name=$name\n";
	return;	# no Hbeta measurements
    }
    $hBeta = sprintf($precisionFmt,$hBeta / $nPts);	# compute the average, avoiding excess precision
    gcDebug $dbgname, 1, "Hbeta nPts=$nPts, average=$hBeta\n";

    my $eT = Hbeta_to_eT($BmY,$m1,$c1,$hBeta);
    if (!&isNumeric($eT))
    {
	gcDebug $dbgname, 1, "Hbeta to eT failed for name=$name, BmY=$BmY, m1=$m1, c1=$c1, Hbeta=$hBeta\n";
	return;
    }

    # we finally have effective temperature, turn it into an output line
    my $out = "";
    if ($eT < 6000.0) # only use ubvy for early-ish type stars
    {
	$out = "#";
    }
    $eT = sprintf "%6.0f", $eT;
    $out .= "T $name $eT # model Effective Temp <= Stromgren photometry\n";
    push @outLines, $out;
    return @outLines;
} ## end procesSimbad_Hbeta ##########################################

####################################################################
## processSimbad_Geneva -- Process Geneva photometry from Simbad
####################################################################
sub processSimbad_Geneva	
{
## declarations procesSimbad_Geneva ###################################
    my $name = shift;
    my $opts = shift;
    my @lines = @_;

    my @outLines;	# the output lines we will return
## begin procesSimbad_Geneva ##########################################

    my $dbgname = "processSimbad_Geneva";
    gcDebug $dbgname, 1, "begin\n";

    # Process and output Geneva photometry from measurement lines returned
    # by SIMBAD, respecting options controlling which bands to output.
    #
    #XXX As with the older version of this from fbolFormat, this code uses hardcoded
    #XXX error values, it does not use the errors from the input data.
    #
    # Measurement lines are assumed to have a fixed tabular format, with
    # fields separated by vertical bars. The first line of input
    # is a header line, it is used here only as a quick check to
    # see whether the output format has changed unexpectedly.
    #
    # Section looks like this (first line is header):
    ## GEN  | U-B  | V-B  | B1-B | B2-B | V1-B | G-B  |Wt : m.e | Vmag |Wt : m.e | Rem |     reference     |
    ## GEN m| 1.462| 0.136| 1.113| 1.286| 0.893| 1.178| 34:0.006| 5.459| 26:0.013|VR   |1976A&AS...26..275R|

    # data header line we expect to see
    my $expectedHeader = "GEN  | U-B  | V-B  | B1-B | B2-B | V1-B | G-B  |Wt : m.e | Vmag |Wt : m.e | Rem |     reference     |";
    my $deferr = 0.08;	#XXX this hardcoded value should be a parameter somewhere
    my $precision = 3;	# computed values will be limited to three places
    my $precisionFmt = "%." . $precision . "f";	# sprintf format string

    # check args
    if (!scalar(@lines))
    {
	gcDebug $dbgname, 1, "no Simbad Geneva measurements found for name=$name\n";
	return;	# no Geneva measurements
    }

    my $header = 1;	# expecting header line to come first
    line: foreach my $line (@lines)
    {
	chomp($line);
	if ($header) 	# examine header line
	{
	    $header = 0;
	    gcDebug $dbgname, 1, "got header line \"$line\"\n";
	    # complain header doesn't match expected
	    if ($line ne $expectedHeader)
	    {
		print STDERR "$dbgname: WARNING: GEN header line does not match expected format\n";
		gcDebug $dbgname, 1, "expected header line \"$expectedHeader\"\n";
		gcDebug $dbgname, 1, "actual header line   \"$line\"\n";
	    }
	    next line;
	}
	gcDebug $dbgname, 1, "processing line \"$line\"\n";

	# Carve up the line.  Looks like:
        #  | U-B  | V-B  | B1-B | B2-B | V1-B | G-B  |Wt : m.e | Vmag |Wt : m.e | Rem |     reference     |
	my @fields = split /\|/, $line;
	my $UmB = trimWhite($fields[1]);
	my $VmB = trimWhite($fields[2]);
	my $B1mB = trimWhite($fields[3]);
	my $B2mB = trimWhite($fields[4]);
	my $V1mB = trimWhite($fields[5]);
	my $GmB = trimWhite($fields[6]);
	my $errC = trimWhite($fields[7]);	#XXX not presently used
	my $V = trimWhite($fields[8]);
	my $errV = trimWhite($fields[9]);	#XXX not presently used
	my $rem = trimWhite($fields[10]);
	my $ref = trimWhite($fields[11]);
	$ref =~ s/\[\d+\]//;	# remove [n] annotation from ref if present (obsolete?)

	my $out;	# workspace for the output line being assembled

	# V
	if (!&isNumeric($V))
	{
	    gcDebug $dbgname, 1, "V not numeric on GEN line \"$line\"\n";
	    next line;	# no more we can do with this line
	}
	gcDebug $dbgname, 1, "name=$name GEN V=$V\n";
	$out = "";
	$out = "#" if ($opts->{noV});
	$out .= formatPhotRec($name,"Geneva","V",$V,$deferr,"Geneva V=$V $ref",$opts->{units});
	push @outLines, $out;

	# compute B, print it later.
	if (!&isNumeric($VmB))
	{
	    gcDebug $dbgname, 1, "V-B not numeric on GEN line \"$line\"\n";
	    next line;	# no more we can do with this line
	}
	gcDebug $dbgname, 1, "name=$name GEN VmB=$VmB\n";
	my $B = sprintf ($precisionFmt, $V - $VmB);	# compute B, avoiding excess precision

	# V1
	if (!&isNumeric($V1mB))
	{
	    gcDebug $dbgname, 1, "V1-B not numeric on GEN line \"$line\"\n";
	}
	else
	{
	    gcDebug $dbgname, 1, "name=$name GEN V1mB=$V1mB\n";
	    my $V1 = sprintf($precisionFmt, $B + $V1mB);  # compute V1 avoiding excess precision
	    $out = "";
	    $out = "#" if ($opts->{noV});
	    $out .= formatPhotRec($name,"Geneva","V1",$V1,$deferr,"Geneva V1=$V1 $ref",$opts->{units});
	    push @outLines, $out;
	}

	# now output B
	$out = "";
	$out = "#" if ($opts->{noB});
	$out .= formatPhotRec($name,"Geneva","B",$B,$deferr,"Geneva B=$B $ref",$opts->{units});
	push @outLines, $out;

	# B1
	if (!&isNumeric($B1mB))
	{
	    gcDebug $dbgname, 1, "B1-B not numeric on GEN line \"$line\"\n";
	}
	else
	{
	    gcDebug $dbgname, 1, "name=$name GEN B1mB=$B1mB\n";
	    my $B1 = sprintf($precisionFmt, $B + $B1mB);  # compute B1 avoiding excess precision
	    $out = "";
	    $out = "#" if ($opts->{noB});
	    $out .= formatPhotRec($name,"Geneva","B1",$B1,$deferr,"Geneva B1=$B1 $ref",$opts->{units});
	    push @outLines, $out;
	}

	# B2
	if (!&isNumeric($B2mB))
	{
	    gcDebug $dbgname, 1, "B2-B not numeric on GEN line \"$line\"\n";
	}
	else
	{
	    gcDebug $dbgname, 1, "name=$name GEN B2mB=$B2mB\n";
	    my $B2 = sprintf($precisionFmt, $B + $B2mB);  # compute B2 avoiding excess precision
	    $out = "";
	    $out = "#" if ($opts->{noB});
	    $out .= formatPhotRec($name,"Geneva","B2",$B2,$deferr,"Geneva B2=$B2 $ref",$opts->{units});
	    push @outLines, $out;
	}

	# U
	if (!&isNumeric($UmB))
	{
	    gcDebug $dbgname, 1, "U-B not numeric on GEN line \"$line\"\n";
	}
	else
	{
	    gcDebug $dbgname, 1, "name=$name GEN UmB=$UmB\n";
	    my $U = sprintf($precisionFmt, $B + $UmB);  # compute U avoiding excess precision
	    $out = "";
	    $out = "#" if ($opts->{noU});
	    $out .= formatPhotRec($name,"Geneva","U",$U,$deferr,"Geneva U=$U $ref",$opts->{units});
	    push @outLines, $out;
	}

	# G
	if (!&isNumeric($GmB))
	{
	    gcDebug $dbgname, 1, "G-B not numeric on GEN line \"$line\"\n";
	}
	else
	{
	    gcDebug $dbgname, 1, "name=$name GEN GmB=$GmB\n";
	    my $G = sprintf($precisionFmt, $B + $GmB);  # compute G avoiding excess precision
	    $out = "";
	    $out .= formatPhotRec($name,"Geneva","G",$G,$deferr,"Geneva G=$G $ref",$opts->{units});
	    push @outLines, $out;
	}
    } # end foreach line
    return @outLines;
} ## end procesSimbad_Geneva ##########################################

####################################################################
## processSimbad_JP11 -- Process JP11 photometry from Simbad
####################################################################
sub processSimbad_JP11	
{
## declarations procesSimbad_JP11 ###################################
    my $name = shift;
    my $opts = shift;
    my @lines = @_;

    my @outLines;	# the output lines we will return
## begin procesSimbad_JP11 ##########################################

    my $dbgname = "processSimbad_JP11";
    gcDebug $dbgname, 1, "begin\n";

    # Process and output JP11 photometry from measurement lines returned
    # by SIMBAD, respecting options controlling which bands to output.
    #
    #XXX As with the older version of this from fbolFormat, this code uses hardcoded
    #XXX error values, it does not use the errors from the input data.
    #
    # Measurement lines are assumed to have a fixed tabular format, with
    # fields separated by vertical bars. The first line of input
    # is a header line, it is used here only as a quick check to
    # see whether the output format has changed unexpectedly.
    #
    # Section looks like this (first line is header):
    ## JP11  | U:360  ( nmes)| B:450  ( nmes)| V:555  ( nmes)| R:670  ( nmes)| I:870  ( nmes)|J:1200  ( nmes)|K:2200  ( nmes)|L:3500  ( nmes)|M:5000  ( nmes)|N:9000  ( nmes)|H:16200 ( nmes)|     reference     |
    ## JP11  |        (     )|        (     )|        (     )|        (     )|        (     )|  4.36  (    7)|  3.99  (    7)|  3.80  (    3)|        (     )|        (     )|        (     )|1968ApJ...152..465J|
    ## JP11  |  6.37  (    3)|  6.17  (    3)|  5.50  (    3)|  4.96  (    3)|  4.62  (    3)|  4.37  (    7)|  3.99  (    7)|  3.78  (?    )|        (     )|        (     )|        (     )|1966CoLPL...4...99J|

    # data header line we expect to see
    my $expectedHeader = "JP11  | U:360  ( nmes)| B:450  ( nmes)| V:555  ( nmes)| R:670  ( nmes)| I:870  ( nmes)|J:1200  ( nmes)|K:2200  ( nmes)|L:3500  ( nmes)|M:5000  ( nmes)|N:9000  ( nmes)|H:16200 ( nmes)|     reference     |";
    my $deferr = 0.05;	#XXX this hardcoded value should be a parameter somewhere

    # check args
    if (!scalar(@lines))
    {
	gcDebug $dbgname, 1, "no Simbad JP11 measurements found for name=$name\n";
	return;	# no measurements
    }

    my $header = 1;	# expecting header line to come first
    line: foreach my $line (@lines)
    {
	chomp($line);
	if ($header) 	# examine header line
	{
	    $header = 0;
	    gcDebug $dbgname, 1, "got header line \"$line\"\n";
	    # complain header doesn't match expected
	    if ($line ne $expectedHeader)
	    {
		print STDERR "$dbgname: WARNING: JP11 header line does not match expected format\n";
		gcDebug $dbgname, 1, "expected header line \"$expectedHeader\"\n";
		gcDebug $dbgname, 1, "actual header line   \"$line\"\n";
	    }
	    next line;
	}
	gcDebug $dbgname, 1, "processing line \"$line\"\n";

	# Carve up the line.  Looks like:
	# JP11  | U:360  ( nmes)| B:450  ( nmes)| V:555  ( nmes)| R:670  ( nmes)| I:870  ( nmes)|J:1200  ( nmes)|K:2200  ( nmes)|L:3500  ( nmes)|M:5000  ( nmes)|N:9000  ( nmes)|H:16200 ( nmes)|     reference     |

	# data field names in order of appearance
	my @fieldNames = ("U","B","V","R","I","J","K","L","M","N","H");
	my @fields = split /\|/, $line;
	my $sys = shift @fields;	# remove first field (system name)
	my $ref = pop @fields;		# reference is the last field
	foreach my $fname (@fieldNames)
	{
	    my $val = shift @fields;	# pick off next value field

	    # clean up a few things
	    $val =~ s/\(.+\)//g;	# Eliminate the nmes field
	    $val =~ s/\~//g;		# Eliminate any null field
	    $val =~ s/\s+//g;		# remove extra whitespace
	    if (&isNumeric($val))
	    {
		gcDebug $dbgname, 1, "fname=$fname val=$val\n";
		my $out = "";
		$out = "#" if (
		    (($opts->{noV}) && ($fname eq "V")) ||
		    (($opts->{noB}) && ($fname eq "B")) ||
		    (($opts->{noU}) && ($fname eq "U"))
		    );
		$out .= formatPhotRec($name,"Johnson",$fname,$val,$deferr,"JP11 Johnson $fname=$val  $ref",$opts->{units});
		push @outLines, $out;
	    }
	}
    } # end foreach line
    return @outLines;
} ## end procesSimbad_JP11 ##########################################

####################################################################
## processSimbad_IRC -- Process IRC photometry from Simbad
####################################################################
sub processSimbad_IRC	
{
## declarations procesSimbad_IRC ###################################
    my $name = shift;
    my $opts = shift;
    my @lines = @_;

    my @outLines;	# the output lines we will return
## begin procesSimbad_IRC ##########################################

    my $dbgname = "processSimbad_IRC";
    gcDebug $dbgname, 1, "begin\n";

    # Process and output IRC photometry from measurement lines returned
    # by SIMBAD, respecting options controlling which bands to output.
    #
    #XXX As with the older version of this from fbolFormat, this code uses hardcoded
    #XXX error values, it does not use the errors from the input data.
    #
    # Measurement lines are assumed to have a fixed tabular format, with
    # fields separated by vertical bars. The first line of input
    # is a header line, it is used here only as a quick check to
    # see whether the output format has changed unexpectedly.
    #
    # Section looks like this (first line is header):
    ## IRC  |  K-mag(mes)m.e :chi-2|  I-mag(mes)m.e :chi-2|     reference      R|
    ## IRC m|   2.50(  4)0.07: 0.25|   3.49(  4)0.05: 0.75|1969IRC...C......0N  |

    # data header line we expect to see
    my $expectedHeader = "IRC  |  K-mag(mes)m.e :chi-2|  I-mag(mes)m.e :chi-2|     reference      R|";
    my $deferr = 0.05;	#XXX this hardcoded value should be a parameter somewhere

    # check args
    if (!scalar(@lines))
    {
	gcDebug $dbgname, 1, "no Simbad IRC measurements found for name=$name\n";
	return;	# no Geneva measurements
    }

    my $header = 1;	# expecting header line to come first
    line: foreach my $line (@lines)
    {
	chomp($line);
	if ($header) 	# examine header line
	{
	    $header = 0;
	    gcDebug $dbgname, 1, "got header line \"$line\"\n";
	    # complain header doesn't match expected
	    if ($line ne $expectedHeader)
	    {
		print STDERR "$dbgname: WARNING: IRC header line does not match expected format\n";
		gcDebug $dbgname, 1, "expected header line \"$expectedHeader\"\n";
		gcDebug $dbgname, 1, "actual header line   \"$line\"\n";
	    }
	    next line;
	}
	gcDebug $dbgname, 1, "processing line \"$line\"\n";

	#XXX SG 2/07 I don't understand why this is here. Orig comment said:
	## Having issues with quantities without leading zeros
	#$line =~ s/\-\s\./\-0\./g; #XXX not needed?

	# Carve up the line.  Looks like:
	# IRC m|   2.50(  4)0.07: 0.25|   3.49(  4)0.05: 0.75|1969IRC...C......0N  |
	my @fields = split /\|/, $line;		# split on the vert bars
	my $Kmeas = trimWhite($fields[1]);	# K measurement
	my $Imeas = trimWhite($fields[2]);	# I measurement
	my $ref = trimWhite($fields[3]);	# reference

	gcDebug $dbgname, 2, "Kmeas=\"$Kmeas\", Imeas=\"$Imeas\"\n";

	# process the K-band measurement
	$Kmeas =~ s/\:.*$//g;			# truncate at colon
	$Kmeas =~ s/\(.+\)/ /g;           	# replace nmeas with whitespace
	$Kmeas = trimWhite($Kmeas);		# remove leading&trailing whitespace
	my ($Kmag, $Kerr) = split /\s+/,$Kmeas;	# split on internal whitespace
	if (&isNumeric($Kmag))
	{
	    $Kerr = $deferr if (!&isNumeric($Kerr));	# if err not present use default
	    gcDebug $dbgname, 2, "I mag=$Kmag err=$Kerr\n";
	    my $out = formatPhotRec($name,"Johnson","K",$Kmag,$Kerr,"IRC Johnson K=$Kmag  $ref",$opts->{units});
	    push @outLines, $out;
	}

	# process the I-band measurement
	$Imeas =~ s/\:.*$//g;			# truncate at colon
	$Imeas =~ s/\(.+\)/ /g;           	# replace nmeas with whitespace
	$Imeas = trimWhite($Imeas);		# remove leading&trailing whitespace
	my ($Imag, $Ierr) = split /\s+/,$Imeas;	# split on internal whitespace
	if (&isNumeric($Imag))
	{
	    $Ierr = $deferr if (!&isNumeric($Ierr));	# if err not present use default
	    gcDebug $dbgname, 2, "I mag=$Imag err=$Ierr\n";
	    my $out = formatPhotRec($name,"Johnson","I",$Imag,$Ierr,"IRC Johnson I=$Imag  $ref",$opts->{units});
	    push @outLines, $out;
	}
    } # end foreach line
    return @outLines;
} ## end procesSimbad_IRC ##########################################

####################################################################
## processSimbad_IRAS -- Process IRAS photometry from Simbad
####################################################################
sub processSimbad_IRAS	
{
## declarations procesSimbad_IRAS ###################################
    my $name = shift;
    my $opts = shift;
    my @lines = @_;

    my @outLines;	# the output lines we will return
## begin procesSimbad_IRAS ##########################################

    my $dbgname = "processSimbad_IRAS";
    gcDebug $dbgname, 1, "begin\n";

    # Process and output IRAS photometry from measurement lines returned
    # by SIMBAD, respecting options controlling which bands to output.
    #
    # Measurement lines are assumed to have a fixed tabular format, with
    # fields separated by vertical bars. The first line of input
    # is a header line, it is used here only as a quick check to
    # see whether the output format has changed unexpectedly.
    #

    # XXX OLD FORMAT:
    ## |  ra, dec (1950)    | err.ellip|     f12      f25      f60     f100 | %flux.err. |conf| v|ns|na|  reference        |
    ## ----------------------------------------------------------------------------------------------------------------------
    ## |07 14 56.5 +01 11 09| 25  5  99|   26.75    11.56     2.00     1.07:|  6  7 12 12|----|99| 3| 6|[42]1988NASAR1190....1B|

    # XXX NEW FORMAT (SIMBAD4 3/07):
    # Section looks like this (first line is header):
    ## IRAS  |RA (1950)  DEC (1950|err    ang|  f12      f25      f60      f100   |e12e25e60e10|conf|v |ns|na|     reference     |
    ## IRAS  |22 04 42.1 +25 06 03| 36  6  61|    3.70     0.82     0.40L    1.00L|  6 12      |----| 6| 0| 4|1988NASAR1190....1B|

    # data header line we expect to see
    my $expectedHeader = "IRAS  |RA (1950)  DEC (1950|err    ang|  f12      f25      f60      f100   |e12e25e60e10|conf|v |ns|na|     reference     |";

    # check args
    if (!scalar(@lines))
    {
	gcDebug $dbgname, 1, "no Simbad IRAS measurements found for name=$name\n";
	return;	# no Geneva measurements
    }

    my $header = 1;	# expecting header line to come first
    line: foreach my $line (@lines)
    {
	chomp($line);
	if ($header) 	# examine header line
	{
	    $header = 0;
	    gcDebug $dbgname, 1, "got header line \"$line\"\n";
	    # complain header doesn't match expected
	    if ($line ne $expectedHeader)
	    {
		print STDERR "$dbgname: WARNING: IRAS header line does not match expected format\n";
		gcDebug $dbgname, 1, "expected header line \"$expectedHeader\"\n";
		gcDebug $dbgname, 1, "actual header line   \"$line\"\n";
	    }
	    next line;
	}
	gcDebug $dbgname, 1, "processing line \"$line\"\n";

	#XXX SG 2/07 I don't understand why this is here. Orig comment said:
	## Having issues with quantities without leading zeros
	## (negative numerics 0.0 <x < -1.0 ?)
	#$line =~ s/\-\s\./\-0\./g; #XXX not needed?

	# Carve up the line.  Looks like:
	## IRAS  |RA (1950)  DEC (1950|err    ang|  f12      f25      f60      f100   |e12e25e60e10|conf|v |ns|na|     reference     |
	## IRAS  |22 04 42.1 +25 06 03| 36  6  61|    3.70     0.82     0.40L    1.00L|  6 12      |----| 6| 0| 4|1988NASAR1190....1B|

	my @fields = split /\|/, $line;		# split on the vert bars
	my $fluxes = trimWhite($fields[3]);	# IRAS flux measurements
	my $ref = trimWhite($fields[9]);	# reference

	my ($f12, $f25, $f60, $f100, $rest) = split /\s+/, $fluxes;
	gcDebug $dbgname, 2,
	    " name=$name, IRAS fluxes:",
	    " f12=\"",nonNull($f12,"?"),"\"",
	    " f25=\"",nonNull($f25,"?"),"\"",
	    " f60=\"",nonNull($f60,"?"),"\"",
	    " f100=\"",nonNull($f100,"?"),"\"";

	# process the 12/25/60 um fluxes in turn

	# 12um
	if (($f12 =~ m/\d+/)                  # If this guy has numeric content
	    && !($f12 =~ m/L$/))               # and isn't just a lower-limit...
	{
	    $f12 = numberOnly($f12);		# remove any non-numeric stuff at the end
	    my $err = $f12 / 10.0;		# XXX hack? carried over from fbolFormat
	    if ($opts->{units} == photometry::UNITS_FLAMBDA)
	    {  ## Convert to Flambda
		## Converting flux from Jy (10^-26 W / m^2 / Hz) to
		## erg / s / cm^2 / um 
		my ($fl,$efl) = &photometry::fNuToFLambda(12000,$f12,$err);
		my $out = sprintf "Fl $name 12000 2000 %.3g %.2g  # IRAS 12um flux  $ref\n",$fl,$efl;
		push @outLines, $out;
	    } ## end converting to Flambda
	    else
	    {
		## Output in native Fnu (Jy) units
		my $out = sprintf "Fn $name 12000 2000 $f12 $err  # IRAS 12um flux  $ref\n";
		push @outLines, $out;
	    }
	} # processing 12um flux

	# 25um
	if (($f25 =~ m/\d+/)                  # If this guy has numeric content
	    && !($f25 =~ m/L$/))               # and isn't just a lower-limit...
	{
	    $f25 = numberOnly($f25);		# remove any non-numeric stuff at the end
	    my $err = $f25 / 10.0;		# XXX hack? carried over from fbolFormat
	    if ($opts->{units} == photometry::UNITS_FLAMBDA)
	    {  ## Convert to Flambda
		## Converting flux from Jy (10^-26 W / m^2 / Hz) to
		## erg / s / cm^2 / um 
		my ($fl,$efl) = &photometry::fNuToFLambda(25000,$f25,$err);
		my $out = sprintf "Fl $name 25000 8000 %.3g %.2g  # IRAS 25um flux  $ref\n",$fl,$efl;
		push @outLines, $out;
	    } ## end converting to Flambda
	    else
	    {
		## Output in native Fnu (Jy) units
		my $out = sprintf "Fn $name 25000 8000 $f25 $err  # IRAS 25um flux  $ref\n";
		push @outLines, $out;
	    }
	} # processing 25um flux

	# 60um
	if (($f60 =~ m/\d+/)                  # If this guy has numeric content
	    && !($f60 =~ m/L$/))               # and isn't just a lower-limit...
	{
	    $f60 = numberOnly($f60);		# remove any non-numeric stuff at the end
	    my $err = $f60 / 10.0;		# XXX hack? carried over from fbolFormat
	    if ($opts->{units} == photometry::UNITS_FLAMBDA)
	    {  ## Convert to Flambda
		## Converting flux from Jy (10^-26 W / m^2 / Hz) to
		## erg / s / cm^2 / um 
		my ($fl,$efl) = &photometry::fNuToFLambda(60000,$f60,$err);
		my $out = sprintf "Fl $name 60000 12000 %.3g %.2g  # IRAS 60um flux  $ref\n",$fl,$efl;
		push @outLines, $out;
	    } ## end converting to Flambda
	    else
	    {
		## Output in native Fnu (Jy) units
		my $out = sprintf "Fn $name 60000 12000 $f60 $err  # IRAS 60um flux  $ref\n";
		push @outLines, $out;
	    }
	} # processing 60um flux
    } # end foreach line
    return @outLines;
} ## end procesSimbad_IRAS ##########################################

# format a magnitude value for printing
sub magval
{
    my ($lbl, $magname, $magval, $errval) = @_;
    my $mv = "$lbl ";
    if (&isNumeric($magval)) {
        $mv .= sprintf("%s=%g ", $magname, $magval);
    } else {
        $mv .= sprintf("%s=? ", $magname);
    }
    if (&isNumeric($errval)) {
        $mv .= sprintf("+/- %g", $errval);
    } elsif (defined($errval)) {
        $mv .= "+/- ?";
    }
    return $mv;
}


#########################################################################
## do2MassPhot -- extract and report photometry from 2MASS (via IRSA)
#########################################################################
sub do2MassPhot
{
    my $name = shift;		# target name to use, with "--spectype" suffix attached
    my $opts = shift;		# ref to options hash
    my $simbadObj = shift;	# simbadresult object

    my $dbgname = "do2MassPhot";

    my $sName = nameOnly($name);	# remove any spectype suffix 
    $sName = normalIdent($sName);	# make it usable for lookups (e.g. remove underscores)

    gcDebug $dbgname,1,"doing IRSA 2Mass lookup for desig \"$name\"\n";
#    my ($data, $result) = search_irsa_nl({catalog => "fp_psc",
#				       cols => [qw/k_m k_cmsig j_m j_cmsig h_m h_cmsig/],
#				       radius => $cfg->TwoMassSearchRadius()},
#				       $sName,
#				       @{$opts->{altTargets}}
#				       );

    # search 2Mass using ra/dec position from SIMBAD
    my $raHMS = $simbadObj->raHMS();
    $raHMS =~ s/\s+/:/g;	# colon-separated H:M:S
    my $decDMS = $simbadObj->decDMS();
    $decDMS =~ s/\s+/:/g;	# colon-separated D:M:S
    my ($data, $result) = search_irsa({catalog => "fp_psc",
	    cols => [qw/k_m k_cmsig j_m j_cmsig h_m h_cmsig/],
	    radius => $cfg->TwoMassSearchRadius(),
	    targetRA => $raHMS,
	    targetDEC => $decDMS,
	    });


    gcDebug $dbgname,1,"IRSA 2Mass lookup returned $result\n";
    if ($result < 0)
    {
	carp "$dbgname: IRSA 2Mass lookup failure for desig \"$name\"\n" if ($result < 0);
	return;		# error in lookup
    }
    elsif ($result > 0)
    {
	return;		# empty results
    }

    my $r = $data->[0];

    my @outLines;
    push @outLines, formatPhotRec($name, "2Mass", "Ks", $r->{k_m}, $r->{k_cmsig},
		magval("2Mass", "Ks", $r->{k_m}, $r->{k_cmsig}),$opts->{units});
    push @outLines, formatPhotRec($name, "2Mass", "J", $r->{j_m}, $r->{j_cmsig},
		magval("2Mass", "J", $r->{j_m}, $r->{j_cmsig}),$opts->{units});
    push @outLines, formatPhotRec($name, "2Mass", "H", $r->{h_m}, $r->{h_cmsig},
		magval("2Mass", "H", $r->{h_m}, $r->{h_cmsig}),$opts->{units});

    return @outLines;
}

#########################################################################
## doHipPhot -- extract and report photometry from Hipparcos catalog
#########################################################################
sub doHipPhot
{
    my $name = shift;		# target name to use, with "--spectype" suffix attached
    my $opts = shift;		# ref to options hash
    my $simbadObj = shift;	# simbadresult object

    my $dbgname = "doHipPhot";


    if (! ($name =~ m/^(HD|HIP)/))
    {
	# name isn't HD or HIP, this just isn't going to work.
	gcDebug $dbgname,1,"can't continue, name isn't HD or HIP: \"$name\"\n";
	return;
    }

    ## get record from Hipparcos catalog
    gcDebug $dbgname,1,"doing Hipparcos lookup for desig \"$name\"\n";
    my $sName = nameOnly($name);
    my @pieces = identPieces($sName);
    my $hRecord = getHipRecord($pieces[0],$pieces[1]);
    if (!defined $hRecord)
    {
	gcDebug $dbgname,1,"can't continue, Hipparcos lookup failed for desig \"$name\"\n";
	return;
    }

    ## make it an object for easy access, then pull out needed fields
    my $hipLine = new hipparcosLine(line => $hRecord);
    my $BTycho = $hipLine->extractField("BTycho");
    my $BTychoErr = $hipLine->extractField("BTychoErr");
    my $VTycho = $hipLine->extractField("VTycho");
    my $VTychoErr = $hipLine->extractField("VTychoErr");
    my $Hpmag = $hipLine->extractField("Hpmag");
    my $HpmagErr = $hipLine->extractField("HpmagErr");

    ## Create the photometry lines
    my @outLines;

    # TychoB
    if (&isNumeric($BTycho))
    {
	my $out = "";
	$out = "#" if ($opts->{noV});
	$out .= formatPhotRec($name,"Tycho","Bt",$BTycho,$BTychoErr, "Tycho Bt magnitude",$opts->{units});
	push @outLines, $out;
    }

    # TychoV
    if (&isNumeric($VTycho))
    {
	my $out = "";
	$out = "#" if ($opts->{noV});
	$out .= formatPhotRec($name,"Tycho","Vt",$VTycho,$VTychoErr, "Tycho Vt magnitude",$opts->{units});
	push @outLines, $out;
    }

    # Hipparcos Hp
    if (&isNumeric($Hpmag))
    {
	my $out = formatPhotRec($name,"Hipparcos","Hp",$Hpmag,$HpmagErr, "Hipparcos Hp magnitude",$opts->{units});
	push @outLines, $out;
    }

    return @outLines;
}

#########################################################################
## doGezariPhot -- extract and report photometry from CIO/Gezari catalog
#########################################################################
sub doGezariPhot
{
    my $name = shift;		# target name to use, with "--spectype" suffix attached
    my $opts = shift;		# ref to options hash
    my $simbadObj = shift;	# simbadresult object

    my $dbgname = "doGezariPhot";

    my $sName = nameOnly($name);	# remove any spectype suffix 
    $sName = normalIdent($sName);	# make it usable for lookups (e.g. remove underscores)

    gcDebug $dbgname,1,"doing IRSA 2Mass lookup for desig \"$name\"\n";

    # search CIO using B1950 (not J2000) ra/dec position from SIMBAD
    my $raHMS = $simbadObj->raHMS1950();
    $raHMS =~ s/\s+/:/g;	# colon-separated H:M:S
    my $decDMS = $simbadObj->decDMS1950();
    $decDMS =~ s/\s+/:/g;	# colon-separated D:M:S
    my $arg = "-RA $raHMS   -Dec $decDMS";
    gcDebug $dbgname,1,"running searchCIO with arg \"$arg\"\n";
    my @cioRecs = `searchCIO $arg`;
    if (@cioRecs)
    {
	gcDebug $dbgname,1,"searchCIO returned records:\n";
	foreach my $rec (@cioRecs)
	{
	    gcDebug $dbgname,1,"CIO record: $rec";
	}
    }
    else
    {
	gcDebug $dbgname,1,"searchCIO returned no results for name=$name, ra1950=$raHMS dec1950=$decDMS\n";
	return;
    }

    # process the CIO lines and reformat for fbol
    my @outLines;
    rec: foreach my $rec (@cioRecs)
    {
	chomp($rec);
	gcDebug $dbgname,1,"processing CIO record $rec\n";
	if ($rec =~ m/\S+/)
	{
	    my $lambda = substr $rec, 37, 7;
	    my $flux = substr $rec, 44, 5;
	    my $unit = substr $rec, 49, 2;
	    my $biblio = substr $rec, 51, 6;

	    if (!&isNumeric($lambda) ||
		!&isNumeric($flux) ||
		(($unit ne "M ") &&
		 ($unit ne "C ") &&
		 ($unit ne "J "))
		 )
	    {
		# problem parsing this line, ignore it and move on
		gcDebug $dbgname,0,"can't parse record (ignoring): $rec\n";
		next rec;
	    }

	    if (($unit eq "M ") || ($unit eq "C "))
	    {
		my $wl = $lambda * 1000.0;  ## nm
		my $pb = 0.2 * $wl;  ## fake passband
		my $out = "";
		if ($wl > 3500.0)
		{
		    $out = "#";
		}
		if ($opts->{units} == photometry::UNITS_FLAMBDA)
		{
		    #XXX not implemented
		    ## convert magnitudes to fLambda
		    gcDebug $dbgname,0,"fLambda conversion not implemented for CIO measurements\n";
		    carp "$dbgname: CIO fLambda conversion not implemented yet\n";
		}
		elsif ($opts->{units} == photometry::UNITS_FNU)
		{
		    #XXX not implemented
		    ## convert magnitudes to fNu
		    gcDebug $dbgname,0,"fNu conversion not implemented for CIO measurements\n";
		    carp "$dbgname: CIO fNu conversion not implemented yet\n";
		}
		else
		{
		    my $fluxErr = 0.05;	#XXX hardcoded error value
		    $out .= "M $name $wl $pb $flux $fluxErr  # CIO magnitude entry -- bibcode $biblio\n";
		    push @outLines, $out;
		}
	    } # end magnitude check

	    if ($unit eq "J ")
	    {
		my $wl = $lambda * 1000.0;  ## nm
		my $pb = 0.2 * $wl;  ###XXX  fake passband
		my $fluxErr = sprintf "%g", 0.1 * $flux;  ###XXX fake fluxerr
		my $out = "";
		if ($wl > 3500.0)
		{
		    $out .=  "#";
		}
		$out .= "F $name $wl $flux $fluxErr  # CIO Jansky flux entry -- bibcode $biblio\n";
		push @outLines, $out;
	    } ## end Jansky flux check
	}
    }
    return @outLines;
}


# Walk through list of measurement data lines returned by SIMBAD,
# and sort them into groups according to their measurement system.
# Return a hash with an entry for each measurement system. Each
# entry is an array of lines for that system.
sub sortMeasurements
{
    my @measLines = @_;

    my $dbgname = "sortMeasurements";

    my %meas;
    my $sysname = undef;
    foreach my $line (@measLines)
    {
	# Each line is either a measurement system name for the
	# data lines to follow, or a data line.

        # JSH 2011-3-22
	# NEW looking for a "|" in the line
        if (($line =~ m/^\w+|/) && ($line =~ m/\S/))
        {
           my $systemName = $line;
           $systemName =~ s/\W.*//;
           # Saw a measurement system name and the line wasn't blank, so save it
            push @{$meas{$systemName}}, $line;
        }

        # ORIG looking for a ":" in the line
        elsif ($line =~ m/^\w+:/)
        {
            # This line is the name of a measurement system for lines to follow.
            $sysname = $line;
            $sysname =~ s/:.*$//;       # remove first colon and everything beyond it
        }
        elsif (defined($sysname) && ($line =~ m/\S/))
        {
            # This is a data line.
            # we saw a measurement system name and the line wasn't blank, so save it
            push @{$meas{$sysname}}, $line;
        }
        else
        {
            # Don't know what this is.
            carp "$dbgname: ignoring unrecognized measurement line \"$line\"\n";
        }

    }
    return %meas;
}

# convert uvby and Hbeta to effective temp
# Algorithm according to Balona, MNRAS 268, 119.
sub Hbeta_to_eT
{
    my $BmY = shift;
    my $m1 = shift;
    my $c1 = shift;
    my $hBeta = shift;

    my $dbgname = "Hbeta_to_eT";

    if (!&isNumeric($BmY) ||
	!&isNumeric($m1) ||
	!&isNumeric($c1) ||
	!&isNumeric($hBeta))
    {
	carp $dbgname,"non-numeric argument\n";
	return undef;
    }
    gcDebug $dbgname, 1, "args BmY=$BmY m1=$m1 c1=$c1 Hbeta=$hBeta\n";

    if (($hBeta <= 2.5) || ($c1 <= -0.25))	# protect the logs we'll do below
    {
	gcDebug $dbgname, 1, "value too small to use, Hbeta=$hBeta, c1=$c1\n";
	return undef;
    }

    # First rough guess on theta (5040/eT)....
    my $theta = -0.5548 + 1.2816 * $BmY
	- 0.4357 * $m1 + 0.1061 * $c1
	    + 0.3607 * $hBeta;

    my $c = log($c1 + 0.250) / log(10.0);
    my $b = log($hBeta - 2.5000) / log(10.0);

    # Now implement the classification scheme, early, late,
    # intermediate...
    if ((5040 / $theta) > 9500)
    { # early group, Balona eq 1
	$theta = 0.7515 + 0.2915 * $c + 0.4875 * $c * $c
	    + 0.1216 * $c * $c * $c
	    + 1.0306 * $b + 0.7958 * $b * $b
	    - 0.4392 * $b * $c;
    }
    elsif ((5040 / $theta) < 7900)
    { # late group, Balona eq 3
	$theta = -0.1030
	    + 0.0513 * $c
	    + 2.6313 * $c * $c
	    - 3.2892 * $c * $c * $c
	    - 2.8581 * $b
	    - 3.9096 * $b * $b
	    - 2.2154 * $b * $b * $b
	    + 0.5443 * $b * $c
	    + 3.7933 * $b * $c * $c
	    + 1.0161 * $b * $b * $c;
    }
    else
    { # middle group, Balona (no eq # ???)
	my $a = 1.36 * $BmY + 0.36 * $m1
	    + 0.18 * $c1 - 0.2448;
	my $r = 0.35 * $c1 - 0.07 * $BmY
	    - $hBeta + 2.565;

	$theta = 0.5236 + 0.9594 * $a - 1.622 * $a * $a
	    - 0.0817 * $r * $r + 0.3087 * $a * $r;
    }

    my $eT = 5040.0 / $theta;  # definition of theta

    return $eT;
}

# remove any "--spectype" suffix from object name
sub nameOnly
{
    my $name = shift;

    $name =~ s/\s*\-\-.*$//g;	# remove '--' and anything after that
    return $name;
}
