#! @PERL@
# -*- perl -*-
# @configure_input@

our $MODVERSION = '$Id: searchCIO.pl,v 1.8 2008/11/12 23:26:50 sgroom Exp $';

##########################################################################
# NASA Exoplanet Science Institute (NExScI) -- http://nexsci.caltech.edu #
#       Copyright (c) 1997-2008 California Institute of Technology       #
#                     U.S. Government Sponsorship                        #
# Under contract with the National Aeronautics and Space Administration  #
##########################################################################

use strict;
use warnings;
use FindBin qw($RealBin);
use lib "$RealBin/../lib/modules";		# location of getCal Perl modules
use gcConfig;
use miscArgs;
use Getopt::Long;
use POSIX;
use coordinates;
use angleBetween;

my $cfg = gcConfig::config();

## searchCIO -- search the Catalog of Infrared Observations (CIO) for
## observations from a given location/source

# program options
my $searchRA;
my $searchDEC;
my $searchRadius = 5.0;  ## default search radius (arcsec)


## 1) Process input arguments

# Setup GetOptions to do option passthrough
Getopt::Long::config("pass_through");
handleCommonOpts(1);
my $result = GetOptions("RA=s" => \$searchRA,
		     "ra=s" => \$searchRA,
		     "Dec=s" => \$searchDEC,
		     "dec=s" => \$searchDEC,
		     "Radius=f" => \$searchRadius,
		     "radius=f" => \$searchRadius);

$searchRadius /= 3600.0; ## Convert to degrees

## convert RA/Dec coordinates
if ($searchRA)
{
    if ($searchRA =~ m/\:|h|m|s|_|\s+/)  ## if $searchRA has a hms format
    {
	$searchRA = RAdeg($searchRA);  ## translate into degrees
    }
}
else
{
    die "Error -- must have search RA\n";
}


# Dec
if ($searchDEC)
{
    if ($searchDEC =~ m/\:|d|m|s|_|\s+/)  ## if $searchDEC has a dms format
    {
	$searchDEC = DECdeg($searchDEC);  ## translate into degrees
    }
}
else
{
    die "Error -- must have a search Dec\n";
}


## 2) Determine which CIO files to search (they are split by RA)
my @files;  ## The list of files to search

my @ra = split /\s+/, RAhms($searchRA);
my $filename = join '.',$ra[0],"dat";
$filename = join '/', $cfg->cioCatalogPath,$filename;
push @files, $filename;


## add an arcminute and see if that pushed us over a boundary
@ra = split /\s+/, RAhms($searchRA + (1.0 / 60.0));
$filename = join '.',$ra[0],"dat";
if (!(grep /$filename/, @files))
{
    $filename = join '/', $cfg->cioCatalogPath,$filename;
    push @files, $filename;
}

## subtract an arcminute and see if that pushed us over a boundary
if (($searchRA - (1.0 / 60.0)) > 0.0)
{
    @ra = split /\s+/, RAhms($searchRA - (1.0 / 60.0));
    $filename = join '.',$ra[0],"dat";
    if (!(grep /$filename/, @files))
    {
	$filename = join '/', $cfg->cioCatalogPath,$filename;
	push @files, $filename;
    }
}
else
{
    $filename = join '.',"23","dat";
    if (!(grep /$filename/, @files))
    {
	$filename = join '/', $cfg->cioCatalogPath,$filename;
	push @files, $filename;
    }
}



## 3) Do a grep the identify "interesting" lines
my $grepStr = &constructGrepStr($searchRA,$searchDEC);
my @lines;
foreach $filename (@files)
{
    open(FILE,$filename) or die "Can't open CIO file $filename\n";
    push @lines, grep {/$grepStr/} <FILE>;
} ## end foreach


##print @lines;


## 4) Now with these "interesting" lines do the precision search
foreach my $line (@lines)
{
    my $raStr = substr $line, 14, 10;
    my $decStr = substr $line, 24, 9;

    my $sep = angleBetween($searchRA,$searchDEC,
			   RAdeg($raStr),DECdeg($decStr));

    if ($sep < $searchRadius)
    {
	print $line;
    }

}

exit(0); ## end searchCIO




sub constructGrepStr
{

    ## constructGrepStr constructs a tedious grep string

    my $arcMin = 1.0 / 60.0;  ## arcmin in degrees

    my ($searchRA,$searchDEC) = @_;

    my (@ra,@dec,$grepStr);
    
    @ra = split /\s+/, RAhms($searchRA);
    @dec = split /\s+/, DECdms($searchDEC);
    $grepStr = "$ra[0] $ra[1] .{4}\\$dec[0] $dec[1]";
    @dec = split /\s+/, DECdms($searchDEC + $arcMin);
    if (!($grepStr =~ m/$ra[0] $ra[1].$dec[0] $dec[1]/)) ## Check for redundancy
    {
	$grepStr .= "|$ra[0] $ra[1] .{4}\\$dec[0] $dec[1]";
    }
    @dec = split /\s+/, DECdms($searchDEC - $arcMin);
    if (!($grepStr =~ m/$ra[0] $ra[1].$dec[0] $dec[1]/))
    {
	$grepStr .= "|$ra[0] $ra[1] .{4}\\$dec[0] $dec[1]";
    }

    
    @ra = split /\s+/, RAhms($searchRA + $arcMin);
    @dec = split /\s+/, DECdms($searchDEC);
    if (!($grepStr =~ m/$ra[0] $ra[1].$dec[0] $dec[1]/))
    {
	$grepStr .= "|$ra[0] $ra[1] .{4}\\$dec[0] $dec[1]";
    }
    @dec = split /\s+/, DECdms($searchDEC + $arcMin);
    if (!($grepStr =~ m/$ra[0] $ra[1].$dec[0] $dec[1]/))
    {
	$grepStr .= "|$ra[0] $ra[1] .{4}\\$dec[0] $dec[1]";
    }
    @dec = split /\s+/, DECdms($searchDEC - $arcMin);
    if (!($grepStr =~ m/$ra[0] $ra[1].$dec[0] $dec[1]/))
    {
	$grepStr .= "|$ra[0] $ra[1] .{4}\\$dec[0] $dec[1]";
    }


    $searchRA -= $arcMin;
    if ($searchRA < 0)
    {
	$searchRA += 360.0;
    }
    @ra = split /\s+/, RAhms($searchRA);
    @dec = split /\s+/, DECdms($searchDEC);
    if (!($grepStr =~ m/$ra[0] $ra[1].$dec[0] $dec[1]/))
    {
	$grepStr .= "|$ra[0] $ra[1] .{4}\\$dec[0] $dec[1]";
    }
    @dec = split /\s+/, DECdms($searchDEC + $arcMin);
    if (!($grepStr =~ m/$ra[0] $ra[1].$dec[0] $dec[1]/))
    {
	$grepStr .= "|$ra[0] $ra[1] .{4}\\$dec[0] $dec[1]";
    }
    @dec = split /\s+/, DECdms($searchDEC - $arcMin);
    if (!($grepStr =~ m/$ra[0] $ra[1].$dec[0] $dec[1]/))
    {
	$grepStr .= "|$ra[0] $ra[1] .{4}\\$dec[0] $dec[1]";
    }

    return $grepStr;

}  ## end constructGrepStr
