#! @PERL@
# -*- perl -*-
# @configure_input@

our $MODVERSION = '$Id: getCal.pl,v 1.93 2008/11/12 23:28:59 sgroom Exp $';

##########################################################################
# NASA Exoplanet Science Institute (NExScI) -- http://nexsci.caltech.edu #
#       Copyright (c) 1997-2008 California Institute of Technology       #
#                     U.S. Government Sponsorship                        #
# Under contract with the National Aeronautics and Space Administration  #
##########################################################################

use strict;		## enable strict error checks
use warnings;
use FindBin qw($RealBin);
use lib "$RealBin/../lib/modules";		# location of getCal Perl modules
use Getopt::Long;
use gcConfig;
use gcDebug;
use coordinates;
use hipparcosLine;
use hipRecord;
use internalLine;
use findScript;
use angles;
use miscString;
use gcSimbad;
use simbadresult;	# testing replacement for Astro::SIMBAD::Result
use astroCatalogue;
use formatConv;
use miscArgs;                       # handlecommonOpts
use angleBetween;
use calCheck;                      # calSearch() and calCheck()
use formatPhotometry;		# collect and format photometry used by fbol

my $cfg = gcConfig::config();
my $GETCAL_RELEASE = gcConfig::version;

# getCal is a perl script/program that automates the process of
# extracting visibility calibrators from astronomical catalogs.
# getCal itself is a glue script that sits on top of a sequence of
# perl programs that do (most of) the actual work.

## internal globals and defaults
my $targetObj = undef;
my $targetCoord = undef;
my $target;			# target name we'll search for
my $tTag = undef;		# target desig we'll use for output
my $targetSimbadName = undef;	# target name as returned by SIMBAD
my $targetSimbadObj = undef;	# target SIMBAD result object 

my $dbgname = "getCal";		# name of module for use in debugging messages

my $usingSearchCoords=0;

## options and their defaults
my $opt_targetName = "";
my $opt_targetHD;
my $opt_targetHIP;
my $opt_targetDM = "";
my $opt_searchRA = "";
my $opt_searchDEC = "";
my @lClass;
my $opt_linearDiam = undef;
my $opt_effTemp = undef;
my $opt_maxAD = undef;
my $opt_passthru = undef;
my $opt_plx;
my $opt_format = $cfg->defaultFormat();
my $opt_messageLevel = 1;   # 0 is quiet, 1 is warnings (default), 2 is verbose (intermediate values)
my $opt_noCalibrators;
my $opt_simbadQ;
my $opt_browser;
my $opt_timeout;
my $opt_noInterspersed;
my $opt_calScript;
my $opt_doFBol;
my $opt_savePhotometry;
my $opt_fbol_plot;
my $opt_fbol_hc;
my $opt_fbol_ps;
my $opt_fbol_eps;
my $opt_fbol_png;
my $opt_fbol_constrainTemp;
my $opt_doTiming;
#my $opt_doNullDepth;
my $opt_doObsCalendar;
my $opt_xEphemDisplay;
my $opt_twoMass;
my $opt_Nband;
my $opt_Lband;
my $opt_Lprime;
my $opt_IRAS;
my $opt_rename = 0;
my @opt_cal;
my $opt_minN;
my $opt_maxN;
my $opt_minL;
my $opt_maxL;
my $opt_useMeasurement;
# options related to fbol photometry
my $opt_phot_fLambda;
my $opt_phot_fNu;
my $opt_phot_noU;
my $opt_phot_noB;
my $opt_phot_noV;
my $opt_phot_noPlx;
my $opt_phot_uvby;
my $opt_phot_longWL;
my $opt_phot_twoMass;
my $opt_phot_geneva;
my $opt_phot_gezari;
my $opt_phot_hipparcos;

# options related to simbadReport
my $opt_simbad_meas;
my $opt_simbad_commonNames;
my $opt_simbad_kmag;
my $opt_simbad_FK5;
my $opt_simbad_HD;
my $opt_simbad_HIP;
my $opt_simbad_SAO;
my $opt_simbad_identifiers;

handleCommonOpts(1, "gcShortHelp.txt", "gcLongHelp.txt");
GetOptions("targetName=s" => \$opt_targetName,
	     "targetHD=s" => \$opt_targetHD,
	     "targetHIP=s" => \$opt_targetHIP,
	     "targetDM=s" => \$opt_targetDM,
	     "searchRA=s" => \$opt_searchRA,
	     "searchDec=s" => \$opt_searchDEC,
             "sigFigures=i" => sub { $cfg->sigFigures($_[1]); },
	     "parallax|plx" => \$opt_plx,
	     "format=s" => \$opt_format,
	     "noMessages|noWarnings" => sub { $opt_messageLevel = 0 if $opt_messageLevel > 0; },
	     "quiet" => sub { $opt_messageLevel--; },
	     "verbose" => sub { $opt_messageLevel++; },
	     "noCalibrators" => \$opt_noCalibrators,
	     "simbad" => \$opt_simbadQ,
	     "simbadQuery" => \$opt_simbadQ,
	     "simbadInteractiveQuery" => \$opt_simbadQ,	# compatibility
	     "browser" => \$opt_browser,
	     "timeout=i" => \$opt_timeout,
	     "noInterspersed" => \$opt_noInterspersed,
	     "calScript" => \$opt_calScript,
	     "fBol" => \$opt_doFBol,
	     "photometry|savePhotometry" => \$opt_savePhotometry,
	     "plot" => \$opt_fbol_plot,
	     "hc" => \$opt_fbol_hc,
	     "ps" => \$opt_fbol_ps,
	     "eps" => \$opt_fbol_eps,
	     "png" => \$opt_fbol_png,
	     "constrainTemp" => \$opt_fbol_constrainTemp,
	     "timing" => \$opt_doTiming,
	     # "nullDepth|nd" => \$opt_doNullDepth, # option now passed thru to timing SG 10/07
	     "obsCalendar" => \$opt_doObsCalendar,
	     "xEphemDisplay" => \$opt_xEphemDisplay,
             "K2Mass" => \$opt_twoMass,
	     "NIRAS" => sub { $opt_IRAS = 1; $opt_Nband = 1; },
	     "Nband" => \$opt_Nband,
	     "Lband" => \$opt_Lband,
	     "Lprime" => \$opt_Lprime,
	     "rename" => \$opt_rename,
             "cal=s"  => \@opt_cal,
             "searchRadius=f" => sub { $cfg->calSearchRadius($_[1]); },
             "minV=f" => sub { $cfg->calMinV($_[1]); },
             "maxV=f" => sub { $cfg->calMaxV($_[1]); },
             "minK=f" => sub { $cfg->calMinK($_[1]); },
             "maxK=f" => sub { $cfg->calMaxK($_[1]); },
             "minN=f" => sub { $cfg->calMinN($_[1]); $opt_Nband = 1; },
             "maxN=f" => sub { $cfg->calMaxN($_[1]); $opt_Nband = 1; },
             "minL=f" => sub { $cfg->calMinL($_[1]); $opt_Lband = 1; },
             "maxL=f" => sub { $cfg->calMaxL($_[1]); $opt_Lband = 1; },
             "maxAD=f" => \$opt_maxAD,
             "lClass=s" => \@lClass,
             "effTemp" => \$opt_effTemp,
             "linearDiam" => \$opt_linearDiam,
             "passthru" => \$opt_passthru,
             "identHDC" => sub { $cfg->identHDC(1); },
             "noIdentHDC" => sub { $cfg->identHDC(0); },
             "useMeasurement" => \$opt_useMeasurement,
	# options related to fbol photometry
	     "fLambda" => \$opt_phot_fLambda,
	     "fNu" => \$opt_phot_fNu,
	     "noU" => \$opt_phot_noU,
	     "noB" => \$opt_phot_noB,
	     "noV" => \$opt_phot_noV,
	     "noPlx" => \$opt_phot_noPlx,
	     "stromgren|uvby" => \$opt_phot_uvby,
	     "longWL|IRAS" => \$opt_phot_longWL,
	     "2Mass" => \$opt_phot_twoMass,
	     "geneva" => \$opt_phot_geneva,
	     "gezari|CIO" => \$opt_phot_gezari,
	     "hipparcos|tycho" => \$opt_phot_hipparcos,
	# options related to simbad queries
	     "meas" => \$opt_simbad_meas,
	     "commonNames" => \$opt_simbad_commonNames,
	     "Kmag" => \$opt_simbad_kmag,
	     "identifiers" => \$opt_simbad_identifiers,
	     "FK5" => \$opt_simbad_FK5,
	     "HD" => \$opt_simbad_HD,
	     "HIP" => \$opt_simbad_HIP,
	     "SAO" => \$opt_simbad_SAO,
	   );

# --- finish setting up options ---

gcDebug $dbgname, 1, "opt_messageLevel = $opt_messageLevel\n";

# If the user didn't specify a luminosity class, assume he wants MS
if (!@lClass)
{
    $lClass[0] = 'V';
}
elsif ((my $tmp = join "", @lClass) =~ m/all/)
{
    @lClass = ("all");
}

# If the user has requested fbol operation, then require simbad
# interactive query with measurement extraction
if ($opt_doFBol) {
    $opt_simbadQ = 1;
}

# --- Target handling ---
print "### GUI catalog from getCal-$GETCAL_RELEASE ###\n" if ($opt_messageLevel >= 0);
gcDebug $dbgname, 1, "step0 - start target handling\n";

if ($opt_targetHD || $opt_targetHIP || $opt_targetDM)
{
    # search by HD/HIP/DM identifier
    $target = cmdTarget();
    my $hipId = $target;
    gcDebug $dbgname, 1, "using supplied target ID $target\n";
    $tTag = mangleIdent($target);	# format target tag per local preference
    $tTag =~ s/\s+//g;	# without the spaces
    gcDebug $dbgname, 1, "using formatted tag $tTag\n";

    # We have an identifier we can use to search hipparcos
    gcDebug $dbgname, 1, "step0b - retrieving target information from hipparcos for \"$hipId\"\n";
    my $iLine = hipById($hipId);
    if (!$iLine)
    {
	print "# Target $hipId not found in the Hipparcos catalog, trying SIMBAD search\n";
	$opt_targetName = $target;	# we want to try SIMBAD search using this name
    }
    else
    {
	gcDebug $dbgname, 1, "step0b - found HIP record: \"",$iLine->lineOutput(),"\"\n";
	$iLine->setField("designation", $tTag);
	$iLine->setField("tagROLE", "TRG");
	$iLine->addComment("# target $target");
	$targetObj = $iLine;
    }
}

if ($opt_targetName)	# search by target name
{
    # If we've been given a target name, translate that name to a catalog
    # number we can use.  If no catalog number found, store the SIMBAD
    # coordinates. Note name translations done through SIMBAD lookup.

    my $hipId = undef;

    $target = $opt_targetName;
    my $searchDesig = normalIdent($target);
    gcDebug $dbgname, 1, "step0a - searching SIMBAD for \"$target\", cleaned up as \"$searchDesig\"\n";
    print "# Resolving target $searchDesig via SIMBAD\n" if ($opt_messageLevel >= 0);
    # searchSimbad does a SIMBAD lookup on the given designation
    # - SIMBAD's preferred desig for this target
    # - a list of alternate desigs for this target
    $targetSimbadObj = searchSimbad($searchDesig);
    if (!$targetSimbadObj)
    {
	print "getCal: SIMBAD lookup failed\n";
	exit(TARGET_NOT_FOUND);
    }
    $targetSimbadName = $targetSimbadObj->desig();

    # examine simbad return, select a "preferred" target name to use
    my $resultDesig = "";
    if ($targetSimbadName) {	# use SIMBAD's preferred desig
	$resultDesig = $targetSimbadName;
	gcDebug $dbgname, 1, "selected name $resultDesig (from targetSimbadName)\n";
    } else {
	$resultDesig = $target;		# use the original target name
	gcDebug $dbgname, 1, "selected name $resultDesig (from supplied target name/desig)\n";
    }

    $hipId = $targetSimbadObj->hip();

    if ((!$hipId) || ($searchDesig ne normalIdent($resultDesig))) {
	print "# SIMBAD identified $searchDesig as $resultDesig";
	if (($hipId) && ($hipId ne $resultDesig))
	{
	    print " ($hipId)";
	}
	print "\n";
    }

    # select the target "tag" to use
    gcDebug $dbgname, 1, "formatting tag, target=\"$target\", resultDesig=\"$resultDesig\"\n";
    if ($opt_rename)
    {
	# use the "preferred" target designation as the target name and tag
	gcDebug $dbgname, 1, "renaming target to $resultDesig\n";
	$target = $resultDesig;	
	$tTag = mangleIdent($resultDesig);	# format target tag per local preference
    }
    else
    {
	gcDebug $dbgname, 1, "using original target $target\n";
	$tTag = $target;
	# SG 8/08: Not using mangleIdent() to deal with HDC since that removes spacesi
	# and underscores, and we want to keep the underscores in target names.
	if ($cfg->identHDC())
	{
	    $tTag =~ s/^HD([^C])/HDC$1/i;	# deal with HD=>HDC for targets
	}
	$tTag =~ s/\s+/_/g;	# replace any spaces with underscores
    }
    gcDebug $dbgname, 1, "using formatted tag $tTag\n";

    if ($hipId)
    {
	# We have an identifier we can use to search hipparcos
	gcDebug $dbgname, 1, "step0c - retrieving target information from hipparcos for \"$hipId\"\n";
	my $iLine = hipById($hipId);
	if (!$iLine)
	{
	    print "No $hipId found in the Hipparcos catalog\n";
	    exit(TARGET_NOT_FOUND);
	}
	gcDebug $dbgname, 1, "step0d - found HIP record: \"",$iLine->lineOutput(),"\"\n";
	$iLine->setField("designation", $tTag);
	$iLine->setField("tagROLE", "TRG");
	$iLine->addComment("# target $target");
	# Fill in any missing details using info returned by simbad
	#XX don't want to do this without reporting it - 3/24/06 SLG
	#XX simbad2ILine($targetSimbadObj, $iLine);
	#XX gcDebug $dbgname, 1, "step0e - after simbad fill: \"",$iLine->lineOutput(),"\"\n";
	$targetObj = $iLine;
    }
    else # no hipId
    {
	## no HIP resolution for target -- try using coordinates if we have them from SIMBAD
	gcDebug $dbgname, 1, "SIMBAD did not return a Hipparcos ID for $target\n";

	if ($targetSimbadObj->raDeg() && $targetSimbadObj->decDeg())
	{
	    gcDebug $dbgname, 1, "step0f - using info from SIMBAD\n";
	    ## Can't find a HIP record for this target, so we'll fake it by creating a
	    ## synthetic target record using the coordinate information we got from SIMBAD.
	    # This value shows up in calibrator lines as the target name

	    ## synthesize a target record from these values
	    my $iLine = new internalLine;
	    $iLine->setField("designation",$tTag);
	    #$iLine->setField("catIndex", $targetSimbadName);
	    $iLine->setField("tagROLE", "TRG");
	    $iLine->addComment("# target $target");
	    $iLine->addComment("#  Warning: No HIP ID found for $target, using information taken from SIMBAD");
	    gcDebug $dbgname, 1, "step0g - before simbad fill: \"",$iLine->lineOutput(),"\"\n";
	    simbad2ILine($targetSimbadObj, $iLine);
	    gcDebug $dbgname, 1, "step0h - after simbad fill: \"",$iLine->lineOutput(),"\"\n";

	    ## create the synthetic record from the information we've supplied
	    $targetObj = $iLine;
	} else {	
	    ## Simbad returned no HIP ID and no coordinates, we're out of luck.
	    gcDebug $dbgname, 1, "SIMBAD did not return coordinates for $target\n";
	    print STDERR "## Unable to resolve target $target using SIMBAD query\n";
	    exit(TARGET_NOT_FOUND);
	}
    }
}
elsif ($opt_searchRA && $opt_searchDEC)
{
    # search by position, create a fake target POSTARGET for this position
    gcDebug $dbgname, 1, "step0 - using supplied coords: $opt_searchRA, $opt_searchDEC\n";

    $tTag = "POSTARGET";

    ## synthesize a target record from these values
    my $iLine = new internalLine;
    $iLine->setField("designation",$tTag);
    $iLine->setField("raDeg",RAdeg($opt_searchRA));
    $iLine->setField("decDeg",DECdeg($opt_searchDEC));
    $iLine->setField("tagROLE", "TRG");
    $iLine->addComment("# target $tTag");

    ## create the synthetic record from the information we've supplied
    $targetObj = $iLine;
}

if (! $targetObj)
{
    ## should never happen
    die "getCal: must specify valid target ID or RA/DEC search coords\n";
}

gcDebug $dbgname, 1, "step0 - tTag=\"$tTag\"\n";
gcDebug $dbgname, 1, "step0 - targetLine=\"", $targetObj->lineOutput(), "\"\n";
gcDebug $dbgname, 1, "step0 - Done with target handling\n";

# 
# At this point:
#  - targetObj contains the internalLine object describing the target
#  - If SIMBAD was used to lookup target, targetSimbadObj contains the SIMBAD result
# 
# --- end target handling
# 

# 
# --- Calibrator extraction/selection ---
# 
# Extract potential calibrators from around the search location
# (hipScan), and downselect the potential calibrators according to
# certain criteria (calCheck).

gcDebug $dbgname, 1, "step1 - start calibrator selection\n";

my @calibrators;	# list of calibrator objects
if (@opt_cal) {
    $opt_passthru = 1;   # force passthru if user specified calibrators
    gcDebug $dbgname, 1, "using specified calibrators\n";
    cal: foreach my $cal (@opt_cal) {
	my $calDesig = normalIdent($cal);
	gcDebug $dbgname, 1, "lookup calibrator $calDesig ($cal)\n";
	my $calObj = hipById($calDesig);
	if (!$calObj)
	{
	    print STDERR "No HIP designation found for calibrator $calDesig ($cal), ignoring.\n";
	    next cal;
	}

	# Calculate angular separation between target and calibrator
	my $sep = &angleBetween($targetObj->extractField("raDeg"),
	    $targetObj->extractField("decDeg"),
	    $calObj->extractField("raDeg"),
	    $calObj->extractField("decDeg"));
	$calObj->setField('tagD', $sep);
	$calObj->setField("tagROLE", "CAL");

	push @calibrators, $calObj;
    }
}
elsif (!$opt_noCalibrators)
{
    @calibrators = calSearch($targetObj->extractField("raDeg"),
	$targetObj->extractField("decDeg"),
	$cfg->calSearchRadius());
    die "getCal:calSearch failed\n" if !@calibrators;
}

if (gcConfig::debug() > 1) {
    gcDebug $dbgname, 1, "step1a - initial calibrators:\n";
    foreach my $calibrator (@calibrators) {
	    print STDERR "DEBUG:   " . $calibrator->lineOutput();
    }
}

gcDebug $dbgname, 1, "step1b - filter cal candidates using search criteria \n";
@calibrators = calCheck(calibrators => \@calibrators,
    tTag => $tTag,
    messageLevel => $opt_messageLevel,
    twoMass => $opt_twoMass,
    Nband => $opt_Nband,
    IRAS => $opt_IRAS,
    minV => $cfg->calMinV(),
    maxV => $cfg->calMaxV(),
    minK => $cfg->calMinK(),
    maxK => $cfg->calMaxK(),
    minN => $cfg->calMinN(),
    maxN => $cfg->calMaxN(),
    minL => $cfg->calMinL(),
    maxL => $cfg->calMaxL(),
    maxAD => $opt_maxAD,
    lClass => \@lClass,
    effTemp => $opt_effTemp,
    linearDiam => $opt_linearDiam,
    passthru => $opt_passthru,
    useMeasurement => $opt_useMeasurement,
    Lband => $opt_Lband,
    Lprime => $opt_Lprime,
);

# Eliminate any target records that made it into the calibrator list
gcDebug $dbgname, 1, "step1c - remove target records\n";
if ($targetObj && (scalar @calibrators))
{
    my $tgtRA = $targetObj->extractField("raDeg");
    my $tgtDEC = $targetObj->extractField("decDeg");
    my $i = 0;
    # go through the calibrator list, removing any entries
    # which match our target's position.
    while ($i < (scalar @calibrators))
    {
	if (($calibrators[$i]->extractField("raDeg") == $tgtRA) &&
	    ($calibrators[$i]->extractField("decDeg") == $tgtDEC))
	{
	    gcDebug $dbgname, 1, "step1d - removing record: \"", $calibrators[$i]->lineOutput(), "\"\n";
	    splice @calibrators, $i, 1;	# remove this entry from the array
	}
	else
	{
	    $i++;	# move on to the next
	}
    }
} # end eliminating target from calibrator set

# 
# --- Additional processing ---
# 
# call calCheck to annotate records with addtional information
gcDebug $dbgname, 1, "step3 - annotate target if needed\n";
if ($targetObj && !$opt_searchRA && !$opt_searchDEC)
{
    gcDebug $dbgname, 1, "step3a - calling calCheck on target=\"", $targetObj->lineOutput(), "\"\n";
    # call calCheck on the target to do any additional annotation
    my @candidates; push @candidates, $targetObj;
    @candidates = calCheck(calibrators => \@candidates,
        messageLevel => $opt_messageLevel,
        twoMass => $opt_twoMass,
        Nband => $opt_Nband,
        IRAS => $opt_IRAS,
        minV => $cfg->calMinV(),
        maxV => $cfg->calMaxV(),
        minK => $cfg->calMinK(),
        maxK => $cfg->calMaxK(),
        minN => $cfg->calMinN(),
        maxN => $cfg->calMaxN(),
        minL => $cfg->calMinL(),
        maxL => $cfg->calMaxL(),
        maxAD => $opt_maxAD,
        lClass => \@lClass,
        effTemp => $opt_effTemp,
        linearDiam => $opt_linearDiam,
        passthru => 1,
        useMeasurement => $opt_useMeasurement,
        Lband => $opt_Lband,
        Lprime => $opt_Lprime,
    );
    $targetObj = $candidates[0];		# update with annotations
    gcDebug $dbgname, 1, "step3b - calCheck returned target=\"", $targetObj->lineOutput(), "\"\n";
}

gcDebug $dbgname, 1, "step 3c - Done with cal selection\n";
my $calCat="";
foreach my $calibrator (@calibrators) {
    $calCat .= $calibrator->lineOutput();
}
gcDebug $dbgname, 1, "step3c - calCat=\"$calCat\"\n";

# 
# --- Assemble some summary information for use by fbol, calScript
# 
# First, assemble a complete list of designations for targets and calibrators
# Do SIMBAD lookups if requested.
my @desigList;		# list of target/cal desigs
my @objList;		# list of target/cal iLine objects
my @simbadObjList;	# list of target/cal SIMBAD result objects
# the target
gcDebug $dbgname, 1, "step4a - adding target desig \"",$tTag,"\"\n";
push @desigList, $tTag;
push @objList, $targetObj;
if ($opt_simbadQ)
{
    # If we haven't done SIMBAD lookup for target,
    # and the target is not a POSTARGET,
    # do the search now
    if (!$targetSimbadObj && !$targetObj && !$opt_searchRA && !$opt_searchDEC)
    {
	gcDebug $dbgname, 1, "step4b - simbad search (target) for desig \"",$tTag,"\n";
	$targetSimbadObj = searchSimbad($tTag);
    }
    push @simbadObjList, $targetSimbadObj;		# save SIMBAD result object - may be null
}
# now the cals
foreach my $calObj (@calibrators)
{
    # for calibrators 
    gcDebug $dbgname, 1, "step4c - adding cal desig \"",$calObj->extractField("designation"),"\n";
    my $desig = $calObj->extractField("designation");
    push @desigList, $desig;
    push @objList, $calObj;
    if ($opt_simbadQ)
    {
	gcDebug $dbgname, 1, "step4d - simbad search (cal) for desig \"",$desig,"\n";
	my $calSimbadObj = searchSimbad($desig);	# SIMBAD lookup for this cal
	push @simbadObjList, $calSimbadObj;		# save SIMBAD result object - may be null
    }
}

# 
# --- SIMBAD reporting section ---
# 
my @simbadRpt;		# output lines for SIMBAD report
if ($opt_simbadQ) # do SIMBAD reporting
{
    #XXX what to do about $opt_browser?
    gcDebug $dbgname, 1, "step5 - start SIMBAD reports\n";
    my %opts = &simbadReportOpts;	# options passed to simbadReport

    for (my $i=0; $i < @desigList; $i++)
    {
	if ($simbadObjList[$i])
	{
	    gcDebug $dbgname, 1, "step5a - formatting simbad report for \"",$desigList[$i],"\"\n";
	    my @rpt = simbadReport($desigList[$i], $simbadObjList[$i],\%opts);	# format the report
	    push @simbadRpt, @rpt;			# save the SIMBAD report
	    push @simbadRpt, "\n";
	    my $comment = join '\n', grep /Simbad Search/, @rpt;	# extract the summary
	    if ($comment)
	    {
		$objList[$i]->addComment("# " . $comment);		# add the summary as a comment
	    }
	}
    }
} # end SIMBAD processing

#
# --- Output section ---
#
gcDebug $dbgname, 1, "step6 - start output \n";
my $outputReport;
foreach my $obj (@objList)
{
    gcDebug $dbgname, 1, "  output line: \"", $obj->lineOutput(), "\"\n";
    if ($opt_format eq 'new')
    {
	$outputReport .= $obj->outputNewFormat($opt_noInterspersed, $cfg->sigFigures(), $opt_plx, $opt_Nband);
    }
    elsif ($opt_format eq 'sky')
    {
	$outputReport .= $obj->outputSkyFormat($opt_noInterspersed, $cfg->sigFigures());
    }
    elsif ($opt_format eq 'old')
    {
	$outputReport .= $obj->outputOldFormat($opt_noInterspersed, $cfg->sigFigures(), $opt_plx,
	    $opt_Nband || $opt_IRAS);
    }
    else {
	die "Output format not recognized: $opt_format\n";
    }
}
gcDebug $dbgname, 1, "output report: \"",$outputReport,"\"\n";
print $outputReport;

#
# --- obsCalendar & timing calculations ---
#
gcDebug $dbgname, 1, "step7 -  obsCalender+timing\n";
if ($opt_doObsCalendar)
{
    my $ocReport = exec_stdin("obsCalendar --stdin --noEcho @ARGV", $outputReport, $cfg->debug());
    print "\n\n$ocReport\n\n";
}

if ($opt_doTiming)
{
    my $timingRpt = exec_stdin("timing -stdin -noEcho @ARGV", $outputReport, $cfg->debug());
    print "\n\n$timingRpt\n\n";
}

# --nullDepth option now passed thru to timing instead of a separate nullDepth utility - SG 10/07
#if ($opt_doNullDepth)
#{
#    my $ndRpt = exec_stdin("nullDepth -stdin -noEcho @ARGV", $outputReport, $cfg->debug());
#    print "\n\n$ndRpt\n\n";
#}

#
# --- fBol operation ---
#
gcDebug $dbgname, 1, "step8 start fbol\n";
if ($opt_doFBol) # Generating diameter estimates through bolometric flux information
{
    # fetch and format photometry for use by fbol
    gcDebug $dbgname, 1, "step8a calling formatPhotometry\n";
    my %popts = &photOpts;
    my @photLines = formatPhotometry(\@desigList,\@simbadObjList,\%popts);
    my $photometry = join "", @photLines;
    gcDebug $dbgname, 1, "using photometry \"$photometry\"\n";

    # run fbol using the formatted photometry
    my $fbolopts = &fbolOpts;
    gcDebug $dbgname, 1, "step8c calling fbol\n";
    my $fBolOut = exec_stdin("fbol $fbolopts --stdin", $photometry, $cfg->debug());
    print "\n\n### Bolometric Flux Diameter Fit results ###\n";
    print $fBolOut;
    print "\n";
    if ($opt_savePhotometry)
    {
	print "### Begin fbol Photometry ###\n";
	print $photometry;
	print "### End fbol Photometry ###\n";
    }
    
    if ($opt_fbol_plot)
    {
	system "rm -f script2.gnu star.dat bbc.dat";
    }
} # end fBol processing

#
# --- Calibration script composition ---
#
gcDebug $dbgname, 1, "step9 start calscript\n";
if ($opt_calScript)
{
    print "\n\n### Begin Calibration script ###\n";
    my $cmdOpts = &calScriptOpts;
    my $calScript;
    my $lines="";
    # create a long string containing the target and cal object lines
    foreach my $obj (@objList)
    {
	$lines .= $obj->lineOutput();
    }
    gcDebug $dbgname, 1, "opt_calScript: lines=\"$lines\"\n";
    if ($targetObj)	# if there is a target defined
    {
        $calScript = exec_stdin("calScript --stdin $cmdOpts", $lines, $cfg->debug());
    }
    else
    {
	my $tName = $target;
	$tName =~ s/\s+//g;
	$calScript = "$tName\n$targetCoord\n";
	$calScript .= exec_stdin("calScript --stdin $cmdOpts --noTarget", $lines, $cfg->debug());
    }

    # cross-sort the calibration script with the summary lines from the SIMBAD
    # report
    if ($opt_simbadQ && !$opt_noInterspersed)
    {
	my @calScrContents = split /\n/, $calScript;
	foreach my $cLine (@calScrContents)
	{
	    if ($cLine =~ m/^\#|^---|^\d+/) # comments/non-object lines
	    {
		print "$cLine\n";
	    }
	    else # object line
	    {
		my @tmp = split /\s+/,$cLine;	# pull desig from calScript line
		# find Simbad report summary line(s) for this desig
		my @rptLines = grep /Simbad Search/, (grep /$tmp[0]/, @simbadRpt);
		foreach my $rLine (@rptLines)	# prefix matching report lines with "# "
		{
		    $rLine =~ s/^Simbad Search/# Simbad Search/g;
		}
		if (@rptLines)
		{
		    print @rptLines;
		}
		print "$cLine\n";
	    } # end object line
	} # end foreach loop
    } # end !$opt_noInterspersed
    else
    {
	print $calScript;
    }
    print "\n\n### End Calibration script ###\n";

} ## end calScript composition


#
# --- Output SIMBAD report
#
gcDebug $dbgname, 1, "step10 simbad output\n";
if ($opt_simbadQ)
{
    print "\n\n### Simbad query results ###\n";
    foreach my $line (@simbadRpt)
    {
	print $line;
    }
    print "\n\n### End Simbad query results ###\n";
}

#
# --- Drive xephem ---
#
# Finally, if requested, write the program list to the xEphem fifo for
# display
gcDebug $dbgname, 1, "step11 start xephem output\n";
if ($opt_xEphemDisplay)
{
    my $xEphemFIFO = $cfg->xephemFifo;
    if ((-w $xEphemFIFO) && (-p $xEphemFIFO))  ## fifo test
    {
	## Open the xephem fifo in read/write (non-blocking) mode (does this work?)
	open(FIFO,"+>>$xEphemFIFO") or die "failed opening dbfifo $xEphemFIFO\n";
	my $xeCat = exec_stdin("xeConvert --stdin", $outputReport, $cfg->debug());
	print FIFO $xeCat;
	close(FIFO);
    }
    else
    {
	print "Cannot write to fifo designation $xEphemFIFO\n";
    }
} # end writing program to xEphem

#
# --- All Done ---
#
gcDebug $dbgname, 1, "normal exit\n";
## Finished -- exit getCal
exit(0);
## end getCal  ##########################################################


#########################################################################
## calScriptOpts -- relay options to calScript
#########################################################################
sub calScriptOpts
{
    my $cmdOpts = "";
    $cmdOpts .= " -sigFigures " . $cfg->sigFigures() if ($cfg->sigFigures());
    $cmdOpts .= " -noMessages" if ($opt_messageLevel < 1);
    $cmdOpts .= " -plx" if ($opt_plx);
    $cmdOpts .= " -IRAS" if ($opt_IRAS);
    $cmdOpts .= " -Nband" if ($opt_Nband);
    return $cmdOpts;
}


#########################################################################
## fbolOpts -- relay options to fbol
#########################################################################
sub fbolOpts
{
    my $cmdOpts = "-fixedError";
    $cmdOpts .= " -plot" if ($opt_fbol_plot);
    $cmdOpts .= " -hc" if ($opt_fbol_hc);
    $cmdOpts .= " -ps" if ($opt_fbol_ps);
    $cmdOpts .= " -eps" if ($opt_fbol_eps);
    $cmdOpts .= " -png" if ($opt_fbol_png);
    $cmdOpts .= " -constrainTemp" if ($opt_fbol_constrainTemp);
    return $cmdOpts;
}

#########################################################################
## photOpts -- relay options to formatPhotometry
## these options are relayed as a hash instead of a string
#########################################################################
sub photOpts
{
    my %opts;
    $opts{units} = photometry::UNITS_FLAMBDA if ($opt_phot_fLambda);
    $opts{units} = photometry::UNITS_FNU if ($opt_phot_fNu);
    $opts{noU} = $opt_phot_noU if (defined $opt_phot_noU);
    $opts{noB} = $opt_phot_noB if (defined $opt_phot_noB);
    $opts{noV} = $opt_phot_noV if (defined $opt_phot_noV);
    $opts{noPlx} = $opt_phot_noPlx if (defined $opt_phot_noPlx);
    $opts{geneva} = $opt_phot_geneva if (defined $opt_phot_geneva);
    $opts{uvby} = $opt_phot_uvby if (defined $opt_phot_uvby);
    $opts{longWL} = $opt_phot_longWL if (defined $opt_phot_longWL);
    $opts{twoMass} = $opt_phot_twoMass if (defined $opt_phot_twoMass);
    $opts{gezari} = $opt_phot_gezari if (defined $opt_phot_gezari);
    $opts{hipparcos} = $opt_phot_hipparcos if (defined $opt_phot_hipparcos);
    return %opts;
}

#########################################################################
## simbadReportOpts -- relay options to simbadReport
## these options are relayed as a hash instead of a string
#########################################################################
sub simbadReportOpts
{
    my %opts;
    $opts{meas} = 1 if ($opt_simbad_meas);
    $opts{commonNames} = 1 if ($opt_simbad_commonNames);
    $opts{Kmag} = 1 if ($opt_simbad_kmag);
    $opts{identifiers} = 1 if ($opt_simbad_identifiers);
    $opts{FK5} = 1 if ($opt_simbad_FK5);
    $opts{HD} = 1 if ($opt_simbad_HD);
    $opts{HIP} = 1 if ($opt_simbad_HIP);
    $opts{SAO} = 1 if ($opt_simbad_SAO);
    return %opts;
}

# Given a target ID for either HIP, HD or DM, look up the target in Hipparcos
# and use that to construct an internalLine
sub hipById {
    my $id = shift;

    my $dbgname = "getCal::hipById";

    my @pieces = identPieces($id);
    my $hipRec = getHipRecord($pieces[0],$pieces[1]);
    if (!$hipRec) {
	gcDebug $dbgname, 1, "no hip record for \"$pieces[0] $pieces[1]\"\n" if gcConfig::debug();
	return undef;
    }
    gcDebug $dbgname, 1, "found record \"$pieces[0] $pieces[1]\" = \"$hipRec\"\n" if gcConfig::debug();
    my $hipLine = new hipparcosLine(line => $hipRec);
    my $iLine = &hip2ILine($hipLine);
    return $iLine;
}

# Returns the target if one was specified on the command line
sub cmdTarget {
    return "HIP " . $opt_targetHIP if ($opt_targetHIP);
    return "HD " . $opt_targetHD if ($opt_targetHD);
    return "DM " . $opt_targetDM if ($opt_targetDM);
    return undef;	# error
}

# end of getCal.pl
