#! @PERL@
# -*- perl -*-
# @configure_input@

our $MODVERSION = '$Id: gcGui.pl,v 1.40 2008/11/12 23:28:12 sgroom Exp $';
our $MODREV = '$Revision: 1.40 $';

##########################################################################
# NASA Exoplanet Science Institute (NExScI) -- http://nexsci.caltech.edu #
#       Copyright (c) 1997-2008 California Institute of Technology       #
#                     U.S. Government Sponsorship                        #
# Under contract with the National Aeronautics and Space Administration  #
##########################################################################

### gcGui -- the getCal perl/Tk gui.

## gcGui provides a gui front-end to the getCal calibrator/catalog
## scanning application.  The implementation mechanism is to have the
## user graphically interact with the gui to compose the appropriate
## getCal command line, and then dispatch the getCal invocation and
## retrieve and display the results.

use strict;
use warnings;
use FindBin qw($RealBin);
use lib "$RealBin/../lib/modules";		# location of getCal Perl modules
use Getopt::Long;
use gcConfig;
use miscArgs;
use interferometers; ## interferometer DB module
use gcGuiAttributes; ## attributes for this gui
use findScript;
use guiSubs;
use Tk;
use Tk::FileSelect;
use Tk::ROText;
use Tk::Wm;
use Tk::Balloon;

use widgetTree;

my $cfg = gcConfig::config();
my @ra = split /\s+/, $MODREV; my $REVISION = $ra[1];
my $PKGSTR = package_string();


my $fbolArgument = "";


####### process command line options
handleCommonOpts(0,"gcGui.txt");
# we don't have any other command line opts beyond those common ones
# otherwise we'd process them here with Getopt

####### set up the main gui
# The main window declaration
my $MW = MainWindow->new();
$MW->title("getCal GUI v" . $REVISION . " ($PKGSTR)");

## In-line help stuff
my $balloon = $MW->Balloon();  ## help balloon object

## At the top -- the menu frame
my $menuFrame = $MW->Frame(-relief => "raised",
			   -borderwidth => "2"
			   )->pack(-side => "top", -fill => "x");

# file menu
my $fileMenu = $menuFrame->Menubutton(-text => "File",
				      -menuitems => [[ 'command' => "Dispatch",
						       -accelerator => 'Alt+g',
						       -command => \&submit],
						     [ 'command' => "Reset",
						       -accelerator => 'Alt+r',
						       -command => \&reset],
						     "-",
						     [ 'command' => "Quit",
						       -accelerator => 'Alt+q',
						       -command => \&exit] ]
				      )->pack(-side => "left", -anchor => "w");

## Keyboard bindings for file menu
$MW->bind('<Alt-g>' => sub {&submit();});
$MW->bind('<Alt-r>' => sub {&reset();});
$MW->bind('<Alt-q>' => sub {$MW->withdraw; exit(0);});



# help menu

# help files
my $gcGuiHelp = findFile("gcGui.txt", $cfg->gcdocs, $cfg->gcdata);
my $gcHelp = findFile("gcLongHelp.txt",$cfg->gcdocs, $cfg->gcdata);
my $shortHelp = findFile("gcShortHelp.txt",$cfg->gcdocs, $cfg->gcdata);
my $format = findFile("format.txt",$cfg->gcdocs, $cfg->gcdata);
my $fbolHelp = findFile("fbol.txt", $cfg->gcdocs, $cfg->gcdata);
my $timingHelp = findFile("timing.txt", $cfg->gcdocs, $cfg->gcdata);
my $ocHelp = findFile("obsCalendar.txt", $cfg->gcdocs, $cfg->gcdata);
my $xephemHelp = findFile("xeConvert.txt",$cfg->gcdocs, $cfg->gcdata);
my $version = findFile("version.txt",$cfg->gcdocs, $cfg->gcdata);
my $copyright = findFile("copyright.txt",$cfg->gcdocs, $cfg->gcdata);

my $helpMenu = $menuFrame->Menubutton(-text => "Help",
				      -menuitems => [ [ 'command' => "gcGui help",
							-accelerator => 'Alt+h',
							-command => sub {&dispHelp($gcGuiHelp)} ],
						      [ 'command' => "about gcGui",
							-command => sub {&aboutgcGui()} ],
						      "-",
						      [ 'command' => "getCal help",
							-command => sub {&dispHelp($gcHelp)} ],
						      [ 'command' => "getCal short help",
							-command => sub {&dispHelp($shortHelp)} ],
						      [ 'command' => "getCal format",
							-command => sub {&dispHelp($format)} ],
						      [ 'command' => "fbol help",
							-command => sub {&dispHelp($fbolHelp)}  ],
						      [ 'command' => "timing help",
							-command => sub {&dispHelp($timingHelp)}  ],
						      [ 'command' => "obsCalendar help",
							-command => sub {&dispHelp($ocHelp)}  ],
						      [ 'command' => "using XEphem with getCal",
							-command => sub {&dispHelp($xephemHelp)}  ],
						      [ 'command' => "getCal version",
							-command => sub {&dispHelp($version)}  ],
						      [ 'command' => "getCal copyright",
							-command => sub {&dispHelp($copyright)}  ] ]
				      )->pack(-side => "right", -anchor => "w");

$MW->bind('<Alt-h>' => sub {&dispHelp($gcGuiHelp)});



###############################################################
# Object designation frame -- this is where the user
# specified the object he wants to look up
###############################################################
my $objFrame = $MW->Frame(-relief => "groove",
			   -borderwidth => "2"
			  )->pack(-side => "top", -fill => "x");

$objFrame->Label(-text => "Object Designation/Pos",
		 )->pack(-side => "left", -anchor => "w");

my $objVal;
my $objEntry = $objFrame->Entry(-textvariable => \$objVal,
				-width => 30,
				-relief => 'raised'
				)->pack(-side => "left", -anchor => "w", -fill => "x", -expand => 1);
$balloon->attach($objEntry,-msg => "Target designation or coordinates");
$objEntry->bind('<Return>' => sub {&submit();});

# Radio (mutually exclusive) buttons to define the type of
# designation, name (to be resolved by Simbad), HD or HIP catalog
# designation, or astrometric position (added 10/99 A.B.).  Default to
# giving an object name.
my $nameType = "name";
my $objRadio;
foreach (qw(name HD HIP Pos))
{
    $objRadio = $objFrame->Radiobutton(-text => $_,
				       -value => $_,
				       -variable => \$nameType
				       )->pack(-side => "left");
} # end foreach

## end objFrame



###############################################################
# Next, the option frame.  This frame holds all the option switches
###############################################################
my $optFrame = $MW->Frame(
			  # -relief => "groove",
			  -borderwidth => "2"
			  )->pack(-side => "top", -fill => "x");


###############################################################
## The photometry options subframe

my $photFrameL = $optFrame->Frame(
					  -relief => "groove",
					  -borderwidth => "2",
				   );
my $photFrameR = $optFrame->Frame(
					  -relief => "groove",
					  -borderwidth => "2",
				   );
$photFrameL->grid($photFrameR,  -sticky => "nsew");

my $photFrameLabel = $photFrameL->Label(-text => "Photometry Search/Output Options:")->pack( -side => "left");

### sources of mag info
#K
my $photFrameR1 = $photFrameR->Frame(
					  # -relief => "groove",
					  # -borderwidth => "2",
					)->pack(-side => "top", -anchor => "w");
my $photMagLabelK = $photFrameR1->Label(-text => "     K-band:")->pack( -side => "left");
my $photMagK;
my $photMagKModelVal=1;
my $photMagK2MassVal=2;
my $photMagKModelButton = $photFrameR1->Radiobutton(-text => "Model",
				       -value => $photMagKModelVal,
				       -variable => \$photMagK,
				       )->pack(-side => "left", -anchor => "w");
$balloon->attach($photMagKModelButton,-msg => "Use V-K model to estimate K mag");
my $photMagK2MassButton = $photFrameR1->Radiobutton(-text => "2Mass",
				       -value => $photMagK2MassVal,
				       -variable => \$photMagK,
				       )->pack(-side => "left", -anchor => "w");
$balloon->attach($photMagK2MassButton,-msg => "Derive K mag from 2MASS");

#N
my $photFrameR2 = $photFrameR->Frame(
					  # -relief => "groove",
					  # -borderwidth => "2",
					)->pack(-side => "top", -anchor => "w");
my $NbandSel = 0;
my $NbandTree;
my $NbandButton = $photFrameR2->Checkbutton(-text => "N-band:",
					   -variable => \$NbandSel,
					   #-relief => 'raised',
					    -command => sub { $NbandTree->updateChildren(); }
					)->pack(-side => "left", -anchor => "w");
$NbandTree = new widgetTree("Nband",$NbandButton,\$NbandSel);
$balloon->attach($NbandButton,-msg => "Include N-band photometry in output");

my $photMagN;
my $photMagNModelVal=1;
my $photMagNIRASVal=2;
my $photMagNModelButton = $photFrameR2->Radiobutton(-text => "Model",
				       -value => $photMagNModelVal,
				       -variable => \$photMagN,
				       )->pack(-side => "left", -anchor => "w");
$NbandTree->addChild("NModel",$photMagNModelButton,undef);
$balloon->attach($photMagNModelButton,-msg => "Use V-N model to estimate N mag");
my $photMagNIRASButton = $photFrameR2->Radiobutton(-text => "from IRAS",
				       -value => $photMagNIRASVal,
				       -variable => \$photMagN,
				       )->pack(-side => "left", -anchor => "w");
$NbandTree->addChild("NIRAS",$photMagNIRASButton,undef);
$balloon->attach($photMagNIRASButton,-msg => "Derive N mag from IRAS");

#L
my $LbandSel = 0;
my $LbandButton = $photFrameR->Checkbutton(-text => "L-band (Model)",
					   -variable => \$LbandSel,
					   #-relief => 'raised',
					)->pack(-side => "top", -anchor => "w");
$balloon->attach($LbandButton,-msg => "Include L-band photometry in output");

#L'
my $LprimeSel = 0;
my $LprimeButton = $photFrameR->Checkbutton(-text => "L'-band (Model)",
					   -variable => \$LprimeSel,
					   #-relief => 'raised',
					)->pack(-side => "top", -anchor => "w");
$balloon->attach($LprimeButton,-msg => "Include L'-band photometry in output");

# "useMeasurement" checkbox
my $useMeasurementSel = 0;
my $useMeasurementButton = $photFrameR->Checkbutton(-text => "Use 2MASS/IRAS measurements for cal search",
						-variable => \$useMeasurementSel,
						# -relief => 'raised'
						)->pack(-side => "left");
$balloon->attach($useMeasurementButton,-msg => "Use 2MASS/IRAS12 measurements for calibrator V/K mag range filter");


###############################################################
## The calibrator options subframe

my $calFrameL = $optFrame->Frame(
					  -relief => "groove",
					  -borderwidth => "2",
				   );
my $calFrameR = $optFrame->Frame(
					  -relief => "groove",
					  -borderwidth => "2",
				   );
$calFrameL->grid($calFrameR,  -sticky => "nsew");

## the calibrator output button
my $calSel = 0;
my $calTree;
my $calButton = $calFrameL->Checkbutton(-text => "Calibrator Output:",
					-variable => \$calSel,
					# -relief => 'raised',
				        -command => sub { $calTree->updateChildren(); }
					)->pack(-side => "left", -anchor => "w");
$calTree = new widgetTree("calOutput",$calButton,\$calSel);
$balloon->attach($calButton,-msg => "Enable calibrator output");

# calFrameR is subdivided as a grid in itself.
# The first row of this grid deals with user-specified calibrators
# The second row of this grid deals with calibrator search, and is subdivided
#	into several additional rows.
# the calibrator selection method (user-spec, cal search) is selected
# by two radio buttons.

my $calFrameR1L = $calFrameR->Frame(
				      # -relief => "groove",
				      -borderwidth => "2",
				    );
my $calFrameR1R = $calFrameR->Frame(
				      -relief => "groove",
				      -borderwidth => "2",
				    );
$calFrameR1L->grid($calFrameR1R,  -sticky => "nsew");

### subframe for calibrator search
### this is also subdivided into subframes
my $calFrameR2L = $calFrameR->Frame(
				      # -relief => "groove",
				      -borderwidth => "2",
				    );
my $calFrameR2R = $calFrameR->Frame(
				      -relief => "groove",
				      -borderwidth => "2",
				    );
$calFrameR2L->grid($calFrameR2R,  -sticky => "nsew");

# Use radiobuttons to choose the calibrator selection method.
# The radiobutton state is stored as a single state variable $calMethod,
# which can have several values depending on which button is selected.
# For combatibility with the widgetTree mechanism, we also have some
# individual state variables, one for each of the buttons
# ($calMethodUserSel and $calMethodSearchSel), indicating
# whether that particular button is currently selected. Whenever
# any of the buttons is selected, a routine (setCalMethod) is called to update
# each of the individual state vars. The widgetTree stuff uses
# these vars to enable/disable any widgets slaved to each button.

# The set of $calMethod values:
my $calMethodUser = 1;		# value for $calMethod when user-spec is selected
my $calMethodSearch = 2;	# value for $calMethod when cal search is selected
my $calMethod = $calMethodSearch;	# radiobutton state variable, shared by all the buttons

# the "user-spec cal" radiobutton
my $calMethodUserSel=0;			# state var indicating whether calMethod = user
my $calMethodUserTree;
my $calMethodUserButton = $calFrameR1L->Radiobutton(-text => "User-specified calibrators:",
				       -value => $calMethodUser,
				       -variable => \$calMethod,
				        -command => sub { &setCalMethod; $calTree->updateChildren(); }
				       )->pack(-side => "left", -anchor => "w");
$calMethodUserTree = $calTree->addChild("calMethodUser",$calMethodUserButton,\$calMethodUserSel);
$balloon->attach($calMethodUserButton,-msg => "Return only user-specified calibrators");

# the "cal search" radiobutton
my $calMethodSearchSel=1;		# state var indicating whether calMethod = search
my $calMethodSearchTree;
my $calMethodSearchButton = $calFrameR2L->Radiobutton(-text => "Calibrator search:",
				       -value => $calMethodSearch,
				       -variable => \$calMethod,
				        -command => sub { &setCalMethod; $calTree->updateChildren(); }
				       )->pack(-side => "left", -anchor => "w");
$calMethodSearchTree = $calTree->addChild("calMethodSearch",$calMethodSearchButton,\$calMethodSearchSel);
$balloon->attach($calMethodSearchButton,-msg => "Search for calibrators");


### subframe for user-specified calibrators
my $calUserVal = "";
my $calUserEntry = $calFrameR1R->Entry(-textvariable => \$calUserVal,
				      -width => 60,
				      -relief => 'raised',
				       )->pack(-side => "left", -anchor => "w", -fill => "x", -expand => 1);
$calMethodUserTree->addChild("calUserEntry",$calUserEntry,undef);
$balloon->attach($calUserEntry,-msg => "comma-separated list of calibrator names");


# cal search row 1
my $calSearchFrame1 = $calFrameR2R->Frame()->pack( -side => "top", -anchor => "w");

my $lcLabel = $calSearchFrame1->Label(-text => "     Luminosity Class:")->pack( -side => "left");
$calMethodSearchTree->addChild("lcLabel",$lcLabel,undef);

my $lcVSel = 1;
my $lcVButton = $calSearchFrame1->Checkbutton(-text => "LC V",
					       -variable => \$lcVSel,
					       -relief => 'raised'
				       ) ->pack( -side => "left", -anchor => "n");
$calMethodSearchTree->addChild("lcVButton",$lcVButton,\$lcVSel);
$balloon->attach($lcVButton,-msg => "Select main-sequence calibrators");

my $lcIIISel = 0;
my $lcIIIButton = $calSearchFrame1->Checkbutton(-text => "LC III",
					    -variable => \$lcIIISel,
					    -relief => 'raised'
				       ) ->pack ( -side => "left", -anchor => "n");
$calMethodSearchTree->addChild("lcIIIButton",$lcIIIButton,\$lcIIISel);
$balloon->attach($lcIIIButton,-msg => "Select giant calibrators");

my $lcISel = 0;
my $lcIButton = $calSearchFrame1->Checkbutton(-text => "LC I",
						  -variable => \$lcISel,
						  -relief => 'raised',
				       ) ->pack ( -side => "left", -anchor => "n");
$calMethodSearchTree->addChild("lcIButton",$lcIButton,\$lcISel);
$balloon->attach($lcIButton,-msg => "Select supergiant calibrators");

### cal search row 2
my $calSearchFrame2 = $calFrameR2R->Frame()->pack( -side => "top", -anchor => "w");

my $maxADSel = 0;
my $maxADTree;
my $maxADButton = $calSearchFrame2->Checkbutton(-text => "Max Angular Diameter (mas):",
					     -variable => \$maxADSel,
					     # -relief => 'raised',
					     -command => sub { $maxADTree->updateChildren(); }
				       ) ->pack (-side => "left", -anchor => "n");
$maxADTree = $calMethodSearchTree->addChild("maxADButton",$maxADButton,\$maxADSel);
$balloon->attach($maxADButton,-msg => "Set maximum calibrator angular diameter (mas)");

my $maxADVal = "";
my $maxADEntry = $calSearchFrame2->Entry(-textvariable => \$maxADVal,
				      -width => 4,
				      -relief => 'raised',
				      ) -> pack(-side => "top", -anchor => "w");
$maxADTree->addChild("maxADEntry",$maxADEntry,undef);
$balloon->attach($maxADEntry,-msg => "Set maximum calibrator angular diameter (mas)");

### cal search row 3
# search radius leading to an entry box
my $calSearchFrame3 = $calFrameR2R->Frame()->pack( -side => "top", -anchor => "w");

my $searchRadiusLabel = $calSearchFrame3->Label(-text => "     Search Radius (deg):")->pack(-side => "left", -anchor => "n");
$calMethodSearchTree->addChild("searchRadiusLabel",$searchRadiusLabel,undef);

my $searchRadiusVal = "";
my $searchRadiusEntry = $calSearchFrame3->Entry(-textvariable => \$searchRadiusVal,
					  -width => 4,
					  -relief => 'raised'
					  )->pack(-side => "left", -anchor => "w");
$calMethodSearchTree->addChild("searchRadiusEntry",$searchRadiusEntry,undef);
$balloon->attach($searchRadiusEntry,-msg => "Set calibrator search radius (deg)");

### cal search row 4 = calMagFrame
# calibrator magnitude specification frame
my $calMagFrame = $calFrameR2R->Frame()->pack( -side => "top", -anchor => "w");
my $calMagFrameL = $calMagFrame->Frame(
				      # -relief => "groove",
				      # -borderwidth => "2",
				    );
my $calMagFrameR = $calMagFrame->Frame(
				      -relief => "groove",
				      -borderwidth => "2",
				    );
$calMagFrameL->grid($calMagFrameR,  -sticky => "nsew");
my $magLabel = $calMagFrameL->Label(-text => "     Magnitude Range:")->pack( -side => "left");
$calMethodSearchTree->addChild("magLabel",$magLabel,undef);


# calMagFrame row 1
my $calMagFrame1 = $calMagFrameR->Frame()->pack( -side => "top", -anchor => "w");
# minV and entry box
my $minVLabel = $calMagFrame1->Label(-text => "Min V")->pack(-side => "left", -anchor => "w");
$calMethodSearchTree->addChild("minVLabel",$minVLabel,undef);
my $minVVal = "";
my $minVEntry = $calMagFrame1->Entry(-textvariable => \$minVVal,
				     -width => 4,
				     -relief => 'raised'
				 )->pack(-side => "left", -anchor => "w");
$calMethodSearchTree->addChild("minVEntry",$minVEntry,undef);
$balloon->attach($minVEntry,-msg => "Set minimum calibrator V");

# maxV and entry box
my $maxVLabel = $calMagFrame1->Label(-text => "Max V")->pack(-side => "left", -anchor => "w");
$calMethodSearchTree->addChild("maxVLabel",$maxVLabel,undef);
my $maxVVal = "";
my $maxVEntry = $calMagFrame1->Entry(-textvariable => \$maxVVal,
				     -width => 4,
				     -relief => 'raised'
				 )->pack(-side => "left", -anchor => "w");
$calMethodSearchTree->addChild("maxVEntry",$maxVEntry,undef);
$balloon->attach($maxVEntry,-msg => "Set maximum calibrator V");


# calMagFrame row 2
my $calMagFrame2 = $calMagFrameR->Frame()->pack( -side => "top", -anchor => "w");
# minK and entry box
my $minKLabel = $calMagFrame2->Label(-text => "Min K")->pack(-side => "left", -anchor => "w");
$calMethodSearchTree->addChild("minKLabel",$minKLabel,undef);
my $minKVal = "";
my $minKEntry = $calMagFrame2->Entry(-textvariable => \$minKVal,
				     -width => 4,
				     -relief => 'raised'
				 )->pack(-side => "left", -anchor => "w");
$calMethodSearchTree->addChild("minKEntry",$minKEntry,undef);
$balloon->attach($minKEntry,-msg => "Set minimum calibrator K");

# maxK and entry box
my $maxKLabel = $calMagFrame2->Label(-text => "Max K")->pack(-side => "left", -anchor => "w");
$calMethodSearchTree->addChild("maxKLabel",$maxKLabel,undef);
my $maxKVal = "";
my $maxKEntry = $calMagFrame2->Entry(-textvariable => \$maxKVal,
				     -width => 4,
				     -relief => 'raised'
				 )->pack(-side => "left", -anchor => "w");
$calMethodSearchTree->addChild("maxKEntry",$maxKEntry,undef);
$balloon->attach($maxKEntry,-msg => "Set maximum calibrator K");

# calMagFrame row 3
my $calMagFrame3 = $calMagFrameR->Frame()->pack( -side => "top", -anchor => "w");
# minN and entry box
my $minNLabel = $calMagFrame3->Label(-text => "Min N")->pack(-side => "left", -anchor => "w");
$calMethodSearchTree->addChild("minNLabel",$minNLabel,undef);
my $minNVal = "";
my $minNEntry = $calMagFrame3->Entry(-textvariable => \$minNVal,
				     -width => 4,
				     -relief => 'raised'
				 )->pack(-side => "left", -anchor => "w");
$calMethodSearchTree->addChild("minNEntry",$minNEntry,undef);
$balloon->attach($minNEntry,-msg => "Set minimum calibrator N");

# maxN and entry box
my $maxNLabel = $calMagFrame3->Label(-text => "Max N")->pack(-side => "left", -anchor => "w");
$calMethodSearchTree->addChild("maxNLabel",$maxNLabel,undef);
my $maxNVal = "";
my $maxNEntry = $calMagFrame3->Entry(-textvariable => \$maxNVal,
				     -width => 4,
				     -relief => 'raised'
				 )->pack(-side => "left", -anchor => "w");
$calMethodSearchTree->addChild("maxNEntry",$maxNEntry,undef);
$balloon->attach($maxNEntry,-msg => "Set maximum calibrator N");

# calMagFrame row 4
my $calMagFrame4 = $calMagFrameR->Frame()->pack( -side => "top", -anchor => "w");
# minL and entry box
my $minLLabel = $calMagFrame4->Label(-text => "Min L")->pack(-side => "left", -anchor => "w");
$calMethodSearchTree->addChild("minLLabel",$minLLabel,undef);
my $minLVal = "";
my $minLEntry = $calMagFrame4->Entry(-textvariable => \$minLVal,
				     -width => 4,
				     -relief => 'raised'
				 )->pack(-side => "left", -anchor => "w");
$calMethodSearchTree->addChild("minLEntry",$minLEntry,undef);
$balloon->attach($minLEntry,-msg => "Set minimum calibrator L");

# maxL and entry box
my $maxLLabel = $calMagFrame4->Label(-text => "Max L")->pack(-side => "left", -anchor => "w");
$calMethodSearchTree->addChild("maxLLabel",$maxLLabel,undef);
my $maxLVal = "";
my $maxLEntry = $calMagFrame4->Entry(-textvariable => \$maxLVal,
				     -width => 4,
				     -relief => 'raised'
				 )->pack(-side => "left", -anchor => "w");
$calMethodSearchTree->addChild("maxLEntry",$maxLEntry,undef);
$balloon->attach($maxLEntry,-msg => "Set maximum calibrator L");

### end calSearchFrame4

# build and pack the buttons which control the calibrator-related objects we've
# defined above

## end calibrator frame
################

###############################################################
## the Simbad Query Frame
my $simbadFrameL = $optFrame->Frame(
				-relief => "groove",
				-borderwidth => "2",
				)
				;
my $simbadFrameR = $optFrame->Frame(
				-relief => "groove",
				-borderwidth => "2",
				)
				;
$simbadFrameL->grid($simbadFrameR, -sticky => "nsew");

# simbad enable button
my $simbadSel = 0;
my $simbadTree;
my $simbadButton = $simbadFrameL->Checkbutton(-text => "Simbad Query:",
					     -variable => \$simbadSel,
					     # -relief => 'raised',
					     -command => sub { $simbadTree->updateChildren(); }
					     )->pack(-side => "left");
$simbadTree = new widgetTree("simbad",$simbadButton,\$simbadSel);
$balloon->attach($simbadButton,-msg => "Query Simbad on calibrators");
### end Simbad query frame

# simbad row 1
my $simbadFrame1 = $simbadFrameR->Frame()->pack(-side => "top", -anchor => "w");

my $commonNamesSel = 0;
my $commonNamesButton = $simbadFrame1->Checkbutton(-text => "Common Names",
						 -variable => \$commonNamesSel,
						 -relief => 'raised'
						)->pack(-side => "left", -anchor => "w");
$simbadTree->addChild("commonNames",$commonNamesButton,\$commonNamesSel);
$balloon->attach($commonNamesButton,-msg => "Retrieve Simbad common names");

my $simbadMeasSel = 0;
my $simbadMeasButton = $simbadFrame1->Checkbutton(-text => "Simbad Meas",
						 -variable => \$simbadMeasSel,
						 -relief => 'raised'
						)->pack(-side => "left", -anchor => "w");
$simbadTree->addChild("simbadMeas",$simbadMeasButton,\$simbadMeasSel);
$balloon->attach($simbadMeasButton,-msg => "Retrieve Simbad measurement report");

my $simbadBrowserSel = 0;
my $simbadBrowserButton = $simbadFrame1->Checkbutton(-text => "Browser",
						 -variable => \$simbadBrowserSel,
						 -relief => 'raised'
						)->pack(-side => "left", -anchor => "w");
$simbadTree->addChild("simbadBrowser",$simbadBrowserButton,\$simbadBrowserSel);
$balloon->attach($simbadBrowserButton,-msg => "Spawn Simbad query in external browser");



###############################################################
## the timing options frame

my $delayVal = "";	# updated when location changes so must declare this one early

## the timing main left and right frames
my $timingFrameL = $optFrame->Frame(
				    -relief => "groove",
				    -borderwidth => "2"
				);
my $timingFrameR = $optFrame->Frame(
				    -relief => "groove",
				    -borderwidth => "2"
				);
$timingFrameL->grid($timingFrameR, -sticky => "nsew");


## Now the timing button itself.  Logic for rendering/unrendering
## subframes and options to timing computations
my $timingSel = 0;
my $timingTree;
my $timingButton = $timingFrameL->Checkbutton(-text => "Timing Info:",
					     -variable => \$timingSel,
					     # -relief => 'raised',
					     -command => sub { $timingTree->updateChildren(); }
					     )->pack(-side => "left");
$timingTree = new widgetTree("timing",$timingButton,\$timingSel);
$balloon->attach($timingButton,-msg => "Compute annual and diurnal accessibility");


## timing subframe 1
my $timingFrame1 = $timingFrameR->Frame()->pack( -side => "top", -anchor => "w");

my $ocDisplaySel = 0;
my $ocDisplayButton = $timingFrame1->Checkbutton(-text => "Observing Calendar Display",
						    -variable => \$ocDisplaySel,
						    -relief => 'raised'
						    )->pack( -side => "left", -anchor => "w");
$timingTree->addChild("ocDisplay",$ocDisplayButton,\$ocDisplaySel);
$balloon->attach($ocDisplayButton,-msg => "Observation calendar display (ocGui)");

my $timingDisplaySel = 0;
my $timingDisplayButton = $timingFrame1->Checkbutton(-text => "Timing Display",
						    -variable => \$timingDisplaySel,
						    -relief => 'raised'
						    )->pack( -side => "left", -anchor => "w");
$timingTree->addChild("timingDisplay",$timingDisplayButton,\$timingDisplaySel);
$balloon->attach($timingDisplayButton,-msg => "Timing display (tGui)");

my $uvDisplaySel = 0;
my $uvDisplayButton = $timingFrame1->Checkbutton(-text => "u-v Display",
						 -variable => \$uvDisplaySel,
						 -relief => 'raised'
						    )->pack( -side => "left", -anchor => "w");
$timingTree->addChild("uvDisplay",$uvDisplayButton,\$uvDisplaySel);
$balloon->attach($uvDisplayButton,-msg => "u-v track display (uvTool)");

		   
## timing subframe 2
my $timingFrame2 = $timingFrameR->Frame()->pack( -side => "top", -anchor => "w");

my $defInterferometer = $cfg->defaultInterferometer;
my $location = $cfg->defaultLocation;
my $interferometer = $interferometers::InterferometerList{$defInterferometer};
my @blList = @{ $$interferometer->{Baselines} };
my $blDesig = $cfg->defaultBaseline; ## "all";
my %blSelect;
my $baselineMenu  ## define $baselineMenu -- used in $locationMenu callback
    = $timingFrame2->Menubutton(-text => "Baseline Selection...",
				-menuitems => [['command' => "All",
						-command => sub
						{
						    $blDesig = "all";
						    my $blTmp;
						    foreach $blTmp (@blList)
						    {
							$blSelect{$blTmp} = 1;
						    } ## end foreach
						}],
					       ['command' => "None",
						-command => sub
						{
						    $blDesig = "none";
						    my $blTmp;
						    foreach $blTmp (@blList)
						    {
							$blSelect{$blTmp} = 0;
						    } ## end foreach
						}],
					       "-"],
				-relief => 'raised'
				);
$timingTree->addChild("baselineMenu",$baselineMenu,undef);
$balloon->attach($baselineMenu,-msg => "Set interferometer baselines");


# The location menu. The menu entries are added individually after
# the menu widget has been created.
my $locLabel = $timingFrame2->Label(-text => "Location:")->pack( -side => "left", -anchor => "w");
$timingTree->addChild("locLabel",$locLabel,undef);
my $locationMenu =
    $timingFrame2->Optionmenu(-variable => \$location,
			      -command => sub
			      {
				  $location = shift;
				  ## When the user selects a location
				  ## we rebuild the baseline menu
				  ## options to accomodate the selection
				  $interferometer
				      = $interferometers::InterferometerList{$location};

				  ## deselect any old baselines
				  foreach my $blTmp (@blList)
				  {
				      $blSelect{$blTmp} = 0;
				  } ## end deselect old baselines
				  ## Remove old baselineMenu entries
				  if ($baselineMenu->menu->index('end') > 3)
				  {
				      $baselineMenu->menu->delete(4,'end');
				  }

				  ## Make new baseline list
				  @blList = @{ $$interferometer->{Baselines} };
				  ## repack the baseline options here
				  foreach my $blTmp (@blList)
				  {
				      ## add menu items
				      $baselineMenu->checkbutton(-label => $blTmp,
								 -command => sub
								 {
								     $blDesig = "individual";
								 },
								 -variable => \$blSelect{$blTmp});
				  } ## end foreach
				  ## Reset baseline to default all
				  $blDesig = "all";

				  # reset default delay range
				  $delayVal = $$interferometer->{DelayRange};
			      }
				)->pack( -side => "left", -anchor => "w");
$timingTree->addChild("locationMenu",$locationMenu,undef);
$balloon->attach($locationMenu,-msg => "Set observing location/interferometer");
$baselineMenu->pack( -side => "left", -anchor => "w");

# now populate the option menu using the list of known interferometers
my $locIndex = 0;
my %locationHash;	# hash containing menu entry index for each known location
foreach my $locname (sort keys %interferometers::InterferometerShortList)
{
    my $interferometer = $interferometers::InterferometerShortList{$locname};
    my $loc = $$interferometer->{Location};	# the official location name

    $locationMenu->addOptions([$locname=>$loc]); # menu maps long name to official name

    #print STDERR "DEBUG: gcGui: location fullname $locname loc $loc index $locIndex\n";
    $locationHash{$loc} = $locIndex;	# stash index of this menu entry
    $locIndex++;
} # end foreach location
# add location aliases in case they will be recognized when supplied as defaults
foreach my $locAlias (sort keys %interferometers::InterferometerList)
{
    my $interferometer = $interferometers::InterferometerList{$locAlias};
    my $loc = $$interferometer->{Location};	# the official location name
    my $idx = $locationHash{$loc};
    if (($locAlias ne $loc) && (defined $idx))
    {
	#print STDERR "DEBUG: gcGui: location alias $locAlias loc $loc index $idx\n";
	$locationHash{$locAlias} = $idx;
    }
}

# set location to default
$location = $cfg->defaultLocation;
if (defined $locationHash{$location})
{
    $locationMenu->menu->invoke($locationHash{$location});
}

## end timing subframe 2

## timing subframe 3
## The za, bias selections go in subframe 3
my $timingFrame3 = $timingFrameR->Frame()->pack( -side => "top", -anchor => "w");

# za
my $zaLabel = $timingFrame3->Label(-text => "Zenith Angle Limit (deg):")->pack( -side => "left", -anchor => "w");
$timingTree->addChild("za",$zaLabel,undef);
my $zaVal = "";
my $zaEntry = $timingFrame3->Entry(-textvariable => \$zaVal,
				     -width => 4,
				     -relief => 'raised'
				    )->pack( -side => "left", -anchor => "w");
$timingTree->addChild("zaEntry",$zaEntry,undef);
$balloon->attach($zaEntry,-msg => "Set zenith angle constraint (deg)");

# bias
my $biasLabel = $timingFrame3->Label(-text => "Delay Bias (m):")->pack( -side => "left", -anchor => "w");
$timingTree->addChild("bias",$biasLabel,undef);
my $biasVal = "";
my $biasEntry = $timingFrame3->Entry(-textvariable => \$biasVal,
				     -width => 4,
				     -relief => 'raised'
				    )->pack( -side => "left", -anchor => "w");
$timingTree->addChild("biasEntry",$biasEntry,undef);
$balloon->attach($biasEntry,-msg => "Set delay bias (LDL) constraint (m)");


# delay options subframe
my $timingFrame3a = $timingFrameR->Frame()->pack( -side => "top", -anchor => "w");
my $delayFrameL = $timingFrame3a->Frame(
				    -relief => "groove",
				    -borderwidth => "2"
				);
my $delayFrameR = $timingFrame3a->Frame(
				    -relief => "groove",
				    -borderwidth => "2"
				);
$delayFrameL->grid($delayFrameR, -sticky => "nsew");

my $delayLabel = $delayFrameL->Label(-text => "Set delay limits:")->pack( -side => "left", -anchor => "w");
$timingTree->addChild("delayLabel",$delayLabel,undef);
$balloon->attach($delayLabel,-msg => "Set delay limits");

### Delay Type (symmetric/asymmetric)
# radio button values:
my $delayTypeDef = 1; 		# default delay
my $delayTypeSym = 2; 		# symmetric delay
my $delayTypeAsym = 3; 		# asymmetric delay
my $delayType = $delayTypeDef;  # current radiobutton setting

### default delay (don't override getCal defaults)

my $delayFrame0 = $delayFrameR->Frame(
				    # -relief => "groove",
				    -borderwidth => "2"
				)->pack( -side => "top", -anchor => "w");
my $delayTypeDefSel = 1;
my $delayTypeDefButton = $delayFrame0->Radiobutton(-text => "Use default delay limits",
					    -variable => \$delayType,
					    -value => $delayTypeDef,
					    #-relief => 'raised',
					    -command => sub { &setDelayType; $timingTree->updateChildren(); }
				    )->pack( -side => "left", -anchor => "w");
$timingTree->addChild("delayDef",$delayTypeDefButton,\$delayTypeDefSel);
$balloon->attach($delayTypeDefButton,-msg => "Use default delay constraints");

### symmetric delay
my $delayFrame1 = $delayFrameR->Frame(
				    # -relief => "groove",
				    -borderwidth => "2"
				)->pack( -side => "top", -anchor => "w");
# symmetric delay radiobutton
my $delayTypeSymSel = 1;
my $delayTypeSymTree;
my $delayTypeSymButton = $delayFrame1->Radiobutton(-text => "Use symmetric delay:  ",
					    -variable => \$delayType,
					    -value => $delayTypeSym,
					    #-relief => 'raised',
					    -command => sub { &setDelayType; $timingTree->updateChildren(); }
				    )->pack( -side => "left", -anchor => "w");
$delayTypeSymTree = $timingTree->addChild("delaySym",$delayTypeSymButton,\$delayTypeSymSel);
$balloon->attach($delayTypeSymButton,-msg => "Set symmetric delay constraint +/- (m)");

# delay (symmetric)
# $delayVal is already declared above
my $delaySymLabel = $delayFrame1->Label(-text => "+/- (m)")->pack( -side => "left");
$delayTypeSymTree->addChild("delaySymLabel",$delaySymLabel,undef);
my $delayEntry = $delayFrame1->Entry(-textvariable => \$delayVal,
				     -width => 4,
				     -relief => 'raised'
				    )->pack( -side => "left", -anchor => "w");
$delayTypeSymTree->addChild("delayEntry",$delayEntry,undef);
$balloon->attach($delayEntry,-msg => "Set delay constraint (m)");

### asymmetric delay
my $delayFrame2 = $delayFrameR->Frame(
				    # -relief => "groove",
				    -borderwidth => "2"
				)->pack( -side => "top", -anchor => "w");
# asymmetric delay radiobutton
my $delayTypeAsymSel = 0;
my $delayTypeAsymTree;
my $delayTypeAsymButton = $delayFrame2->Radiobutton(-text => "Use asymmetric delay:",
					    -variable => \$delayType,
					    -value => $delayTypeAsym,
					    #-relief => 'raised',
					    -command => sub { &setDelayType; $timingTree->updateChildren(); }
				    )->pack( -side => "left", -anchor => "w");
$delayTypeAsymTree = $timingTree->addChild("delayAsym",$delayTypeAsymButton,\$delayTypeAsymSel);
$balloon->attach($delayTypeAsymButton,-msg => "Set asymmetric delay constraint (m)");

# delayMin
my $delayMinLabel = $delayFrame2->Label(-text => "Min (m):")->pack( -side => "left");
$delayTypeAsymTree->addChild("delayMinLabel",$delayMinLabel,undef);
my $delayMinVal = "";
my $delayMinEntry = $delayFrame2->Entry(-textvariable => \$delayMinVal,
				     -width => 4,
				     -relief => 'raised'
				    )->pack( -side => "left", -anchor => "w");
$delayTypeAsymTree->addChild("delayMinEntry",$delayMinEntry,undef);
$balloon->attach($delayMinEntry,-msg => "Set delayMin constraint (m)");

# delayMax
my $delayMaxLabel = $delayFrame2->Label(-text => "Max (m):")->pack( -side => "left");
$delayTypeAsymTree->addChild("delayMaxLabel",$delayMaxLabel,undef);
my $delayMaxVal = "";
my $delayMaxEntry = $delayFrame2->Entry(-textvariable => \$delayMaxVal,
				     -width => 4,
				     -relief => 'raised'
				    )->pack( -side => "left", -anchor => "w");
$delayTypeAsymTree->addChild("delayMaxEntry",$delayMaxEntry,undef);
$balloon->attach($delayMaxEntry,-msg => "Set delayMax constraint (m)");
### end timing subframe 3

### Date selection options now go in subframe 4
my $timingFrame4 = $timingFrameR->Frame()->pack( -side => "top", -anchor => "w");

my $selDate = 0;
my $selDateTree;
my $selDateButton = $timingFrame4->Checkbutton(-text => "Select Date",
					      -variable => \$selDate,
					      -relief => 'raised',
					      -command => sub { $selDateTree->updateChildren(); }
					     )->pack( -side => "left", -anchor => "w");
$selDateTree = $timingTree->addChild("selDate",$selDateButton,\$selDate);
$balloon->attach($selDateButton,-msg => "Set observing date (UTC)");

my ($sec,$min,$hour,$day,$mon,$year,$wday,$yday,$isdst) = gmtime(time);
##$mon += 1; ## These months start at zero
$yday += 1; ## Adding 1 to conform to "date -u +%j" (Jan 1 = 001)
$year += 1900;  ## We know how the dates come out of gmtime...
my $month;
my $monthMenu = $timingFrame4->Optionmenu(-options => [["January",1], ["Febuary",2], ["March",3],
						      ["April",4], ["May",5], ["June",6],
						      ["July",7], ["August",8], ["September",9],
						      ["October",10], ["November",11], ["December",12]],
					 -command => sub {$month = shift},
					 -variable => \$month
					 )->pack( -side => "left", -anchor => "w");
$selDateTree->addChild("month",$monthMenu,undef);
$monthMenu->menu->invoke($mon);
my $dayLabel = $timingFrame4->Label(-text => "day")->pack( -side => "left", -anchor => "w");
$selDateTree->addChild("dayLabel",$dayLabel,undef);
my $dayEntry = $timingFrame4->Entry(-textvariable => \$day,
				   -width => 4,
				   -relief => 'raised'
				   )->pack( -side => "left", -anchor => "w");
$selDateTree->addChild("dayEntry",$dayEntry,undef);
my $yearLabel = $timingFrame4->Label(-text => "year")->pack( -side => "left", -anchor => "w");
$selDateTree->addChild("yearLabel",$yearLabel,undef);
my $yearEntry = $timingFrame4->Entry(-textvariable => \$year,
				    -width => 6,
				    -relief => 'raised',
				    )->pack( -side => "left", -anchor => "w");
$selDateTree->addChild("yearEntry",$yearEntry,undef);
## end timing subframe 4 (dates)

## Null Depth
my $nulldepthSel = 0;
my $nulldepthButton = $timingFrameR->Checkbutton(-text => "Compute Null Depth Estimates",
					      -variable => \$nulldepthSel,
					      -relief => 'raised'
						    )->pack( -side => "left", -anchor => "w");
$timingTree->addChild("nulldepthDisplay",$nulldepthButton,\$nulldepthSel);
$balloon->attach($nulldepthButton,-msg => "Compute null depth estimate");

### end timing frame

print STDERR "DEBUG: gcGui: done with timing setup\n" if gcConfig::debug();

###############################################################
## the fbol options frame

my $fbolFrameL = $optFrame->Frame(
				    -relief => "groove",
				    -borderwidth => "2"
				);
my $fbolFrameR = $optFrame->Frame(
				    -relief => "groove",
				    -borderwidth => "2"
				);
$fbolFrameL->grid($fbolFrameR, -sticky => "nsew");

### fbol enable button

my $fbolSel = 0;
my $fbolTree;
my $fbolButton = $fbolFrameL->Checkbutton(-text => "fbol Diameters:",
					  -variable => \$fbolSel,
					  # -relief => 'raised',
					  -command => sub { $fbolTree->updateChildren(); }
					  )->pack(-side => "left");
$fbolTree = new widgetTree("fbolButton",$fbolButton,\$fbolSel);
$balloon->attach($fbolButton,-msg => "Perform fbol SED modelling");

# fbol row 1
my $fbolFrame1 = $fbolFrameR->Frame()->pack(-side => "top", -anchor => "w");

my $twoMassSel = 0;
my $twoMassButton = $fbolFrame1->Checkbutton(-text => "2Mass IR Phot",
						   -variable => \$twoMassSel,
						   -relief => 'raised'
						   )->pack(-side => "left");
$fbolTree->addChild("twoMass",$twoMassButton,\$twoMassSel);
$balloon->attach($twoMassButton,-msg => "Retrieve 2Mass Photometry for output");

my $IRASSel = 0;
my $IRASButton = $fbolFrame1->Checkbutton(-text => "IRAS Long WL Phot",
                                              -variable => \$IRASSel,
                                              -relief => 'raised'
                                              )->pack(-side => "left");
$fbolTree->addChild("IRAS",$IRASButton,\$IRASSel);
$balloon->attach($IRASButton,-msg => "Retrieve IRAS long wavelength photometry");


my $constTempSel = 0;
my $constTempButton = $fbolFrame1->Checkbutton(-text => "Constrain Temp",
					      -variable => \$constTempSel,
					      -relief => 'raised'
					      )->pack(-side => "left", -anchor => "w");
$fbolTree->addChild("constTemp",$constTempButton,\$constTempSel);
$balloon->attach($constTempButton,-msg => "Constrain eff temp in SED model");

# fbol subframe 2
my $fbolFrame2 = $fbolFrameR->Frame()->pack(-side => "top", -anchor => "w");

my $photometrySel = 0;
my $photometryButton = $fbolFrame2->Checkbutton(-text => "Save Photometry",
						-variable => \$photometrySel,
						-relief => 'raised'
					      )->pack(-side => "left", -anchor => "w");
$fbolTree->addChild("photometry",$photometryButton,\$photometrySel);
$balloon->attach($photometryButton,-msg => "Save retrieved photometry");


# fbol plot button
my $plotSel = 0;
my $plotTree;
my $plotButton = $fbolFrame2->Checkbutton(-text => "fbol Plots",
					  -variable => \$plotSel,
					  -relief => 'raised',
					  -command => sub { $plotTree->updateChildren(); }
					  )->pack(-side => "left", -anchor => "w");
$plotTree = $fbolTree->addChild("plot",$plotButton,\$plotSel);
$balloon->attach($plotButton,-msg => "Make SED model plots");

# PostScript hardcopy suboptions to plot button
my $postScriptSel = 0;
my $postScriptButton = $fbolFrame2->Checkbutton(-text => "PostScript",
					     -variable => \$postScriptSel,
					     -relief => 'raised'
					  )->pack(-side => "left", -anchor => "w");
$plotTree->addChild("postScript",$postScriptButton,\$postScriptSel);
$balloon->attach($postScriptButton,-msg => "PostScript SED model plots");

my $epsSel = 0;
my $epsButton = $fbolFrame2->Checkbutton(-text => "EPS",
					     -variable => \$epsSel,
					     -relief => 'raised'
					  )->pack(-side => "left", -anchor => "w");
$plotTree->addChild("eps",$epsButton,\$epsSel);
$balloon->attach($epsButton,-msg => "EPS SED model plots");

my $pngSel = 0;
my $pngButton = $fbolFrame2->Checkbutton(-text => "PNG",
					     -variable => \$pngSel,
					     -relief => 'raised'
					  )->pack(-side => "left", -anchor => "w");
$plotTree->addChild("png",$pngButton,\$pngSel);
$balloon->attach($pngButton,-msg => "PNG SED model plots");

### end fbol frames

print STDERR "DEBUG: gcGui: done with fbol setup\n" if gcConfig::debug();


###############################################################
## the output options frame
my $addlOptsFrameL = $optFrame->Frame(
				    -relief => "groove",
				    -borderwidth => "2",
				    );
my $addlOptsFrameR = $optFrame->Frame(
				    -relief => "groove",
				    -borderwidth => "2",
				    );
$addlOptsFrameL->grid($addlOptsFrameR, -sticky => "nsew");

my $addlOptsLabel = $addlOptsFrameL->Label(-text => "Additional Options:")->pack( -side => "left");

# output opts row 1 - message level
my $addlOptsFrame1 = $addlOptsFrameR->Frame()->pack(-side => "top", -anchor => "w");
my $addlOptsMessageLabel = $addlOptsFrame1->Label(-text => "Message Level:")->pack( -side => "left", -anchor => "w");
my $messageLevel;
my @messageLevelButtons;
foreach (qw(Quiet Normal Verbose))
{
    push @messageLevelButtons, $addlOptsFrame1->Radiobutton(-text => $_,
				       -value => $_,
				       -variable => \$messageLevel,
				       )->pack(-side => "left");
} # end foreach

# output opts row 2 - output format
my $addlOptsFrame2 = $addlOptsFrameR->Frame()->pack(-side => "top", -anchor => "w");

my $outputFormatLabel = $addlOptsFrame2->Label(-text => "Output Format:")->pack( -side => "left", -anchor => "w");

# output format selection
# @formatList contains the list of formats getCal knows about (the -format option)
# we'll create a reverse mapping (name=>num) in %formatNum
my @formatList = ("new","old","sky");
my %formatNum;
for (my $idx=0; $idx < @formatList; $idx++)
{
    $formatNum{$formatList[$idx]} = $idx;
}
my $outputFormat = $cfg->defaultFormat();
my $outputFmtOpt = $formatNum{$outputFormat};
my $formatMenu = $addlOptsFrame2->Optionmenu(-options => [
		["New",$formatNum{"new"}],
		["Old",$formatNum{"old"}],
		["Keck Sky",$formatNum{"sky"}],
		],
		 -command => sub {$outputFmtOpt = shift},
		 -variable => \$outputFmtOpt);
$balloon->attach($formatMenu,-msg => "Select getCal output format");
$formatMenu->menu->invoke($outputFmtOpt);
$formatMenu->pack(-side => "left");

# output opts row 3 - plx
my $addlOptsFrame3 = $addlOptsFrameR->Frame()->pack(-side => "top", -anchor => "w");

my $plxSel = 0;
my $plxButton = $addlOptsFrame3->Checkbutton(-text => "Parallax",
					      -variable => \$plxSel,
					      -relief => 'raised'
					      )->pack(-side => "left");
$balloon->attach($plxButton,-msg => "Write parallax in output line");

my $renameSel = 0;
my $renameButton = $addlOptsFrame3->Checkbutton(-text => "Rename targets to HD/HIP ID",
					      -variable => \$renameSel,
					      -relief => 'raised'
					      )->pack(-side => "left");
$balloon->attach($renameButton,-msg => "Rename target with preferred (e.g. HD/HIP) desig");

my $hdcSel = 0;
my $hdcButton = $addlOptsFrame3->Checkbutton(-text => "Use PTI-style HDC prefix",
					      -variable => \$hdcSel,
					      -relief => 'raised'
					      )->pack(-side => "left");
$balloon->attach($hdcButton,-msg => "Use PTI-style HDC prefix in output desigs");


# output opts row 4 - misc
my $addlOptsFrame4 = $addlOptsFrameR->Frame()->pack(-side => "top", -anchor => "w");

my $calScriptSel = 0;
my $calScriptButton = $addlOptsFrame4->Checkbutton(-text => "Cal Script Composition",
						   -variable => \$calScriptSel,
						   -relief => 'raised'
						   )->pack(-side => "left");
$balloon->attach($calScriptButton,-msg => "Compose provisional calibration script (for wbCalib)");

my $xEphemSel = 0;
my $xEphemButton = $addlOptsFrame4->Checkbutton(-text => "xEphem Display",
						   -variable => \$xEphemSel,
						   -relief => 'raised'
						   )->pack(-side => "left");
$balloon->attach($xEphemButton,-msg => "Write output objects to Xephem FIFO");

# debug options
my $noOpSel = 0;
my $noOpButton = $addlOptsFrame4->Checkbutton(-text => "No-Op",
						   -variable => \$noOpSel,
						   -relief => 'raised'
						   );
$balloon->attach($noOpButton,-msg => "Display getCal command without running it");

my $debugSel = 0;
my $debugButton = $addlOptsFrame4->Checkbutton(-text => "Debug",
						   -variable => \$debugSel,
						   -relief => 'raised'
						   );

if (gcConfig::debug())
{
    $noOpButton->pack(-side => "left");
    $debugButton->pack(-side => "left");
}



### end output options frame

print STDERR "DEBUG: gcGui: done with output options setup\n" if gcConfig::debug();

###############################################################
## the control frame (dispatch, reset, quit)
my $ctrlFrame = $MW->Frame(-relief => 'groove',
			   -borderwidth => "2"
			   )->pack(-side => "bottom", -fill => "x");
my $dispatch = $ctrlFrame->Button(-text => 'Dispatch',
				  -relief => 'raised',
				  -command => sub{submit()}
				  )->pack(-side => "left", -anchor => "w");
$balloon->attach($dispatch,-msg => "Start getCal run...");

my $resetOpts = $ctrlFrame->Button(-text => 'Reset',
				   -command => sub{&reset}
				   )->pack(-side => "left", -anchor => "w");
$balloon->attach($resetOpts,-msg => "Reset gcGui options");



my $quit = $ctrlFrame->Button(-text => 'Quit',
			      -command => sub{exit}
			      )->pack(-side => "right", -anchor => "w");
$balloon->attach($quit,-msg => "Exit gcGui");

## working-in the logo
my $logoFile = findFile("NExScI_logo_micro.gif",$cfg->gcdocs . "/images", $cfg->gcdata . "/images");
if (defined $logoFile)
{
    my $logo = $ctrlFrame->Photo(-file => $logoFile);
    my $logoB =
	$ctrlFrame->Button(-text => '',
			   -relief => 'flat',
			   -borderwidth => '0',
			   -command => sub {&aboutgcGui();},
			   -image => $logo)->pack(-side => "right");
    $balloon->attach($logoB,-msg => "NASA Exoplanet Science Institute");
} ## end logo file

### end control frame

## Call the reset routine as a workaround for some apparent
## strangeness in perl 5.6.0

print STDERR "DEBUG: gcGui: done setting up everything, calling reset\n" if gcConfig::debug();
&reset();
print STDERR "DEBUG: gcGui: done with reset, entering MainLoop\n" if gcConfig::debug();

# force focus to the object entry box
$objEntry->focus;

MainLoop;  ## Perl/TK main event loop

exit(0);  ## end gcGui  #################################################




#########################################################################
## submit -- compose the getCal invocation and process the output
#########################################################################
sub submit
{
## declarations submit  #################################################

    # This subroutine composes the appropriate command-line string
    # invocation to getCal, submits it, and records and displays the
    # return information.  In terms of processing, this is the heart
    # of gcGui

    # We start by composing the invocation command
    my $gcStr = "getCal"; # duh


## begin submit  ########################################################
    ## Object designation composition
    if ($objVal)
    {
	if ($nameType eq "name")
	{
	    $objVal =~ s/\s+/_/g;  # replace spaces with underlines
	    $gcStr .= " -targetName $objVal";
	}
	elsif ($nameType eq "HD")
	{
	    $objVal =~ s/\D+//g;   # eliminate all non-digits
	    $gcStr .= " -targetHD $objVal";
	    $objVal = join ' ', "HD",$objVal;
	}
	elsif ($nameType eq "HIP")
	{
	    $objVal =~ s/\D+//g;   # eliminate all non-digits
	    $gcStr .= " -targetHIP $objVal";
	    $objVal = join ' ', "HIP",$objVal;
	}
	elsif ($nameType eq "Pos")
	{
	    ## parsing coordinates.  For now assume user has given a
	    ## space-separated RA/Dec string
	    my ($raStr,$decStr,$nonsense) = split /\s+/, $objVal;
	    $gcStr .= " -searchRA $raStr  -searchDec $decStr";
	}
	    
    } ## end if $objVal
    else
    {
	print "gcGui: no object designation supplied\n";
	# force focus to the object entry box
	$objEntry->focus;
	return;
    }

    ## Photometry search/output options
    # 2Mass K-band photometry
    if ($photMagK == $photMagK2MassVal)
    {
	$gcStr .= " -K2Mass";
    }

    if ($NbandSel)
    {
	$gcStr .= " -Nband";
    }

    if ($LbandSel)
    {
	$gcStr .= " -Lband";
    }

    if ($LprimeSel)
    {
	$gcStr .= " -Lprime";
    }

    if ($photMagN == $photMagNIRASVal)
    {
      $gcStr .= " -NIRAS";
    }

    if ($useMeasurementSel)
    {
      $gcStr .= " -useMeasurement";
    }


    ## Calibrator specifications
    if (!$calSel)
    {
	# not processing calibrators
	$gcStr .= " -noCal";
    }
    else
    {
	# processing calibrators

	if ($calMethod == 1) # user-specified calibrators
	{
	    # clean up the user-spec cal string and turn into -cal arguments

	    my $calStr = $calUserVal;

	    # split on commas, then process each calibrator
	    my @calList = split ',', $calStr;
	    foreach my $cal (@calList)
	    {
		$cal =~ s/^\s+//g;  # zap leading whitespace
		$cal =~ s/\s+$//g;  # zap trailing whitespace
		$cal =~ s/\s+/_/g;  # replace interior whitespace with underlines
		$cal =~ s/[^a-zA-Z0-9_\-\+\.]//g; # eliminate potentially troublesome punctutation
		if ($cal) # if there's anything left, use it
		{
		    print STDERR "DEBUG: gcGui: using cal \"$cal\"\n" if gcConfig::debug();
		    $gcStr .= " -cal $cal";
		}
	    }
	}
	else # calibrator search
	{
	    # geometry spec
	    if ($searchRadiusVal ne "")
	    {
		$gcStr .= " -searchRadius $searchRadiusVal";
	    }

	    # luminosity class specification
	    if ($lcVSel)
	    {
		$gcStr .= " -lClass V";
	    }
	    if ($lcIIISel)
	    {
		$gcStr .= " -lClass III";
	    }
	    if ($lcISel)
	    {
		$gcStr .= " -lClass I";
	    }
	    if ($maxADSel && ($maxADVal ne ""))
	    {
		$gcStr .= " -maxAD $maxADVal";
	    }

	    ## calibrator brightness spec
	    if ($minVVal)
	    {
		$gcStr .= " -minV $minVVal";
	    }
	    if ($maxVVal)
	    {
		$gcStr .= " -maxV $maxVVal";
	    }
	    if ($minKVal)
	    {
		$gcStr .= " -minK $minKVal";
	    }
	    if ($maxKVal)
	    {
		$gcStr .= " -maxK $maxKVal";
	    }
	    if ($minNVal)
	    {
		$gcStr .= " -minN $minNVal";
	    }
	    if ($maxNVal)
	    {
		$gcStr .= " -maxN $maxNVal";
	    }
	    if ($minLVal)
	    {
		$gcStr .= " -minL $minLVal";
	    }
	    if ($maxLVal)
	    {
		$gcStr .= " -maxL $maxLVal";
	    }
	} # end cal specification

	# request calibration script
	if ($calScriptSel)
	{
	    $gcStr .= " -calScript";
	}
    }


    ## simbad query and suboptions
    if ($simbadSel)
    {
	$gcStr .= " -simbadInteractiveQuery";
	if ($commonNamesSel)
	{
	    $gcStr .= " -commonNames";
	}

	if ($simbadMeasSel)
	{
	    $gcStr .= " -meas";
	}

    } ## end if $simbadSel

    ## timing information and suboptions
    if ($timingSel)
    {
	$gcStr .= " -timing -obsCalendar";  ## invoke both timing and obsCalendar

	if ($nulldepthSel)
	{
	    $gcStr .= " -nullDepth";  ## invoke nullDepth
	}

	if ($location)
	{
	    $gcStr .= " -location $location";
	}

	if ($blDesig =~ m/all/)  ## requesting all baselines
	{
	    $gcStr .= " -baseline all";
	}
	elsif ($blDesig =~ m/none/)  ## requesting no baselines
	{
	    $gcStr .= " -baseline none";
	}
	else ## selecting individual baselines
	{
	    my $blTmp;
	    foreach $blTmp (keys %blSelect)
	    {
		if ($blSelect{$blTmp})
		{
		    $gcStr .= " -baseline $blTmp";
		}
	    } ## end foreach

	} ## end 


	if ($zaVal ne "")
	{
	    $gcStr .= " -zenithAngle $zaVal";
	}

	if ($biasVal ne "")
	{
	    $gcStr .= " -biasOffset $biasVal";
	}

	if ($delayTypeSymSel && ($delayVal ne ""))
	{
	    $gcStr .= " -delayLimit $delayVal";
	}

	if ($delayTypeAsymSel && ($delayMinVal ne ""))
	{
	    $gcStr .= " -delayMin $delayMinVal";
	}

	if ($delayTypeAsymSel && ($delayMaxVal ne ""))
	{
	    $gcStr .= " -delayMax $delayMaxVal";
	}

	if ($selDate)
	{
	    $gcStr .= " -month $month";
	    if ($day ne "")
	    {
		$gcStr .= " -day $day";
	    }
	    if ($year ne "")
	    {
		$gcStr .= " -year $year";
	    }
	}
    } ## end if $timingSel

    ## fbol calculation and suboptions
    if ($fbolSel)
    {
	$gcStr .= " -fbol -strom -geneva -noU";  ## default photometric specifications

	if ($twoMassSel)
	{
	    $gcStr .= " -2Mass";
	}

	if ($IRASSel)
	{
	    $gcStr .= " -longWL";
	}

	if ($constTempSel)
	{
	    $gcStr .= " -constrainTemp";
	}

	if ($photometrySel)
	{
	    $gcStr .= " -savePhotometry";
	}

	if ($plotSel)
	{
	    if ((!$postScriptSel) && (!$epsSel) && (!$pngSel))
	    {
		$gcStr .= " -plot"; ## screen plot
	    }
	    if ($postScriptSel)
	    {
		$gcStr .= " -ps"; ## PostScript plots
	    }
	    if ($epsSel)
	    {
		$gcStr .= " -eps"; ## Encapsulated PostScript
	    }
	    if ($pngSel)
	    {
		$gcStr .= " -png"; ## Portable Network Graphics
	    }
	}

    } # end if $fbolSel

    ## Additional output options
    # output format
    my $tmpFmt = $formatList[$outputFmtOpt];
    $gcStr .= " -format=" . $formatList[$outputFmtOpt];

    # message level
    # default message level of 1 requires no output
    if ($messageLevel eq "Quiet")
    {
	$gcStr .= " -noMessages";
    }
    elsif ($messageLevel eq "Normal")
    {
	# default, do nothing
    }
    elsif ($messageLevel eq "Verbose")
    {
	$gcStr .= " -verbose";
    }

    # parallax
    if ($plxSel)
    {
	$gcStr .= " -plx";
    } ## end parallax

    # rename
    if ($renameSel)
    {
	$gcStr .= " -rename";
    } ## end rename

    # hdc
    if ($hdcSel)
    {
	$gcStr .= " -identHDC";
    } ## end rename

    # xephem
    if ($xEphemSel)
    {
	$gcStr .= " -xEphemDisplay";
    }

    # debug
    if ($debugSel)
    {
	$gcStr .= " -debug";
    }

    ## end gcStr composition/invocation composition


    ## Next, execution/dispach of the getCal query
    print "\nexecuting $gcStr\n";

    if ($noOpSel)
    {
	# No-Op option for debugging - just print the command we
	# would execute, without actually doing it.
	print "No-Op selected, skipping execution\n";
	return;
    }

    ## This can take a while -- Make the cursor into a watch to
    ## signify that getCal is off doing its thing...
    $MW->Busy;
    my $gcReturn = `$gcStr` or die;  # actual getCal invocation
    print $gcReturn;
    ## We're done -- Revert to normal cursor
    $MW->Unbusy;


    ## create a text window to display the return info
    my $tl = $MW->Toplevel();

    $tl->title("getCal Return -- $objVal");
    my $text = $tl->Scrolled("ROText",
			     -wrap => "none",
			     -width => 110, -height => 32,
			     -relief => "groove"
			     )->pack(-side => "top",
				-expand => "yes",
				-fill => "both");

    # Add the text with some color-coding to help the reader
    $text->tagConfigure('colorBlue',-foreground => "blue");
    $text->tagConfigure('colorRed',-foreground => "red");

    $text->insert('end',"$gcStr\n\n",'colorBlue');

    my @tmpTxt = split/\n/, $gcReturn;
    my ($line,@targetSet);
    foreach $line (@tmpTxt)
    {
	$line .= "\n";
	if ($line =~ m/^\#/)
	{
	    $text->insert('end',$line);
	}
	else
	{
	    $text->insert('end',$line,'colorRed');
	}

	my ($desig,$rest) = split /\s+/, $line,2;
	if (($desig) && ($rest =~ m/^\d{2}\s+\d{2}\s+\d+/))  ## GUI/Astrometry line
	{
	    push @targetSet,&simbadName($desig);
	} ## end extracting object designation

    } # end foreach

    ###$text->insert('end',$gcReturn);

    my $tlCntrl = $tl->Frame(-relief => "groove",
			     -borderwidth => "2"
			     )->pack(-side => "bottom", -fill => "x");

    my $b1 = 
	$tlCntrl->Button(-text => "Save",
			 -relief => "raised",
			 -command => sub
			 {

			     ## my $FSref = $MW->FileSelect(-directory => $ENV{PWD});
			     ## print "FileSelect returned $FSref\n";

			     my $fn = $tl->getSaveFile(-initialdir => $ENV{PWD},
						       -initialfile => "$objVal.getCal");
			     if ($fn)
			     {
				 open (OUTPUT, ">>$fn") or print "Couldnt open $fn";
				 print OUTPUT "$gcStr\n\n";
				 print OUTPUT "$gcReturn\n\n";
				 close OUTPUT;
				 print "getCal output appended to $fn\n";
			     }
			 }
			 )->pack(-side => "left");
    $balloon->attach($b1,-msg => "Save getCal output...");

    my $b2 = 
	$tlCntrl->Button(-text => "Simbad Browser",
			 -relief => "raised",
			 -command => sub
			 {
			     ## Issue simbad query on returned object list
			    my $cmd = "simbadInteractiveQuery --browser";
			    foreach my $t (@targetSet) {
				$cmd .= " \"$t\"";
			    }
			    system $cmd;
			 })->pack(-side => "left");
    $balloon->attach($b2,-msg => "Simbad query in external browser");


    my $b3 =
	$tlCntrl->Button(-text => "XEphem Output",
			 -relief => "raised",
			 -command => sub
			 {
			     ## route getCal return through xeConvert
			     &xephemOutput($MW, $gcReturn);
			 })->pack(-side => "left");
    $balloon->attach($b3,-msg => "Save objects in XEphem catalog file...");
    


    my $b4 = $tlCntrl->Button(-text => "Close",
			      -relief => "raised",
			      -command => sub {$tl->withdraw}
			      )->pack(-side => "right");
    $balloon->attach($b4,-msg => "Close output window");


    if ($simbadSel && $simbadBrowserSel && scalar (@targetSet))  ## User wants to see Simbad output in browser
    {
	## Issue simbad query on returned object list
	my $cmd = "simbadInteractiveQuery --browser";
	foreach my $t (@targetSet) {
	    $cmd .= " \"$t\"";
	}
	system $cmd;
    } ## end simbad browser query
    

    if ($timingSel && $timingDisplaySel) ## user wants timing display
    {
	# push the return info to the timing display gui
	system "echo \"$gcReturn\" | tGui \&";
    }

    if ($timingSel && $uvDisplaySel) ## user wants u-v display
    {
	# push the return info to the uv display gui
	system "echo \"$gcReturn\" | uvTool \&";
    }

    if ($timingSel && $ocDisplaySel) ## user wants observing calendar display
    {
	# push the return info to the obsCalendar display gui
	system "echo \"$gcReturn\" | ocGui \&";
    }


    if ($calScriptSel)  ## if we generated a calibration script...
    {
	## We're going to display the calScript in a window of its own
	## First, extract the calScript portion of the getCal output

	my @tmp = split /\#\#\# Begin Calibration script \#\#\#\n|\#\#\# End Calibration script \#\#\#/, $gcReturn;

	## Shove the calibration script into its own window
	## create a text window to display the cs
	my $cs = $MW->Toplevel();

	$cs->title("getCal Calibration Script -- $objVal");
	my $text = $cs->Scrolled("Text",
				 -wrap => "none",
				 -width => 90, -height => 32,
				 -relief => "groove",
				 -exportselection => 1
				 )->pack(-side => "top",
				    -expand => "yes",
				    -fill => "both");

	# Add the text with some color-coding to help the reader
	$text->tagConfigure('colorRed',-foreground => "red");

	my @tmpTxt = split/\n/, $tmp[1];
	my $line;
	foreach $line (@tmpTxt)
	{
	    $line .= "\n";
	    if ($line =~ m/^\#/)
	    {
		$text->insert('end',$line);
	    }
	    else
	    {
		$text->insert('end',$line,'colorRed');
	    }
	} # end foreach

	my $csCntrl = $cs->Frame(-relief => "groove",
				 -borderwidth => "2"
				 )->pack(-side => "bottom", -fill => "x");

	my $b5 = 
	    $csCntrl->Button(-text => "Save",
			     -relief => "raised",
			     -command => sub
			     {

				 my $fn = $cs->getSaveFile(-initialfile => "cs.$objVal");
				 if ($fn)
				 {
				     open (OUTPUT, ">$fn") or print "Couldnt open $fn";
				     $tmp[1] = $text->get('1.0','end');
				     print OUTPUT "$tmp[1]\n\n";
				     close OUTPUT;
				     print "\ncalibration script output written to $fn\n";
				 }
			     }
			     )->pack(-side => "left");
	$balloon->attach($b5,-msg => "Save calibration script...");


	my $b6 = $csCntrl->Button(-text => "Close",
				  -relief => "raised",
				  -command => sub {$cs->withdraw}
				  )->pack(-side => "right");
	$balloon->attach($b6,-msg => "Close calibration script window");

    
    } ## end rendering calScript



    if ($photometrySel)  ## if we generated a photometry dataset...
    {
	## We're going to display the fbol photometry in a window of
	## its own.  First, extract the photometry portion of the
	## getCal output
	my @tmp = split /\#\#\# Begin fbol Photometry \#\#\#\n|\#\#\# End fbol Photometry \#\#\#/, $gcReturn;
	if (defined $tmp[1])
	{
	    &processPhotometry($objVal,$tmp[1]);
	}
    } ## end rendering photometry


} ## end submit  ########################################################


#########################################################################
## reset -- restore default gui settings
#########################################################################
sub reset
{

## begin reset  #########################################################

    ## reset all options/switches/entries to start-up values
    print STDERR "gcGui: resetting\n" if gcConfig::debug();

    # object designation
    $objEntry->delete(0,'end');
    $objVal ="";
    $nameType = "name";

    # phot options reset
    $photMagKModelButton->select();
    $NbandButton->deselect();
    $photMagNModelButton->select();
    $NbandTree->updateChildren();
    $LbandButton->deselect();
    $LprimeButton->deselect();
    $useMeasurementButton->deselect();

    # calibrator specification options
    $calButton->select();
    $calMethod = $calMethodSearch;
    &setCalMethod;
    $calMethodSearchButton->select();
    $lcVButton->select();
    $lcIIIButton->deselect();
    $lcIButton->deselect();
    $maxADVal = "";
    $maxADButton->deselect();

    $searchRadiusVal = $cfg->calSearchRadius;

    $minVVal = $cfg->calMinV;
    $maxVVal = $cfg->calMaxV;
    $minKVal = $cfg->calMinK;
    $maxKVal = $cfg->calMaxK;
    $minNVal = $cfg->calMinN;
    $maxNVal = $cfg->calMaxN;
    $minLVal = $cfg->calMinL;
    $maxLVal = $cfg->calMaxL;

    $calTree->updateChildren();

    # simbad options reset
    $simbadButton->deselect();
    $commonNamesButton->deselect();
    $simbadMeasButton->deselect();
    $simbadBrowserButton->deselect();
    $simbadTree->updateChildren();

    # timing options reset
    $timingButton->deselect();
    $selDateButton->deselect();
    $ocDisplayButton->deselect();
    $timingDisplayButton->deselect();
    $uvDisplayButton->deselect();
    $nulldepthButton->deselect();
    $location = $cfg->defaultLocation;
    if (defined $locationHash{$location})
    {
	$locationMenu->menu->invoke($locationHash{$location});

	# reset defaults specific to this location
	my $interferometer = $interferometers::InterferometerList{$location};
	$delayVal = $$interferometer->{DelayRange};
    }
    $blDesig = $cfg->defaultBaseline;

    $zaVal = $cfg->zenithAngleLimit();	# default zenith angle limit
    $biasVal = $cfg->defaultBiasOffset();	# default bias (LDL) value

    $delayType = $delayTypeDef;
    &setDelayType;
    $delayTypeDefButton->select();
    $delayMinVal = "";
    $delayMaxVal = "";

    my ($rsec,$rmin,$rhour,$rday,$rmon,$ryear,$rwday,$ryday,$risdst) = gmtime(time);
    ##$rmon += 1; ## These months start at zero
    $day = $rday;
    $year = $ryear + 1900;  ## We know how the dates come out of gmtime...
    $monthMenu->menu->invoke($rmon);

    $timingTree->updateChildren();

    # fbol options reset
    $fbolButton->deselect();
    $twoMassButton->deselect();
    $IRASButton->deselect();
    $constTempButton->deselect();
    $photometryButton->deselect();
    $plotButton->deselect();
    $postScriptButton->deselect();
    $epsButton->deselect();
    $pngButton->deselect();

    $fbolTree->updateChildren();

    # additional options

    $messageLevelButtons[1]->select(); # messageLevel 1=Normal
    $formatMenu->menu->invoke($formatNum{$cfg->defaultFormat()});
    $plxButton->deselect();
    $calScriptButton->deselect();
    $xEphemButton->deselect();
    $noOpButton->deselect();
    $debugButton->deselect();

}  ## end reset  ########################################################


#########################################################################
## dispHelp -- display help message contained in a file
#########################################################################
sub dispHelp
{

## declarations dispHelp  ###############################################
    my $filename = shift @_;

## begin dispHelp  ######################################################
    if (!defined $filename)
    {
	print "Error: undefined document link in gcGui\n";
	return;
    }

    ## Basically we open a text window and display the help file -- in
    ## all it's glory.

    ## create a text window to display the return info
    my $helpWindow = $MW->Toplevel();
    $helpWindow->title("help -- $filename");
    my $text = $helpWindow->Scrolled("ROText",
				     -wrap => "none",
				     -width => 90, -height => 32,
				     -relief => "groove"
				     )->pack(-side => "top",
				       -expand => "yes",
				       -fill => "both");

    # dump the contents of the help file into the text window
    my $helpMsg = `cat $filename` or die;

    $text->insert('end',$helpMsg);

    # add a close button
    my $hwCntrl = $helpWindow->Frame(-relief => "groove",
				     -borderwidth => "2"
				     )->pack(-side => "bottom", -fill => "x");
    $hwCntrl->Button(-text => "Close",
		     -relief => "raised",
		     -command => sub {$helpWindow->withdraw}
		     )->pack(-side => "right");

    $helpWindow->bind('<Return>' => sub {$helpWindow->withdraw});


}  ## end dispHelp  #####################################################


#########################################################################
## aboutgcGui -- display an "about" window for gcGui
#########################################################################
sub aboutgcGui
{

## begin aboutgcGui  ####################################################
    ## aboutgcGui creates a splash screen with gcGui information

    my $aboutWindow = $MW->Toplevel();
    $aboutWindow->title("about gcGui");

    ## Display logo image in a button
    my $logoFile = findFile("NExScI_logo_tiny.gif",$cfg->gcdocs . "/images", $cfg->gcdata . "/images");
    return if (!defined $logoFile);

    my $logo = $aboutWindow->Photo(-file => $logoFile);
    $aboutWindow->Button(-text => '',
			 -command => sub {$aboutWindow->withdraw},
			 -image => $logo)->pack(-side => "top", -fill => "x");
    $aboutWindow->Label(-text => "http://nexsci.caltech.edu")->pack(-side => "top", -fill => "x");
    $aboutWindow->Label(-text => "gcGui $REVISION ($PKGSTR)")->pack(-side => "top", -fill => "x");
    $aboutWindow->Label(-text => $gcGuiAttributes::gcGuiCopyright)->pack(-side => "top", -fill => "x");
    $aboutWindow->bind('<Return>' => sub { $aboutWindow->withdraw });
} ## end aboutgcGui  ####################################################


#########################################################################
## processPhotometry
#########################################################################
sub processPhotometry
{
## declarations processPhotometry  ######################################
    my $objName = shift @_;
    my $photometry = shift @_;

## begin processPhotometry  #############################################

    ## Shove the photometry into its own window
    ## create a text window to display
    my $phot = $MW->Toplevel();

    $phot->title("getCal fbol Photometry -- $objName");
    my $text = $phot->Scrolled("Text",
			       -wrap => "none",
			       -width => 90, -height => 32,
			       -relief => "groove",
			       -exportselection => 1
			       )->pack(-side => "top",
			           -expand => "yes",
				   -fill => "both");

    # Add the text with some color-coding to help the reader
    $text->tagConfigure('colorRed',-foreground => "red");

    ##my @tmpTxt = split/\n/, $photometry;
    foreach my $line (split /\n/, $photometry)
    {
	$line .= "\n";
	if ($line =~ m/^\#/)
	{
	    $text->insert('end',$line);
	}
	else
	{
	    $text->insert('end',$line,'colorRed');
	}
    } # end foreach

    my $photCntrl = $phot->Frame(-relief => "groove",
				 -borderwidth => "2"
				 )->pack(-side => "bottom", -fill => "x");

    my $b1 = 
	$photCntrl->Button(-text => "Save",
			   -relief => "raised",
			   -command => sub
			   {
			       my $fn = $phot->getSaveFile(-initialfile => "photometry.$objName");
			       if ($fn)
			       {
				   open (OUTPUT, ">$fn") or print "Couldnt open $fn";
				   my $txt = $text->get('1.0','end');
				   print OUTPUT "$txt\n\n";
				   close OUTPUT;
				   print "\nphotometry written to $fn\n";
			       }
			   }
			   )->pack(-side => "left");
    $balloon->attach($b1,-msg => "Save photometry to file...");


    my $b2 = 
	$photCntrl->Button(-text => "Run fbol",
			   -relief => "raised",
			   -command => sub
			   {
			       my $cancel;
			       ($fbolArgument,$cancel) = &editFbolArgument($fbolArgument);
			       if (!$cancel)
			       {
				   $photometry = $text->get('1.0','end');
				   my $fbolOutput
				       = `echo "$photometry" | fbol -stdin $fbolArgument` or die;
				   &dispText($MW,$fbolOutput,"fbol Output",undef);
			       } ## end no $cancel
			   }
			   )->pack(-side => "left");
    $balloon->attach($b2,-msg => "Run fbol on this photometry...");



    my $b3 = $photCntrl->Button(-text => "Close",
				-relief => "raised",
				-command => sub {$phot->withdraw}
				)->pack(-side => "right");
    $balloon->attach($b3,-msg => "Close photometry window");


## end processPhotometry  ###############################################
}



#########################################################################
## editFbolArgument -- facility for controling timing recomputation
#########################################################################
sub editFbolArgument
{
    ## editFbolArgument opens up a simple window to get the fbol
    ## arguments from the user
## declarations editFbolArgument  #######################################
    my $argument = shift @_;

## begin editFbolArgument  ##############################################
    my $editArg = $MW->Toplevel();

    $editArg->title("fbol argument input");


    my $argFrame = $editArg->Frame(-relief => "raised",
				   -borderwidth => "2"
				   )->pack(-side => "top", -fill => "x");

    $argFrame->Label(-text => "fbol arguments: ",
		      )->pack(-side => "left", -anchor => "w");
    
    $argFrame->Entry(-textvariable => \$argument,
		      -relief => 'raised', -width => 60
		      )->pack(-side => "left", -anchor => "w", -fill => "x", -expand => 1);

    ## A second frame for buttons (AFB 12/2003)
    my $buttonFrame = $editArg->Frame(-relief => "raised",
				      -borderwidth => "2"
				      )->pack(-side => "top", -fill => "x");

    my $b1 = 
	$buttonFrame->Button(-text => "Go",
			     -relief => 'groove',
			     -borderwidth => "5",
			     -command => sub 
			     {
				 $editArg->destroy();
			     }
			     )->pack(-side => "left");
    $balloon->attach($b1,-msg => "Run fbol...");

    my $b2 = 
	$buttonFrame->Button(-text => "fbol Help",
			     -relief => 'groove',
			     -borderwidth => "5",
			     -command => sub 
			     {
				 &dispHelp($fbolHelp);
			     }
			     )->pack(-side => "left");
    $balloon->attach($b2,-msg => "Display fbol help...");

    my $cancel = 0;  ## Give the option to cancel re-fbol
    my $b3 = 
	$buttonFrame->Button(-text => "Cancel",
			     -relief => 'groove',
			     -borderwidth => "5",
			     -command => sub 
			     {
				 $cancel = 1;
				 $editArg->destroy();
			     }
			     )->pack(-side => "right");
    $balloon->attach($b3,-msg => "Cancel fbol run");

    $editArg->bind('<Return>' => sub  ## bind Return to go
		    {
			$editArg->destroy();
		    });

    $editArg->waitWindow();

    return ($argument,$cancel);

} ## end editFbolArgument  ##############################################

# callback for calibrator search method radiobuttons
# updates individual status flags according to current radiobutton state
sub setCalMethod
{
    $calMethodUserSel = ($calMethod == $calMethodUser);
    $calMethodSearchSel = ($calMethod == $calMethodSearch);
}

sub setDelayType
{
    $delayTypeDefSel = ($delayType == $delayTypeDef);
    $delayTypeSymSel = ($delayType == $delayTypeSym);
    $delayTypeAsymSel = ($delayType == $delayTypeAsym);
}
