package widgetTree;
my $MODVERSION = '$Id: widgetTree.pm,v 1.5 2008/11/12 23:28:13 sgroom Exp $';

##########################################################################
# NASA Exoplanet Science Institute (NExScI) -- http://nexsci.caltech.edu #
#       Copyright (c) 1997-2008 California Institute of Technology       #
#                     U.S. Government Sponsorship                        #
# Under contract with the National Aeronautics and Space Administration  #
##########################################################################

#
# widgetTree.pm - This module provides a widgetTree "class" for tracking
# a hierarchy of Tk widgets, so that when a widget is enabled/disabled,
# it's children can also be automatically enabled/disabled. Storing these
# relationships as a hierarchical data structure simplies managing these
# relationships when there is a large number of objects involved.
#

use warnings;
use strict;

use Exporter();
our @ISA = qw(Exporter);
our @EXPORT = qw();

use gcConfig;
use Tk;

my $cfg = config();


# create and return a new widgetTree object
sub new
{
    my $class = shift @_;
    my $name = shift @_;	# the name of the object - useful for debugging
    my $widget = shift @_;	# the Tk widget reference
    my $selRef = shift @_;	# a reference to this widget's associated variable, if any


    # see if this widget's configure method understands -state option
    my @opts = $widget->configure();	# dump list of valid options
    my $hasConfigState = 0;
    optloop: foreach my $opt (@opts)
    {
	if ((@$opt)[0] =~ m/-state/)
	{
	    $hasConfigState = 1;
	    last optloop;
	}
    }
    if (gcConfig::debug() >= 3)
    {
	if ($hasConfigState)
	{
	    print STDERR "widgetTree::new: widget $name has -state\n";
	}
	else
	{
	    print STDERR "widgetTree::new: widget $name MISSING -state\n";
	}
    }

    my $objRef = { name => $name,
		   widget => $widget,
    		   selRef => $selRef,
		   children => [], 	# empty list of children
		   hasConfigState => $hasConfigState
		 };
    bless $objRef;
    return $objRef;
}


# create and add a child to a widgetTree object
sub addChild
{
    my $this = shift @_;
    my $name = shift @_;
    my $widget = shift @_;
    my $selRef = shift @_;

    my $objRef = new widgetTree($name,$widget,$selRef);

    my $hasConfigState = $objRef->{hasConfigState};
    print STDERR "widgetTree::addChild: created widget $name hasConfigState $hasConfigState\n" if (gcConfig::debug() >= 2);

    push @{ $this->{children} }, $objRef;
    return $objRef;
}


# accessor to retrieve name
sub getName
{
    my $this = shift @_;
    return $this->{name};
}


# accessor to retrieve selection state
sub getSel
{
    my $this = shift @_;
    my $selRef = $this->{selRef};
    return undef if (!defined $selRef);
    my $sel = $$selRef;
    return $sel;
}


# recursively enable or disable the descendent children of a widget,
# depending on the current selection state of the given (parent) widget
sub updateChildren
{
    my $this = shift @_;
    my $state = shift @_; # state of the parent (optional)

    my $name = $this->{name};
    my $defState = 0;	# default state for widgets with no selection state variable

    # if state was not provided, use the parent widget's associated variable as
    # an indicator of state, and use that in deciding how to update children.
    if (!defined $state)
    {
	my $selRef = $this->{selRef};
	if (defined $selRef)
	{
	    $state = $$selRef;	# use selection state of this widget
	}
	else
	{
	    $state = $defState;
	}
    }
    
    print STDERR "DEBUG: widgetTree::updateChildren $name state=$state\n" if (gcConfig::debug() >= 2);
    foreach my $child (@{$this->{children}})
    {
	my $childname = $child->{name};
	my $widget = $child->{widget};
	my $hasConfigState = $child->{hasConfigState};

	print STDERR "DEBUG: widgetTree::updateChildren $name updating child $childname hasConfigState $hasConfigState\n" if (gcConfig::debug() >= 2);
	if ($state)	
	{
	    # enabling this widget, and possibly descendants

	    # If this widget's configure method understands the -state option,
	    # use that to enable the widget
	    if ($hasConfigState)
	    {
		print STDERR "DEBUG: widgetTree::updateChildren $name enable child $childname\n" if (gcConfig::debug() >= 2);
		$widget->configure(-state => 'normal');	# enable the child widget
	    }

	    # if that child widget is already selected,
	    # recurse to enable the child's children
	    # use child's selection state if it has one.
	    my $cstate = $defState;
	    my $cSelRef = $child->{selRef};
	    if (defined $cSelRef)
	    {
		$cstate = $$cSelRef;
	    }
	    print STDERR "DEBUG: widgetTree::updateChildren $name updating children of $childname state=$cstate\n" if (gcConfig::debug() >= 2);
	    $child->updateChildren($cstate); # children enable/disable according to state of child
	}
	else
	{
	    # disable the child widget and all descendants
	    # If this widget's configure method understands the -state option,
	    # use that to disable the widget
	    if ($hasConfigState)
	    {
		print STDERR "DEBUG: widgetTree::updateChildren $name disable child $childname\n" if (gcConfig::debug() >= 2);
		$widget->configure(-state => 'disabled');
	    }
	    print STDERR "DEBUG: widgetTree::updateChildren $name updating children of $childname state=0\n" if (gcConfig::debug() >= 2);
	    $child->updateChildren(0);	# disable all children, ignoring their own setting
	}
    }
}
