#! @PERL@
# -*- perl -*-
# @configure_input@

our $MODVERSION = '$Id: ocGui.pl,v 1.22 2008/11/12 23:28:13 sgroom Exp $';
our $MODREV = '$Revision: 1.22 $';

##########################################################################
# NASA Exoplanet Science Institute (NExScI) -- http://nexsci.caltech.edu #
#       Copyright (c) 1997-2008 California Institute of Technology       #
#                     U.S. Government Sponsorship                        #
# Under contract with the National Aeronautics and Space Administration  #
##########################################################################

## Observing Calendar GUI -- part of the getCal suite.  A.B. 2/02

##  The Observing Calendar GUI renders target accessibility timing
##  information.  It takes input information calculated by an external
##  observing calendar program (obsCalendar) and renders it for
##  planning purposes.


## Modules and pieces we use...
use strict;
use warnings;
use FindBin qw($RealBin);
use lib "$RealBin/../lib/modules";		# location of getCal Perl modules
use Getopt::Long;
use Time::DaysInMonth;  ## leapyear calculation
use Tk;
use Tk::ROText;
use Tk::FileSelect;
use Tk::Balloon;
use gcConfig;
use guiSubs;
use coordinates;  ## Coordinate handling utilities
use yearDay;  ## yearday calculations
use ocGuiAttributes; ## Colors and such for the oCalGui display
use novasConstants; ## astronomical constants package
use findScript;
use miscArgs;

## Variable declarations
my $cfg = gcConfig::config();
my @ra = split /\s+/, '$Revision: 1.22 $'; my $REVISION = $ra[1];
my $PKGSTR = package_string();

my ($input,$preambleText);
my (%sunriseTransit,%sunriseYday,%sunriseHA,%sunriseYdayHA,
    %midnightTransit,%midnightYday,
    %sunsetTransit,%sunsetYday,%sunsetHA,%sunsetYdayHA,
    %target,%targetOn);
my (@targets);
my $oCalArgument = "-replace ";

## Declare calendar info and define defaults
my ($sec,$min,$hour,$mday,$mon,$calYear,$wday,$yday,$isdst) = gmtime(time);
$mon += 1; ## These months start at zero
$yday += 1; ## Adding 1 to conform to "date -u +%j" (Jan 1 = 001)
$calYear += 1900;  ## We know how the dates come out of gmtime...
my $yearDays = 365;

my $targDesig = "all"; ## default to all targets
my $calOn = 0;
my $yDaysOn = 0;
my $dayLineOn = 0;

# keywords to highlight in text output window
my $highlightKeys = "Observing calendar|for timings in|Using .* Location|Using .* hour transit offsets in accessibilty calculations|is near transit at .* on|is .* available in the sky|is .* hours .*-transit at sun.* on";

## Adding command-line arguments to tGui  A.B. 5/2003
my ($help,$version,$stdin,$filename,$PostScript,$pdf,$gif,$jpg,$png);
my $batchMode;

handleCommonOpts(0,"ocGui.txt");
Getopt::Long::config("pass_through");
my $result = GetOptions("stdin" => \$stdin,
			"filename=s" => \$filename,
			"yearDay" => \$yDaysOn,
			"currentDay|day" => \$dayLineOn,
			"calibrators" => \$calOn,
			"ps|postscript|PostScript|hc" => \$PostScript,
			"pdf" => \$pdf,
			"gif" => \$gif,
			"jpg" => \$jpg,
			"png" => \$png);

## Process some command-line things
if ((defined $PostScript) || (defined $pdf)
    || (defined $gif) || (defined $jpg) || (defined $png))
{
    $batchMode = 1;
} ## end setting batch mode




## Process initial input
if ($stdin) ## explicit stdin direction
{
    &readCalendarRpt(0);
}
elsif ($filename)  ## If given a filename argument
{
    &readCalendarRpt(1,$filename);
}
elsif ((scalar @ARGV) && !($stdin)) ## otherwise check command line
{
    &readCalendarRpt(1,$ARGV[0]);  ## read from filename
}
else ## otherwise nothing left on command line, assume stdin
{
    $stdin = 1;
    &readCalendarRpt(0);  ## read from stdin
}

# quick check to make sure there was some input
if (scalar(@targets) == 0)
{
    print "ocGui: No target information found in input stream, exiting.\n";
    exit(0);
}

## TLE -- 7/12/04 set up original target hash (so -calibrators option behaves)
my $trg;
foreach $trg (@targets)
{
  if (!($target{$trg}))
  {
    $targetOn{$trg} = $calOn;
  }
}


## The main window declaration
my $MW = new MainWindow;
$MW->title("obsCalendar GUI v$REVISION ($PKGSTR)");

my $balloon = $MW->Balloon();  ## help balloon object


## Retrieve pertinent parameters from ocGuiAttributes module
my $xCanvasSize = $ocGuiAttributes::ocGuiXCanvasSize;
my $yCanvasSize = $ocGuiAttributes::ocGuiYCanvasSize;

##################################################
### timing display canvas parameter definition ###
##################################################
my $xScale = $xCanvasSize / ($yearDays * 1.3);

my $xZero = int ($xCanvasSize / 8.0);

my $yZero = int ($yCanvasSize / 10.0);
my $maxY;



## Define the main display canvas
my $displayCanvas = $MW->Scrolled('Canvas',
			   -width => $xCanvasSize,
			   -height => $yCanvasSize
			   );
$balloon->attach($displayCanvas,-msg => "Calendar display canvas");


## At the top -- the menu frame
my $menuFrame = $MW->Frame(-relief => "raised",
			   -borderwidth => "2"
			   )->pack(-side => "top", -fill => "x");

## Do batchMode/hardcopy as required
if (defined $batchMode)
{
    ## Call print routine
    &printCalDiagram();

    exit(0);  ## we've generated the graphic -- exit
} ## end batchMode


# file menu
my $fileMenu = $menuFrame->Menubutton(-text => "File",
				      -menuitems => [## Recompute timings
						     [ 'command' => "Recompute Calendar...",
						       -accelerator => 'Alt+r',
						       -command => sub {
							   my $cancel;
							   ($oCalArgument,$cancel) =
							       &editCalendarArgument($oCalArgument);

							   if (!$cancel)
							   {

							       my $revisedInput
								   = `echo "$input" | obsCalendar -stdin $oCalArgument` or die;
							       &readCalendarRpt(2,$revisedInput);
							       &drawPreamble();
							       &packTargetMenu();
							       &packSimbadMenu();
							       &drawAxis($displayCanvas);
							       &drawTargets($displayCanvas,$targDesig);
							       &drawCurrentDay(\$displayCanvas);
							   } ## end if !$cancel
						       
						       }],
						     "-",
						     ## Open new file
						     [ 'command' => "Open...",
						       -accelerator => 'Alt+o',
						       -command => sub {
							   my $file
							       = $MW->getOpenFile(-defaultextension => ".cat",
										  -filetypes        =>
										  [['catalog files',    ['.cat','.cal','.calendar'] ],
										   ['Text Files',       ['.txt','.text']],
										   ['All Files',        '*',            ],
										   ],
										  -initialfile      => "getopenfile",
										  -title            => "Open calendar file",
										  );
							   if ($file)
							   {
							       &readCalendarRpt(1,$file);
							       &drawPreamble();
							       &packTargetMenu();
							       &packSimbadMenu();
							       &drawAxis($displayCanvas);
							       &drawTargets($displayCanvas,$targDesig);
							       &drawCurrentDay(\$displayCanvas);
							   } ## end if file
						       }],
						     ## Open new file and recompute timings
						     [ 'command' => "Open Through obsCalendar...",
						       -accelerator => 'Alt+O',
						       -command => sub {
							   my $file
							       = $MW->getOpenFile(-defaultextension => ".cat",
										  -filetypes        =>
										  [['catalog files',   ['.cat','.cal','.calendar'] ],
										   ['Text Files',       ['.txt','.text']],
										   ['All Files',        '*',            ],
										   ],
										  -initialfile      => "getopenfile",
										  -title            => "Open catalog file through obsCalendar",
										  );
							   if ($file)
							   {
							       my $cancel;
							       ($oCalArgument,$cancel) =
								   &editCalendarArgument($oCalArgument);
							       if (!$cancel)
							       {
								   my $revisedInput
								       = `obsCalendar $oCalArgument -filename $file` or die;
								   &readCalendarRpt(2,$revisedInput);
								   &drawPreamble();
								   &packTargetMenu();
								   &packSimbadmenu();
								   &drawAxis();
								   &drawTargets($displayCanvas,$targDesig);
								   &drawCurrentDay(\$displayCanvas);
							       } ## end if not $cancel
							   } ## end if file
						       }],
						     "-",
						     [ 'command' => "View Input",
						       -accelerator => 'Alt+v',
						       -command => sub {&dispText($MW,$input,"ocGui input",$highlightKeys)}],
						     [ 'command' => "tGui",
						       -accelerator => 'Alt+t',
						       -command => sub
						       { system "echo \"$input\" | tGui &";}],
						     "-",
						     [ 'command' => "XEphem Catalog",
						       -accelerator => 'Alt+x',
						       -command => sub {&xephemOutput($MW, $input)}],
						     [ 'command' => "XEphem FIFO",
						       -accelerator => 'Alt+X',
						       -command => sub {&xephemOutput($MW, $input,"fifo")}],
						     "-",
						     [ 'command' => "Print...",
						       -accelerator => 'Alt+p',
						       -command => sub {&printCalDiagram()}],
						     "-",
						     [ 'command' => "Quit",
						       -accelerator => 'Alt+q',
						       -command => sub {$MW->withdraw; exit(0); }] ]
				      )->pack(-side => "left", -anchor => "w");
$balloon->attach($fileMenu,-msg => "File menu -- options for input and output");

## Define keyboard bindings for File menu
## Recompute timings
$MW->bind('<Alt-r>' => sub
	  {
	      my $cancel;
	      ($oCalArgument,$cancel) =
		  &editCalendarArgument($oCalArgument);

	      if (!$cancel)
	      {
		  my $revisedInput
		      = `echo "$input" | obsCalendar -stdin $oCalArgument` or die;
		  &readCalendarRpt(2,$revisedInput);
		  &drawPreamble();
		  &packTargetMenu();
		  &packSimbadMenu();
		  &drawAxis($displayCanvas);
		  &drawTargets($displayCanvas,$targDesig);
		  &drawCurrentDay(\$displayCanvas);
	      } ## end if !$cancel
	  });

## Open new file
$MW->bind('<Alt-o>' => sub {
    my $file
	= $MW->getOpenFile(-defaultextension => ".cat",
			   -filetypes        =>
			   [['catalog files',    ['.cat','.cal','.calendar'] ],
			    ['Text Files',       ['.txt','.text']],
			    ['All Files',        '*',            ],
			    ],
			   -initialfile      => "getopenfile",
			   -title            => "Open calendar file",
			   );
    if ($file)
    {
	&readCalendarRpt(1,$file);
	&drawPreamble();
	&drawAxis($displayCanvas);
	&drawTargets($displayCanvas,$targDesig);
	&drawCurrentDay(\$displayCanvas);
    } ## end if file
}); ## end open keyboard accelerator


## Open through obsCalendar
$MW->bind('<Alt-O>' => sub {
    my $file
	= $MW->getOpenFile(-defaultextension => ".cat",
			   -filetypes        =>
			   [['catalog files',   ['.cat','.cal','.calendar'] ],
			    ['Text Files',       ['.txt','.text']],
			    ['All Files',        '*',            ],
			    ],
			   -initialfile      => "getopenfile",
			   -title            => "Open catalog file through obsCalendar",
			   );
    if ($file)
    {
	my $cancel;
	($oCalArgument,$cancel) =
	    &editCalendarArgument($oCalArgument);
	if (!$cancel)
	{
	    my $revisedInput
		= `obsCalendar $oCalArgument -filename $file` or die;
	    &readCalendarRpt(2,$revisedInput);
	    &drawPreamble();
	    &packTargetMenu();
	    &packSimbadmenu();
	    &drawAxis();
	    &drawTargets($displayCanvas,$targDesig);
	    &drawCurrentDay(\$displayCanvas);
	} ## end if not $cancel
    } ## end if file
}); ## end open through obsCalendar accelerator

$MW->bind('<Alt-v>' => sub {&dispText($MW,$input,"ocGui input",$highlightKeys)});
$MW->bind('<Alt-t>' => sub {system "echo \"$input\" | tGui &";});
$MW->bind('<Alt-x>' => sub {&xephemOutput($MW, $input)});
$MW->bind('<Alt-X>' => sub {&xephemOutput($MW, $input,"fifo")});
$MW->bind('<Alt-p>' => sub {&printCalDiagram()});
$MW->bind('<Alt-q>' => sub {$MW->withdraw; exit(0); });

## target menu

my $targetMenu = $menuFrame->Menubutton(-text => "Target",
					-menuitems => [['command' => "Targets",
							-accelerator => 'Alt+t',
							-command => sub
							{
							    ## draw target menu entry -- set $targetOn for
							    ## all targets -- set off for cals
							    my $trg;
							    foreach $trg (@targets)
							    {
								if ($target{$trg})
								{
								    $targetOn{$trg} = 1;
								}
								else
								{
								    $targetOn{$trg} = 0;
								}
							    }
							    $targDesig = "all";
							    $calOn = 0;
							    &drawTargets($displayCanvas,$targDesig);
							    &drawCurrentDay(\$displayCanvas);
							}],
						       ['command' => "Calibrators",
							-accelerator => 'Alt+c',
							-command => sub
							{
							    ## draw calibrator menu entry -- set $targetOn for
							    ## all calibrators
							    ## Toggle the calibrator rendering	
							    $calOn = !($calOn);
							    my $trg;
							    foreach $trg (@targets)
							    {
								if (!($target{$trg}))
								{
								    $targetOn{$trg} = $calOn;
								}
							    }
							    &drawTargets($displayCanvas,$targDesig);
							    &drawCurrentDay(\$displayCanvas);
							}],
						       ['command' => "All",
							-accelerator => 'Alt+a',
							-command => sub
							{
							    ## draw all menu entry -- set $targetOn for
							    ## everybody
							    $targDesig = "all";
							    $calOn = 1;
							    my $trg;
							    foreach $trg (@targets)
							    {
								$targetOn{$trg} = 1;
							    }
							    &drawTargets($displayCanvas,$targDesig);
							    &drawCurrentDay(\$displayCanvas);
							}],
						       ['command' => "Invert",
							-accelerator => 'Alt+i',
							-command => sub
							{
							    ## invert menu entry -- toggle $targetOn for
							    ## everybody
							    $targDesig = "all";
							    $calOn = !($calOn);  ## vestigial
							    my $trg;
							    foreach $trg (@targets)
							    {
								$targetOn{$trg} = !($targetOn{$trg});
							    }
							    &drawTargets($displayCanvas,$targDesig);
							    &drawCurrentDay(\$displayCanvas);
							}],
						       ['command' => "None",
							-accelerator => 'Alt+n',
							-command => sub
							{
							    ## draw no menu entries -- set $targetOn for
							    ## everybody
							    $targDesig = "all";
							    $calOn = 0;
							    my $trg;
							    foreach $trg (@targets)
							    {
								$targetOn{$trg} = 0;
							    }
							    &drawTargets($displayCanvas,$targDesig);
							    &drawCurrentDay(\$displayCanvas);
							}],
						       "-"]
					)->pack(-side => "left", -anchor => "w");
$balloon->attach($targetMenu,-msg => "Target menu -- options for objects");


## Keyboard bindings for the target menu
$MW->bind('<Alt-t>' => sub
	  {
	      ## draw target menu entry -- set $targetOn for
	      ## all targets -- set off for cals
	      my $trg;
	      foreach $trg (@targets)
	      {
		  if ($target{$trg})
		  {
		      $targetOn{$trg} = 1;
		  }
		  else
		  {
		      $targetOn{$trg} = 0;
		  }
	      }
	      $targDesig = "all";
	      $calOn = 0;
	      &drawTargets($displayCanvas,$targDesig);
	      &drawCurrentDay(\$displayCanvas);
	  });
$MW->bind('<Alt-c>' => sub
	  {
	      ## draw calibrator menu entry -- set $targetOn for
	      ## all calibrators
	      ## Toggle the calibrator rendering	
	      $calOn = !($calOn);
	      my $trg;
	      foreach $trg (@targets)
	      {
		  if (!($target{$trg}))
		  {
		      $targetOn{$trg} = $calOn;
		  }
	      }
	      &drawTargets($displayCanvas,$targDesig);
	      &drawCurrentDay(\$displayCanvas);
	  });
$MW->bind('<Alt-a>' => sub	
	  {
	      ## draw all menu entry -- set $targetOn for
	      ## everybody
	      $targDesig = "all";
	      $calOn = 1;
	      my $trg;
	      foreach $trg (@targets)
	      {
		  $targetOn{$trg} = 1;
	      }
	      &drawTargets($displayCanvas,$targDesig);
	      &drawCurrentDay(\$displayCanvas);
	  });
$MW->bind('<Alt-i>' => sub
	  {
	      ## invert menu entry -- toggle $targetOn for
	      ## everybody
	      $targDesig = "all";
	      $calOn = !($calOn);  ## vestigial
	      my $trg;
	      foreach $trg (@targets)
	      {
		  $targetOn{$trg} = !($targetOn{$trg});
	      }
	      &drawTargets($displayCanvas,$targDesig);
	      &drawCurrentDay(\$displayCanvas);
	  });
$MW->bind('<Alt-n>' => sub
	  {
	      ## draw no menu entries -- set $targetOn for
	      ## everybody
	      $targDesig = "all";
	      $calOn = 0;
	      my $trg;
	      foreach $trg (@targets)
	      {
		  $targetOn{$trg} = 0;
	      }
	      &drawTargets($displayCanvas,$targDesig);
	      &drawCurrentDay(\$displayCanvas);
	  });


## Use subroutine to pack individual objects in the target menu
&packTargetMenu(1);


## Simbad query menu
my $simbadMenu = $menuFrame->Menubutton(-text => "Simbad",
					-menuitems => [['command' => "Targets",
							##-accelerator => 'Alt-t',
							-command => sub
							{
							    ## Simbad target menu entry -- query all targets
							    my @targetSet = ();
							    foreach my $trg (@targets)
							    {
								if ($target{$trg})
								{
								    push @targetSet,&simbadName($trg);
								}
							    }
							    if (scalar @targetSet)
							    {
								doSimbadGui($MW,@targetSet);
							    }

							}],
						       ['command' => "Calibrators",
							##-accelerator => 'Alt+c',
							-command => sub
							{
							    ## Simbad calibrator menu entry -- query all calibrators
							    my @targetSet = ();
							    foreach my $trg (@targets)
							    {
								if (!($target{$trg}))
								{
								    push @targetSet,&simbadName($trg);
								}
							    }
							    if (scalar @targetSet)
							    {
								doSimbadGui($MW,@targetSet);
							    }
							}],
						       ['command' => "All",
							##-accelerator => 'Alt+a',
							-command => sub
							{
							    ## Simbad all menu entry -- query all
							    my @targetSet = ();
							    foreach my $trg (@targets)
							    {
								push @targetSet,&simbadName($trg);
							    }
							    if (scalar @targetSet)
							    {
								doSimbadGui($MW,@targetSet);
							    } ## end if @targetSet
							}],
						       "-"]
					)->pack(-side => "left", -anchor => "w");
$balloon->attach($simbadMenu,-msg => "Simbad menu -- Simbad query options");

## Use subroutine to pack individual objects in the simbad menu
&packSimbadMenu(1);




# help menu
# help files
my $ocGuiHelp = findFile("ocGui.txt",$cfg->gcdocs, $cfg->gcdata);
my $ocHelp = findFile("obsCalendar.txt",$cfg->gcdocs, $cfg->gcdata);
$version = findFile("version.txt",$cfg->gcdocs, $cfg->gcdata);
my $copyright = findFile("copyright.txt",$cfg->gcdocs, $cfg->gcdata);

my $helpMenu = $menuFrame->Menubutton(-text => "Help",
				      -menuitems => [ [ 'command' => "ocGui help",
							-accelerator => 'Alt+h',
							-command => sub {&dispHelp($ocGuiHelp)} ],
						      [ 'command' => "about ocGui",
							-command => sub {&aboutocGui()}],
						      "-",
						      [ 'command' => "obsCalendar help",
							-command => sub {&dispHelp($ocHelp)}  ],
						      [ 'command' => "getCal version",
							-command => sub {&dispHelp($version)}  ],
						      [ 'command' => "getCal copyright",
							-command => sub {&dispHelp($copyright)}  ],
						    ]
				      )->pack(-side => "right", -anchor => "w");
$balloon->attach($helpMenu,-msg => "Help menu -- available documentation options");

$MW->bind('<Alt-h>' => sub {&dispHelp($ocGuiHelp)});


# Next a text frame with informational comments about the calendar
# summary

my $msgFrame = $MW->Frame(-relief => 'groove',
			  -borderwidth => "2"
			  )->pack(-side => "top", -fill => "x");

## Replacing the older labels with a text window containing the preamble
my $preamble = $msgFrame->Scrolled("ROText",
				   -wrap => "none",
				   -width => 80, -height => 5,
				   -relief => "groove"
				   )->pack(-side => "left",
					   -expand => "yes",
					   -fill => "x");
$balloon->attach($preamble,-msg => "Calendar preamble");

## Insert the preamble text in this frame
&drawPreamble();


## A time frame to hold current time displays
my $tFrame = $msgFrame->Frame(-relief => 'groove',
			     -borderwidth => "1",
			     )->pack(-side => "right",-fill => "x");

## Logic for current time displays
my ($currentDate,$currentYearDay,$currentUT);
&setCurrentTime();
$tFrame->Label(-text => "Current")->pack(-side => "top",-anchor => "w");
$tFrame->Label(-textvariable => \$currentDate,
	       -relief => 'raised',
	       -width => 10
	       )->pack(-side => "top", -anchor => "w");

$tFrame->Label(-textvariable => \$currentYearDay,
	       -relief => 'raised',
	       -width => 10
	       )->pack(-side => "top", -anchor => "w");

$tFrame->Label(-textvariable => \$currentUT,
	       -relief => 'raised',
	       -width => 10
	       )->pack(-side => "top", -anchor => "w");
$balloon->attach($tFrame,-msg => "Current date and time");



## Define the main display canvas
$displayCanvas->pack(-side => "top",-expand => "yes",-fill => "both");


### Define mini-canvas with display legend
my $yCanvasSize2 = int ($yCanvasSize / 6.0);
my $legendCanvas = $MW->Canvas(-width => $xCanvasSize,
			       -height => $yCanvasSize2,
			       -relief => "groove",
			       -borderwidth => "2"
			       )->pack(-side => "top");

## First line: Legend text, intersect and obscure definitions
my $yPos = int (0.25 * $yCanvasSize2);
my $xPos = int (0.05 * $xCanvasSize);
$legendCanvas->createText($xPos,$yPos,-text => "Legend:");

my $offset = 60.0;
my $length = int (0.05 * $xCanvasSize);
$xPos += int (0.15 * $xCanvasSize);

$legendCanvas->createText($xPos,$yPos,-text => "Target Accessible");
$legendCanvas->createLine($xPos + $offset,$yPos,
			  $xPos + $offset + $length,$yPos,
			  -width => 3,
			  -fill => $ocGuiAttributes::intersectionColor);



## Second Line: sunrise, midnight, and sunset transit definitions
$yPos = int (3/4 * $yCanvasSize2);
$xPos = int (0.1 * $xCanvasSize);

$legendCanvas->createText($xPos,$yPos,-text => "Sunrise Transit");
my $tSize = 5;
my @tCenter = ($xPos + $offset,$yPos);
$legendCanvas->createPolygon($tCenter[0],$tCenter[1] - $tSize,
			     $tCenter[0] - $tSize,$tCenter[1] + $tSize,
			     $tCenter[0] + $tSize,$tCenter[1] + $tSize,
			     -fill => $ocGuiAttributes::sunriseColor);

$xPos += int (0.3 * $xCanvasSize);
$legendCanvas->createText($xPos,$yPos,-text => "Sunset Transit");
@tCenter = ($xPos + $offset,$yPos);
$legendCanvas->createPolygon($tCenter[0],$tCenter[1] + $tSize,
			     $tCenter[0] - $tSize,$tCenter[1] - $tSize,
			     $tCenter[0] + $tSize,$tCenter[1] - $tSize,
			     -fill => $ocGuiAttributes::sunsetColor);

$xPos += int (0.3 * $xCanvasSize);
$legendCanvas->createText($xPos,$yPos,-text => "Midnight Transit");
@tCenter = ($xPos + $offset,$yPos);
$tSize = 6;
$legendCanvas->createPolygon($tCenter[0],$tCenter[1] - $tSize,
			     $tCenter[0] + $tSize,$tCenter[1],
			     $tCenter[0],$tCenter[1] + $tSize,
			     $tCenter[0] - $tSize,$tCenter[1],
			     -fill => $ocGuiAttributes::transitColor);

## A micro-logo for institutional awarness purposes
my $logoFile = findFile("NExScI_logo_micro.gif",$cfg->gcdocs . "/images", $cfg->gcdata . "/images");
my $logo = $legendCanvas->Photo( 'IMG', -file => $logoFile);
$xPos = int (0.94 * $xCanvasSize);
$yPos = int (0.55 * $yCanvasSize2);
$legendCanvas->create('image', $xPos, $yPos,
                     '-image'  => $logo );



## Add a couple of simple control frames at the bottom
my $ctrlFrame = $MW->Frame(-relief => 'groove',
			   -borderwidth => "2"
			   )->pack(-side => "bottom", -fill => "x");


## First control frame holds intersection/ZA/delay.obscured track
## controls

my $ctrlFrame1 = $ctrlFrame->Frame(-relief => 'groove',
				   -borderwidth => "2"
				   )->pack(-side => "top", -fill => "x");




## Second control frame holds UT, sunrise/sunset, calibrator, print,
## and quit options
my $ctrlFrame2 = $ctrlFrame->Frame(-relief => 'groove',
				   -borderwidth => "2"
				   )->pack(-side => "top", -fill => "x");

## Quit button
my $qButton =
    $ctrlFrame2->Button(-text => 'Close',
			-command => sub{$MW->withdraw; exit(0); }
			)->pack(-side => "right", -anchor => "w");
$balloon->attach($qButton,-msg => "Vaya con Dios <Alt+q>");


## Print button
my $pButton = 
    $ctrlFrame2->Button(-text => 'Print',
			-command => sub
			{
			    &printCalDiagram();
			}
			)->pack(-side => "right", -anchor => "w");
$balloon->attach($pButton,-msg => "Print current rendering <Alt+p>");

## xephemOutput control button
my $xButton =
    $ctrlFrame2->Button(-text => 'XEphem',
			-command => sub
			{
			    ## Output the object catalog to xephem
			    &xephemOutput($MW, $input);
			}
			)->pack(-side => "right", -anchor => "w");
$balloon->attach($xButton,-msg => "Output objects to XEphem catalog <Alt+x>");



## Controls for current UT/LST indicator display

## current date control button
my $dButton =
    $ctrlFrame2->Button(-text => 'Current Date',
			-command => sub
			{
			    $dayLineOn = (!$dayLineOn);
			    &drawTargets($displayCanvas,$targDesig);
			    &drawCurrentDay(\$displayCanvas);
			}
			)->pack(-side => "left", -anchor => "w");
$balloon->attach($dButton,-msg => "Toggle current date display <Cntrl+d>");

$MW->bind('<Control-d>' => sub
	  {
	      $dayLineOn = (!$dayLineOn);
	      &drawTargets($displayCanvas,$targDesig);
	      &drawCurrentDay(\$displayCanvas);
	  });


## yDay control button
my $yButton =
    $ctrlFrame2->Button(-text => 'Year Day',
			-command => sub
			{
			    $yDaysOn = (!$yDaysOn);
			    &drawTargets($displayCanvas,$targDesig);
			    &drawCurrentDay(\$displayCanvas);
			}
			)->pack(-side => "left", -anchor => "w");
$balloon->attach($yButton,-msg => "Toggle yday display  <Cntrl+y>");

$MW->bind('<Control-y>' => sub
	  {
	      $yDaysOn = (!$yDaysOn);
	      &drawTargets($displayCanvas,$targDesig);
	      &drawCurrentDay(\$displayCanvas);
	  });

# Controls for calibrator rendering
my $calButton =
    $ctrlFrame2->Button(-text => 'Calibrators',
			-command => sub
			{
			    ## Toggle the calibrator rendering
			    $calOn = !($calOn);  ## toggle
			    my $trg;
			    foreach $trg (@targets)
			    {
				if (!($target{$trg}))
				{
				    $targetOn{$trg} = $calOn;
				}
			    }
			    &drawTargets($displayCanvas,$targDesig);
			    &drawCurrentDay(\$displayCanvas);
			}
			)->pack(-side => "left", -anchor => "w");
$balloon->attach($calButton,-msg => "Toggle calibrator display");


## Tonight report button
my $tButton =
    $ctrlFrame2->Button(-text => 'Tonight',
			-command => sub
			{
			    my $rpt = `timing -noTargets` or die;
			    &dispText($MW, $rpt, "ocGui timing report for tonight",$highlightKeys);
			}
			)->pack(-side => "left", -anchor => "w");
$balloon->attach($tButton,-msg => "Display information for tonight <Cntrl+t>");

$MW->bind('<Control-t>' => sub
	  {
	      my $rpt = `timing -noTargets` or die;
	      &dispText($MW, $rpt, "ocGui timing report for tonight",$highlightKeys);
	  });



### end control frames

### Do the initial rendering
### Draw axis lines
&drawAxis($displayCanvas);

## Draw the targets and the y axis
## (this gets reinvoked each time we change settings) ##
&drawTargets($displayCanvas,$targDesig);

## Draw the current day as required
&drawCurrentDay(\$displayCanvas) if ($dayLineOn);




MainLoop;  ## Perl/TK main event loop

exit;  ## end tGui main





#########################################################################
## xCoord -- compute the canvas x position for a given date
#########################################################################
sub xCoord
{
    ## Subroutine for computing x canvas position for a particular
    ## time.  This is where potential time offsets and clipping logic
    ## should go.
    my $dayArg = shift @_;
    my $xRet = int($xZero + $dayArg * $xScale);

    return $xRet;

} ## end xCoord  ########################################################



#########################################################################
## drawAxis -- draw the calendar horizontal axis
#########################################################################
sub drawAxis
{
## declarations drawAxis  ###############################################
    ## draws calendar horizontal axis
    my $canvas = shift @_;  ## canvas argument

## begin drawAxis  ######################################################
    ## Get rid of any old stuff
    $canvas->delete("CalAxis");


    ## the (persistent) calendar axis
    my $month;
    my @monthNames = ("Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec");
    my $minMonth = 1;
    my $maxMonth = 12;
    $canvas->createText(xCoord(-10),$yZero / 2,-tags => "CalAxis",
			-anchor => "e", -text => "Month");  ## Denoting Month axis
    for ($month = $minMonth; $month <= $maxMonth; $month += 1)
    {
	my $yd = yearDay($calYear,$month,15);
	my $dateText = $monthNames[$month - 1]; ###join '/',$month,"1";
	$canvas->createText(xCoord($yd),$yZero / 2,
			    -tags => "CalAxis",-text => "$dateText");
    }
    $canvas->createText(xCoord($yearDays + 10),$yZero / 2,-tags => "CalAxis",
			-anchor => "w", -text => $calYear);  ## Denoting UT axis

    $canvas->createLine(xCoord(0),$yZero,
			xCoord($yearDays),$yZero,-tags => "CalAxis"); # x axis


} ## end drawAxis  ######################################################




#########################################################################
## readCalendarRpt -- the name says it all
#########################################################################
sub readCalendarRpt
{
    ## readCalendarRpt reads the input stream for observing calendar
    ## annotations

## declarations readCalendarRpt  ########################################
    my ($modeSwitch,$arg) = @_;  ## extract call arguments
    my @inputStream;

## begin readCalendarRpt  ###############################################
    ## Branch on the mode switch used to call readCalendarRpt
    if ($modeSwitch == 0) ## mode == 0 -- read from stdin
    {
	@inputStream = <STDIN>;
    } ## end reading from stdin
    elsif (($modeSwitch == 1) && ($arg))  ## mode == 1 -- read from a filename argument
    {
	open (READ,$arg) or die "Unable to open $arg\n";
	@inputStream = <READ>;
    } ## end reading from file
    elsif (($modeSwitch == 2) && ($arg))  ## mode == 2 -- read from the stream passed in
    {
	@inputStream = split /\n/, $arg;
	foreach my $tmp (@inputStream)  ## need to add the linebreaks back in...
	{
	    $tmp .= "\n";
	}
    } ## end reading from call argument
	

    ## Reading the syntax of the observing calendar report as composed
    ## by the obsCalendar program -- now contained in @inputStream


    ## Initialize storage we use to contain timing information
    $preambleText = $input = "";
    $yearDays = 365;

    %midnightTransit = %sunriseTransit = %sunsetTransit
	= %sunriseHA = %sunriseYdayHA
	= %midnightYday = %sunriseYday = %sunsetYday
	= %sunsetHA = %sunsetYdayHA
	= %target = %targetOn = ();
    @targets = ();

    ## Reset obsCalendar argument
    ## (Now building custom argument based on input calendar)
    $oCalArgument = "-replace ";

    foreach my $line (@inputStream)
    {
	$input .= $line;  ## accumulate input for display/later access

        # Observing calendar (obsCalendar v0.1dev) run at 2/14/2002 UTC, day 2002045
	if ($line =~ m/^\# Observing calendar /)
	{
	    $preambleText .= $line;
	}

        #  for timings in 2002 UTC
	if ($line =~ m/^\#  for timings in/)
	{
	    my @tmp = split /\s+/, $line;
	    $preambleText .= $line;
	    $calYear = $tmp[4];
	    $yearDays++ if (is_leap($calYear));
	    $oCalArgument .= "-year $calYear ";
	}

        # Using Palomar Location (long: -116:51:48   lat: +33:21:24)
	if ($line =~ m/^\# Using (.*) Location/)
	{
	    $preambleText .= $line;
	    my $trg = "-location $1"; ## use captured search target
	    if (!($oCalArgument =~ m/$trg/))  ## avoid duplications
	    {
		$oCalArgument .= "$trg ";
	    }
	} ## end reading location string

        # Using 2.0 hour transit offsets in accessibilty calculations
	if ($line =~ m/^\# Using (.*) hour transit offsets in accessibilty calculations/)
	{
	    $preambleText .= $line;
	    $oCalArgument .= "-hourAngle $1 ";  ## use captured search target
	} ## end reading location string



# HDC40084 05 59 21.780  +49 55 28.344  is near transit at sunrise on 10/4/2002 (2002277)
	if ($line =~ m/^\# \S+ \S+ \S+ \S+  \S+ \S+ \S+  is near transit at sunrise on/)
	{
	    # Extract name, transit time
	    my @tmp = split /\s+/, $line;
	    my $name = $tmp[1];
	    my $tDate = $tmp[14];
	    my $tyDay = $tmp[15];

	    my $tList = join '|', @targets;  ## join with or for match operation
	    if (!($name =~ m/$tList/))
	    {
		my $indx = scalar @targets;
		$targets[$indx] = $name;  ## Create list of targets, avoiding duplications

		$sunriseTransit{$name} = $tmp[14];
		$sunriseYday{$name} = $tmp[15];
		$sunriseYday{$name} =~ s/$calYear//;  ## mask-off the year
		$sunriseYday{$name} =~ s/\(|\)//g;  ## mask-off parentheses

		if ($line =~ m/target|Target/)  ## Add support for differentiating targets and cals
		{
		    $target{$name} = $targetOn{$name} = 1;
		}
		else
		{
		    $target{$name} = $targetOn{$name} = 0;
		}

	    }  ## end if new info

	} # end if sunrise transits logic


# HDC40084 is 2.0 hours pre-transit at sunrise on 9/10/2002 (2002253)
	if ($line =~ m/^\# \S+ is \S+ hours pre-transit at sunrise on/)
	{
	    # Extract name, transit time
	    my @tmp = split /\s+/, $line;
	    my $name = $tmp[1];
	    $sunriseHA{$name} = $tmp[9];
	    $sunriseYdayHA{$name} = $tmp[10];
	    $sunriseYdayHA{$name} =~ s/$calYear//;  ## mask-off the year
	    $sunriseYdayHA{$name} =~ s/\(|\)//g;  ## mask-off parentheses
	} ## end setting supplimental sunrise date



# HDC40084 is near transit at midnight on 12/17/2002 (2002351)
	if ($line =~ m/^\# \S+ is near transit at midnight on/)
	{
	    my @tmp = split /\s+/, $line;

	    my $name = $tmp[1];
	    $midnightTransit{$name} = $tmp[8];
	    $midnightYday{$name} = $tmp[9];
	    $midnightYday{$name} =~ s/$calYear//;  ## mask-off the year
	    $midnightYday{$name} =~ s/\(|\)//g;  ## mask-off parentheses
	} # end midnight transit logic


# HDC40084 is near transit at sunset on 3/7/2002 (2002066)
	if ($line =~ m/^\# \S+ is near transit at sunset on/)
	{
	    my @tmp = split /\s+/, $line;
	    
	    my $name = $tmp[1];
	    $sunsetTransit{$name} = $tmp[8];
	    $sunsetYday{$name} = $tmp[9];
	    $sunsetYday{$name} =~ s/$calYear//;  ## mask-off the year
	    $sunsetYday{$name} =~ s/\(|\)//g;  ## mask-off parentheses
	} # end sunset transit logic


# HDC40084 is 2.0 hours post-transit at sunset on 4/3/2002 (2002093)
	if ($line =~ m/^\# \S+ is \S+ hours post-transit at sunset on/)
	{
	    # Extract name, transit time
	    my @tmp = split /\s+/, $line;
	    my $name = $tmp[1];
	    $sunsetHA{$name} = $tmp[9];
	    $sunsetYdayHA{$name} = $tmp[10];
	    $sunsetYdayHA{$name} =~ s/$calYear//;  ## mask-off the year
	    $sunsetYdayHA{$name} =~ s/\(|\)//g;  ## mask-off parentheses
	} ## end setting supplimental sunset date



# HDC40084 05 59 21.780  +86 55 28.344  is always available in the sky target
	if ($line =~ m/^\# \S+ \S+ \S+ \S+  \S+ \S+ \S+  is always available in the sky/)
	{
	    my @tmp = split /\s+/, $line;
	    
	    my $name = $tmp[1];

	    my $tList = join '|', @targets;  ## join with or for match operation

	    if (!($name =~ m/$tList/))
	    {
		my $indx = scalar @targets;
		$targets[$indx] = $name;  ## Create list of targets, avoiding duplications
	    }

	    $sunriseTransit{$name} = "1/1/";
	    $sunriseTransit{$name} .= $calYear;
	    $sunriseYday{$name} = 1;
	    $sunsetTransit{$name} = "12/31/";
	    $sunsetTransit{$name} .= $calYear;
	    $sunsetYday{$name} = $yearDays;

	    if ($line =~ m/target|Target/)  ## Add support for differentiating targets and cals
	    {
		$target{$name} = $targetOn{$name} = 1;
	    }
	    else
	    {
		$target{$name} = $targetOn{$name} = 0;
	    }

	} ## end always available


# HDC32537 05 06 40.631  -61 35 51.793  is never available in the sky
	if ($line =~ m/^\# \S+ \S+ \S+ \S+  \S+ \S+ \S+  is never available in the sky/)
	{
	    my @tmp = split /\s+/, $line;
	    
	    my $name = $tmp[1];

	    my $tList = join '|', @targets;  ## join with or for match operation

	    if (!($name =~ m/$tList/))
	    {
		my $indx = scalar @targets;
		$targets[$indx] = $name;  ## Create list of targets, avoiding duplications
	    }

	    if ($line =~ m/target|Target/)  ## Add support for differentiating targets and cals
	    {
		$target{$name} = $targetOn{$name} = 1;
	    }
	    else
	    {
		$target{$name} = $targetOn{$name} = 0;
	    }

	} ## end never available


    } # end while



    ## Clean-up the preamble text a little
    $preambleText =~ s/\#//g;

} ## end readCalendarRpt  ###############################################



#########################################################################
## drawTargets -- render accessibility of targets on canvas
#########################################################################
sub drawTargets
{
    ## drawTargets renders the target accesibility limits

## declarations drawTargets  ############################################
    ## Extract call arguments -- canvas, target designations
    my $canvas = shift @_;
    my $trg = shift @_;

## begin drawTargets  ###################################################
    ## Clean-up the old lines
    $canvas->delete("target");
    $canvas->delete("calibrator");
    $canvas->delete("yAxis");


    my $key;
    my $tCount = 0;
    my $ySpace = 20;

    foreach $key (@targets)  ## walk target list in input order
    {
	if ($targetOn{$key}) ## Render the targets that are turned "on"
	{

	    my $tag;
	    if ($target{$key})  ## render targets and calibrators with different tags
	    {
		$tag = "target";
		$tCount++;
	    }
	    else
	    {
		$tag = "calibrator";
		$tCount++;
	    }
	    my $targetY = $yZero + $tCount * $ySpace;
	    # target label
	    $canvas->createText(xCoord(-5),$targetY, -text => $key,
				-anchor => "e", -tags => $tag);  ## left side label
	    $canvas->createText(xCoord($yearDays+5),$targetY,
				-anchor => "w", -text => $key, -tags => $tag);  ## right side label


	    my $offset = 0;
	    my $color;

	    &drawTargetLine($canvas,$sunriseYday{$key},$sunsetYday{$key},
			    $targetY + $offset,3,
			    $ocGuiAttributes::intersectionColor,$tag);

	    if ($sunriseYdayHA{$key})  ## if we have a supplemental sunrise date
	    {
		&drawTargetLine($canvas,$sunriseYdayHA{$key},$sunriseYday{$key},
				$targetY + $offset,3,
				$ocGuiAttributes::intersectionColor,$tag);
	    } ## end supplimental sunrise date


	    if ($sunsetYdayHA{$key})  ## if we have a supplemental sunrise date
	    {
		&drawTargetLine($canvas,$sunsetYday{$key},$sunsetYdayHA{$key},
				$targetY + $offset,3,
				$ocGuiAttributes::intersectionColor,$tag);
	    } ## end supplimental sunrise date


	    # midnight transit marker
	    if ($midnightYday{$key})
	    {
		my @tCenter = (xCoord($midnightYday{$key}),$targetY);
		my $tSize = 6;
		if ($target{$key})  ## render targets and calibrators with slightly different colors
		{
		    $canvas->createPolygon($tCenter[0],$tCenter[1] - $tSize,
					   $tCenter[0] + $tSize,$tCenter[1],
					   $tCenter[0],$tCenter[1] + $tSize,
					   $tCenter[0] - $tSize,$tCenter[1],
					   -fill => $ocGuiAttributes::transitColor,
					   -tags => $tag);
		}
		else  ## otherwise this is a calibrator -- render muted
		{
		    $canvas->createPolygon($tCenter[0],$tCenter[1] - $tSize,
					   $tCenter[0] + $tSize,$tCenter[1],
					   $tCenter[0],$tCenter[1] + $tSize,
					   $tCenter[0] - $tSize,$tCenter[1],
					   -stipple => 'gray75',
					   -fill => $ocGuiAttributes::transitColor,
					   -tags => $tag);
		}


		## sunrise transit marker
		$tSize = 5;
		@tCenter = (xCoord($sunriseYday{$key}),$targetY);
		if ($target{$key})  ## render targets and calibrators with slightly different colors
		{
		    $canvas->createPolygon($tCenter[0],$tCenter[1] - $tSize,
					   $tCenter[0] - $tSize,$tCenter[1] + $tSize,
					   $tCenter[0] + $tSize,$tCenter[1] + $tSize,
					   -fill => $ocGuiAttributes::sunriseColor,
					   -tags => $tag);
		}
		else  ## otherwise this is a calibrator -- render muted
		{
		    $canvas->createPolygon($tCenter[0],$tCenter[1] - $tSize,
					   $tCenter[0] - $tSize,$tCenter[1] + $tSize,
					   $tCenter[0] + $tSize,$tCenter[1] + $tSize,
					   -stipple => 'gray75',
					   -fill => $ocGuiAttributes::sunriseColor,
					   -tags => $tag);
		}

		## sunset transit marker
		@tCenter = (xCoord($sunsetYday{$key}),$targetY);
		if ($target{$key})  ## render targets and calibrators with slightly different colors
		{
		    $canvas->createPolygon($tCenter[0],$tCenter[1] + $tSize,
					   $tCenter[0] - $tSize,$tCenter[1] - $tSize,
					   $tCenter[0] + $tSize,$tCenter[1] - $tSize,
					   -fill => $ocGuiAttributes::sunsetColor,
					   -tags => $tag);
		}
		else  ## otherwise this is a calibrator -- render muted
		{
		    $canvas->createPolygon($tCenter[0],$tCenter[1] + $tSize,
					   $tCenter[0] - $tSize,$tCenter[1] - $tSize,
					   $tCenter[0] + $tSize,$tCenter[1] - $tSize,
					   -stipple => 'gray75',
					   -fill => $ocGuiAttributes::sunsetColor,
					   -tags => $tag);
		}

	    } ## if $trg check


	} ## end if $midnightYday

    } # end foreach $key loop

    ## draw the y axis to adjust to the number of objects
    $tCount++;
    $maxY = $yZero + $tCount * $ySpace;
    $canvas->createLine(xCoord(0),$yZero,xCoord(0),$maxY + 5,-tags => "yAxis"); # New y axis
    $canvas->createLine(xCoord($yearDays),$yZero,xCoord($yearDays),$maxY + 5,-tags => "yAxis"); # New y axis

    ## and minor lines for month boundaries
    my $month;
    my $minMonth = 2;
    my $maxMonth = 12;
    for ($month = 2; $month <= 12; $month += 1)
    {
	my $yd = yearDay($calYear,$month,1);
	$canvas->createLine(xCoord($yd),$yZero,xCoord($yd),$maxY + 5,
			    -stipple => 'gray50',
			    -tags => "yAxis"); # minor y axis
	if ($yDaysOn)
	{
	    $canvas->createText(xCoord($yd),$maxY + 15,
				-text => $yd, -tags => "yAxis");  ## yearDay label
	} ## drawing yday labels
    }

    if ($yDaysOn)
    {
	$canvas->createText(xCoord(-10),$maxY + 15,-anchor => "e",
			    -text => "Yday", -tags => "yAxis");  ## yearDay label

    }



    # Finally set the canvas scrollable region to include everything
    $canvas->configure(-scrollregion => [ $canvas->bbox("all") ] );


} ## end drawTargets  ###################################################



#########################################################################
## drawTargetLine -- draw the line for an individual target
#########################################################################
sub drawTargetLine
{
    ## drawTargetLine renders an individual target line of the desired
    ## color and thickness at the desired position.  This version
    ## checks for wraps of the lines around the HARDWIRED 0/360 deg
    ## boundary.

##  declarations drawTargetLine  ########################################
    my ($canvas,$x1,$x2,$y,$width,$color,$tag) = @_;  ## extract call arguments

## begin drawTargetLine  ################################################
    ## Check for wraps
    if ($x1 > $x2)
    {
	## render targets and calibrators with slightly
	## different colors/patterns
	if ($tag =~ /target/)
	{
	    $canvas->createLine(xCoord($x1),$y,xCoord($yearDays),$y,
				-width => $width,-fill => $color,
				-capstyle => 'projecting',
				-tags => $tag);
	    $canvas->createLine(xCoord(0),$y,xCoord($x2),$y,
				-width => $width,-fill => $color,
				-capstyle => 'projecting',
				-tags => $tag);
	} ## end target
	else
	{
	    $canvas->createLine(xCoord($x1),$y,xCoord($yearDays),$y,
				-width => $width,-fill => $color,
				-capstyle => 'round',-stipple => 'gray75',
				-tags => $tag);
	    $canvas->createLine(xCoord(0),$y,xCoord($x2),$y,
				-width => $width,-fill => $color,
				-capstyle => 'round',-stipple => 'gray75',
				-tags => $tag);
	} ## end calibrator

    } ## end $x1 > $x2 -- wrap
    else
    {

	## render targets and calibrators with slightly
	## different colors/patterns
	if ($tag =~ /target/)
	{
	    $canvas->createLine(xCoord($x1),$y,xCoord($x2),$y,
				-width => $width,-fill => $color,
				-capstyle => 'projecting',
				-tags => $tag);
	} ## end target
	else
	{
	    $canvas->createLine(xCoord($x1),$y,xCoord($x2),$y,
				-width => $width,-fill => $color,
				-capstyle => 'round',-stipple => 'gray75',
				-tags => $tag);
	} ## end calibrator
    } ## end normal line


} ## end drawTargetLine  ################################################



#########################################################################
## setCurrentTime -- compose strings for current time
#########################################################################
sub setCurrentTime
{

    ## setCurrentTime composes time strings for current date, yearday,
    ## and UT time for running display

## begin setCurrentTime  ################################################
    # get the current UT
    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = gmtime(time);
    my $ut = $hour + $min / 60.0 + $sec / 3600.0;  ## Current UTC


    ## Compose the date string
    $year += 1900;
    $mon += 1;
    $currentDate = join '/', $mon,$mday,$year;


    ## Compose the yearday string
    $yday += 1;
    $yday = join '',"0", $yday if ($yday < 10);
    $yday = join '',"0", $yday if ($yday < 100);
    $currentYearDay = join '', $year,$yday;


    ## Compose UT string
    $currentUT = &RAhms(15.0 * $ut,-1);
    $currentUT =~ s/\s+/:/g;
    $currentUT .= " UTC";

    $MW->after(20000, \&setCurrentTime);  ## recompute current time every 20 seconds

} ## end setCurrentTime  ################################################



#########################################################################
## drawCurrentDay -- render current day on canvas
#########################################################################
sub drawCurrentDay
{
## declarations drawCurrentDay  #########################################
    ## drawCurrentDay draws a vertical line indicating the present day
    my $canvasRef = shift @_;
    my $canvas = $$canvasRef;

## begin drawCurrentDay  ################################################
    $canvas->delete("DayLine");  ## delete previous UT line and labels

    if ($dayLineOn) ## If we're going to draw a day line
    {
	# get the current UT
	my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = gmtime(time);
	$year += 1900;
	$mon += 1;
	$yday += 1;

	## create a string...(and why are we not using $currentDate?)
	my $dateStr = join '/', $mon,$mday,$year;

	# render the day line and date/yday labels
	$canvas->createLine(xCoord($yday),$yZero,
			    xCoord($yday),$maxY,
			    -tags => "DayLine",
			    -fill => $ocGuiAttributes::currentDayColor);
	$canvas->createText(xCoord($yday),$yZero - 5,
			    -text => $dateStr,
			    -tags => "DayLine");
	$canvas->createText(xCoord($yday),$maxY + 5,
			    -text => $yday,
			    -tags => "DayLine");

        ## re-render day line every 120 seconds
	##  Note: strange syntax for invoking callback with argument
	$MW->after(120000, [\&drawCurrentDay, Ev($canvasRef)]);  

    } ## end if $dayLineOn

} ## end drawCurrentDay  ################################################


#########################################################################
## dispHelp -- display text from a file
#########################################################################
sub dispHelp
{

    ## dispHelp opens a read-only text window with help information
    ## for tGui

## declarations dispHelp  ###############################################
    my $filename = shift @_;

## begin dispHelp  ######################################################

    if (!defined $filename)
    {
	print "Error: undefined document link in ocGui\n";
	return;
    }

    ## Basically we open a text window and display the help file -- in
    ## all its glory.

    ## create a text window to display the return info
    my $helpWindow = $MW->Toplevel();
    $helpWindow->title("help -- $filename");
    my $text = $helpWindow->Scrolled("ROText",
				     -wrap => "none",
				     -width => 90, -height => 32,
				     -relief => "groove"
				     )->pack(-side => "top",
					-expand => "yes",
					-fill => "both");

    # dump the contents of the help file into the text window
    my $helpMsg = `cat $filename` or die;

    $text->insert('end',$helpMsg);

    # add a close button
    my $hwCntrl = $helpWindow->Frame(-relief => "groove",
				     -borderwidth => "2"
				     )->pack(-side => "bottom", -fill => "x");
    $hwCntrl->Button(-text => "Close",
		     -relief => "raised",
		     -command => sub {$helpWindow->withdraw}
		     )->pack(-side => "right");

    $helpWindow->bind('<Return>' => sub { $helpWindow->withdraw });


} ## end dispHelp  ######################################################



#########################################################################
## printCalDiagram -- handle printing tasks of various types
#########################################################################
sub printCalDiagram
{
    ## printCalDiagram prints a PostScript summary of the obsCalendar diagram

## begin printCalDiagram  ###############################################

    ## Make a new canvas especially for printing
    my $printCanvas = $MW->Scrolled('Canvas',
				    -width => $xCanvasSize,
				    -height => $yCanvasSize);

    &drawAxis($printCanvas);
    &drawTargets($printCanvas,$targDesig);
    &drawCurrentDay(\$printCanvas);


    ## Add reporting information on the plot
    my @tmp = grep /Using .* Location/, (split /\n/, $preambleText);
    if ($tmp[0])
    {
	$tmp[0] =~ s/ Using //g;
	$printCanvas->createText($xCanvasSize * 0.05,$maxY + 30,
				 -anchor => "w",-text => $tmp[0]);

    }

    $printCanvas->createText($xCanvasSize * 0.6,$maxY + 30,
			     -fill => $ocGuiAttributes::intersectionColor,
			     -anchor => "w",-text => "Target Accessible");


    $printCanvas->createText($xCanvasSize * 0.30,$maxY + 45,
			     -anchor => "w",-text => "Transit Markers:");

    $printCanvas->createText($xCanvasSize * 0.45,$maxY + 45,
			     -fill => $ocGuiAttributes::sunriseColor,
			     -anchor => "w",-text => "Sunrise");

    $printCanvas->createText($xCanvasSize * 0.55,$maxY + 45,
			     -fill => $ocGuiAttributes::transitColor,
			     -anchor => "w",-text => "Midnight");

    $printCanvas->createText($xCanvasSize * 0.65,$maxY + 45,
			     -fill => $ocGuiAttributes::sunsetColor,
			     -anchor => "w",-text => "Sunset");
    
    my $tmpTxt = "Produced by obsCalendar GUI v$REVISION ($PKGSTR)";

    $printCanvas->createText($xCanvasSize * 0.05,$maxY + 60,
			     -anchor => "w",-text => $tmpTxt);

    $tmpTxt = "NASA Exoplanet Science Institute -- http://nexsci.caltech.edu/";
    $printCanvas->createText($xCanvasSize * 0.55,$maxY + 60,
			     -fill => 'blue',
			     -anchor => "w",-text => $tmpTxt);

    ## Convert the canvas to PostScript
    ###TLE -- 6/23/04 need -pageheight, -pagewidth, -width, -height to fit
    ###    complete plot on PS page (auto-rescales to fit); -page* from input
    ###    ocGuiAttributes.pm module (default 300, 700)
    my ($bboxminX,$bboxminY,$bboxmaxX,$bboxmaxY) = $printCanvas->bbox("all");
    my $ps = $printCanvas->postscript( '-x' => 0,
                                  '-y' => 0,
                                  -rotate => 1,
                                  -pagewidth => "9i",
                                  -width => $bboxmaxX,
                                  -height => $bboxmaxY + 70
                                  );


    my $fileName = "oCalendar";
    if (defined $batchMode)
    {
	## Do batch mode output. Allow multiple output formats to be specified.
	writePsOutput("$fileName.ps",$ps) if (defined $PostScript); ## output PS
	writePsOutput("$fileName.gif",$ps) if (defined $pdf);   ## output GIF
	writePsOutput("$fileName.jpg",$ps) if (defined $jpg);   ## output JPEG
	writePsOutput("$fileName.png",$ps) if (defined $png);   ## output PNG
	writePsOutput("$fileName.pdf",$ps) if (defined $pdf);   ## output PDF
    }
    else
    {
	## Do "Print..." dialog to get filename or print command to use, 
	## then output the postscript as requested
	my $fn = &getOutputName($MW->Toplevel(), $balloon, $fileName);
	writePsOutput($fn,$ps);
    }
} ## end printCalDiagram  ###############################################

#########################################################################
## drawPreamble -- render calendar preamble
#########################################################################
sub drawPreamble
{

    ## drawPreamble flushes the contents of the existing preamble
    ## frame and replace it with the new $preambleText

    $preamble->delete('1.0','end'); ## clear the existing text
    $preamble->insert('end',$preambleText);  ## Insert the new

} ## end drawPreamble  ##################################################



#########################################################################
## packTargetMenu -- pack target menu with available options
#########################################################################
sub packTargetMenu
{
    ## packTargetMenu manages the dynamic part of the target menu
    ## contents

## declarations packTargetMenu  #########################################
    my $init = shift @_;

## begin packTargetMenu  ################################################

    ## Eliminate the extra stuff (if not init)...
    if (!$init)
    {
	$targetMenu->menu->delete('7', 'end');
    }

    my $nTargets = scalar @targets;
    my $nTargMenu = $ocGuiAttributes::targMenuItems;  ## Max Items 
    if ($nTargets > $nTargMenu)
    {
	my ($count,@indices);
	$count = 0;
	while ($count < $nTargets)
	{
	    push @indices, $count;
	    $count += $nTargMenu;
	} ## end while
	push @indices, $nTargets;

	my ($i,$j);
	$count = 1;
	for ($i = 0; $i < (scalar @indices - 1); $i++)
	{
	    my $addlTargets = $targetMenu->cascade(-label => "Target List $count");
	    for ($j = $indices[$i]; $j < $indices[$i + 1]; $j++)
	    {
		my $targTmp = $targets[$j];
		$addlTargets->checkbutton(-label => $targTmp,
					  -command => sub
					  {
					      ## Note the checkbutton code handles
					      ## the variable toggling for you
					      &drawTargets($displayCanvas,$targDesig);
					      &drawCurrentDay(\$displayCanvas);
					  },
					  -variable => \$targetOn{$targTmp});
	    } ## end j for loop
	    $count++;
	} ## end i for loop
    }
    else ## no need for sub-menus -- all in one single menu
    {
	foreach my $targTmp (@targets)
	{
	    $targetMenu->checkbutton(-label => $targTmp,
				     -command => sub
				     {
					 ## Note the checkbutton code handles
					 ## the variable toggling for you
					 &drawTargets($displayCanvas,$targDesig);
					 &drawCurrentDay(\$displayCanvas);
				     },
				     -variable => \$targetOn{$targTmp});
	} ## end target menu packing/for loop
    } ## end else


} ## end packTargetMenu  ################################################



#########################################################################
## packSimbadMenu -- pack Simbad menu with available options
#########################################################################
sub packSimbadMenu
{
    ## packTargetMenuSimbad manages the dynamic part of the Simbad
    ## menu contents

## declarations packSimbadMenu ##########################################
    my $init = shift @_;

## begin packSimbadMenu  ################################################
    ## Eliminate the extra stuff (if not init)...
    if (!$init)
    {
	$simbadMenu->menu->delete('5', 'end'); ### this needs to be fixed?
    }

    my $nTargets = scalar @targets;
    my $nTargMenu = $ocGuiAttributes::targMenuItems;  ## Max Items 
    if ($nTargets > $nTargMenu)
    {
	my ($count,@indices);
	$count = 0;
	while ($count < $nTargets)
	{
	    push @indices, $count;
	    $count += $nTargMenu;
	} ## end while
	push @indices, $nTargets;

	my ($i,$j);
	$count = 1;
	for ($i = 0; $i < (scalar @indices - 1); $i++)
	{
	    my $addlTargets = $simbadMenu->cascade(-label => "Target List $count");
	    for ($j = $indices[$i]; $j < $indices[$i + 1]; $j++)
	    {
		my $targTmp = $targets[$j];
		$addlTargets->command(-label => $targTmp,
				      -command => sub  ## query individual object
				      {
					  my $trgName = &simbadName($targTmp);
					  doSimbadGui($MW,$trgName);
				      });
	    } ## end j for loop
	    $count++;
	} ## end i for loop
    }
    else ## no need for sub-menus -- all in one single menu
    {
	foreach my $targTmp (@targets)
	{
	    $simbadMenu->command(-label => $targTmp,
				 -command => sub  ## query individual object
				      {
					  my $trgName = &simbadName($targTmp);
					  doSimbadGui($MW,$trgName);
				      });
	} ## end target menu packing/for loop
    } ## end else


} ## end packSimbadMenu  ################################################



#########################################################################
## aboutocGui -- pop up a informational window
#########################################################################
sub aboutocGui
{
    ## abouttGui creates a splash screen with tGui information

## begin aboutocGui  ####################################################
    my $aboutWindow = $MW->Toplevel();
    $aboutWindow->title("about ocGui");

    ## Display logo image in a button
    my $logoFile = findFile("NExScI_logo_tiny.gif",$cfg->gcdocs . "/images", $cfg->gcdata . "/images");
    return if (!defined $logoFile);  ## or not...

    my $logo = $aboutWindow->Photo(-file => $logoFile);
    $aboutWindow->Button(-text => '',
			 -command => sub {$aboutWindow->withdraw},
			 -image => $logo)->pack(-side => "top", -fill => "x");
    $aboutWindow->Label(-text => "http://nexsci.caltech.edu")->pack(-side => "top", -fill => "x");
    $aboutWindow->Label(-text => "ocGui v$REVISION ($PKGSTR)")->pack(-side => "top", -fill => "x");
    $aboutWindow->Label(-text => $ocGuiAttributes::ocGuiCopyright)->pack(-side => "top", -fill => "x");
    $aboutWindow->bind('<Return>' => sub { $aboutWindow->withdraw });
} ## end aboutocGui  ####################################################



#########################################################################
## editCalendarArgument -- handle options for recomputing calendar
#########################################################################
sub editCalendarArgument
{
    ## editCalendarArgument opens up a simple window to get the timing
    ## arguments from the user

## declarations editCalendarArgument  ###################################
    my $argument = shift @_;

## begin editCalendarArgument  ##########################################
    my $editArg = $MW->Toplevel();

    $editArg->title("obsCalendar argument input");


    my $argFrame = $editArg->Frame(-relief => "raised",
				   -borderwidth => "2"
				   )->pack(-side => "top", -fill => "x");

    $argFrame->Label(-text => "obsCalendar arguments: ",
		      )->pack(-side => "left", -anchor => "w");
    
    $argFrame->Entry(-textvariable => \$argument,
		      -relief => 'raised', -width => 60
		      )->pack(-side => "left", -anchor => "w", -fill => "x", -expand => 1);


    ## A second frame for buttons (AFB 12/2003)
    my $buttonFrame = $editArg->Frame(-relief => "raised",
				      -borderwidth => "2"
				      )->pack(-side => "top", -fill => "x");

    $buttonFrame->Button(-text => "Go",
			 -relief => 'groove',
			 -borderwidth => "5",
			 -command => sub 
			 {
			     $editArg->destroy();
			 }
			 )->pack(-side => "left");

    $buttonFrame->Button(-text => "obsCalendar Help",
			 -relief => 'groove',
			 -borderwidth => "5",
			 -command => sub 
			 {
			     &dispHelp($ocHelp);
			 }
			 )->pack(-side => "left");

    my $cancel = 0;  ## Give the option to cancel re-timing
    $buttonFrame->Button(-text => "Cancel",
			 -relief => 'groove',
			 -borderwidth => "5",
			 -command => sub 
			 {
			     $cancel = 1;
			     $editArg->destroy();
			 }
			 )->pack(-side => "right");

    $editArg->bind('<Return>' => sub  ## bind Return to go
		   {
		       $editArg->destroy();
		   });

    $editArg->waitWindow();

    return ($argument,$cancel);

} ## end editCalendarArgument  ##########################################
