#! @PERL@
# -*- perl -*-
# @configure_input@

our $MODVERSION = '$Id: tGui.pl,v 1.39 2008/11/12 23:28:13 sgroom Exp $';
our $MODREV = '$Revision: 1.39 $';

##########################################################################
# NASA Exoplanet Science Institute (NExScI) -- http://nexsci.caltech.edu #
#       Copyright (c) 1997-2008 California Institute of Technology       #
#                     U.S. Government Sponsorship                        #
# Under contract with the National Aeronautics and Space Administration  #
##########################################################################

## Timing GUI -- part of the getCal suite.  A.B. 6/99 -- 2/04

##  The timing GUI renders target accessibility timing information for
##  optical (direct combination) interferometers.  It takes input
##  information calculated by an external timing program and renders
##  it for planning purposes.  It (the GUI) also performs real-time
##  display functions to indicate current target accessibility.

## Modules and pieces we use
use strict;
use warnings;
use FindBin qw($RealBin);
use lib "$RealBin/../lib/modules";		# location of getCal Perl modules
use Getopt::Long;
use Tk;
use Tk::ROText;
use Tk::FileSelect;
use Tk::NoteBook;
use Tk::Balloon;
use guiSubs;
use coordinates;  ## Coordinate handling utilities
use tGuiAttributes; ## Colors and such for the tGui display
use novasConstants; ## astronomical constants package
use gcConfig;
use gcDebug;
use findScript;
use miscArgs;
use interval;	## computing intersection of intervals

## Command-line switch/default variables
my $cfg = gcConfig::config();
my @ra = split /\s+/, $MODREV; my $REVISION = $ra[1];
my $PKGSTR = package_string();
my ($help,$version,$stdin,$PostScript,$pdf,$gif,$jpg,$png,$batchMode);
my ($filename);

my $dbgname = "tGui";	# program name used in debugging messages

my $sunOn = 0;
my $utOn = 0;
my $intersectionTrack = 1;
my $zaTrack = 0;
my $delayTrack = 0;
my $pointingTrack = 0;
my $timeLabels = 1; ## default to LST on
my $blDesig = undef;
my $calOn = 0;
my $moonOn = 0;
my $timingArgument = "-replace ";

# keywords to highlight in text output window
my $highlightKeys = "Timing summary|for timings|Using .* Location|Using .* Baseline|The JD at 0 hr UT is|The LST at 0 hr UT is|The Local Time at 0 hr UT is|Approximate UT sunset -- sunrise times|Approximate LST at sunset -- sunrise|Approximate UT moonrise -- moonset times|moon transit|% moon illumination -- |Using .* deg zenith angle constraint|Using .* deg zenith angle .* constraint|Using .* m delay constraint|Using .* delay interval|transits at .*, zenAng|is within zenith angle range from|is never within zenith angle range|is always within zenith angle range|is within delay range from|is always within delay range|is never within delay range|is within pointing range|is never within pointing range|is always within pointing range|is out of pointing range at transit";

## Adding command-line arguments to tGui  A.B. 5/2003
handleCommonOpts(0,"tGui.txt");
Getopt::Long::config("pass_through");
my $result = GetOptions("stdin" => \$stdin,
			"filename=s" => \$filename,
			"sun:i" => \$sunOn,
			"moon" => \$moonOn,
			"timeLabels:i" => \$timeLabels,
                        "currentTime:i" => \$utOn,
			"baseline=s" => \$blDesig,
			"intersectionTrack:i" => \$intersectionTrack,
			"zenithTrack:i" => \$zaTrack,
			"delayTrack:i" => \$delayTrack,
			"pointingTrack:i" => \$pointingTrack,
                        "calibrators" => \$calOn,
			"ps|postscript|PostScript|hc" => \$PostScript,
			"pdf" => \$pdf,
			"gif" => \$gif,
			"jpg" => \$jpg,
			"png" => \$png);


## Process some command-line things

if ((defined $PostScript) || (defined $pdf)
    || (defined $gif) || (defined $jpg) || (defined $png))
{
    $batchMode = 1;
} ## end setting batch mode


## The main window declaration
my $MW = new MainWindow;
$MW->title("timing GUI v$REVISION ($PKGSTR)");

## In-line help stuff
##my $statusFrame = $MW->Frame(-relief => "raised",
##			  -borderwidth => "2");
##$msgArea = $statusFrame->Label(-borderwidth => 2,
##			    -relief => 'groove')->pack(-side => 'bottom', fill => 'x');

my $balloon = $MW->Balloon();  ## help balloon object


##################################################
### timing display canvas parameter definition ###
##################################################

## Retrieve pertinent parameters from tGuiAttributes module

my $xCanvasSize = $tGuiAttributes::tGuiXCanvasSize;
my $yCanvasSize = $tGuiAttributes::tGuiYCanvasSize;

my $displayHours = $tGuiAttributes::tGuiDisplayHours;

my $xScale = $xCanvasSize / ($displayHours * 15.0);

my $xZero = int ($xCanvasSize / 8.0);
my $timeOffsetHR = $tGuiAttributes::timeOffsetHR;
my $timeOffset = RAdeg($timeOffsetHR);

my $yZero = int ($yCanvasSize / 10.0);
my $maxY;

## Define the main display canvas notebook (but don't pack it yet)
my $displayNB = $MW->NoteBook();


## Timing summary variable declarations
my ($timingDate, $yDay, $JDzeroUT, $LSTzeroUT, $LocalZeroUT,
    $sunRise, $sunSet, $moonRise, $moonSet,$moonIllumination);
$timingDate = $sunRise = $sunSet = "";
my ($input,$preambleText);
my (%transit, %leadZA, %trailZA, %newDelay, %target, %targetOn, %pointing, %transitWarn);
my (%DisplayCanvas);
my (@targets, @baseline, @clusters);


########################### BEGIN MAIN PROGRAM

# Start by reading the input timing report

## Process initial input
if ($stdin) ## explicit stdin direction
{
    &readTimingRpt(0);
}
elsif ($filename)  ## If given a filename argument
{
    &readTimingRpt(1,$filename);
}
elsif ((scalar @ARGV) && !($stdin)) ## otherwise check command line
{
    &readTimingRpt(1,$ARGV[0]);  ## read from filename
}
else ## otherwise nothing left on command line, assume stdin
{
    $stdin = 1;
    &readTimingRpt(0);  ## read from stdin
}

## Setup more default values
my $targDesig = "all"; ## default to all targets
## default to the first baseline listed if not already set in options
$blDesig = $baseline[0] if (!(defined $blDesig));

## Track drawing setting variables and defaults
if ($zaTrack || $delayTrack || $pointingTrack)
{
    $intersectionTrack = 0;
}


if (!($preambleText =~ m/Using .* Baseline/))
{
    $intersectionTrack = 0;
    $zaTrack = 1;
    $delayTrack = 0;
    $pointingTrack = 0;
} ## no baseline/delay timing info -- reset defaults

## TLE -- 7/8/04 set up original target hash
my $trg;
foreach $trg (@targets)
{
  if (!($target{$trg}))
  {
    $targetOn{$trg} = $calOn;
  }
}

### end intro stuff ###


## Do batchMode/hardcopy as required
if (defined $batchMode)
{
    ## Call print routine
    printTDiagram();

    exit(0);  ## we've generated the graphic -- exit
} ## end batchMode


## Gui setup.  If we're still here we're running interactively...so do
## the menus and keyboard options...

## At the top -- the menu frame
my $menuFrame = $MW->Frame(-relief => "raised",
			   -borderwidth => "2"
			   )->pack(-side => "top", -fill => "x");



# file menu
my $fileMenu = $menuFrame->Menubutton(-text => "File",
				      -menuitems => [## Recompute timings
						     [ 'command' => "Recompute Timings...",
						       -accelerator => 'Alt+r',
						       -command => sub {
							   my $cancel;
							   ($timingArgument,$cancel) =
							       &editTimingArgument($timingArgument, $MW->Toplevel());

							   if (!$cancel)
							   {

							       my $revisedInput
								   = `echo "$input" | timing -stdin $timingArgument`;
							       &readTimingRpt(2,$revisedInput);
							       &drawPreamble();
							       &packTargetMenu();
							       &packBlineMenu();
							       &packSimbadMenu();

							       ## Raise the first page on re-compute
							       $displayNB->raise($baseline[0]);

							   } ## end if !$cancel
						       
						       }],
						     "-",
						     ## Open new file
						     [ 'command' => "Open...",
						       -accelerator => 'Alt+o',
						       -command => sub {
							   my $file
							       = $MW->getOpenFile(-defaultextension => ".cat",
										  -filetypes        =>
										  [['catalog files',   '.cat' ],
										   ['Text Files',       ['.txt','.text']],
										   ['All Files',        '*',            ],
										   ],
										  -initialfile      => "getopenfile",
										  -title            => "Open schedule file",
										  );
							   if ($file)
							   {
							       &readTimingRpt(1,$file);
							       &drawPreamble();
							       &packTargetMenu();
							       &packBlineMenu();
							       &packSimbadMenu();

							       ## Raise the first page on re-open
							       $displayNB->raise($baseline[0]);

							   } ## end if file
						       }],
						     ## Open new file and recompute timings
						     [ 'command' => "Open Through timing...",
						       -accelerator => 'Alt+O',
						       -command => sub {
							   my $file
							       = $MW->getOpenFile(-defaultextension => ".cat",
										  -filetypes        =>
										  [['catalog files',   '.cat' ],
										   ['Text Files',       ['.txt','.text']],
										   ['All Files',        '*',            ],
										   ],
										  -initialfile      => "getopenfile",
										  -title            => "Open catalog file through timing",
										  );
							   if ($file)
							   {
							       my $cancel;
							       ($timingArgument,$cancel) =
								   &editTimingArgument($timingArgument, $MW->Toplevel());
							       if (!$cancel)
							       {
								   my $revisedInput
								       = `timing $timingArgument -filename $file`;
								   &readTimingRpt(2,$revisedInput);
								   &drawPreamble();
								   &packTargetMenu();
								   &packBlineMenu();
								   &packSimbadMenu();

								   ## Raise the first page on re-open
								   $displayNB->raise($baseline[0]);

							       } ## end if not $cancel
							   } ## end if file
						       }],
						     "-",
						     [ 'command' => "View Input",
						       -accelerator => 'Alt+v',
						       -command => sub {&dispText($MW,$input,"tGui input",$highlightKeys)}],
						     [ 'command' => "uvTool",
						       -accelerator => 'Alt+u',
						       -command => sub
						       { system "echo \"$input\" | uvTool &";}],
						     "-",
						     [ 'command' => "XEphem Catalog",
						       -accelerator => 'Alt+x',
						       -command => sub {&xephemOutput($MW, $input)}],
						     [ 'command' => "XEphem FIFO",
						       -accelerator => 'Alt+X',
						       -command => sub {&xephemOutput($MW, $input,"fifo")}],
						     "-",
						     [ 'command' => "Print...",
						       -accelerator => 'Alt+p',
						       -command => sub {&printTDiagram()}],
						     "-",
						     [ 'command' => "Quit",
						       -accelerator => 'Alt+q',
						       -command => sub {$MW->withdraw; exit(0); }] ]
				      )->pack(-side => "left", -anchor => "w");
## Build the balloon help list for the file menu
$balloon->attach($fileMenu,-msg => "File menu -- options for input and output");

## Define keyboard bindings for File menu
## Recompute timings
$MW->bind('<Alt-r>' => sub
	  {
	      my $cancel;
	      ($timingArgument,$cancel) =
		  &editTimingArgument($timingArgument, $MW->Toplevel());

	      if (!$cancel)
	      {
		  my $revisedInput
		      = `echo "$input" | timing -stdin $timingArgument`;
		  &readTimingRpt(2,$revisedInput);
		  &drawPreamble();
		  &packTargetMenu();
		  &packBlineMenu();
		  &packSimbadMenu();

		  ## Raise the first page on re-compute
		  $displayNB->raise($baseline[0]);

	      } ## end if !$cancel
	  });

## Open new file
$MW->bind('<Alt-o>' => sub {
                              my $file
				  = $MW->getOpenFile(-defaultextension => ".cat",
						     -filetypes        =>
						     [['catalog files',   '.cat' ],
						      ['Text Files',       ['.txt','.text']],
						      ['All Files',        '*',            ],
						      ],
						     -initialfile      => "getopenfile",
						     -title            => "Open schedule file",
						     );
			     if ($file)
			     {
				 &readTimingRpt(1,$file);
				 &drawPreamble();
				 &packBlineMenu();

				 ## Raise the first page on re-open
				 $displayNB->raise($baseline[0]);

			     } ## end if file
			 });
$MW->bind('<Alt-O>' => sub {
    my $file
	= $MW->getOpenFile(-defaultextension => ".cat",
			   -filetypes        =>
			   [['catalog files',   '.cat' ],
			    ['Text Files',       ['.txt','.text']],
			    ['All Files',        '*',            ],
			    ],
			   -initialfile      => "getopenfile",
			   -title            => "Open catalog file through timing",
			   );
    if ($file)
    {
	my $cancel;
	($timingArgument,$cancel) =
	    &editTimingArgument($timingArgument, $MW->Toplevel());
	if (!$cancel)
	{
	    my $revisedInput
		= `timing $timingArgument -filename $file`;
	    &readTimingRpt(2,$revisedInput);
	    &drawPreamble();
	    &packTargetMenu();
	    &packBlineMenu();
	    &packSimbadMenu();

	    ## Raise the first page on re-open
	    $displayNB->raise($baseline[0]);

	} ## end if not $cancel
    } ## end if file
});  ## end open through timing accelerator

$MW->bind('<Alt-v>' => sub {&dispText($MW,$input,"tGui input",$highlightKeys)});
$MW->bind('<Alt-u>' => sub {system "echo \"$input\" | uvTool &";});
$MW->bind('<Alt-x>' => sub {&xephemOutput($MW, $input)});
$MW->bind('<Alt-X>' => sub {&xephemOutput($MW, $input,"fifo")});
$MW->bind('<Alt-p>' => sub {&printTDiagram()});
$MW->bind('<Alt-q>' => sub {$MW->withdraw; exit(0); });

## target menu

my $targetMenu = $menuFrame->Menubutton(-text => "Target",
					-menuitems => [['command' => "Targets",
							-accelerator => 'Alt+t',
							-command => sub
							{
							    ## draw target menu entry -- set $targetOn for
							    ## all targets -- set off for cals
							    my $trg;
							    foreach $trg (@targets)
							    {
								if ($target{$trg})
								{
								    $targetOn{$trg} = 1;
								}
								else
								{
								    $targetOn{$trg} = 0;
								}
							    }
							    $targDesig = "all";
							    $calOn = 0;

							    my $page = $displayNB->raised;
							    &drawTargets($DisplayCanvas{$page},$targDesig,$blDesig);
							    &drawSun($DisplayCanvas{$page});
							    &drawClusters($DisplayCanvas{$page});
							    &drawCurrentUT(\$DisplayCanvas{$page});
							    &drawTimeLabels($DisplayCanvas{$page});
							}],
						       ['command' => "Calibrators",
							-accelerator => 'Alt+c',
							-command => sub
							{
							    ## draw calibrator menu entry -- set $targetOn for
							    ## all calibrators
							    ## Toggle the calibrator rendering	
							    $calOn = !($calOn);
							    my $trg;
							    foreach $trg (@targets)
							    {
								if (!($target{$trg}))
								{
								    $targetOn{$trg} = $calOn;
								}
							    }

							    my $page = $displayNB->raised;
							    &drawTargets($DisplayCanvas{$page},$targDesig,$blDesig);
							    &drawSun($DisplayCanvas{$page});
							    &drawClusters($DisplayCanvas{$page});
							    &drawCurrentUT(\$DisplayCanvas{$page});
							    &drawTimeLabels($DisplayCanvas{$page});
							}],
						       ['command' => "All",
							-accelerator => 'Alt+a',
							-command => sub
							{
							    ## draw all menu entry -- set $targetOn for
							    ## everybody
							    $targDesig = "all";
							    $calOn = 1;
							    my $trg;
							    foreach $trg (@targets)
							    {
								$targetOn{$trg} = 1;
							    }

							    my $page = $displayNB->raised;
							    &drawTargets($DisplayCanvas{$page},$targDesig,$blDesig);
							    &drawSun($DisplayCanvas{$page});
							    &drawClusters($DisplayCanvas{$page});
							    &drawCurrentUT(\$DisplayCanvas{$page});
							    &drawTimeLabels($DisplayCanvas{$page});
							}],
						       ['command' => "Invert",
							-accelerator => 'Alt+i',
							-command => sub
							{
							    ## invert menu entry -- toggle $targetOn for
							    ## everybody
							    $targDesig = "all";
							    $calOn = !($calOn);  ## vestigial
							    my $trg;
							    foreach $trg (@targets)
							    {
								$targetOn{$trg} = !($targetOn{$trg});
							    }

							    my $page = $displayNB->raised;
							    &drawTargets($DisplayCanvas{$page},$targDesig,$blDesig);
							    &drawSun($DisplayCanvas{$page});
							    &drawClusters($DisplayCanvas{$page});
							    &drawCurrentUT(\$DisplayCanvas{$page});
							    &drawTimeLabels($DisplayCanvas{$page});
							}],
						       ['command' => "None",
							-accelerator => 'Alt+n',
							-command => sub
							{
							    ## draw no menu entries -- set $targetOn for
							    ## everybody
							    $targDesig = "all";
							    $calOn = 0;
							    my $trg;
							    foreach $trg (@targets)
							    {
								$targetOn{$trg} = 0;
							    }

							    my $page = $displayNB->raised;
							    &drawTargets($DisplayCanvas{$page},$targDesig,$blDesig);
							    &drawSun($DisplayCanvas{$page});
							    &drawClusters($DisplayCanvas{$page});
							    &drawCurrentUT(\$DisplayCanvas{$page});
							    &drawTimeLabels($DisplayCanvas{$page});
							}],
						       "-"]
					)->pack(-side => "left", -anchor => "w");
$balloon->attach($targetMenu,-msg => "Target menu -- options for objects");

## Keyboard bindings for the target menu
$MW->bind('<Alt-t>' => sub
	  {
	      ## draw target menu entry -- set $targetOn for
	      ## all targets -- set off for cals
	      my $trg;
	      foreach $trg (@targets)
	      {
		  if ($target{$trg})
		  {
		      $targetOn{$trg} = 1;
		  }
		  else
		  {
		      $targetOn{$trg} = 0;
		  }
	      }
	      $targDesig = "all";
	      $calOn = 0;

	      my $page = $displayNB->raised;
	      &drawTargets($DisplayCanvas{$page},$targDesig,$blDesig);
	      &drawSun($DisplayCanvas{$page});
	      &drawClusters($DisplayCanvas{$page});
	      &drawCurrentUT(\$DisplayCanvas{$page});
	      &drawTimeLabels($DisplayCanvas{$page});
	  });
$MW->bind('<Alt-c>' => sub
	  {
	      ## draw calibrator menu entry -- set $targetOn for
	      ## all calibrators
	      ## Toggle the calibrator rendering	
	      $calOn = !($calOn);
	      my $trg;
	      foreach $trg (@targets)
	      {
		  if (!($target{$trg}))
		  {
		      $targetOn{$trg} = $calOn;
		  }
	      }

	      my $page = $displayNB->raised;
	      &drawTargets($DisplayCanvas{$page},$targDesig,$blDesig);
	      &drawSun($DisplayCanvas{$page});
	      &drawClusters($DisplayCanvas{$page});
	      &drawCurrentUT(\$DisplayCanvas{$page});
	      &drawTimeLabels($DisplayCanvas{$page});
	  });
$MW->bind('<Alt-a>' => sub	
	  {
	      ## draw all menu entry -- set $targetOn for
	      ## everybody
	      $targDesig = "all";
	      $calOn = 1;
	      my $trg;
	      foreach $trg (@targets)
	      {
		  $targetOn{$trg} = 1;
	      }

	      my $page = $displayNB->raised;
	      &drawTargets($DisplayCanvas{$page},$targDesig,$blDesig);
	      &drawSun($DisplayCanvas{$page});
	      &drawClusters($DisplayCanvas{$page});
	      &drawCurrentUT(\$DisplayCanvas{$page});
	      &drawTimeLabels($DisplayCanvas{$page});
	  });
$MW->bind('<Alt-i>' => sub
	  {
	      ## invert menu entry -- toggle $targetOn for
	      ## everybody
	      $targDesig = "all";
	      $calOn = !($calOn);  ## vestigial
	      my $trg;
	      foreach $trg (@targets)
	      {
		  $targetOn{$trg} = !($targetOn{$trg});
	      }

	      my $page = $displayNB->raised;
	      &drawTargets($DisplayCanvas{$page},$targDesig,$blDesig);
	      &drawSun($DisplayCanvas{$page});
	      &drawClusters($DisplayCanvas{$page});
	      &drawCurrentUT(\$DisplayCanvas{$page});
	      &drawTimeLabels($DisplayCanvas{$page});
	  });
$MW->bind('<Alt-n>' => sub
	  {
	      ## draw no menu entries -- set $targetOn for
	      ## everybody
	      $targDesig = "all";
	      $calOn = 0;
	      my $trg;
	      foreach $trg (@targets)
	      {
		  $targetOn{$trg} = 0;
	      }

	      my $page = $displayNB->raised;
	      &drawTargets($DisplayCanvas{$page},$targDesig,$blDesig);
	      &drawSun($DisplayCanvas{$page});
	      &drawClusters($DisplayCanvas{$page});
	      &drawCurrentUT(\$DisplayCanvas{$page});
	      &drawTimeLabels($DisplayCanvas{$page});
	  });


## Use subroutine to pack individual objects in the target menu
&packTargetMenu(1);


## baseline menu
my $baselineMenu =
    $menuFrame->Menubutton(-text => "Baseline",
			   -menuitems => [['command' => "None",
					   -command => sub
					   {
					       $blDesig = "None";

					       my $page = $displayNB->raised;
					       &drawTargets($DisplayCanvas{$page},$targDesig,$blDesig);
					       &drawSun($DisplayCanvas{$page});
					       &drawClusters($DisplayCanvas{$page});
					       &drawCurrentUT(\$DisplayCanvas{$page});
					       &drawTimeLabels($DisplayCanvas{$page});
					   }],
					  "-"]
			   )->pack(-side => "left", -anchor => "w");
$balloon->attach($baselineMenu,-msg => "Baseline menu -- options for baselines");

## Make an entry in the baseline menu for each baseline designation in
## the input file
&packBlineMenu(1);


## Simbad query menu
my $simbadMenu = $menuFrame->Menubutton(-text => "Simbad",
					-menuitems => [['command' => "Targets",
							##-accelerator => 'Alt-t',
							-command => sub
							{
							    ## Simbad target menu entry -- query all targets
							    my @targetSet = ();
							    foreach my $trg (@targets)
							    {
								if ($target{$trg})
								{
								    push @targetSet,&simbadName($trg);
								}
							    }
							    if (scalar @targetSet)
							    {
								doSimbadGui($MW,@targetSet);
							    }

							}],
						       ['command' => "Calibrators",
							##-accelerator => 'Alt+c',
							-command => sub
							{
							    ## Simbad calibrator menu entry -- query all calibrators
							    my @targetSet = ();
							    foreach my $trg (@targets)
							    {
								if (!($target{$trg}))
								{
								    push @targetSet,&simbadName($trg);
								}
							    }
							    if (scalar @targetSet)
							    {
								doSimbadGui($MW,@targetSet);
							    }
							}],
						       ['command' => "All",
							##-accelerator => 'Alt+a',
							-command => sub
							{
							    ## Simbad all menu entry -- query all
							    my @targetSet = ();
							    foreach my $trg (@targets)
							    {
								push @targetSet,&simbadName($trg);
							    }
							    if (scalar @targetSet)
							    {
								doSimbadGui($MW,@targetSet);
							    }
							}],
						       "-"]
					)->pack(-side => "left", -anchor => "w");
$balloon->attach($simbadMenu,-msg => "Simbad menu -- Simbad query options");

## Use subroutine to pack individual objects in the simbad menu
&packSimbadMenu(1);



# help menu
# help files

my $tgHelp = findFile("tGui.txt",$cfg->gcdocs, $cfg->gcdata);
my $timingHelp = findFile("timing.txt",$cfg->gcdocs, $cfg->gcdata);
$version = findFile("version.txt",$cfg->gcdocs, $cfg->gcdata);
my $copyright = findFile("copyright.txt",$cfg->gcdocs, $cfg->gcdata);

my $helpMenu = $menuFrame->Menubutton(-text => "Help",
				      -menuitems => [ [ 'command' => "tGui help",
							-accelerator => 'Alt+h',
							-command => sub {&dispHelp($tgHelp)} ],
						      [ 'command' => "about tGui",
							-command => sub {&abouttGui()}],
						      "-",
						      [ 'command' => "timing help",
							-command => sub {&dispHelp($timingHelp)}  ],
						      [ 'command' => "getCal version",
							-command => sub {&dispHelp($version)}  ],
						      [ 'command' => "getCal copyright",
							-command => sub {&dispHelp($copyright)}  ],
						    ]
				      )->pack(-side => "right", -anchor => "w");
$balloon->attach($helpMenu,-msg => "Help menu -- available documentation options");

$MW->bind('<Alt-h>' => sub {&dispHelp($tgHelp)});


# Next a text frame with informational comments about the timing
# summary

my $msgFrame = $MW->Frame(-relief => 'groove',
			  -borderwidth => "2"
			  )->pack(-side => "top", -fill => "x");

## Replacing the older labels with a text window containing the preamble
my $preamble = $msgFrame->Scrolled("ROText",
				   -wrap => "none",
				   -width => 80, -height => 8,
				   -relief => "groove"
				   )->pack(-side => "left",
					   -expand => "yes",
					   -fill => "x");
$balloon->attach($preamble,-msg => "Timing preamble");


## Insert the preamble text in this frame
&drawPreamble();


## Now pack the canvas notebook
$displayNB->pack(-side => "top",-expand => "yes",-fill => "both");



## A time frame to hold current time displays
my $tFrame = $msgFrame->Frame(-relief => 'groove',
			     -borderwidth => "1",
			     )->pack(-side => "right",-fill => "x");

## Logic for current time displays
my ($currentUT,$currentLST,$currentLocal);
&setCurrentTime();
$tFrame->Label(-text => "Current")->pack(-side => "top",-anchor => "w");
$tFrame->Label(-textvariable => \$currentUT,
	       -relief => 'raised',
	       -width => 10
	       )->pack(-side => "top", -anchor => "w");

$tFrame->Label(-textvariable => \$currentLST,
	       -relief => 'raised',
	       -width => 10
	       )->pack(-side => "top", -anchor => "w");
$tFrame->Label(-textvariable => \$currentLocal,
	       -relief => 'raised',
	       -width => 10
	       )->pack(-side => "top", -anchor => "w");
$balloon->attach($tFrame,-msg => "Current date and time");


### Draw axes lines



### Define mini-canvas with display legend
my $yCanvasSize2 = int ($yCanvasSize / 6.0);
my $legendCanvas = $MW->Canvas(-width => $xCanvasSize,
			       -height => $yCanvasSize2,
			       -relief => "groove",
			       -borderwidth => "2"
			       )->pack(-side => "top");

## First line: Legend text, intersect and pointing definitions
my $yPos = int (0.25 * $yCanvasSize2);
my $xPos = int (0.05 * $xCanvasSize);
$legendCanvas->createText($xPos,$yPos,-text => "Legend:");

$xPos += int (0.3 * $xCanvasSize);
my $offset = 50.0;
my $length = int (0.05 * $xCanvasSize);

$legendCanvas->createText($xPos,$yPos,-text => "Intersection Track");
$legendCanvas->createLine($xPos + $offset,$yPos,
			  $xPos + $offset + $length,$yPos,
			  -width => 3,
			  -fill => $tGuiAttributes::intersectionColor);


$xPos += int (0.3 * $xCanvasSize);
$legendCanvas->createText($xPos,$yPos,-text => "Pointing Track");
$legendCanvas->createLine($xPos + $offset,$yPos,
			  $xPos + $offset + $length,$yPos,
			  -width => $tGuiAttributes::pointingThickness,
			  -fill => $tGuiAttributes::pointingColor);


## Second Line: delay, zenith, and transit definitions
$yPos = int (3/4 * $yCanvasSize2);
$xPos = int (0.1 * $xCanvasSize);

$legendCanvas->createText($xPos,$yPos,-text => "Delay Track");
$legendCanvas->createLine($xPos + $offset,$yPos,
			  $xPos + $offset + $length,$yPos,
			  -width => 3,
			  -fill => $tGuiAttributes::delayColor);


$xPos += int (0.3 * $xCanvasSize);
$legendCanvas->createText($xPos,$yPos,-text => "Zenith Track");
$legendCanvas->createLine($xPos + $offset,$yPos,
			  $xPos + $offset + $length,$yPos,
			  -width => 3,
			  -fill => $tGuiAttributes::zaColor);

$xPos += int (0.3 * $xCanvasSize);
$legendCanvas->createText($xPos,$yPos,-text => "Transit");
my @tCenter = ($xPos + $offset,$yPos);
my $tSize = 6;	# size of transit marker triangle
$legendCanvas->createPolygon($tCenter[0],$tCenter[1] - $tSize,
			     $tCenter[0] - $tSize,$tCenter[1] + $tSize,
			     $tCenter[0] + $tSize,$tCenter[1] + $tSize,
			     -fill => $tGuiAttributes::transitColor);

## A micro-logo for institutional awarness purposes
my $logoFile = findFile("NExScI_logo_micro.gif",$cfg->gcdocs . "/images", $cfg->gcdata . "/images");
my $logo = $legendCanvas->Photo( 'IMG', -file => $logoFile);
$xPos = int (0.94 * $xCanvasSize);
$yPos = int (0.55 * $yCanvasSize2);
$legendCanvas->create('image', $xPos, $yPos,
                     '-image'  => $logo );



## Add a couple of simple control frames at the bottom
my $ctrlFrame = $MW->Frame(-relief => 'groove',
			   -borderwidth => "2"
			   )->pack(-side => "bottom", -fill => "x");


## First control frame holds intersection/ZA/delay/pointing track
## controls

my $ctrlFrame1 = $ctrlFrame->Frame(-relief => 'groove',
				   -borderwidth => "2"
				   )->pack(-side => "top", -fill => "x");
## Intersection display control
my $iButton = 
    $ctrlFrame1->Button(-text => 'Intersection Track',
			-command => sub
			{
			    $intersectionTrack = (!$intersectionTrack);
			    if ($intersectionTrack)
			    {			    
				$zaTrack = $delayTrack = $pointingTrack = 0;
			    }
			    else
			    {
				$zaTrack = $delayTrack = $pointingTrack = 1;
			    }

			    my $page = $displayNB->raised;
			    &drawTargets($DisplayCanvas{$page},$targDesig,$blDesig);
			    &drawSun($DisplayCanvas{$page});
			    &drawClusters($DisplayCanvas{$page});
			    &drawCurrentUT(\$DisplayCanvas{$page});
			}
			)->pack(-side => "left", -anchor => "w");
$balloon->attach($iButton,-msg => "Toggle intesection track rendering <Cntrl+i>");

$MW->bind('<Control-i>' => sub
	  {
	      $intersectionTrack = (!$intersectionTrack);
	      if ($intersectionTrack)
	      {			    
		  $zaTrack = $delayTrack = 0;
	      }
	      else
	      {
		  $zaTrack = $delayTrack = 1;
	      }

	      my $page = $displayNB->raised;
	      &drawTargets($DisplayCanvas{$page},$targDesig,$blDesig);
	      &drawSun($DisplayCanvas{$page});
	      &drawClusters($DisplayCanvas{$page});
	      &drawCurrentUT(\$DisplayCanvas{$page});
	  });

## ZA display control
my $zButton =
    $ctrlFrame1->Button(-text => 'ZA Track',
			-command => sub
			{
			    $intersectionTrack = 0;
			    $zaTrack = (!$zaTrack);

			    my $page = $displayNB->raised;
			    &drawTargets($DisplayCanvas{$page},$targDesig,$blDesig);
			    &drawSun($DisplayCanvas{$page});
			    &drawClusters($DisplayCanvas{$page});
			    &drawCurrentUT(\$DisplayCanvas{$page});
			}
			)->pack(-side => "left", -anchor => "w");
$balloon->attach($zButton,-msg => "Toggle zenith angle track rendering <Cntrl+z>");

$MW->bind('<Control-z>' => sub
	  {
	      $intersectionTrack = 0;
	      $zaTrack = (!$zaTrack);

	      my $page = $displayNB->raised;
	      &drawTargets($DisplayCanvas{$page},$targDesig,$blDesig);
	      &drawSun($DisplayCanvas{$page});
	      &drawClusters($DisplayCanvas{$page});
	      &drawCurrentUT(\$DisplayCanvas{$page});
	  });


## Delay control button
my $dButton =
    $ctrlFrame1->Button(-text => 'Delay Track',
			-command => sub
			{
			    $intersectionTrack = 0;
			    $delayTrack = (!$delayTrack);

			    my $page = $displayNB->raised;
			    &drawTargets($DisplayCanvas{$page},$targDesig,$blDesig);
			    &drawSun($DisplayCanvas{$page});
			    &drawClusters($DisplayCanvas{$page});
			    &drawCurrentUT(\$DisplayCanvas{$page});
			}
			)->pack(-side => "left", -anchor => "w");
$balloon->attach($dButton,-msg => "Toggle delay track rendering <Cntrl+d>");

$MW->bind('<Control-d>' => sub
	  {
	      $intersectionTrack = 0;
	      $delayTrack = (!$delayTrack);

	      my $page = $displayNB->raised;
	      &drawTargets($DisplayCanvas{$page},$targDesig,$blDesig);
	      &drawSun($DisplayCanvas{$page});
	      &drawClusters($DisplayCanvas{$page});
	      &drawCurrentUT(\$DisplayCanvas{$page});
	  });


## Pointing control button
my $ptButton =
    $ctrlFrame1->Button(-text => 'Pointing Track',
			-command => sub
			{
			    $intersectionTrack = 0;
			    $pointingTrack = (!$pointingTrack);

			    my $page = $displayNB->raised;
			    &drawTargets($DisplayCanvas{$page},$targDesig,$blDesig);
			    &drawSun($DisplayCanvas{$page});
			    &drawClusters($DisplayCanvas{$page});
			    &drawCurrentUT(\$DisplayCanvas{$page});
			}
			)->pack(-side => "left", -anchor => "w");
$balloon->attach($ptButton,-msg => "Toggle pointing track rendering <Cntrl+p>");

$MW->bind('<Control-p>' => sub
	  {
	      $pointingTrack = (!$pointingTrack);

	      my $page = $displayNB->raised;
	      &drawTargets($DisplayCanvas{$page},$targDesig,$blDesig);
	      &drawSun($DisplayCanvas{$page});
	      &drawClusters($DisplayCanvas{$page});
	      &drawCurrentUT(\$DisplayCanvas{$page});
	  });

## xephemOutput control button
my $xButton =
    $ctrlFrame1->Button(-text => 'XEphem',
			-command => sub
			{
			    ## Output the object catalog to xephem
			    &xephemOutput($MW, $input);
			}
			)->pack(-side => "left", -anchor => "w");
$balloon->attach($xButton,-msg => "Output objects to XEphem catalog <Alt+x>");

## display/edit the active baseline
my $blineEntry = $ctrlFrame1->Entry(-textvariable => \$blDesig,
				    -relief => 'raised',
				    -width => 10
				    )->pack(-side => "right", -anchor => "w");
$balloon->attach($blineEntry,-msg => "Baseline designation entry");
$ctrlFrame1->Label(-text => "Baseline")->pack(-side => "right",-anchor => "w");

$blineEntry->bind('<Return>' => sub
		  {
		      ## Must put notebook page selection stuff here!!!
		      $displayNB->raise($blDesig);
		  });


## display/edit the UT offset
my $tOffEntry = $ctrlFrame1->Entry(-textvariable => \$timeOffsetHR,
				    -relief => 'raised',
				    -width => 5
				    )->pack(-side => "right", -anchor => "w");
$balloon->attach($tOffEntry,-msg => "UT time offset entry");
$ctrlFrame1->Label(-text => "UT Offset")->pack(-side => "right",-anchor => "w");

$tOffEntry->bind('<Return>' => sub
		  {
		      $timeOffset = fold360(RAdeg($timeOffsetHR));

		      $timeOffsetHR = RAhms($timeOffset,-1);

		      my $page = $displayNB->raised;
		      &drawTopAxis($DisplayCanvas{$page});
		      &drawTargets($DisplayCanvas{$page},$targDesig,$blDesig);
		      &drawSun($DisplayCanvas{$page});
		      &drawClusters($DisplayCanvas{$page});
		      &drawCurrentUT(\$DisplayCanvas{$page});
		      &drawTimeLabels($DisplayCanvas{$page});
		  });




## Second control frame holds UT, sunrise/sunset, calibrator, print,
## and quit options
my $ctrlFrame2 = $ctrlFrame->Frame(-relief => 'groove',
				   -borderwidth => "2"
				   )->pack(-side => "top", -fill => "x");

## Quit button
my $qButton = 
    $ctrlFrame2->Button(-text => 'Close',
			-command => sub{$MW->withdraw; exit(0); }
			)->pack(-side => "right", -anchor => "w");
$balloon->attach($qButton,-msg => "Vaya con Dios <Alt+q>");


## Print button
my $pButton = 
    $ctrlFrame2->Button(-text => 'Print',
			-command => sub
			{
			    printTDiagram();
			}
			)->pack(-side => "right", -anchor => "w");
$balloon->attach($pButton,-msg => "Print current rendering <Alt+p>");


## Controls for current UT/LST indicator display
my $uButton = 
    $ctrlFrame2->Button(-text => 'Current UT/LST',
			-command => sub
			{
			    ## Toggle the Current UT/LST indicator state
			    $utOn++;
			    $utOn = 0 if ($utOn > 2);

			    my $page = $displayNB->raised();
			    &drawCurrentUT(\$DisplayCanvas{$page});
			}
			)->pack(-side => "left", -anchor => "w");
$balloon->attach($uButton,-msg => "Toggle current UTC/LST rendering <Cntrl+u>");

$MW->bind('<Control-u>' => sub
	  {
	      ## Toggle the Current UT/LST indicator state
	      $utOn++;
	      $utOn = 0 if ($utOn > 2);

	      my $page = $displayNB->raised;
	      &drawCurrentUT(\$DisplayCanvas{$page});
	  });



## Controls for sunrise/sunset display
my $sButton =
    $ctrlFrame2->Button(-text => 'Sunrise/Sunset',
			-command => sub
			{
			    ## Toggle the current sunrise/set rendering state
			    $sunOn++;
			    $sunOn = 0 if ($sunOn > 2);

			    my $page = $displayNB->raised;
			    &drawSun($DisplayCanvas{$page});
			}
			)->pack(-side => "left", -anchor => "w");
$balloon->attach($sButton,-msg => "Toggle sunrise/sunset rendering <Cntrl+s>");

$MW->bind('<Control-s>' => sub
	  {
	      ## Toggle the current sunrise/set rendering state
	      $sunOn++;
	      $sunOn = 0 if ($sunOn > 2);

	      my $page = $displayNB->raised;
	      &drawSun($DisplayCanvas{$page});
	  });

## Controls for timeLabels display
my $tButton =
    $ctrlFrame2->Button(-text => 'Time Labels',
			-command => sub
			{
			    ## Toggle the time labels
			    $timeLabels++;
			    if ($timeLabels >= 4)
			    {
				$timeLabels = 0;
			    }

			    my $page = $displayNB->raised;
			    &drawCurrentUT(\$DisplayCanvas{$page});
			    &drawClusters($DisplayCanvas{$page});
			    &drawTimeLabels($DisplayCanvas{$page});
			}
			)->pack(-side => "left", -anchor => "w");
$balloon->attach($tButton,-msg => "Toggle time labels <Cntrl+t>");

$MW->bind('<Control-t>' => sub
	  {
	      ## Toggle the time labels
	      $timeLabels++;
	      if ($timeLabels == 4)
	      {
		  $timeLabels = 0;
	      }
	      
	      my $page = $displayNB->raised;
	      &drawCurrentUT(\$DisplayCanvas{$page});
	      &drawClusters($DisplayCanvas{$page});
	      &drawTimeLabels($DisplayCanvas{$page});
	  });


## Controls for cluster display
my $clusterOn = 0;
my $cButton = 
    $ctrlFrame2->Button(-text => 'Clusters',
			-command => sub
			{
			    ## Toggle the current cluster transition rendering state
			    ## $clusterOn = !($clusterOn);
			    $clusterOn++;
			    if ($clusterOn == 3)
			    {
				$clusterOn = 0;
			    }

			    my $page = $displayNB->raised;
			    &drawClusters($DisplayCanvas{$page});
			}
			)->pack(-side => "left", -anchor => "w");
$balloon->attach($cButton,-msg => "Toggle cluster rendering <Cntrl+c>");

$MW->bind('<Control-c>' => sub
	  {
	      ## Toggle the current cluster transition rendering state
	      ## $clusterOn = !($clusterOn);
	      $clusterOn++;
	      if ($clusterOn == 3)
	      {
		  $clusterOn = 0;
	      }
	      
	      my $page = $displayNB->raised;
	      &drawClusters($DisplayCanvas{$page});
	  });


# Controls for calibrator rendering
my $c2Button = 
    $ctrlFrame2->Button(-text => 'Calibrators',
			-command => sub
			{
			    ## Toggle the calibrator rendering
			    $calOn = !($calOn);  ## toggle
			    my $trg;
			    foreach $trg (@targets)
			    {
				if (!($target{$trg}))
				{
				    $targetOn{$trg} = $calOn;
				}
			    }

			    my $page = $displayNB->raised;
			    &drawTargets($DisplayCanvas{$page},$targDesig,$blDesig);
			    &drawSun($DisplayCanvas{$page});
			    &drawClusters($DisplayCanvas{$page});
			    &drawCurrentUT(\$DisplayCanvas{$page});
			    &drawTimeLabels($DisplayCanvas{$page});
			}
			)->pack(-side => "left", -anchor => "w");
$balloon->attach($c2Button,-msg => "Toggle calibrator rendering");

## Moon control button
my $mButton = 
    $ctrlFrame2->Button(-text => 'Moon',
			-command => sub
			{
			    $moonOn = (!$moonOn);

			    my $page = $displayNB->raised;
			    &drawTargets($DisplayCanvas{$page},$targDesig,$blDesig);
			    &drawSun($DisplayCanvas{$page});
			    &drawClusters($DisplayCanvas{$page});
			    &drawCurrentUT(\$DisplayCanvas{$page});
			    &drawTimeLabels($DisplayCanvas{$page});
			}
			)->pack(-side => "left", -anchor => "w");
$balloon->attach($mButton,-msg => "Toggle moon rendering <Cntrl+m>");

$MW->bind('<Control-m>' => sub
	  {
	      $moonOn = (!$moonOn);

	      my $page = $displayNB->raised;
	      &drawTargets($DisplayCanvas{$page},$targDesig,$blDesig);
	      &drawSun($DisplayCanvas{$page});
	      &drawClusters($DisplayCanvas{$page});
	      &drawCurrentUT(\$DisplayCanvas{$page});
	      &drawTimeLabels($DisplayCanvas{$page});
	  });



### end control frames

if (defined $blDesig)
{
    $displayNB->raise($blDesig);
}

## Help/status message area
##$statusFrame->pack(-side => 'bottom', fill => 'x');

MainLoop;  ## Perl/TK main event loop

exit(0);  ## end tGui main
## end tGui  ############################################################





#########################################################################
## xCoord -- compute canvas x coordinate corresponding to input time
#########################################################################
sub xCoord
{
## declarations xCoord  #################################################
    ## Subroutine for computing x canvas position for a particular
    ## time.
    my $timeArg = shift @_;

## begin xCoord  ########################################################
    my $dbgname = "tGui::xCoord";

    ## debug: range checking
    if (($timeArg < 0) || ($timeArg > 360.0))
    {
	gcDebug $dbgname, 1, "WARNING: arg $timeArg out of range 0-360\n";
    }

    my $xRet = int($xZero + $timeArg * $xScale);

    return $xRet;

} ## end xCoord  ########################################################



#########################################################################
## readTimingRpt -- read the input timing report
#########################################################################
sub readTimingRpt
{

## declarations readTimingRpt  ##########################################

    my ($modeSwitch,$arg) = @_;  ## extract call arguments
    my @inputStream;

## begin readTimingRpt  #################################################
    my $dbgname = "tGui::readTimingRpt";

    ## Branch on the mode switch used to call readTimingRpt
    if ($modeSwitch == 0) ## mode == 0 -- read from stdin
    {
	@inputStream = <STDIN>;
    } ## end reading from stdin
    elsif (($modeSwitch == 1) && ($arg))  ## mode == 1 -- read from a filename argument
    {
	open (READ,$arg) or die "Unable to open $arg\n";
	@inputStream = <READ>;
	close READ;
    } ## end reading from file
    elsif (($modeSwitch == 2) && ($arg))  ## mode == 2 -- read from the stream passed in
    {
	@inputStream = split /\n/, $arg;
	foreach my $tmp (@inputStream)  ## need to add the linebreaks back in...
	{
	    $tmp .= "\n";
	}
    } ## end reading from call argument
	

    ## Reading the syntax of the timing report as composed by the
    ## timing program -- now contained in @inputStream


    ## Initialize storage we use to contain timing information

    ## Delete old notebook pages for each baseline
    foreach my $bl (@baseline)
    {
	$displayNB->delete($bl);  ## deleting the notebook page
    }

    ## Reset arrays to zero
    $timingDate = $yDay = $JDzeroUT = $LSTzeroUT
	= $LocalZeroUT = $sunRise = $sunSet
	= $moonRise = $moonSet = $moonIllumination = "";
    $preambleText = $input = "";

    %transit = %leadZA = %trailZA = %newDelay
	= %target = %targetOn = %pointing = ();
    @targets = @baseline = @clusters = ();

    ## Reset timing argument
    ## (Now building custom timing argument based on input timings)
    $timingArgument = "-replace ";

    ## Process new input stream
    foreach my $line (@inputStream)
    {
	$input .= $line;  ## accumulate input for display/later access

	if ($line =~ m/^\# Timing summary /)
	{
	    $preambleText .= $line;
	}

	if ($line =~ m/^\#  for timings on/)
	{
	    my @tmp = split /\s+/, $line;
	    $timingDate = $tmp[4];
	    $yDay = $tmp[6];
	    $preambleText .= $line;
	    my @tmp2 = split /\//, $timingDate;
	    $timingArgument .= "-month $tmp2[0] -day $tmp2[1] -year $tmp2[2] ";
	}

	if ($line =~ m/^\# The JD at 0 hr UT is/)
	{
	    my @tmp = split /\s+/, $line;
	    $JDzeroUT = $tmp[8];
	    $preambleText .= $line;
	}

	if ($line =~ m/^\# The LST at 0 hr UT is/)
	{
	    my @tmp = split /\s+/, $line;
	    $LSTzeroUT = $tmp[8];
	    $preambleText .= $line;
	} ## end reading LST

	if ($line =~ m/^\# The Local Time at 0 hr UT is/)
	{
	    my @tmp = split /\s+/, $line;
	    $LocalZeroUT = $tmp[9];
	    $preambleText .= $line;
	} ## end reading LST

	if ($line =~ m/^\# Approximate UT sunset -- sunrise times/)
	{
	    my @tmp = split /\s+/, $line;
	    $sunSet = $tmp[7];
	    $sunRise = $tmp[9];
	    $preambleText .= $line;
	} ## end reading sunrise/sunset


	if ($line =~ m/^\# Approximate LST at sunset -- sunrise/)
	{
	    $preambleText .= $line;
	} ## end reading sunrise/sunset LST


# Approximate UT moonrise -- moonset times: 22:05:36 -- 11:56:39
	if ($line =~ m/^\# Approximate UT moonrise -- moonset times/)
	{
	    my @tmp = split /\s+/, $line;
	    $moonRise = &RAdeg($tmp[7]);  ## convert to decimal degrees
	    $moonSet = &RAdeg($tmp[9]);
	    $preambleText .= $line;

	    if ($moonRise > $moonSet)
	    {
		$moonRise -= 360.0;
	    }
	} ## end reading moonrise/moonset


#   94% moon illumination -- 2.2 days until full moon (moon at 04:26 RA  +19:46 Dec)
	if ($line =~ m/^\#\s+\d+\% moon illumination -- /)
	{
	    my @tmp = split /\s+/, $line;
	    $preambleText .= $line;

	    if ($tmp[1] =~ m/\d+/)
	    {
		$moonIllumination = $tmp[1];
	    }

	} ## end reading moon illimination


        # Using Palomar location (long,lat: -116.8633, 33.3567)
	if ($line =~ m/^\# Using (.*) Location/)
	{
	    $preambleText .= $line;
	    my $trg = "-location $1"; ## use captured search target
	    if (!($timingArgument =~ m/$trg/))  ## avoid duplications
	    {
		$timingArgument .= "$trg ";
	    }
	} ## end reading location string

        # Using PTI_NS Baseline (ENUBias: -37.116352, -103.264746, 3.319338, -12.915245)
	if ($line =~ m/^\# Using (.*) Baseline/)
	{
	    $preambleText .= $line;
	    my $bName = $1;  ## use the captured search target
	    if (!(grep /$bName/, @baseline))  ## check for redundancy
	    {
		push @baseline, $bName;	## append new baseline designation
		$timingArgument .= "-baseline $bName ";
	    }
	} ## end reading baseline string

	if ($line =~ m/zenith angle constraint|zenith angle (.+) constraint|delay constraint|delay interval/)
	{
	    $preambleText .= $line;  ## add to preamble
	    if ($line =~ m/^\# Using (.*) deg zenith angle/)
	    {
		$timingArgument .= "-zenith $1 ";  ## used captured search target
	    }
	    if ($line =~ m/^\# Using (.*) m delay constraint/)
	    {
		$timingArgument .= "-delayTarget $1 ";  ## used captured search target
	    }
	    if ($line =~ m/^\# Using (.*) to (.*) m delay interval/)
	    {
		$timingArgument .= "-delayMin $1 -delayMax $2 ";  ## used captured search targets
		if ($line =~ m/with (.*) m bias offset/)
		{
		    $timingArgument .= "-bias $1 ";  ## used captured search target
		}
	    }
	    
	} ## end dealing with zenith angle/delay specification lines



	## mwc342 20 23 19.5  +39 31 15.6  transits at 16:59 UT, zenAng 19.7 deg (1.06 am) target

	if ($line =~ m/^\# \S+ \S+ \S+ \S+  \S+ \S+ \S+  transits at/)
	{
	    # Extract name, transit time
	    my @tmp = split /\s+/, $line;
	    my $name = $tmp[1];
	    my $tTime = $tmp[10];

	    if (!(exists $transit{$name}))
	    {
		push @targets, $name;  ## Create list of targets, avoiding duplications
	    }

	    $transit{$name} = RAdeg($tTime);

	    if ($line =~ m/target/)  ## Add support for differentiating targets and cals
	    {
		$target{$name} = $targetOn{$name} = 1;
	    }
	    else
	    {
		$target{$name} = $targetOn{$name} = 0;
	    }
	} # end if transits line

	if ($line =~ m/^\# \S+ is within zenith angle range from/)
	{
	    my @tmp = split /\s+/, $line;

	    my $name = $tmp[1];
	    ## New -- reference _everything_ to the transit timing -- A.B. 1/2002
	    my ($leadZA,$trailZA)
		= &relativeTimings(&RAdeg($tmp[8]),&RAdeg($tmp[10]),$transit{$name});

	    $leadZA{$name} = $leadZA;
	    $trailZA{$name} = $trailZA;
	} # end zenith angle line

	if ($line =~ m/^\# \S+ is always within zenith angle range/)
	{
	    my @tmp = split /\s+/, $line;
	    my $name = $tmp[1];
	    $leadZA{$name} = -180.0;
	    $trailZA{$name} = +180.0;
	} # end always within zenith angle range


	## HDC131511 is within delay range from 01:12:58 to 05:06:13 UT for PTI_NW
	if ($line =~ m/^\# \S+ is within delay range from/)
	{
	    my @tmp = split /\s+/, $line;
	    
	    my $name = $tmp[1];
	    ## New -- reference everything to the transit timing -- A.B. 1/2002
	    my ($leadDA,$trailDA)
		= &relativeTimings(RAdeg($tmp[7]),RAdeg($tmp[9]),$transit{$name});
	    my $bl = $tmp[12];

	    ## If this data hasn't already been inserted...
	    my $delaystr = join ':', $bl,$leadDA,$trailDA;
	    gcDebug $dbgname, 1, "object $name checking delay $delaystr\n";
	    if (!(exists $newDelay{$name})
		|| !($newDelay{$name} =~ m/$delaystr/))
	    {
		## compose the string encoding and append to the entry
		if (exists $newDelay{$name})
		{
		    gcDebug $dbgname, 1, "object $name appending delay $delaystr\n";
		    $newDelay{$name} .= "|" . $delaystr;
		}
		else
		{
		    gcDebug $dbgname, 1, "object $name new delay $delaystr\n";
		    $newDelay{$name} = $delaystr;
		}
	    } ## end !$newDelay{$name}
	} # end delay line range


	# HDC147933 is always within delay range for KI_K1K2
	if ($line =~ m/^\# \S+ is always within delay range/)
	{
	    my @tmp = split /\s+/, $line;
	    
	    my $name = $tmp[1];
	    my $leadDA = -180.0;  ## Again, relative to transit
	    my $trailDA = +180.0;
	    my $bl = $tmp[8];

	    ## If this data hasn't already been inserted...
	    my $delaystr = join ':', $bl,$leadDA,$trailDA;
	    if (!(exists $newDelay{$name})
		|| !($newDelay{$name} =~ m/$delaystr/))
	    {
		## compose the string encoding and append to the entry
		if (exists $newDelay{$name})
		{
		    $newDelay{$name} .= "|" . $delaystr;
		}
		else
		{
		    $newDelay{$name} = $delaystr;
		}
	    } ## end !$newDelay{$name}
	} ## end always in dl range


	## HDC131511 is within pointing range from 01:12:58 to 05:06:13 UT for PTI_NW
	if ($line =~ m/^\# \S+ is within pointing range/)
	{
	    my @tmp = split /\s+/, $line;

	    my $name = $tmp[1];
	    ## New -- reference everything to the transit timing -- A.B. 1/2002
	    my ($leadPt,$trailPt)
		= &relativeTimings(RAdeg($tmp[7]),RAdeg($tmp[9]),$transit{$name});
	    my $bl = $tmp[12];

	    ## If this data hasn't already been inserted...
	    my $ptstr = join ':', $bl,$leadPt,$trailPt;
	    gcDebug $dbgname, 1, "object $name checking accessibility $ptstr\n";
	    if (!(exists $pointing{$name})
		|| !($pointing{$name} =~ m/$ptstr/))
	    {
		## compose the string encoding and append to the entry
		if (exists $pointing{$name})
		{
		    $pointing{$name} .= "|" . $ptstr;
		    gcDebug $dbgname, 1, "object $name appending accessibility $ptstr\n";
		}
		else
		{
		    $pointing{$name} = $ptstr;
		    gcDebug $dbgname, 1, "object $name new accessibility $ptstr\n";
		}
	    } ## end !$pointing{$name}
	} # end within pointing range


	# HDC147933 is always within pointing range for KI_K1K2
	if ($line =~ m/^\# \S+ is always within pointing range/)
	{
	    my @tmp = split /\s+/, $line;
	    
	    my $name = $tmp[1];
	    my $leadPt = -180.0;  ## Again, relative to transit
	    my $trailPt = +180.0;
	    my $bl = $tmp[8];

	    ## If this data hasn't already been inserted...
	    my $ptstr = join ':', $bl,$leadPt,$trailPt;
	    if (!(exists $pointing{$name})
		|| !($pointing{$name} =~ m/$ptstr/))
	    {
		## compose the string encoding and append to the entry
		if (exists $pointing{$name})
		{
		    $pointing{$name} .= "|" . $ptstr;
		}
		else
		{
		    $pointing{$name} = $ptstr;
		}
	    } ## end !$pointing{$name}
	} ## end always in pointing range

	# HDC147933 is never within pointing range for KI_K1K2
	# Note: we must record the empty pointing range so we can distinguish
	# this "never in range" case from the case where no pointing limits
	# are in effect for the given baseline. This is used when
	# computing the intersection track. Empty zenith/delay ranges
	# don't need to be stored since the intersection track code assumes
	# those are always applicable.
	if ($line =~ m/^\# \S+ is never within pointing range/)
	{
	    my @tmp = split /\s+/, $line;
	    my $name = $tmp[1];
	    my $bl = $tmp[8];
	    my $ptstr = $bl . "::";	# store empty pointing range
	    if (!(exists $pointing{$name}))
	    {
		## compose the string encoding and append to the entry
		if (exists $pointing{$name})
		{
		    $pointing{$name} .= "|" . $ptstr;
		}
		else
		{
		    $pointing{$name} = $ptstr;
		}
	    } ## end !$pointing{$name}
	} ## end never in pointing range

	# HDC147933 is out of pointing range at transit for KI_K1K2
	if ($line =~ m/^\# \S+ is out of pointing range at transit/)
	{
	    my @tmp = split /\s+/, $line;
	    my $name = $tmp[1];
	    my $bl = $tmp[8];
	    $transitWarn{$name} = 1;
	} ## end out of pointing range at transit

    } # end while

    # 5/07 SLG: As of getCal release 2.10, timing will generate information
    # for a "Composite" baseline if more than one baseline was included, but
    # not a "... Using Composite Baseline ..." line, so it won't
    # be picked up above. We have to prepend it to the baseline list here.
    if ((scalar @baseline > 1) && !(grep /Composite/, @baseline))
    {
	unshift @baseline, "Composite";	# insert "Composite" at the front of the baseline list
    }
    foreach my $bl (@baseline)
    {
	gcDebug $dbgname, 1, "using baseline $bl\n";
    }

    ## Look for Cluster transition messages A.B. 8/01
    my @clusterList = grep /\#\# Cluster/, split /\n/, $input;
    my $cluster;
    foreach $cluster (@clusterList)
    {
	## Cluster 1: sunrise to 4:30 UT (zenith angle 45 deg)
        ## Cluster 2: 4:30 to 7:30 UT

	my ($first,$last) = split /:\s+/, $cluster;
	my ($time,$rest) = split /\s+/, $last;
	if ($time =~ m/\d/)
	{
	    push @clusters, &RAdeg($time,-1);
	}

    } ## end foreach $cluster


    ## Clean-up the preamble and input text a little
    $preambleText =~ s/\#//g;
    $input =~ s/\"//g; ## quotes (as arcsec) interact badly on recomputation

    ## Check that we got baseline information
    if  (!($preambleText =~ m/Using .* Baseline/))
    {
	print "tGui: No baseline information found in input stream, exiting.\n";
	exit(0);
    } ## no baseline/delay timing info -- exit

    ## 5/30/07 SLG: as of getCal release 2.10, timing reports covering
    ## multiple baselines will also contain information for the "Composite"
    ## baseline, so there is no longer any need to compute it here.

    ## Create notebook pages for each baseline
    foreach my $bl (@baseline)
    {
	gcDebug $dbgname, 2, "Creating notebook page for baseline $bl\n";
	my $page = $displayNB->add($bl,-label => $bl,  ## Notebook page
				   -raisecmd => sub
				   {
				       $blDesig = $bl;
				       &drawTopAxis($DisplayCanvas{$bl});
				       &drawTargets($DisplayCanvas{$bl},$targDesig,$bl);
				       &drawSun($DisplayCanvas{$bl});
				       &drawClusters($DisplayCanvas{$bl});
				       &drawCurrentUT(\$DisplayCanvas{$bl});
				       &drawTimeLabels($DisplayCanvas{$bl});
				   }
				   );
				       
	$DisplayCanvas{$bl}
	= $page->Scrolled('Canvas',-width => $xCanvasSize,-height => $yCanvasSize
			  )->pack(-side => "top",-expand => "yes",-fill => "both");
	$balloon->attach($DisplayCanvas{$bl},-msg => "$bl display canvas");

    }


} ## end readTimingRpt  #################################################


#########################################################################
## relativeTimings -- compute transit-referenced interval times
#########################################################################
sub relativeTimings
{
## declarations relativeTimings  ########################################
    ## relativeTimings computes the relative (transit-referenced) timings

    ## relativeTimings consolidates the logic for correctly
    ## interpreting the relative timing information between limits
    ## (zenith, delay, pointing) and the transit of an object.  Of
    ## course the tricky part of this is the 0/360 degree wrap encoded
    ## in the input data.  It begs the question of whether I should
    ## change the output format of timing...?

    my ($lead,$trail,$transit) = @_;

## begin relativeTimings  ###############################################
    ## Step 1: By definition the $trail must follow $lead, so if this
    ## is not the case we augment the $trail -- this must be a wrap
    ## effect...and now $trail is by definition on the same branch as
    ## $lead...
    $trail += 360.0 if ($trail < $lead);
    ## Now compute the midpoint...
    my $midPt = ($lead + $trail) / 2.0;


    ## Step 2: Choose the branch that minimizes the distance between
    ## $lead and $transit (this seems most reasonable)...
    if ($midPt > $transit)
    {
	if (abs(($midPt - 360.0) - $transit) < abs($midPt - $transit))
	{
	    $lead -= 360.0;
	    $trail -= 360.0;
	}
    } ## $lead > $transit
    else ## otherwise $lead < $transit
    {
	if (abs(($midPt + 360.0) - $transit) < abs($midPt - $transit))
	{
	    $lead += 360.0;
	    $trail += 360.0;
	}
    } ## end $lead < $transit

    ## Step 3: Reference to $transit...
    $lead -= $transit;
    $trail -= $transit;

    return ($lead,$trail);

} ## end relativeTimings  ###############################################



#########################################################################
## drawTargets -- draw targets on the timing plot canvas
#########################################################################
sub drawTargets
{
## declarations drawTargets  ############################################
    ## drawTargets renders the target accesibility limits (zenith, delay)

    ## Extract call arguments -- target and baseline designations
    my $canvas = shift @_;  ## canvas we're drawing to
    my $trg = shift @_;     ## target keyword
    my $bl = shift @_;      ## baseline designation

## begin drawTargets  ###################################################
    my $dbgname = "tGui::drawTargets";

    ## Clean-up the old lines
    $canvas->delete("target");
    $canvas->delete("calibrator");
    $canvas->delete("moon");
    $canvas->delete("yAxis");

    my $key;
    my $tCount = 0;
    my $ySpace = 25;
    my $width = 3;	# standard thickness of bars
    my $offset = 6;	# separation between bars

    foreach $key (@targets)  ## walk target list in input order
    {
	if ($targetOn{$key}) ## Render the targets that are turned "on"
	{
	    my $tag;
	    if ($target{$key})  ## render targets and calibrators with different tags
	    {
		$tag = "target";
		$tCount++;
	    }
	    else
	    {
		$tag = "calibrator";
		$tCount++;
	    }
	    my $targetY = $yZero + $tCount * $ySpace;
	    # target label
	    $canvas->createText(xCoord(0) - 5,$targetY,
				-anchor => "e", -text => $key, -tags => $tag);  ## left side label
	    $canvas->createText(xCoord(360) + 5,$targetY,
				-anchor => "w", -text => $key, -tags => $tag);  ## right side label


	    # transit marker
	    my $transitTime = fold360($transit{$key} - $timeOffset);
	    my @tCenter = (xCoord($transitTime),$targetY);
	    my $tSize = 8;
	    my $color = $tGuiAttributes::transitColor;
	    my $triangle;
###	    if ($transitWarn{$key})
###	    {
###		# highlight the transit symbol with a yellow background
###		my $wSize = $tSize+2;
###		$triangle =
###		    $canvas->createPolygon($tCenter[0],$tCenter[1]-$wSize,
###					   $tCenter[0] - $wSize,$tCenter[1] + $wSize,
###					   $tCenter[0] + $wSize,$tCenter[1] + $wSize,
###					   -fill => 'yellow',
###					   -tags => $tag);
###	    }
	    if ($target{$key})  ## render targets and calibrators with slightly different colors
	    {
		$triangle =
		    $canvas->createPolygon($tCenter[0],$tCenter[1] - $tSize,
					   $tCenter[0] - $tSize,$tCenter[1] + $tSize,
					   $tCenter[0] + $tSize,$tCenter[1] + $tSize,
					   -fill => $color,
					   -tags => $tag);
	    }
	    else  ## otherwise this is a calibrator -- render muted
	    {
		$triangle =
		    $canvas->createPolygon($tCenter[0],$tCenter[1] - $tSize,
					   $tCenter[0] - $tSize,$tCenter[1] + $tSize,
					   $tCenter[0] + $tSize,$tCenter[1] + $tSize,
					   -stipple => 'gray75',
					   -fill => $color,
					   -tags => $tag);
	    }
	    if ($transitWarn{$key})
	    {
		# draw a special warning symbol on top of the transit marker,
		# and on top of any range lines
		my $wSize = $tSize+2;
		$canvas->createLine($tCenter[0],$tCenter[1] - $wSize,
			            $tCenter[0],$tCenter[1] + $wSize,
			            -width => 5,
			            -fill => "black",
				    -tags => $tag);
	    }

	    if ($delayTrack)
	    {
		gcDebug $dbgname, 1, "drawing delay track for bl $bl\n";
		## Render delay limits as selected
		## new extraction algorithms for multiple baselines
		if ($newDelay{$key})
		{
		    my @delay = grep /$bl/, (split '\|', $newDelay{$key});
		    foreach my $dseg (@delay)
		    {
			if ($dseg)
			{
			    (undef, my $dEntryTime, my $dExitTime) = split /:/, $dseg;
			    if (($dEntryTime =~ m/\d+/) &&
				($dExitTime =~ m/\d+/))
			    {
				gcDebug $dbgname, 1, "drawing delay for $key, entry $dEntryTime, exit $dExitTime\n";
				## New -- all times are relative to transit
				&drawTargetLine($canvas,$dEntryTime + $transit{$key},
					$dExitTime + $transit{$key},
					$targetY + $offset, $width,
					$tGuiAttributes::delayColor,$tag);
			    }
			}
		    }
		}
	    } ## end if $delayTrack


	    if ($zaTrack 
		&& (defined $leadZA{$key})
		&& (defined $trailZA{$key}))
	    {
		gcDebug $dbgname, 1, "drawing zenith track for bl $bl\n";
		# Render zenith limits as selected
		my $zEntryTime = $leadZA{$key};
		my $zExitTime = $trailZA{$key};

		&drawTargetLine($canvas,$zEntryTime + $transit{$key},
				$zExitTime + $transit{$key},
				$targetY, $width,
				$tGuiAttributes::zaColor,$tag);
	    } ## end render ZA tracks

	    ## Check for pointing range
	    if ($pointingTrack
		&& (exists $pointing{$key}) && ($pointing{$key} =~ m/$bl/))
	    {
		gcDebug $dbgname, 1, "drawing pointing track for bl $bl\n";
		my @pt = grep /$bl/, (split '\|', $pointing{$key});
		foreach my $pseg (@pt)
		{
		    (undef, my $pEntryTime, my $pExitTime) = split /:/, $pseg;
		    if (($pEntryTime =~ m/\d+/) &&
		    	($pExitTime =~ m/\d+/))
		    {
			## Back to angle-time space for the plot coordinates
			$pEntryTime += $transit{$key};
			$pExitTime += $transit{$key};
			gcDebug $dbgname, 1, "drawing pointing range for $key, entry $pEntryTime, exit $pExitTime\n";
			&drawTargetLine($canvas,$pEntryTime,$pExitTime,
					$targetY - $offset,
					$tGuiAttributes::pointingThickness,
					$tGuiAttributes::pointingColor,
					$tag);
		    } ## if $oEntryTime and $oExitTime
		}
	    } ## end pointing range on this baseline

	    ## Render intersection track
	    if ($intersectionTrack)
	    {
		gcDebug $dbgname, 1, "drawing intersection track\n";
		# Initialize the intersection list with the ZA limits
		if ((defined $leadZA{$key}) &&
		    (defined $trailZA{$key}))
		{
		    # 1) start the intersection list with the ZA range
		    my @iList = ( [$leadZA{$key}, $trailZA{$key}] ) ;

		    # 2) intersect that with a list of delay limits
		    if ((exists $newDelay{$key}) && ($newDelay{$key} =~ m/$bl/))
		    {
			my @delay = grep /$bl/, (split '\|', $newDelay{$key});
			my @dList;
			foreach my $dseg (@delay)
			{
			    (undef, my $dEntryTime, my $dExitTime) = split /:/, $dseg;
			    if (($dEntryTime =~ m/\d+/)
				&& ($dExitTime =~ m/\d+/))
			    {
				push @dList, [$dEntryTime, $dExitTime];
			    }
			}
			## Do the intersection computation (in hour angle to avoid wraps)
			intersectList(\@iList, \@dList, \@iList);	# intersect composite list with delay

			# 3) If baseline has pointing limits defined, intersect with pointing limits
			if ((exists $pointing{$key}) && ($pointing{$key} =~ m/$bl/))
			{
			    my @pt = grep /$bl/, (split '\|', $pointing{$key});
			    my @pList;
			    foreach my $pseg (@pt)
			    {
				(undef, my $pEntryTime, my $pExitTime) = split /:/, $pseg;
				if (($pEntryTime =~ m/\d+/)
				    && ($pExitTime =~ m/\d+/))
				{
				    push @pList, [$pEntryTime, $pExitTime];
				}
			    }
			    intersectList(\@iList, \@pList, \@iList);	# intersect composite list with pointing
			}

			# Now, @iList contains a list of intersected entry/exit times covering
			# zenith, delay , and pointing (if pointing limits are defined for this baseline).
			# Draw that intersection.
			for (my $i = 0; $i < scalar(@iList); $i++)
			{
			    my $iEntryTime = $iList[$i][0];
			    my $iExitTime = $iList[$i][1];

			    ## Back to angle-time space for the plot coordinates
			    $iEntryTime += $transit{$key};
			    $iExitTime += $transit{$key};
			    gcDebug $dbgname, 1, "drawing intersection for $key, entry $iEntryTime, exit $iExitTime\n";
			    &drawTargetLine($canvas,$iEntryTime,$iExitTime,$targetY,
					    $width,$tGuiAttributes::intersectionColor,
					    $tag);
			} # end interval
		    } # end delay
		} # end za
	    } ## end render intersection tracks
	} ## if $trg check
    } # end foreach $key loop

    if ($moonOn    ## Draw moon info as requested/available
	&& (defined $moonRise)
	&& (defined $moonSet))
    {

	## drawing the moon
	$tCount++;
	my $targetY = $yZero + $tCount * $ySpace;
	# target label
	my $moonLabel = "Moon";
	if ($moonIllumination)
	{
	    $moonLabel .= " ($moonIllumination)";
	}
	$canvas->createText(xCoord(0) - 5,$targetY,
			    -text => $moonLabel, -tags => "moon",
			    -anchor => "e");  ## left side label
	$canvas->createText(xCoord(360) + 5,$targetY,
			    -text => $moonLabel, -tags => "moon",
			    -anchor => "w");  ## right side label

	&drawTargetLine($canvas,$moonRise,$moonSet,
			$targetY,$width,$tGuiAttributes::moonColor,
			"moon");

    } ## end drawing the moon

    ## draw the y axis to adjust to the number of objects
    $tCount++;
    $maxY = $yZero + $tCount * $ySpace;
    $canvas->createLine(xCoord(0),$yZero,
			xCoord(0),$maxY + 5,-tags => "yAxis"); # New y axis
    $canvas->createLine(xCoord(360),$yZero,
			xCoord(360),$maxY + 5,-tags => "yAxis"); # New y axis

    ## Experiment with extra time markers for reference
    for (my $ut = 0; $ut <= 24; $ut += 1)
    {
	my $t = fold360(($ut*15) - $timeOffset);
	my $xT = xCoord($t);
	if (!($ut % 3))
	{
	    $canvas->createLine($xT,$yZero,$xT,$maxY,
				-stipple=>"gray50",-tags => "yAxis");
	}
	else
	{
	    $canvas->createLine($xT,$yZero,$xT,$yZero + 5,
				-stipple=>"gray50",-tags => "yAxis");
	}
    } ## end ut loop

    # Finally set the canvas scrollable region to include everything
    $canvas->configure(-scrollregion => [ $canvas->bbox("all") ] );

} ## end drawTargets  ###################################################



#########################################################################
## drawTargetLine -- render an individual target line
#########################################################################
sub drawTargetLine
{
## declarations drawTargetLine  #########################################
    ## drawTargetLine renders an individual target line of the desired
    ## color and thickness at the desired position.  This version
    ## checks for wraps of the lines around the HARDWIRED 0/360 deg
    ## boundary.

    my $canvas = shift @_; ## extract canvas we're drawing to

    my ($x1,$x2,$y,$width,$color,$tag) = @_;  ## extract call arguments

## begin drawTargetLine  ################################################
    my $dbgname = "tGui::drawTargetLine";

    my $capwidth = 1;
    my $capoff = 3;


    # shift x coords by timeOffset
    my $tx1 = $x1 - $timeOffset;	# time x1 with offset applied
    my $tx2 = $x2 - $timeOffset;	# time x2 with offset applied

    my $t0 = 0.0;			# time 0
    my $t360 = $t0 + 360.0;		# time 360

    gcDebug $dbgname, 2, "timeOffset=$timeOffset, x1=$x1 x2=$x2, tx1=$tx1 tx2=$tx2 y=$y\n";

    # do some simple sanity checks on the inputs
    my $drawCaps = 1;
    if (($tx2 - $tx1) > 360.0)
    {
	gcDebug $dbgname, 2, "*** length check tx1=$tx1 tx2=$tx2 y=$y\n";
    }
    if (($tx2 - $tx1) == 360.0)
    {
	$drawCaps = 0;	# line spans the complete range so skip drawing the end caps
    }
    if ($tx1 > $tx2)
    {
	gcDebug $dbgname, 2, "*** order check tx1=$tx1 tx2=$tx2 y=$y\n";
    }

    ## check for wrapping and values outside the 0-360 range
    my $wrapped = 0;
    if (($tx1 < $t0) || ($tx1 > $t360) ||
	($tx2 < $t0) || ($tx2 > $t360))
    {
	gcDebug $dbgname, 2, "*** range check tx1=$tx1 tx2=$tx2 y=$y\n";

	## start by getting tx1 into range
	if ($tx1 < $t0)
	{
	    $tx1 += 360.0;
	    $tx2 += 360.0;
	    gcDebug $dbgname, 2, "shifted right to tx1=$tx1 tx2=$tx2 y=$y\n";
	}
	elsif ($tx1 > $t360)
	{
	    $tx1 -= 360.0;
	    $tx2 -= 360.0;
	    gcDebug $dbgname, 2, "shifted left to tx1=$tx1 tx2=$tx2 y=$y\n";
	}
	# $tx1 should be in range. If $tx2 is still out, we have a wrap.
	if ($tx2 > $t360)
	{
	    $wrapped = 1;
	    gcDebug $dbgname, 2, "*** wrap tx1=$tx1 tx2=$tx2 y=$y\n";

	    # $tx2 is out of range so we need to draw it as two parts.
	    # draw the wrapped part first.
	    my $tx1Tmp = $t0;
	    my $tx2Tmp = $tx2 - 360.0;
	    $tx2 = $t360;
	    gcDebug $dbgname, 2, "drawing wrap: y=$y tx1Tmp=$tx1Tmp tx2Tmp=",$tx2Tmp,"\n";

	    ## render targets and calibrators with slightly
	    ## different colors/patterns
	    if ($tag =~ /target/)
	    {
		$canvas->createLine(xCoord($tx1Tmp),$y,xCoord($tx2Tmp),$y,
				    -width => $width,-fill => $color,
				    # -capstyle => 'projecting',
				    -tags => $tag);
	    } ## end target
	    else
	    {
		$canvas->createLine(xCoord($tx1Tmp),$y,xCoord($tx2Tmp),$y,
				    -width => $width,-fill => $color,
				    # -capstyle => 'round',
				    -stipple => 'gray75',
				    -tags => $tag);
	    } ## end calibrator

	    if ($drawCaps)
	    {
		## draw a "cap" at the end of the segment
		$canvas->createLine(xCoord($tx2Tmp),$y+$capoff,xCoord($tx2Tmp),$y-$capoff,
				    -width => $capwidth, -fill => $color,
				    # -capstyle => 'projecting',
				    -tags => $tag);
	    }
	}
    }

    ## draw the segment
    gcDebug $dbgname, 2, "drawing: y=$y tx1=$tx1 tx2=",$tx2,"\n";

    ## render targets and calibrators with slightly
    ## different colors/patterns
    if ($tag =~ /target/)
    {
	$canvas->createLine(xCoord($tx1),$y,xCoord($tx2),$y,
			    -width => $width,-fill => $color,
			    # -capstyle => 'projecting',
			    -tags => $tag);
    } ## end target
    else
    {
	$canvas->createLine(xCoord($tx1),$y,xCoord($tx2),$y,
			    -width => $width,-fill => $color,
			    # -capstyle => 'round',
			    -stipple => 'gray75',
			    -tags => $tag);
    } ## end calibrator

    if ($drawCaps)
    {
	## draw a "cap" at the start of the segment
	$canvas->createLine(xCoord($tx1),$y+$capoff,xCoord($tx1),$y-$capoff,
			    -width => $capwidth, -fill => $color,
			    # -capstyle => 'projecting',
			    -tags => $tag);
	if (!$wrapped)
	{
	    ## draw a "cap" at the end of the segment
	    $canvas->createLine(xCoord($tx2),$y+$capoff,xCoord($tx2),$y-$capoff,
			    -width => $capwidth, -fill => $color,
			    # -capstyle => 'projecting',
			    -tags => $tag);
	}
    } ## end drawCaps

} ## end drawTargetLine  ################################################



#########################################################################
## drawTopAxis -- render top UT axis for timing plot
#########################################################################
sub drawTopAxis
{
## declarations drawTopAxis  ############################################
    ## drawTopAxis renders the top UT axis for tGui
    my $canvas = shift @_; ## extract canvas we're drawing to

## begin drawTopAxis  ###################################################
    ## Get rid of any old stuff
    $canvas->delete("UTAxis");


    ## Now the new stuff
    $canvas->createText(xCoord(0) - 25,$yZero / 2,
			-text => "UTC",-anchor => "e",-tags => "UTAxis");  ## Denoting UT axis
    ## Major Tic Labels
    for (my $ut = 0; $ut < 24; $ut += 3)
    {
	my $t = fold360(($ut*15) - $timeOffset);
	my $utTxt = &RAhms($ut*15,-1);
	$utTxt =~ s/\s+/:/g;
	$canvas->createText(xCoord($t),$yZero / 2,
			    -text => "$utTxt",-tags => "UTAxis");
	if ($t == 0)
	{
	    # repeat label on right side
	    $canvas->createText(xCoord(360),$yZero / 2,
			    -text => "$utTxt",-tags => "UTAxis");
	}
    } ## end for loop

    $canvas->createLine(xCoord(0),$yZero,
			xCoord(360),$yZero,-tags => "UTAxis"); # x axis

} ## end drawTopAxis  ###################################################


#########################################################################
## setCurrentTime -- compose time strings for current times
#########################################################################
sub setCurrentTime
{
    ## setCurrentTime composes time strings for current UT, LST, and
    ## local time for running display

## begin setCurrentTime  ################################################
    # get the current UT
    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = gmtime(time);
    my $ut = $hour + $min / 60.0 + $sec / 3600.0;  ## Current UTC

    ## Compose UT string
    $currentUT = &RAhms(15.0 * $ut,-1);
    $currentUT =~ s/\s+/:/g;
    $currentUT .= " UTC";

    
    ## Now the LST string
    my ($lstH,$lstM,$lstS) = split /:/, $LSTzeroUT;
    my $lst = 15.0 * ($lstH + $lstM / 60.0 + $lstS / 3600.0
		      + (novasConstants::SolarToSidereal * $ut));  ## correction 12/01 AB
    if ($lst > 360.0) ## Fix wraps
    {
	$lst -= 360.0;
    }
    $currentLST = &RAhms($lst,-1);
    $currentLST =~ s/\s+/:/g;
    $currentLST .= " LST";


    ## Finally the local time string
    if ($LocalZeroUT)
    {
	($lstH,$lstM,$lstS) = split /:/, $LocalZeroUT;
	$lst = 15.0 * ($lstH + $lstM / 60.0 + $lstS / 3600.0 + $ut);
	if ($lst > 360.0) ## Fix wraps
	{
	    $lst -= 360.0;
	}
	$currentLocal = &RAhms($lst,-1);
	$currentLocal =~ s/\s+/:/g;
	$currentLocal .= " Local";
    }
    else
    {
	$currentLocal = "Not Avail";
    }


    $MW->after(15000, \&setCurrentTime);  ## recompute current time every 15 seconds

} ## end setCurrentTime  ################################################



#########################################################################
## drawCurrentUT -- renders information about the current time
#########################################################################
sub drawCurrentUT
{
## declarations drawCurrentUT  ##########################################
    ## drawCurrentUT draws a vertical line indicating the present time

    my $canvasRef = shift @_; ## extract canvas we're drawing to
    my $canvas = $$canvasRef;

## begin drawCurrentUT  #################################################
    $canvas->delete("UTLine");  ## delete previous UT line and labels

    if ($utOn) ## If we're going to draw a UT line
    {
	# get the current UT
	my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = gmtime(time);
	my $ut = $hour + $min / 60.0 + $sec / 3600.0;  ## Current UTC

	my $t = fold360($ut*15 - $timeOffset);
	my $xT = xCoord($t);

	# render the UT line and UT/LST labels
	$canvas->createLine($xT,$yZero,
			    $xT,$maxY,
			    -tags => "UTLine",
			    -fill => $tGuiAttributes::currentUTColor);

	if ($utOn > 1)  ## If adding annotations
	{

	    ## beautify the format and create a string
	    my $utStr = RAhms($ut * 15.0,-1);
	    $utStr =~ s/\s+/:/g;
	    $utStr .= " UTC";

	    $canvas->createText($xT,$yZero - 5,
				-text => $utStr,
				-tags => "UTLine");

	    my $yAxisLoc = $maxY + 5;

	    ## New Feb 2004 -- draw bottom time labels appropriate for
	    ## the bottom axis  AFB
	    if ($timeLabels == 1) ## doing LST
	    {
		my $lst = &lstFromUT($ut);
		$lst = &RAhms($lst,-1);
		$lst =~ s/\s+/:/g;
		$lst .= " LST";
		$canvas->createText($xT,$yAxisLoc,
				    -text => $lst,-tags => "UTLine");
	    } ## end LST labels
	    elsif (($timeLabels == 2) && ($LocalZeroUT)) ## doing local
	    {
		my $loc = &localFromUT($ut);
		$loc = &RAhms($loc,-1);
		$loc =~ s/\s+/:/g;
		$loc .= " Local";
		$canvas->createText($xT,$yAxisLoc,
				    -text => $loc,-tags => "UTLine");
	    } ## end local labels
	    elsif ($timeLabels == 3)  ## doing UT again
	    {
		$canvas->createText($xT,$yAxisLoc,
				    -text => $utStr,-tags => "UTLine");

	    } ## end UT labels

	} ## end if $utOn > 1

        ## re-render UT line every 10 seconds
	##  Note: strange syntax for invoking callback with argument
	$MW->after(10000, [\&drawCurrentUT, Ev($canvasRef)]);  

    } ## end if $utOn

} ## end drawCurrentUT  #################################################



#########################################################################
## drawSun -- draw info about sunrise/sunset
#########################################################################
sub drawSun
{
## declarations drawSun  ################################################
    ## drawSun renderes the sunrise/sunset info on the timing gui

    my $canvas = shift @_; ## extract canvas we're drawing to

## begin drawSun  #######################################################
    $canvas->delete("SunLine");  ## Delete previous sunrise/sunset lines and labels

    if ($sunOn)  ## If we're going to render the sunset/sunrise...
    {

	if ($sunSet)
	{
	    my $sunset = fold360(RAdeg($sunSet) - $timeOffset);
	    $canvas->createLine(xCoord($sunset),$yZero,xCoord($sunset),$maxY,
				-fill => $tGuiAttributes::sunColor,
				-tags => "SunLine"); # Sunset line 

	    if ($sunOn > 1)
	    {
		my @tmp = split /\:/, $sunSet;
		my $sunSetStr = join ':', $tmp[0],$tmp[1];
		$sunSetStr .= " UTC";
		$canvas->createText(xCoord($sunset),$yZero - 5,
				    -text => $sunSetStr,-tags => "SunLine");
		$canvas->createText(xCoord($sunset),$maxY + 5,
				    -text => 'Sunset',-tags => "SunLine");
	    } ## end if $sunOn > 1
	} ## end sunset

	if ($sunRise)
	{
	    my $sunrise = fold360(RAdeg($sunRise) - $timeOffset);
	    $canvas->createLine(xCoord($sunrise),$yZero,xCoord($sunrise),$maxY,
				-fill => $tGuiAttributes::sunColor,
				-tags => "SunLine"); # Sunrise line
	    if ($sunOn > 1)
	    {
		my @tmp = split /\:/, $sunRise;
		my $sunRiseStr = join ':', $tmp[0],$tmp[1];
		$sunRiseStr .= " UTC";
		$canvas->createText(xCoord($sunrise),$yZero - 5,
				    -text => $sunRiseStr,-tags => "SunLine");
		$canvas->createText(xCoord($sunrise),$maxY + 5,
				    -text => 'Sunrise',-tags => "SunLine");
	    } ## end if $sunOn > 1
	} ## end sunrise

    } ## end $sunOn

} ## end drawSun  #######################################################



#########################################################################
## drawClusters -- draw info about schedule clusters 
#########################################################################
sub drawClusters
{
## declarations drawClusters  ###########################################
    ## drawClusters renderes the cluster transition info on the timing gui

    my $canvas = shift @_; ## extract canvas we're drawing to
    my $yAxisLoc = $maxY + 10;

## begin drawClusters  ##################################################
    $canvas->delete("ClusterLine");  ## Delete previous cluster lines and labels

    if ($clusterOn && (scalar @clusters))  ## If we're going to render the clusters
    {

	my $cluster;
	foreach $cluster (@clusters)
	{
	    my $xC = xCoord(fold360($cluster - $timeOffset));
	    $canvas->createLine($xC,$yZero,$xC,$maxY,
				-fill => $tGuiAttributes::clusterColor,
				-stipple=>"gray50",
				-tags => "ClusterLine"); # Cluster line

	    if ($clusterOn > 1)
	    {
		my $cText = &RAhms($cluster,-1);
		$cText =~ s/\s+/:/g;
		$canvas->createText($xC,$yZero - 5,
				    -text => $cText,-tags => "ClusterLine");

		## New Feb 2004 -- draw bottom time labels appropriate for
		## the bottom axis  AFB
		if ($timeLabels == 1) ## doing LST
		{
		    my $lst = &lstFromUT($cluster * 24.0 / 360.0);
		    $lst = &RAhms($lst,-1);
		    $lst =~ s/\s+/:/g;
		    $canvas->createText($xC,$yAxisLoc,
					-text => $lst,-tags => "ClusterLine");
		} ## end LST labels
		elsif (($timeLabels == 2) && ($LocalZeroUT)) ## doing local
		{
		    my $loc = &localFromUT($cluster * 24.0 / 360.0);
		    $loc = &RAhms($loc,-1);
		    $loc =~ s/\s+/:/g;
		    $canvas->createText($xC,$yAxisLoc,
					-text => $loc,-tags => "ClusterLine");
		} ## end local labels
		elsif ($timeLabels == 3)  ## doing UT again
		{
		    $canvas->createText($xC,$yAxisLoc,
					-text => $cText,-tags => "ClusterLine");
		} ## end UT labels
	    } ## end if $clusterOn > 1/adding time text
	} ## end foreach $cluster
    } ## end $clusterOn
} ## end drawClusters  ##################################################



#########################################################################
## drawTimeLabels -- draw bottom time axis
#########################################################################
sub drawTimeLabels
{
## declarations drawTimeLabels  #########################################
    ## timeLabels draws a second axis labels with either LST or local
    ## time

    my $canvas = shift @_; ## extract canvas we're drawing to

## begin drawTimeLabels  ################################################
    $canvas->delete("TimeLabels");  ## Delete previous elements

    if ($timeLabels)
    {

	my $yAxisLoc = $maxY + 10;

	# label with units
	my $timeUnit = "";
	if ($timeLabels == 1)
	{
	    $timeUnit = "LST";
	}
	elsif ($timeLabels == 2)
	{
	    $timeUnit = "Local";
	}
	elsif ($timeLabels == 3)
	{
	    $timeUnit = "UTC";
	}
	$canvas->createText(xCoord(0) - 25,$yAxisLoc + 5,-anchor => "e",
		-text => "$timeUnit",-tags => "TimeLabels");

	for (my $ut = 0; $ut < 24; $ut += 3)
	{
	    my $t = fold360(($ut*15) - $timeOffset);

	    my $timeTxt;
	    if ($timeLabels == 1)	# 1=LST
	    {
		my $lst = &lstFromUT($ut);
		$timeTxt = &RAhms($lst,-1);
	    }
	    elsif ($timeLabels == 2)	# 2=Local
	    {
		my $loc = &localFromUT($ut);
		$timeTxt = &RAhms($loc,-1);
	    }
	    elsif ($timeLabels == 3)	# 3=UTC
	    {
		$timeTxt = &RAhms($ut*15,-1);
	    }
	    $timeTxt =~ s/\s+/:/g;

	    $canvas->createText(xCoord($t),$yAxisLoc + 5,
				-text => "$timeTxt",-tags => "TimeLabels");
	    if ($t == 0)
	    {
		# repeat label on right side
		$canvas->createText(xCoord(360),$yAxisLoc + 5,
				-text => "$timeTxt",-tags => "TimeLabels");
	    }
	} ## end for loop

	# reset the canvas scrollable region
	$canvas->configure(-scrollregion => [ $canvas->bbox("all") ] );

    } ## end if $timeLabels
} ## end drawTimeLabels  ################################################


#########################################################################
## dispHelp -- display text contained in a file
#########################################################################
sub dispHelp
{
## declarations dispHelp  ###############################################
    ## dispHelp opens a read-only text window with help information
    ## for tGui
    my $filename = shift @_;

## begin dispHelp  ######################################################
    if (!defined $filename)
    {
	print "Error: undefined document link in tGui\n";
	return;
    }

    ## Basically we open a text window and display the help file -- in
    ## all it's glory.

    ## create a text window to display the return info
    my $helpWindow = $MW->Toplevel();
    $helpWindow->title("help -- $filename");
    my $text = $helpWindow->Scrolled("ROText",
				     -wrap => "none",
				     -width => 90, -height => 32,
				     -relief => "groove"
				     )->pack(-side => "top",
				       -expand => "yes",
				       -fill => "both");

    # dump the contents of the help file into the text window
    my $helpMsg = `cat $filename`;

    $text->insert('end',$helpMsg);

    # add a close button
    my $hwCntrl = $helpWindow->Frame(-relief => "groove",
				     -borderwidth => "2"
				     )->pack(-side => "bottom", -fill => "x");
    $hwCntrl->Button(-text => "Close",
		     -relief => "raised",
		     -command => sub {$helpWindow->withdraw}
		     )->pack(-side => "right");

    $helpWindow->bind('<Return>' => sub { $helpWindow->withdraw });


} ## end dispHelp  ######################################################


#########################################################################
## printTDiagram -- print timing diagram in number of formats
#########################################################################
sub printTDiagram
{
    ## printTDiagram prints a PostScript summary of the timing diagram

## begin printTDiagram  #################################################
    my $dbgname = "tGui::printTDiagram";

    ## Make a new canvas especially for printing
    my $printCanvas = $MW->Scrolled('Canvas',
				    -width => $xCanvasSize,
				    -height => $yCanvasSize);

    &drawTopAxis($printCanvas);
    &drawTargets($printCanvas,$targDesig,$blDesig);
    &drawSun($printCanvas);
    &drawClusters($printCanvas);
    &drawCurrentUT(\$printCanvas);
    &drawTimeLabels($printCanvas);


    ## Add reporting information on the plot

    ## TLE -- 7/1/2004 add Location to plot
    my $tmpTxt;
    my @tmp = grep /Using .* Location/, (split /\n/, $preambleText);
    if ($tmp[0])
    {
        $tmp[0] =~ s/ Using\s+//g;
        $tmpTxt = $tmp[0];
	gcDebug $dbgname, 1, "location: $tmpTxt\n";
	$printCanvas->createText(xCoord(0) - 35,$maxY + 35,
                             -anchor => "w",-text => $tmpTxt);
    }

    if ($intersectionTrack)
    {
        $printCanvas->createText(xCoord(0) + ($xCanvasSize * 0.55),$maxY + 35,
                                 -fill => $tGuiAttributes::intersectionColor,
                                 -anchor => "w",-text => "Intersection Track");
    }   
    else
    {   
        $printCanvas->createText(xCoord(0) + ($xCanvasSize * 0.45),$maxY + 35,
                                 -fill => $tGuiAttributes::zaColor,
                                 -anchor => "w",-text => "Zenith Track");

        $printCanvas->createText(xCoord(0) + ($xCanvasSize * 0.75),$maxY + 35,
                                 -fill => $tGuiAttributes::pointingColor,
                                 -anchor => "w",-text => "Pointing Track");
        
        $printCanvas->createText(xCoord(0) + ($xCanvasSize * 1.2),$maxY + 35,
                                 -fill => $tGuiAttributes::delayColor,
                                 -anchor => "c",-text => "Delay Track");
    }   

    $tmpTxt = "Baseline: " . $blDesig;
    gcDebug $dbgname, 1, "baseline: $tmpTxt\n";
    $printCanvas->createText(xCoord(0)  - 35,$maxY + 85,
                             -anchor => "w",-text => $tmpTxt);


    ## Print info on zenith and delay constraints
    ## Zenith Angle
    @tmp = grep /Using .* zenith angle/, (split /\n/, $preambleText);
    if ($tmp[0])
    {
	$tmp[0] =~ s/ Using\s+//g;
	$tmp[0] =~ s/ \(.*\)//g;
	gcDebug $dbgname, 1, "zenith: $tmp[0]\n";
	$printCanvas->createText(xCoord(0) + ($xCanvasSize * 0.40),$maxY + 60,
				 -fill => $tGuiAttributes::zaColor,
				 -anchor => "w",-text => $tmp[0]);
    } ## end printing ZA constraint

    ## delay
    @tmp = grep /Using .* delay interval .* on baseline $blDesig/, (split /\n/, $preambleText);
    if ($tmp[0])
    {
	$tmp[0] =~ s/ Using\s+//g;
	$tmp[0] =~ s/ \(.*\)//g;
	$tmp[0] =~ s/ on baseline $blDesig$//g;
	gcDebug $dbgname, 1, "delay: $tmp[0]\n";
	$printCanvas->createText(xCoord(0) + ($xCanvasSize * 1.2),$maxY + 60,
				 -fill => $tGuiAttributes::delayColor,
				 -anchor => "c",-text => $tmp[0]);
    } ## end printing delay constraint

    ## my @preambleLines = split /\n/, $preambleText;
    @tmp = grep /for timings on/, (split /\n/, $preambleText);
    if ($tmp[0])
    {
        $tmp[0] =~ s/ for timings on\s+//g;
	gcDebug $dbgname, 1, "date: $tmp[0]\n";
        $printCanvas->createText(xCoord(0) - 35, $maxY + 60,
                                 -anchor => "w",-text => $tmp[0]);
    } ## end printing timing date



    ## Print program/institutional info
    $tmpTxt = "Produced by timing GUI v$REVISION ($PKGSTR)";
    gcDebug $dbgname, 1, "program: $tmpTxt\n";
    $printCanvas->createText(xCoord(0) + ($xCanvasSize * 1.2),$maxY + 85,
			     -anchor => "c",-text => $tmpTxt);

    $tmpTxt = "NASA Exoplanet Science Institute -- http://nexsci.caltech.edu/";
    gcDebug $dbgname, 1, "org: $tmpTxt\n";
    $printCanvas->createText(xCoord(0) + ($xCanvasSize * 0.65),$maxY + 85,
			     -fill => 'blue',
			     -anchor => "c",-text => $tmpTxt);

    ## A micro-logo for institutional awarness purposes
    ####  logo disabled, causes failure of postscript output in older Tk (800.025) - SG 8/28/06
    #my $logoFile = findFile("NExScI_logo_micro.gif",$cfg->gcdocs . "/images", $cfg->gcdata . "/images");
    #my $logo = $printCanvas->Photo( 'IMG', -file => $logoFile);
    #$printCanvas->create('image', xCoord(0) + ($xCanvasSize * 1.1),$maxY + 85,
    # 			    -anchor => "c", '-image'  => $logo );

    ## Convert the canvas to PostScript
    my ($bboxminX,$bboxminY,$bboxmaxX,$bboxmaxY) = $printCanvas->bbox("all");
    gcDebug $dbgname, 1, "bbox ($bboxminX,$bboxminY) ($bboxmaxX,$bboxmaxY)\n";

    my $ps = $printCanvas->postscript(
				-x => 0,
				-y => 0,
				-rotate => 1,
				-pagewidth => "9i",
				-width => $bboxmaxX,
				-height => $bboxmaxY
				   );
    my $fileName = "timing";
    if (defined $batchMode)
    {
	## Do batch mode output. Allow multiple output formats to be specified.
	writePsOutput("$fileName.ps",$ps) if (defined $PostScript); ## output PS
	writePsOutput("$fileName.gif",$ps) if (defined $gif);	## output GIF
	writePsOutput("$fileName.jpg",$ps) if (defined $jpg);	## output JPEG
	writePsOutput("$fileName.png",$ps) if (defined $png);	## output PNG
	writePsOutput("$fileName.pdf",$ps) if (defined $pdf);	## output PDF
    }
    else
    {
	## Do "Print..." dialog to get filename or print command to use, 
	## then output the postscript as requested
	my $fn = &getOutputName($MW->Toplevel(), $balloon, $fileName);
	writePsOutput($fn,$ps);
    }
} ## end printTDiagram  #################################################


#########################################################################
## drawPreamble -- draw timing preamble
#########################################################################
sub drawPreamble
{
    ## drawPreamble flushes the contents of the existing preamble
    ## frame and replace it with the new $preambleText

## begin drawPreamble  ##################################################

    $preamble->delete('1.0','end'); ## clear the existing text
    $preamble->insert('end',$preambleText);  ## Insert the new

} ## end drawPreamble  ##################################################



#########################################################################
## packBlineMenu -- back baseline menu with available options
#########################################################################
sub packBlineMenu
{
    ## packBlineMenu flushes the transient baseline entries from the
    ## baseline menu and replaces them with the contents of @baseline
## declarations packBlineMenu  ##########################################
    my $init = shift @_;

## begin packBlineMenu  #################################################
    if (!$init)
    {
	$baselineMenu->menu->delete('3', 'end');
    }


    my $blTmp;
    foreach $blTmp (@baseline)
    {
	$baselineMenu->AddItems(['command' => $blTmp,
				 -command => sub
				 {
				     $displayNB->raise($blTmp);
				 }]);
    } ## end baseline loop

} ## end packBlineMenu  #################################################



#########################################################################
## packTargetMenu -- pack target menu with available options
#########################################################################
sub packTargetMenu
{
    ## packTargetMenu manages the dynamic part of the target menu
    ## contents

## declarations packTargetMenu  #########################################
    my $init = shift @_;

## begin packTargetMenu  ################################################
    ## Eliminate the extra stuff (if not init)...
    if (!$init)
    {
	$targetMenu->menu->delete('7', 'end');
    }

    my $nTargets = scalar @targets;
    my $nTargMenu = $tGuiAttributes::targMenuItems;  ## Max Items 
    if ($nTargets > $nTargMenu)
    {
	my ($count,@indices);
	$count = 0;
	while ($count < $nTargets)
	{
	    push @indices, $count;
	    $count += $nTargMenu;
	} ## end while
	push @indices, $nTargets;

	my ($i,$j);
	$count = 1;
	for ($i = 0; $i < (scalar @indices - 1); $i++)
	{
	    my $addlTargets = $targetMenu->cascade(-label => "Target List $count");
	    for ($j = $indices[$i]; $j < $indices[$i + 1]; $j++)
	    {
		my $targTmp = $targets[$j];
		$addlTargets->checkbutton(-label => $targTmp,
					  -command => sub
					  {
					      ## Note the checkbutton code handles
					      ## the variable toggling for you
					      &drawTargets($DisplayCanvas{$blDesig},$targDesig,$blDesig);
					      &drawSun($DisplayCanvas{$blDesig});
					      &drawClusters($DisplayCanvas{$blDesig});
					      &drawCurrentUT(\$DisplayCanvas{$blDesig});
					      &drawTimeLabels($DisplayCanvas{$blDesig});
					  },
					  -variable => \$targetOn{$targTmp});
	    } ## end j for loop
	    $count++;
	} ## end i for loop
    }
    else ## no need for sub-menus -- all in one single menu
    {
	foreach my $targTmp (@targets)
	{
	    $targetMenu->checkbutton(-label => $targTmp,
				     -command => sub
				     {
					 ## Note the checkbutton code handles
					 ## the variable toggling for you
					 &drawTargets($DisplayCanvas{$blDesig},$targDesig,$blDesig);
					 &drawSun($DisplayCanvas{$blDesig});
					 &drawClusters($DisplayCanvas{$blDesig});
					 &drawCurrentUT(\$DisplayCanvas{$blDesig});
					 &drawTimeLabels($DisplayCanvas{$blDesig});
				     },
				     -variable => \$targetOn{$targTmp});
	} ## end target menu packing/for loop
    } ## end else


} ## end packTargetMenu  ################################################



#########################################################################
## packSimbadMenu -- pack Simbad menu with available options
#########################################################################
sub packSimbadMenu
{
    ## packTargetMenuSimbad manages the dynamic part of the Simbad
    ## menu contents

## declarations packSimbadMenu  #########################################
    my $init = shift @_;

## begin packSimbadMenu  ################################################
    ## Eliminate the extra stuff (if not init)...
    if (!$init)
    {
	$simbadMenu->menu->delete('5', 'end'); ### this needs to be fixed?
    }

    my $nTargets = scalar @targets;
    my $nTargMenu = $tGuiAttributes::targMenuItems;  ## Max Items 
    if ($nTargets > $nTargMenu)
    {
	my ($count,@indices);
	$count = 0;
	while ($count < $nTargets)
	{
	    push @indices, $count;
	    $count += $nTargMenu;
	} ## end while
	push @indices, $nTargets;

	my ($i,$j);
	$count = 1;
	for ($i = 0; $i < (scalar @indices - 1); $i++)
	{
	    my $addlTargets = $simbadMenu->cascade(-label => "Target List $count");
	    for ($j = $indices[$i]; $j < $indices[$i + 1]; $j++)
	    {
		my $targTmp = $targets[$j];
		$addlTargets->command(-label => $targTmp,
				      -command => sub  ## query individual object
				      {
					  my $trgName = &simbadName($targTmp);
					  doSimbadGui($MW,$trgName);
				      });
	    } ## end j for loop
	    $count++;
	} ## end i for loop
    }
    else ## no need for sub-menus -- all in one single menu
    {
	foreach my $targTmp (@targets)
	{
	    $simbadMenu->command(-label => $targTmp,
				 -command => sub  ## query individual object
				      {
					  my $trgName = &simbadName($targTmp);
					  doSimbadGui($MW,$trgName);
				      });
	} ## end target menu packing/for loop
    } ## end else


} ## end packSimbadMenu  ################################################



#########################################################################
## abouttGui
#########################################################################
sub abouttGui
{
    ## abouttGui creates a splash screen with tGui information
## begin abouttGui  #####################################################
    my $aboutWindow = $MW->Toplevel();
    $aboutWindow->title("about tGui");

    ## Display logo image in a button
    my $logoFile = findFile("NExScI_logo_tiny.gif",$cfg->gcdocs . "/images", $cfg->gcdata . "/images");
    return if (!defined $logoFile);

    my $logo = $aboutWindow->Photo(-file => $logoFile);
    $aboutWindow->Button(-text => '',
			 -command => sub {$aboutWindow->withdraw},
			 -image => $logo)->pack(-side => "top", -fill => "x");
    $aboutWindow->Label(-text => "http://nexsci.caltech.edu")->pack(-side => "top", -fill => "x");
    $aboutWindow->Label(-text => "tGui v$REVISION ($PKGSTR)")->pack(-side => "top", -fill => "x");
    $aboutWindow->Label(-text => $tGuiAttributes::tGuiCopyright)->pack(-side => "top", -fill => "x");
    $aboutWindow->bind('<Return>' => sub { $aboutWindow->withdraw });
} ## end abouttGui  #####################################################


#########################################################################
## lstFromUT -- compute the LST at a given UT
#########################################################################
sub lstFromUT
{
## begin lstFromUT  #####################################################
    my $ut = shift @_;

    $ut -= 24.0 if ($ut >= 24.0);
    my ($lstH,$lstM,$lstS) = split /:/, $LSTzeroUT;
    my $lst = 15.0 * ($lstH + $lstM / 60.0 + $lstS / 3600.0
		      + (novasConstants::SolarToSidereal * $ut));
    if ($lst > 360.0) ## Fix wraps
    {
	$lst -= 360.0;
    }
    
    return $lst;

## end lstFromUT  #######################################################
}


#########################################################################
## localFromUT -- compute the local time at a given UT
#########################################################################
sub localFromUT
{
## begin localFromUT  ###################################################
    my $ut = shift @_;

    if (!(defined $LocalZeroUT))
    {
	return undef;
    }

    $ut -= 24.0 if ($ut >= 24.0);
    my ($locH,$locM,$locS) = split /:/, $LocalZeroUT;
    my $loc = 15.0 * ($locH + $locM / 60.0 + $locS / 3600.0 + $ut);

    if ($loc >= 360.0) ## Fix wraps
    {
	$loc -= 360.0;
    }

    return $loc;

## end localFromUT  #####################################################
}


#########################################################################
## fold360 - fold argument into the range  [0, 360]
#########################################################################
sub fold360
{
    my $val = shift;

    while ($val < 0)
    {
	$val += 360.0;
    }
    while ($val >=  360.0)
    {
	$val -= 360.0;
    }
    return $val;
}
