#! @PERL@
# -*- perl -*-
# @configure_input@

our $MODVERSION = '$Id: uvTool.pl,v 1.25 2008/11/12 23:28:13 sgroom Exp $';
our $MODREV = '$Revision: 1.25 $';

##########################################################################
# NASA Exoplanet Science Institute (NExScI) -- http://nexsci.caltech.edu #
#       Copyright (c) 1997-2008 California Institute of Technology       #
#                     U.S. Government Sponsorship                        #
# Under contract with the National Aeronautics and Space Administration  #
##########################################################################

use strict;
use warnings;
use FindBin qw($RealBin);
use lib "$RealBin/../lib/modules";		# location of getCal Perl modules
use Getopt::Long;
use Tk;
use Tk::ROText;
use Tk::FileSelect;
use Tk::Balloon;
use guiSubs;
use vector;
use geometry;
use coordinates;
use uvToolAttributes; ## Colors and such for the uvTool display
use novasConstants;
use findScript;
use baseline;
use gcConfig;
use miscArgs;
use interval;	## computing intersection of intervals

my @ra = split /\s+/, $MODREV; my $REVISION = $ra[1];
my $PKGSTR = package_string();

## uvTool -- part of the getCal suite

## uvTool is a Perl/Tk program to render the uv coverage for targets
## including zenith and delay constraints, based on input from the
## getCal timing module.

# Start by reading the timing report
my $cfg = gcConfig::config();
my ($timingDate, $yDay, $JDzeroUT, $LSTzeroUT, $LocalZeroUT,
    $sunRise, $sunSet, $moonRise, $moonSet, $moonIllumination);
$timingDate = $sunRise = $sunSet = $moonRise = $moonSet = "";
my ($input,$preambleText,$latitude);
my $maxBModulus = 0;
my $minBModulus = 1000;
my $lambda = 2.2; ## default to Kband
## Maximum and minimum spatial frequencies
my ($maxSPFreq,$minSPFreq,$RmaxSPFreq,$RminSPFreq);

## Command-line switch/default variables
my ($help,$version,$stdin,$PostScript,$pdf,$gif,$jpg,$png,$batchMode);
my ($filename);

my $intersectionTrack = 1;
my $zaTrack = 0;
my $delayTrack = 0;
my $obscuredTrack = 1;
my $haMarks = 1;
my $timingArgument = "-replace ";
my $defaultScale = 1.0e6;
my $scale;
my $anglesOn = 0;
my $utOn = 0;
my $targetID;

# keywords to highlight in text output windows
my $highlightKeys = "Timing summary|for timings|Using .* Location|Using .* Baseline|The JD at 0 hr UT is|The LST at 0 hr UT is|The Local Time at 0 hr UT is|Approximate UT sunset -- sunrise times|Approximate LST at sunset -- sunrise|Approximate UT moonrise -- moonset times|moon transit|% moon illumination -- |Using .* deg zenith angle constraint|Using .* deg zenith angle .* constraint|Using .* m delay constraint|Using .* delay interval|transits at .*, zenAng|is within zenith angle range from|is never within zenith angle range|is within delay range from|is always within delay range|is never within delay range|is obscured for";

## Adding command-line arguments to uvTool  A.B. 6/2003
handleCommonOpts(0,"uvTool.txt");
Getopt::Long::config("pass_through");
my $result = GetOptions("stdin" => \$stdin,
			"filename=s" => \$filename,
			"intersectionTrack:i" => \$intersectionTrack,
			"zenithTrack:i" => \$zaTrack,
			"delayTrack:i" => \$delayTrack,
			"hourAngle|haMarks:i" => \$haMarks,
			"anglesOn:i" => \$anglesOn,
			"currentTime|utOn" => \$utOn,
			"targetID=s" => \$targetID,
                        "wavelength=f" => \$lambda,
			"ps|postscript|PostScript|hc" => \$PostScript,
			"pdf" => \$pdf,
			"gif" => \$gif,
			"jpg" => \$jpg,
			"png" => \$png);
## TLE -- 7/8/04 added "wavelength" to command-line arguments


## Process some command-line things

if ((defined $PostScript) || (defined $pdf)
    || (defined $gif) || (defined $jpg) || (defined $png))
{
    $batchMode = 1;
} ## end setting batch mode




### uv track display canvas parameter definition (used globally)
my $xCanvasSize = $uvToolAttributes::XCanvasSize;
my $yCanvasSize = $uvToolAttributes::YCanvasSize;

## Main window declaration
my $MW = new MainWindow;
$MW->title("u-v Tool v$REVISION ($PKGSTR)");
my $balloon = $MW->Balloon();  ## help balloon object


# define the display canvas and frame
my $displayCanvasFrame = $MW->Frame(-relief => "raised",-borderwidth => "2");
my $displayCanvas = $displayCanvasFrame->Canvas(-width => $xCanvasSize,
				  -height => $yCanvasSize,
				  -relief => "groove",
				  -borderwidth => "2"
				  );
my $displayCanvasPacked = 0;


## Timing input variables
my (%transit, %leadZA, %trailZA, %newDelay, %target, %raStr, %decStr, %bSelect, %bHash);
my (@targets, @baseline);

## Reading input information
## Process initial input
if ($stdin) ## explicit stdin direction
{
    &readTimingRpt(0);
}
elsif ($filename)  ## If given a filename argument
{
    &readTimingRpt(1,$filename);
}
elsif ((scalar @ARGV) && !($stdin)) ## otherwise check command line
{
    &readTimingRpt(1,$ARGV[0]);  ## read from filename
}
else ## otherwise nothing left on command line, assume stdin
{
    $stdin = 1;
    &readTimingRpt(0);  ## read from stdin
}


## Scale variables
my ($xZero,$xScale,$yZero,$yScale);
## Should go away
my $maxY;

## Do batchMode/hardcopy as required
if (defined $batchMode)
{
    ## Call print routine
    printUVDiagram();

    exit(0);  ## we've generated the graphic -- exit
} ## end batchMode



## MW Contents
## At the top -- the menu frame
my $menuFrame = $MW->Frame(-relief => "raised",
			   -borderwidth => "2"
			   )->pack(-side => "top", -fill => "x");

# file menu
my $fileMenu = $menuFrame->Menubutton(-text => "File",
				      -menuitems => [## Recompute timings
						     [ 'command' => "Recompute Timings...",
						       -accelerator => 'Alt+r',
						       -command => sub {
							   my $cancel;
							   ($timingArgument,$cancel) =
							       &editTimingArgument($timingArgument, $MW->Toplevel());

							   if (!$cancel)
							   {
							       my $revisedInput
								   = `echo "$input" | timing -stdin $timingArgument`;
							       &readTimingRpt(2,$revisedInput);
							       &drawPreamble();
							       &packTargetMenu(0);
							       &packBlineMenu(0);
							       &packSimbadMenu(0);
							       &drawTarget($displayCanvas,$targetID,&baselineList());

							   } ## end if !$cancel
						       }],
						     "-",
						     ## Open new file
						     [ 'command' => "Open...",
						       -accelerator => 'Alt+o',
						       -command => sub {
							   my $file
							       = $MW->getOpenFile(-defaultextension => ".cat",
										  -filetypes        =>
										  [['catalog files',   '.cat' ],
										   ['Text Files',       ['.txt','.text']],
										   ['All Files',        '*',            ],
										   ],
										  -initialfile      => "getopenfile",
										  -title            => "Open schedule file",
										  );
							   if ($file)
							   {
							       &readTimingRpt($file,1);
							       &drawPreamble();
							       &packTargetMenu(0);
							       &packBlineMenu(0);
							       &packSimbadMenu(0);
							       &plotScales(); ## recompute the plot scales
							       &drawAxes($displayCanvas);  ## redraw the axes
							       &drawTarget($displayCanvas,$targetID,&baselineList());
							       &drawCurrentUT(\$displayCanvas); ## update current UT display (if any)
							   }}],
						     ## Open new file through timing
						     [ 'command' => "Open through timing...",
						       -accelerator => 'Alt+O',
						       -command => sub {
							   my $file
							       = $MW->getOpenFile(-defaultextension => ".cat",
										  -filetypes        =>
										  [['catalog files',   '.cat' ],
										   ['Text Files',       ['.txt','.text']],
										   ['All Files',        '*',            ],
										   ],
										  -initialfile      => "getopenfile",
										  -title            => "Open catalog file through timing",
										  );
							   if ($file)
							   {
							       my $cancel;
							       ($timingArgument,$cancel) =
								   &editTimingArgument($timingArgument, $MW->Toplevel());

							       if (!$cancel)
							       {
								   my $revisedInput
								       = `echo "$input" | timing -stdin $timingArgument`;
								   &readTimingRpt($revisedInput,2);
								   &drawPreamble();
								   &packTargetMenu(0);
								   &packBlineMenu(0);
								   &packSimbadMenu(0);
								   &plotScales(); ## recompute the plot scales
								   &drawAxes($displayCanvas);  ## redraw the axes
								   &drawTarget($displayCanvas,$targetID,&baselineList());
								   &drawCurrentUT(\$displayCanvas); ## update current UT display (if any)
							       } ## end !$cancel
							   }}],
						     "-",
						     [ 'command' => "View Input",
						       -accelerator => 'Alt+v',
						       -command => sub {&dispText($MW,$input,"uvTool input",$highlightKeys)}],
						     [ 'command' => "tGui",
						       -accelerator => 'Alt+t',
						       -command => sub
						       { system "echo \"$input\" | tGui &";}],
						     "-",
						     [ 'command' => "XEphem Catalog",
						       -accelerator => 'Alt+x',
						       -command => sub {&xephemOutput($MW, $input)}],
						     [ 'command' => "XEphem FIFO",
						       -accelerator => 'Alt+X',
						       -command => sub {&xephemOutput($MW, $input,"fifo")}],
						     "-",
						     [ 'command' => "Print...",
						       -accelerator => 'Alt+p',
						       -command => sub {&printUVDiagram()}],
						     "-",
						     [ 'command' => "Quit",
						       -accelerator => 'Alt+q',
						       -command => sub{ $MW->withdraw; exit(0);} ] ]
				      )->pack(-side => "left", -anchor => "w");
$balloon->attach($fileMenu,-msg => "File menu -- options for input and output");

## keyboard bindings for file menu
$MW->bind('<Alt-r>' => sub {  ## recompute timings
    my $cancel;
    ($timingArgument,$cancel) =
	&editTimingArgument($timingArgument, $MW->Toplevel());
    
    if (!$cancel)
    {
	my $revisedInput
	    = `echo "$input" | timing -stdin $timingArgument`;
	&readTimingRpt(2,$revisedInput);
	&drawPreamble();
	&packTargetMenu(0);
	&packBlineMenu(0);
	&packSimbadMenu(0);
	my @bList = &baselineList();
	&drawTarget($displayCanvas,$targetID,@bList);
	
    } ## end if !$cancel
});
$MW->bind('<Alt-o>' => sub  ## Open new file
	  {
	      my $file
		  = $MW->getOpenFile(-defaultextension => ".cat",
				     -filetypes        =>
				     [['catalog files',   '.cat' ],
				      ['Text Files',       ['.txt','.text']],
				      ['All Files',        '*',            ],
				      ],
				     -initialfile      => "getopenfile",
				     -title            => "Open schedule file",
				     );
	      if ($file)
	      {
		  &readTimingRpt($file);
		  &drawPreamble();
		  &packTargetMenu(0);
		  &packBlineMenu(0);
		  &packSimbadMenu(0);
		  &plotScales(); ## recompute the plot scales
		  &drawAxes($displayCanvas);  ## redraw the axes
		  my @bList = &baselineList();
		  &drawTarget($displayCanvas,$targetID,@bList);
		  &drawCurrentUT(\$displayCanvas); ## update current UT display (if any)
	      }
	  });  ## end open new file keyboard accelerator


$MW->bind('<Alt-O>' => sub  ## Open new file through timing
	  {
	      my $file
		  = $MW->getOpenFile(-defaultextension => ".cat",
				     -filetypes        =>
				     [['catalog files',   '.cat' ],
				      ['Text Files',       ['.txt','.text']],
				      ['All Files',        '*',            ],
				      ],
				     -initialfile      => "getopenfile",
				     -title            => "Open catalog file through timing",
				     );
	      if ($file)
	      {
		  my $cancel;
		  ($timingArgument,$cancel) =
		      &editTimingArgument($timingArgument, $MW->Toplevel());

		  if (!$cancel)
		  {
		      my $revisedInput
			  = `echo "$input" | timing -stdin $timingArgument`;
		      &readTimingRpt($revisedInput,2);
		      &drawPreamble();
		      &packTargetMenu(0);
		      &packBlineMenu(0);
		      &packSimbadMenu(0);
		      &plotScales(); ## recompute the plot scales
		      &drawAxes($displayCanvas);  ## redraw the axes
		      &drawTarget($displayCanvas,$targetID,&baselineList());
		      &drawCurrentUT(\$displayCanvas); ## update current UT display (if any)
		  } ## end !$cancel
	      } ## end if $file
	      });


$MW->bind('<Alt-v>' => sub {&dispText($MW,$input,"uvTool input",$highlightKeys)});
$MW->bind('<Alt-t>' => sub { system "echo \"$input\" | tGui &";});
$MW->bind('<Alt-x>' => sub {&xephemOutput($MW, $input)});
$MW->bind('<Alt-X>' => sub {&xephemOutput($MW, $input,"fifo")});
$MW->bind('<Alt-p>' => sub {&printUVDiagram()});
$MW->bind('<Alt-q>' => sub {$MW->withdraw; exit(0); });


## target menu

my $targetMenu = $menuFrame->Menubutton(-text => "Target")->pack(-side => "left", -anchor => "w");
$balloon->attach($targetMenu,-msg => "Target menu -- options for objects");

## Pack the target menu with selections for individual targets (now in
## subroutine)
&packTargetMenu(1);



## baseline menu
my $baselineMenu = $menuFrame->Menubutton(-text => "Baseline",
					  -menuitems => [['command' => "All",
							  -accelerator => 'Alt+a',
							  -command => sub
							  {
							      foreach my $blTmp (@baseline)
							      {
								  $bSelect{$blTmp} = 1;
							      }
							      my @bList = &baselineList();
							      &drawTarget($displayCanvas,$targetID,@bList);
							      &drawCurrentUT(\$displayCanvas); ## update current UT display (if any)
							  }],
							 ['command' => "Invert",
							  -accelerator => 'Alt+i',
							  -command => sub
							  {
							      foreach my $blTmp (@baseline)
							      {
								  $bSelect{$blTmp} = !($bSelect{$blTmp});
							      }
							      my @bList = &baselineList();
							      &drawTarget($displayCanvas,$targetID,@bList);
							      &drawCurrentUT(\$displayCanvas); ## update current UT display (if any)
							  }],
							 "-"]
					  )->pack(-side => "left", -anchor => "w");
$balloon->attach($baselineMenu,-msg => "Baseline menu -- options for baselines");

## keyboard bindings for baseline menu
$MW->bind('<Alt-a>' => sub
	  {
	      foreach my $blTmp (@baseline)
	      {
		  $bSelect{$blTmp} = 1;
	      }
	      my @bList = &baselineList();
	      &drawTarget($displayCanvas,$targetID,@bList);
	      &drawCurrentUT(\$displayCanvas); ## update current UT display (if any)
$MW
	  });
$MW->bind('<Alt-i>' => sub
	  {
	      foreach my $blTmp (@baseline)
	      {
		  $bSelect{$blTmp} = !($bSelect{$blTmp});
	      }
	      my @bList = &baselineList();
	      &drawTarget($displayCanvas,$targetID,@bList);
	      &drawCurrentUT(\$displayCanvas); ## update current UT display (if any)
$MW
	  });

## Make an entry in the baseline menu for each baseline designation in
## the input file
$bSelect{$baseline[0]} = 1;  ## Select the first baseline by default
## Pack the baseline menu with individual baseline selections (now in
## a subroutine).
&packBlineMenu(1);


## Simbad query menu
my $simbadMenu = $menuFrame->Menubutton(-text => "Simbad",
					-menuitems => [['command' => "Targets",
							##-accelerator => 'Alt-t',
							-command => sub
							{
							    ## Simbad target menu entry -- query all targets
							    my @targetSet = ();
							    foreach my $trg (@targets)
							    {
								if ($target{$trg})
								{
								    push @targetSet,&simbadName($trg);
								}
							    }
							    if (scalar @targetSet)
							    {
								doSimbadGui($MW,@targetSet);
							    }
							}],
						       ['command' => "Calibrators",
							##-accelerator => 'Alt+c',
							-command => sub
							{
							    ## Simbad calibrator menu entry -- query all calibrators
							    my @targetSet = ();
							    foreach my $trg (@targets)
							    {
								if (!($target{$trg}))
								{
								    push @targetSet,&simbadName($trg);
								}
							    }
							    if (scalar @targetSet)
							    {
								doSimbadGui($MW,@targetSet);
							    }
							}],
						       ['command' => "All",
							##-accelerator => 'Alt+a',
							-command => sub
							{
							    ## Simbad all menu entry -- query all
							    my @targetSet = ();
							    foreach my $trg (@targets)
							    {
								push @targetSet,&simbadName($trg);
							    }
							    if (scalar @targetSet)
							    {
								doSimbadGui($MW,@targetSet);
							    }
							}],
						       "-"]
					)->pack(-side => "left", -anchor => "w");
$balloon->attach($simbadMenu,-msg => "Simbad menu -- Simbad query options");

## Use subroutine to pack individual objects in the simbad menu
&packSimbadMenu(1);




# help menu
# help files
my $uvHelp = findFile("uvTool.txt",$cfg->gcdocs, $cfg->gcdata);
my $timingHelp = findFile("timing.txt",$cfg->gcdocs, $cfg->gcdata);
my $versionTxt = findFile("version.txt",$cfg->gcdocs, $cfg->gcdata);
my $copyright = findFile("copyright.txt",$cfg->gcdocs, $cfg->gcdata);

my $helpMenu = $menuFrame->Menubutton(-text => "Help",
				      -menuitems => [ [ 'command' => "uvTool help",
							-accelerator => 'Alt+h',
							-command => sub {&dispHelp($uvHelp)} ],
						      [ 'command' => "about uvTool",
							-command => sub {&aboutuvTool()} ],
						      "-",
						      [ 'command' => "timing help",
							-command => sub {&dispHelp($timingHelp)}  ],
						      [ 'command' => "getCal version",
							-command => sub {&dispHelp($versionTxt)}  ],
						      [ 'command' => "getCal copyright",
							-command => sub {&dispHelp($copyright)}  ] ]
				      )->pack(-side => "right", -anchor => "w");
$balloon->attach($helpMenu,-msg => "Help menu -- available documentation options");

$MW->bind('<Alt-h>' => sub {&dispHelp($uvHelp)});


## Next a control frame that allows the user to define units and
## wavelength scale
my $scaleFrame = $MW->Frame(-relief => "raised",
			    -borderwidth => "2"
			    )->pack(-side => "top", -fill => "x");

## First an optionmenu selector to draw the axes in angle or Mlambda
## units
$scaleFrame->Label(-text => "  Plot scale ")->pack(-side => "left", -anchor => "w");
my $scaleMenu = $scaleFrame->Optionmenu(-options => [["Mlambda","1e6"], ["Klambda","1e3"]],
					-command => sub 
					{
					    $scale = shift;
					    if ($displayCanvasPacked)
					    {
						&plotScales(); ## recompute the plot scales
						&drawAxes($displayCanvas);  ## redraw the axes
						&drawTarget($displayCanvas,$targetID,&baselineList());  ## redraw the tracks
						&drawCurrentUT(\$displayCanvas); ## redraw the current information (if selected)
						&drawAngleCircles($displayCanvas); ## redraw fringe spacing indocators (if selected)
					    } ## end checking $displayCanvas
					},
					-variable => $scale
					)->pack(-side => "left");
$balloon->attach($scaleMenu,-msg => "Set display scale");


## Next a wavelength entry box
## Default display values
$scaleFrame->Label(-text => "  wavelength (microns) ")->pack(-side => "left", -anchor => "w");
my $lambdaEntry = $scaleFrame->Entry(-textvariable => \$lambda,
				     -width => 8,
				     -relief => 'raised')->pack(-side => "left");
$balloon->attach($lambdaEntry,-msg => "Wavelength entry (microns)");

## Return in the wl entry box rescales the plot (equivalent to rescale button)
$lambdaEntry->bind('<Return>' => sub
		   {
		       &plotScales(); ## recompute the plot scales
		       &drawAxes($displayCanvas);  ## redraw the axes
		       &drawTarget($displayCanvas,$targetID,&baselineList());  ## redraw the tracks
		       &drawCurrentUT(\$displayCanvas); ## redraw the current information (if selected)
		       &drawAngleCircles($displayCanvas); ## redraw fringe spacing indocators (if selected)
		   });

## Finally the rescale command
my $b1 =
    $scaleFrame->Button(-text => 'Rescale',
			-command => sub
			{
			    &plotScales(); ## recompute the plot scales
			    &drawAxes($displayCanvas);  ## redraw the axes
			    &drawTarget($displayCanvas,$targetID,&baselineList());  ## redraw the tracks
			    &drawCurrentUT(\$displayCanvas); ## redraw the current information (if selected)
			    &drawAngleCircles($displayCanvas); ## redraw fringe spacing indocators (if selected)
			}
			)->pack(-side => "left", -anchor => "w");
$balloon->attach($b1,-msg => "Rescale u-v plot");


## Next a text frame with informational comments about the timing
# summary
my $msgFrame = $MW->Frame(-relief => 'groove',
			  -borderwidth => "2"
			  )->pack(-side => "top", -fill => "x");

my $preamble = $msgFrame->Scrolled("ROText",
				   -wrap => "none",
				   -width => 80, -height => 8,
				   -relief => "groove"
				   )->pack(-side => "left",
					   -expand => "yes",
					   -fill => "x");
$balloon->attach($preamble,-msg => "Timing preamble");

$preamble->insert('end',$preambleText);


## A time frame to hold current time displays
my $tFrame = $MW->Frame(-relief => 'groove',
			     -borderwidth => "1",
			     )->pack(-side => "top",-fill => "x");

## Logic for current time displays
my ($currentUT,$currentLST,$currentLocal);
&setCurrentTime();
$tFrame->Label(-text => "Current Time ")->pack(-side => "left",-anchor => "w");
$tFrame->Label(-textvariable => \$currentUT,
	       -relief => 'raised',
	       -width => 10
	       )->pack(-side => "left", -anchor => "w");

$tFrame->Label(-textvariable => \$currentLST,
	       -relief => 'raised',
	       -width => 10
	       )->pack(-side => "left", -anchor => "w");

$tFrame->Label(-textvariable => \$currentLocal,
	       -relief => 'raised',
	       -width => 10
	       )->pack(-side => "left", -anchor => "w");
$balloon->attach($tFrame,-msg => "Current time display");


## Pack the canvas frame and $displayCanvas
$displayCanvasFrame->pack(-side => "top",-expand => "yes",-fill => "both");
$displayCanvas->pack(-side => "left",-expand => "yes",-fill => "both");
$balloon->attach($displayCanvas,-msg => "u-v display canvas");
$displayCanvasPacked = 1;



### mini-canvas with legends
my $xCanvasSize2 = int ($xCanvasSize / 4.0);
my $legendCanvas = $displayCanvasFrame->Canvas(-width => $xCanvasSize2,
					-height => $yCanvasSize,
					-relief => "groove",
					-borderwidth => "2"
					)->pack(-side => "right");

my $xPos = int (0.1 * $xCanvasSize2);
my $yPos = int (0.05 * $yCanvasSize);
$legendCanvas->createText($xPos,$yPos,-anchor => "w", -text => "Legend:");

$yPos = int (0.15 * $yCanvasSize);
my $offset = 20;
my $length = int ($xCanvasSize2 / 2.0);

$legendCanvas->createText($xPos,$yPos,-anchor => "w",-text => "Intersection Range");
$legendCanvas->createLine($xPos + $offset,$yPos + $offset,
			  $xPos + $offset + $length,$yPos + $offset,
			  -width => 3,
			  -fill => $uvToolAttributes::intersectionColor);

$yPos += int (0.15 * $yCanvasSize);
$legendCanvas->createText($xPos,$yPos,-anchor => "w",-text => "Delay Range");
$legendCanvas->createLine($xPos + $offset,$yPos + $offset,
			  $xPos + $offset + $length,$yPos + $offset,
			  -width => 2,
			  -fill => $uvToolAttributes::delayColor);


$yPos += int (0.15 * $yCanvasSize);
$legendCanvas->createText($xPos,$yPos,-anchor => "w",-text => "Zenith Range");
$legendCanvas->createLine($xPos + $offset,$yPos + $offset,
			  $xPos + $offset + $length,$yPos + $offset,
			  -width => 5,
			  -fill => $uvToolAttributes::zaColor);

$yPos += int (0.15 * $yCanvasSize);
$legendCanvas->createText($xPos,$yPos,-anchor => "w",-text => "Transit/HA");
my @tCenter = ($xPos + $offset + $length / 2,$yPos + $offset);
my $tSize = 6;
$legendCanvas->createOval($tCenter[0] - $tSize,$tCenter[1] - $tSize,
			  $tCenter[0] + $tSize,$tCenter[1] + $tSize,
			  -fill => $uvToolAttributes::transitColor,
			  -outline => $uvToolAttributes::transitColor);

$yPos += int (0.15 * $yCanvasSize);
@tCenter = ($xPos + $offset + $length / 2,$yPos + $offset);
$legendCanvas->createText($xPos,$yPos,-anchor => "w",-text => "Current u-v");

## Now drawing a single arrow -- A.B. 7/2001
$legendCanvas->createLine($tCenter[0] + $tSize,$tCenter[1] - $tSize,
			  $tCenter[0] - $tSize,$tCenter[1] + $tSize,
			  -width => 2,
			  -arrow => "first",
			  -fill => $uvToolAttributes::currentUTColor);

## Finally a micro-logo for institutional awarness purposes
$yPos += int (0.2 * $yCanvasSize);
$xPos = int (0.5 * $xCanvasSize2);
my $microLogo = findFile("NExScI_logo_micro.gif",$cfg->gcdocs . "/images", $cfg->gcdata . "/images");
if (defined $microLogo)
{
    my $logo = $legendCanvas->Photo( 'IMG', -file => $microLogo);
    $legendCanvas->create('image', $xPos, $yPos,
			  '-image'  => $logo );
} ## end logo on the legend



# Add a couple of control frames at the bottom
my $ctrlFrame = $MW->Frame(-relief => 'groove',
			    -borderwidth => "2"
			    )->pack(-side => "bottom", -fill => "x");

my $ctrlFrame1 = $ctrlFrame->Frame(-relief => 'groove',
				    -borderwidth => "1"
				    )->pack(-side => "top", -fill => "x");

my $iButton =
    $ctrlFrame1->Button(-text => 'Intersection Track',
			-command => sub
			{
			    $intersectionTrack = (!$intersectionTrack);
			    if ($intersectionTrack)
			    {			    
				$zaTrack = $delayTrack = 0;
			    }
			    else
			    {
				$zaTrack = $delayTrack = 1;
			    }
			    &drawTarget($displayCanvas,$targetID,&baselineList());  ## redraw the tracks
			}
			)->pack(-side => "left", -anchor => "w");
$balloon->attach($iButton,-msg => "Toggle intesection track rendering <Cntrl+i>");

$MW->bind('<Control-i>' => sub
	  {
	      $intersectionTrack = (!$intersectionTrack);
	      if ($intersectionTrack)
	      {			    
		  $zaTrack = $delayTrack = 0;
	      }
	      else
	      {
		  $zaTrack = $delayTrack = 1;
	      }
	      &drawTarget($displayCanvas,$targetID,&baselineList());  ## redraw the tracks
	  });


my $zButton =
    $ctrlFrame1->Button(-text => 'ZA Track',
			-command => sub
			{
			    $intersectionTrack = 0;
			    $zaTrack = (!$zaTrack);
			    &drawTarget($displayCanvas,$targetID,&baselineList());  ## redraw the tracks
			}
			)->pack(-side => "left", -anchor => "w");
$balloon->attach($zButton,-msg => "Toggle zenith angle track rendering <Cntrl+z>");
$MW->bind('<Control-z>' => sub
	  {
	      $intersectionTrack = 0;
	      $zaTrack = (!$zaTrack);
	      &drawTarget($displayCanvas,$targetID,&baselineList());  ## redraw the tracks
	  });


my $dButton = 
    $ctrlFrame1->Button(-text => 'Delay Track',
			-command => sub
			{
			    $intersectionTrack = 0;
			    $delayTrack = (!$delayTrack);
			    &drawTarget($displayCanvas,$targetID,&baselineList());  ## redraw the tracks
			}
			)->pack(-side => "left", -anchor => "w");
$balloon->attach($dButton,-msg => "Toggle delay track rendering <Cntrl+d>");

$MW->bind('<Control-d>' => sub
	  {
	      $intersectionTrack = 0;
	      $delayTrack = (!$delayTrack);
	      &drawTarget($displayCanvas,$targetID,&baselineList());  ## redraw the tracks
	  });


## Controls for ghost track
my $ghostTrackOn = 0;

my $gButton = 
    $ctrlFrame1->Button(-text => 'Ghost Track',
			-command => sub
			{
			    $ghostTrackOn = (!$ghostTrackOn);
			    &drawTarget($displayCanvas,$targetID,&baselineList());  ## redraw the tracks
			}
			)->pack(-side => "left", -anchor => "w");
$balloon->attach($gButton,-msg => "Toggle ghost track (virtual u-v ellipse) rendering <Cntrl+g>");

$MW->bind('<Control-g>' => sub
	  {
	      $ghostTrackOn = (!$ghostTrackOn);
	      &drawTarget($displayCanvas,$targetID,&baselineList());  ## redraw the tracks
	  });


## Simbad query button
my $sButton =
    $ctrlFrame1->Button(-text => 'Simbad',
			-command => sub
			{
			    my $trgName = &simbadName($targetID);
			    doSimbadGui($MW,$trgName);
			}
			)->pack(-side => "left", -anchor => "w");
$balloon->attach($sButton,-msg => "Simbad query on current object");

## control frame2 (UT/LST, HA, FringeSpacing, Print, Quit)
my $ctrlFrame2 = $ctrlFrame->Frame(-relief => 'groove',
			    -borderwidth => "1"
			    )->pack(-side => "bottom", -fill => "x");

my $cButton =
    $ctrlFrame2->Button(-text => 'Close',
			-command => sub{exit}
			)->pack(-side => "right", -anchor => "w");
$balloon->attach($cButton,-msg => "Vaya con Dios <Alt+q>");

my $pButton =
    $ctrlFrame2->Button(-text => 'Print',
			-command => sub
			{
			    printUVDiagram();
			}
			)->pack(-side => "right", -anchor => "w");
$balloon->attach($pButton,-msg => "Print current rendering <Alt+p>");

## Controls for current UT/LST 
my $uButton =
    $ctrlFrame2->Button(-text => 'Current UT/LST',
			-command => sub
			{
			    $utOn = (!$utOn);
			    &drawCurrentUT(\$displayCanvas); ##($targetID,@bList);
			}
			)->pack(-side => "left", -anchor => "w");
$balloon->attach($uButton,-msg => "Toggle current UTC/LST rendering <Cntrl+u>");

$MW->bind('<Control-u>' => sub
	  {
	      $utOn = (!$utOn);
	      &drawCurrentUT(\$displayCanvas); ##($targetID,@bList);
	  });


## Controls for ha marks
my $hButton =
    $ctrlFrame2->Button(-text => 'Hour Angle',
			-command => sub
			{
			    ##$haMarksOn = (!$haMarksOn);
			    $haMarks++;
			    if ($haMarks == 3)
			    {
				$haMarks = 0;
			    }

			    &drawTarget($displayCanvas,$targetID,&baselineList());  ## redraw the tracks
			}
			)->pack(-side => "left", -anchor => "w");
$balloon->attach($hButton,-msg => "Toggle rendering of hour angle markers <Cntrl+h>");

$MW->bind('<Control-h>' => sub
	  {
	      ##$haMarksOn = (!$haMarksOn);
	      $haMarks++;
	      if ($haMarks == 3)
	      {
		  $haMarks = 0;
	      }

	      &drawTarget($displayCanvas,$targetID,&baselineList());  ## redraw the tracks
	  });


## Controls for current fringe spacing indicators
my $fButton =
    $ctrlFrame2->Button(-text => 'Fringe Spacing',
			-command => sub
			{
			    ##$anglesOn = (!$anglesOn);
			    $anglesOn++;
			    if ($anglesOn == 4)  ## 4-phase display modality
			    {
				$anglesOn = 0;
			    }

			    &drawAngleCircles($displayCanvas);
			}
			)->pack(-side => "left", -anchor => "w");
$balloon->attach($fButton,-msg => "Toggle fringe spacing rendering <Cntrl+f>");

$MW->bind('<Control-f>' => sub
	  {
	      $anglesOn++;
	      if ($anglesOn == 4)  ## 4-phase display modality
	      {
		  $anglesOn = 0;
	      }
	      &drawAngleCircles($displayCanvas);
	  });




### end control frame 2

## Draw the initial content
&plotScales();  ## define scales
&drawAxes($displayCanvas);  ## draw axes
&drawTarget($displayCanvas,$targetID,&baselineList());  ## u-v plot
&drawCurrentUT(\$displayCanvas); ## update current UT display (if any)
&drawAngleCircles($displayCanvas); ## redraw fringe spacing indocators (if selected)

MainLoop;  ## Perl/TK main event loop

exit(0);  ## end uvTool main
## end uvTool  ##########################################################



#########################################################################
## plotScales -- compute plot scales
#########################################################################
sub plotScales
{

## begin plotScales  ####################################################
    ## the subroutine plotScales defines the plot scales we will be
    ## using in the rendering of the uv tracks

    ## Start by redefining the maximum and minimum spatial frequencies
    ## (in case $lambda has changed)
    $maxSPFreq = $maxBModulus / ($lambda * 1.0e-6);
    $minSPFreq = $minBModulus / ($lambda * 1.0e-6);

    ## $maxSPFreq and $minSPFreq are the extremum spatial frequencies
    ## probed by the input baselines/wavelength.  We need to round
    ## them UP/DOWN respectively to the nearest whole-looking number
    ## to make the plots more reasonable
    $RmaxSPFreq = &round($maxSPFreq,+1.0);
    $RminSPFreq = &round($minSPFreq,0.0);

    if (!$scale)
    {
	## Default scale value -- Mlambda
	$scale = $defaultScale;
    }

    ## $xScale is negative because RA increases left (we think)
    $xScale = - ($xCanvasSize / (1.3 * 2.0 * $RmaxSPFreq));
    $xZero = int ($xCanvasSize / 2.0);

    $yScale = $xScale;  ## $yScale is negative because screen coordinates increase down
    $yZero = int ($yCanvasSize / 2.0);

} ## end plotScales  ####################################################



#########################################################################
## drawAxes -- draw u and v axes
#########################################################################
sub drawAxes
{
    ## drawAxes draws the axes under a scale or unit change

## declarations drawAxes  ###############################################
    my $canvas = shift @_;  ## canvas argument

## begin drawAxes  ######################################################
    ## Eliminate old axes
    $canvas->delete("Axes");

    ### Draw axis lines depending on $scale

    # x
    my $x;
    my $maxX = $RmaxSPFreq;
    my $minX = - $maxX;
    $canvas->createText(xCoord(1.1 * $RmaxSPFreq),yCoord(0),
			-text => "u",-tags => "Axes");  ## Denoting u axis
    for ($x = $minX; $x <= $maxX; $x += (2 * $RmaxSPFreq))
    {
	my $tmp = sprintf "%3d", int($x / $scale);
	$canvas->createText(xCoord($tmp * $scale),yCoord(0) + 8,
			    -text => $tmp,-tags => "Axes");
    } ## end x loop
    $canvas->createLine(xCoord($minX),yCoord(0),xCoord($maxX),yCoord(0),
			-tags => "Axes"); # x axis

    # y
    my $y;
    my $maxY = $RmaxSPFreq;
    my $minY = - $maxY;
    $canvas->createText(xCoord(0),yCoord(1.1 * $RmaxSPFreq),
			-text => "v",-tags => "Axes");  ## Denoting v axis
    for ($y = $minY; $y <= $maxY; $y += (2.0 * $RmaxSPFreq))
    {
	my $tmp = sprintf "%3d", int ($y / $scale);
	$canvas->createText(xCoord(0) + 12,yCoord($tmp * $scale),
			    -text => $tmp,-tags => "Axes");
    }
    $canvas->createLine(xCoord(0),yCoord($minY),xCoord(0),yCoord($maxY),
			-tags => "Axes"); # y axis

    ## Draw E-N indicator lines (added AB 7/2003)
    $canvas->createLine(xCoord($maxX),yCoord($maxY),xCoord($maxX * 1.2),yCoord($maxY),
			-arrow => "last",-tags => "Axes"); # East indicator
    $canvas->createText(xCoord($maxX * 1.2),yCoord($maxY) + 8,
			    -text => "E",-tags => "Axes");

    $canvas->createLine(xCoord($maxX),yCoord($maxY),xCoord($maxX),yCoord($maxY * 1.2),
			-arrow => "last",-tags => "Axes"); # North indicator
    $canvas->createText(xCoord($maxX) + 8,yCoord($maxY * 1.2) + 8,
			    -text => "N",-tags => "Axes");

} ## end drawAxes  ######################################################



#########################################################################
## xCoord 
#########################################################################
sub xCoord
{

    my $arg = shift @_;

    my $xRet = int($xZero + $arg * $xScale);

    return $xRet;
} ## end xCoord  ########################################################


#########################################################################
## yCoord
#########################################################################
sub yCoord
{

    my $arg = shift @_;

    my $yRet = int($yZero + $arg * $yScale);

    return $yRet;
} ## end yCoord  ########################################################


#########################################################################
## readTimingRpt -- name says it all
#########################################################################
sub readTimingRpt
{

## declarations readTimingRpt  ##########################################
    my ($modeSwitch,$arg) = @_;   ## extract call arguments
    my @inputStream;

## begin readTimingRpt  #################################################
    ## Branch on the mode switch used to call readTimingRpt
    if ($modeSwitch == 0) ## mode == 0 -- read from stdin
    {
	@inputStream = <STDIN>;
    } ## end reading from stdin
    elsif (($modeSwitch == 1) && ($arg))  ## mode == 1 -- read from filename argument
    {
	open (READ,$arg) or die "Unable to open $arg\n";
	@inputStream = <READ>;
    } ## end reading from file
    elsif (($modeSwitch == 2) && ($arg))  ## mode == 2 -- read from the input stream
    {
	@inputStream = split /\n/, $arg;
	foreach my $tmp (@inputStream)  ## need to add the linebreaks back in...
	{
	    $tmp .= "\n";
	}
    } ## end reading from call argument


    ## Reading the syntax of the timing report as composed by the
    ## timing program -- now contained in @inputStream

    ## Initialize variables that contain timing information
    $timingDate = $yDay = $JDzeroUT = $LSTzeroUT = $LocalZeroUT
	= $sunRise = $sunSet = $moonRise = $moonSet = $moonIllumination = "";
    $input = $preambleText = $latitude = "";
    $maxBModulus = 0;
    $minBModulus = 1000;
    ## TLE -- 7/8/04 do NOT reset lambda
    ##$lambda = 2.2; ## default to Kband

    %transit =  %leadZA = %trailZA = %newDelay
	= %target = %raStr = %decStr = %bSelect = %bHash = ();
    @targets = @baseline = ();

    ## Reset timing argument
    ## (Now building custom timing argument based on input timings)
    $timingArgument = "-replace ";

    my $line;

    ##open (TEST,"testCases/multiBaseline.timing");  ## used for testing
    ##open (TEST,"testCases/23july00.cat");  ## used for testing
    ##open (TEST,"testCases/NPOI.cat");  ## used for testing

    foreach my $line (@inputStream)
    {
	$input .= $line;  ## accumulate input for display/later access

	if ($line =~ m/^\# Timing summary/)
	{
	    $preambleText .= $line;
	}

	if ($line =~ m/^\#  for timings on/)
	{
	    my @tmp = split /\s+/, $line;
	    $timingDate = $tmp[4];
	    $yDay = $tmp[6];
	    $preambleText .= $line;
	    my @tmp2 = split /\//, $timingDate;
	    $timingArgument .= "-month $tmp2[0] -day $tmp2[1] -year $tmp2[2] ";
	}

	if ($line =~ m/^\# The JD at 0 hr UT is/)
	{
	    my @tmp = split /\s+/, $line;
	    $JDzeroUT = $tmp[8];
	    $preambleText .= $line;
	}

	if ($line =~ m/^\# The LST at 0 hr UT is/)
	{
	    my @tmp = split /\s+/, $line;
	    $LSTzeroUT = $tmp[8];
	    $preambleText .= $line;
	} ## end reading LST

	if ($line =~ m/^\# The Local Time at 0 hr UT is/)
	{
	    my @tmp = split /\s+/, $line;
	    $LocalZeroUT = $tmp[9];
	    $preambleText .= $line;
	} ## end reading Local

	if ($line =~ m/^\# Approximate UT sunset -- sunrise times/)
	{
	    my @tmp = split /\s+/, $line;
	    $sunSet = $tmp[7];
	    $sunRise = $tmp[9];
	    $preambleText .= $line;
	} ## end reading sunrise/sunset

        # Approximate LST at sunset -- sunrise: 05:53:05 -- 16:28:14
	if ($line =~ m/^\# Approximate LST at sunset -- sunrise/)
	{
	    my @tmp = split /\s+/, $line;
	    $preambleText .= $line;
	} ## end reading sunrise/sunset

        # Approximate UT moonrise -- moonset times: 10:12:45 -- 20:12:17
	if ($line =~ m/^\# Approximate UT moonrise -- moonset times/)
	{
	    my @tmp = split /\s+/, $line;
	    $moonRise = $tmp[7];
	    $moonSet = $tmp[9];
	    $preambleText .= $line;
	} ## end reading moonrise/moonset


#   94% moon illumination -- 2.2 days until full moon (moon at 04:26 RA  +19:46 Dec)
	if ($line =~ m/^\#\s+\d+\% moon illumination -- /)
	{
	    my @tmp = split /\s+/, $line;
	    $preambleText .= $line;

	    if ($tmp[1] =~ m/\d+/)
	    {
		$moonIllumination = $tmp[1];
	    }

	} ## end reading moon illimination


        # Using Palomar location (long,lat: -116.8633, 33.3567)
        # Using Palomar Location (long: -116:51:48   lat: +33:21:24)
	if ($line =~ m/^\# Using (.*) Location/)
	{
	    $preambleText .= $line;
	    my $trg = "-location $1"; ## use captured search target
	    if (!($timingArgument =~ m/$trg/))  ## avoid duplications
	    {
		$timingArgument .= "$trg ";
	    }
	    chomp($line);
	    my @tmp = split /\s+/, $line;
	    $latitude = $tmp[(scalar @tmp) - 1];
	    $latitude =~ s/\)//g;
	    if ($latitude =~ m/:/)
	    {
		$latitude = &DECdeg($latitude);
	    }
	} ## end reading location string

        # Using PTI_NS Baseline (ENUBias: -37.116352, -103.264746, 3.319338, -12.915245)
	if ($line =~ m/^\# Using (.*) Baseline/)
	{
	    $preambleText .= $line;
	    my $bName = $1;  ## use the captured search target
	    my @tmp = split /\s+/, $line;
	    if (!(grep /$bName/, @baseline))  ## check for redundancy
	    {
		push @baseline, $bName;
		$bSelect{$bName} = 0;
		$timingArgument .= "-baseline $bName ";
		if ($line =~ m/\(ENUBias: (.*)\)/)
		{
		    my $list = $1;
		    $list =~ s/,//g;  ## get rid of any commas that might be there
		    my @b = split /\s+/, $list;
		    $bHash{$bName} = new baseline(name => $bName,
						  latitude => $latitude,
						  bVector => \@b,
						  delayBias => $b[3]);
		    ## Collect extrema in baseline lengths for uv plot scaling
		    my $bModulus = &modulus(@b);
		    $maxBModulus = $bModulus if ($bModulus > $maxBModulus);
		    $minBModulus = $bModulus if ($bModulus < $minBModulus);
		} ## end creating baseline object
	    } ## end grep/redundancy check
	} ## end reading baseline string


	if ($line =~ m/zenith angle constraint|zenith angle (.+) constraint|delay constraint|delay interval/)
	{
	    $preambleText .= $line;  ## add to preamble
	    if ($line =~ m/^\# Using (.*) deg zenith angle/)
	    {
		$timingArgument .= "-zenith $1 ";  ## used captured search target
	    }
	    if ($line =~ m/^\# Using (.*) m delay constraint/)
	    {
		$timingArgument .= "-delayTarget $1 ";  ## used captured search target
	    }
	    if ($line =~ m/^\# Using (.*) to (.*) m delay interval/)
	    {
		$timingArgument .= "-delayMin $1 -delayMax $2 ";  ## used captured search targets
		if ($line =~ m/with (.*) m bias offset/)
		{
		    $timingArgument .= "-bias $1 ";  ## used captured search target
		}
	    }
	    
	} ## end dealing with zenith angle/delay specification lines


	if ($line =~ m/^\# \S+ \S+ \S+ \S+  \S+ \S+ \S+  transits at/)
	{
	    # Extract name, transit time
	    my @tmp = split /\s+/, $line;
	    my $name = $tmp[1];
	    my $raH = $tmp[2];
	    my $raM = $tmp[3];
	    my $raS = $tmp[4];
	    my $decD = $tmp[5];
	    my $decM = $tmp[6];
	    my $decS = $tmp[7];
	    my $tTime = $tmp[10];

	    my $indx = scalar @targets;
	    if (!(exists $transit{$name}))
	    {
		$targets[$indx] = $name;  ## Create list of targets, avoiding duplications
	    }

	    $transit{$name} = RAdeg($tTime);
	    $raStr{$name} = join ' ', $raH,$raM,$raS;
	    $decStr{$name} = join ' ', $decD,$decM,$decS;

	    if ($line =~ m/target/)  ## Add support for differentiating targets and cals
	    {
		$target{$name} = 1;
	    }
	    else
	    {
		$target{$name} = 0;
	    }
	} # end if transits line

	if ($line =~ m/^\# \S+ is within zenith angle range from/)
	{
	    my @tmp = split /\s+/, $line;

	    my $name = $tmp[1];
	    ## New -- reference _everything_ to the transit timing -- A.B. 1/2002
	    my ($leadZA,$trailZA)
		= &relativeTimings(&RAdeg($tmp[8]),&RAdeg($tmp[10]),$transit{$name});

	    $leadZA{$name} = $leadZA;
	    $trailZA{$name} = $trailZA;

	} # end zenith angle line

	## HDC131511 is within delay range from 01:12:58 to 05:06:13 UT for PTI_NW
	if ($line =~ m/^\# \S+ is within delay range from/)
	{
	    my @tmp = split /\s+/, $line;
	    
	    my $name = $tmp[1];
	    ## New -- reference everything to the transit timing -- A.B. 1/2002
	    my ($leadDA,$trailDA)
		= &relativeTimings(RAdeg($tmp[7]),RAdeg($tmp[9]),$transit{$name});
	    my $bl = $tmp[12];

	    ## If this data hasn't already been inserted...
	    my $delaystr = join ':', $bl,$leadDA,$trailDA;
	    if (!(exists $newDelay{$name})
		|| !($newDelay{$name} =~ m/$delaystr/))
	    {
		## compose the string encoding and append to the entry
		if (exists $newDelay{$name})
		{
		    $newDelay{$name} .= "|" . $delaystr;
		}
		else
		{
		    $newDelay{$name} = $delaystr;
		}
	    } ## end !$newDelay{$name}

	} # end delay line

        ## HDC163506 is always within delay range for NPOI_AN0AE0
	if ($line =~ m/^\# \S+ is always within delay range/)
	{
	    my @tmp = split /\s+/, $line;
	    my $name = $tmp[1];

	    ## Always in delay range -- hardwire the limits relative
	    ## to the transit point

	    my $leadDA = -179.99;  ## Again, relative to transit
	    my $trailDA = +179.99;
	    my $bl = $tmp[8];

	    ## If this data hasn't already been inserted...
	    if (!(exists $newDelay{$name})
		|| !($newDelay{$name} =~ m/$bl/))
	    {
		## compose the string encoding and append to the entry
		my $str = join ':', $bl,$leadDA,$trailDA;
		if (exists $newDelay{$name})
		{
		    $newDelay{$name} .= "|" . $str;
		}
		else
		{
		    $newDelay{$name} = $str;
		}
	    } ## end !$newDelay{$name}
    

	} ## end always in delay range
    } # end foreach $line in @inputStream


    ## Clean-up the preamble text a little
    $preambleText =~ s/\#//g;

    ## Check that we got baseline information
    if (!($preambleText =~ m/Using .* Baseline/))
    {
	print "uvTool: No baseline information found in input stream, exiting.\n";
	exit(0);
    } ## no baseline/delay timing info -- exit


    ## (re)Set spatial frequecy info
    $maxSPFreq = $maxBModulus / ($lambda * 1.0e-6);
    $minSPFreq = $minBModulus / ($lambda * 1.0e-6);
    $RmaxSPFreq = &round($maxSPFreq,+1.0);  ## Rounded versions are used for plotting
    $RminSPFreq = &round($minSPFreq,0.0);

    ## (re)set default target and baseline information as required
    if (!(defined $targetID) || !(grep m/$targetID/, @targets))
    {
	$targetID = $targets[0];  ## default to first target entered
    }
    foreach my $bl (@baseline)   ## default to all baseline in the input set
    {
	$bSelect{$bl} = 1;
    } ## end foreach


} ## end readTimingRpt  #################################################



#########################################################################
## relativeTimings -- compute transit-referenced timings
#########################################################################
sub relativeTimings
{
    ## relativeTimings computes the relative (transit-referenced) timings

    ## relativeTimings consolidates the logic for correctly
    ## interpreting the relative timing information between limits
    ## (zenith, delay, obscuration) and the transit of an object.  Of
    ## course the tricky part of this is the 0/360 degree wrap encoded
    ## in the input data.  It begs the question of whether I should
    ## change the output format of timing...?

## declarations relativeTimings  ########################################
    my ($lead,$trail,$transit) = @_;


## begin relativeTimings  ###############################################
    ## Step 1: By definition the $trail must follow $lead, so if this
    ## is not the case we augment the $trail -- this must be a wrap
    ## effect...and now $trail is by definition on the same branch as
    ## $lead...
    $trail += 360.0 if ($trail < $lead);
    ## Now compute the mid-point
    my $midPt = ($lead + $trail) / 2.0;


    ## Step 2: Choose the branch that minimizes the distance between
    ## $midPt and $transit (this seems most reasonable)...
    if ($midPt > $transit)
    {
	if (abs(($midPt - 360.0) - $transit) < abs($midPt - $transit))
	{
	    $lead -= 360.0;
	    $trail -= 360.0;
	}
    } ## $lead > $transit
    else ## otherwise $midPt < $transit
    {
	if (abs(($midPt + 360.0) - $transit) < abs($midPt - $transit))
	{
	    $lead += 360.0;
	    $trail += 360.0;
	}
    } ## end $lead < $transit


    ## Step 3: Reference to $transit...
    $lead -= $transit;
    $trail -= $transit;

    
    ## "Home is where you wear your hat!"
    return ($lead,$trail);

} ## end relativeTimings  ###############################################



#########################################################################
## drawTarget -- draw u-v tracks for a target
#########################################################################
sub drawTarget
{
## declarations drawTarget  #############################################
    ## Draws uv tracks for a single target over multiple baselines
    my $canvas = shift @_;  ## canvas argument
    my $key = shift @_;  ## target argument
    my @bList = @_;  ## The baseline list to draw


## begin drawTarget  ####################################################
    $canvas->delete("uvTrack");  ## Erase existing tracks

    ## Initialize some variables
    my $transitTime = $transit{$key};
    my $targDec = DECdeg($decStr{$key});

    ## Draw some descriptive text (designation, position) about this
    ## target at the bottom of the canvas
    my $tmpX = int (0.5 * $xCanvasSize / 20);
    my $tmpY = int (8.6 * $yCanvasSize / 10);
    $canvas->createText($tmpX,$tmpY,-anchor => "w",-text => $key,-tags => 'uvTrack');
    my $coords = join '  ', $raStr{$key},$decStr{$key};
    $canvas->createText($tmpX,$tmpY + 16,-anchor => "w",-text => $coords,-tags => 'uvTrack');
    my $tmpTime = &RAhms($transitTime,-1);
    $tmpTime =~ s/\s+/:/g;
    my $transitStr = join ' ',"Transit",$tmpTime,"UT";
    $canvas->createText($tmpX,$tmpY + 32,-anchor => "w",-text => $transitStr,-tags => 'uvTrack');
    if ($LocalZeroUT)
    {
	my ($lH,$lM,$lS) = split /:/, $LocalZeroUT;
	my $loc = (15.0 * ($lH + $lM / 60.0 + $lS / 3600.0)) + $transitTime;
	if ($loc > 360.0) ## Fix wraps
	{
	    $loc -= 360.0;
	}
	$tmpTime = &RAhms($loc,-1);
	$tmpTime =~ s/\s+/:/g;
	$transitStr = join ' ',"Transit",$tmpTime, "Local";
	$canvas->createText($tmpX,$tmpY + 48,-anchor => "w",-text => $transitStr,-tags => 'uvTrack');
    } ## if we have local time info from preamble


    ## Loop over baselines
    my $bl;
    foreach $bl (@bList)
    {
	## Logic for rendering the intersection track (default)
	my $color = $uvToolAttributes::intersectionColor;
	my $ha;
	my $trackInc = 1.5;  ## rendering step size (degrees)  Effects GUI response!

	## compute and render intersection track
	if ($intersectionTrack)
	{
	    # compute the intersection of ZA range and delay range
	    # initialize the intersection list with the ZA limits
	    if ((defined $leadZA{$key}) &&
		(defined $trailZA{$key}))
	    {

		# 1) start the intersection list with the ZA range
		my @iList = ( [$leadZA{$key}, $trailZA{$key}] ) ;

		# 2) intersect that with a list of delay limits
		if ((exists $newDelay{$key}) && ($newDelay{$key} =~ m/$bl/))
		{
		    my @delay = grep /$bl/, (split '\|', $newDelay{$key});
		    my @dList;
		    foreach my $dseg (@delay)
		    { 
			(undef, my $dEntryTime, my $dExitTime) = split /:/, $dseg;
			if (($dEntryTime =~ m/\d+/)
			    && ($dExitTime =~ m/\d+/))
			{
			    push @dList, [$dEntryTime, $dExitTime];
			}
		    }
		    ## Do the intersection computation (in hour angle to avoid wraps)
		    my $iflag = intersectList(\@iList, \@dList, \@iList);       # intersect composite list with delay

		    ## Render the intersection track
		    if ($iflag)
		    {
			# draw each segment of the intersection
			for (my $idx = 0; $idx < scalar(@iList); $idx++)
			{
			    # break out entry/exit times for this segment
			    my $iEntry = $iList[$idx][0];
			    my $iExit = $iList[$idx][1];

			    # draw the segment
			    my $first = 1;
			    my ($u,$v,$w,$oldU,$oldV);
			    for ($ha = $iEntry;
				 $ha <= $iExit;
				 $ha += $trackInc)
			    {
				### ($u,$v) = uvPt($ha,$targDec,$latitude,@bline);
				($u,$v,$w) = $bHash{$bl}->uvw($ha,$targDec,$lambda * 1.0e-6);
				
				if (!($first))
				{
				    $canvas->createLine(xCoord($oldU),yCoord($oldV),
							xCoord($u),yCoord($v),
							-width => 3,
							-capstyle => 'projecting',
							-fill => $color,
							-tags => 'uvTrack');
				    $canvas->createLine(xCoord(-$oldU),yCoord(-$oldV),
							xCoord(-$u),yCoord(-$v),
							-width => 3,
							-capstyle => 'projecting',
							-fill => $color,
							-tags => 'uvTrack');
				} ## end first
				$first = 0;
				$oldU = $u;
				$oldV = $v;
			    } ## end for point on segment
			} ## end for segment
		    } ## end $intersect check
		} ## end delay range check
	    } # end ZA range
	} ## end $intersectionTrack

	## render ZA track
	if ($zaTrack)
	{
	    ## zenith-limited track in green
	    $color = $uvToolAttributes::zaColor;
	    my $zEntryTime = $leadZA{$key};
	    my $zExitTime = $trailZA{$key};

	    if ($zEntryTime && $zExitTime)
	    {
		my $first = 1;
		my ($u,$v,$oldU,$oldV, $w);

		for ($ha = $zEntryTime;
		     $ha <= $zExitTime;
		     $ha += $trackInc)
		{
		    ### ($u,$v) = uvPt($ha,$targDec,$latitude,@bline);
		    ($u,$v,$w) = $bHash{$bl}->uvw($ha,$targDec,$lambda * 1.0e-6);

		    if (!($first))
		    {
			$canvas->createLine(xCoord($oldU),yCoord($oldV),
					    xCoord($u),yCoord($v),
					    -width => 5,
					    -capstyle => 'projecting',
					    -fill => $color,
					    -tags => 'uvTrack');
			$canvas->createLine(xCoord(-$oldU),yCoord(-$oldV),
					    xCoord(-$u),yCoord(-$v),
					    -width => 5,
					    -capstyle => 'projecting',
					    -fill => $color,
					    -tags => 'uvTrack');
		    } ## end first

		    $first = 0;
		    $oldU = $u;
		    $oldV = $v;

		} ## end for
	    } ## end zenith limit check
	} ## end if zaTrack


	## render delay track
	if ($delayTrack && $newDelay{$key})
	{
	    print STDERR "DEBUG: delay string is \"$newDelay{$key}\"\n" if gcConfig::debug();
	    $color = $uvToolAttributes::delayColor;
	    my @delay = grep /$bl/, (split '\|', $newDelay{$key});
	    foreach my $dseg (@delay)
	    {
		print STDERR "DEBUG: delay seg is \"$dseg\"\n" if gcConfig::debug();
		my ($null,$dEntryTime,$dExitTime) = split /:/, $dseg;
		if ($dEntryTime && $dExitTime)
		{
		    my $first = 1;
		    my ($u,$v,$oldU,$oldV, $w);
		    print STDERR "DEBUG: drawing delay seg $dEntryTime:$dExitTime\n" if gcConfig::debug();

		    for ($ha = $dEntryTime;
			 $ha <= $dExitTime;
			 $ha += $trackInc)
		    {
			### ($u,$v) = uvPt($ha,$targDec,$latitude,@bline);
			($u,$v,$w) = $bHash{$bl}->uvw($ha,$targDec,$lambda * 1.0e-6);

			if (!($first))
			{
			    $canvas->createLine(xCoord($oldU),yCoord($oldV),
						xCoord($u),yCoord($v),
						-width => 2,
						-capstyle => 'projecting',
						-fill => $color,
						-tags => 'uvTrack');
			    $canvas->createLine(xCoord(-$oldU),yCoord(-$oldV),
						xCoord(-$u),yCoord(-$v),
						-width => 2,
						-capstyle => 'projecting',
						-fill => $color,
						-tags => 'uvTrack');
			} ## end first

			$first = 0;
			$oldU = $u;
			$oldV = $v;

		    } ## end for
		} ## end zenith limit check
	    } ## end $delayTrack
	} ## end delay segment
	

	## ghost-track in black
	$color = $uvToolAttributes::ghostColor;

	if ($ghostTrackOn)
	{
	    my $first = 1;
	    my ($u,$v,$oldU,$oldV, $w);

	    for ($ha = -180.0; $ha <= 180.0; $ha += $trackInc)
	    {
		($u,$v,$w) = $bHash{$bl}->uvw($ha,$targDec,$lambda * 1.0e-6);
		
		if (!($first))
		{
		    $canvas->createLine(xCoord($oldU),yCoord($oldV),
					xCoord($u),yCoord($v),
					-width => 1,
					-capstyle => 'projecting',
					-fill => $color,
					-stipple => "gray75",
					-tags => 'uvTrack');
		    $canvas->createLine(xCoord(-$oldU),yCoord(-$oldV),
					xCoord(-$u),yCoord(-$v),
					-width => 1,
					-capstyle => 'projecting',
					-fill => $color,
					-stipple => "gray75",
					-tags => 'uvTrack');
		} ## end first

		$first = 0;
		$oldU = $u;
		$oldV = $v;

	    } ## end for
	} ## end if ghostTrack



	## Transit/ha markers as requested ($haMarks)
	if ($haMarks)
	{
	    ## Transit marker in red
	    my $tSize = 3;
	    $color = $uvToolAttributes::transitColor;
	    my ($u,$v,$w) = $bHash{$bl}->uvw(0,$targDec,$lambda * 1.0e-6);

	    $canvas->createOval(xCoord($u) - $tSize,xCoord($v) - $tSize,
				xCoord($u) + $tSize,xCoord($v) + $tSize,
				-fill => $color, -outline => $color,
				-tags => 'uvTrack');

	    $canvas->createOval(xCoord(-$u) - $tSize,yCoord(-$v) - $tSize,
				xCoord(-$u) + $tSize,yCoord(-$v) + $tSize,
				-fill => $color, -outline => $color,
				-tags => 'uvTrack');

	} ## end if $haMarks (transit marker)

	## Now optionally draw minor hour angle marks smaller and also in red
	if ($haMarks > 1)
	{
	    $color = $uvToolAttributes::transitColor;
	    $tSize = 1.5;
	    for ($ha = 1; $ha <= 3; $ha++)
	    {
		my ($u,$v,$w) = $bHash{$bl}->uvw($ha * 15.0,$targDec,$lambda * 1e-6);  ## angles in degrees

		$canvas->createOval(xCoord($u) - $tSize,yCoord($v) - $tSize,
				    xCoord($u) + $tSize,yCoord($v) + $tSize,
				    -fill => $color, -outline => $color,
				    -tags => 'uvTrack');

		$canvas->createOval(xCoord(-$u) - $tSize,yCoord(-$v) - $tSize,
				    xCoord(-$u) + $tSize,yCoord(-$v) + $tSize,
				    -fill => $color, -outline => $color,
				    -tags => 'uvTrack');

		## Negative hour angle marks
		($u,$v,$w) = $bHash{$bl}->uvw(-$ha * 15.0,$targDec,$lambda * 1e-6);  ## angles in degrees
		$canvas->createOval(xCoord($u) - $tSize,yCoord($v) - $tSize,
				    xCoord($u) + $tSize,yCoord($v) + $tSize,
				    -fill => $color, -outline => $color,
				    -tags => 'uvTrack');

		$canvas->createOval(xCoord(-$u) - $tSize,xCoord(-$v) - $tSize,
				    xCoord(-$u) + $tSize,xCoord(-$v) + $tSize,
				    -fill => $color, -outline => $color,
				    -tags => 'uvTrack');

	    } ## end ha/ha marker loop
	} ## end if $haMarks > 1


    } ## end foreach baseline loop

} ## end drawTarget  ####################################################


#########################################################################
## baselineList -- construct the baseline list
#########################################################################
sub baselineList()
{
    ## baselineList constructs a list of list of currently active
    ## baselines specified in the %bSelect hash

## declarations baselineList  ###########################################
    my ($bl,@bList);
    foreach $bl (keys %bSelect)
    {
	if ($bSelect{$bl})
	{
	    push @bList, $bl;
	}
    } ## end foreach

    return @bList;

} ## end baselineList  ##################################################



#########################################################################
## setCurrentTime -- compose strings reflecting current time
#########################################################################
sub setCurrentTime
{
    ## setCurrentTime composes time strings for current UT, LST, and local time

## begin setCurrentTime  ################################################
    # get the current UT
    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = gmtime(time);
    my $ut = $hour + $min / 60.0 + $sec / 3600.0;

    ## Compose UT string
    $currentUT = &RAhms(15.0 * $ut,-1);
    $currentUT =~ s/\s+/:/g;
    $currentUT .= " UTC";

    
    ## Now the LST string
    my ($lstH,$lstM,$lstS) = split /:/, $LSTzeroUT;
    my $lst = 15.0 * ($lstH + $lstM / 60.0 + $lstS / 3600.0
		      + (novasConstants::SolarToSidereal * $ut));
    if ($lst > 360.0) ## Fix wraps
    {
	$lst -= 360.0;
    }
    $currentLST = &RAhms($lst,-1);
    $currentLST =~ s/\s+/:/g;
    $currentLST .= " LST";


    ## Finally the local time string
    if ($LocalZeroUT)
    {
	($lstH,$lstM,$lstS) = split /:/, $LocalZeroUT;
	$lst = 15.0 * ($lstH + $lstM / 60.0 + $lstS / 3600.0 + $ut);
	if ($lst > 360.0) ## Fix wraps
	{
	    $lst -= 360.0;
	}
	$currentLocal = &RAhms($lst,-1);
	$currentLocal =~ s/\s+/:/g;
	$currentLocal .= " Local";
    }
    else
    {
	$currentLocal = "Not Avail";
    }


    $MW->after(10000, \&setCurrentTime);  ## recompute current time every 10 seconds

} ## end setCurrentTime  ################################################



#########################################################################
## drawCurrentUT -- draw markers for current time
#########################################################################
sub drawCurrentUT
{

## declarations drawCurrentUT  ##########################################
    my $key = $targetID; ## Using global access to avoid recursion issues
    my @bList = &baselineList();  ## These were originally call arguments.  Sorry.

    my $canvasRef = shift @_; ## extract canvas we're drawing to
    my $canvas = $$canvasRef;

## begin drawCurrentUT  #################################################
    $canvas->delete("UTLine");  ## delete previous UT line and labels

    if ($utOn) ## If we're going to draw a current time marker
    {

	# get the current UT
	my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = gmtime(time);
	my $ut = $hour + $min / 60.0 + $sec / 3600.0;

	## Create UT string
	my $utStr = &RAhms(15.0 * $ut,-1);
	$utStr =~ s/\s+/:/g;
	$utStr .= " UTC";


	## Create an LST and HA
	## Now the LST string
	my ($lstH,$lstM,$lstS) = split /:/, $LSTzeroUT;
	my $lst = 15.0 * ($lstH + $lstM / 60.0 + $lstS / 3600.0 
			  + (novasConstants::SolarToSidereal * $ut));
	if ($lst > 360.0) ## Fix wraps
	{
	    $lst -= 360.0;
	}
	my $lstStr = &RAhms($lst,-1);
	$lstStr =~ s/\s+/:/g;
	$lstStr .= " LST";

        ## HA on the source
	my $targDec = DECdeg($decStr{$key});
	my $ha = &hourAngle(RAdeg($raStr{$key}),$lst,1);

	my $bl;
	foreach $bl (@bList)
	{
	    my ($u,$v,$w) = $bHash{$bl}->uvw($ha,$targDec,$lambda * 1e-6);

	    # render the UT marker and UT/HA labels
	    my $markerLength = 15;
	    my $mag = sqrt(($u * $u) + ($v * $v));
	    my $xOffset = $markerLength * ($u / $mag);
	    my $yOffset = $markerLength * ($v / $mag);

	    ## Now rendering u-v markers as short line segments with
	    ## arrow heads -- A.B. 7/01
	    $canvas->createLine(xCoord($u),yCoord($v),
				xCoord($u) + $xOffset,yCoord($v) + $yOffset,
				-arrow => "first",
				-width => 2,
				-tags => "UTLine",
				-fill => $uvToolAttributes::currentUTColor);

	    $canvas->createLine(xCoord(-$u),yCoord(-$v),
				xCoord(-$u) - $xOffset,yCoord(-$v) - $yOffset,
				-arrow => "first",
				-width => 2,
				-tags => "UTLine",
				-fill => $uvToolAttributes::currentUTColor);


	} ## end foreach baseline loop

	## Draw some UT/LST/HA text information
	my $tmpX = int (16.5 * $xCanvasSize / 20);
	my $tmpY = int (0.78 * $yCanvasSize);

	## Date info
	$mon += 1;
	$year += 1900;
	my $dateStr = join '/',$mon,$mday,$year;
	$canvas->createText($tmpX,$tmpY,-anchor => "w",-text => $dateStr,-tags => "UTLine");

	## UT
	$tmpY += 15;
	$canvas->createText($tmpX,$tmpY,-anchor => "w",-text => $utStr,-tags => "UTLine");

	## LST
	$tmpY += 15;
	$canvas->createText($tmpX,$tmpY,-anchor => "w",-text => $lstStr,-tags => "UTLine");

	## local time
	if ($LocalZeroUT)
	{
	    $tmpY += 15;
	    my ($lH,$lM,$lS) = split /:/, $LocalZeroUT;
	    my $loc = 15.0 * ($lH + $lM / 60.0 + $lS / 3600.0 + $ut);
	    if ($loc > 360.0) ## Fix wraps
	    {
		$loc -= 360.0;
	    }
	    my $localStr = &RAhms($loc,-1);
	    $localStr =~ s/\s+/:/g;
	    $localStr .= " Local";
	    $canvas->createText($tmpX,$tmpY,-anchor => "w",-text => $localStr,-tags => "UTLine");
	} ## end testing local time info available

	## hour angle
	$tmpY += 15;
	my $haStr = &RAhms($ha,-1);
	$haStr =~ s/\s+/:/g;
	$haStr .= " HA";
	$canvas->createText($tmpX,$tmpY,-anchor => "w",-text => $haStr,-tags => "UTLine");

	## zenith angle
	$tmpY += 15;
	my $za = zenithAngle($ha,$targDec,$latitude);
	my $zaStr = sprintf "%3.1f", $za;
	$zaStr .= " deg ZA";
	$canvas->createText($tmpX,$tmpY,-anchor => "w",-text => $zaStr,-tags => "UTLine");

	## airmass
	if ($za < 90)
	{
	    $tmpY += 15;
	    my $am = 1.0 / cos($za * 3.14159 / 180.0);
	    my $amStr = sprintf "%3.2f", $am;
	    $amStr .= " airmass";
	    $canvas->createText($tmpX,$tmpY,-anchor => "w",-text => $amStr,-tags => "UTLine");
	} ## end if $za < 90
	

        ## re-render UT line every 15 seconds
	##  Note: strange syntax for invoking callback with argument
	$MW->after(15000, [\&drawCurrentUT, Ev($canvasRef)]);  

    } ## end if $utOn

} ## end drawCurrentUT  #################################################


#########################################################################
## drawAngleCircles -- draw circle references
#########################################################################
sub drawAngleCircles
{

## declarations drawAngleCircles  #######################################
    ## drawAngleCircles renders concentric circles corresponding to
    ## angular fringe spacings on the spatial frequency plot
    my $canvas = shift @_;
    

## begin drawAngleCircles  ##############################################
    $canvas->delete("AngleCircles");  ## delete previous angle indicators

    if ($anglesOn) ## If we're going to draw angle indicators
    {
	## Start by drawing circles at the maximum and minimum spatial
	## frequencies

	## Maximum
	$canvas->createOval(xCoord(-$RmaxSPFreq),yCoord(-$RmaxSPFreq),
			    xCoord(+$RmaxSPFreq),yCoord(+$RmaxSPFreq),
			    -outline => $uvToolAttributes::maxFSpaceColor,
			    -tags => "AngleCircles",);


	my $angle = 1.0 / $RmaxSPFreq / 4.85e-9; ## convert from spfreq to rad to mas
	$angle = sprintf "%4.1f", $angle;
	if ($angle > 500.0)
	{
	    $angle /= 1000.0;
	    $angle = sprintf "%4.1f", $angle;
	    $angle .= " as";
	}
	else
	{
	    $angle .= " mas";
	}

	$canvas->createText(xCoord(-$RmaxSPFreq/ 1.4),yCoord($RmaxSPFreq / 1.4),
			    -fill => $uvToolAttributes::maxFSpaceColor,
			    -anchor => "w",-text => $angle,
			    -tags => "AngleCircles");

    } ## end if $anglesOn

    if ($anglesOn > 1)  ### check for minimum
    {
	## Minimum
	$canvas->createOval(xCoord(-$RminSPFreq),yCoord(-$RminSPFreq),
			    xCoord(+$RminSPFreq),yCoord(+$RminSPFreq),
			    -outline => $uvToolAttributes::medFSpaceColor,
			    -tags => "AngleCircles",);


	my $angle = 1.0 / $RminSPFreq / 4.85e-9; ## convert from spfreq to rad to mas
	$angle = sprintf "%4.1f", $angle;
	if ($angle > 500.0)
	{
	    $angle /= 1000.0;
	    $angle = sprintf "%4.1f", $angle;
	    $angle .= " as";
	}
	else
	{
	    $angle .= " mas";
	}

	$canvas->createText(xCoord(-$RminSPFreq/ 1.4),yCoord($RminSPFreq / 1.4),
			    -fill => $uvToolAttributes::medFSpaceColor,
			    -anchor => "w",-text => $angle,-tags => "AngleCircles");
    } ## end if $anglesOn > 1


    if ($anglesOn > 2)
    {
	## Minimum rounded down
	my $rMinSP = &round($RminSPFreq,-3.0);
	$canvas->createOval(xCoord(-$rMinSP),yCoord(-$rMinSP),
			    xCoord(+$rMinSP),yCoord(+$rMinSP),
			    -outline => $uvToolAttributes::minFSpaceColor,
			    -tags => "AngleCircles",);


	my $angle = 1.0 / $rMinSP / 4.85e-9; ## convert from spfreq to rad to mas
	$angle = sprintf "%4.1f", $angle;
	if ($angle > 500.0)
	{
	    $angle /= 1000.0;
	    $angle = sprintf "%4.1f", $angle;
	    $angle .= " as";
	}
	else
	{
	    $angle .= " mas";
	}

	$canvas->createText(xCoord(-$rMinSP / 1.4),yCoord($rMinSP / 1.4),
			    -fill => $uvToolAttributes::minFSpaceColor,
			    -anchor => "w",-text => $angle,-tags => "AngleCircles");

    }  ## end if $anglesOn > 2

} ## end drawAngleCircles  ##############################################



#########################################################################
## dispHelp -- display help text from file
#########################################################################
sub dispHelp
{
## declarations dispHelp  ###############################################
    my $filename = shift @_;


## begin dispHelp  ######################################################
    if (!defined $filename)
    {
	print "Error: undefined document link in uvTool\n";
	return;
    }

    ## Basically we open a text window and display the help file -- in
    ## all it's glory.

    ## create a text window to display the return info
    my $helpWindow = $MW->Toplevel();
    $helpWindow->title("help -- $filename");
    my $text = $helpWindow->Scrolled("ROText",
				     -wrap => "none",
				     -width => 90, -height => 32,
				     -relief => "groove"
				     )->pack(-side => "top",
					-expand => "yes",
					-fill => "both");

    # dump the contents of the help file into the text window
    my $helpMsg = `cat $filename`;

    $text->insert('end',$helpMsg);

    # add a close button
    my $hwCntrl = $helpWindow->Frame(-relief => "groove",
				     -borderwidth => "2"
				     )->pack(-side => "bottom", -fill => "x");
    $hwCntrl->Button(-text => "Close",
		     -relief => "raised",
		     -command => sub {$helpWindow->withdraw}
		     )->pack(-side => "right");

    $helpWindow->bind('<Return>' => sub {$helpWindow->withdraw});


} ## end dispHelp  ######################################################



#########################################################################
## printUVDiagram -- handle printing tasks for uvTool
#########################################################################
sub printUVDiagram
{

## begin printUVDiagram  ################################################

    ## TLE -- 7/7/2004 get Baseline list for subroutine
    my @bList = &baselineList();
    my $tmpTxt;

    ## Make a new canvas especially for printing
    my $printCanvas = $MW->Scrolled('Canvas',
				    -width => $xCanvasSize,
				    -height => $yCanvasSize);
    ## draw stuff into it
    &plotScales(); ## recompute the plot scales
    &drawAxes($printCanvas);  ## redraw the axes
    ##&drawTarget($printCanvas,$targetID,&baselineList());
    &drawTarget($printCanvas,$targetID,@bList);
    &drawCurrentUT(\$printCanvas); ## update current UT display (if any)
    &drawAngleCircles($printCanvas); ## redraw fringe spacing indocators (if selected)

    ## Add reporting information on the plot
    if ($intersectionTrack)
    {
	$printCanvas->createText($xCanvasSize * 1.0,$yCanvasSize * 0.15,
				 -fill => $uvToolAttributes::intersectionColor,
				 -anchor => "w",-text => "Intersection Track");
    }
    else
    {
	$printCanvas->createText($xCanvasSize * 1.0,$yCanvasSize * 0.15,
				 -fill => $uvToolAttributes::zaColor,
				 -anchor => "w",-text => "Zenith Angle Track") if ($zaTrack);

	$printCanvas->createText($xCanvasSize * 1.0,$yCanvasSize * 0.25,
				 -fill => $uvToolAttributes::delayColor,
				 -anchor => "w",-text => "Delay Line Track") if ($delayTrack);


    }

    $printCanvas->createText($xCanvasSize * 1.0,$yCanvasSize * 0.35,
			     -fill => $uvToolAttributes::transitColor,
			     -anchor => "w",-text => "Transit/HA") if ($haMarks);




    ## my @preambleLines = split /\n/, $preambleText;
    my @tmp = grep /for timings on/, (split /\n/, $preambleText);
    if ($tmp[0])
    {
	$tmp[0] =~ s/ for timings on //g;
	$printCanvas->createText($xCanvasSize * 1.0,$yCanvasSize * 0.5,
				 -anchor => "w",-text => $tmp[0]);

    }

    ## TLE -- 7/7/2004 add wavelength between date and scale
    $tmpTxt = sprintf "Wavelength: %g microns", $lambda;
    $printCanvas->createText($xCanvasSize * 1.0,$yCanvasSize * 0.6,
                             -anchor => "w",-text => $tmpTxt);

    $tmpTxt = sprintf "Scale: %g lambda", $scale;
    $printCanvas->createText($xCanvasSize * 1.0,$yCanvasSize * 0.7,
			     -anchor => "w",-text => $tmpTxt);

    ## TLE -- 7/7/2004 add Baseline(s), Location to plot (moving "Produced"
    ##      and link downwards)
    @tmp = grep /Using .* Location/, (split /\n/, $preambleText);
    if ($tmp[0])
    {
        $tmp[0] =~ s/ Using\s+//g;
        $tmpTxt = $tmp[0];
    }
    $printCanvas->createText($xCanvasSize * 0.05,$yCanvasSize * 1.03,
                             -anchor => "w",-text => $tmpTxt);
    $tmpTxt = "Baseline(s): @bList";
    $printCanvas->createText($xCanvasSize * 0.05,$yCanvasSize * 1.06,
                             -anchor => "w",-text => $tmpTxt);

    $tmpTxt = "Produced by uvTool GUI v$REVISION ($PKGSTR)";
 
    $printCanvas->createText($xCanvasSize * 0.05,$yCanvasSize * 1.1,
                             -anchor => "w",-text => $tmpTxt);
        
    $tmpTxt = "NASA Exoplanet Science Institute -- http://nexsci.caltech.edu/";
    $printCanvas->createText($xCanvasSize * 0.05,$yCanvasSize * 1.13,
                             -fill => 'blue',
                             -anchor => "w",-text => $tmpTxt);

    ## make the PostScript rendering
    my ($bboxminX,$bboxminY,$bboxmaxX,$bboxmaxY) = $printCanvas->bbox("all");
    print STDERR "DEBUG: uvTool: canvas bounding box ($bboxminX,$bboxminY) ($bboxmaxX,$bboxmaxY)\n" if gcConfig::debug();
    #my $canvasWidth = $bboxmaxX;
    #my $canvasHeight = $bboxmaxY;
    my $canvasWidth = $xCanvasSize * 1.4;
    my $canvasHeight = $yCanvasSize * 1.2;
    print STDERR "DEBUG: uvTool: postscript canvas width=$canvasWidth, height=$canvasHeight\n" if gcConfig::debug();


    my $ps = $printCanvas->postscript( '-x' => 0,
				       '-y' => 0,
				       -rotate => 1,
				       -pagewidth => "6i",
				       -width => $canvasWidth,
				       -height => $canvasHeight);

    my $fileName = "uvPlot";
    if (defined $batchMode)
    {
	## Do batch mode output. Allow multiple output formats to be specified.
	writePsOutput("$fileName.ps",$ps) if (defined $PostScript); ## output PS
	writePsOutput("$fileName.gif",$ps) if (defined $pdf);   ## output GIF
	writePsOutput("$fileName.jpg",$ps) if (defined $jpg);   ## output JPEG
	writePsOutput("$fileName.png",$ps) if (defined $png);   ## output PNG
	writePsOutput("$fileName.pdf",$ps) if (defined $pdf);   ## output PDF
    }
    else
    {
	## Do "Print..." dialog to get filename or print command to use, 
	## then output the postscript as requested
	my $fn = &getOutputName($MW->Toplevel(), $balloon, $fileName);
	writePsOutput($fn,$ps);
    }
} ## end printUVDiagram  ################################################


#########################################################################
## round -- round argument to nearest 1.5 significant digits
#########################################################################
sub round
{
## declarations round  ##################################################
    my $arg = shift @_;
    my $direction = shift @_;

## begin round  #########################################################
    ## Idea: round up/down to 1.5 significant digits
    ## $direction > 0 -- rounding up by 0.5 * $direction significant digits
    ## $direction < 0 -- rounding down by 0.5 * $direction significant digits

    my $lg = (log($arg) / log(10.0));  ## base 10 log
    $lg = int($lg);
    my $halfMag = ((10.0)**$lg) / 2.0;

    $arg /= $halfMag;

    my $result = int ($arg + $direction);
    return ($result * $halfMag);

} ## end round  #########################################################


#########################################################################
## hourAngle -- compute hour angle for source
#########################################################################
sub hourAngle
{
    ## hourAngle computes the hour angle on a source at right
    ## ascention $ra (deg) at the current local siderial time ($lst)

## declarations hourAngle  ##############################################
    my ($ra, $lst, $primeInterval) = @_;


## begin hourAngle  #####################################################
    my $ha = $lst - $ra;

    ## Condition the hour angle to be in the interval -180 to 180 (A.B. 7/01)
    if ($primeInterval)
    {
	if ($ha < -180.0)
	{
	    $ha += 360.0;
	}

	if ($ha > 180.0)
	{
	    $ha -= 360.0;
	}
    } ## end prime interval calculation


    return $ha;


} ## end hourAngle  #####################################################



#########################################################################
## drawPreamble -- dump preamble text to gui window
#########################################################################
sub drawPreamble
{

    ## drawPreamble flushes the contents of the existing preamble
    ## frame and replace it with the new (global var) $preambleText

    $preamble->delete('1.0','end'); ## clear the existing text
    $preamble->insert('end',$preambleText);  ## Insert the new

} ## end drawPreamble  ##################################################




#########################################################################
## packBlineMenu -- pack baseline menu with available (dynamic) options 
#########################################################################
sub packBlineMenu
{
    ## packBlineMenu flushes the transient baseline entries from the
    ## baseline menu and replaces them with the contents of @baseline

## declarations packBlineMenu  ##########################################
    my $init = shift @_;

## begin packBlineMenu  #################################################
    if (!$init)
    {
	$baselineMenu->menu->delete('4', 'end');
    }


    foreach my $blTmp (@baseline)
    {
	## The baseline menu has check buttons (individually selectable)
	## for rendering multiple baselines
	$baselineMenu->checkbutton(-label => $blTmp,
				   -command => sub
				   {
				       my @bList = &baselineList();
				       &drawTarget($displayCanvas,$targetID,@bList);
				       &drawCurrentUT(\$displayCanvas); ## update current UT display (if any)
				   },
				   -variable => \$bSelect{$blTmp});
    } ## end foreach baseline menu packing loop

} ## end packBlineMenu  #################################################



#########################################################################
## packTargetMenu -- pack target menu with available (dynamic) options
#########################################################################
sub packTargetMenu
{
## declarations packTargetMenu  #########################################
    my $init = shift @_;

## begin packTargetMenu  ################################################
    ## Eliminate the extra stuff (if not init)...
    if (!$init)
    {
	$targetMenu->menu->delete('1', 'end');
    }

    my $nTargets = scalar @targets;
    my $nTargMenu = $uvToolAttributes::targMenuItems;  ## Max Items 

    if ($nTargets > $nTargMenu)
    {

	my ($count,@indices);
	$count = 0;
	while ($count < $nTargets)
	{
	    push @indices, $count;
	    $count += $nTargMenu;
	} ## end while
	push @indices, $nTargets;

	my ($i,$j);
	$count = 1;
	for ($i = 0; $i < (scalar @indices - 1); $i++)
	{
	    my $addlTargets = $targetMenu->cascade(-label => "Target List $count");
	    for ($j = $indices[$i]; $j < $indices[$i + 1]; $j++)
	    {
		my $targTmp = $targets[$j];

		$addlTargets->radiobutton(-label => $targTmp,
					  -command => sub
					  {
					      my @bList = &baselineList();
					      &drawTarget($displayCanvas,$targetID,@bList);
					      &drawCurrentUT(\$displayCanvas); ## update current UT display (if any)
					  },
					  -variable => \$targetID,
					  -value => $targTmp);
	    } ## end j for loop
	    $count++;
	} ## end i for loop
    }
    else ## no need for sub-menus -- all in one single menu
    {

	foreach my $targTmp (@targets)
	{
	    ## The target menu has radiobuttons (mutually exclusive) for each
	    ## of the targets in the input stream
	    $targetMenu->radiobutton(-label => $targTmp,
				     -command => sub
				     {
					 my @bList = &baselineList();
					 &drawTarget($displayCanvas,$targetID,@bList);
					 &drawCurrentUT(\$displayCanvas); ## update current UT display (if any)
				     },
				     -variable => \$targetID,
				     -value => $targTmp);
	} ## end target menu packing/foreach loop
    } ## end else


} ## end packTargetMenu  ################################################



#########################################################################
## packSimbadMenu -- pack simbad menu with available (dynamic) options 
#########################################################################
sub packSimbadMenu
{
    ## packSimbadMenu manages the dynamic part of the Simbad menu
    ## contents

## declarations packSimbadMenu  #########################################
    my $init = shift @_;


## begin packSimbadMenu  ################################################
    ## Eliminate the extra stuff (if not init)...
    if (!$init)
    {
	$simbadMenu->menu->delete('5', 'end'); ### this needs to be fixed?
    }

    my $nTargets = scalar @targets;
    my $nTargMenu = $uvToolAttributes::targMenuItems;  ## Max Items 
    if ($nTargets > $nTargMenu)
    {
	my ($count,@indices);
	$count = 0;
	while ($count < $nTargets)
	{
	    push @indices, $count;
	    $count += $nTargMenu;
	} ## end while
	push @indices, $nTargets;

	my ($i,$j);
	$count = 1;
	for ($i = 0; $i < (scalar @indices - 1); $i++)
	{
	    my $addlTargets = $simbadMenu->cascade(-label => "Target List $count");
	    for ($j = $indices[$i]; $j < $indices[$i + 1]; $j++)
	    {
		my $targTmp = $targets[$j];
		$addlTargets->command(-label => $targTmp,
				      -command => sub  ## query individual object
				      {
					  my $trgName = &simbadName($targTmp);
					  doSimbadGui($MW,$trgName);
				      });
	    } ## end j for loop
	    $count++;
	} ## end i for loop
    }
    else ## no need for sub-menus -- all in one single menu
    {
	foreach my $targTmp (@targets)
	{
	    $simbadMenu->command(-label => $targTmp,
				 -command => sub  ## query individual object
				      {
					  my $trgName = &simbadName($targTmp);
					  doSimbadGui($MW,$trgName);
				      });
	} ## end target menu packing/for loop
    } ## end else


} ## end packSimbadMenu  ################################################



#########################################################################
## aboutuvTool -- about splashscreen
#########################################################################
sub aboutuvTool
{
    ## aboutuvTool creates a splash screen with uvTool information

## begin aboutuvTool  ###################################################
    my $aboutWindow = $MW->Toplevel();
    $aboutWindow->title("about uvTool");

    ## Display logo image in a button
    my $logoFile = findFile("NExScI_logo_tiny.gif",$cfg->gcdocs . "/images", $cfg->gcdata . "/images");
    return if (!defined $logoFile);

    my $logo = $aboutWindow->Photo(-file => $logoFile);
    $aboutWindow->Button(-text => '',
			 -command => sub {$aboutWindow->withdraw},
			 -image => $logo)->pack(-side => "top", -fill => "x");

    $aboutWindow->Label(-text => "http://nexsci.caltech.edu")->pack(-side => "top", -fill => "x");
    $aboutWindow->Label(-text => "uvTool v$REVISION ($PKGSTR)")->pack(-side => "top", -fill => "x");
    $aboutWindow->Label(-text => $uvToolAttributes::uvToolCopyright)->pack(-side => "top", -fill => "x");
    $aboutWindow->bind('<Return>' => sub { $aboutWindow->withdraw });

} ## end aboutuvTool  ###################################################
