#! @PERL@
# -*- perl -*-
# @configure_input@

our $MODVERSION = '$Id: irsaminer.pl,v 1.6 2006/06/16 01:46:30 claude Exp $';

# irsaminer - command line utility for performing queries of the IRSA archive
# http://irsa.ipac.caltech.edu

# irsaminer is a standalone Perl program that does not depend on
# any components of the getCal suite.

use strict;
use warnings;
use Getopt::Long;
use LWP::UserAgent;
use HTTP::Request;

use constant EXIT_SUCCESS => 0;
use constant EXIT_NO_DATA => 1;
use constant EXIT_ERROR => -1;

# Boolean options
my ($opt_catlist, $opt_ddlist, $opt_help, $opt_titles);

my $opt_catalog;
my $opt_colnames = "*"; # comma-separated list of column names, or * for all
my $opt_dd;
my $opt_coldelim = "|"; # column delimiter
my $opt_objstr;
my $opt_rowdelim = "\n"; # row delimiter
my $opt_server = 'irsa.ipac.caltech.edu';
my $opt_url = 0;
my $opt_verbose = 0;
my $opt_where;
my $opt_within;

# Titles for the catlist table columns. The order here determines output order.
my @CATLIST_COLS = ('archive', 'server', 'database', 'catname', 'ddname', 'desc');
my @DDLIST_COLS = ('archive', 'dd', 'colname', 'dbtype', 'format', 'nulls', 'units', 'desc');

my $helpstr = qq!
irsaminer - command line tool for IRSA

irsaminer queries IRSA and prints the results on stdout in tabular form.
It is essentially a command line version of a subset of the Catalog Holdings
List, Catalog Data Dictionary and Catalog Search services offered at
http://irsa.ipac.caltech.edu/applications/Oasis/svc/index.html.

Usage:
	irsaminer --catlist [ --delim= --server= --titles ]
	irsaminer --ddlist --dd= [ --delim= --server= --titles ]
	irsaminer --catalog= | --dd= [ --cols= --where= --within= --objstr= ]

--catalog specifies the catalog to be queried.  Note that most IRSA catalog
	names are unique, but a couple are not.  It is a bit safer to use --dd.
--catlist prints a list of IRSA catalogs
--cols is either a comma-separated list of names or * for all (the default)
--dd specifies a data dictionary.  Data dictionary names in IRSA are unique.
--ddlist prints the contents of a data dictionary (e.g.,
	operation\@rmt_boulder:irasfscr_dd)
--delim sets the field delimiter; default is $opt_coldelim
--objstr names a reference body for spatial searches
--server specifies the IRSA web server; default is $opt_server
--titles turns on the printing of column titles
--url restricts output to the query URL
--verbose (or -v) turns on verbose output.  Using -vv makes it very verbose.
--where supplies an SQL 'where' clause for the query.
--within supplies a spatial parameter (e.g., '2 degrees') for searching
	around a target (indicated by objstr)
!;

# Main
parse_options();
if ($opt_catlist) {
    print_catlist();
} elsif ($opt_ddlist) {
    print_ddlist();
} else {
    print_search();
}

exit(0);
# End main

sub parse_options {
    Getopt::Long::config("pass_through");
    GetOptions("catalog=s" => \$opt_catalog,
	       "catlist" => \$opt_catlist,
	       "cols=s" => \$opt_colnames,
	       "dd=s" => \$opt_dd,
	       "ddlist" => \$opt_ddlist,
	       "delim=s" => \$opt_coldelim,
	       "h|help" => \$opt_help,
	       "objstr=s" => \$opt_objstr,
	       "rdelim=s" => \$opt_rowdelim,
	       "server=s" => \$opt_server,
	       "titles" => \$opt_titles,
	       "url" => \$opt_url,
	       "v|verbose+" => \$opt_verbose,
	       "where=s" => \$opt_where,
	       "within=s" => \$opt_within,
	       );
      if ($opt_help
	  || !($opt_catlist || $opt_ddlist || $opt_catalog || $opt_dd)) {
	  print STDERR $helpstr;
	  exit(EXIT_ERROR);
      }
}

sub print_catlist {
    if ($opt_url) {
	print catlist_url();
    } else {
	print_table(catlist_as_table(), \@CATLIST_COLS);
    }
}

sub print_ddlist {
    if ($opt_url) {
	print ddlist_url(cd_args());
    } else {
	print_table(ddlist_as_table(cd_args()), \@DDLIST_COLS);
    }
}

sub print_search {
    my $irsa_url = irsa_search_url(cd_args(), $opt_colnames, $opt_where);
    if ($opt_url) {
	print $irsa_url;
	return;
    }
    my $irsa_results = get_url($irsa_url);
    print STDERR "$irsa_results\n" if ($opt_verbose > 1);
    if ($irsa_results =~ /^ERROR/) {
	handle_error($irsa_results);
    }
    my @lines = split "\n", $irsa_results;
    my (@title_lines, @data_lines);
    foreach my $line (@lines) {
	if ($line =~ m/^[\\\|]/) { # title lines start with backslash or pipe
	    push @title_lines, $line;
	} else { # data line
	    $line =~ s/^\s+//; # Remove leading space
	    $line =~ s/\s+/$opt_coldelim/g; # Replace other spaces with delim
	    push @data_lines, $line;
	}
    }
    if ($opt_titles) {
	print join($opt_rowdelim, @title_lines);
	print $opt_rowdelim;
    }
    print join($opt_rowdelim, @data_lines);
    print $opt_rowdelim;
}

# Distinguish between empty results and a bona fide error message from IRSA
sub handle_error {
    my $errtext = shift;
    if ($errtext =~ m/found no data/) {
	exit(EXIT_NO_DATA);
    } else {
	print STDERR "irsaminer: query returned error:\n$errtext\n";
	exit(EXIT_ERROR);
    }
}

# Returns a (param, value) pair for either catalog or data dictionary
sub cd_args {
    my ($param_name, $param_val);
    if ($opt_catalog) {
	$param_name = 'catname';
	$param_val = $opt_catalog;
    } elsif ($opt_dd) {
	$param_name = 'ddname';
	$param_val = $opt_dd;
    } else {
	die "No catalog or data dictionary specified for search";
    }
    return ($param_name, $param_val);
}

sub catlist_url {
    return sprintf("http://%s/cgi-bin/Oasis/CatList/nph-catlist?", $opt_server);
}

# Returns the IRSA catalog list as an XML document
sub catlist_as_xml {
    return get_xml_url(catlist_url());
}

# Returns the IRSA catalog list as a reference to a 2-D array
sub catlist_as_table {
    my $xml = catlist_as_xml();
    return xml2array($xml, 'catalog', \@CATLIST_COLS);
}

sub ddlist_url {
    my $param_name = shift;
    my $param_val = shift;
    my ($server, $database, undef, $dd) = sdcdd_lookup($param_name, $param_val);
    return sprintf("http://%s/cgi-bin/Oasis/DD/nph-dd?server=%s&database=%s&datadict=%s&short=0", $opt_server, $server, $database, $dd);
}

# Returns XML text describing a data dictionary
sub ddlist_as_xml {
    return get_xml_url(ddlist_url(@_));
}

# Returns a 2-D array describing a data dictionary
sub ddlist_as_table {
    my $dd_xml = ddlist_as_xml(@_);
    my $ddrows = xml2array($dd_xml, 'column', \@DDLIST_COLS);
    return $ddrows;
}

# Return the URL for an IRSA catalog search
sub irsa_search_url {
    my ($param_name, $param_val, $columns, $where) = @_;
    my ($server, $database, $catalog) = sdcdd_lookup($param_name, $param_val);
    my $url = sprintf("http://%s/cgi-bin/Oasis/CatSearch/nph-catsearch?server=%s&database=%s&catalog=%s", $opt_server, $server, $database, $catalog);

    my $sql = sprintf("select %s from %s", $columns, $catalog);
    if ($where) {
	$sql .= " where " . $where;
    }
    $url = add_arg($url, "sql", $sql);
    if ($opt_within) {
	$url = add_arg($url, "within", $opt_within);
	$url = add_arg($url, "objstr", $opt_objstr);
    }
    return $url;
}

# Append a filtering argument to a query URL
sub add_arg {
    my $url = shift;
    my $argname = shift;
    my $argval = shift;
    $argval =~ s/\s/\+/g;
    return sprintf("%s&%s=%s", $url, $argname, $argval);
}

# Given a catalog name or data dictionary name, return the appropriate
# server, database, catalog and data dictionary
# Arguments:
# 0: Either 'catalog' or 'ddname'
# 1: Name of the catalog or data dictionary being sought
sub sdcdd_lookup {
    my $param_name = shift;
    my $param_val = shift;
    my $catlist = catlist_as_table();
    my $ddeq = sub {
	return ($_[0]->[list_index($_[1], \@CATLIST_COLS)] eq $_[2]);
	};
    my $rows = find_rows($catlist, $ddeq, $param_name, $param_val);
    die "Entry not found for $param_name = $param_val\n" if ($#$rows < 0);
    print STDERR "Warning: Multiple entries found for $param_name = $param_val\n" if ($#$rows);
    return ($rows->[0][list_index('server', \@CATLIST_COLS)],
	    $rows->[0][list_index('database', \@CATLIST_COLS)],
	    $rows->[0][list_index('catname', \@CATLIST_COLS)],
	    $rows->[0][list_index('ddname', \@CATLIST_COLS)]);
}

# Convert a 2-level XML document to a 2-D Perl array.
# The XML document must fit the pattern of a root node containing one or more
# sub-elements.  This subroutine finds first-level elements of a given name,
# and from those elements, extracts the values of second-level elements whose
# names reside in a list of columns.  This explanation might sound a bit
# confusing, but both the catalog list and the data dictionary XML documents
# returned by IRSA fit this pattern.  If you look at those, you'll probably
# get a good idea of what this function does.
# Arguments:
# 0: XML document to be parsed
# 1: Name of first-level elements
# 2: Reference to an array of 2nd-level elements to be extracted
sub xml2array {
    my $xml = shift;
    my $sub1_name = shift;
    my $col_headers = shift;
    my @rows;
    my @tagstack;
    my $root_attrhash;
    my $colvals;
    while ($xml =~ /<(\/?[[:alpha:]_][^>]*)>([^<]*)/g) {
	my $tag = $1;
	my $tagtext = $2;
	$tagtext =~ s/(^\s+)|(\s+$)//g;
	$tag =~ /(\/?[^[:space:]]+)(([[:space:]]+[[:alnum:]]+=((\"[^\"]*\")|(\'[^\']*\')))*)/;
	my $tagname = $1;
	my $attrlist = $2;
	if (substr($tagname, 0, 1) eq "/") { # Closing tag
	    if (substr($tagname, 1) ne $tagstack[$#tagstack]) {
		die sprintf("Parse error: expected /%s but got %s",
			    $tagstack[$#tagstack], $tagname);
	    }
	    pop @tagstack;
	} else { # Opening tag
	    push @tagstack, $tagname;
	    if (1 == scalar @tagstack) {
		$root_attrhash = attribute_hash($attrlist);
	    } elsif (2 == scalar @tagstack && $tagname eq $sub1_name) {
		my @newarray = ();
		$colvals = \@newarray;
		push @rows, $colvals;
		for my $key (keys %$root_attrhash) {
		    my $lidx = list_index($key, $col_headers);
		    $colvals->[$lidx] = $root_attrhash->{$key} if ($lidx >= 0);
		}
	    } elsif (3 == scalar @tagstack) {
		my $lidx = list_index($tagname, $col_headers);
		$colvals->[$lidx] = $tagtext if ($lidx >= 0);
	    }
	}
    }
    return \@rows;
}

###############################################################################
# Low-level utility routines
###############################################################################

# Convert a string of XML tag attributes to a hash
sub attribute_hash {
    my $attrlist = shift;
    $attrlist =~ s/(^\s+)|(\s+$)//g;
    my @attrs = split /\s+/, $attrlist;
    my %ahash;
    foreach my $attr (@attrs) {
	my ($key, $value) = split '=', $attr;
	$value =~ s/(^\s*[\'\"])|(\s*[\'\"]\s*)//g;
	$ahash{$key} = $value;
    }
    return \%ahash;
}

# Given a 2-D array, extract rows from that table
# Arguments:
# 0: Reference to the table array
# 1: Reference to a subroutine that returns True for match and False for no match
# 2-*: Any additional arguments are passed verbatim to the test subroutine
sub find_rows {
    my $table = shift;
    my $test = shift;
    my $r = 0;
    my @matchrows;
    while ($table->[$r]) {
	if ($test->($table->[$r], @_)) {
	    push @matchrows, $table->[$r];
	}
	$r++;
    }
    return \@matchrows;
}


# Given an array of strings, return the index of the first element matching
# a particular value.
# Arguments:
# 0: The value sought
# 1: Array reference
sub list_index {
    my $str = shift;
    my $a = shift;
    my $idx = -1;
    for (my $i = 0; $i <= $#$a && $idx == -1; ++$i) {
	$idx = $i if ($str eq $a->[$i]);
    }
    return $idx;
}

# Print a table of data
# Arguments:
# 0: 2-D array of data
# 1: 1-D array of titles
# 2: File handle; if none given, defaults to STDOUT
sub print_table {
    my $body = shift;
    my $headers = shift;
    my $file = shift;
    if ($headers && $opt_titles) {
	my $title = join($opt_coldelim, @{$headers}) . $opt_rowdelim;
	if ($file) {
	    print $file $title;
	} else {
	    print $title;
	}
    }
    my $r = 0;
    while ($body->[$r]) {
	my $line = join($opt_coldelim, @{$body->[$r]}) . $opt_rowdelim;
	if ($file) {
	    print $file $line;
	} else {
	    print $line;
	}
	++$r;
    }
}

# Get an XML document from a URL; clean it before returning
sub get_xml_url {
    my $url = shift;
    my $xml = get_url($url);
    $xml =~ s/&nbsp;//g;
    return $xml;
}

# Given a URL, return its content
sub get_url {
    my $url = shift;
    print STDERR "Fetch URL:\n$url\n" if ($opt_verbose);
    my $agent = LWP::UserAgent->new();
    $agent->env_proxy();  # try to use HTTP_PROXY if avail
    my $req = new HTTP::Request(GET => $url);
    my $response = $agent->request($req);
    if ($response->is_success) {
	print "HTTP response content:\n" . $response->content() . "\n" if ($opt_verbose >= 2);
	return $response->content();
    }
    die "Error retrieving $url\n" . $response->status_line . "\n\n";
}
