#! @PERL@
# -*- perl -*-
# @configure_input@

our $MODVERSION = '$Id: legacyIRSAQuery.pl,v 1.12 2007/01/29 20:40:42 sgroom Exp $';

# This program exists solely for backwards compatibility with getCal 2.7.
# It is a back end for the "legacy" IRASQuery and 2MASSQuery scripts.
# No other getCal tools depend on these three scripts, so they can be safely
# removed from the project once it is clear that nobody needs the legacy stuff
# anymore.
#
# The replacements for the legacy tools are irsaminer and gcIrsa.pm,
# which allow for much greater flexibility than the old tools.

use strict;
use warnings;
use FindBin qw($RealBin);
use lib "$RealBin/../lib/modules";		# location of getCal Perl modules
use gcConfig;
use miscString;
use gcIrsa;
use Getopt::Long;
use coordinates;
use miscArgs;

my $cfg = gcConfig::config();

my ($opt_browser, $opt_coord, $opt_raw, $opt_extended, $opt_stdin, $opt_photometry, $opt_error) = (0,0,0,0,0,0,0);
my ($opt_tool, @opt_filenames, $opt_searchRA, $opt_searchDEC);
my $opt_radius = 3;

my $helpFileName;

# IRAS options
my ($opt_f12, $opt_f25, $opt_f60, $opt_f100) = (1,0,0,0);

# 2Mass options
my ($opt_kmag, $opt_hmag, $opt_jmag) = (1,0,0);

# Main
parse_options();
my @targets = getArgs($opt_stdin, \@opt_filenames);
if (scalar @targets) { # query targets by name
    foreach my $target (@targets) {
	query_for_target(search_args($target), $target);
    }
} else { # query target by its coordinates
    my $args = search_args();
    if (!$args)
    {
	# print help message and exit
	helpFile(1,$helpFileName);
	exit(1);
    }
    my $target = sprintf("%s+%s", $args->{targetRA}, $args->{targetDEC});
    query_for_target($args, $target);
}
exit(0);
# End main

sub parse_options {
    Getopt::Long::config("pass_through");
    GetOptions("browser" => \$opt_browser,
	       "coord" => \$opt_coord,
	       "data|allData" => \$opt_raw,
	       "error" => \$opt_error,
	       "extended" => \$opt_extended,
	       "filename=s" => \@opt_filenames,
	       "radius|searchRadius=s" => \$opt_radius,
	       "searchRA=s" => \$opt_searchRA,
	       "searchDEC=s" => \$opt_searchDEC,
	       "stdin" => \$opt_stdin,
	       "tool=s" => \$opt_tool,

	       # IRAS options
	       "f12" => \$opt_f12,
	       "f25" => \$opt_f25,
	       "f60" => \$opt_f60,
	       "f100" => \$opt_f100,

	       # 2Mass options
	       "Kmag" => \$opt_kmag,
	       "Jmag" => \$opt_jmag,
	       "Hmag" => \$opt_hmag,
	       "photometry" => \$opt_photometry,
	       );
    $helpFileName = sprintf("%sHelp.txt", $opt_tool);
    handleCommonOpts(0, $helpFileName, $helpFileName);
    $opt_jmag = $opt_hmag = 1 if ($opt_photometry);
    $opt_f25 = $opt_f60 = $opt_f100 = 1 if ($opt_photometry);
}

sub search_args {
    my $target = shift;
    my $catalog;
    my @cols;

    if ($opt_tool eq "IRASQuery") {
	$catalog = "iraspsc";
	@cols = qw/ra dec fnu_12 relunc_12 fnu_25 relunc_25 fnu_60 relunc_60 fnu_100 relunc_100/;
	if ($opt_extended) {
	    push @cols, qw/pscname major minor posang tsnr_12 tsnr_25 tsnr_60 tsnr_100/;
	}
    } elsif ($opt_tool eq "2MassQuery") {
	$catalog = "fp_psc";
	@cols = qw/ra dec k_m k_cmsig j_m j_cmsig h_m h_cmsig/;
	if ($opt_extended) {
	    push @cols, qw/designation k_msigcom k_snr j_msigcom j_snr h_msigcom h_snr/;
	}
    } else {
	die "Invalid --tool value: $opt_tool";
    }

    my %opts = (catalog => $catalog,
		cols => \@cols,
		radius => $opt_radius,
		);
    if (defined($target)) {
	$opts{target} = $target;
    } elsif ((defined $opt_searchRA) && (defined $opt_searchDEC)) {
	$opts{targetRA} = $opt_searchRA;
	$opts{targetDEC} = $opt_searchDEC;
    } else {
	# no targets and no search coords, we can't continue.
	return undef;
    }

    if ($opt_raw) {
	$opts{delim} = "\t";
	$opts{titles} = 1;
    }
    return \%opts;
}

sub query_for_target {
    my $args = shift;
    my $target = shift;
    my $results = query_irsa($args);
    output_results($target, $results) if (defined($results));
    print "\n";
}

sub query_irsa {
    my $opts = shift;
    if ($opt_browser) {
	my $cmd = irsaminer_command($opts) . " --url";
	my $url = `$cmd`;
	die "Command failed: $cmd" if ($?);
	system(sprintf("%s \"%s\" &", $cfg->browser(), $url));
	return;
    } elsif ($opt_raw) {
	my $cmd = irsaminer_command($opts);
	die "Command failed: $cmd" if (system($cmd));
	return;
    }
    return grab_irsa($opts);
}

sub output_results {
    my $target = $_[0];
    my $r = $_[1]->[0]; # First and only result row
    printf "# %s Search %s: ", substr($opt_tool, 0, length($opt_tool) - 5), $target;
    output_coord($r) if ($opt_coord);
    output_iras($r) if ($opt_tool eq "IRASQuery");
    output_2mass($r) if ($opt_tool eq "2MassQuery");
    print "\n";
}

sub output_coord {
    my $r = shift;
    printf(" %s %s ", RAhms($r->{ra}, 1), DECdms($r->{dec}, 1));
}

sub output_iras {
    my $r = shift;
    outval("fnu_12", $r->{fnu_12}, $r->{relunc_12}) if ($opt_f12);
    outval("fnu_25", $r->{fnu_25}, $r->{relunc_25}) if ($opt_f25);
    outval("fnu_60", $r->{fnu_60}, $r->{relunc_60}) if ($opt_f60);
    outval("fnu_100", $r->{fnu_100}, $r->{relunc_100}) if ($opt_f100);
}

sub output_2mass {
    my $r = shift;
    outval("H", $r->{h_m}, $r->{h_cmsig}) if ($opt_hmag);
    outval("J", $r->{j_m}, $r->{j_cmsig}) if ($opt_jmag);
    outval("K", $r->{k_m}, $r->{k_cmsig}) if ($opt_kmag);
}

sub outval {
    my ($tag, $val, $verr) = @_;
    printf(" %s = %g", $tag, $val);
    printf(" +/- %g", $verr) if ($opt_error);
    print " ";
}
