#! @PERL@
# -*- perl -*-
# @configure_input@

our $MODVERSION = '$Id: makeHipIdx.pl,v 1.10 2008/11/12 23:28:59 sgroom Exp $';

##########################################################################
# NASA Exoplanet Science Institute (NExScI) -- http://nexsci.caltech.edu #
#       Copyright (c) 1997-2008 California Institute of Technology       #
#                     U.S. Government Sponsorship                        #
# Under contract with the National Aeronautics and Space Administration  #
##########################################################################

# makeHipIdx.pl
# Utility to create indexes for the Hipparcos catalog, facilitating
# speedy retrieval of Hipparcos records by HIP, HD, or DM designation
#
# Each index file is represented as a tied hash
# using the HD/HIP/DM designation
# as the key, and the byte offset of that Hipparcos record as the value.
#
# The index file is stored using the DB_File Perl package, which
# creates Berkeley DB format database file.
#
# The index files created with this utility are used by the
# perl module hipRecord.pm
#
# 3/18/2006
# Steve Groom

use strict;
use warnings;
use FindBin qw($RealBin);
use lib "$RealBin/../lib/modules";		# location of getCal Perl modules
use Getopt::Long;
use gcConfig;
use hipparcosKeys;
use miscString;
use astroCatalogue;

BEGIN
{
    my $HAVE_DB_FILE = eval("use DB_File; 1");
    if (!$HAVE_DB_FILE)
    {
	die("makeHipIdx: required Perl module DB_File not found, please install it.");
    }
}



# initialize
my $cfg = config();
my $idxpath = $cfg->hipparcosIdxPath();
my $debug = 0;

# process command options
my $result = GetOptions("d|debug+" => \$debug);
optDebug($debug);

open CATALOG, $cfg->hipparcosCatalog()
	or die "Can't open Hipparcos catalog " . $cfg->hipparcosCatalog() . "\n";

my $idxHdName = "$idxpath/idx_hd.db";		# the HD index file
my $idxHipName = "$idxpath/idx_hip.db";		# the HIP index file
my $idxDmName = "$idxpath/idx_dm.db";		# the DM index file
my $DB_FORMAT_VERSION = "1.0";                  # version of the DB format
my $DB_FORMAT_VERSION_TAG = "__DB_FORMAT_VERSION__";    # tag for stashing version info in the DB itself


my %IDXHD;
my %IDXHIP;
my %IDXDM;

# delete the db's if they already exist, we are going to recreate them
unlink $idxHdName;
unlink $idxHipName;
unlink $idxDmName;

# open the db files and tie them to perl hashes
print "Creating Hipparcos HD index file $idxHdName\n";
tie (%IDXHD, "DB_File", $idxHdName, O_RDWR|O_CREAT, 0666, $DB_HASH)
    or die "Can't create Hipparcos-HD index file $idxHdName\n";
print "Creating Hipparcos HIP index file $idxHipName\n";
tie (%IDXHIP, "DB_File", $idxHipName, O_RDWR|O_CREAT, 0666, $DB_HASH)
    or die "Can't create Hipparcos-HIP index file $idxHipName\n";
print "Creating Hipparcos DM index file $idxDmName\n";
tie (%IDXDM, "DB_File", $idxDmName, O_RDWR|O_CREAT, 0666, $DB_HASH)
    or die "Can't create Hipparcos-DM index file $idxDmName\n";

print "Building index files, this will take a few minutes...\n";

# Insert a format version info tag into each db, to be used on
# the lookup side to detect format changes if they happen.
$IDXHD{$DB_FORMAT_VERSION_TAG} = $DB_FORMAT_VERSION;
$IDXHIP{$DB_FORMAT_VERSION_TAG} = $DB_FORMAT_VERSION;
$IDXDM{$DB_FORMAT_VERSION_TAG} = $DB_FORMAT_VERSION;

# process each entry in the hipparcos catalog, creating entries in
# each of the HD/HIP/DM hashes if the record has that type of ID
# Store the byte offset of the record from the original hipparcos catalog
# as the index, for use in quickly seeking to that record.
my $line;
my $counthd = 0;
my $counthip = 0;
my $countdm = 0;
my $recno = 0;
my $pos = tell CATALOG; # remember the file offset BEFORE the read!
while ($line = <CATALOG>)
{
    my @contents = split /\|/, $line;
    my $hd = trimWhite($contents[$HIP_HDnum]);
    my $hip = trimWhite($contents[$HIP_Num]);
    my $dm = trimWhite($contents[$HIP_DMname]);
    if ($dm)
    {
	$dm =~ s/[ ]+/_/g;	# replace any remaining spaces with underscores
    }

    if ($hd)
    {
	print STDERR "rec $recno indexed as HD $hd at pos $pos\n" if $cfg->debug();
	$IDXHD{$hd} = $pos;
	$counthd++;
    }
    if ($hip)
    {
	print STDERR "rec $recno indexed as HIP $hip at pos $pos\n" if $cfg->debug();
	$IDXHIP{$hip} = $pos;
	$counthip++;
    }
    if ($dm)
    {
    	print STDERR "rec $recno indexed as DM $dm at pos $pos\n" if $cfg->debug();
    	$IDXDM{$dm} = $pos;
    	$countdm++;
    }
    $recno++;
    $pos = tell CATALOG;
}
untie %IDXHD;
untie %IDXHIP;
untie %IDXDM;

print "Wrote $counthd records to Hipparcos-HD index file $idxHdName\n";
print "Wrote $counthip records to Hipparcos-HIP index file $idxHipName\n";
print "Wrote $countdm records to Hipparcos-DM index file $idxDmName\n";
