package angularDiameters;
my $MODVERSION = '$Id: angularDiameters.pm,v 1.6 2008/11/12 23:28:22 sgroom Exp $';
use strict;
use warnings;
require 5.000;

our @ISA = qw(Exporter);
our @EXPORT = qw(angDiamET);



##########################################################################
# NASA Exoplanet Science Institute (NExScI) -- http://nexsci.caltech.edu #
#       Copyright (c) 1997-2008 California Institute of Technology       #
#                     U.S. Government Sponsorship                        #
# Under contract with the National Aeronautics and Space Administration  #
##########################################################################

# Angular diameter subroutines.  

use effectiveTemperature;
use bolometricCorrection;
use gcConfig;

my $cfg = gcConfig::config();

#########################################################################
## angDiamET -- estimate angular diameter by effective temp means
#########################################################################
# This routine estimates apparent diameters from spectral type,
# v-mag, distance (for extinction correction) and a little
# thermodynamics.
#
# Parameters:
#	spType
#	vMag
#	distance
#
# Normally returns:
#	estDiamET - estimated diameter based on effective temperature
#	mBol - bolometric magnitude of object
#	fBol - bolometric flux of object
#	eT - effective temperature of object
#
# If any of the above values cannot be provided, that value will
# returned as undef.
#
sub angDiamET
{
    my $spType = shift @_;
    my $vMag = shift @_;
    my $dist = shift @_;

    # return values with defaults
    my $estDiamET = undef;
    my $mBol = undef;
    my $fBol = undef;
    my $eT = undef;

    # get bolCorr
    my $bc = &bolCorr($spType);
    if (!defined $bc)
    {
	print STDERR "DEBUG: angularDiameters::angDiamET: bolCorr failed for spType \"$spType\"\n" if ($cfg->debug());
    }

    # get effTemp
    $eT = &effTemp($spType);
    if (!defined $eT)
    {
	print STDERR "DEBUG: angularDiameters::angDiamET: effTemp failed for spType \"$spType\"\n" if ($cfg->debug());
    }

    # if we got both bolCorr and effTemp, proceed with mbol, fbol and estDiamET
    if ((defined $bc) && (defined $eT))
    {
	# Extinction correction: V-band extinction (8e-4 mag/pc) times the
	# distance in pc.
	my $redCorrection = 8.0e-4 * $dist;

	# Compute the bolometric magnitude and flux of the object from the
	# V magnitude.  Units of bolometric flux are W cm^-2 (yes, I know
	# they are mixed units, talk to Gerard).  (2.48e-12 is the
	# bolometric flux for a mBol = 0 source.)  
	$mBol = $vMag + $bc - $redCorrection;
	$fBol = (2.48e-12) * 10**(- $mBol / 2.5);

	# Calculate the estimated angular diameter (form from Dyck, van
	# Belle, and Benson, AJ 112, 294).  This is really just the
	# definition of effective temperature and a black-body model.
	$estDiamET = (1.316e7 / $eT)**2 * sqrt($fBol);
    }
    return ($estDiamET,$mBol,$fBol,$eT);
} ## end angDiamET  #####################################################

return 1;
