package astroCatalogue;
my $MODVERSION = '$Id: astroCatalogue.pm,v 1.15 2007/04/23 17:52:04 sgroom Exp $';

use strict;
use warnings;
use miscString;
use gcConfig;

use Exporter();

our @ISA = qw(Exporter);
our @EXPORT = qw(TARGET_SUCCESS TARGET_MISNAMED
		 TARGET_NOT_FOUND BAD_PARAMETER HTTP_ERROR
		 identPieces stdCatalogue normalIdent
		 needTargetNameChange sansSpaces cmpIdentPieces
		 identIsInCatalogue mangleIdent
		 );

my $cfg = gcConfig::config();

# Exit codes for catalogue searches
use constant TARGET_SUCCESS => 0;
use constant TARGET_NOT_FOUND => -1;
use constant BAD_PARAMETER => -2;
use constant HTTP_ERROR => -3; # network timeout, web server down, etc.
use constant TARGET_MISNAMED => 1;

# Order two arrays of identifier pieces
sub cmpIdentPieces
{
    my $pL = shift;
    my $pR = shift;
    my $cval = (uc($pL->[0]) cmp uc($pR->[0]));
    if (!$cval) {
	if (defined($pL->[1]) && !defined($pR->[1])) {
	    $cval = 1;
	} elsif (!defined($pL->[1]) && defined($pR->[1])) {
	    $cval = -1;
	} elsif (isNumeric($pL->[1]) && isNumeric($pR->[1])) {
	    $cval = ($pL->[1] <=> $pR->[1]);
	} else {
	    $cval = (uc($pL->[1]) cmp uc($pR->[1]));
	}
    }
    if (!$cval) {
	if (defined($pL->[2]) && !defined($pR->[2])) {
	    $cval = 1;
	} elsif (!defined($pL->[2]) && defined($pR->[2])) {
	    $cval = -1;
	} else {
	    $cval = (uc($pL->[2]) cmp uc($pR->[2]));
	}
    }
    return $cval;
}

# Given an identifier name, break it into an array containing
# (acronym, sequence, specifier).  Be reasonably smart about doing so,
# but don't alter the string other than breaking it apart.
sub identPieces
{
    my $target = shift;
    my ($acronym, $sequence, $specifier) = ("", "", "");
    my @pieces;
    my $tmp = '';
    ($target, @pieces) = _breakApart($target);
    if (scalar(@pieces) == 1) {
	my @inner0 = ();
	my @inner1 = ();
	if ($target =~ m/^([pP]\. +)(.+)$/)
	{
	    ($tmp, @inner0) = _breakApart($2);
	    @pieces = @inner0;
	    $pieces[0] = "p. " . $inner0[0];
	}
	elsif ($target =~ m/^com +([^=]+)=([^=]+)$/)
	{
	    my ($t0, $t1) = ($1, $2);
	    ($tmp, @inner0) = _breakApart($t0);
	    ($tmp, @inner1) = _breakApart($t1);
	    if (cmpIdentPieces(\@inner0, \@inner1) <= 0)
	    {
		@pieces = @inner0;
	    }
	    else
	    {
		@pieces = @inner1;
	    }
	}
    }

    $acronym = $pieces[0];
    $sequence = $pieces[1] if defined($pieces[1]);
    $specifier = $pieces[2] if defined($pieces[2]);
    return ($acronym, $sequence, $specifier)
}

sub _breakApart
{
    my $target = trimWhite(shift);
    study $target;

    my @pieces = ();
    # Check for the "standard" naming convention as outlined at
    # http://vizier.u-strasbg.fr/Dic/iau-spec.htx
    if ($target =~ m/^([[:alnum:]]+[\#]?)[[:space:]_]+([[:alnum:]\:\+\-\.]*)(([[:space:]_]+[[:^space:]]+)*)$/)
    {
	push @pieces, ($1, $2, trimWhite($3));
    }
    # Recognize a misstated but still clear form like 'HD174638', 'HD174638A' or 'G148-13'
    elsif ($target =~ m/^([[:alpha:]]+)([[:digit:]\:\+\-\.]+)([[:space:]]?[[:alpha:]]?)$/)
    {
	push @pieces, ($1, $2 . trimWhite($3));
    }
    # Recognize a form like 'BD+33 3223' or 'CSI+33 3223 1'
    elsif ($target =~ m/^([[:alpha:]]+[\+\-][[:digit:]]+)[[:space:]_]+([[:digit:]]+)(([[:space:]_]+[[:digit:]]+)*)$/)
    {
	push @pieces, ($1, $2, trimWhite($3));
    }
    # CD-31 3520B
    elsif ($target =~ m/^([[:alnum:]\-\+]+)[[:space:]]([[:digit:]]+)([[:alpha:]]?)$/)
    {
	push @pieces, ($1, $2 . $3);
    }
    # [SC93b] I 209
    elsif ($target =~ m/^(\[[^[:space:]]+\])(( +[[:alpha:]])?) +([[:digit:]]+)/)
    {
	push @pieces, ($1 . $2, $4);
    }
    # [M60b]+36 72
    elsif ($target =~ m/^(\[[^[:space:]]+\])([[:digit:][:space:]\+\-]+)([[:alpha:]]?)$/)
    {
	push @pieces, ($1, $2, $3);
    }
    # XXX J01234.5-6789
    elsif ($target =~ m/^([[:alnum:]]+) (J[[:digit:]\.\-\+]+)$/)
    {
	push @pieces, ($1, $2);
    }
    # V* V362 Per
    elsif ($target =~ m/^([[:alpha:]]+\*) +([[:alnum:]]+( [[:alnum:]]+)?)/)
    {
	push @pieces, ($1, $2);
    }
    else
    {
	push @pieces, $target;
    }
    return ($target, (@pieces));
}

# Return the "standard" name for a catalogue, or else return the input
# string if it isn't recognized as a name needing transformation.
sub stdCatalogue
{
    # Table of catalogue substitutions. Identifiers to be found are the keys;
    # their replacements are the referent elements.
    my %catSubst = ("HDC" => "HD");
    my $catName = shift;
    my $retName = $catSubst{$catName};
    if (!$retName) {
	$retName = $catName;
    }
    return $retName;
}

# Normalize an identifier by inserting spaces, capitalizing letters, and
# standardizing catalogue name
sub normalIdent
{
    my $ident = shift;

    # Replace underscores with spaces
    $ident =~ tr/_/ /;

    my ($acronym, $sequence, $specifier) = identPieces($ident);
    $acronym = stdCatalogue($acronym);
    my $normalName = "";
    if ($specifier) {
	$normalName = join(' ', $acronym, $sequence, $specifier);
    } elsif ($sequence) {
	$normalName = join(' ', $acronym, $sequence);
    } else {
	$normalName = $ident;
	#print STDERR "Couldn't normalize identifier: $normalName\n";
    }
    return uc($normalName);
}

sub identIsInCatalogue {
    my $ident = shift;
    my $catExpr = shift;
    return ($ident =~ m/^($catExpr)/);
}

# Sometimes the result of a catalogue query (e.g., Simbad) returns an
# identifier that doesn't match the one requested.  This function returns
# true if the output identifier should be preferred over the search
# criterion.
# Parameters:
# inTarget - the search target
# outTarget - the identifier returned from the catalogue
# catPrefs - a pipe-delimited list of catalogues that are "preferred".
#     If inTarget is is one of those catalogues but outTarget is not,
#     the return value will be 0.
sub needTargetNameChange {
    my $inTarget = normalIdent(shift);
    my $outTarget = normalIdent(shift);
    my $catPrefs = shift || "HD|HIP|DM";

    return ($inTarget ne $outTarget
	    && (&identIsInCatalogue($outTarget, $catPrefs)
		|| !&identIsInCatalogue($inTarget, $catPrefs)));
}

# Normalize an identifier and replace each space with an underscore
# or with a delimiter of the caller's choosing
sub sansSpaces {
    my $inIdent = shift;
    my $delim = shift;
    if (!($delim)) {
	$delim = '_';
    }
    my @pieces;
    @pieces = identPieces($inIdent) if ($inIdent);
    my $outIdent = "";
    foreach my $piece (@pieces) {
	$outIdent .= $delim if ($outIdent && $piece);
	$outIdent .= $piece;
    }
    return $outIdent;
}

# Given a catalogue identifier, clean it up to prepare it for output.
# This may include translation of "HD" into "HDC" according to config settings.
sub mangleIdent {
    my $ident = normalIdent(shift);
    my %subst = ();
    if ($cfg->identHDC()) {
	$subst{HD} = "HDC";
    }
    
    my ($acronym, $sequence, $specifier) = identPieces($ident);
    my $newacr = $subst{$acronym};
    if ($newacr) {
	$acronym = $newacr;
    }
    return $acronym . $sequence . $specifier;
}
