package baseline;
my $MODVERSION = '$Id: baseline.pm,v 1.5 2008/11/12 23:28:22 sgroom Exp $';
use strict;
use warnings;

require 5.000;


##########################################################################
# NASA Exoplanet Science Institute (NExScI) -- http://nexsci.caltech.edu #
#       Copyright (c) 1997-2008 California Institute of Technology       #
#                     U.S. Government Sponsorship                        #
# Under contract with the National Aeronautics and Space Administration  #
##########################################################################

use POSIX;

use angles;  ## angle constants
use vector;  ## vector utilities
use geometry; ## geometry utilities



## baseline class to hold data structures and methods pertaining to
## baseline computations



#########################################################################
#  baseline class Constructor                                           #
#########################################################################
sub new
{
## declarations baseline constructor  ###################################
    my $class = shift @_;  ## yank off the class designation

    ## Create the hash with the rest of the stuff from the call argument
    ##  list
    my $objRef = { @_ };

##  begin baseline constructor  #########################################
    ## Check that required pieces are in place

    ## We require a latitude (input units of degrees)
    if (defined $objRef->{latitude})
    {
	$objRef->{latitude} *= DEG_TO_RAD;  ## convert to radians
    }
    else
    {
	die "Error: latitude must be defined in baseline constructor\n";
    }

    ## Longitude is a niceity (input units of degrees)
    if (defined $objRef->{longitude})
    {
	$objRef->{longitude} *= DEG_TO_RAD;  ## convert to radians
    }


    ## We obviously need a baseline 3-vector (input units meters)
    my @WorkB = ();
    if (defined $objRef->{bVector})  ## came in as a vector
    {
	@WorkB = ($objRef->{bVector}[0],
			$objRef->{bVector}[1],
			$objRef->{bVector}[2]);
	$objRef->{baseline} = \@WorkB;
    }
    elsif ((defined $objRef->{bEast})  ## came in as discrete components
	   && (defined $objRef->{bNorth})
	   && (defined $objRef->{bUp}))
    {
	@WorkB = ($objRef->{bEast},$objRef->{bNorth},$objRef->{bUp});
	$objRef->{baseline} = \@WorkB;
    }
    else  ## otherwise sound the alarm
    {
	die "Error: need baseline 3-vector specification in baseline constructor\n";
    }


    ## Hour angles of delay extrema are independent of star
    ## declination -- who knew?!?  AB 1April2004 (no joke)
    my @baseline = @{ $objRef->{baseline} };
    ## Guaranteed to be in the interval -PI to PI
    my $haExt = atan2($baseline[0],
		      ($baseline[1] * sin($objRef->{latitude})
		       - $baseline[2] * cos($objRef->{latitude})));
    $objRef->{haMin} = $haExt; ## Don't forget we're storing angles in radians

    ## The other extremum time (make certain in principle interval
    $objRef->{haMax} = $haExt + PI;
    $objRef->{haMax} -= TWO_PI if ($objRef->{haMax} > PI);


    ## Finished.  "Objectify" and return
    bless $objRef;
    return $objRef;

##  end baseline constructor  ###########################################
}




#########################################################################
#  uvw member function                                                  #
#########################################################################
sub uvw
{
## delarations uvw  #####################################################
    my $this = shift @_;  ## strip-off class

    my $hourAngle = (shift @_) * DEG_TO_RAD;  ## convert input degrees
    my $targetDec = (shift @_) * DEG_TO_RAD;  ## into radians

    my $lambda = shift @_;

## begin uvw ############################################################
    my @uStar = &unitStar($hourAngle,$targetDec,$this->{latitude});
    my @zHat = (0,cos($this->{latitude}),sin($this->{latitude}));

    my @raV = &cross(@zHat,@uStar);
    @raV = &unit(@raV);
    my @decV = &cross(@uStar,@raV);

    my $u = &dot(@{ $this->{baseline} },@raV);
    my $v = &dot(@{ $this->{baseline} },@decV);
    my $w = &dot(@{ $this->{baseline} },@uStar);

    if (defined $lambda)
    {
	$u /= $lambda;
	$v /= $lambda;
	$w /= $lambda;
    }

    return ($u,$v,$w);
## end uvw ##############################################################
}



#########################################################################
##  delay member function                                               #
#########################################################################
sub delay
{
## delarations delay ####################################################
    my $this = shift @_;  ## strip-off class

    my $hourAngle = (shift @_) * DEG_TO_RAD;  ## convert input degrees
    my $targetDec = (shift @_) * DEG_TO_RAD;  ## into radians

## begin delay ##########################################################
    my @uStar = &unitStar($hourAngle,$targetDec,$this->{latitude});
    my $delay = &dot(@{ $this->{baseline} },@uStar);

    if (defined $this->{delayBias})  ## If this baseline object has delayBias
    {
	$delay += $this->{delayBias};
    }

    return $delay;

## end delay ############################################################
}



#########################################################################
##  hour angle delay extrema member function                            #
#########################################################################
sub haDelayExtrema
{
## declarations haDelayExtrema  #########################################
    my $this = shift @_;  ## strip-off class

## begin haDelayExtrema  ################################################
    ## This one's trivial -- we did the work in the constructor
    return ($this->{haMin} * RAD_TO_DEG,
	    $this->{haMax} * RAD_TO_DEG)

} ## end haDelayExtrema  ################################################



#########################################################################
##  delay extrema member function                                       #
#########################################################################
sub delayExtrema
{
## declarations delayExtrema  ###########################################
    my $this = shift @_;  ## strip-off class
    
    my $targetDec = shift @_;  ## DO NOT convert into radians (delay does)


## begin delayExtrema  ##################################################

    my $delayMin = $this->delay($this->{haMin} * RAD_TO_DEG,
				$targetDec);

    my $delayMax = $this->delay($this->{haMax} * RAD_TO_DEG,
				$targetDec);

    ## Make certain we have the min/max "parity" right...
    if ($delayMin <= $delayMax)
    {
	return ($this->{haMin} * RAD_TO_DEG,$delayMin,
		$this->{haMax} * RAD_TO_DEG,$delayMax);
    }
    else ## Otherwise we've the parity wrong -- fix
    {
	return ($this->{haMax} * RAD_TO_DEG,$delayMax,
		$this->{haMin} * RAD_TO_DEG,$delayMin);
    }

} ## end delayExtrema  ##################################################





return 1;




