package bolometricCorrection;
my $MODVERSION = '$Id: bolometricCorrection.pm,v 1.9 2008/11/12 23:28:22 sgroom Exp $';
use strict;
use warnings;

require 5.000;

use Exporter();

our @ISA = qw(Exporter);
our @EXPORT = qw(bolCorr);


##########################################################################
# NASA Exoplanet Science Institute (NExScI) -- http://nexsci.caltech.edu #
#       Copyright (c) 1997-2008 California Institute of Technology       #
#                     U.S. Government Sponsorship                        #
# Under contract with the National Aeronautics and Space Administration  #
##########################################################################


use tableInterpolate;
use parseSpType;
use gcConfig;

my $cfg = gcConfig::config();


# Associative array of bolometric correction
my %BolCorrTable = (

# June 2006: RMG updated using Allen 4th edition

 # Main sequence -- from Allen

    	"O5V" => -4.4,
	"O9V" => -3.33,     	

    	"B0V" => -3.16,
	"B2V" => -2.35,
    	"B5V" => -1.46,
	"B8V" => -0.80,

    	"A0V" => -0.30,
	"A2V" => -2.00,
    	"A5V" => -0.15,

    	"F0V" => -0.09,
	"F2V" => -0.11,
    	"F5V" => -0.14,
	"F8V" => -0.16,

    	"G0V" => -0.18,
	"G2V" => -0.20,
    	"G5V" => -0.21,
	"G8V" => -0.40,

    	"K0V" => -0.31,
	"K2V" => -0.42,
    	"K5V" => -0.72,

    	"M0V" => -1.38,
	"M2V" => -1.89,
    	"M5V" => -2.73,


# Giant sequence -- again from Allen

    	"G0III" => -0.03,
    	"G5III" => -0.34,
	"G8III" => -0.42,

    	"K0III" => -0.50,
	"K2III" => -0.61,
    	"K5III" => -1.02,

    	"M0III" => -1.25,
	"M2III" => -1.62,
    	"M5III" => -2.48,

# Supergiant sequence -- Allen

	"O9I" => -3.18,

    	"B0I" => -3,
	"B2I" => -1.58,
	"B5I" => -0.95,
	"B8I" => -0.66,

    	"A0I" => -0.41,
	"A2I" => -0.28,
	"A5I" => -0.13,

    	"F0I" => -0.01,
	"F2I" => -0.00,
	"F5I" => -0.03,
	"F8I" => -0.09,

    	"G0I" => -0.15,
	"G2I" => -0.21,
    	"G5I" => -0.33,
	"G8I" => -0.42,

    	"K0I" => -0.50,
	"K2I" => -0.61,
    	"K5I" => -1.01,

    	"M0I" => -1.29,
	"M2I" => -1.62,
    	"M5I" => -3.47,

	   );



#########################################################################
## bolCorr -- compute bolometric correction based on spectral type
#########################################################################
# bolometric correction subroutine.  This just does a table lookup and
# interpolation into our bolometric correction table based on spectral
# type.
#
# The luminosity class must not be blank. Also, because the table
# only contains values for LC I, III, and V, this
# routine treats LC II as LC I, and LC IV as LC V.
#
# Returns bolometric correction on success.
# Returns undef if LC was missing or table interpolation failed.
#
sub bolCorr
{
    my $spType = shift @_;

    # check and remap LC's if needed
    my ($major,$minor,$lc) = parseSpType($spType);
    if (!$lc) {
	print STDERR "DEBUG: bolometricCorrection::bolCorr: LC is missing for spType \"$spType\"\n" if ($cfg->debug());
	return undef;
    } elsif ($lc eq "II") {
	$lc = "I";
    } elsif ($lc eq "IV") {
	$lc = "V";
    }
    # Reassemble the spectral type
    $spType = join '', $major, $minor, $lc;

    my $bc = &tableInterpolate($spType,\%BolCorrTable);
    if (!defined($bc))
    {
	print STDERR "DEBUG: bolometricCorrection::bolCorr: lookup failed for spType \"$spType\"\n" if ($cfg->debug());
	return undef
    }
    return $bc;
} ## end bolCorr  #######################################################

return 1;  ## end bolometricCorrection.pm


