package calCheck;
my $MODVERSION = '$Id: calCheck.pm,v 1.25 2008/11/12 23:28:22 sgroom Exp $';

##########################################################################
# NASA Exoplanet Science Institute (NExScI) -- http://nexsci.caltech.edu #
#       Copyright (c) 1997-2008 California Institute of Technology       #
#                     U.S. Government Sponsorship                        #
# Under contract with the National Aeronautics and Space Administration  #
##########################################################################

#########################################################################
# calCheck.pm - provides functions that search and validate calibrators

use strict;
use warnings;
use coordinates;				# RAhms
use linearDiameter;             # lDiam
use angularDiameters;           # angDiamET
use parseSpType;                # parseSpType
use colors;                     # VminusK, VminusN, KminusL, KminusLprime
use effectiveTemperature;		# effTemp
use findScript;					# findFile
use formatConv;                 # hip2ILine
use angles;                     # DEG_TO_RAD
use angleBetween;               # angleBetween
use hipRecord;					# getHipRecord
use astroCatalogue;				# mangleIdent
use gcIrsa;						# search_irsa
use miscString;					# isNumeric

use Exporter();

our @ISA = qw(Exporter);
our @EXPORT = qw(calSearch calCheck);


# get global config 
my $cfg = gcConfig::config();

#########################################################################
# calSearch - search for calibrators
# 			- given search RA, DEC and radius.  all required

sub calSearch
{
    my $searchRA = shift; die "calCheck::calSearch: searchRA not specified" if !isNumeric $searchRA;
    my $searchDEC = shift; die "calCheck::calSearch: searchDEC not specified" if !isNumeric $searchDEC;
    my $searchRadius = shift; die "calCheck::calSearch: searchRadius not specified" if !defined $searchRadius;

    my @calibrators;

    my $CatalogBytes = 53316318;
    my $SafetyFactor = 1.2;

# Start the search
# Compute the approximate entry point into the catalog for the search
    open(CATALOG,$cfg->hipparcosCatalog) or die "calCheck::calSearch: Can't open Hipparcos catalog " . $cfg->hipparcosCatalog;

    # Compute the size of the RA window (a function of declination)
    my $RAWindow = $searchRadius / cos($searchDEC * DEG_TO_RAD);

# check to see if the search region straddles the 0/24 hr RA boundary
    my $wrapCondition = 0;
    my $OffsetPt;
    if (($searchRA - ($RAWindow * $SafetyFactor) < 0.0)
        || ($searchRA + ($RAWindow * $SafetyFactor) > 360.0))
    {
        $wrapCondition = 1;
        $OffsetPt = 0;
        if ($searchRA > $RAWindow)
        {
            $searchRA -= 360.0;
        }
    }
    else # otherwise, simple search
    {
        $OffsetPt = ($searchRA - ($RAWindow * $SafetyFactor)) / 360.0;
        $OffsetPt *= $CatalogBytes;
        $OffsetPt = int $OffsetPt;
    }

# Step into the catalog at the defined offset
    if ($OffsetPt < $CatalogBytes)
    {
        seek CATALOG, $OffsetPt, 0;
        if ($OffsetPt)
        {
            my $line = <CATALOG>;  # flush any partial line
        }
    }
    else
    {
        print STDERR "Warning: entry offset calculation failed\n";
    }

    @calibrators = do_scan($searchRA, $searchDEC, $searchRadius, $RAWindow);

    if ($wrapCondition) # we've got a end straddle -- search the end of the catalog
    {
        $searchRA += 360.0;
        $OffsetPt = ($searchRA - ($searchRadius * $SafetyFactor)) / 360.0;
        $OffsetPt *= $CatalogBytes;
        $OffsetPt = int $OffsetPt;

        if ($OffsetPt < $CatalogBytes)
        {
            seek CATALOG, $OffsetPt, 0;
            if ($OffsetPt)
            {
                my $line = <CATALOG>;  # flush any partial line
            }
        }
        else
        {
            print STDERR "Warning: entry offset calculation failed\n";
        }
        push @calibrators, do_scan($searchRA, $searchDEC, $searchRadius, $RAWindow);
    } # end wrap search

    return @calibrators;

} # end calSearch

# do_scan - perform a positional search of the hipparcos catalog starting with a RA/DEC bounding box
# followed by a radius check.
sub do_scan
{
    my $searchRA = shift; die "calCheck::do_scan: searchRA not specified" if !isNumeric $searchRA;
    my $searchDEC = shift; die "calCheck::do_scan: searchDEC not specified" if !isNumeric $searchDEC;
    my $searchRadius = shift || die "calCheck::do_scan: searchRadius not specified";
    my $RAWindow = shift || die "calCheck::do_scan: RAWindow not specified";

    my @calibrators;

    my $limitExceeded = 0;
    while ((!$limitExceeded) && (my $hipRec = <CATALOG>))
    {
# caf -- rather than always parsing into hipparcosline structure, consider peeking at RA, DEC and
#        possibly compute separation angle and final internalLine object only if needed.
        my $hipLine = new hipparcosLine(line => $hipRec);
        my $recordRA = $hipLine->extractField("raDeg");
        if (($searchRA - $recordRA) < $RAWindow)
        {
            if (($recordRA - $searchRA) < $RAWindow)
            {
                my $recordDec = $hipLine->extractField("decDeg");
                if (abs($searchDEC - $recordDec) < $searchRadius)
                {
                    my $sep = &angleBetween($searchRA, $searchDEC, $recordRA, $recordDec);
                    if ($sep < $searchRadius)
                    {
                        my $iline = &hip2ILine($hipLine);
                        $iline->setField("tagD",$sep);
                        push @calibrators, $iline;
                    } # end actual vector check
                } # end declination check
            } # end RA check 2
            else # otherwise, we're more than a search radius away from the target RA
            {
                $limitExceeded = 1;
            }
        } # end RA check 1
    } # end while loop

    return @calibrators;
} # end do_scan




#########################################################################
# calCheck - check calibrators
# check V by default using minV and maxV.  sources is hipparcos
# check K by default using minK and maxK.  sources are model and 2mass
# check N if minN or maxN.  sources are model and IRAS

sub calCheck
{
    my %options = @_;

    my $ref = $options{calibrators} || die "calCheck.pm::calCheck: calibrators not specified";
    my @candidates = @$ref;
    my $opt_tTag = $options{tTag};
    my $messageLevel = $options{messageLevel};
    my $opt_twoMass = $options{twoMass};
    my $opt_IRAS = $options{IRAS};
    my $opt_Nband = $options{Nband};
    my $opt_Lband = $options{Lband};
    my $opt_Lprime = $options{Lprime};
    my $minV = $options{minV};
    my $maxV = $options{maxV};
    my $minK = $options{minK};
    my $maxK = $options{maxK};
    my $minN = $options{minN};
    my $maxN = $options{maxN};
    my $minL = $options{minL};
    my $maxL = $options{maxL};
    my $maxAD = $options{maxAD};
    my @lClass = @{$options{lClass}};
    my $effTemp = $options{effTemp};
    my $linearDiam = $options{linearDiam};
    my $passthru = $options{passthru};
    my $useMeasurement = $options{useMeasurement};

    my $firstPassK = $cfg->firstPassK();
    my $firstPassN = $cfg->firstPassN();

    my @calibrators;

    # Process array of calibrators - assumed to be array of internalLine's
    CALIB: foreach my $catLine (@candidates)
    {
		## If the output designation for this object hasn't already been set, set it now
		my $desig = $catLine->getIdent();

#		print STDERR "DEBUG: calCheck: ident is $desig\n" if gcConfig::debug();
		my $outputDesig = $catLine->extractField("designation");
		if (!$outputDesig)
		{
			$outputDesig = mangleIdent($desig);
			$catLine->setField("designation", $outputDesig);
		}
#		print STDERR "DEBUG: calCheck: output desig is $outputDesig\n" if gcConfig::debug();

        # get spectral type as it's used by various models
        my $spType = $catLine->extractField("tagSPECTYP");
        $spType =~ s/\s+//g;  ## eliminate spaces
        $catLine->setField("tagSPECTYP", $spType);
        if (!($spType =~ m/\S+/))
        {
            $spType = "";
            next CALIB if !$passthru;
        }

		my $source   = $catLine->extractField("source");

        # FIRST Check the luminosity class
        if (1) # check always
        {
            my ($major,$minor,$lc) = parseSpType($spType) if $spType;
            my $okay = 0;
            foreach my $lClass (@lClass)
            {
                if ($passthru || (($lClass eq "all") || ($lc eq $lClass)))  # got a luminosity class match
                {
                    # Check diameter estimates.

                    # Estimate the apparent diameter of the object.
                    my ($estDiam,$diamErr) = &angDiam($catLine, $linearDiam, $effTemp) if $opt_tTag;
                    if (!defined $estDiam)
                    {
                        print STDERR "DEBUG: calCheck: no angular diameter for $desig\n" if ($cfg->debug());
                    }
                    if (!defined $diamErr)
                    {
                        print STDERR "DEBUG: calCheck: no angular diameter error for $desig\n" if ($cfg->debug());
                    }

                    if ($passthru || (!defined $estDiam) || # no diam available
                        ((!$maxAD) || ($estDiam <= $maxAD))) # check diam
                    {
			if ($cfg->debug())
			{
			    my $dstr = $estDiam;
			    my $estr = $diamErr;
			    $dstr = "undef" if (!defined $dstr);
			    $estr = "undef" if (!defined $estr);
			    print STDERR "DEBUG: calCheck: desig $desig diam/err = $dstr/$estr\n";
			}
                        $catLine->setField("tagDIAM",$estDiam);
                        $catLine->setField("tagDIAMERR",$diamErr);
                        $okay = 1;
                    } # end maxAD check

                    last;   # skip remaining lClass checks
                } # end check luminosity class match

            } # end while loop on lClass

            # skip to next calibrator if out of limits
            next CALIB if !$passthru && !$okay;

        } # check luminosity class


# check V-band brightness limits
        my $vmag = undef;
        if (1)  # always check
        {
            $vmag = $catLine->extractField("Vmag");

            # skip to next calibrator if out of limits
            next CALIB if !$passthru && ($vmag < $minV || $vmag > $maxV);
        } # check V-band


# check K-band brightness limits
        my $vmk = undef;
        my $kmag = undef;
        my $K2Mass = undef;
        if (1) # always check
        {
            ## Estimate the K mag from color model
            $vmk = VminusK($spType) if $spType;
            if (!defined $vmk)
            {
                print STDERR "DEBUG: calCheck: can't get VminusK for $desig\n" if ($cfg->debug());
		if ($messageLevel > 0)
		{
		    $catLine->addComment("#  Warning: Kmag model failed (VminusK failed for spectype $spType)");
		}
            }
            else
            {
                $kmag = sprintf("%f", $vmag - $vmk);	# be careful to do rounding after subtraction
		print STDERR "DEBUG: calCheck: vmag=$vmag, vmk=$vmk, model Kmag=$kmag\n" if gcConfig::debug();
                $catLine->setField("Kmag",$kmag); ## set the model Kmag field
                my $kmagstr = sprintf("%3.1f", $kmag);	# format for display
                $catLine->addComment("# K mag (model): $kmagstr") if $messageLevel > 1;
            }

            # skip calibrator if magnitude is way off
            next CALIB if !$passthru && defined $kmag && isNumeric($kmag) &&
                ((defined $minK && $kmag < $minK - $firstPassK) || 
                    (defined $maxK && $kmag > $maxK + $firstPassK));

            ## If requested, query 2Mass for actual Kmagnitude
            if ($opt_twoMass && $useMeasurement)
            {
                print STDERR "DEBUG: calCheck: query 2MASS\n" if gcConfig::debug();
                $K2Mass = &twoMassKmag($catLine->extractField("raDeg"),
                    $catLine->extractField("decDeg"));
                if ($K2Mass)
                {
                    print STDERR "DEBUG: calCheck: 2MASS Kmag query returned $K2Mass\n" if gcConfig::debug();
		    my $kmagstr = sprintf("%3.1f", $K2Mass);
                    $catLine->addComment("# K mag (2Mass): $kmagstr") if $messageLevel > 1;
                    $catLine->addComment( &twoMassWarnings($kmag,$K2Mass) ) if ($messageLevel > 0);
                    $kmag = $K2Mass;
                    $catLine->setField("Kmag", $kmag);
                }
                else
                {
                    # 2MASS Kmag lookup failed
                    print STDERR "DEBUG: calCheck: 2MASS Kmag query failed for designation=$desig\n" if gcConfig::debug();
                    if ($messageLevel > 0)
                    {
                        $catLine->addComment("#  Warning: 2MASS Kmag query failed, using model Kmag");
                    }
                }
            }

            # skip to next calibrator if out of limits
            next CALIB if !$passthru && defined $kmag && isNumeric($kmag) &&
                ((defined $minK && $kmag < $minK) || (defined $maxK && $kmag > $maxK));

        } # check K-band

# Report estimated N-band photometry as requested
# check N-band brightness limits
        my $nmag = undef;
		if ($opt_Nband)
        {
            $catLine->setField("tagN", undef);      # force dummy value if N-band is requested
            if ($vmag && $spType)
            {
                my $vmn = VminusN($spType) if $spType;
                if (defined $vmn)
                {
                    $nmag = sprintf("%f", $vmag - $vmn);	# be careful about rounding after subtraction
		    print STDERR "DEBUG: calCheck: vmag=$vmag, vmn=$vmn, model Nmag=$nmag\n" if gcConfig::debug();
                    $catLine->setField("tagN", $nmag);
                    my $nmagstr = sprintf("%3.1f", $nmag);	# format for display
                    $catLine->addComment("# N mag (model): $nmagstr") if $messageLevel > 1;
                }
                else
                {
                    print STDERR "DEBUG: calCheck: Nmag model failed for designation=$desig (VminusN failed for spectype $spType)\n" if gcConfig::debug();
                    if ($messageLevel > 0)
                    {
                        $catLine->addComment("#  Warning: Nmag model failed (VminusN failed for spectype $spType)");
                    }
                }
            }
            else
            {
                # missing kmag or spType
                print STDERR "DEBUG: calCheck: Nmag model failed for designation=$desig (missing Vmag or spectype)\n" if gcConfig::debug();
                if ($messageLevel > 0)
                {
                    $catLine->addComment("#  Warning: Nmag model failed (missing Vmag or spectype)");
                }
            }
            # skip calibrator if magnitude is way off
            next CALIB if !$passthru && defined $nmag && isNumeric($nmag) &&
                ((defined $minN && $nmag < $minN - $firstPassN) || 
                    (defined $maxN && $nmag > $maxN + $firstPassN));

            ## If requested, query IRAS for actual Nmag
            my $NIRAS;
            if ($opt_IRAS && $useMeasurement)
            {
                print STDERR "DEBUG: calCheck: query IRAS for Nmag estimate\n" if gcConfig::debug();
                my $NIRAS = &IRASNmag($catLine->extractField("raDeg"), $catLine->extractField("decDeg"), $spType);
                if ($NIRAS)
                {
                    print STDERR "DEBUG: calCheck: IRAS Nmag query returned $NIRAS\n" if gcConfig::debug();
		    my $nmagstr = sprintf("%3.1f", $NIRAS);
                    $catLine->addComment("# N mag (IRAS model): $nmagstr") if $messageLevel > 1;
                    $catLine->addComment( &IRASWarnings($nmag,$NIRAS) ) if ($messageLevel > 0);
                    $nmag = $NIRAS;
                    $catLine->setField("tagN", $nmag);
                }
                else
                {
                    # IRAS Nmag query failed
                    print STDERR "DEBUG: calCheck: IRAS Nmag query failed for designation=$desig\n" if gcConfig::debug();
                    if ($messageLevel > 0)
                    {
                        $catLine->addComment("#  Warning: IRAS Nmag query failed, using model Nmag");
                    }
                }
            }

            # skip to next calibrator if N-band brighness out of limits
            next CALIB if !$passthru && defined $nmag && isNumeric($nmag) &&
                    ((defined $minN && $nmag < $minN) || (defined $maxN && $nmag > $maxN));
        } # check N-band


# Report estimated L-band photometry as requested
# check L-band brightness limits
	if ($opt_Lband)
        {
            $catLine->setField("tagL", undef);      # force dummy value if L-band is requested
	    my $lmag = undef;
            if ($kmag && $spType)
            {
                my $kml = KminusL($spType) if $spType;
                if (defined $kml)
                {
                    $lmag = sprintf("%f", $kmag - $kml);	# be careful to do rounding after subtraction
		    print STDERR "DEBUG: calCheck: kmag=$kmag, kml=$kml, model Lmag=$lmag\n" if gcConfig::debug();
                    $catLine->setField("tagL", $lmag);
		    my $lmagstr = sprintf("%3.1f", $lmag);	# format for display
                    $catLine->addComment("# L mag (model): $lmagstr") if $messageLevel > 1;
                }
                else
                {
                    print STDERR "DEBUG: calCheck: Lmag model failed for designation=$desig (VminusL failed for spectype $spType)\n" if gcConfig::debug();
                    if ($messageLevel > 0)
                    {
                        $catLine->addComment("#  Warning: Lmag model failed (VminusL failed for spectype $spType)");
                    }
                }
            }
            else
            {
                # missing kmag or spType
                print STDERR "DEBUG: calCheck: Lmag model failed for designation=$desig (missing Kmag or spectype)\n" if gcConfig::debug();
                if ($messageLevel > 0)
                {
                    $catLine->addComment("#  Warning: Lmag model failed (missing Kmag or spectype)");
                }
            }

            # skip to next calibrator if out of limits
            next CALIB if !$passthru && defined $lmag && isNumeric($lmag) &&
                ((defined $minL && $lmag < $minL) || (defined $maxL && $lmag > $maxL));
	}

# Report estimated Lprime-band photometry as requested
# check Lprime-band brightness limits
	if ($opt_Lprime)
        {
            $catLine->setField("tagLPRIME", undef);      # force dummy value if L-band is requested
	    my $lprimemag = undef;
            if ($kmag && $spType)
            {
                my $kmlprime = KminusLprime($spType) if $spType;
                if (defined $kmlprime)
                {
		    $lprimemag = sprintf("%f", $kmag - $kmlprime);	# be careful to do rounding after subtraction
                    print STDERR "DEBUG: calCheck: LPRIME model is $lprimemag\n" if gcConfig::debug();
                    $catLine->setField("tagLPRIME", $lprimemag);
		    my $lprimemagstr = sprintf("%3.1f", $lprimemag);	# format for display
                    $catLine->addComment("# LPRIME mag (model): $lprimemagstr") if $messageLevel > 1;
                }
                else
                {
                    print STDERR "DEBUG: calCheck: LPRIME model failed for designation=$desig (VminusLprime failed for spectype $spType)\n" if gcConfig::debug();
                    if ($messageLevel > 0)
                    {
                        $catLine->addComment("#  Warning: LPRIME model failed (VminusLprime failed for spectype $spType)");
                    }
                }
            }
            else
            {
                # missing kmag or spType
                print STDERR "DEBUG: calCheck: LPRIME model failed for designation=$desig (missing Kmag or spectype)\n" if gcConfig::debug();
                if ($messageLevel > 0)
                {
                    $catLine->addComment("#  Warning: LPRIME model failed (missing Kmag or spectype)");
                }
            }

            # skip to next calibrator if out of limits
	    # Note we use the same set of minL/maxL limits for L and L' mags.
            next CALIB if !$passthru && defined $lprimemag && isNumeric($lprimemag) &&
                ((defined $minL && $lprimemag < $minL) || (defined $maxL && $lprimemag > $maxL));
	}


## Make sure we have B-V
	my $BmV = $catLine->extractField("BminusV");
	if ((!$BmV) && $spType)
	{
		$BmV = BminusV($spType) if $spType;
		$catLine->setField("BminusV", $BmV);
	}

        ## go back and fill in with exact 2mass values if requested
        if ($opt_twoMass && !$useMeasurement)
        {
            print STDERR "DEBUG: calCheck: query 2MASS\n" if gcConfig::debug();
		$K2Mass = &twoMassKmag($catLine->extractField("raDeg"),
                $catLine->extractField("decDeg"));
            if ($K2Mass)
            {
                print STDERR "DEBUG: calCheck: 2MASS Kmag query returned $K2Mass\n" if gcConfig::debug();
		my $kmagstr = sprintf("%3.1f", $K2Mass);
                $catLine->addComment("# K mag (2Mass): $kmagstr") if $messageLevel > 1;
                $catLine->addComment( &twoMassWarnings($kmag,$K2Mass) ) if ($messageLevel > 0);
                $kmag = $K2Mass;
                $catLine->setField("Kmag", $kmag);

		# update other mags that are modeled on K
		# L mag
		if ($opt_Lband)
		{
		    my $kml = KminusL($spType) if $spType;
		    if (defined $kml)
		    {
			my $lmag = sprintf("%f", $kmag - $kml);	# be careful to do rounding after subtraction
			print STDERR "DEBUG: calCheck: Lmag model from 2MASS K is $lmag\n" if gcConfig::debug();
			$catLine->setField("tagL", $lmag);
			my $lmagstr = sprintf("%3.1f", $lmag);	# format for display
			$catLine->addComment("# L mag (model using 2MASS K): $lmagstr") if $messageLevel > 1;
		    }
		    else
		    {
			print STDERR "DEBUG: calCheck: Lmag model failed for designation=$desig (VminusL failed for spectype $spType)\n" if gcConfig::debug();
			if ($messageLevel > 0)
			{
			    $catLine->addComment("#  Warning: Lmag model failed (VminusL failed for spectype $spType)");
			}
		    }
		}

		# L' mag
		if ($opt_Lprime)
		{
		    my $kmlprime = KminusLprime($spType) if $spType;
		    if (defined $kmlprime)
		    {
			my $lprimemag = sprintf("%f", $kmag - $kmlprime);	# be careful to do rounding after subtraction
			print STDERR "DEBUG: calCheck: LPRIME model from 2MASS K is $lprimemag\n" if gcConfig::debug();
			$catLine->setField("tagLPRIME", $lprimemag);
			my $lprimemagstr = sprintf("%3.1f", $lprimemag);	# format for display
			$catLine->addComment("# LPRIME mag (model using 2MASS K): $lprimemagstr") if $messageLevel > 1;
		    }
		    else
		    {
			print STDERR "DEBUG: calCheck: LPRIME model failed for designation=$desig (VminusLprime failed for spectype $spType)\n" if gcConfig::debug();
			if ($messageLevel > 0)
			{
			    $catLine->addComment("#  Warning: LPRIME model failed (VminusL failed for spectype $spType)");
			}
		    }
		}

            }
            else
            {
                # 2MASS Kmag lookup failed
                print STDERR "DEBUG: calCheck: 2MASS Kmag query failed for designation=$desig\n" if gcConfig::debug();
                if ($messageLevel > 0)
                {
                    $catLine->addComment("#  Warning: 2MASS Kmag query failed, using model Kmag");
                }
            }
        }

        ## go back and fill in N with exact iras values if requested
        if ($opt_IRAS && !$useMeasurement)
        {
            print STDERR "DEBUG: calCheck: query IRAS for Nmag estimate\n" if gcConfig::debug();
            my $NIRAS = &IRASNmag($catLine->extractField("raDeg"), $catLine->extractField("decDeg"), $spType);
            if ($NIRAS)
            {
                print STDERR "DEBUG: calCheck: IRAS Nmag query returned $NIRAS\n" if gcConfig::debug();
		my $nmagstr = sprintf("%3.1f",$NIRAS);
                $catLine->addComment("# N mag (IRAS model): $nmagstr") if $messageLevel > 1;
                $catLine->addComment( &IRASWarnings($nmag,$NIRAS) ) if ($messageLevel > 0);
                $nmag = $NIRAS;
                $catLine->setField("tagN", $nmag);

            }
            else
            {
                # IRAS Nmag query failed
                print STDERR "DEBUG: calCheck: IRAS Nmag query failed for designation=$desig\n" if gcConfig::debug();
                if ($messageLevel > 0)
                {
                    $catLine->addComment("#  Warning: IRAS Nmag query failed, using model Nmag");
                }
            }
        }

        # Annotate line with comments/warnings pertaining to the object's attributes
        if ($messageLevel > 0)
        {
            if (($source) && ($source =~ m/HIP/))
            {
                $catLine->addComment( &hipparcosWarnings($catLine) );
            }
        }

        # mark it as a true calibrator if not a target
        if ($opt_tTag) 
        {
            $catLine->setField("tagROLE", "CAL");
			$catLine->setField("tagCALFOR", $opt_tTag);
        }

	my $lineout = $catLine->lineOutput();
	print STDERR "DEBUG: calCheck: after : \"$lineout\"\n" if gcConfig::debug();

        # everything passed, we have a keeper.
        push @calibrators, $catLine;

    } # end foreach loop on @candidates

    return @calibrators;

} # end calCheck


#########################################################################
sub angDiam
{
    my $catLine = shift @_;  ## internal line object
    my $linearDiam = shift;
    my $effTemp = shift;

    # Estimate the apparent diameter of the object.
    # Two methods: model linear diameter and hipparcos
    # parallax, and model effective temperature,
    # bolometric correction, and hipparcos parallax
    # for extinction.
    # estAngDiam in mas (lDiam in AU, Plx in mas)
    ## TLE - 6/2005   now internal pm's and PLX all in arcsec -- put parallax into mas
    my $parallax = $catLine->extractField("tagPLX")*1000.;

    my ($estDiamLinear,  $estDiamEffTemp, $mBol, $fBol, $eT) =
    (undef, undef, undef, undef, undef);

    my $spType = $catLine->extractField("tagSPECTYP");
    my $desig = $catLine->getIdent();

    if ($parallax > 0.0) {
        # get diam estimate from model linear diameter and parallax
        my $ld = lDiam($spType);
        if (defined ($ld))
        {
            $estDiamLinear = $ld * $parallax;
	    print STDERR "DEBUG: calCheck.pm::angDiam: $desig estDiamLinear=$estDiamLinear\n" if ($cfg->debug());
        }

        # get diam estimate from model effective temperature
        my $distance = 1000.0 / $parallax;
        ($estDiamEffTemp,$mBol,$fBol,$eT) =
        angDiamET($spType, $catLine->extractField("Vmag"), $distance);
        # set the record fields
        if (defined $estDiamEffTemp)
        {
	    print STDERR "DEBUG: calCheck.pm::angDiam: $desig estDiamEffTemp=$estDiamEffTemp mBol=$mBol fBol=$fBol eT=$eT\n" if ($cfg->debug());
            $catLine->setField("bolMag",$mBol);
	    $catLine->setField("bolFlux",$fBol);
	    $catLine->setField("effTemp",$eT);
	}
    } else {
	print STDERR "DEBUG: calCheck.pm::angDiam: Warning: Parallax is $parallax for $desig\n" if ($cfg->debug());
    }

    # Decide which estimate to use, in this order:
    # - user selected
    # - average of both effTemp and linear, if avail
    # - effTemp, if avail
    # - linear, if avail
    my $estDiam = undef;
    if ($effTemp)
    {
	# user asked for effTemp diameter
	$estDiam = $estDiamEffTemp;
    }
    elsif ($linearDiam)
    {
	# user asked for linear diameter
	$estDiam = $estDiamLinear;
    }
    elsif (defined $estDiamEffTemp && defined $estDiamLinear)
    {
	# if both estimates worked return the average
	$estDiam = ($estDiamLinear + $estDiamEffTemp) / 2.0;
    }
    elsif (defined $estDiamEffTemp)
    {
	# if effTemp worked, return that
	$estDiam = $estDiamEffTemp;
    }
    elsif (defined $estDiamLinear)
    {
	# if linear worked, return that
	$estDiam = $estDiamLinear;
    }

    # quote the error as the disagreement of the two methods.
    my $estDiamErr = undef;
    if (defined $estDiam && defined $estDiamEffTemp && defined $estDiamLinear) 
    {
	$estDiamErr = abs($estDiamLinear - $estDiamEffTemp);
    }
    return ($estDiam,$estDiamErr);
} # end angDiam


#########################################################################
## IRASNmag -- compute Nmag estimate based on IRAS fnu_12
## Search for IRAS 12um measurement in IRSA archive and convert to N band magnitude.
## Applies IRAS 12um color correction
## Also applies passband flux correction (IRAS 12um to N)
#########################################################################
sub IRASNmag
{
## declarations  ########################################################
    my $raDeg = shift @_;
    my $decDeg = shift @_;
    my $spType = shift @_;

    # IRAS 12um color correction factors, excerpted from IRAS explanatory supplement
    # (Table Suppl.VI.C.6, Color Correction Factos, K)
    my @eTemps = (2000, 3000, 4000, 5000, 10000);
    my @k12 = (1.38, 1.41, 1.42, 1.43, 1.45);
    my $kcor_default = 1.42;	# default value used when temp estimate is not available
    my $numET = $#eTemps;
    my $kcor;

    # IRAS Nband flux correction for IRAS 12um passband to N passband
    # per Rachel Akeson email, 8/10/2006
    my $nfluxcor = 1.15;

## begin ################################################################
    ## retrieve IRAS fnu_12 from IRSA
    my $sigFigures = 3;
    my $ra = RAhms($raDeg, $sigFigures);
    $ra =~ s/ /:/g;
    my $dec = DECdms($decDeg, $sigFigures);
    $dec =~ s/ /:/g;
    print STDERR "DEBUG: IRASNmag: calling search_irsa ra=$ra dec=$dec spType=$spType\n" if gcConfig::debug();
    my ($idata, $result) = search_irsa({catalog => "iraspsc",
		    cols => [qw/fnu_12/],
		    radius => $cfg->IRASSearchRadius(),
		    targetRA => $ra,
		    targetDEC => $dec});
    print STDERR "DEBUG: IRASNmag: search_irsa returned $result\n" if gcConfig::debug();
    if ($result < 0)
    {
	# IRSA search error
	print STDERR "IRASNmag: error during IRSA search, search_irsa returned error $result\n";
	return undef;
    }
    elsif ($result > 0 || !isNumeric($idata->[0]{fnu_12}))
    {
	# lookup failed
	print STDERR "DEBUG: IRASNmag: lookup failed\n" if gcConfig::debug();
	return undef;
    }
    my $iras_f12 = $idata->[0]{fnu_12};
    print STDERR "DEBUG: IRASNmag: raw IRAS f12 = $iras_f12\n" if gcConfig::debug();

    ## compute color correction factor to apply
    my $eT = &effTemp($spType);
    if (!defined $eT)
    {
	$kcor = $kcor_default;	# temp not avail, use default kcor
	print STDERR "DEBUG: IRASNmag: effTemp undefined, using default kcor = $kcor\n" if gcConfig::debug();
    }
    elsif ($eT >= $eTemps[-1])
    {
	$kcor = $k12[-1]; # out of table range - use range max
    }
    elsif ($eT < $eTemps[0])
    {
	$kcor = $k12[0]; # out of table range - use range min
    }
    else
    {
	# do table lookup/interpolation to compute kcor
	foreach my $i (0..$numET-1) 
	{
	    if ($eT == $eTemps[$i])
	    {
		$kcor = $k12[$i];
	    }
	    elsif (($eT >= $eTemps[$i]) && ($eT < $eTemps[$i+1]))
	    {
		$kcor = ( (($k12[$i+1] - $k12[$i])/($eTemps[$i+1] - $eTemps[$i])) 
			    * ($eT - $eTemps[$i]) ) + $k12[$i];
	    }
	}
    }
    print STDERR "DEBUG: IRASNmag: kcor = $kcor\n" if gcConfig::debug();

    ## apply flux corrections to get corrected N flux
    my $nflux = $iras_f12 / $kcor * $nfluxcor;
    print STDERR "DEBUG: IRASNmag: corrected N flux = $nflux\n" if gcConfig::debug();

    ## convert flux to mag
    my $nmag = (-2.5 * (log($nflux)/log(10)) ) + 3.629;
    print STDERR "DEBUG: IRASNmag: computed N mag = $nmag\n" if gcConfig::debug();
    return sprintf "%f", $nmag;

} ## end IRASNmag  ###################################################



#########################################################################
## twoMassKmag -- query 2Mass database on object
#########################################################################
sub twoMassKmag
{

## declarations  ########################################################

	## my $desig = shift @_;
	##  TLE -- 5/2005  use searchRA, searchDec instead of $desig to give
	##                 more accurate coords (w/o simbad name lookup)
	my $raDeg = shift @_;
	my $decDeg = shift @_;

## begin ################################################################
    my $sigFigures = 3;
	my $ra = RAhms($raDeg, $sigFigures);
	$ra =~ s/ /:/g;
	my $dec = DECdms($decDeg, $sigFigures);
	$dec =~ s/ /:/g;
	my ($data, $result) = search_irsa({catalog => "fp_psc",
			cols => [ "k_m" ],
			radius => $cfg->TwoMassSearchRadius(),
			targetRA => $ra,
			targetDEC => $dec});
	return undef if ($result > 0 || !isNumeric($data->[0]{k_m}));
	die "IRSA search failed with exit code $result" if ($result < 0);
	return sprintf("%f", $data->[0]{k_m});
} ## end twoMassKmag  ###################################################



#########################################################################
## hipparcosWarnings -- printout information for hipparcos records
#########################################################################
sub hipparcosWarnings
{
## declarations  ########################################################
	my $catLine = shift @_;  ## internal catalog line object
	my $msg = "";

## begin  ###############################################################

	my $hipID = $catLine->extractField("catIndex2") || "";
	$hipID =~ s/HIP//;
	$hipID =~ s/\s+//g;
	my $hipRec = getHipRecord("HIP",$hipID);
	if (!$hipRec)
	{
		print STDERR "DEBUG: calCheck: hipparcosWarnings: getHipRecord failed for HIP $hipID\n" if gcConfig::debug();
		return $msg;
	}
	my $hipLine = new hipparcosLine(line => $hipRec);
	my $hd = $hipLine->extractField("HDnum");
	$hd =~ s/\s+//g;

	## First check photometric variability
	if ((my $vFlag = $hipLine->extractField("coarseVarFlag")) =~ m/\S+/)
	{
		my $scat = $hipLine->extractField("Hpscat");
		my $nObs = $hipLine->extractField("o_Hpmag");
		$msg .= "# HIP $hipID (HD $hd) has his variability flag set ($vFlag)\n";
		$msg .= "#  with $scat mag scatter in $nObs observations\n";
	} # end photometric variability

	## Next check various mulitplicity items
	if ($hipLine->extractField("DSFlag") =~ m/\S+/)
	{
		my $dsFlag = $hipLine->extractField("DSFlag");
		$msg .= "# HIP $hipID (HD $hd) has his multiple component flag set to $dsFlag\n";
		## component solutons
		if ($hipLine->extractField("DSFlag") =~ m/C/)
		{
			$msg .= "#  the C designation indicates solutions were found for individual components\n";
			# Extract pertinent info from the hipparcos component solution annex
			if (defined (my $componentFile = findFile("h_dm_com.dat",$cfg->hipparcosPath)))
			{
				my $command = $cfg->egrep." \"\\\|*".$hipID."\\\|\" ".$componentFile;
				print STDERR "DEBUG: calCheck: hipparcosWarnings: running command \"$command\"\n" if gcConfig::debug();
				my $tmp = `$command` or die;
				my @tmp2 = split /\n/, $tmp;
				my $count = 0;
				my $rpt = "";
				my $cmp;
				foreach $cmp (@tmp2)
				{
					my @tmp3 = split /\|/, $cmp;
					$tmp3[13] =~ s/\s+//g;
					$hipID =~ s/\s+//g;
					if ($tmp3[13] eq $hipID)
					{
						my $ref = $tmp3[12];
						my $sep = $tmp3[32];
						my $pa = $tmp3[31];
						my $V = $tmp3[14];
						if (!($sep =~ m/\S+/))
						{
							$rpt .= "#    $ref component -- V=$V\n";
						}
						else
						{
							$sep =~ s/\s+//g;
							$pa = sprintf "%3.0f", $pa;
							$rpt .= "#    $ref component -- V=$V at sep $sep arcsec/PA $pa deg\n";
						}
						$count++;
					} # end item check
				} # end foreach
				$msg .= "#    $count components:\n$rpt";
			} # end component file check

		} # end $hipLine->extractField("DSFlag") = C
		elsif ($hipLine->extractField("DSFlag") =~ m/G/)  ## higher-order terms...
		{
			$msg .= "#  the G designation indicates acceleration or higher-order terms in astrometric solution\n";
		} # end $contents[$HIP_DSFlag] = G
		elsif ($hipLine->extractField("DSFlag") =~ m/O/)  ## orbital motion...
		{
			$msg .= "#  Warning: the O designation indicates an orbital solution was found\n";
			# Extract pertinent info from the hipparcos orbital solution annex
			if (defined (my $orbitFile = findFile("hip_dm_o.dat",$cfg->hipparcosPath)))
			{
				my $command = $cfg->egrep." \"".$hipID."\\\|\" ".$orbitFile;
				print STDERR "DEBUG: calCheck: hipparcosWarnings: running command \"$command\"\n" if gcConfig::debug();
				my $tmp = `$command` or die;
				my @tmp2 = split /\|/, $tmp;
				$tmp2[3] =~ s/\s+//g;
				$tmp2[1] =~ s/\s+//g;
				$msg .= "#   with photocentric SMA $tmp2[3] mas, $tmp2[1] day period\n";
			} # end orbit file check
		} # end $contents[$HIP_DSFlag] = O
		elsif ($hipLine->extractField("DSFlag") =~ m/V/)  ## variability-induced mover
		{
			$msg .= "#  Warning: the V designation indicates suspected variability-induced movement\n";
		} # end $contents[$HIP_DSFlag] = V
		elsif ($hipLine->extractField("DSFlag") =~ m/X/)  ## stochastic mover
		{
			$msg .= "#  Warning: the X designation indicates stochastic motion/suspected astrometric binary\n";
		} # end $contents[$HIP_DSFlag] = O
	} # end multiple components


	## Check astrometric source flag
	if ($hipLine->extractField("AstSrcFlag") =~ m/\S+/)
	{
		my $aFlag = $hipLine->extractField("AstSrcFlag");
		my $hd = $hipLine->extractField("HDnum");
		$hd =~ s/\s+//g;
		my $quality = $hipLine->extractField("SolQualFlag");
		$msg .= "# HIP $hipID (HD $hd) has his astrometric source flag set to $aFlag\n";
		$msg .= "#  with solution quality listed as $quality\n";
		if ((my $rho = $hipLine->extractField("rho")) =~ m/\S+/)
		{
			$msg .= "# and a listed separation between components of $rho arcsec\n";
		}
	} ## end check astrometric source flag

	return $msg;

} ## end hipparcosWarnings  #############################################



#########################################################################
## print warnings for 2Mass/model K-photometry
#########################################################################
sub twoMassWarnings
{
	my $KModel = shift @_;
	my $K2Mass = shift @_;

	my $tol = $cfg->toleranceK();
	my $msg = "";

## begin  ###############################################################

	## Check agreement of model and 2Mass K
	if ($KModel && $K2Mass && isNumeric($KModel) && isNumeric($K2Mass)
		&& (abs($K2Mass - $KModel) > $tol))
	{
		my $diff = sprintf "%4.2f", abs($K2Mass - $KModel);
		my $kmodStr = sprintf "%4.2f", $KModel;
		my $k2massStr = sprintf "%4.2f", $K2Mass;
		$msg .= "#  Warning: model K ($kmodStr) and 2Mass K ($k2massStr) differ by $diff\n";
	}

	return $msg;

}  ## end twoMassWarnings  ##############################################

#########################################################################
## print warnings for IRAS/model N-photometry
#########################################################################
sub IRASWarnings
{
	my $NModel = shift @_;
	my $NIRAS = shift @_;

	my $tol = $cfg->toleranceN();
	my $msg = "";

## begin  ###############################################################

	## Check agreement of model and IRAS N
	if ($NModel && $NIRAS && isNumeric($NModel) && isNumeric($NIRAS)
		&& (abs($NIRAS - $NModel) > $tol))
	{
		my $diff = sprintf "%4.2f", abs($NIRAS - $NModel);
		my $nmodStr = sprintf "%4.2f", $NModel;
		my $nirasStr = sprintf "%4.2f", $NIRAS;
		$msg .= "#  Warning: model N ($nmodStr) and IRAS model N ($nirasStr) differ by $diff\n";
	}

	return $msg;
}  ## end IRASWarnings  ##############################################

1; ## end of calCheck.pm
