package colors;
my $MODVERSION = '$Id: colors.pm,v 1.14 2008/11/12 23:28:22 sgroom Exp $';
use strict;
use warnings;

require 5.000;

use Exporter();

our @ISA = qw(Exporter);
our @EXPORT = qw(VminusK BminusV VminusN KminusL KminusLprime);


##########################################################################
# NASA Exoplanet Science Institute (NExScI) -- http://nexsci.caltech.edu #
#       Copyright (c) 1997-2008 California Institute of Technology       #
#                     U.S. Government Sponsorship                        #
# Under contract with the National Aeronautics and Space Administration  #
##########################################################################


use tableInterpolate;
use parseSpType;
use gcConfig;

my $cfg = gcConfig::config();


# Associative array of V - K
my %VminusKtable = (
 # Main sequence -- from Tokunaga
 # (found at UKIRT webpage http://lanalana.jach.hawaii.edu/UKIRT.new/astronomy/temp.html
 # Not really any different than the Bessell and Brett 1988 colors

    "O5V" => -0.91,
    "O8V" => -0.91,
    "O9V" => -0.87,
    "O9.5V" => -0.85,

    "B0V" => -0.83,
    "B1V" => -0.74,
    "B2V" => -0.66,
    "B3V" => -0.56,
    "B4V" => -0.49,
    "B5V" => -0.42,
    "B6V" => -0.36,
    "B7V" => -0.29,
    "B8V" => -0.24,

    "A0V" => 0.0,
    "A2V" => 0.02,
    "A5V" => 0.38,
    "A7V" => 0.50,

    "F0V" => 0.70,
    "F2V" => 0.82,
    "F5V" => 1.10,
    "F7V" => 1.32,

    "G0V" => 1.41,
    "G2V" => 1.46,
    "G4V" => 1.53,
    "G6V" => 1.64,

    "K0V" => 1.96,
    "K2V" => 2.22,
    "K4V" => 2.63,
    "K5V" => 2.85,
    "K7V" => 3.16,

    "M0V" => 3.65,
    "M1V" => 3.87,
    "M2V" => 4.11,
    "M3V" => 4.65,
    "M4V" => 5.28,
    "M5V" => 6.17,
    "M6V" => 7.37,


# Giant sequence -- again from Tomanaga

    "G0III" => 1.75,
    "G4III" => 2.05,
    "G6III" => 2.15,
    "G8III" => 2.16,

    "K0III" => 2.31,
    "K1III" => 2.50,
    "K2III" => 2.70,
    "K3III" => 3.00,
    "K4III" => 3.26,
    "K5III" => 3.60,

    "M0III" => 3.85,
    "M1III" => 4.05,
    "M2III" => 4.30,
    "M3III" => 4.64,
    "M4III" => 5.10,
    "M5III" => 5.96,
    "M6III" => 6.84,
    "M7III" => 7.70,

# Supergiant sequence -- again from Tomanaga
# These were NOT in Bessell and Brett!!!

    "O9I" => -0.82,

    "B0I" => -0.69,
    "B1I" => -0.55,
    "B2I" => -0.40,
    "B3I" => -0.28,
    "B4I" => -0.20,
    "B5I" => -0.13,
    "B6I" => -0.07,
    "B7I" => 0.01,
    "B8I" => 0.07,
    "B9I" => 0.13,

    "A0I" => 0.19,
    "A1I" => 0.26,
    "A2I" => 0.32,
    "A5I" => 0.37,

    "F0I" => 0.64,
    "F2I" => 0.75,
    "F5I" => 0.93,
    "F8I" => 1.21,

    "G0I" => 1.44,
    "G3I" => 1.67,
    "G8I" => 1.99,

    "K0I" => 2.15,
    "K1I" => 2.28,
    "K2I" => 2.43,
    "K3I" => 2.90,
    "K5I" => 3.50,

    "M0I" => 3.80,
    "M1I" => 3.90,
    "M2I" => 4.10,
    "M3I" => 4.60,
    "M4I" => 5.20,

	   );



#########################################################################
# VminusK -- estimate V-K for input spectral type
#########################################################################
# V - K color index subroutine.  This just does a table lookup and
# interpolation into our V - K color index table based on spectral
# type.
#
# The luminosity class must not be blank. Also, because the table
# only contains values for LC I, III, and V, this
# routine treats LC II as LC I, and LC IV as LC V.
#   
# Returns estimated V-K on success.
# Returns undef if LC was missing or table interpolation failed.
#
sub VminusK
{
    my $spType = shift @_;

    # check and remap LC's if needed
    my ($major,$minor,$lc) = parseSpType($spType);
    if (!$lc) {
	print STDERR "DEBUG: colors::VminusK: LC is missing for spType \"$spType\"\n" if (gcConfig::debug() >= 2);
	return undef;
    } elsif ($lc eq "II") {
	$lc = "I";
    } elsif ($lc eq "IV") {
	$lc = "V";
    }
    # Reassemble the spectral type
    $spType = join '', $major, $minor, $lc;

    my $val = &tableInterpolate($spType,\%VminusKtable);
    if (!defined $val)
    {
	print STDERR "DEBUG: colors::VminusK: lookup failed for spType \"$spType\"\n" if (gcConfig::debug() >= 2);
	return undef;
    }

    return sprintf "%5.3f", $val;
} ## end VminusK  #######################################################




# Associative array of B - V
my %BminusVtable = (
 # Main sequence -- from Allen 4th edition pp 388-389

    "O5V" => -0.33,
    "O9V" => -0.31,

    "B0V" => -0.30,
    "B2V" => -0.24,
    "B5V" => -0.17,
    "B8V" => -0.11,

    "A0V" => -0.02,
    "A2V" => +0.05,
    "A5V" => +0.15,

    "F0V" => +0.30,
    "F2V" => +0.35,
    "F5V" => +0.44,
    "F8V" => +0.52,

    "G0V" => +0.58,
    "G2V" => +0.63,
    "G5V" => +0.68,
    "G8V" => +0.74,

    "K0V" => +0.81,
    "K2V" => +0.91,
    "K5V" => +1.15,

    "M0V" => +1.40,
    "M2V" => +1.49,
    "M5V" => +1.64,


# Giant sequence -- again from Tomanaga
# June 2006 RMG comment: looks like Allen values ... except G0

    "G0III" => +0.58,
    "G5III" => +0.86,
    "G8III" => +0.94,

    "K0III" => +1.00,
    "K2III" => +1.16,
    "K5III" => +1.50,

    "M0III" => +1.56,
    "M2III" => +1.60,
    "M5III" => +1.63,

# Supergiant sequence -- again from Tomanaga
# June 2006 RMG comment: looks like Allen values

    "O9I" => -0.27,

    "B2I" => -0.17,
    "B5I" => -0.10,
    "B8I" => -0.03,

    "A0I" => -0.01,
    "A2I" => +0.03,
    "A5I" => +0.09,

    "F0I" => +0.17,
    "F2I" => +0.23,
    "F5I" => +0.32,
    "F8I" => +0.56,

    "G0I" => +0.76,
    "G2I" => +0.87,
    "G8I" => +1.14,

    "K0I" => +1.25,
    "K2I" => +1.36,
    "K5I" => +1.60,

    "M0I" => +1.67,
    "M2I" => +1.71,
    "M5I" => +1.80,

	   );


#########################################################################
# BminusV -- estimate B-V for input spectral type
#########################################################################
# BminusV subroutine.  This just does a table lookup and interpolation
# into our effective temperature table based on spectral type.
#
# The luminosity class must not be blank. Also, because the table
# only contains values for LC I, III, and V, this
# routine treats LC II as LC I, and LC IV as LC V.
#   
# Returns estimated B-V on success.
# Returns undef if LC was missing or table interpolation failed.
#
sub BminusV
{
    my $spType = shift @_;

    # check and remap LC's if needed
    my ($major,$minor,$lc) = parseSpType($spType);
    if (!$lc) {
	print STDERR "DEBUG: colors::BminusV: LC is missing for spType \"$spType\"\n" if (gcConfig::debug() >= 2);
	return undef;
    } elsif ($lc eq "II") {
	$lc = "I";
    } elsif ($lc eq "IV") {
	$lc = "V";
    }
    # Reassemble the spectral type
    $spType = join '', $major, $minor, $lc;

    my $val = &tableInterpolate($spType,\%BminusVtable);
    if (!defined $val)
    {
	print STDERR "DEBUG: colors::BminusV: lookup failed for spType \"$spType\"\n" if (gcConfig::debug() >= 2);
	return undef;
    }
    return sprintf "%4.2f", $val;
} ## end subroutine BminusV  ############################################



# RMG: june 2006
# real table of intrinsic (V-N) table
# from Ducati et al. ApJ, 558, 2001

# Associative array of V - N
my %VminusNtable = (
		 
 ## Main-sequence (LC V)

    	"B2.5V" => -0.96,
	"B3V" => -0.91,
	"B4V" => -0.84,
	"B5V" => -0.75,
	"B6V" => -0.66,
	"B7V" => -0.58,
	"B8V" => -0.48,
	"B9V" => -0.30,

	"A0V" => -0.14,
	"A1V" => -0.08,
	"A2V" => -0.02,
	"A3V" => +0.03,
	"A4V" => +0.09,
	"A5V" => +0.16,
	"A6V" => +0.21,
	"A7V" => +0.26,
	"A8V" => +0.34,
	"A9V" => +0.41,
	
	"F0V" => +0.48,
	"F1V" => +0.54,
	"F2V" => +0.60,
	"F5V" => +0.80,
	"F8V" => +0.91,

	"G0V" => +1.01,
	"G2V" => +1.11,
	"G3V" => +1.13,
	"G5V" => +1.17,
	"G8V" => +1.30,

	"K0V" => +1.54,
	"K1V" => +1.68,
	"K2V" => +1.84,
	"K3V" => +2.01,
	"K4V" => +2.15,
	"K5V" => +2.44,
	"K7V" => +2.83,

	"M0V" => +3.16,
	"M1V" => +3.39,
	"M2V" => +3.66,
	"M3V" => +3.89,
	"M4V" => +4.11,

 ## Giant stars (LC III)	 

	"G3III" => +1.37,
	"G4III" => +1.44,
	"G5III" => +1.51,
	"G8III" => +1.75,
	"G9III" => +1.83,
	
	"K0III" => +1.92,
	"K1III" => +2.11,
	"K2III" => +2.37,
	"K3III" => +2.73,
	"K4III" => +3.02,
	"K5III" => +3.21,

	"M0III" => +3.54,
	"M1III" => +3.74,
	"M2III" => +4.00,
	"M3III" => +4.26,
	"M4III" => +4.56,
	"M5III" => +4.90,
	"M6III" => +5.23,

 ## SuperGiant stars (LC I)

	"O9I" => -0.65,	
	
	"B0I" => -0.63,
	"B1I" => -0.50,
	"B2I" => -0.44,
	"B3I" => -0.35,
	"B4I" => -0.28,
	"B5I" => -0.19,
	"B6I" => -0.14,
	"B7I" => -0.06,
	"B8I" => +0.05,
	"B9I" => +0.19,
	
	"A0I" => +0.29,
	"A1I" => +0.39,
	"A2I" => +0.43,
	"A5I" => +0.57,
	
	"F0I" => +0.82,
	"F2I" => +0.95,
	"F5I" => +1.15,
	"F8I" => +1.58,

	"G0I" => +1.98,
	"G2I" => +2.28,
	"G3I" => +2.38,
	"G4I" => +2.50,
	"G5I" => +2.63,
	"G8I" => +2.91,

	"K0I" => +3.14,
	"K1I" => +3.26,
	"K2I" => +3.38,
	"K3I" => +3.63,
	"K4I" => +3.84,
	"K5I" => +4.01,
	
	"M0I" => +4.38,
	"M1I" => +4.58,
	"M2I" => +4.80,
	"M3I" => +5.37,
	"M4I" => +5.95,
	"M5I" => +6.47,

	);

# 
# RMG June 2006
#
#########################################################################
# VminusN -- estimate V-N for input spectral type
#########################################################################
# V - N color index subroutine.  This just does a table lookup and
# interpolation into our V - N color index table based on spectral
# type.
#
# The luminosity class must not be blank. Also, because the table
# only contains values for LC I, III, and V, this
# routine treats LC II as LC I, and LC IV as LC V.
#   
# Returns estimated V-N on success.
# Returns undef if LC was missing or table interpolation failed.
#
sub VminusN
{
    my $spType = shift @_;

    # check and remap LC's if needed
    my ($major,$minor,$lc) = parseSpType($spType);
    if (!$lc) {
	print STDERR "DEBUG: colors::VminusN: LC is missing for spType \"$spType\"\n" if (gcConfig::debug() >= 2);
	return undef;
    } elsif ($lc eq "II") {
	$lc = "I";
    } elsif ($lc eq "IV") {
	$lc = "V";
    }
    # Reassemble the spectral type
    $spType = join '', $major, $minor, $lc;

    my $val = &tableInterpolate($spType,\%VminusNtable);
    if (!defined $val)
    {
	print STDERR "DEBUG: colors::VminusN: lookup failed for spType \"$spType\"\n" if (gcConfig::debug() >= 2);
	return undef;
    }
    return sprintf "%5.3f", $val;
} ## end VminusN  #######################################################


# SLG: May 2007
# K-L table from Allen 4th ed, p151
# Associative array of K - L
my %KminusLtable = (
		 
 ## Main-sequence (LC V)

	"O9V" => -0.06,
	"O9.5V" => -0.06,

	"B0V" => -0.06,
	"B1V" => -0.05,
	"B2V" => -0.05,
	"B3V" => -0.05,
	"B4V" => -0.05,
	"B5V" => -0.04,
	"B6V" => -0.04,
	"B7V" => -0.04,
	"B8V" => -0.04,
	"B9V" => -0.03,

	"A0V" => +0.00,
	"A2V" => +0.01,
	"A5V" => +0.02,
	"A7V" => +0.03,

	"F0V" => +0.03,
	"F2V" => +0.03,
	"F5V" => +0.04,
	"F7V" => +0.04,

	"G0V" => +0.05,
	"G2V" => +0.05,
	"G4V" => +0.05,
	"G6V" => +0.05,

	"K0V" => +0.06,
	"K2V" => +0.07,
	"K4V" => +0.09,
	"K5V" => +0.10,
	"K7V" => +0.11,

	"M0V" => +0.14,
	"M1V" => +0.15,
	"M2V" => +0.16,
	"M3V" => +0.20,
	"M4V" => +0.23,
	"M5V" => +0.29,
	"M6V" => +0.36,

 ## Giant stars (LC III)	 

	"G0III" => +0.04,
	"G4III" => +0.05,
	"G6III" => +0.06,
	"G8III" => +0.06,

	"K0III" => +0.07,
	"K1III" => +0.08,
	"K2III" => +0.09,
	"K3III" => +0.10,
	"K4III" => +0.11,
	"K5III" => +0.12,

	"M0III" => +0.12,
	"M1III" => +0.13,
	"M2III" => +0.15,
	"M3III" => +0.17,
	"M4III" => +0.18,
	"M5III" => +0.20,

 ## SuperGiant stars (LC I)

	"O9I" => -0.08,	

	"B0I" => -0.07,	
	"B1I" => -0.07,	
	"B2I" => -0.07,	
	"B3I" => -0.05,	
	"B4I" => -0.01,	
	"B5I" => +0.02,	
	"B6I" => +0.03,	
	"B7I" => +0.04,	
	"B8I" => +0.05,	
	"B9I" => +0.06,	

	"A0I" => +0.07,	
	"A1I" => +0.07,	
	"A2I" => +0.08,	
	"A5I" => +0.07,	

	"F0I" => +0.06,	
	"F2I" => +0.06,	
	"F5I" => +0.07,	
	"F8I" => +0.07,	

	"G0I" => +0.08,	
	"G3I" => +0.08,	
	"G8I" => +0.09,	

	"K0I" => +0.10,	
	"K1I" => +0.11,	
	"K2I" => +0.12,	

	);

# 
# SLG May 2007
#
#########################################################################
# KminusL -- estimate K-L for input spectral type
#########################################################################
# K - L color index subroutine.  This just does a table lookup and
# interpolation into our K - L color index table based on spectral
# type.
#
# The luminosity class must not be blank. Also, because the table
# only contains values for LC I, III, and V, this
# routine treats LC II as LC I, and LC IV as LC V.
#   
# Returns estimated K-L on success.
# Returns undef if LC was missing or table interpolation failed.
#
sub KminusL
{
    my $spType = shift @_;

    # check and remap LC's if needed
    my ($major,$minor,$lc) = parseSpType($spType);
    if (!$lc) {
	print STDERR "DEBUG: colors::KminusL: LC is missing for spType \"$spType\"\n" if (gcConfig::debug() >= 2);
	return undef;
    } elsif ($lc eq "II") {
	$lc = "I";
    } elsif ($lc eq "IV") {
	$lc = "V";
    }
    # Reassemble the spectral type
    $spType = join '', $major, $minor, $lc;

    my $val = &tableInterpolate($spType,\%KminusLtable);
    if (!defined $val)
    {
	print STDERR "DEBUG: colors::KminusL: lookup failed for spType \"$spType\"\n" if (gcConfig::debug() >= 2);
	return undef;
    }
    return sprintf "%5.3f", $val;
} ## end KminusL  #######################################################


# Associative array of K - L'
# K-L' table from Allen 4th ed, pp151-153
# Note some subranges are really K-L values from that reference,
# or from other sources. See notes inline
# SLG, May 2007
my %KminusLprimeTable = (
		 
 ## Main-sequence (LC V)

	# values for O9V through B9V are really K-L from the same reference
	"O9V" => -0.06,
	"O9.5V" => -0.06,

	"B0V" => -0.06,
	"B1V" => -0.05,
	"B2V" => -0.05,
	"B3V" => -0.05,
	"B4V" => -0.05,
	"B5V" => -0.04,
	"B6V" => -0.04,
	"B7V" => -0.04,
	"B8V" => -0.04,
	"B9V" => -0.03,

	# values for A0V through M6V are K-L' from Allen p151
	"A0V" => +0.00,
	"A2V" => +0.01,
	"A5V" => +0.02,
	"A7V" => +0.03,

	"F0V" => +0.03,
	"F2V" => +0.03,
	"F5V" => +0.04,
	"F7V" => +0.04,

	"G0V" => +0.05,
	"G2V" => +0.05,
	"G4V" => +0.05,
	"G6V" => +0.05,

	"K0V" => +0.06,
	"K2V" => +0.07,
	"K4V" => +0.10,
	"K5V" => +0.11,
	"K7V" => +0.13,

	"M0V" => +0.17,
	"M1V" => +0.21,
	"M2V" => +0.23,
	"M3V" => +0.32,
	"M4V" => +0.37,
	"M5V" => +0.42,
	"M6V" => +0.48,

	# The M7V..M9V values below provided by Rachel Akeson,
	# based on (Reid and Cruz 2002) which shows K-L'
	# is linear through the M spectral type.
	"M7V" => +0.53,
	"M8V" => +0.58,
	"M9V" => +0.63,

 ## Giant stars (LC III)	 

	"G0III" => +0.05,
	"G4III" => +0.06,
	"G6III" => +0.07,
	"G8III" => +0.07,

	"K0III" => +0.08,
	"K1III" => +0.09,
	"K2III" => +0.10,
	"K3III" => +0.12,
	"K4III" => +0.14,
	"K5III" => +0.16,

	"M0III" => +0.17,
	"M1III" => +0.17,
	"M2III" => +0.19,
	"M3III" => +0.20,
	"M4III" => +0.21,
	"M5III" => +0.22,

 ## SuperGiant stars (LC I)

	# These LC I values are really K-L (not K-L') 
	# taken from Allen 4th ed, p153

	"O9I" => -0.08,	

	"B0I" => -0.07,	
	"B1I" => -0.07,	
	"B2I" => -0.07,	
	"B3I" => -0.05,	
	"B4I" => -0.01,	
	"B5I" => +0.02,	
	"B6I" => +0.03,	
	"B7I" => +0.04,	
	"B8I" => +0.05,	
	"B9I" => +0.06,	

	"A0I" => +0.07,	
	"A1I" => +0.07,	
	"A2I" => +0.08,	
	"A5I" => +0.07,	

	"F0I" => +0.06,	
	"F2I" => +0.06,	
	"F5I" => +0.07,	
	"F8I" => +0.07,	

	"G0I" => +0.08,	
	"G3I" => +0.08,	
	"G8I" => +0.09,	

	"K0I" => +0.10,	
	"K1I" => +0.11,	
	"K2I" => +0.12,	
	);

# 
# SLG May 2007
#
#########################################################################
# KminusLprime -- estimate K - L' for input spectral type
#########################################################################
# K - L' color index subroutine.  This just does a table lookup and
# interpolation into our K - L' color index table based on spectral
# type.
#
# Note this routine deals with L', not L.
#
# The luminosity class must not be blank. Also, because the table
# only contains values for LC I, III, and V, this
# routine treats LC II as LC I, and LC IV as LC V.
#   
# Returns estimated K - L' on success.
# Returns undef if LC was missing or table interpolation failed.
#
sub KminusLprime
{
    my $spType = shift @_;

    # check and remap LC's if needed
    my ($major,$minor,$lc) = parseSpType($spType);
    if (!$lc) {
	print STDERR "DEBUG: colors::KminusLprime: LC is missing for spType \"$spType\"\n" if (gcConfig::debug() >= 2);
	return undef;
    } elsif ($lc eq "II") {
	$lc = "I";
    } elsif ($lc eq "IV") {
	$lc = "V";
    }
    # Reassemble the spectral type
    $spType = join '', $major, $minor, $lc;

    my $val = &tableInterpolate($spType,\%KminusLprimeTable);
    if (!defined $val)
    {
	print STDERR "DEBUG: colors::KminusLprime: lookup failed for spType \"$spType\"\n" if (gcConfig::debug() >= 2);
	return undef;
    }
    return sprintf "%5.3f", $val;
} ## end KminusLprime  #######################################################


return 1;  ## end colors.pm
