package effectiveTemperature;
my $MODVERSION = '$Id: effectiveTemperature.pm,v 1.6 2008/11/12 23:28:22 sgroom Exp $';
use strict;
use warnings;

require 5.000;

use Exporter();

our @ISA = qw(Exporter);
our @EXPORT = qw(effTemp);


##########################################################################
# NASA Exoplanet Science Institute (NExScI) -- http://nexsci.caltech.edu #
#       Copyright (c) 1997-2008 California Institute of Technology       #
#                     U.S. Government Sponsorship                        #
# Under contract with the National Aeronautics and Space Administration  #
##########################################################################


use tableInterpolate;
use parseSpType;
use gcConfig;

my $cfg = gcConfig::config();


# Associative array of Effective Temperature
my %EffTempTable = (
 # Main sequence -- from Tokunaga
 # (found at UKIRT webpage http://lanalana.jach.hawaii.edu/UKIRT.new/astronomy/temp.html

    "O5V" => 40000,
    "O8V" => 38000,
    "O9V" => 33200,
    "O9.5V" => 31450,

    "B0V" => 29700,
    "B1V" => 25600,
    "B2V" => 22300,
    "B3V" => 19000,
    "B4V" => 17200,
    "B5V" => 15500,
    "B6V" => 14100,
    "B7V" => 13000,
    "B8V" => 10700,

    "A0V" => 9480,
    "A2V" => 8810,
    "A5V" => 8160,
    "A7V" => 7930,

    "F0V" => 7020,
    "F2V" => 6750,
    "F5V" => 6530,
    "F7V" => 6240,

    "G0V" => 5930,
    "G2V" => 5830,
    "G4V" => 5740,
    "G6V" => 5620,

    "K0V" => 5240,
    "K2V" => 5010,
    "K4V" => 4560,
    "K5V" => 4340,
    "K7V" => 4040,

    "M0V" => 3800,
    "M1V" => 3680,
    "M2V" => 3530,
    "M3V" => 3380,
    "M4V" => 3180,
    "M5V" => 3030,
    "M6V" => 2850,
    "M8V" => 2400,


# Giant sequence -- again from Tomanaga

    "G0III" => 5910,
    "G4III" => 5190,
    "G6III" => 5050,
    "G8III" => 4960,

    "K0III" => 4810,
    "K1III" => 4610,
    "K2III" => 4500,
    "K3III" => 4320,
    "K4III" => 4080,
    "K5III" => 3980,

    "M0III" => 3820,
    "M1III" => 3780,
    "M2III" => 3710,
    "M3III" => 3630,
    "M4III" => 3560,
    "M5III" => 3420,
    "M6III" => 3250,

# Supergiant sequence -- again from Tomanaga

    "O9I" => 32500,

    "B0I" => 26000,
    "B1I" => 20700,
    "B2I" => 17800,
    "B3I" => 15600,
    "B4I" => 13900,
    "B5I" => 13400,
    "B6I" => 12700,
    "B7I" => 12000,
    "B8I" => 11200,
    "B9I" => 10500,

    "A0I" => 9730,
    "A1I" => 9230,
    "A2I" => 9080,
    "A5I" => 8510,

    "F0I" => 7700,
    "F2I" => 7170,
    "F5I" => 6640,
    "F8I" => 6100,

    "G0I" => 5510,
    "G3I" => 4980,
    "G8I" => 4590,

    "K0I" => 4420,
    "K1I" => 4330,
    "K2I" => 4260,
    "K3I" => 4130,
    "K5I" => 3850,

    "M0I" => 3650,
    "M1I" => 3550,
    "M2I" => 3450,
    "M3I" => 3200,
    "M4I" => 2980,

	   );


#########################################################################
## effTemp -- estimate effective temperature from spectral type
#########################################################################
# Effective temperature subroutine.  This just does a table lookup and
# interpolation into our effective temperature table based on spectral
# type.
#
# The luminosity class must not be blank. Also, because the table
# only contains values for LC I, III, and V, this
# routine treats LC II as LC I, and LC IV as LC V.
#   
# Returns estimated effective temperature on success.
# Returns undef if LC was missing or table interpolation failed.
#
sub effTemp
{
    my $spType = shift @_;

    # check and remap LC's if needed
    my ($major,$minor,$lc) = parseSpType($spType);
    if (!$lc) {
	print STDERR "DEBUG: effectiveTemperature::effTemp: LC is missing for spType \"$spType\"\n" if ($cfg->debug());
	return undef;
    } elsif ($lc eq "II") {
	$lc = "I";
    } elsif ($lc eq "IV") {
	$lc = "V";
    }
    # Reassemble the spectral type
    $spType = join '', $major, $minor, $lc;

    my $val = &tableInterpolate($spType,\%EffTempTable);
    if (!defined $val)
    {
	print STDERR "DEBUG: effectiveTemperature::effTemp: lookup failed for spType \"$spType\"\n" if ($cfg->debug());
	return undef;
    }
    return sprintf "%5.0f", $val;
} ## end effTemp  #######################################################

return 1;
