package gcIrsa;

my $MODVERSION = '$Id: gcIrsa.pm,v 1.11 2006/03/24 02:34:07 sgroom Exp $';

use strict;
use warnings;
use gcConfig;
use astroCatalogue;
use Exporter();
our @ISA = qw(Exporter);
our @EXPORT = qw(search_irsa grab_irsa search_irsa_nl irsaminer_command);

my $cfg = gcConfig::config();
my $delim = ";";

# search_irsa runs an IRSA search and returns an array of results.
#
# Named arguments to this function
# catalog: IRSA catalog name (see irsaminer --catlist)
# dd: IRSA data dictionary name
# cols: Array of requested column names (see irsaminer --ddlist)
# radius: Search radius in arcsec
# target: Target name
# targetRA, targetDEC: Search coordinates in HMS/DMS. There appears to be some
#      leeway here--e.g., they may contain optional colons
#
# The return value is an array containing:
# - A reference to an array of hashes. Each hash represents one result line
#      and maps column names to values.
# - The exit code of the irsaminer command.
sub search_irsa {
    my $args = shift;
    my $cmd = &irsaminer_command($args);
    print STDERR "DEBUG: search_irsa: running command \"$cmd\"\n" if $cfg->debug();
    my $output = `$cmd`;
    my $result = $?;
    print STDERR "DEBUG: search_irsa: result \"$result\" output \"$output\"\n" if $cfg->debug();
    my @datarows;
    if (!$result) {
	my @output_rows = split("\n", $output);
	foreach my $row (@output_rows) {
	    my %rowhash;
	    my @rowdata = split($delim, $row);
	    for (my $i = 0; $i <= $#rowdata; ++$i) {
		$rowhash{$args->{cols}[$i]} = $rowdata[$i];
	    }
	    push(@datarows, \%rowhash);
	}
    }
    return (\@datarows, $result);
}

# This is very much like search_irsa, except that it takes an additional 
# argument containing the number of expected results, and fails noisily if
# the search doesn't return that number.  Use it when you have a pretty good
# idea of what you're expecting from IRSA.
# If no second argument is supplied, the default is to expect 1 result.
#
# This function does not return the exit code of irsaminer, but only the
# result data.
sub grab_irsa {
    my $args = shift;
    my $expected_rows = shift;
    $expected_rows = 1 if (!defined($expected_rows));
    my ($results, $exitcode) = search_irsa($args);
    die sprintf("irsaminer exit code: %d\ncommand was: %s", $exitcode, irsaminer_command($args)) if ($exitcode);
    die sprintf("%d results returned by irsaminer; expected %d", scalar(@$results), $expected_rows) if (scalar(@$results) != $expected_rows);
    return $results;
}

# Search IRSA with a list of target names; try successive target names in
# the list until one works.
# Arguments:
# 1: $args, same as for search_irsa()
# 2-: target names to try
#
# Output is identical to search_irsa()
sub search_irsa_nl {
    my $args = shift;
    my ($results, $exitcode);
    while (defined(my $target = shift)) {
	my %tmpargs = %$args;
	$tmpargs{target} = $target;
	($results, $exitcode) = search_irsa(\%tmpargs);
	if (!$exitcode) {
	    return ($results, $exitcode);
	}
    }
    return ($results, $exitcode);
}

# Given a hashref of arguments identical to those for search_irsa, return the
# command string to invoke irsaminer.
sub irsaminer_command {
    my $args = shift;
    my $cmd = "irsaminer ";
    $cmd = append($cmd, "server", $cfg->irsaHost());
    $cmd = append($cmd, "catalog", $args->{catalog});
    $cmd = append($cmd, "dd", $args->{dd});
    $cmd = append($cmd, "delim", ($args->{delim} || $delim), 1);
    $cmd = append($cmd, "cols", join(',', @{$args->{cols}}), 1);
    $cmd = "$cmd --titles" if ($args->{titles});
    my $within;
    $within = sprintf("%g arcsec", $args->{radius}) if ($args->{radius});
    $cmd = append($cmd, "within", $within, 1);
    my $objstr;
    if ($args->{target}) {
	$objstr = normalIdent($args->{target});
    } elsif ($args->{targetRA} && $args->{targetDEC}) {
	$objstr = sprintf("%s+%s", $args->{targetRA}, $args->{targetDEC});
    }
    $cmd = append($cmd, "objstr", $objstr, 1);
    return $cmd;
}

sub append {
    my $cmd = shift;
    my $arg = shift;
    my $value = shift;
    my $quotevalue = shift;
    if ($value) {
	$value = "\"$value\"" if ($quotevalue);
	$cmd .= sprintf(" --%s=%s", $arg, $value);
    }
    return $cmd;
}


return 1;
