package gcSimbad;
my $MODVERSION = '$Id: gcSimbad.pm,v 1.27 2007/04/23 17:54:48 sgroom Exp $'; 
use strict;
use warnings;

use gcConfig;
use gcDebug;
use miscString;
use coordinates;
use angles;
use gcSimbad4;		# simbad4 interface
use simbadresult;	# getCal simbad result object
use astroCatalogue;
use colors;

use Exporter();
our @ISA = qw(Exporter);
our @EXPORT = qw(searchSimbad simbadReport
	    identsByCatalogue orderedIdentsByCatalogue);

my $cfg = gcConfig::config();

my $pkg = "gcSimbad";	# package name used in error/debug messages

# Given a target name, search SIMBAD(4) return:
# - Normalized target name according to SIMBAD
# - Catalogue ID for Hipparcos
# - Result object from the query
sub searchSimbad4
{
    my $desig = $_[0];

    my $dbgname = $pkg . "::searchSimbad4";

    # SG 12/15/06: testing new SIMBAD code
    #my ($ok, $sr) = gcSimbad4::queryBasic($desig);
    my ($ok, $sr) = gcSimbad4::query($desig,1);	# 1 = "full" query
    if (!$ok)
    {
	print "SIMBAD4 database search failed for designation $desig - unable to query SIMBAD service\n";
	return undef, undef;
    }
    elsif (!$sr)
    {
	# query succeeded - but SIMBAD returned error
	print "SIMBAD4 database search failed for designation $desig - SIMBAD query returned error\n";
	return undef, undef;
    }

    # we should have a good result
    if (gcConfig::debug())
    {
	gcDebug $dbgname, 1, "desig   \"",($sr->desig() || ""),"\"\n";
	gcDebug $dbgname, 1, "HD      \"",($sr->hd() || ""),"\"\n";
	gcDebug $dbgname, 1, "HIP     \"",($sr->hip() || ""),"\"\n";
	gcDebug $dbgname, 1, "Name    \"",($sr->name() || ""),"\"\n";
	gcDebug $dbgname, 1, "descr   \"",($sr->description() || ""),"\"\n";
	gcDebug $dbgname, 1, "raHMS   \"",($sr->raHMS() || ""),"\"\n";
	gcDebug $dbgname, 1, "decDMS  \"",($sr->decDMS() || ""),"\"\n";
	gcDebug $dbgname, 1, "raDeg   \"",($sr->raDeg() || ""),"\"\n";
	gcDebug $dbgname, 1, "decDeg  \"",($sr->decDeg() || ""),"\"\n";
	gcDebug $dbgname, 1, "raHMS1950   \"",($sr->raHMS1950() || ""),"\"\n";
	gcDebug $dbgname, 1, "decDMS1950  \"",($sr->decDMS1950() || ""),"\"\n";
	gcDebug $dbgname, 1, "raDeg1950   \"",($sr->raDeg1950() || ""),"\"\n";
	gcDebug $dbgname, 1, "decDeg1950  \"",($sr->decDeg1950() || ""),"\"\n";
	gcDebug $dbgname, 1, "pmra    \"",($sr->pmra() || ""),"\"\n";
	gcDebug $dbgname, 1, "pmdec   \"",($sr->pmdec() || ""),"\"\n";
	gcDebug $dbgname, 1, "spec    \"",($sr->spec() || ""),"\"\n";
	gcDebug $dbgname, 1, "plx     \"",($sr->plx() || ""),"\"\n";
	gcDebug $dbgname, 1, "plxfull \"",($sr->plxfull() || ""),"\"\n";
	gcDebug $dbgname, 1, "vmag    \"",($sr->vmag() || ""),"\"\n";
	gcDebug $dbgname, 1, "bmag    \"",($sr->bmag() || ""),"\"\n";
    }
    return ($sr);
}

# Given a target name, search SIMBAD and return:
# - Normalized target name according to SIMBAD
# - Catalogue ID for Hipparcos
# - Result object from the query
sub searchSimbad
{
    my $desig = normalIdent($_[0]);

    my $dbgname = $pkg . "::searchSimbad";
    
    my $simhost = $cfg->simbadHost;
    gcDebug $dbgname, 1, "using SIMBAD host $simhost\n";

    # if more than one version of SIMBAD was supported, the logic
    # to decide which version to use would go here.

    my $sr4 = searchSimbad4($desig);
    return $sr4;
}


# Prepare a text format report of SIMBAD output for
# the given designation. If a simbadresult object is provided
# as the second argument, that information is used to format
# the report. Otherwise, a SIMBAD query is performed.
#
# The $optsRef argument is a reference to a hash containing options,
# described below
sub simbadReport
{
    my $searchdesig = shift;	# name used for the search
    my $obj = shift;		# SIMBAD search result object
    my $optsRef = shift;	# options hash

    my $dbgname = $pkg . "::simbadReport";
    
    # make sure we have the SIMBAD data
    if (!defined $obj)
    {
	print STDERR "$dbgname: no object supplied\n";
	return undef;
    }

    ### examine options
    # (we'll avoid referencing the options hash below this section)
    dumpopts($dbgname,$optsRef) if gcConfig::debug();	# debugging dump of options passed in
    my $opt_Kmag = $optsRef->{Kmag};	# estimate K from spec type and show on summary line
    my $opt_commonNames = $optsRef->{commonNames}; # show common names on summary line
    my $opt_identifiers = $optsRef->{identifiers}; # show all identifiers
    my $opt_HD = $optsRef->{HD};	# show HD identifier
    my $opt_HIP = $optsRef->{HIP};	# show HIP identifier
    my $opt_SAO = $optsRef->{SAO};	# show SAO identifier
    my $opt_FK5 = $optsRef->{FK5};	# show FK5 identifier
    my $opt_meas = $optsRef->{meas};	# show measurements

    # extract some useful stuff
    my $desig = ($obj->desig() || "?");		# preferred desig
    my $descr = ($obj->description() || "");	# short description given by SIMBAD
    my $spec = ($obj->spec() || "");		# object spec type
    my $vmag = ($obj->vmag() || "?");		# object V mag

    #### Prepare the formatted report
    my @report;
    push @report, "## Simbad search results for $searchdesig:\n";
    
    # do the summary line
    my $summary = "Simbad Search $searchdesig: $desig -- $descr  $spec V=$vmag";
    if ($opt_Kmag)		# append K mag to summary line
    {
	my $vmk = VminusK($spec);
	if (defined $vmk)
	{
	    my $kmag = sprintf "%.2f", $vmag - $vmk;
	    $summary .= " K=$kmag";
	}
    }
    if ($opt_commonNames)	# append common names to summary line
    {
	# search list of identifers for ones containing '*' or 'NAME'
	# and append them to the summary line
	my @idlines = $obj->ident();
	foreach my $id (grep (m/\*|NAME/, @idlines))
	{
	    $id =~ s/^\s+|\s+$//g;	# remove leading & trailing whitespace
	    $id =~ s/\s+/ /g;		# collapse internal whitespace
	    $summary .= " $id";	# append to summary line
	}
    }
    $summary .= "\n";
    push @report, $summary;
    
    # do the notes
    my @noteslines = $obj->notes();
    if (@noteslines)
    {
	push @report, "==========\n";
	push @report, "Notes:\n";
	foreach my $line (@noteslines)
	{
	    $line .= "\n";
	    push @report, $line;
	}
    }

    # do the identifiers list
    my @identlines;
    if ($opt_identifiers)
    {
	# display all identifiers
	@identlines = $obj->ident();
    }
    else
    {
	# display identifiers from selected catalogs only
	my @catalogs;
	if ($opt_HD) { push @catalogs, "HD"; }
	if ($opt_HIP) { push @catalogs, "HIP"; }
	if ($opt_SAO) { push @catalogs, "SAO"; }
	if ($opt_FK5) { push @catalogs, "FK5"; }
	if (@catalogs)
	{
	    @identlines = identsByCatalogue(join('|',@catalogs), $obj->ident());
	}
    }
    if (@identlines)
    {
	push @report, "==========\n";
	push @report, "Identifiers:\n";
	foreach my $line (@identlines)
	{
	    $line .= "\n";
	    push @report, $line;
	}
	push @report, "\n";
    }

    # do the measurements
    if ($opt_meas)
    {
	push @report, "==========\n";
	push @report, "Measurements:\n";
	gcDebug $dbgname,1,"meas: \"",$obj->meas(),"\"\n";
	my @measlines = $obj->meas();
	foreach my $line (@measlines)
	{
	    $line .= "\n";
	    push @report, $line;
	}
    }

    #push @report, $raw;
    #push @report, "\n";

    return @report;
}


# Filter a list of identifiers by catalogue. Arguments are a string
# containing one or more catalogue names separated by pipes
# followed by a list of target names.
sub identsByCatalogue {
    my $catalogues = shift;
    my $matchexpr = "^($catalogues)";
    return grep /$matchexpr/, @_;
}

# Like identsByCatalogue, but also sort the returned values in order of their
# position in the catalogue list
sub orderedIdentsByCatalogue {
    my $catstr = $_[0];
    my @catlist = split /\|/, $catstr;
    my @tmpIdents;
    foreach my $ident (&identsByCatalogue(@_)) {
	my @pieces = &identPieces($ident);
	my $catalogue = $pieces[0];
	my $tmpEntry = sprintf("Z%04d|%s", index($catstr, $catalogue), $ident);
	push @tmpIdents, $tmpEntry;
    }
    my @idents;
    foreach my $tmp (sort @tmpIdents) {
	my @sections = split /\|/, $tmp;
	push @idents, $sections[1];
    }
    return @idents;
}
