package gcSimbad3;
my $MODVERSION = '$Id: gcSimbad3.pm,v 1.3 2007/03/02 22:38:05 sgroom Exp $';

# getCal routines for interacting with SIMBAD3
# Most were formerly contained in gcSimbad.pm, but have been relocated here since
# they are specific to SIMBAD3 and may go away someday maybe-soon.

use strict;
use warnings;
use gcConfig;
use Astro::SIMBAD::Query;
use astroCatalogue;
use miscString;
use coordinates;
use angles;
use gcDebug;
use gcSimbad; # orderedIdentsByCatalogue

use Exporter();
our @ISA = qw(Exporter);
our @EXPORT = qw(prepSimbadQuery querySimbadCheckId checkSimbadResult
		 simbadLookupIdent
		 SIMBAD_ASCII_LINE_WIDTH
		 );

my $cfg = gcConfig::config();
my $pkg = "gcSimbad3";		# package name used in error/debug messages

use constant SIMBAD_ASCII_LINE_WIDTH => 248;

# prepSimbadQuery initializes an Astro::SIMBAD::Query object.
# It takes a named argument list--see the documentation for the
# Astro::SIMBAD::Query constructor at
# http://search.cpan.org/~aallan/Astro-SIMBAD-2.0.0/Query/Query.pm.
sub prepSimbadQuery {
    my %opts = @_;
    my %defaults = (Frame => 'FK5',
		    Epoch => 2000,
		    Equinox => 2000,
		    URL => $cfg->simbadHost(),
		    Timeout => $cfg->httpTimeout(),
		    Proxy => $cfg->httpProxy()
		    );

    my $dbgname = $pkg . "::prepSimbadQuery";

    foreach my $param (keys %defaults) {
	if (!$opts{$param} && $defaults{$param}) {
	    $opts{$param} = $defaults{$param};
	}
    }

    # Astro::SIMBAD doesn't appear to be doing this...
    $opts{Target} = urlQuote($opts{Target}) if ($opts{Target});

    # Make it an object-type query unless explicitly stated otherwise
    if ($opts{'Target'} && !$opts{'Error'}) {
	$opts{'Error'} = 0;
    }
    return new Astro::SIMBAD::Query(%opts);
}

# Given a target ID, run a query against Simbad and validate that the
# returned target matches the requested target.
# Inputs: target identifier
# Outputs: Astro::SIMBAD::Result, returned target name, raw query results as text
# If the returned target name matches, the name will be identical to the
# target ID passed into the function.
sub querySimbadCheckId {
    my $targetId = shift;
    my $dbgname = $pkg . "::querySimbadCheckId";

    gcDebug $dbgname, 2, "prep query for targetId \"$targetId\"\n";
    my $query = prepSimbadQuery(Target => $targetId);
    my $url = $query->queryurl();
    gcDebug $dbgname, 1, "query url $url\n";
    my $objname = "";
    my $report = "";
    my $result;
    gcDebug $dbgname, 2, "issue query for targetId \"$targetId\"\n";
    eval { $result = $query->querydb(); };
    gcDebug $dbgname, 2, "after query for targetId \"$targetId\"\n";
    my $ok = checkSimbadResult($result);
    if (!$ok)
    {
	gcDebug $dbgname, 1, "query failed for targetId \"$targetId\"\n";
	return (undef, "", "");
    }
    gcDebug $dbgname, 2, "after checkSimbadResult\n";
    if (($ok) && ($result->objects())) {
	my @objects = $result->objects();
	my $obj = $objects[0];
	$objname = normalIdent($obj->name());
	if (normalIdent($targetId) eq $objname) {
	    $objname = $targetId; # Reset to target to provide easy check
	}
	$report = html2txt($query->{BUFFER});
    }
    return ($result, $objname, $report);
}

# Check the result object returned from a SIMBAD search
sub checkSimbadResult
{
    my $result = shift;
    if (!defined($result)) {
	return 0;	# error
    }
    return 1;		# ok
}

# Given an identifier and an ordered list of preferred catalogues, do a Simbad
# lookup and return the catalogue identifier that best matches your request.
# Argument	Meaning
# 0		Identifier to use in the Simbad search
# 1		Pipe-delimited list of catalogues (e.g., "HIP|HD")
#
# The return value is a two-valued list:
#
# Return value	Meaning
# 0		An identifier from one of the requested catalogues, or a blank if no match was found
# 1		Simbad's preferred identifier for the object
#
# Be warned, though:  Timewise, this function is rather expensive!
sub simbadLookupIdent {
    my $inIdent = &normalIdent(shift);
    my $catalogs = shift;
    my ($result, $outIdent) = &querySimbadCheckId($inIdent);
    if ($outIdent) {
	my (@objects, $obj, @idents);
	@objects = $result->objects();
	$obj = $objects[0] if (@objects);
	@idents = &orderedIdentsByCatalogue($catalogs, @{$obj->ident()}) if $obj;
	return ($idents[0], $outIdent) if (@idents);
    }
    return ("", $outIdent);
}

# Return proper motion in arcsec/year; also normalize to account for Simbad's
# representation of pmRA as milli-RA arcsec/year--has to be divided by cos(DEC)
# to get "real" pmRA mas/year
# Input: Simbad result object
# Output: Array of (pmRA, pmDEC) in arcsec/year
sub simbad2NormalPM
{
    my $simbadObj = shift;
    my $pm_in = $simbadObj->pm();
    my @pm_out;
    if (@$pm_in) {
	my $dec_degrees = simbad2DECdeg($simbadObj);
	my $dec_radians = $dec_degrees * angles::DEG_TO_RAD;
	$pm_out[0] = $pm_in->[0] / (1000.0 * cos($dec_radians));
	$pm_out[1] = $pm_in->[1] / 1000.0;
    }
    return @pm_out;
}
