package gcSimbad4;
my $MODVERSION = '$Id: gcSimbad4.pm,v 1.16 2007/06/25 17:40:54 sgroom Exp $';
#
# Routines for interacting with SIMBAD4
# SLG 1/31/2007
#

use strict;
use warnings;

use LWP::UserAgent;
#use URI::Escape;

use simbadresult;
use coordinates;
use angles;
use miscString;
use gcConfig;
use gcDebug;
use Exporter();
use astroCatalogue;

our @ISA=qw(Exporter);
our @EXPORT=qw(query);

my $cfg = gcConfig::config();
my $pkg = "gcSimbad4";				# package name for error/debug messages
my $timeout = 30;

#############################
#
# Public Methods
#
#############################

# query: perform SIMBAD query by identifier
sub query
{
    my $searchdesig = shift;	# identifier string to lookup
    my $qType = shift;	# type of query: 0=basic, 1=full

    my $dbgname = $pkg . "::query";	# function name for debug messages


    if (!$searchdesig)
    {
	print STDERR "$dbgname: no searchdesig supplied\n";
	return 0, undef;
    }

    my $cacheFileFound = 0;	# remember whether we found entry in the cache
    # use the designation as the cache filename, after doing some cleanup.
    # be careful to avoid potentially dangerous characters in the filename.
    my $filedesig = &sansSpaces($searchdesig);	# filename will have spaces replaced by underscore
    $filedesig =~ s/\.\.//g;	# security: remove dot-dot from filenames
    $filedesig =~ s/\///g;	# security: remove / from filenames
    $filedesig =~ s/\~//g;	# security: remove ~ from filenames
    $filedesig =~ s/\*//g;	# security: remove * from filenames
    $filedesig =~ s/\+//g;	# security: remove + from filenames
    $filedesig =~ s/\\//g;	# security: remove \ from filenames
    
    my $content;	# where we will store the query's response content
    my $cachedir = $cfg->simbadCacheDir() || ""; 
    gcDebug $dbgname, 1, "SIMBAD cache dir set to \"$cachedir\"\n";
    if ($cachedir)	# check for cached query if dir is defined
    {
	# Instead of doing a live SIMBAD query,
	# look for a previously cached query matching this desig
	my $filename = $cachedir . "/" . $filedesig;
	gcDebug $dbgname, 1, "looking for SIMBAD result cache filename \"$filename\"\n";
	if (open CACHEFILE, "<", $filename)
	{
	    $cacheFileFound = 1;	# note that file was found
	    gcDebug $dbgname, 1, "loading cached query result for desig \"$searchdesig\"\n";
	    my @lines;
	    push @lines, <CACHEFILE>;
	    foreach my $line (@lines)
	    {
		$content .= $line;
	    }
	    #$content = join /\n/, @lines;	# read the whole file as giant string
	    close CACHEFILE;
	    gcDebug $dbgname, 1, "got cached query result:\n",$content,"\n";
	}
	else
	{
	    if ($cfg->simbadCacheOnly())
	    {
		# the "cache only" option was set, meaning lookup failures should not fall
		# back to doing a live query.
		print STDERR "GC_SIMBAD_CACHEONLY is set and cache filename \"$filename\" not found for desig \"$searchdesig\", giving up\n";
		return 0, undef;	# not found in the cache, lookup failed.
	    }
	    else
	    {
		# not found in the cache, but we'll fall through and do a live query.
		gcDebug $dbgname, 1, "cache filename \"$filename\" not found for desig \"$searchdesig\", doing live query\n";
	    }
	}
    }

    if (!$content)
    {
	# do a live SIMBAD query
	# get URL to use
	my $url = _composeQuery($searchdesig,$qType);
	gcDebug $dbgname, 1, "getting url: $url\n";

	# do lookup at that URL
	my ($ok, $response) = _makeQuery($url);
	if (!$ok)
	{
	    # query failed. XXX NEED TO DISTINGUISH HTTP FAILURE FROM SIMBAD FAILURE
	    gcDebug $dbgname, 1, "_makeQuery failed\n";
	    return 0, undef;
	}
	gcDebug $dbgname, 1, "got response:\n",$response->content,"\n";
	$content = $response->content;
    }

    if ($cachedir && (!$cacheFileFound))
    {
	# cache option was used, but query was not found in cache,
	# so put it there now
	my $filename = $cachedir . "/" . $filedesig;
	if (open CACHEFILE, ">", $filename)
	{
	    gcDebug $dbgname, 1, "caching SIMBAD result to cache filename \"$filename\"\n";
	    print CACHEFILE $content;
	    close CACHEFILE;
	}
	else
	{
	    gcDebug $dbgname, 1, "unable to cache SIMBAD result to cache filename \"$filename\"\n";
	}
    }

    # Parse the response content into a result object
    my $obj = _parseResponse($content,$qType);
    return 1, $obj;
}

#############################
#
# Private Methods
#
#############################

# _composeQuery: form URL to do a SIMBAD query
# takes identifier to use and query type (0=basic, 1=full)
# returns URL
sub _composeQuery
{
    my $searchdesig = shift;
    my $qtype = shift || 0;

    my $dbgname = $pkg . "::_composeQuery";	# function name for debug messages

    # create SIMBAD query script
    # The format string here must match that expected by _parseResponse()
    my $script;
    if ($qtype == 1)
    {
	gcDebug $dbgname, 1, "composing full query\n";
	# "full" query script
	# This is the same as the "basic" script with the addition of notes and measurements.
	$script = "format object \"OBJECT: %IDLIST(HD)|%IDLIST(HIP)|%IDLIST(NAME,HD,HIP,1)|%OTYPE(V)|" .
	    "%COO(;A,D;ICRS;J2000;2000)|%COO(;A,D;FK4;B1950;1950)|%PM(A,D)|%FLUXLIST[%*(N=F U,)]|%SP(S)|%PLX" .
	    #"\\nMEASUREMENTS:\\n%MEASLIST[%*(HA)]" .
	    "\\n::IDENTIFIERS::\\n%IDLIST" .
	    "\\n::NOTES::\\n%NOTELIST" .
	    "\\n::MEASUREMENTS::\\n%MEASLIST[%*(HA)]" .
	    "\"\n";
    }
    else
    {
	gcDebug $dbgname, 1, "composing basic query\n";
	# "basic" query script
	$script = "format object \"OBJECT: %IDLIST(HD)|%IDLIST(HIP)|%IDLIST(NAME,HD,HIP,1)|%OTYPE(V)|" .
	    "%COO(;A,D;ICRS;J2000;2000)|%COO(;A,D;FK4;B1950;1950)|%PM(A,D)|%FLUXLIST[%*(N=F U,)]|%SP(S)|%PLX" .
	    "\"\n";
    }
    $script .= "query id $searchdesig\n";

    # escape any unsafe characters in the script so they can be safely encoded as a URL
    #my $safeScript = URI::Escape::uri_escape($script);
    my $safeScript = urlQuote($script);
    gcDebug $dbgname, 1, "script=     \"$script\"\n";
    gcDebug $dbgname, 1, "safescript= \"$safeScript\"\n";

    # form a complete URL from the script
    my $simhost = $cfg->simbadHost();
    my $url = "http://$simhost/simbad/sim-script?script=$safeScript";
    # gcDebug $dbgname, 1, "url=\"$url\"\n";

    return $url;
}

# _parseResponse: populate a result object from a SIMBAD response
# takes: response content returned by LWP::UserAgent::get()
# returns: simbadresult object populated with information returned by SIMBAD
sub _parseResponse
{
    my $content = shift;	# text of query response
    my $qtype = shift || 0;	# query type 0=basic (default), 1=full. NOT CURRENTLY USED

    my $dbgname = $pkg . "::_parseResponse";	# function name for debug messages

    # The SIMBAD return information is divided into sections, each
    # introduced by a section header that looks like:
    # 	::script:::::::::::::::::::::::::::::::::::::::::::::::::
    # where "script" is the name of the section.
    # Valid section names are:
    #	script - the script we sent to SIMBAD
    #	console - normal messages generated during the query
    #	error - error messages returned by SIMBAD
    #	data - actual query results with format as we specified in _composeQuery
    # The last two we use are not true SIMBAD sections, they are
    # created by the formatting we requested in the query
    #	IDENTIFIERS - object identifiers
    #	NOTES - object notes
    #	MEASUREMENTS - object measurements

    # sort response lines into sections
    my @responselines = split /\n/, $content;
    my $section = 0;
    my @scriptlines;
    my @consolelines;
    my @errorlines;
    my @objectlines;
    my @identlines;
    my @noteslines;
    my @measurementlines;
    foreach my $line (@responselines)
    {
	next if (!$line);	# skip blank lines
	if ($line =~ /^::/)	# section header line?
	{
	    # section header - decide which
	    if ($line =~ /^::script::/)
	    {
		$section = 1;
	    }
	    elsif ($line =~ /^::console::/)
	    {
		$section = 2;
	    }
	    elsif ($line =~ /^::error::/)
	    {
		$section = 3;
	    }
	    elsif ($line =~ /^::data::/)
	    {
		$section = 4;
	    }
	    elsif ($line =~ /^::IDENTIFIERS::/)
	    {
		$section = 5;
	    }
	    elsif ($line =~ /^::NOTES::/)
	    {
		$section = 6;
	    }
	    elsif ($line =~ /^::MEASUREMENTS::/)
	    {
		$section = 7;
	    }
	    else
	    {
		# warn and ignore it?
		print STDERR "gcSimbad4::_parseResponse: unrecognized section header line \"$line\"\n";
	    }
	}
	else
	{
	    # non-header line - add to correct section
	    if ($section == 1)		# ::script::
	    {
		push @scriptlines, $line;
	    }
	    elsif ($section == 2)	# ::console::
	    {
		push @consolelines, $line;
	    }
	    elsif ($section == 3)	# ::error::
	    {
		push @errorlines, $line;
	    }
	    elsif ($section == 4)	# ::data::
	    {
		push @objectlines, $line;
	    }
	    elsif ($section == 5)	# ::IDENTIFIERS:: (subsection of data)
	    {
		push @identlines, $line;
	    }
	    elsif ($section == 6)	# ::NOTES:: (subsection of data)
	    {
		push @noteslines, $line;
	    }
	    elsif ($section == 7)	# ::MEASUREMENTS:: (subsection of data)
	    {
		push @measurementlines, $line;
	    }
	    else
	    {
		# warn and ignore it?
		print STDERR "simbadQuery::_parseResponse: unknown section has line \"$line\"\n";
	    }
	}
    }
    # now we have all lines grouped into four arrays, one for each section.

    # check for any error messages. We don't want to see any.
    if (@errorlines)
    {
	# SIMBAD returned an error, so the query has failed.
	# print the message on STDERR and return.
	print STDERR "simbadQuery::_parseResponse: SIMBAD returned error:\n";
	foreach my $line (@errorlines)
	{
	    print STDERR "simbadQuery::_parseResponse: errormsg: $line\n";
	}
	return undef;
    }

    #####################
    # The query succeeded.
    # Extract the interesting parts of the result

    # First, any  "OBJECT:" lines.
    # If multiple lines are returned, we use the first one only.

    if ($cfg->debug())
    {
	foreach my $oline (@objectlines)
	{
	    gcDebug $dbgname, 1, "got object line: \"$oline\"\n";
	}
    }
    my $objectline  = $objectlines[0];
    if (!defined $objectline)
    {
	print STDERR "simbadQuery::parseResponse: no object information found in SIMBAD results\n";
	return undef;
    }
    $objectline =~ s/^OBJECT: //;

    #XXX TEST:
    $objectline =~ s/~//g;	# '~' is simbad's way of representing a missing value, just get rid of any

    # Parse the object data line and
    # build a simbadresult object with that info.
    # Do some simple string cleanup first
    $objectline =~ s/^\s+|\s+$//;	# zap leading & trailing space
    $objectline =~ s/\s*([|,])\s*/$1/g;	# zap space adjacent to field separators
    $objectline =~ s/\s+/ /g;	# collapse internal whitespace


    # major field separator is '|'
    my @fields = split /\|/,$objectline;
    my $hd = $fields[0];	# HD identifier
    my $hip = $fields[1];	# HIP identifier
    my $name = $fields[2];	# NAME identifier, or HD, or HIP, or anything available (in that order)
    my $description = $fields[3];	# description string
    my $coordStr = $fields[4];		# coordinates (ICRS, epoch J2000, equi 2000.0)
    my $coordStr1950 = $fields[5];	# coordinates (FK4, epoch B1950, equi 1950.0)
    my $pmStr = $fields[6];	# PM RA and DEC
    my $fluxdata = $fields[7];	# list of "flux" values (actually mags)
    my $spec = $fields[8];	# spec type
    my $plxfull = trimWhite($fields[9]);	# plx,qual,err,ref of the form "65.10 A [0.76] A 1997A&A...323L..49P"

    # some cleanup on information returned
    $name =~ s/^NAME\s*//;	# strip any "NAME " prefix from returned name
    
    # now construct the return object
    # handle the simple stuff first
    my $obj = new simbadresult;
    $obj->raw($content);	# store the raw output from the SIMBAD query
    $obj->desig($hd || $hip || $name);	# set preferred designation: HD/HIP/NAME, in that order
    $obj->hd($hd) if (defined $hd);
    $obj->hip($hip) if (defined $hip);
    $obj->name($name) if (defined $name);
    $obj->description($description) if (defined $description);
    $obj->spec($spec) if (defined $spec);
    if ((defined $plxfull) &&
	($plxfull =~ /\S+/) &&
	($plxfull ne "[]"))	# avoid using an empty field (after the ~'s have been removed)
    {
	my ($plx,undef) = split /\s+/,$plxfull;	# pick off first field only
	$obj->plx($plx) if ((defined $plx) && ($plx =~ /\d+/));
	$obj->plxfull($plxfull);
    }

    # parse coord string, a pair of sexgesimal values separated by a comma
    # ICRS 2000.0
    my ($raHMS, $decDMS) = split /,/, $coordStr;
    $obj->raHMS($raHMS) if (defined $raHMS);	# stash HMS
    $obj->decDMS($decDMS) if (defined $decDMS); # stash DMS
    $obj->raDeg(RAdeg($raHMS)) if (defined $raHMS);	# convert HMS to deg
    $obj->decDeg(DECdeg($decDMS)) if (defined $decDMS);	# convert DMS to deg

    # FK4 1950.0/1950.0
    my ($raHMS1950, $decDMS1950) = split /,/, $coordStr1950;
    $obj->raHMS1950($raHMS1950) if (defined $raHMS1950);	# stash HMS
    $obj->decDMS1950($decDMS1950) if (defined $decDMS1950); # stash DMS
    $obj->raDeg1950(RAdeg($raHMS1950)) if (defined $raHMS1950);	# convert HMS to deg
    $obj->decDeg1950(DECdeg($decDMS1950)) if (defined $decDMS1950);	# convert DMS to deg

    # parse PM string, two floats separated by comma
    my ($pmraSimbad, $pmdecSimbad) = split /,/, $pmStr;
    if ($pmraSimbad && $pmdecSimbad)
    {
	# Before storing the value returned by Simbad,
	# need to convert from milliarcsec/yr to arcsec/year.
	# Also need to correct PMRA for declination to get "real" pmRA arcsec/year
	my $decRadians = $obj->decDeg() * angles::DEG_TO_RAD;
	my $pmra = $pmraSimbad / 1000.0 / cos($decRadians);
	my $pmdec = $pmdecSimbad / 1000.0;
	$obj->pmra($pmra);
	$obj->pmdec($pmdec);
    }

    # parse "fluxdata"
    # SIMBAD calls these values "fluxes" but they are usually mags.
    # this is a list of "X=v u" separated by commas, where
    # 'X' is band, 'v' is value, 'u' is units
    flux: foreach my $flux (split /,/,$fluxdata)
    {
	next flux if ($flux !~ m/\S/); # ignore blank flux items

	my ($band,$rest) = split /=/, $flux;
	my ($val,$unit) = split / /, $rest;
	if ($unit ne "mag")
	{
	    print STDERR "simbadQuery::_parseResponse: ignoring non-mag flux value $flux\n";
	    next flux;
	}
	if ($band eq "B")
	{
	    $obj->bmag($val);
	} elsif ($band eq "V")
	{
	    $obj->vmag($val);
	}
    }

    #####################
    # Process any identifier lines that were returned.
    if (@identlines)
    {
	if ($cfg->debug())
	{
	    foreach my $line (@identlines)
	    {
		gcDebug $dbgname, 1, "got ident line: \"$line\"\n";
	    }
	}
	$obj->ident(@identlines);
    }

    #####################
    # Process any notes lines that were returned.
    if (@noteslines)
    {
	if ($cfg->debug())
	{
	    foreach my $line (@noteslines)
	    {
		gcDebug $dbgname, 1, "got notes line: \"$line\"\n";
	    }
	}
	$obj->notes(@noteslines);
    }

    #####################
    # Process any measurement lines that were returned.

    if (@measurementlines)
    {
	if ($cfg->debug())
	{
	    foreach my $line (@measurementlines)
	    {
		gcDebug $dbgname, 1, "got measurement line: \"$line\"\n";
	    }
	}
	$obj->meas(@measurementlines);
    }

    return $obj;	# return the simbadresult object we created
}


# _makeQuery: Send HTTP query to SIMBAD using the URL provided
# Takes URL to invoke
# Returns:
# 	- success flag (0=failure, 1=success)
# 	- on success, response object returned by LWP::UserAgent::get()
sub _makeQuery
{
    my $url = shift; 	# url to query

    my $dbgname = $pkg . "::_makeQuery";	# function name for debug messages

    # set up user agent
    my $timeout = $cfg->httpTimeout();
    my $ua = new LWP::UserAgent( timeout => $timeout );
    $ua->agent("gcSimbad4/0.0");	# set agent ident string that will be sent along with HTTP request
    $ua->env_proxy();			# respect HTTP_PROXY env var

    # send the request and wait for response
    my $response = $ua->get($url);
    gcDebug $dbgname, 1, "request returned ", $response->code, " : ", $response->message,"\n";

    # check for query errors
    if ($response->is_error)
    {
	gcDebug $dbgname, 1, "request returned error: ", $response->code, " : ", $response->message,"\n";
	return 0, undef;	# error return
    }
    return 1, $response;
}
