package geometry;
my $MODVERSION = '$Id: geometry.pm,v 1.7 2008/11/12 23:28:22 sgroom Exp $';
use strict;
use warnings;
require 5.000;

use Exporter();

our @ISA = qw(Exporter);
our @EXPORT = qw(zenithAngle zaLimit azAngle azFromEl unitStar precession);

##########################################################################
# NASA Exoplanet Science Institute (NExScI) -- http://nexsci.caltech.edu #
#       Copyright (c) 1997-2008 California Institute of Technology       #
#                     U.S. Government Sponsorship                        #
# Under contract with the National Aeronautics and Space Administration  #
##########################################################################

## geometry.pm  #########################################################

use POSIX;

use angles;
use vector;


#########################################################################
## zenithAngle -- compute the zenith angle (deg) of a star given HA, dec, lat (deg)
#########################################################################
sub zenithAngle
{
## declarations zenithAngle  ############################################    
    my $ha = shift @_;	# hour angle, deg)
    my $dec = shift @_;
    my $lat = shift @_;

## begin zenithAngle  ###################################################
    my $haR = $ha * DEG_TO_RAD;
    my $decR = $dec * DEG_TO_RAD;
    my $latR = $lat * DEG_TO_RAD;

    my $cos_za = + sin($latR) * sin($decR) + cos($latR) * cos($haR) * cos($decR);
    my $za;
    if ($cos_za > 1.0)
    {
	print STDERR "zenithAngle: ha=$ha, dec=$dec, lat=$lat, cos_za = $cos_za\n";
	$za = 0.0;
    }
    elsif ($cos_za < -1.0)
    {
	print STDERR "zenithAngle: ha=$ha, dec=$dec, lat=$lat, cos_za = $cos_za\n";
	$za = 180.0;	# converted to degrees
    }
    else
    {
	$za = POSIX::acos($cos_za) * RAD_TO_DEG;
    }
    return $za;
} ## end zenithAngle  ###################################################


#########################################################################
## zaLimit -- compute hour angle(degrees) corresponding to given dec/lat/za
#########################################################################
sub zaLimit
{
    # zaLimit finds the transition time into the acceptable zenith
    # angle region in a particular temporal direction.  This
    # implementation is stupid in that it needs to be protected from
    # invocation if a solution doesn't exist.

    # SG 6/2006: Replaced algorithm with a direct calculation method
    # instead of the iterative method the older version was using.
## declarations zaLimit  ################################################
    my $dec = shift @_;		# target dec (deg)
    my $lat = shift @_;		# observer lat (deg)
    my $za = shift @_;		# zenith angle we're looking for (deg)
    my $direction = shift @_;	# which side of transit: -1 = rising, +1 = setting
## begin zaLimit  #######################################################

    my $elR = (90.0 - $za) * DEG_TO_RAD;
    my $decR = $dec * DEG_TO_RAD;
    my $latR = $lat * DEG_TO_RAD;

    my $cos_ha = (sin($elR) - (sin($decR) * sin($latR))) / (cos($decR) * cos($latR));
    my $ha;
    if ($cos_ha > 1.0)
    {
	# TODO: fix this
	die "zaLimit: cos out of range: dec=$dec, lat=$lat, za=$za, cos_ha = $cos_ha\n";
    }
    elsif ($cos_ha < -1.0)
    {
	# TODO: fix this
	die "zaLimit: cos out of range: dec=$dec, lat=$lat, za=$za, cos_ha = $cos_ha\n";
    }
    else
    {
	$ha = POSIX::acos ($cos_ha) * RAD_TO_DEG;
    }
    $ha *= $direction;	# set sign (east or west of meridian)
    return $ha;
} ## end zaLimit  #######################################################


#########################################################################
## azAngle -- compute the azimuth angle (deg) of a star given ha, dec, lat (deg)
#########################################################################
sub azAngle
{
## declarations azAngle  ############################################    
    my $ha = shift @_;	# star hour angle in degrees
    my $dec = shift @_;		# star declination in degrees
    my $lat = shift @_;		# observer latitude in degrees

## begin azAngle  ###################################################
    my $haR = $ha * DEG_TO_RAD;
    my $decR = $dec * DEG_TO_RAD;
    my $latR = $lat * DEG_TO_RAD;

    my @ustar = unitStar($haR, $decR, $latR);	# convert to ENU
    my $azR = POSIX::atan2($ustar[0],$ustar[1]); # atan2(east, north)
    my $az = $azR * RAD_TO_DEG;
    if ($az < 0.0)
    {
	$az += 360.0;
    }
    elsif ($az > 360.0)
    {
	$az -= 360.0;
    }

    # TESTING: could we still fall victim to rounding errors?
    if ($az < 0.0)
    {
	print STDERR "azAngle: ERROR: AZ<0: $az\n";
    }
    elsif ($az > 360.0)
    {
	print STDERR "azAngle: ERROR: AZ>360: $az\n";
    }

    return $az;
} ## end azAngle  ###################################################


#########################################################################
## unitStar -- compute ENU unit vector for a star
#########################################################################
sub unitStar
{
    ## unitStar computes a unit vector in the ENU coordinate system
    ## for a star at hour angle $hrAngle, $dec, as viewed from a
    ## location at latitude $lat.  All arguments are assumed in
    ## radians.

## declarations unitStar  ###############################################
    my $hrAngle = shift @_;
    my $dec = shift @_;
    my $lat = shift @_;

    my @uStar = (-sin($hrAngle) * cos($dec),
		 cos($lat) * sin($dec) - sin($lat) * cos($hrAngle) * cos($dec),
		 + sin($lat) * sin($dec) + cos($lat) * cos($hrAngle) * cos($dec));

    return @uStar;
} ## end unitStar  ######################################################




#########################################################################
## precession
#########################################################################
sub precession
{
    ## precession computes the apparent position at an arbitrary epoch
    ## (Julian Date) given input positions referenced to J2000.
    ## Algorithm is taken from Astrophysical Quantities Fourth Edition
    ## section 27.3 (contributed by A.D. Fiala)

    ## Inputs are the J2000 reference positions and Julian date of
    ## epoch.  Units of the input and output positions are degrees

## declarations precession  #############################################
    my ($alpha0,$delta0,$jd) = @_;
    $alpha0 *= DEG_TO_RAD;  ## convert to radians
    $delta0 *= DEG_TO_RAD;

## begin precession  ####################################################    
    ## First evaluate the precession "constants"
    my $T = ($jd - 2451545.0) / 36525;  ## Units of years/10

    my $M = (1.2812323 * $T
	     + 3.879e-4 * $T * $T
	     + 1.01e-5 * $T * $T * $T) * DEG_TO_RAD; ## units of rad

    my $N = (0.5567530 * $T
	     - 1.185e-4 * $T * $T
	     - 1.16e-5 * $T * $T * $T) * DEG_TO_RAD; ## units of rad

    ## From these, "mean epoch" quantities are
    my $alphaM = $alpha0 + 0.5 * ($M + $N * sin($alpha0) * tan($delta0));
    my $deltaM = $delta0 + 0.5 * $N * cos($alphaM);

    ## Finally the quantities at epoch
    my $alpha = $alpha0 + $M + $N * sin($alphaM) * tan($deltaM);
    my $delta = $delta0 + $N * cos($alphaM);
    ## Note there is a typo in the AQ equation for $delta -- b_m
    ##  should read delta_m

    ## Return the quantities in degrees
    return ($alpha * RAD_TO_DEG,$delta * RAD_TO_DEG);

}  ## end precession  ###################################################



return 1;  ## end geometry.pm

