package hipRecord;
my $MODVERSION = '$Id: hipRecord.pm,v 1.12 2008/11/12 23:28:23 sgroom Exp $';
require 5.000;

use Exporter();

our @ISA = qw(Exporter);
our @EXPORT = qw(getHipRecord getHipRecordOld);

##########################################################################
# NASA Exoplanet Science Institute (NExScI) -- http://nexsci.caltech.edu #
#       Copyright (c) 1997-2008 California Institute of Technology       #
#                     U.S. Government Sponsorship                        #
# Under contract with the National Aeronautics and Space Administration  #
##########################################################################

use strict;
use warnings;
use English;		# allow use of $ERRNO

use FindBin qw($RealBin);
use lib "$RealBin/../lib/modules";

use gcConfig;
use miscString;
use hipparcosKeys;

# Check for the DB_File module at runtime, and only "use" it if found.
my $HAVE_DB_FILE;
my $MY_DB_FILETYPE;
my $MY_DB_MODE;
BEGIN {
    $HAVE_DB_FILE = eval ("use DB_File; 1") ;
    if ($HAVE_DB_FILE)
    {
	use Fcntl;
	$MY_DB_FILETYPE = $DB_File::DB_HASH;
	$MY_DB_MODE = O_RDONLY;
    }
    else
    {
	$MY_DB_FILETYPE = undef;
	$MY_DB_MODE = undef;
    }
}

my $cfg = gcConfig::config();
my $idxpath = $cfg->hipparcosIdxPath();

my $idxHdName = "$idxpath/idx_hd.db";		# the HD index file
my $idxHipName = "$idxpath/idx_hip.db";		# the HIP index file
my $idxDmName = "$idxpath/idx_dm.db";		# the DM index file
my $DB_FORMAT_VERSION = "1.0";			# version of the DB format
my $DB_FORMAT_VERSION_TAG = "__DB_FORMAT_VERSION__";	# tag for stashing version info in the DB itself

my %IDXHIP;
my %IDXHD;
my %IDXDM;
my $CATALOG;

my $initialized=0;	# flag controlling one-time initialization
my $useIndex=0;		# default: don't use the Hipparcos indexes

# initialization routine to open files the first time
# the module is used.
sub init
{
    my $result;

    if ($initialized)
    {
	return 1;	# initialization has already been done
    }

    if ($HAVE_DB_FILE)	# if we have the DB_File module needed to use the indexes...
    {
	$useIndex = 1;	# enable index use, if anything fails we'll turn it off again.

	print STDERR "DEBUG: hipRecord: opening Hipparcos HD index file $idxHdName\n" if gcConfig::debug();
	$result = tie(%IDXHD,"DB_File",$idxHdName,$MY_DB_MODE,0,$MY_DB_FILETYPE);
	if (!$result)
	{
	    print STDERR "DEBUG: hipRecord: Can't open Hipparcos HD index file $idxHdName:  $ERRNO\n" if gcConfig::debug();
	    $useIndex = 0;
	}
	else
	{
	    my $dbversion = $IDXHD{$DB_FORMAT_VERSION_TAG};
	    if ((!$dbversion) || ($dbversion ne $DB_FORMAT_VERSION))
	    {
		print STDERR "Warning: hipRecord: Hipparcos HD index file $idxHdName is not the current format and must be rebuilt using makeHipIdx\n";
		$useIndex = 0;
		undef $result;
		untie %IDXHD;
	    }
	}

	print STDERR "DEBUG: hipRecord: opening Hipparcos HIP index file $idxHipName\n" if gcConfig::debug();
	$result = tie(%IDXHIP,"DB_File",$idxHipName,$MY_DB_MODE,0,$MY_DB_FILETYPE);
	if (!$result)
	{
	    print STDERR "DEBUG: hipRecord: Can't open Hipparcos HIP index file $idxHipName:  $ERRNO\n" if gcConfig::debug();
	    $useIndex = 0;
	}
	else
	{
	    my $dbversion = $IDXHIP{$DB_FORMAT_VERSION_TAG};
	    if ((!$dbversion) || ($dbversion ne $DB_FORMAT_VERSION))
	    {
		print STDERR "Warning: hipRecord: Hipparcos HIP index file $idxHipName is not the current format and must be rebuilt using makeHipIdx\n";
		$useIndex = 0;
		undef $result;
		untie %IDXHIP;
	    }
	}

	print STDERR "DEBUG: hipRecord: opening Hipparcos DM index file $idxDmName\n" if gcConfig::debug();
	$result = tie(%IDXDM,"DB_File",$idxDmName,$MY_DB_MODE,0,$MY_DB_FILETYPE);
	if (!$result)
	{
	    print STDERR "DEBUG: hipRecord: Can't open Hipparcos HIP index file $idxDmName:  $ERRNO\n" if gcConfig::debug();
	    $useIndex = 0;
	}
	else
	{
	    my $dbversion = $IDXDM{$DB_FORMAT_VERSION_TAG};
	    if ((!$dbversion) || ($dbversion ne $DB_FORMAT_VERSION))
	    {
		print STDERR "Warning: hipRecord: Hipparcos DM index file $idxHipName is not the current format and must be rebuilt using makeHipIdx\n";
		$useIndex = 0;
		undef $result;
		untie %IDXDM;
	    }
	}
    }

    # if this one fails, we're really out of luck.
    my $path = $cfg->hipparcosCatalog();
    if (! open(CATALOG, $path))
    {
    	print STDERR "hipRecord: Can't open Hipparcos catalog file $path:  $ERRNO\n";
	return 0; # fatal error
    }

    $initialized=1;
    return 1;
}

# retrieve a Hipparcos record based in HD, HIP, or DM identifier
# returns record on success, "undef" on failure
sub getHipRecord
{
    my $idType = shift @_;
    my $id = shift @_;

    # open files if they haven't already been opened
    if (!$initialized)
    {
	if (!init())
	{
	    return undef;
	}
    }
    
    if (!$useIndex)
    {
	# Index initialization failed, revert to old style lookups.
	print STDERR "DEBUG: getHipRecord: Hipparcos index files not available, reverting to old-style lookups\n" if gcConfig::debug();
	return getHipRecordOld($idType,$id);
    }

    # Lookup the given id in the appropriate index
    my $pos;
    if ($idType eq "HD")
    {
	$pos = $IDXHD{$id};
    }
    elsif ($idType eq "HIP")
    {
	$pos = $IDXHIP{$id};
    }
    elsif ($idType eq "DM")
    {
	$pos = $IDXDM{$id};
    }
    else
    {
	print STDERR "getHipRecord: invalid id type $idType. Only HD, HIP and DM lookups are currently supported.\n";
	return undef;
    }

    # if the index lookup failed, return error
    if (!defined $pos)
    {
	print STDERR "DEBUG: getHipRecord: $idType $id not found (not found in index)\n" if gcConfig::debug();
	return undef;
    }

    print STDERR "DEBUG: getHipRecord: getting $idType $id, seeking to offset $pos\n" if gcConfig::debug();

    # index lookup succeeded, read record
    seek CATALOG, $pos, 0;
    my $line = <CATALOG>; 

    if ($line)
    {
	# seek/read succeeded, return record
	return $line;
    }
    else
    {
	# seek/read failed, return error
	print STDERR "DEBUG: getHipRecord: $idType $id not found at offset $pos (read failed)\n" if gcConfig::debug();
	return undef;
    }
}


# use the old-style hipparcos search, without the benefit of an index
# returns record on success, "undef" on failure
sub getHipRecordOld
{
    my $idType = shift @_;
    my $id = shift @_;

    if (!$initialized)
    {
	if (!init())
	{
	    return undef;
	}
    }

    if (($idType ne 'HD') &&
	($idType ne 'HIP') &&
	($idType ne 'DM'))
    {
	print STDERR "getHipRecordOld: invalid id type $idType\n";
	print "\n";
	return;
    }
    
    my $CatalogBytes = 53316318;
    my $SafetyFactor = 0.9;
    my %lastNum = ('HD' => 272150,
		    'HIP' => 118322,
		    );

    # Estimate where to start looking within the catalog if we can,
    # otherwise start from the beginning (offset=0)
    my $OffsetPt = 0;
    if ($lastNum{$idType} && isNumeric($id)) {
        $OffsetPt = int($CatalogBytes * $SafetyFactor * $id / $lastNum{$idType});
	if ($OffsetPt >= $CatalogBytes)
	{
	    $OffsetPt = 0;
	}
    }

    # search until we find a match or we search the whole file
    while (1)
    {
	seek CATALOG, $OffsetPt, 0;	# seek to starting point
	if ($OffsetPt > 0)
	{
	    my $line = <CATALOG>; # After seeking, start on the next fresh line
	    print STDERR "DEBUG: getHipRecordOld: Attempting smart search from offset $OffsetPt\n" if gcConfig::debug();
	}
	else
	{
	    print STDERR "DEBUG: getHipRecordOld: Using brute force search of entire file\n" if gcConfig::debug();
	}

	while (my $line = <CATALOG>)
	{
	    # read a line and compare desired identifier
	    my @contents = split /\|/, $line;
	    if ($idType eq 'HD')
	    {
		if (trimWhite($contents[$HIP_HDnum]) eq $id)
		{
		    return $line;	# return HD match
		}
	    }
	    elsif ($idType eq 'HIP')
	    {
		if (trimWhite($contents[$HIP_Num]) eq $id)
		{
		    return $line;	# return HIP match
		}
	    }
	    if ($idType eq 'DM')
	    {
		my $dm = trimWhite($contents[$HIP_DMname]);
		if ($dm)
		{
		    $dm =~ s/[ ]+/_/g;	# replace any remaining spaces with underscores
		}
		if ($dm eq $id)
		{
		    return $line;	# return DM match
		}
	    }
	} # end while

	# if we started the search from the beginning of the file and didn't
	# find what we wanted, return failure.
	if ($OffsetPt == 0)
	{
	    print STDERR "DEBUG: getHipRecordOld: Brute force search failed for $idType $id\n" if gcConfig::debug();
	    return "\n";	# no match, return empty record
	}

	# Smart search didn't work, reset pointer to beginning of file and try one more time
	print STDERR "DEBUG: getHipRecordOld: The smart search failed; trying brute force.\n" if gcConfig::debug();
	$OffsetPt = 0;	# guarantee loop termination after next pass through file
    }
    # not reached
}

return 1;
