package interferometers;
my $MODVERSION = '$Id: interferometers.pm,v 1.16 2008/11/12 23:28:23 sgroom Exp $';
use strict;
use warnings;

##########################################################################
# NASA Exoplanet Science Institute (NExScI) -- http://nexsci.caltech.edu #
#       Copyright (c) 1997-2008 California Institute of Technology       #
#                     U.S. Government Sponsorship                        #
# Under contract with the National Aeronautics and Space Administration  #
##########################################################################

## interferometers.pm -- interferometers database perl module
##
## This module defines the parameters for each location, telescope, and
## baseline that getCal knows about.

#########################################################################
## PTI -- Palomar Testbed Interferometer
#########################################################################
my $PTI =  ### PTI Hash
{

    ## Names and places
    Name => "Palomar Testbed Interferometer",
    Nickname => "PTI",
    Pseudonyms => ["PTI", "Palomar", "PalomarMtn"],
    Location => "Palomar",
    Place => "Mt. Palomar, CA",

    ## Geographic information
    ## Palomar: -116:52, 33:21
    Longitude => -116.8633,
    Latitude => +33.3567,
    Elevation => 1680,
    LatLongV => [-116.8633, +33.3567],
    TimeZone => [-8,1],   ## California uses US DST convention
    
    ## Station information
    Telescopes => ["N","S","W"],
    N_Location => [0,0,0],
    S_Location => [-37.116352,-103.264746,3.319338],
    W_Location => [-44.566402,75.048984,-0.214424],


    ## Baseline information
    # Baseline list...
    Baselines => ["PTI_NS","PTI_NW","PTI_SW"],
    # Baseline vectors (ENUBias)
    PTI_NS => [-37.116352,-103.264746,3.319338,-12.915245],
    PTI_NW => [-81.685124,-28.214086,3.105647,0.031416],
    ## PTI_SW => [ -(-37.116352 - (-81.685124)), -(-103.264746 - (-28.214086)),-(3.319338-3.105647), -(-12.915245-0.031416)  ],
    ## PTI_SW => [-44.568772,75.05066,-0.213691,12.946661],
    ## PTI_SW Updated with bFit result 16 Oct 2003 AFB
    PTI_SW => [-44.566402,75.048984,-0.214424,14.726679],
    ## Delay range
    DelayRange => 38.3,

};  ### End PTI Hash


#########################################################################
## KI -- Keck Interferometer
#########################################################################
my $KI =  ### KI Hash
{
    ## Names and places
    Name => "Keck Interferometer",
    Nickname => "KI",
    Pseudonyms => ["KI", "Keck", "MaunaKea", "WMKO"],
    Location => "MaunaKea",
    Place => "Mauna Kea, HI",

    ## http://www2.keck.hawaii.edu:3636/realpublic/observing/obswmkpl.html
    ## Geographic information
    ## Keck/Mauna Kea: -155:28.4, +19:49.6
    Longitude => -155.473333,
    Latitude => 19.826667,
    Elevation => 4123.0,
    LatLongV => [-155.473333, +19.826667],
    TimeZone => [-10, 0],  ## Hawaii doesn't use Daylight Savings Time

    ## Telescope information
    ## Telescope list
    Telescopes => ["K1","K2"],
    ## Telescope Locations (ENU, m)
    K1_Location => [0,0,4159.58],
    K2_Location => [51.702317,67.455618,4159.58],	## K-K baseline fit per RLA 1/31/07

    ## Telescope dome radii (the ones that matter)
    K1_Radius => 18.7,
    K2_Radius => 18.7,

    ## Baseline information
    ## Baseline list
    Baselines => ["KI_K1K2"],
    ## Baseline vectors (ENUBias, m)
    # KI_K1K2 => [51.704,67.455,0.001,+40.39], ## K-K baseline fit 3/14/01 -51.7039  -67.4552  -0.0015  1.3993
    KI_K1K2 => [51.702317,67.455618,-0.003674,+41.006818], ## K-K baseline fit per RLA 1/31/07

    ## FDL Range
    DelayRange => 15.0,

    ## LDL Range
    StaticDelayRange => 100.0,

    ## Maps of elevation limits for each baseline.
    ## Each map indicates the composite limits for the baseline,
    ## combining the individual limits of each telescope on that baseline.
    ## The map must start at az=0.0 and end at az=360.0, in
    ## order of increasing az. Each segment az must begin
    ## where the previous one ended, no overlap.
    ## When no limits are defined for a baseline, it is assumed
    ## that there are no pointing limitations for that baseline.
    ## Each entry in the *_Limits list is:
    ##       az_begin, az_end, el_min, el_max (all in deg)
    ## SG 6/06

    # K1 and K2 telescope pointing limits taken from Keck Telescope and Facility
    # Instrument Guide, August 2002.  The KI_K1K2 baseline pointing limits were
    # found by merging the individual K1 and K2 limits given
    # in that reference:
    # K1:
    #	az 5.3 to 146.2, el 33.3 to 88.9
    #   elsewhere, el 18.0 to 88.9
    # K2:
    #   az 185.3 to 332.8, el 36.8 to 89.5
    #   elsewhere, el 18.0 to 89.5
    # SG 6/06
    KI_K1K2_Limits => [
 		    [   0.0,     5.3,   18.0,   88.9],	# az_begin, az_end, el_min, el_max (deg)
 		    [   5.3,   146.2,   33.3,   88.9],
 		    [ 146.2,   185.3,   18.0,   88.9],
 		    [ 185.3,   332.8,   36.8,   88.9],
 		    [ 332.8,   360.0,   18.0,   88.9]
 		    ]
};  ### End KI Hash

#########################################################################
## KI_Nulling -- Keck Interferometer Nulling mode
#########################################################################
my $KI_Nulling =  ### KI Hash
{
    ## Names and places
    Name => "Keck Interferometer (Nulling Mode)",
    Nickname => "KI_Nulling",
    Pseudonyms => ["KI_Nulling", "Keck_Nulling", "MaunaKea_Nulling", "WMKO_Nulling"],
    Location => "MaunaKea_Nulling",
    Place => "Mauna Kea, HI",

    ## http://www2.keck.hawaii.edu:3636/realpublic/observing/obswmkpl.html
    ## Geographic information
    ## Keck/Mauna Kea: -155:28.4, +19:49.6
    Longitude => -155.473333,
    Latitude => 19.826667,
    Elevation => 4123.0,
    LatLongV => [-155.473333, +19.826667],
    TimeZone => [-10, 0],  ## Hawaii doesn't use Daylight Savings Time

    ## Telescope information
    ## Telescope list
    Telescopes => ["K1","K2"],
    ## Telescope Locations (ENU, m)
    K1_Location => [0,0,4159.58],
    K2_Location => [51.702317,67.455618,4159.58],	## K-K baseline fit per RLA 1/31/07

    ## Telescope dome radii (the ones that matter)
    K1_Radius => 18.7,
    K2_Radius => 18.7,

    ## Baseline information
    ## Baseline list
    Baselines => ["KI_PK","KI_PN","KI_SK","KI_SN"],

    ## Baseline vectors (ENUBias, m)
    KI_PK =>   [51.702317,67.455618,-0.003674,+41.043], ## KI Nuller Primary K, RMG 3/8/07
    KI_PN =>   [51.702317,67.455618,-0.003674,+40.627], ## KI Nuller Primary N, RMG 3/8/07
    KI_SK =>   [51.702317,67.455618,-0.003674,+41.142], ## KI Nuller Secondary K, RMG 3/8/07
    KI_SN =>   [51.702317,67.455618,-0.003674,+40.708], ## KI Nuller Secondary N, RMG 3/8/07

    ## 6/25/07 SLG: removed KI baseline delay offsets
    ##KI_PK_DelayOffset =>  0.00,	## KI Nuller Primary K delay offset, RMG/SLG 6/25/07
    ##KI_PN_DelayOffset =>  0.00,	## KI Nuller Primary N delay offset, RMG/SLG 6/25/07
    ##KI_SK_DelayOffset =>  0.00,	## KI Nuller Secondary K delay offset, RMG/SLG 6/25/07
    ##KI_SN_DelayOffset =>  0.00,	## KI Nuller Secondary N delay offset, RMG/SLG 6/25/07

    ## FDL Range
    DelayRange => 15.0,			# standard delay range for the site
    KI_PN_DelayRange => 14.04,		# nonstandard delay range for the KI_PN baseline, RMG, 10/30/07
    KI_SN_DelayRange => 14.04,		# nonstandard delay range for the KI_SN baseline, RMG, 10/30/07

    ## LDL Range
    StaticDelayRange => 100.0,

    ## Maps of elevation limits for each baseline.
    ## Each map indicates the composite limits for the baseline,
    ## combining the individual limits of each telescope on that baseline.
    ## The map must start at az=0.0 and end at az=360.0, in
    ## order of increasing az. Each segment az must begin
    ## where the previous one ended, no overlap.
    ## When no limits are defined for a baseline, it is assumed
    ## that there are no pointing limitations for that baseline.
    ## Each entry in the *_Limits list is:
    ##       az_begin, az_end, el_min, el_max (all in deg)
    ## SG 6/06

    # K1 and K2 telescope pointing limits taken from Keck Telescope and Facility
    # Instrument Guide, August 2002.  The KI_K1K2 baseline pointing limits were
    # found by merging the individual K1 and K2 limits given
    # in that reference:
    # K1:
    #	az 5.3 to 146.2, el 33.3 to 88.9
    #   elsewhere, el 18.0 to 88.9
    # K2:
    #   az 185.3 to 332.8, el 36.8 to 89.5
    #   elsewhere, el 18.0 to 89.5
    # SG 6/06

    # These limits are the same for all four Nulling baselines, and the
    # same as the regular KI_K1K2 baseline.
    KI_PK_Limits => [
 		    [   0.0,     5.3,   18.0,   88.9],	# az_begin, az_end, el_min, el_max (deg)
 		    [   5.3,   146.2,   33.3,   88.9],
 		    [ 146.2,   185.3,   18.0,   88.9],
 		    [ 185.3,   332.8,   36.8,   88.9],
 		    [ 332.8,   360.0,   18.0,   88.9]
 		    ],
    KI_PN_Limits => [
 		    [   0.0,     5.3,   18.0,   88.9],	# az_begin, az_end, el_min, el_max (deg)
 		    [   5.3,   146.2,   33.3,   88.9],
 		    [ 146.2,   185.3,   18.0,   88.9],
 		    [ 185.3,   332.8,   36.8,   88.9],
 		    [ 332.8,   360.0,   18.0,   88.9]
 		    ],
    KI_SK_Limits => [
 		    [   0.0,     5.3,   18.0,   88.9],	# az_begin, az_end, el_min, el_max (deg)
 		    [   5.3,   146.2,   33.3,   88.9],
 		    [ 146.2,   185.3,   18.0,   88.9],
 		    [ 185.3,   332.8,   36.8,   88.9],
 		    [ 332.8,   360.0,   18.0,   88.9]
 		    ],
    KI_SN_Limits => [
 		    [   0.0,     5.3,   18.0,   88.9],	# az_begin, az_end, el_min, el_max (deg)
 		    [   5.3,   146.2,   33.3,   88.9],
 		    [ 146.2,   185.3,   18.0,   88.9],
 		    [ 185.3,   332.8,   36.8,   88.9],
 		    [ 332.8,   360.0,   18.0,   88.9]
 		    ],

};  ### End KI_Nulling Hash


#########################################################################
## NPOI -- Navy Prototype Optical Interferometer
#########################################################################
my $NPOI =  ### NPOI Hash
{
    
    ## Names and pseudonyms
    Name => "Navy Prototype Optical Interferometer",
    Nickname => "NPOI",
    Pseudonyms => ["NPOI", "Lowell", "Flagstaff", "AndersonMesa"],
    Location => "Lowell",
    Place => "Anderson Mesa, Flagstaff, AZ",

    ## Geographic information
    ## Anderson_Mesa
    ## Latitude =   35 05 48.0
    ## Longitude= -111 32 06.0
    ## Height   = 2200.66 m
    Longitude => -111.6017,
    Latitude => +35.1933,
    Elevation => 2200.66,
    LatLongV => [-111.6017, +35.1933],
    TimeZone => [-8,1],  ## Arizona uses US DST convention (???)
    

    ## Baseline information
    Baselines => ["NPOI_AN0AC0", "NPOI_AN0AE0", "NPOI_AN0AW0",
		  "NPOI_AC0AE0", "NPOI_AC0AW0",
		  "NPOI_AE0AW0",
		  "NPOI_AN0N00", "NPOI_AN0N01",
		  "NPOI_AC0N00", "NPOI_AC0N01",
		  "NPOI_AE0N00", "NPOI_AE0N01",
		  "NPOI_AW0N00", "NPOI_AW0N01"],

    # Baseline vectors (ENUBias)
    NPOI_AN0AC0 => [-0.1400,22.8500,0.0000,7.2822],
    NPOI_AN0AE0 => [-17.6300,30.0900,0.0000,-10.9590],
    NPOI_AN0AW0 => [19.7500,32.7100,0.0000,-6.7248],
    NPOI_AC0AE0 => [-17.4900,7.2400,0.0000,-18.2412],
    NPOI_AC0AW0 => [19.8900,9.8600,0.0000,-14.0070],
    NPOI_AE0AW0 => [37.3800,2.6200,0.0000,4.2342],
    NPOI_AN0N00 => [-1.8300,16.5100,0.0000,16.61],
    NPOI_AN0N01 => [-1.580,14.300,0.000,14.388],
    NPOI_AC0N00 => [-1.690,-6.340,0.000,9.329],
    NPOI_AC0N01 => [-1.440,-8.550,0.000,7.106],
    NPOI_AE0N00 => [15.800,-13.580,0.000,27.571],
    NPOI_AE0N01 => [16.050,-15.790,0.000,25.347],
    NPOI_AW0N00 => [-21.580,-16.200,0.000,23.336],
    NPOI_AW0N01 => [-21.330,-18.410,0.000,21.113],
    NPOI_N00N01 => [0.250,-2.210,0.000,-2.224],

    ## FDL Range
    DelayRange => 40.0,

    ## LDL Range
    StaticDelayRange => 100.0

};  ### end NPOI Hash



#########################################################################
## CHARA -- Center for High Angular Resolution Astronomy
#########################################################################
my $CHARA = ### Begin CHARA Hash
{
    ## Names and pseudonyms
    Name => "Center for High Angular Resolution Astronomy Array",
    Nickname => "CHARA",
    Pseudonyms => ["CHARA", "MtWilson"],
    Location => "MtWilson",
    Place => "Mt Wilson, CA",

    ## Geographic information
    ## Mt Wilson, CA
    ## Latitude =   34 13 27.78
    ## Longitude= -118 03 25.31
    ## Height   = 2200.66 m
    Longitude => -118.05703,
    Latitude => +34.224383,
    Elevation => 2200.66,
    LatLongV => [-118.05703, +34.224383],
    TimeZone => [-8,1],  ## California uses US DST convention

    ## Telescope information
    ## Telescope list
    Telescopes => ["S1","S2","E1","E2","W1","W2","K7","K8"],

    ## Baseline list -- updates from GvB 2/2004
    Baselines => ["CHARA_W2S1","CHARA_W2S2","CHARA_W2E1","CHARA_W2E2","CHARA_W2W1",
		  "CHARA_W1S1","CHARA_W1S2","CHARA_W1E1","CHARA_W1E2",
		  "CHARA_E2S1","CHARA_E2S2","CHARA_E2E1",
		  "CHARA_E1S1","CHARA_E1S2","CHARA_S2S1"],

    ## Baseline vectors (ENUBias, m)
    ## CHARA baselines from Theo 2/1/04 via GvB
    ## constant term components for S1S2 and E1S1 from Theo 10/16/01
    ## constant terms not known for these baselines    
    CHARA_W2S1 => [-69.659,199.152,0.470,0.000],    
    CHARA_W2S2 => [-63.911,165.571,-0.174,0.000],    
    CHARA_W2E1 => [-195.001,-106.784,6.375,0.000],    
    CHARA_W2E2 => [-140.060,-70.565,3.258,0.000],    
    CHARA_W2W1 => [105.409,-17.170,11.298,0.000],    

    CHARA_W1S1 => [-175.069,216.322,-10.828,0.000],    
    CHARA_W1S2 => [-169.320,182.741,-11.472,0.000],    
    CHARA_W1E1 => [-300.410,-89.614,-4.923,0.000],    
    CHARA_W1E2 => [-245.469,-53.395,-8.040,0.000],    

    CHARA_E2S1 => [70.401,269.717,-2.788,0.000],    
    CHARA_E2S2 => [76.149,236.135,-3.432,0.000],    
    CHARA_E2E1 => [-54.941,-36.219,3.117,0.000],    

# CHARA_S1E1 from Theo 10/16/01
# XOFFSET  125319571.070 # YOFFSET  305955414.637 # ZOFFSET   -5926542.809 # AIRPATH   15648305.165
    CHARA_E1S1 => [125.342,305.936,-5.905,15.648],    
    CHARA_E1S2 => [131.090,272.354,-6.549,0.000],    

# CHARA_S1S2 from Theo 10/16/01
# XOFFSET -5750984.274 # YOFFSET 33578992.485 # ZOFFSET   643418.132 # AIRPATH  4780257.962
    CHARA_S2S1 => [-5.748,33.581,0.644,4.780],


    ## FDL Range
    DelayRange => 88.0,

    ## LDL Range
    StaticDelayRange => 100.0

};  ## end CHARA Hash



#########################################################################
## VLTI -- Very Large Telescope Interferometer
#########################################################################
my $VLTI =  ### VLTI Hash
{
    
    ## Names and places
    Name => "Very Large Telescope Interferometer",
    Nickname => "VLTI",
    Pseudonyms => ["VLTI", "Paranal", "ESO"],
    Location => "Paranal",
    Place => "Cerro Paranal, Chile",

    ## http://www2.keck.hawaii.edu:3636/realpublic/observing/obswmkpl.html
    ## Geographic information
    ## Paranal: -4h 41m 36s, -24d 37.4m
    Longitude => -70.4,
    Latitude => -24.6233,
    Elevation => 2635.0,
    LatLongV => [-70.4, -24.6233],
    TimeZone => [-4, -1],  ## Paranal uses Chilean DST convention

    ## Telescope information
    ## Telescope list
    Telescopes => ["K1","K2","K3","K4","K5","K6","K7","K8","S1","S2"],
    ## Telescope Locations (ENU, m)
    ##K1_Location => [0,0,4159.58],
    ##K2_Location => [51.66,67.41,4159.58],

    ##S1_Location => [67.01,44.35,4147.01],
    ##S2_Location => [79.38,60.58,4147.02],
    
    ## Telescope dome radii (the ones that matter)
    ##K1_Radius => 18.7,
    ##K2_Radius => 18.7,


    ## Baseline information
    ## Baseline list
    Baselines => ["VLTI_U1U2","VLTI_U1U3","VLTI_U1U4","VLTI_U2U3","VLTI_U2U4","VLTI_U3U4"],
    ## Baseline vectors (ENUBias, m)
    VLTI_U1U2 => [24.31,51.08,0,0],
    VLTI_U1U3 => [53.97,87.08,0,0],
    VLTI_U1U4 => [112.59,65.45,0,0],
    VLTI_U2U3 => [29.66,36.00,0,0],
    VLTI_U2U4 => [88.28,14.37,0,0],
    VLTI_U3U4 => [56.62,-21.63,0,0],

    ## FDL Range
    DelayRange => 40.0,

    ## LDL Range
    StaticDelayRange => 100.0

};  ### End VLTI Hash



#########################################################################
## SUSI -- Sydney University Stellar Interferometer
#########################################################################
my $SUSI =  ### SUSI Hash
{
    
    ## Names and places
    Name => "Sydney University Stellar Interferometer",
    Nickname => "SUSI",
    Pseudonyms => ["SUSI", "Narrabri"],
    Location => "Narrabri",
    Place => "Narrabri, Australia",

    ## Geographic information from http://www.physics.usyd.edu.au/astron/susi/
    ## N: +149d 33m E, -30d 32m S
    ## refined position data from John Davis email 26 July 2003
    ## 149 32 53.61 E,  -30 19 20.19 S
    Longitude => +149.5482,
    Latitude => -30.3223,
    Elevation => 211.5,
    LatLongV => [+149.5482, -30.3223],
    TimeZone => [+10, -1],  ## Narrabri uses DST convention

    ## Telescope information
    ## Telescope list
    Telescopes => ["K1","K2","K3","K4","K5","K6","K7","K8","S1","S2"],
    ## Telescope Locations (ENU, m)

    ## Baseline information
    ## Baseline list
    Baselines => ["SUSI_N1S1","SUSI_N1S2","SUSI_N3S1","SUSI_N3S2",
		  "SUSI_N1S3","SUSI_N3S3","SUSI_N4S1","SUSI_N4S3",
		  "SUSI_N1S4","SUSI_N4S4"],

    ## Baseline vectors (ENUBias, m)
    SUSI_N1S1 => [0,  -5.0, 0,   0],
    SUSI_N1S2 => [0, -10.0, 0,   5],
    SUSI_N3S1 => [0, -15.0, 0, -10],
    SUSI_N3S2 => [0, -20.0, 0,  -5],

    SUSI_N1S3 => [0, -30.0, 0,  25],
    SUSI_N3S3 => [0, -40.0, 0,  15],
    SUSI_N4S1 => [0, -55.0, 0, -50],
    SUSI_N4S3 => [0, -80.0, 0, -25],

    SUSI_N1S4 => [0,-110.0, 0, 105],
    SUSI_N4S4 => [0,-160.0, 0,  55],

    ## FDL Range
    DelayRange => 140.0,

    ## LDL Range
    StaticDelayRange => 100.0

};  ### End SUSI Hash


#########################################################################
## IOTA -- Infrared Optical Telescope Array
#########################################################################
my $IOTA =  ### IOTA Hash
{
    
    ## Names and places
    Name => "Infrared Optical Telescope Array",
    Nickname => "IOTA",
    Pseudonyms => ["IOTA", "MtHopkins"],
    Location => "MtHopkins",
    Place => "Mt. Hopkins, Arizona",

    ## Geographic information from
    ## http://cfa-www.harvard.edu/cfa/oir/IOTA/TECHNICAL/index.html
    ## 31d41m29.58s  	 (31.6916 deg)      N
    ## 110d53m05.73s 	 (110.8849 deg)     W
    ## Altitude: 8412ft / 2564m
    Longitude => -110.8849,
    Latitude => +31.6916,
    Elevation => 2564.0,
    LatLongV => [-110.8849, +31.6916],
    TimeZone => [-8,1],  ## Arizona uses US DST convention (???)

    ## Telescope information
    ## Telescope list
    Telescopes => ["IOTA_1","IOTA_2","IOTA_3"],
    ## Telescope Locations (ENU, m)

    ## Baseline information
    ## Baseline list
    Baselines => [],

    ## Baseline vectors (ENUBias, m)

    ## FDL Range
    DelayRange => 60.0,

    ## LDL Range
    ## StaticDelayRange => 0.0  ## No LDLs at IOTA

};  ### End IOTA Hash



# List of all the known interferometers, pseudonyms and locations
%interferometers::InterferometerList =
    (
     "PTI" => \$PTI,
     "Palomar" => \$PTI,
     "PalomarMtn" => \$PTI,
     "KI" => \$KI,
     "Keck" => \$KI,
     "MaunaKea" => \$KI,
     "WMKO" => \$KI,
     "NPOI" => \$NPOI,
     "Lowell" => \$NPOI,
     "Flagstaff" => \$NPOI,
     "AndersonMesa" => \$NPOI,
     "CHARA" => \$CHARA,
     "MtWilson" => \$CHARA,
     "VLTI" => \$VLTI,
     "Paranal" => \$VLTI,
     "ESO" => \$VLTI,
     "SUSI" => \$SUSI,
     "Narrabri" => \$SUSI,
     "MtHopkins" => \$IOTA,
     "IOTA" => \$IOTA,
     "KI_Nulling" => \$KI_Nulling,
     "Keck_Nulling" => \$KI_Nulling,
     "MaunaKea_Nulling" => \$KI_Nulling,
     "WMKO_Nulling" => \$KI_Nulling,
    );

# List of all the known interferometers by preferred full name,
# used by gcGui's location menu
%interferometers::InterferometerShortList =
    (
     "Palomar (PTI)" => \$PTI,
     "Mauna Kea (KI)" => \$KI,
     "Flagstaff (NPOI)" => \$NPOI,
     "Mt Wilson (CHARA)" => \$CHARA,
     "Paranal (VLTI)" => \$VLTI,
     "Narrabri (SUSI)" => \$SUSI,
     "Mt Hopkins (IOTA)" => \$IOTA,
     "Mauna Kea Nulling (KI)" => \$KI_Nulling,
    );

