package internalLine;
my $MODVERSION = '$Id: internalLine.pm,v 1.46 2008/11/12 23:28:23 sgroom Exp $';

use warnings;
use strict;
require 5.000;
use coordinates;
use angles;  ## angle constants module
use gcConfig;
use miscString;
use astroCatalogue;

use Exporter();
our @ISA = qw(Exporter);
our @EXPORT = qw();

##########################################################################
# NASA Exoplanet Science Institute (NExScI) -- http://nexsci.caltech.edu #
#       Copyright (c) 1997-2008 California Institute of Technology       #
#                     U.S. Government Sponsorship                        #
# Under contract with the National Aeronautics and Space Administration  #
##########################################################################

## internalLine.pm -- code/class library for dealing with the new
## getCal internal line format

## MODIFIED T. Evans   8 Dec 2004  -- for release 2.7 with new output formats
##   add tags to internalLine object, new comment & source type elements
##   "new": read in any of 4 available formats and create obj
##   output methods: accept flag for format type, write any of 4 formats

## MODIFIED T. Evans   Jun 2005  -- for release 2.8 
##   updated to iron out units wrinkles for proper motions and parallaxes:
##     PLX, pmRA and pmDec now will always be in units of arcsec internally
##   deal properly with input and internal "undefined" values so they are
##   returned properly as "undef" instead of "?" or "0" or whatever


#########################################################################
## global declarations
#########################################################################
my $cfg = gcConfig::config();

my $fieldDelimiter = "|";  ## Follow Hipparcos example


my $designation = 0;
my $catIndex = 1;
my $catIndex2 = 2;
#my $spType = 3;
my $tagSPECTYP = 3;
my $raDeg = 4;
my $decDeg = 5;
#my $parallax = 6;
my $tagPLX = 6;
my $pmRA = 7;
my $pmDec = 8;
my $Vmag = 9;
my $Kmag = 10;
my $BminusV = 11;
my $source = 12;
#my $separation = 13;
my $tagD = 13;
my $bolMag = 14;
my $bolFlux = 15;
my $effTemp = 16;
#my $angDiam = 17;
my $tagDIAM = 17;
#my $angDiamErr = 18;
my $tagDIAMERR = 18;

## new fields
my $tagROLE = 19;
my $tagPID = 20;
my $tagCALFOR = 21;
my $tagR = 22;
my $tagJ = 23;
my $tagH = 24;
my $tagL = 25;
my $tagN = 26;
my $tagCMNT = 27;
my $raHMS = 28;
my $decDMS = 29;
my $tagLPRIME = 30;

## Hash labeling the fields in the internal line format
my %InternalKeys =
    (
     "designation" => 0,
     "catIndex" => 1,
     "catIndex2" => 2,
     #"spType" => 3,
     "tagSPECTYP" => 3,
     "raDeg" => 4,
     "decDeg" => 5,
     #"parallax" => 6,
     "tagPLX" => 6,
     "pmRA" => 7,
     "pmDec" => 8,
     "Vmag" => 9,
     "Kmag" => 10,
     "BminusV" => 11,
     "source" => 12,
     #"separation" => 13,
     #"sep" => 13,
     "tagD" => 13,
     "bolMag" => 14,
     "bolFlux" => 15,
     "effTemp" => 16,
     #"angDiam" => 17,
     "tagDIAM" => 17,
     #"angDiamErr" => 18,
     "tagDIAMERR" => 18,
     "tagROLE" => 19,
     "tagPID" => 20,
     "tagCALFOR" => 21,
     "tagR" => 22,
     "tagJ" => 23,
     "tagH" => 24,
     "tagL" => 25,
     "tagN" => 26,
     "tagCMNT" => 27,
     "raHMS" => 28,
     "decDMS" => 29,
     "tagLPRIME" => 30,
     );

## lists of required tagged fields
my @iLineFields =
	qw / designation catIndex catIndex2 tagSPECTYP raDeg decDeg
	    tagPLX pmRA pmDec Vmag Kmag BminusV source tagD
	    bolMag bolFlux effTemp tagDIAM tagDIAMERR
	    tagROLE tagPID tagCALFOR
	    tagR tagJ tagH tagL tagN tagCMNT raHMS decDMS tagLPRIME/ ;
##my @trgReqTags = qw/ tagSPECTYP tagROLE / ;
my @newTrgReqTags = qw/ tagSPECTYP tagROLE tagPID / ;  ## also tagPLX iff requested
my @calReqTags = qw/ tagSPECTYP tagDIAM tagDIAMERR tagROLE tagCALFOR / ;  ## also tagPLX iff requested
my @engReqTags = qw/ tagSPECTYP tagDIAM tagDIAMERR tagROLE tagPID / ;  ## also tagPLX iff requested
my @trgOptTags = qw/ tagR tagJ tagH tagN tagL tagLPRIME/ ;
my @calOptTags = qw/ tagD tagR tagJ tagH tagN tagL tagLPRIME/ ;
##my @skyReqFields = qw/ designation raDeg decDeg raHMS decDMS pmRA pmDec Vmag BminusV / ;
##my @oldnewReqFields = qw/ designation raDeg decDeg raHMS decDMS pmRA pmDec Vmag Kmag BminusV / ;
my @skyReqFieldsOut = qw/ designation raHMS decDMS pmRA pmDec 
                          Vmag BminusV tagSPECTYP / ;
my @oldReqFieldsOut = qw/ designation raHMS decDMS pmRA pmDec 
                          tagPLX Vmag Kmag BminusV tagN tagSPECTYP / ;
my @newReqFieldsOut = qw/ designation raHMS decDMS pmRA pmDec Vmag Kmag BminusV / ;

## array indexes: 0 = uninitialized string / missing value
##                1 = lineOutput (internal line string) format
##                2 = New and most Old output file format 
##                3 = Sky output file format
my %fieldFmts = (
   "designation" => [ "desig?",   "%-15s",  "%s",    "%-15s"            ],
   "catIndex"    => [ "?",        "%10s",   "NULL",  "NULL"             ],
   "catIndex2"   => [ "?",        "%10s",   "NULL",  "NULL"             ],
   "tagSPECTYP"  => [ "SPECTYP?", "%s",     "%s",    "%s"               ],
   "raDeg"       => [ "?",        "%12.8f", "NULL",  "NULL"             ],
   "decDeg"      => [ "?",        "%12.8f", "NULL",  "NULL"             ],
   "tagPLX"      => [ "PLX?",     "%8.6f",  "%5.3f", "NULL"             ],
   "pmRA"        => [ "pmRA?",    "%8.6f",  "%6.3f", "2000 pmra=%-6.4f" ],
   "pmDec"       => [ "pmDec?",   "%8.6f",  "%6.3f", "pmdec=%-5.3f"     ],
   "Vmag"        => [ "Vmag?",    "%f",     "%3.1f", "vmag=%-3.1f"      ],
   "Kmag"        => [ "Kmag?",    "%f",     "%3.1f", "NULL"             ],
   "BminusV"     => [ "B-V?",     "%f",     "%5.2f", "b-v=%-4.2f"       ],
   "source"      => [ "?",        "%s",     "NULL",  "NULL"             ],
   "tagD"        => [ "D?",       "%4.1f",  "%3.1f", "%3.1f"            ],
   "bolMag"      => [ "?",        "%6.2f",  "NULL",  "NULL"             ],
   "bolFlux"     => [ "?",        "%8.2e",  "NULL",  "NULL"             ],
   "effTemp"     => [ "?",        "%5.0f",  "NULL",  "NULL"             ],
   "tagDIAM"     => [ "DIAM?",    "%f",     "%3.1f", "%3.1f"            ],
   "tagDIAMERR"  => [ "DIAMERR?", "%f",     "%3.1f", "+/-%3.1f"         ],
   "tagROLE"     => [ "ROLE?",    "%s",     "%s",    "NULL"             ],
   "tagPID"      => [ "PID?",     "%s",     "%s",    "NULL"             ],
   "tagCALFOR"   => [ "CALFOR?",  "%s",     "%s",    "%s"               ],
   "tagR"        => [ "?",        "NULL",   "NULL",  "NULL"             ],
   "tagJ"        => [ "?",        "NULL",   "NULL",  "NULL"             ],
   "tagH"        => [ "?",        "NULL",   "NULL",  "NULL"             ],
   "tagL"        => [ "L?",       "%f",     "%3.1f", "NULL"             ],
   "tagLPRIME"   => [ "LPRIME?",  "%f",     "%3.1f", "NULL"             ],
   "tagN"        => [ "N?",       "%f",     "%3.1f", "NULL"             ],
   "tagCMNT"     => [ "?",        "%s",     "%s",    "NULL"             ],
   "raHMS"       => [ "raHMS?",   "%s",     "%s",    "%s"               ],
   "decDMS"      => [ "decDMS?",  "%s",     "%s",    "%s"               ],
    );


#########################################################################
## object constructor -- create internal line object
#########################################################################
sub new
{
##  declarations  #######################################################

    my $class = shift @_;  ## class designation

    ## Create the hash with the rest of the stuff from the call argument
    ##  list
    my $objRef = { @_ };
    bless $objRef;

## begin  ###############################################################
    if ((grep m/line/, keys %$objRef))
    {
      chomp($objRef->{line});		# there may be a newline at the end
      print STDERR "DEBUG: internalLine new: \"$objRef->{line}\"\n" if (gcConfig::debug() >= 2);
      ##  TLE - 12/2004  check if internal line string
      if (grep m/\|/, $objRef->{line})
      {
	$objRef->newFromILine();
      }
      else
      {
        ##  TLE - 12/2004  must be a file format (sky || old || new)
	print STDERR "DEBUG: using file constructor\n" if (gcConfig::debug() >= 3);
	$objRef->newFromFileLine();

      }  ## end file formats 

    } ## end input line
    else
    {
	# this means someone called "new" without supply the "line =>" intializer.
	# not an error, but we should fix those calls to use a better
	# calling interface - slg
	print STDERR "DEBUG: internalLine constructor called without \"line=>\"\n" if (gcConfig::debug() >= 2);
    }

    ##print STDERR "DEBUG: new version\n";
    print STDERR "DEBUG: internalLine constructor returning object:\n" if (gcConfig::debug() >= 2);
    $objRef->dump() if (gcConfig::debug() >= 3);	## DEBUG -- TLE 3/2005 added 'if' for debugging

    ## return object
    return $objRef;

} ## end constructor  ###################################################


#########################################################################
# newFromILine
# constructor helper - initialize object from "internal line" string format
# format parsed is that produced by internalLine::lineOutput()
#########################################################################
sub newFromILine
{
    my $this = shift @_;

    ## creating an internal line object from an internal line string
    ## perform the catalog line split
    my @lineContents = split /\|/, $this->{line};

    # Use the list of field names in iLineFields to read each field
    # from the array in order, and store it into the
    # hash using its field name
    foreach my $tag ( @iLineFields )
    {
	my $val = shift @lineContents;
	if ( (defined $val) && ($val ne $fieldFmts{$tag}[0]) && ($val ne '?') )
	{
            # is defined and != null strings (in %fieldFmts or '?')
	    # get rid of leading and trailing spaces
	    $val =~ s/^\s+//;
	    $val =~ s/\s+$//;
            if ($tag eq "tagSPECTYP")
            {
	      $val =~ s/\s+//g;        ## eliminate spaces
            }
	    $this->{$tag} = "$val";
	}
	else
	{
	    ## field was empty or null, just ignore it
	    print STDERR "DEBUG: newFromILine: val undefined for tag $tag\n" if (gcConfig::debug() >= 2);
	}
    }
}

#########################################################################
# newFromFileLine
# constructor helper - initialize object from file-formatted line
# Format could be one of:
#   getCal "Old" format (pre December 2004)
#   getCal "New" format (December 2004)
#   Keck "Sky" format
#
#########################################################################
sub newFromFileLine
{
    my $objRef = shift @_;

    @{ $objRef->{lineContents} } = split " ", $objRef->{line};
    my $nfields = @{ $objRef->{lineContents} };
    ##my @reqFields;
    my $ind = 0;
    my ($key, $val);
    my $isSky = 0;
    my $isOld = 0;
    my $isNew = 0;

    ## set up line format types: Sky, Old, New
    if (grep m/pmra/, $objRef->{line})
    {
      $isSky = 1;
      ##@reqFields = (@skyReqFields);
    }
    elsif (grep m/ROLE/, $objRef->{line})
    {
      $isNew = 1;
      ##@reqFields = (@oldnewReqFields);
    }
    else
    {
      $isOld = 1;
      ##@reqFields = (@oldnewReqFields);
    }

    # desig, ra, dec same for all formats

    $val = ${ $objRef->{lineContents} }[$ind];
    $objRef->{designation} = $val
        if ( defined($val) && ($val ne $fieldFmts{designation}[0]) && ($val ne '?') );
    $ind++;

    # get decimal RA, Dec 
    ## now adding sexiges position strings
    $val = ${ $objRef->{lineContents} }[$ind];
    if ( defined($val) && ($val ne $fieldFmts{raHMS}[0]) && ($val ne '?') )
    {
      my $raString = join ' ', $val, ${ $objRef->{lineContents} }[$ind+1], 
            ${ $objRef->{lineContents} }[$ind+2];
      ## raHMS field no longer filled on input
      ##$objRef->{raHMS} = $raString;
      $objRef->{raDeg} = &RAdeg($raString);
      $ind += 2;
    }
    $ind++;
    $val = ${ $objRef->{lineContents} }[$ind];
    if ( defined($val) && ($val ne $fieldFmts{decDMS}[0]) && ($val ne '?') )
    {
      my $decString = join ' ', $val, ${ $objRef->{lineContents} }[$ind+1], 
            ${ $objRef->{lineContents} }[$ind+2];
      ## decDMS field no longer filled on input
      ##$objRef->{decDMS} = $decString;
      $objRef->{decDeg} = &DECdeg($decString);
      $ind += 2;
    }
    $ind++;
        
    foreach my $tag (qw/ pmRA pmDec Vmag Kmag BminusV /) 
    {
      ## these are mostly the same for each format...

      ## skip Kmag for Sky format
      next if ( ($isSky) && ($tag eq "Kmag") );
      if ($isSky) 
      {
        ## have to split tag=val for Sky format
        ($key, $val) = split /=/,  ${ $objRef->{lineContents} }[$ind];
        if ($tag eq 'pmRA')
        {
          ## TLE - 6/2005   Sky format input: pmRA in time seconds/yr, now put pmRA 
          ##       in arcsec and keep pmDec in arcsec
          $val *= 15.
        }
      }
      else
      {
        $val = ${ $objRef->{lineContents} }[$ind];
        ## TLE - 6/2005   Old or New format input: now keep pmRA, pmDec in arcsec 
      }
      $objRef->{$tag} = $val
          if ( defined($val) && ($val ne $fieldFmts{$tag}[0]) && ($val ne '?') );
      $ind++;

      if ( $isOld && ($tag eq "pmDec") && ( ($nfields == 19) ||
             (($nfields == 18) && (${ $objRef->{lineContents} }[$ind] < 1.)) ) )
      {
	  ## if is Old format and right after pmDec:
	  ##   if has max # of params,  must have PLX, N (mag)
	  ##   OR if has either PLX or N and current field < 1.0 (most likely PLX)
          ## TLE - 6/2005   no longer multiply PLX value by 1000 -- keep in arcsec
          $val = ${ $objRef->{lineContents} }[$ind];
          $objRef->{tagPLX} = $val
              if ( defined($val) && ($val ne $fieldFmts{tagPLX}[0]) && ($val ne '?') );
          $ind++;
	  ## continue on
      }

    }  ## finished through BminusV

    if ( $isOld && ( ($nfields == 19) ||
	     (($nfields == 18) && !(exists $objRef->{tagPLX})) ) )
    {
      ## if is Old format and right after BminusV:
      ## max # of params - must have PLX, N (mag)
      ##   OR has either PLX or N and PLX key not defined
      $val = ${ $objRef->{lineContents} }[$ind];
      $objRef->{tagN} = $val
          if ( defined($val) && ($val ne $fieldFmts{tagN}[0]) && ($val ne '?') );
      $ind++;
    }

    if ($isNew)
    {
      # New format line -- run through tags (in any order)

      foreach ($ind .. $nfields-1)
      {
        ($key, $val) = split /=/,  ${ $objRef->{lineContents} }[$_];
        $objRef->{"tag".$key} = $val
            if ( defined($val) && ($val ne $fieldFmts{"tag".$key}[0]) && ($val ne '?') );
      }
      ##if (grep m/TRG/, $objRef->{line})
      ##{
        ##push @reqFields, @newTrgReqTags;
      ##}
      ##elsif (grep m/CAL/, $objRef->{line})
      ##{
        ##push @reqFields, @calReqTags;
      ##}
      ##else
      ##{
        ##push @reqFields, @engReqTags;
      ##}
    }
    else
    {
      # Old or Sky format line 

      $val = ${ $objRef->{lineContents} }[$ind];
      $objRef->{tagSPECTYP} = $val
          if ( defined($val) && ($val ne $fieldFmts{tagSPECTYP}[0]) && ($val ne '?') );
      $ind++;

      if (grep m/trg/, $objRef->{line})
      {
        # is target line
	$objRef->{tagROLE} = "TRG";
        ##push @reqFields, @trgReqTags;
      }
      else
      {
	# is calibrator/eng line

        $val = ${ $objRef->{lineContents} }[$ind];
        $objRef->{tagD} = $val
            if ( defined($val) && ($val ne $fieldFmts{tagD}[0]) && ($val ne '?') );
        $ind++;
	my ($val1, $val2) = 
	     split /\+\/-/, ${ $objRef->{lineContents} }[$ind];
        $objRef->{tagDIAM} = $val1
            if ( defined($val1) && ($val1 ne $fieldFmts{tagDIAM}[0]) && ($val1 ne '?') );
        $objRef->{tagDIAMERR} = $val2
            if ( defined($val2) && ($val2 ne $fieldFmts{tagDIAMERR}[0]) && ($val2 ne '?') );
        $ind += 2;
        $val = ${ $objRef->{lineContents} }[$ind];
        $objRef->{tagCALFOR} = $val
            if ( defined($val) && ($val ne $fieldFmts{tagCALFOR}[0]) && ($val ne '?') );
        $ind++;

        if (grep m/cal/, $objRef->{line})
        {
          $objRef->{tagROLE} = "CAL";
          ##push @reqFields, @calReqTags;
        }
        else
        {
          $objRef->{tagROLE} = "ENG";
          ##push @reqFields, @engReqTags;
        }
      }

    }

    #### TLE - 6/2005  for all formats -- check for missing required fields
    ##foreach my $itag ( @reqFields )
    ##{
    ##  if (! defined ($objRef->extractField($itag)) )
    ##  {
    ##    print STDERR 
    ##       "Warning: internalLine::newFromFileLine is missing field \"$itag\" for desig \"",
    ##            $objRef->extractField("designation"),"\"\n";
    ##  }
    ##}
}


#########################################################################
## field extraction member function
#########################################################################
sub extractField
{
## declarations  ########################################################
    my $this = shift @_;
    my $field = shift @_;

## begin  ###############################################################
    if (defined $this->{$field})  ## see if there's a field already
    {
	return $this->{$field};
    }
    elsif (!(grep m/$field/, keys %InternalKeys)) ## else see if its a valid key
    {
	# DEBUG - does this need to be reported?
	print STDERR 
           "Warning: internalLine::extractField: requesting invalid field name \"$field\"\n" if (gcConfig::debug() >= 2);
	return undef;
    }
    else  ## otherwise return undefined
    {
	return undef;
    }
}  ## end extractField  #################################################

#########################################################################
## field test member function
#########################################################################
sub fieldExists
{
## declarations  ########################################################
    my $this = shift @_;
    my $field = shift @_;

## begin  ###############################################################
    ## see if this field has been given some value, even if it's "undef"
    if (exists $this->{$field})
    {
	return 1;
    }
    else
    {
	return 0;
    }
}  ## end fieldExists  #################################################


#########################################################################
## required fields check member function
#########################################################################
sub checkFields
{
## declarations  ########################################################
    my $this = shift @_;
    my @reqFields = @_;
    my $retStr = "";

## begin  ###############################################################
    foreach my $tag ( @reqFields )
    {
      if (! defined ($this->extractField($tag)) )
      {
        print STDERR 
           "Warning: internalLine::checkFields: internalLine is missing field \"$tag\" for desig \"",
                $this->extractField("designation"),"\"\n";
        $retStr .= $tag . " ";
      }
    }

    return $retStr;

}  ## end checkFields  #################################################



#########################################################################
## field set member function
#########################################################################
sub setField
{
## declarations  ########################################################
    my $this = shift @_;
    my $field = shift @_;
    my $value = shift @_;

## begin  ###############################################################

    if (grep m/$field/, keys %InternalKeys)
    {
        if ($field eq "tagSPECTYP" && defined($value))
        {
	  $value =~ s/\s+//g;        ## eliminate spaces
        }
	$this->{$field} = $value;
    }
    else
    {
	print STDERR 
           "Warning: internalLine::setField: setting unknown field name \"$field\"\n";
    }

}  ## end setField  #####################################################


#########################################################################
## add comment function
#########################################################################
sub addComment
{
## declarations  ########################################################
    my $this = shift @_;
    my $cLines = shift @_;

## begin  ###############################################################

    #print STDERR "DEBUG: addComment: \"$cLines\"\n";
    #print STDERR "DEBUG: addComment: was \"$this->{"tagCMNT"}\"\n";
    chomp($cLines);	# strip trailing newline, if any

    # Squash multiline comments into a single long string by replacing
    # newlines with a delimiter.
    # This is gonna break if the comment has this delimiter in it already!
    # The delimiter is "%%".
    # The delimiters will be changed back into newlines on output.
    $cLines =~ s/\n/%%/g;	# turn newlines into an internal delimiter,

    if ($cLines)	# only add non-empty comments
    {
	if ($this->{"tagCMNT"})
	{
	    $this->{"tagCMNT"} .= "%%" . $cLines;
	}
	else
	{
	    $this->{"tagCMNT"} = $cLines;
	}
    }
    #print STDERR "DEBUG: addComment: now \"$this->{"tagCMNT"}\"\n";
}  ## end addComment  #####################################################


#########################################################################
## lineOutput -- make a string format version of the internalLine object
## This format is used as an encoding to pass internalLine objects
## between getCal modules.  It should not be used for anything else
## since it is subject to change at the whim of the getCal development
## team!
#########################################################################
sub lineOutput
{
## declarations  ########################################################
    my $this = shift @_;
    my $outputLine = "";

## begin lineOutput  ####################################################

    # Use the list of field names in iLineFields to get each field
    # from the array in order, and write it into the output line
    # using its format spec from the fieldFmts hash: $fieldFmts{tag}[1]

    foreach my $tag ( @iLineFields )
    {
        $outputLine .= valToStr($fieldFmts{$tag}[1],$this->{$tag});
        $outputLine .= $fieldDelimiter;
    }

    $outputLine .= "\n";  ## terminate the output line

    print STDERR "DEBUG: internalLine lineOutput: ", $outputLine if (gcConfig::debug() >= 2);

    return $outputLine;

} ## end lineOutput  ####################################################


#########################################################################
## outputNewFormat -- make a file output format string with the contents 
##               of the internalLine object
#########################################################################
sub outputNewFormat
{
## declarations  ########################################################
    my $this = shift @_;
    my $noIntrsp = shift @_;
    my $sigFigures = shift @_;
    my $opt_plx = shift @_;
    my $opt_Nband = shift @_;
    ## TLE - 6/2005 now check $opt_plx for output

    my $outputLine = "";
    my $type; 
    my $val;
    my @reqFields;
    my @optFields;

## begin outputNewFormat  ####################################################

    # Write data into the output line using its format spec from the 
    # fieldFmts hash: $fieldFmts{tag}[2]

    $this->dump() if (gcConfig::debug() >= 3);	## DEBUG -- TLE 3/2005 added 'if' for debugging

    # Compose the output line(s) -- first any comments
    if (! $noIntrsp)
    {
        my $cmnt = $this->extractField("tagCMNT");
        if ($cmnt)
	{
	    $cmnt =~ s/%%/\n/g;	# turn comment delimiters back into a newlines
	    $outputLine .= $cmnt . "\n";
	}
    }

    ## TLE - 8/2005  set raHMS and decDMS fields
    $this->setField("raHMS",&RAhms($this->extractField("raDeg"), $sigFigures));
    $this->setField("decDMS",&DECdms($this->extractField("decDeg"), $sigFigures));
    
    # Next print out fields
    foreach my $tag ( @newReqFieldsOut )
    {
      $val = $this->extractField($tag);
      if (defined $val && ($val ne '' || isNumeric ($val)))
      {
        $outputLine .= (sprintf $fieldFmts{$tag}[2], $val) . ' ';
      }
      else
      {
        $outputLine .= $fieldFmts{$tag}[0] . ' ';
      }
    }

    ## print tagged required and optional fields

    $type = $this->extractField("tagROLE");
    if ($type eq 'TRG')
    {
      ## target line
      @reqFields = (@newTrgReqTags);
      @optFields = (@trgOptTags);
    }
    elsif ($type eq 'CAL')
    {
      ## calibrator line
      @reqFields = (@calReqTags);
      @optFields = (@calOptTags);
    }
    else
    {
      ## engineering line
      @reqFields = (@engReqTags);
      @optFields = (@calOptTags);
    }
    push @reqFields, "tagPLX" if ($opt_plx);  ## iff requested, add tagPLX to list

    foreach my $tag (@reqFields)
    {
      ## print required tags
      my $shortTag = $tag;
      $shortTag =~ s/^tag//;	# remove the "tag" prefix
      $val = $this->extractField($tag);
      if (defined $val && ($val ne '' || isNumeric ($val)))
      {
          $outputLine .= (sprintf $shortTag . "=" . $fieldFmts{$tag}[2], $val) . ' ';
      }
      else
      {
          # required value wasn't defined, use uninit string
          $outputLine .= $shortTag . "=" . $fieldFmts{$tag}[0] . ' ';
      }
    }

    foreach my $tag (@optFields)
    {
      ## print optional tags
      my $shortTag = $tag;
      $shortTag =~ s/^tag//;	# remove the "tag" prefix
      $val = $this->extractField($tag);
      if (defined $val && ($val ne '' || isNumeric ($val)))
      {
          $outputLine .= (sprintf $shortTag . "=" . $fieldFmts{$tag}[2], $val) . ' ';
      }
      #elsif ( $opt_Nband && ($tag eq "tagN") )
      elsif ($this->fieldExists($tag))
      {
	  # optional field was set but value was undef, blank, or not numeric
	  # print the field with it's "missing" indicator
          $outputLine .= $shortTag . "=" . $fieldFmts{$tag}[0] . ' ';
      }
    }

    $outputLine .= "\n";  ## terminate the output line

    return $outputLine;

} ## end outputNewFormat  ####################################################



#########################################################################
## outputOldFormat -- make a file output format string with the contents 
##               of the internalLine object
#########################################################################
sub outputOldFormat
{
## declarations  ########################################################
    my $this = shift @_;
    my $noIntrsp = shift @_;
    my $sigFigures = shift @_;
    ##my $noMsg = shift @_;
    my $opt_plx = shift @_;
    my $opt_Nband = shift @_;
    #print STDERR "DEBUG: outputOldFormat: noIntrsp=$noIntrsp, sigFigures=$sigFigures, plx=$opt_plx n=$opt_Nband\n";

    my $outputLine = "";
    my $type; 
    my $tag;
    my $val;

## begin outputOldFormat  ####################################################
    # Write data into the output line using its format spec from the 
    # fieldFmts hash: $fieldFmts{tag}[2] (usually)

    #print STDERR "DEBUG: outputOldFormat: outputting line:\n";	## SLG DEBUG
    $this->dump() if (gcConfig::debug() >= 3);	## DEBUG -- TLE 3/2005 added 'if' for debugging

    # Compose the output line(s) -- first any comments
    if (! $noIntrsp)
    {
        my $cmnt = $this->extractField("tagCMNT");
        if ($cmnt)
	{
	    $cmnt =~ s/%%/\n/g;	# turn comment delimiters back into a newlines
	    $outputLine .= $cmnt . "\n";
	}
    }

    ## TLE - 8/2005  set raHMS and decDMS fields
    $this->setField("raHMS",&RAhms($this->extractField("raDeg"), $sigFigures));
    $this->setField("decDMS",&DECdms($this->extractField("decDeg"), $sigFigures));
    
    # Next print out fields through tagSPECTYP

    foreach my $tag ( @oldReqFieldsOut )
    {
      next if ( (!$opt_plx) && ($tag eq "tagPLX") );
      next if ( (!$opt_Nband) && ($tag eq "tagN") );
      $val = $this->extractField($tag);
      if ($val)
      {
        $outputLine .= (sprintf $fieldFmts{$tag}[2], $val) . ' ';
      }
      else
      {
        $outputLine .= $fieldFmts{$tag}[0] . ' ';
      }
    }

    # print rest of fields

    $type = $this->extractField("tagROLE");
    if ($type eq 'TRG')
    {
      ## finish target line
      $outputLine .= "0.0 xxx   xxx  trg";
    }

    else
    {
      ## cal/eng line -- do rest of tags
      $val = $this->extractField("tagD");
      if ($val)
      {
        $outputLine .= (sprintf $fieldFmts{tagD}[2], $val) . ' ';
      }
      else
      {
        ## optional -- output if null?
        $outputLine .= $fieldFmts{tagD}[0] . ' ';
      }
      $val = $this->extractField("tagDIAM");
      if ($val)
      {
        $outputLine .= sprintf $fieldFmts{tagDIAM}[2], $val;
      }
      else
      {
        $outputLine .= $fieldFmts{tagDIAM}[0] . ' ';
      }
      $val = $this->extractField("tagDIAMERR");
      if ($val)
      {
        ## use Sky format with "+/-"!
        $outputLine .= (sprintf $fieldFmts{tagDIAMERR}[3], $val) . ' ';
      }
      else
      {
        $outputLine .= $fieldFmts{tagDIAMERR}[0] . ' ';
      }

      if ($type eq 'CAL')
      {
        ## calibrator line
        $outputLine .= " cal ";
        $val = $this->extractField("tagCALFOR");
        if ($val)
        {
          $outputLine .= sprintf $fieldFmts{tagCALFOR}[2], $val;
        }
        else
        {
          $outputLine .= $fieldFmts{tagCALFOR}[0] . ' ';
        }
      }
      else
      {
        ## engineering line
        $outputLine .= " eng ";
        $val = $this->extractField("tagPID");
        if ($val)
        {
          $outputLine .= "PID=" . (sprintf $fieldFmts{tagPID}[2], $val);
        }
        else
        {
          $outputLine .= $fieldFmts{tagPID}[0] . ' ';
        }
      }

    }

    $outputLine .= "\n";  ## terminate the output line

    return $outputLine;

} ## end outputOldFormat  ####################################################



#########################################################################
## outputSkyFormat -- make a file output format string with the contents 
##               of the internalLine object
#########################################################################
sub outputSkyFormat
{
## TLE - 3/2005   changed field specs (widths) to comply with Sky format spec 
##                and previous versions' output
##                also added valid-value checks so output will have '*?' when NA

## declarations  ########################################################
    my $this = shift @_;
    my $noIntrsp = shift @_;
    my $sigFigures = shift @_;
    ##my $noMsg = shift @_;

    my $outputLine = "";
    my $type; 
    my $tag;
    my $val;

## begin outputSkyFormat  ####################################################
    # Write data into the output line using its format spec from the 
    # fieldFmts hash: $fieldFmts{tag}[3]

    print STDERR "DEBUG: outputSkyFormat: outputting line:\n" if (gcConfig::debug() >= 2);	## SLG DEBUG
    $this->dump() if (gcConfig::debug() >= 3);	## DEBUG -- TLE 3/2005 added 'if' for debugging

    ## TLE - 8/2005  set raHMS and decDMS fields
    $this->setField("raHMS",&RAhms($this->extractField("raDeg"), $sigFigures));
    $this->setField("decDMS",&DECdms($this->extractField("decDeg"), $sigFigures));
    
    # Next print out fields through tagSPECTYP

    foreach my $tag ( @skyReqFieldsOut )
    {
      $val = $this->extractField($tag);
      if ($val)
      {
        if ($tag eq "pmRA")
        {
          # change pmRA to be in time seconds
          ## TLE - 6/2005   pmRA value now in arcsec 
          $val = $val/15.0;
        }
        $outputLine .= (sprintf $fieldFmts{$tag}[3], $val) . ' ';
      }
      else
      {
        $outputLine .= $fieldFmts{$tag}[0] . ' ';
      }
    }

    # print rest of fields

    $type = $this->extractField("tagROLE");
    if ($type eq 'TRG')
    {
      ## finish target line
      $outputLine .= "0.0 xxx   xxx  trg";
    }

    else
    {
      ## cal/eng line -- do rest of tags
      $val = $this->extractField("tagD");
      if ($val)
      {
        $outputLine .= (sprintf $fieldFmts{tagD}[3], $val) . ' ';
      }
      else
      {
        ## optional -- output if null?
        $outputLine .= $fieldFmts{tagD}[0] . ' ';
      }
      $val = $this->extractField("tagDIAM");
      if ($val)
      {
        $outputLine .= sprintf $fieldFmts{tagDIAM}[3], $val;
      }
      else
      {
        $outputLine .= $fieldFmts{tagDIAM}[0] . ' ';
      }
      $val = $this->extractField("tagDIAMERR");
      if ($val)
      {
        $outputLine .= (sprintf $fieldFmts{tagDIAMERR}[3], $val) . ' ';
      }
      else
      {
        $outputLine .= $fieldFmts{tagDIAMERR}[0] . ' ';
      }

      if ($type eq 'CAL')
      {
        ## calibrator line
        $outputLine .= " cal ";
        $val = $this->extractField("tagCALFOR");
        if ($val)
        {
          $outputLine .= sprintf $fieldFmts{tagCALFOR}[3], $val;
        }
        else
        {
          $outputLine .= $fieldFmts{tagCALFOR}[0] . ' ';
        }
      }
      else
      {
        ## engineering line
        $outputLine .= " eng ";
        $val = $this->extractField("tagPID");
        if ($val)
        {
          $outputLine .= "PID=" . (sprintf $fieldFmts{tagPID}[3], $val);
        }
        else
        {
          $outputLine .= $fieldFmts{tagPID}[0] . ' ';
        }
      }

    }
 
    $outputLine .= "\n";  ## terminate the output line

    return $outputLine;

} ## end outputSkyFormat  ####################################################



#########################################################################
## outputXEphemFormat -- make an XEphem output format string with the contents 
##               of the internalLine object
#########################################################################
sub outputXEphemFormat
{
## declarations  ########################################################
    my $this = shift @_;
    my $sigFigures = shift @_;

    my $outputLine = "";
    my $type; 
    my $tag;
    my $fld;

## begin outputXEphemFormat  ####################################################
    print STDERR "DEBUG: outputXEphemFormat: outputting line:\n" if (gcConfig::debug() >= 2);	## SLG DEBUG
    $this->dump() if (gcConfig::debug() >= 3);	## DEBUG -- TLE 3/2005 added 'if' for debugging

    # Compose the GUI line -- 
    $outputLine .= sprintf "%s,f|S|", $this->extractField("designation");

    $outputLine .= sprintf "%s,", $this->extractField("tagSPECTYP");

    # the astrometry (J2000!) 
    my ($hd, $min, $sec) = split " ", (&RAhms($this->extractField("raDeg"),$sigFigures), 3);
    $outputLine .= sprintf "%s:%s:%s,", $hd, $min, $sec; 
    ($hd, $min, $sec) = split " ", (&DECdms($this->extractField("decDeg"),$sigFigures), 3);
    $outputLine .= sprintf "%s:%s:%s,", $hd, $min, $sec; 

    # magnitudes
    $outputLine .= sprintf "%-3.1f,2000", $this->extractField("Vmag");

 
    $outputLine .= "\n";  ## terminate the output line


    return $outputLine;


} ## end outputXEphemFormat  ####################################################


#########################################################################
## valToStr
## use sprintf to convert a value to a string, with the given sprintf
## format descriptor.
## If the value is undefined or is just whitespace, returns empty string.
#########################################################################
sub valToStr
{
    my $sprintfFmt = shift @_;	# format descriptor given to sprintf
    my $val = shift @_;		# the value to be converted

    # if the value is defined and is not whitespace or just "?", sprintf it
    if ((defined $val) && ($val =~ m/\S+/) && ($val ne '?'))
    {
	my $str = sprintf "$sprintfFmt", $val;
	print STDERR 
           "DEBUG: valToStr: format $sprintfFmt, value \"$val\", returning \"$str\"\n" if (gcConfig::debug() >= 3);
	return $str;
    }
    else
    {
	print STDERR "DEBUG: valToStr: value undefined or blank, returning \"\"\n" if (gcConfig::debug() >= 3);
	return "";
    }
}

## end valToStr  ####################################################


#########################################################################
## dump
## dump object to STDERR for debugging purposes
#########################################################################
sub dump
{
    print STDERR "DEBUG: *** Dumping object: ***\n";
    my $this = shift@_;
    foreach my $key (sort keys %$this)
    {
	print STDERR "DEBUG: dump key ",$key, " value \"", $this->{$key},"\"\n";
    }
    print STDERR "DEBUG: *** dump done ***\n"
}

## end dump  ####################################################


# Extract the identifier. Also strip any leading or trailing whitespace.
# By default, ths identifier is pulled from catIndex, catIndex2,
# or designation, in that order of preference.  However, if you want, you can
# pass in the member names in your order of preference.
sub getIdent
{
    my $this = shift;
    my $f1 = shift || "catIndex";
    my $f2 = shift || "catIndex2";
    my $f3 = shift || "designation";
    my $d = trimWhite($this->extractField($f1));
    $d = trimWhite($this->extractField($f2)) if (!$d);
    $d = trimWhite($this->extractField($f3)) if (!$d);
    return $d;
}


# Return the identifier, mangled for output
sub getOutputIdent
{
    my $this = shift;
    return mangleIdent($this->getIdent());
}
