package interval;
my $MODVERSION = '$Id: interval.pm,v 1.3 2008/11/12 23:28:23 sgroom Exp $';

##########################################################################
# NASA Exoplanet Science Institute (NExScI) -- http://nexsci.caltech.edu #
#       Copyright (c) 1997-2008 California Institute of Technology       #
#                     U.S. Government Sponsorship                        #
# Under contract with the National Aeronautics and Space Administration  #
##########################################################################

# This module contains subroutines dealing with lists of intervals
# expressed as [begin,end] value pairs.
# Used by "timing" and "tGui" in processing delay and pointing ranges.

use strict;
use warnings;

use Exporter();
our @ISA = qw(Exporter);
our @EXPORT = qw(intersection intersectList);

use gcDebug;


#########################################################################
## intersection -- compute intersection of two intervals
#########################################################################
sub intersection
{
## declarations intersection  ###########################################
    ## intersect: computes the intersection of two input intervals
    ## [a1,a2], and [b1,b2]
    ## Note that logic assumes all wraps have been removed
    my $a1 = shift @_;	# first low
    my $a2 = shift @_;	# first high
    my $b1 = shift @_;  # second low
    my $b2 = shift @_;	# second high

## begin intersection  ##################################################
    ## Define the minimum of the intersection interval
    my $minValue = $a1;
    if ($minValue < $b1)
    {
	$minValue = $b1;
    }

    ## Define the maximum of the intersection interval
    my $maxValue = $a2;
    if ($maxValue > $b2)
    {
	$maxValue = $b2;
    }

    ## Make sure there is intersection (min < max)
    if ($minValue < $maxValue)
    {
	return (1,$minValue,$maxValue);
    }
    else ## otherwise return a null intersection
    {
	return 0;
    }
} ## end intersection  ##################################################


#########################################################################
## intersectList -- compute intersection of two lists of intervals
#########################################################################
sub intersectList
{
    # Compute the intersection of two lists of intervals, storing
    # the intersection in a third list. The return value is 1 if
    # any intersections were found, 0 if no intersection was found.
    # The three lists must be passed by reference. The third list will be
    # overwritten with the computed list of intersected intervals.
## declarations intersectList  ##################################################

    my $aListRef = shift @_;	# list of [a1, a2] pairs
    my $bListRef = shift @_;	# list of [b1, b2] pairs
    my $iListRef = shift @_;	# list of [i1, i2] pairs to be returned

    my $intersectFlag = 0;	# did we find any intersections?

## begin intersectList  ##################################################

    my $dbgname = "interval::intersectList";

    # List where we'll store the intersection, and
    # copy it to the destination at the end.
    my @tmpList;	# initially empty
    
    my $aSize = scalar(@$aListRef);	# number of [a1, a2] pairs
    my $bSize = scalar(@$bListRef);	# number of [b1, b2] pairs
    my $iSize = 0;			# number of [i1, i2] pairs
    gcDebug $dbgname, 2, "A has " . $aSize . " entries, B has " . $bSize . " entries\n";

    # intersect each interval [a1,a2] with each interval [b1,b2]
    for (my $a = 0; $a < $aSize; $a++)
    {
	my $a1 = $aListRef->[$a][0];
	my $a2 = $aListRef->[$a][1];
	for (my $b = 0; $b < $bSize; $b++)
	{
	    my $b1 = $bListRef->[$b][0];
	    my $b2 = $bListRef->[$b][1];
	    # compare the two intervals, and record the intersection, if any.
	    gcDebug $dbgname, 2, "checking [$a1, $a2] with [$b1, $b2]\n";
	    my ($iFlag, $i1, $i2) =
	    	intersection($a1, $a2, $b1, $b2);
	    if ($iFlag == 1)
	    {
		gcDebug $dbgname, 2, "found intersection [$i1, $i2]\n";
		$intersectFlag = 1;
		push @tmpList, [$i1, $i2];
		$iSize++;
	    }
	}
    }

    # finally, copy the result list to the destination
    @$iListRef = @tmpList;

    gcDebug $dbgname, 2, "returning intersection list with " . $iSize . " entries\n";
    for (my $i = 0; $i < $iSize; $i++)
    {
	gcDebug $dbgname, 2, "returning [" . $iListRef->[$i][0] . ", " . $iListRef->[$i][1] ."]\n";
    }
    return $intersectFlag;
} ## end intersectList  ##################################################
