package linearDiameter;
my $MODVERSION = '$Id: linearDiameter.pm,v 1.8 2008/11/12 23:28:23 sgroom Exp $';
use strict;
use warnings;
require 5.000;

use Exporter();

our @ISA = qw(Exporter);
our @EXPORT = qw(lDiam);


##########################################################################
# NASA Exoplanet Science Institute (NExScI) -- http://nexsci.caltech.edu #
#       Copyright (c) 1997-2008 California Institute of Technology       #
#                     U.S. Government Sponsorship                        #
# Under contract with the National Aeronautics and Space Administration  #
##########################################################################


use tableInterpolate;
use parseSpType;
use gcConfig;

my $cfg = gcConfig::config();


# Linear diameter model values from Allen

my %LdiamTable = (
    # main sequence
    "O0V" => +2.0,
    "O5V" => 1.25,

    "B0V" => 0.87,
    "B5V" => 0.58,

    "A0V" => 0.40,
    "A5V" => 0.24,

    "F0V" => 0.13,
    "F5V" => 0.08,

    "G0V" => 0.02,
    "G5V" => -0.03,

    "K0V" => -0.07,
    "K5V" => -0.13,

    "M0V" => -0.20,
    "M2V" => -0.3,
    "M5V" => -0.50,
    "M8V" => -0.90,

    # giant sequence
    "B0III" => 1.2,
    "B5III" => 1.0,

    "A0III" => 0.8,

    "F5III" => 0.6,

    "G0III" => 0.8,
    "G5III" => 1.0,

    "K0III" => 1.2,
    "K5III" => 1.4,

    # supergiant sequence
    "B0I" => 1.3,
    "B5I" => 1.5,

    "A0I" => 1.6,
    "A5I" => 1.7,

    "F0I" => 1.8,
    "F5I" => 1.9,

    "G0I" => 2.0,
    "G5I" => 2.1,

    "K0I" => 2.3,
    "K5I" => 2.6,

    "M0I" => 2.7,
    "M2I" => 2.9,

);


#########################################################################
## lDiam -- estimate linear diameter from spectral type
#########################################################################
# linear diameter estimation subroutine.  This just does a table lookup
# and interpolation into our linear diameter table based on spectral
# type.
#
# The luminosity class must not be blank. Also, because the table
# only contains values for LC I, III, and V, this
# routine treats LC II as LC I, and LC IV as LC V.
#
# Returns estimated diameter on success.
# Returns undef if LC was missing or table interpolation failed.
#
sub lDiam
{
    my $spType = shift @_;

    # check and remap LC's if needed
    my ($major,$minor,$lc) = parseSpType($spType);
    if (!$lc) {
	# can't estimate diameter for missing LC
	print STDERR "DEBUG: linearDiameter::lDiam: LC is missing for spType \"$spType\"\n" if ($cfg->debug());
	return undef;
    } elsif ($lc eq "II") {
	$lc = "I";
    } elsif ($lc eq "IV") {
	$lc = "V";
    }
    # Reassemble the spectral type
    $spType = join '', $major, $minor, $lc;

    my $output = &tableInterpolate($spType,\%LdiamTable);
    ##print STDERR "DEBUG: lDiam: spType=$spType, output=$output\n";
    if (!defined $output)
    {
	print STDERR "DEBUG: linearDiameter::lDiam: lookup failed for spType \"$spType\"\n" if ($cfg->debug());
	return undef;
    }
    my $solarDiamAU = 2.0 * 6.96e8 / 1.496e11;
    return ((10**$output) * $solarDiamAU);
} ## end lDiam #########################################################

return 1;
