package miscString;
my $MODVERSION = '$Id: miscString.pm,v 1.17 2007/06/13 21:03:24 sgroom Exp $';

# This module contains miscellaneous string- and text-related functions

use strict;
use warnings;

use Exporter();
our @ISA = qw(Exporter);
our @EXPORT = qw(trimWhite trimMultiSpace floatString isNumeric html2txt
		 urlQuote formatValues exec_stdin numberOnly nonNull
		 );

my $UINT_REGEX = '[0-9]+';
my $UFLOAT_SIMPLE_REGEX = '[0-9]*\.[0-9]+'; # float without exponent
my $SIGN_PREFIX_REGEX = '(\+|\-)';
my $INT_REGEX = sprintf('(%s)?%s', $SIGN_PREFIX_REGEX, $UINT_REGEX); # signed int
my $FLOAT_SIMPLE_REGEX = sprintf('(%s)?%s', $SIGN_PREFIX_REGEX, $UFLOAT_SIMPLE_REGEX); # signed float
my $MANTISSA_REGEX = sprintf('((%s)|(%s))', $FLOAT_SIMPLE_REGEX, $INT_REGEX);
my $EXP_DELIM_REGEX = '[eE]';
my $EXPONENT_REGEX = sprintf('%s%s', $EXP_DELIM_REGEX, $INT_REGEX);
my $NUMBER_REGEX = sprintf('%s(%s)?', $MANTISSA_REGEX, $EXPONENT_REGEX); # signed int or float, optionally in scientific notation

# Trim whitespace from the front and back of a string
sub trimWhite
{
    my $s = shift @_;
    if (defined ($s)) {
	$s =~ s/(^\s+)|(\s+$)//g;
    } else {
	$s = '';
    }
    return $s;
}

# Replace any multi-space runs with single spaces
sub trimMultiSpace
{
    my $s = shift @_;
    if (defined ($s)) {
	$s =~ s/\s+/ /g;
    }
    return $s;
}

# Return 0 instead of a null string or whitespace
sub floatString
{
    my $f = 0.0;
    my $s = shift @_;
    if (defined ($s)) {
	if ((my $s2 = trimWhite($s))) {
	    $f = $s2;
	}
    }
    return $f;
}

# Return 1 if the string can be converted to a number
sub isNumeric {
    my $v = shift;
    my $nr = sprintf('^\s*%s\s*$', $NUMBER_REGEX);
    return 1 if (defined($v) && $v =~ m/$nr/);
    return 0;
}

sub numberOnly {
    my $v = shift;
    my $nr = sprintf('^[^0-9\+\-\.]*(%s).*$', $NUMBER_REGEX);
    $v =~ m/$nr/;
    my $number = $1;
    return '' if (!isNumeric($number));
    return $number;
}

sub nonNull {
    my $s = shift;
    my $default = shift;
    return $s if (defined($s));
    return $default;
}

# Convert HTML to ASCII for a given line width
sub html2txt {
    use HTML::TreeBuilder;
    use HTML::FormatText;
    my $html = shift;
    my $lineWidth = shift;
    $lineWidth = 80 if (!defined($lineWidth));
    my $tree = HTML::TreeBuilder->new_from_content($html);
    my $formatter = HTML::FormatText->new(leftmargin => 0, rightmargin => $lineWidth);
    return $formatter->format($tree);
}

# Quote a string for insertion into a URL
# SG 12/06: this isn't really the complete set, see RFC 2396
# Only allowed chars should be alphanum and 
# "-" | "_" | "." | "!" | "~" | "*" | "'" | "(" | ")"
# should instead match on only valid chars and escape everything else,
# instead of trying to enumerate what should be escaped.
# HOWEVER...
# some of those chars ('?','&','=') are commonly used in URL's
# so we'll leave them alone for now.
sub urlQuote {
    my $s = shift;
    #my @hex_needed_chars = ('%', '+', '?', '&', '=', ' ', '#', ';', '|', ',', '"', ':', '[', ']');
    my @hex_needed_chars = ('%', '+', ' ', '#', ';', '|', ',', '"', ':', '[', ']');
    foreach my $qc (@hex_needed_chars) {
	my $qqc = sprintf("%%%X", ord($qc));
	eval "\$s =~ s/\\$qc/\$qqc/g";
    }
    return $s;
}

# Return a string of formatted values, where each value may or may not
# be defined.
# Arguments:
# 0: Default string to print for an undefined value
# 1: A separator to print between values, defined or undefined
# 2: printf()-style format string for the first value
# 3: first value
# 4: printf()-style format string for the second value
# 5: second value
# ...
# The return value is a line containing the formatted values
sub formatValues
{
    my $undefValue = shift;
    my $sep = shift;
    my $fline = "";
    my ($fmt, $value) = (shift, shift);
    while(defined($fmt)) {
	if (defined($value)) {
	    $fline .= sprintf($fmt, $value);
	} else {
	    $fline .= $undefValue;
	}
	$fline .= $sep if defined($sep);
	($fmt, $value) = (shift, shift);
    }
    return $fline;
}

# Execute a command, given that command and the text to be fed to its
# standard input.  Return the standard output.
# This function is safer than running `echo \"input data\" | command` directly.
sub exec_stdin {
    my $cmd = shift;
    my $input = shift;
    my $debug = shift || 0;

    # Escape double-quotes in the input to avoid an error in $use_cmd
    my $mod_input = $input;

    my ($dq, $dqr) = ('"', '\"');
    $mod_input =~ s/$dq/$dqr/g;
    
    my $use_cmd = "echo \"$mod_input\" | $cmd";
    print STDERR "DEBUG: exec_stdin: running command: \"$cmd\"\n" if ($debug);
    print STDERR "DEBUG: exec_stdin: command standard input:\n$input\n" if (($debug) && ($debug>=2));
    print STDERR "DEBUG: exec_stdin: exact command is: \"$use_cmd\"\n" if (($debug) && ($debug>=2));
    my $outlines = "";
    open(CMDOUT, "-|", $use_cmd) or die "Command failed: $cmd\nStandard input:\n$input\n";
    while (my $l = <CMDOUT>) {
	$outlines .= $l;
    }
    close(CMDOUT);
    printf STDERR "DEBUG: exec_stdin: command standard output:\n$outlines\n" if (($debug) && ($debug>=2));
    chomp($outlines);
    return $outlines;
}

# mimic the perl "system" function, letting us trace who's calling whom
# returns exit code from the system() call
sub exec_system
{
    my $cmd = shift;
    my $debug = shift || 0;

    print STDERR "DEBUG: exec_system: running command: \"$cmd\"\n" if ($debug);
    my $outlines = system($cmd);
    my $result = $?;
    return $result;	# returns exit code
}

# mimic the perl backtick operator, letting us trace who's calling whom
# returns command's stdout
sub exec_backtick
{
    my $cmd = shift;
    my $debug = shift || 0;

    print STDERR "DEBUG: exec_backtick: command: \"$cmd\"\n" if ($debug);
    my $outlines = `$cmd`;
    print STDERR "DEBUG: exec_backtick: command standard output:\n$outlines\n" if (($debug) && ($debug>=2));
    return $outlines;	# returns the output of the command
}


return 1
