package siderealTime;
my $MODVERSION = '$Id: siderealTime.pm,v 1.4 2008/11/12 23:28:23 sgroom Exp $';
use strict;
use warnings;

require 5.000;

use Exporter();

our @ISA = qw(Exporter);
our @EXPORT = qw(siderealTime localSiderealTime);


##########################################################################
# NASA Exoplanet Science Institute (NExScI) -- http://nexsci.caltech.edu #
#       Copyright (c) 1997-2008 California Institute of Technology       #
#                     U.S. Government Sponsorship                        #
# Under contract with the National Aeronautics and Space Administration  #
##########################################################################


use novasConstants;


#########################################################################
## siderealTime -- compute sidereal time for input JD
#########################################################################
sub siderealTime
{
# This is a perl implementation of the novas sidereal time function
# Original credit goes to George Kaplan, USNO
# novasC call arguments (double julianhi, double julianlo, double ee, double *gst)

# our call arguments ($JulianDate,$EqnEqinoxes)

# Novas routine header:
#   PURPOSE:    
#      Computes the Greenwich apparent sidereal time, at Julian date
#      'julianhi' + 'julianlo'.
#
#   REFERENCES: 
#      Aoki, et al. (1982) Astronomy and Astrophysics 105, 359-361.
#      Kaplan, G. H. "NOVAS: Naval Observatory Vector Astrometry
#         Subroutines"; USNO internal document dated 20 Oct 1988;
#         revised 15 Mar 1990.
#
#   INPUT
#   ARGUMENTS:
#      julianhi (double)
#         Julian date, integral part.
#      julianlo (double)
#         Julian date, fractional part.
#      ee (double)
#         Equation of the equinoxes (seconds of time)
#
#   OUTPUT
#   ARGUMENTS:
#      *gst (double)
#         Greenwich apparent sidereal time, in hours.
#
#  RETURNED
#   VALUE:
#      None.
#
#   GLOBALS
#   USED:
#      T0
#
#   FUNCTIONS
#   CALLED:
#      pow     math.h
#      fmod    math.h
#
#   VER./DATE/
#   PROGRAMMER:
#      V1.0/06-92/TKB (USNO/NRL Optical Interfer.) Translate Fortran.
#      V1.1/08-93/WTH (USNO/AA) Update to C programing standards.
#      6/98 perl version by Andy Boden, JPL/Caltech 
#
#   NOTES:
#      1. This function is based on Fortran NOVAS routine 'sidtim'.
#      2. The Julian date may be split at any point, but for highest
#      precision, set 'julianhi' to be the integral part of the Julian
#      date, and set 'julianlo' to be the fractional part.
#

## declarations siderealTime  ###########################################
# let perl do the optimal julian date split
    my $JD = shift @_;
    my ($jdHi,$jdLo) = split /\./, $JD;
    $jdLo = "0." . $jdLo;

    my $EqnEquinoxes = 0.0;
    if (scalar @_)
    {
	$EqnEquinoxes = shift @_;
    }

## begin siderealTime  ##################################################
    my $timeHi = ($jdHi -  novasConstants::T0) / 36525.0;
    my $timeLo = $jdLo / 36525.0;

    my $time = $timeHi + $timeLo;

    my $st =  - 6.2e-6 * ($time**3.0)
              + 0.093104 * ($time**2.0)
	      + 67310.54841 + 8640184.812866 * $timeLo
	      + 3155760000.0 * $timeLo
	      + 8640184.812866 * $timeHi + 3155760000.0 * $timeHi;

    my $gst = $st / 3600.0;
    if ($EqnEquinoxes)
    {
	$gst += $EqnEquinoxes / 3600.0;
    }
    $gst -= 24.0 * (int ($gst / 24.0));
    

    if ($gst < 0.0)
    {
	$gst += 24.0;
    }

    return $gst;
} ## end siderealTime  ##################################################



#########################################################################
## localSiderealTime -- local sidereal time
#########################################################################
sub localSiderealTime
{
    # this subroutine is trivial, computing the local sidereal time
    # from the Greenwich apparent sidereal time

## declarations localSiderealTime  ######################################
    my $myJD = shift @_;
    my $localLongitude = shift @_;
    my $EqnEquinoxes = shift @_;

## begin localSiderealTime  #############################################
    $localLongitude /= 15.0; # convert to hours

    my $lst = &siderealTime($myJD,$EqnEquinoxes) + $localLongitude;

    if ($lst < 0.0)
    {
	$lst += 24.0;
    }

    return $lst;

} ## end localSiderealTime  #############################################


return 1; # for the require statement
