package sunMoon;
our $MODVERSION = '$Id: sunMoon.pm,v 1.2 2008/11/12 23:28:23 sgroom Exp $';

##########################################################################
# NASA Exoplanet Science Institute (NExScI) -- http://nexsci.caltech.edu #
#       Copyright (c) 1997-2008 California Institute of Technology       #
#                     U.S. Government Sponsorship                        #
# Under contract with the National Aeronautics and Space Administration  #
##########################################################################

#########################################################################
# This module contains routines for calculations related to sun and moon
# position and moon phase.
#########################################################################

use strict;
use warnings;

use Exporter();
our @ISA = qw(Exporter);
our @EXPORT = qw(sunPos moonPos moonPhase);

use POSIX;
use angles;
use vector;

#########################################################################
## sunPos - compute RA & DEC (deg) of sun at given date (JD)
#########################################################################
sub sunPos
{
    my $jd = shift;
    if (!defined $jd)
    {
	return undef;
    }
     
    my ($ra,$dec) = &sunCoord($jd);

    while ($ra < 0)
    {
	$ra += TWO_PI;
    }

    $ra /= DEG_TO_RAD;
    $dec /= DEG_TO_RAD;
    return ($ra, $dec);
}


#########################################################################
## moonPos - compute RA & DEC (deg) of moon at given date (JD)
#########################################################################
sub moonPos
{
    my $jd = shift;	# date as JD - required
    my $sid = shift;	# sidereal angle (deg) - optional
    my $lat = shift;	# latitude (deg) - optional

    if (!defined $jd)
    {
	return undef;
    }
     
    my ($ra,$dec) = &moonCoord($jd,$sid,$lat);

    while ($ra < 0)
    {
	$ra += TWO_PI;
    }

    $ra /= DEG_TO_RAD;
    $dec /= DEG_TO_RAD;
	 
    return ($ra, $dec);
}

#########################################################################
## moonPhase - compute moon phase at given date (JD), with
## optional topocentric correction
#########################################################################
sub moonPhase
{
    my $jd = shift;	# date as JD - required
    my $sid = shift;	# sidereal angle (deg) - optional
    my $lat = shift;	# latitude (deg) - optional
 
    my $result = &moonIlluminationDesc($jd,$sid,$lat);

    $result .= " -- ";
    $result .= &moonPhaseDesc($jd);

    return($result);
}


#########################################################################
######
###### Private (internal) functions below
######
#########################################################################


#########################################################################
## deltaT
## returns an approximate ET_UT difference in seconds */
## t is time in centuries since 1900 */
#########################################################################
sub deltaT
{
    my $t = shift @_;
    return (0.41 + 1.2053*$t + 0.4992*$t*$t) / 60.0;
} ## end deltaT



#########################################################################
## interpolate
## interpolates y, where y is an array of three 
##   doubles, into the interval n 
#########################################################################
sub interpolate
{
    my ($n, @y) = @_;
    my ($a,$b,$c);

    $a = $y[1] - $y[0];
    $b = $y[2] - $y[1];
    $c = $b - $a;
    return ($y[1] + ($n * ($a + $b + $n * $c)) / 2.0);
} ## end interpolate



#########################################################################
## sunCoord - compute sun RA, DEC (deg) as a function of time (JD)
#########################################################################
sub sunCoord
{
    my $jd = shift @_;

    ## Calculate the Julian century number */

    my $t = $jd - 2415020.0;
    $t /= 36525.0;

    ## Calculate the geometric mean longitude of the Sun in degrees 

    my $l = 279.69668 + 36000.76892*$t + 0.0003025*$t*$t;
    $l -= 360.0 * (int ($l / 360.0)); ## modulus 


    ## Calculate the Suns mean anomaly, in degrees 

    my $m = 358.47583 + 35999.04975*$t - 0.000150*$t*$t - 0.0000033*$t*$t*$t;
    $m = $m - 360.0 * (int ($m / 360.0)); ## modulus 
  
    ## The eccentricity of the earths' orbit 

    my $e = 0.01675104 - 0.0000418*$t - 0.000000126*$t*$t;

    ## Now, to find the true longitude and anomaly of the Sun, we
    ## first solve for the 'center' as follows */

    my $c = (1.919469 - 0.004789*$t - 0.000014*$t*$t) * sin($m * DEG_TO_RAD);
    $c += (0.020094 - 0.000100*$t) * sin(2.0 * $m * DEG_TO_RAD);
    $c += 0.000293 * sin(3.0 * $m * DEG_TO_RAD);


    ## The Suns' true longitude is then 

    my $longitude = $l + $c;

    ## And its true anomaly:

    my $anomaly = $m + $c;

    ## The Suns' radius vector (in AU) 

    my $r = 1.0000002 * (1.0 - $e * $e);
    $r /= 1.0 + $e * cos($anomaly * DEG_TO_RAD);

    ## In case we need to solve for the apparent longitude of the Sun

    my $omega = 259.18 - 1934.142*$t;
    $omega = $omega - 360.0 * (int ($omega / 360.0));

    my $longitude_app = $longitude - 0.00569 - 0.00479 * sin($omega * DEG_TO_RAD);

    ## Finally, one more thing is needed - the obliquity */

    my $obl = 23.4562294 - 0.0130125*$t - 0.00000164*$t*$t + 0.000000503*$t*$t*$t;

    ## If you are computing apparent solar position then add */

    $obl += 0.00256 * cos($omega * DEG_TO_RAD);

    ## Now solve for ra, dec */

    my $ra = atan2(cos($obl * DEG_TO_RAD) * sin($longitude_app * DEG_TO_RAD),
		   cos($longitude * DEG_TO_RAD) );

    my $sindec = sin($obl * DEG_TO_RAD) * sin($longitude_app * DEG_TO_RAD);
    my $dec = POSIX::asin($sindec);

    return ($ra,$dec);

} ## end sunCoord


#########################################################################
## moonCoord - compute moon RA, DEC (deg) as a function of time (JD) 
## and observer location (lon, lat)
#########################################################################
sub moonCoord
{

    my ($jd,$sid,$lat) = @_;

    
    ## implements "low precision" moon algorithms from Astronomical
    ## Almanac (p. D46 in 1992 version).  Does apply the topocentric
    ## correction.


    ## Units are as follows
    ## jd,lat, sid;   decimal hours
    ## *ra, *dec,   decimal hours, degrees
    ## *dist;      earth radii */


    ## double T, lambda, beta, pie, l, m, n, x, y, z, alpha, delta,
    ## rad_lat, rad_lst, distance, topo_dist;

    my $T = ($jd - 2451545.0) / 36525.0;  ## jul cent. since J2000.0 

    my $lambda = 218.32 + 481267.883 * $T
	+ 6.29 * sin((134.9 + 477198.85 * $T) / RAD_TO_DEG)
	- 1.27 * sin((259.2 - 413335.38 * $T) / RAD_TO_DEG)
	+ 0.66 * sin((235.7 + 890534.23 * $T) / RAD_TO_DEG)
	+ 0.21 * sin((269.9 + 954397.70 * $T) / RAD_TO_DEG)
	- 0.19 * sin((357.5 + 35999.05 * $T) / RAD_TO_DEG)
	- 0.11 * sin((186.6 + 966404.05 * $T) / RAD_TO_DEG);

    $lambda /= RAD_TO_DEG;

    my $beta = 5.13 * sin((93.3 + 483202.03 * $T) / RAD_TO_DEG)
	+ 0.28 * sin((228.2 + 960400.87 * $T) / RAD_TO_DEG)
	- 0.28 * sin((318.3 + 6003.18 * $T) / RAD_TO_DEG)
	- 0.17 * sin((217.6 - 407332.20 * $T) / RAD_TO_DEG);

    $beta /= RAD_TO_DEG;

    my $plx = 0.9508
	+ 0.0518 * cos((134.9 + 477198.85 * $T) / RAD_TO_DEG)
	+ 0.0095 * cos((259.2 - 413335.38 * $T) / RAD_TO_DEG)
	+ 0.0078 * cos((235.7 + 890534.23 * $T) / RAD_TO_DEG)
	+ 0.0028 * cos((269.9 + 954397.70 * $T) / RAD_TO_DEG);

    $plx /= RAD_TO_DEG;
    my $distance = 1.0 / sin($plx);

    my $l = cos($beta) * cos($lambda);
    my $m = 0.9175 * cos($beta) * sin($lambda) - 0.3978 * sin($beta);
    my $n = 0.3978 * cos($beta) * sin($lambda) + 0.9175 * sin($beta);

    my $x = $l * $distance;
    my $y = $m * $distance;
    my $z = $n * $distance;  ## for topocentric correction 

    ## Topocentric correction
    if (defined $lat && defined $sid)
    {
	$lat *= DEG_TO_RAD;
	$sid *= DEG_TO_RAD;
	$x -= cos($lat) * cos($sid);
	$y -= cos($lat) * sin($sid);
	$z -= sin($lat);
    }


    my $topo_dist = sqrt($x * $x + $y * $y + $z * $z);

    $l = $x / $topo_dist;
    $m = $y / $topo_dist;
    $n = $z / $topo_dist;

    my $ra = atan2($m,$l);
    my $dec = asin($n);

    return ($ra,$dec,$topo_dist);

}  ## end moonCoord


#########################################################################
## moonIlluminationDesc - returns a verbal description of
## moon illumination given date (JD) and optional observer location
#########################################################################
sub moonIlluminationDesc
{
    my ($jd,$sid,$lat) = @_;

    ## Compute the illuminated fraction of the moon
    my ($sunRAdeg,$sunDecdeg) = &sunPos($jd);
    my ($moonRAdeg,$moonDecdeg,$leftover2) = &moonPos($jd,$sid,$lat);


    my $illFrac = 0.5 * (1.0 - cos(&subtend($moonRAdeg,$moonDecdeg,
					    $sunRAdeg,$sunDecdeg)));
    $illFrac = sprintf "%2.0f", ($illFrac * 100.0);
    $illFrac .= "%";
    return " $illFrac moon illumination";

} ## end moonIllumination


#########################################################################
## moonPhaseDesc - return a verbal description of moon phase, given the
## julian date.
#########################################################################
sub moonPhaseDesc
{
    my $jd = shift @_;
    my $result;

    ## returns a verbal description of moon phase, given the julian
    ## date.

    ## Find current lunation
    my $nlast = int (($jd - 2415020.5) / 29.5307 - 1);  ## approximate?
    ## Find bounding new moons
    my $lastnewjd;
    my $newjd = &flmoon($nlast,0);
    my $count = 0;
    do
    {
	$lastnewjd = $newjd;
	$nlast++;
	$newjd = &flmoon($nlast,0);
	$count++;
    }
    while (($newjd < $jd) && ($count < 40));

    if ($count > 35)
    {
	$result = "Didn't find phase in moonPhase";
    }
    else  ## found lunation okay
    {
	my $x = $jd - $lastnewjd;
	$nlast--;
	my $noctiles = int ($x / 3.69134);  ## 3.69134 = 1/8 month; truncate. 
	if ($noctiles == 0)
	{
	    $result = sprintf "%3.1f days since new moon",$x;
	}  ## end new moon 1
	elsif ($noctiles <= 2)  ## nearest first quarter 
	{  
	    my $fqjd = flmoon($nlast,1);
	    $x = $jd - $fqjd;
	    if ($x < 0.0)
	    {
		$result = sprintf "%3.1f days before first quarter",(-1.0 * $x);
	    }
	    else
	    {
		$result = sprintf "%3.1f days since first quarter",$x;
	    }
	}  ## end first quarter
	elsif ($noctiles <= 4) ## nearest full
	{
	    my $fljd = &flmoon($nlast,2);
	    $x = $jd - $fljd;
	    if ($x < 0.0)
	    {
		$result = sprintf "%3.1f days until full moon",(-1.0 * $x);
	    }
	    else
	    {
		$result = sprintf "%3.1f days after full moon",$x;
	    }
	}  ## end full
	elsif ($noctiles <= 6) ## nearest last quarter
	{
	    my $lqjd = &flmoon($nlast,3);
	    $x = $jd - $lqjd;
	    if ($x < 0.0)
	    {
		$result = sprintf "%3.1f days before last quarter",(-1.0 * $x);
	    }
	    else
	    {
		$result = sprintf "%3.1f days after last quarter",$x;
	    }
	} ## end last quarter
	else
	{
	    $result = sprintf "%3.1f days before new moon",($newjd - $jd);
	} ## end new 2

    }  ## end found lunation

    return $result;

}  ## end moonPhaseDesc



sub flmoon
{

    my ($n,$nph) = @_;

    ## Gives jd (+- 2 min) of phase nph on lunation n; replaces less
    ## accurate Numerical Recipes routine.  This routine implements
    ## formulae found in Jean Meeus' *Astronomical Formulae for
    ## Calculators*, 2nd edition, Willman-Bell.  A very useful book!!

    ## n, nph lunation and phase; nph = 0 new, 1 1st, 2 full, 3 last
    ## jdOut   jd of requested phase 

    ##double jd, cor;
    ##double M, Mpr, F;
    ##double T;
    ##double lun;

    my $lun = $n + $nph / 4.0;
    my $T = $lun / 1236.85;
    my $jdOut = 2415020.75933 + 29.53058868 * $lun
	      + 0.0001178 * $T * $T
	      - 0.000000155 * $T * $T * $T
	      + 0.00033 * sin((166.56 + 132.87 * $T - 0.009173 * $T * $T) * DEG_TO_RAD);

    my $M = 359.2242 + 29.10535608 * $lun - 0.0000333 * $T**2 - 0.00000347 * $T**3;
    ##M = M / DEG_IN_RADIAN;
    $M *= DEG_TO_RAD;
    my $Mpr = 306.0253 + 385.81691806 * $lun + 0.0107306 * $T**3 + 0.00001236 * $T**3;
    ##Mpr = Mpr / DEG_IN_RADIAN;
    $Mpr *= DEG_TO_RAD;
    my $F = 21.2964 + 390.67050646 * $lun - 0.0016528 * $T**2 - 0.00000239 * $T**3;
    ##F = F / DEG_IN_RADIAN;
    $F *= DEG_TO_RAD;
    if (($nph == 0) || ($nph == 2))  ## new or full
    {
	my $correction =
	    (0.1734 - 0.000393 * $T) * sin($M)
		+ 0.0021 * sin(2 * $M)
		- 0.4068 * sin($Mpr)
		+ 0.0161 * sin(2 * $Mpr)
		- 0.0004 * sin(3 * $Mpr)
		+ 0.0104 * sin(2 * $F)
		- 0.0051 * sin($M + $Mpr)
		- 0.0074 * sin($M - $Mpr)
		+ 0.0004 * sin(2 * $F + $M)
		- 0.0004 * sin(2 * $F - $M)
		- 0.0006 * sin(2 * $F + $Mpr)
		+ 0.0010 * sin(2 * $F - $Mpr)
		+ 0.0005 * sin($M + 2 * $Mpr);
	$jdOut += $correction;
    }
    else
    {
	my $correction =
	    (0.1721 - 0.0004 * $T) * sin($M)
		+ 0.0021 * sin(2 * $M)
		- 0.6280 * sin($Mpr)
		+ 0.0089 * sin(2 * $Mpr)
		- 0.0004 * sin(3 * $Mpr)
		+ 0.0079 * sin(2 * $F)
		- 0.0119 * sin($M + $Mpr)
		- 0.0047 * sin($M - $Mpr)
		+ 0.0003 * sin(2 * $F + $M)
		- 0.0004 * sin(2 * $F - $M)
		- 0.0006 * sin(2 * $F + $Mpr)
		+ 0.0021 * sin(2 * $F - $Mpr)
		+ 0.0003 * sin($M + 2 * $Mpr)
		+ 0.0004 * sin($M - 2 * $Mpr)
		- 0.0003 * sin(2 * $M + $Mpr);
	if ($nph == 1)
	{
	    $correction +=  0.0028 - 0.0004 * cos($M) + 0.0003 * cos($Mpr);
	}
	if ($nph == 3)
	{
	    $correction += - 0.0028 + 0.0004 * cos($M) - 0.0003 * cos($Mpr);
	}
	$jdOut += $correction;

    }  ## end else

    return $jdOut;

}  ## end flmoon



sub subtend
{
    my ($ra1,$dec1,$ra2,$dec2) = @_;

    ## args in degrees

    ## angle subtended by two positions in the sky --
    ## return value is in radians.  Hybrid algorithm works down
    ## to zero separation except very near the poles.

    $ra1 *= DEG_TO_RAD;
    $dec1 *= DEG_TO_RAD;
    $ra2 *= DEG_TO_RAD;
    $dec2 *= DEG_TO_RAD;

    my @x1 = (cos($ra1) * cos($dec1), sin($ra1) * cos($dec1), sin($dec1));
    my @x2 = (cos($ra2) * cos($dec2), sin($ra2) * cos($dec2), sin($dec2));

    my $theta = acos(&dot(@x1,@x2));

    ## use flat Pythagorean approximation if the angle is very small
    ## *and* you're not close to the pole; avoids roundoff in arccos.


    if ($theta < 1.0e-5)
    {  ## seldom the case, so don't combine test */
	if ((abs($dec1) < (PI_OVER_TWO - 0.001))
	    && (abs($dec2) < (PI_OVER_TWO - 0.001)))
	{
	    ## recycled variables here... */
	    my $x1 = ($ra2 - $ra1) * cos(($dec1 + $dec2) / 2.0);
	    my $x2 = $dec2 - $dec1;
	    $theta = sqrt($x1**2 + $x2**2);
	}
    }

    return $theta;

}  ## end subtend


