package tableInterpolate;
my $MODVERSION = '$Id: tableInterpolate.pm,v 1.6 2008/11/12 23:28:23 sgroom Exp $';
use strict;
use warnings;
require 5.000;

use Exporter();

our @ISA = qw(Exporter);
our @EXPORT = qw(tableInterpolate);


##########################################################################
# NASA Exoplanet Science Institute (NExScI) -- http://nexsci.caltech.edu #
#       Copyright (c) 1997-2008 California Institute of Technology       #
#                     U.S. Government Sponsorship                        #
# Under contract with the National Aeronautics and Space Administration  #
##########################################################################

use parseSpType;

#########################################################################
## tableInterpolate -- interpolate values in hashes
#########################################################################
sub tableInterpolate($\%)
{
## declarations tableInterpolate  #######################################
    my $spType = shift @_;
    my $table = shift @_;
    ##print STDERR "DEBUG: tableInterpolate.pm: spType=$spType, table=$table\n";

## begin tableInterpolate  ##############################################
    # Split the input spectral type, and create a template key for
    # this object.  This is necessary because sometimes the spectral
    # type contains information beyond the MK classification.
    my ($major,$minor,$lc) = parseSpType($spType);
    my $thisKey = join '',$major,$minor,$lc;
    ##print STDERR "DEBUG: tableInterpolate.pm: major=$major, minor=$minor, lc=$lc\n";

    # Check in fact that this lumonosity class is currently
    # implemented.  If not, bag with an undef
    if (!(grep /\d$lc\Z/, keys %$table))
    {
        print STDERR "tableInterpolate.pm: Warning: lclass \"$lc\" not implemented in table\n";
	return undef;
    }

    # Save ourself a little work.  If we've got a table entry that
    # matches the input type, return that value (duh).
    if (grep /\A$thisKey\Z/, keys %$table)
    {
        ##print STDERR "DEBUG: tableInterpolate.pm: thisKey is in table\n";
	return $$table{$thisKey}
    }

    # We're still around, so find the array keys that _bound_ the
    # input value.  Do this by first pulling out all the table keys
    # that match this luminosity class (the grep call), and then
    # sorting the template key for this object into this reduced list
    # of table keys (the sort call).  We're sorting using the function
    # bySpectralType, which sorts the objects in order of decreasing
    # mass/temp (OBAFGKM), and then by luminosity class.  (Sometimes I
    # want to kiss Larry's feet.)
    my @keyList = sort bySpectralType ((grep /\d$lc\Z/, keys %$table), $thisKey);
    ##print STDERR "DEBUG: tableInterpolate.pm: keyList = @keyList\n";

    # Find the occurance of the object key in the sorted list.
    # Currently stupid linear search, maybe someday a binary search.
    my $indx = 0;
    while ($keyList[$indx] ne $thisKey)
    {
	$indx++;
    }

    # If it's at the begining or the end return the nearest entry
    if ($indx == 0)
    {
	return $$table{$keyList[1]};
    }
    elsif ($indx == (scalar @keyList - 1))
    {
	return $$table{$keyList[scalar @keyList - 2]};
    }
    else # otherwise, interpolate between the two entries that bound our
    {    # object key.
	my $lower = $keyList[$indx - 1];
	my ($lMajor,$lMinor,$lLC) = parseSpType($lower);
	my $upper = $keyList[$indx + 1];
	my ($uMajor,$uMinor,$uLC) = parseSpType($upper);
	if ($lMajor ne $uMajor)
	{
	    $uMinor += 10;
	}
        ##print STDERR "DEBUG: tableInterpolate.pm: lower=$lower, upper=$upper, lMajor=$lMajor, lMinor=$lMinor, lLC=$lLC\n";
        ##print STDERR "DEBUG: tableInterpolate.pm: uMajor=$uMajor, uMinor=$uMinor, uLC=$uLC\n";

	return $$table{$lower}
	 + ($$table{$upper} - $$table{$lower}) * ($minor - $lMinor) / ($uMinor - $lMinor);

    } # end interpolation

} ## end tableInterpolate  ##############################################



#########################################################################
## bySpectralType -- spectral type sorting
#########################################################################
sub bySpectralType
{
# bySpectralType -- mechanizes a sort order for sorting hash
# (associative array) keys by star spectral type.

## begin bySpectralType  ################################################
    # signed less than equal to.  I can never remember how this goes,
    # so I hard-code it in.

    my $good = 1 <=> 5;
    my $reverse = 5 <=> 1;

    # parse the spectral type keys
    my ($majorA,$minorA,$lcA) = parseSpType($a);
    my ($majorB,$minorB,$lcB) = parseSpType($b);

    # Start the sort order check.  First check spectral class
    if ($majorA eq $majorB) # same spectral classes
    {
	if ($minorA eq $minorB) # same subclasses
	{
	    return $lcA cmp $lcB; # sort by luminosity class
	}
	else
	{
	    return $minorA <=> $minorB; # sort by spectral subclass
	}
    } # end $majorA == $majorB
    elsif ($majorA eq "O")
    {
	return $good;
    } # end $majorA == O-star
    elsif ($majorA eq "B")
    {
	if ($majorB =~ m/O/)
	{
	    return $reverse;
	}
	else
	{
	    return $good;
	}
    } # end $majorA == B-star
    elsif ($majorA eq "A")
    {
	if ($majorB =~ m/O|B/)
	{
	    return $reverse;
	}
	else
	{
	    return $good;
	}
    } # end $majorA == A-star
    elsif ($majorA eq "F")
    {
	if ($majorB =~ m/O|B|A/)
	{
	    return $reverse;
	}
	else
	{
	    return $good;
	}
    } # end $majorA == F-star
    elsif ($majorA eq "G")
    {
	if ($majorB =~ m/O|B|A|F/)
	{
	    return $reverse;
	}
	else
	{
	    return $good;
	}
    } # end $majorA == G-star
    elsif ($majorA eq "K")
    {
	if ($majorB =~ m/O|B|A|F|G/)
	{
	    return $reverse;
	}
	else
	{
	    return $good;
	}
    } # end $majorA == K-star
    else # $majorA == M-star
    {
	return $reverse;
    }

} ## end bySpectralType  ################################################



return 1;
