#! @PERL@
# -*- perl -*-
# @configure_input@

our $MODVERSION = '$Id: obscure.pl,v 1.6 2008/11/12 23:28:35 sgroom Exp $';

##########################################################################
# NASA Exoplanet Science Institute (NExScI) -- http://nexsci.caltech.edu #
#       Copyright (c) 1997-2008 California Institute of Technology       #
#                     U.S. Government Sponsorship                        #
# Under contract with the National Aeronautics and Space Administration  #
##########################################################################

use strict;
use warnings;

use FindBin qw($RealBin);
use lib "$RealBin/../lib/modules";		# location of getCal Perl modules

use coordinates;
use geometry;
use angles;
use vector;
use interferometers;  ## Interferometers DB module
use miscArgs;
use gcConfig;
use Getopt::Long;



## obscure -- compute telescope/telescope obscurations along star tracks

## Part of getCal suite -- A. Boden 12/00


## obscure is written at the request of Mark Colavita for Keck
## Interferometer -- to identify times when objects are obscured to
## the KI sids by the Keck domes.

## I'm doing this in the context of the getCal timing annotations, so
## this may eventually become of the getCal tool suite.  Or maybe not.

## Input is a schedule complete with getCal timing annotations, coming
## in on stdin.  Output is an addition to the timing annotations with
## obscuration times.  

## Design will be to be as stateless as possible -- react entirely to
## input stream.


## Open input stream (debugging only)

##open (TEST,"test1.cat");  ## used for testing
##open (TEST,"HD_9939.getCal");  ## used for testing
##open (TEST,"HD_24546.getCal");  ## used for testing
##open (TEST,"64_Psc.getCal");  ## used for testing


# handle the standard command line options
my $cfg = gcConfig::config();
handleCommonOpts(0);

print STDERR "DEBUG: obscure: debugging enabled\n" if gcConfig::debug();

my @bl = undef;
my @delay = undef;
my $name = undef;

line: while (my $line = <STDIN>)
{
    print $line;

    ## Timing information (annotations) looks like:
    # HDC134323 15 08 53.512  +13 14 06.872  transits at 02:52:11 UT, zenAng 20.1 deg
    # HDC134323 is within zenith angle range from 00:46:11 to 04:58:11 UT
    # HDC134323 is within delay range from 22:39:35 to 11:20:16 UT for PTI_NS
    # HDC134323 is within delay range from 00:25:34 to 04:10:34 UT for PTI_NW

    if (!($line =~ m/^\# /))
    {
	next line;
    }

    if ($line =~ m/^\# Using .* Baseline/)
    {
	my @tmp = split /\s+/, $line;
	print STDERR "DEBUG: obscure: found baseline $tmp[1]\n" if (gcConfig::debug());
	push @bl, $tmp[1];
	next line;
    }

    if ($line =~ m/transits at/)
    {
	# Extract name, coordinates, transit time
	my @tmp = split /\s+/, $line;
	my $name = $tmp[1];
	my $raDeg = RAdeg(join " ", $tmp[2],$tmp[3],$tmp[4]);  ## We work in deg
	my $decDeg = DECdeg(join " ", $tmp[5],$tmp[6],$tmp[7]);
	my $tTime = RAdeg($tmp[10]);
	print STDERR "DEBUG: obscure: $name transit found\n" if (gcConfig::debug());

	@delay = ();	# delete any saved delay range info
	next line;
    }

    ## Start by looking for a comment transit marker
    if (($line =~ m/^\# /)
	&& ($line =~ m/transits at/))
    {

	## Loop over remaining lines in this timing annotation
	obj: while ($line = <STDIN>) 
	{
	    if ($line =~ m/^#>>/)
	    {
		print STDERR "DEBUG: obscure: skipping '#>>' line\n" if (gcConfig::debug());
		next obj;
	    }
	    if (!($line =~ m/^\# $name/))
	    {
		print STDERR "DEBUG: obscure: next object\n" if (gcConfig::debug());
		last obj;
	    }
	    print STDERR "DEBUG: obscure: checking for a delay range line\n" if (gcConfig::debug());
	    print $line;

	    ## Look for a delay limit line (signifies baseline)
	    if (($line =~ m/is within delay range from/)
		|| ($line =~ m/is always within delay range/))
	    {
		print STDERR "DEBUG: obscure: found delay range line\n" if (gcConfig::debug());
		@tmp = split /\s+/, $line;
		my ($leadDA,$trailDA,$bl);
	    
		if ($line =~ m/is within delay range from/) ## finite delay duration
		{
		    $leadDA = RAdeg($tmp[7]);
		    $trailDA = RAdeg($tmp[9]);
		    $bl = $tmp[12];

		    if (($leadDA > $tTime)  ## Check for modulo 24 hour wraps
			&& ($leadDA > $trailDA))
		    {
			$leadDA -= 360.0;            ## Working in degrees
		    }

		    if (($trailDA < $leadDA)         ## Check for modulo 24 hour wraps
			&& ($trailDA < $tTime))
		    {
			$trailDA += 360.0;           ## Working in degrees
		    }
		} ## end finite delay duration
		else  ## object always in delay range
		{
		    $leadDA = $tTime - 179.9;
		    $trailDA = $tTime + 179.9;
		    $bl = $tmp[8];
		} ## end always in delay line range

		## Get the telescope list for this baseline
		## Start by assuming KI-specific stuff.
		my $I = $interferometers::InterferometerList{KI};

		my @tScopes;
		foreach my $tl (@{ $$I->{Telescopes}})
		{
		    push @tScopes, $tl if ($bl =~ m/$tl/);
		}

		## See if there are any obscurations for the
		## telescopes composing this baseline
		my @potentialObscurations = ("K1","K2");
		my ($tel,$obs);
		foreach $tel (@tScopes)
		{
		    print STDERR "DEBUG: obscure: checking tel $tel\n" if (gcConfig::debug());
		    my $tLName = $tel . "_Location";
		    foreach $obs (@potentialObscurations)
		    {
			my $oLName = $obs . "_Location";
			my $oRName = $obs . "_Radius";
			if (!($tel =~ m/$obs/))
			{
			    my ($obscured,$startObs,$stopObs)
			       = &obscuration($raDeg,$decDeg,$$I->{Latitude},
					      ($leadDA - $tTime),
					      ($trailDA - $tTime),
					      $tel,$obs,@{ $$I->{$tLName} },
					      @{ $$I->{$oLName} },
					      $$I->{$oRName});
			    if ($obscured)
			    {
				my $t1 = RAhms($startObs + $tTime,0);
				$t1 =~ s/\s+/:/g;
				my $t2 = RAhms($stopObs + $tTime,0);
				$t2 =~ s/\s+/:/g;
				print "# $name is obscured for $bl ($tel) by $obs from $t1 to $t2 UT\n";
			    } ## end if obscured
			} ## end no self-obscuration test
		    } ## end potential obscuration loop

		} ## end telescope loop


	    } ## end normal delay line


	} ## end while processing clustered timing annotation lines
	print $line;
    } ## end process timing annotation
} ## end outer while loop


exit; ## end obscure




sub obscuration
{

    ## Extract arguments from the complicated call list
    my $ra = shift @_;
    my $dec = shift @_;
    my $lat = shift @_;

    my $leadHA = shift @_;
    my $trailHA = shift @_;

    my $tName = shift @_;
    my $obsName = shift @_;

    my (@tLoc,@obsLoc);
    $tLoc[0] = shift @_;
    $tLoc[1] = shift @_;
    $tLoc[2] = shift @_;

    $obsLoc[0] = shift @_;
    $obsLoc[1] = shift @_;
    $obsLoc[2] = shift @_;

    my $obsRadius = shift @_;
    my $minDCA = 1e10;


    ## Iterate from $leadHA to $trailHA

    my $ha;
    #my DEG_TO_RAD = 3.14159 / 180.0;	# get this from angles.pm now - SG 5/06
    my $obscured = 0;
    my ($startObscuration,$stopObscuration);
    for ($ha = $leadHA; $ha <= $trailHA; $ha += 2.0) ## 2 deg steps
    {
	## Evaluate instantaneous direction of object
	my @uStar = unitStar($ha * DEG_TO_RAD,
			     $dec * DEG_TO_RAD,
			     $lat * DEG_TO_RAD);

	## Time for some analytic geometry -- compute the point of
	## closest approach between the line of sight to the object
	## and the potential obscuring telescope
	my ($range,$DCA) = dca(@uStar,@tLoc,@obsLoc);
	## print "$ha\t$range\t$DCA\n";
	$minDCA = $DCA if (($range > 0) && ($DCA < $minDCA));

	## Check that the dca range is positive (between telescope and
	## star) and DCA is less that the obscuration radius
	if (($range > 0) && ($DCA < $obsRadius))
	{
	    # We've got an obscuration
	    if (!$obscured)
	    {
		$obscured = 1;
		$startObscuration = $ha;
	    } ## end start obscuration logic
	} ## end obscuration check
	elsif ($obscured)
	{
	    $obscured = 0;
	    $stopObscuration = $ha;
	} ## end terminating obscuration

    } ## end $ha loop

    if ($startObscuration && $stopObscuration)
    {
	return (1,$startObscuration,$stopObscuration);
    }
    else
    {
	return (0,$minDCA);
    }

} ## end sub obscuration




sub dca
{

    ## dca -- distance of closest approach
    ## dca evaluates the distance of closest approach between the line
    ## of sight to an astronomical object and the location of a
    ## potential obscuration.

    ## Extract call arguments

    my (@uStar,@tLoc,@obsLoc);

    $uStar[0] = shift @_;
    $uStar[1] = shift @_;
    $uStar[2] = shift @_;

    $tLoc[0] = shift @_;
    $tLoc[1] = shift @_;
    $tLoc[2] = shift @_;

    ## Map coordinates to center on @tLoc (telescope location) during
    ## extraction (@tLoc becomes a throw-away)
    $obsLoc[0] = (shift @_) - $tLoc[0];
    $obsLoc[1] = (shift @_) - $tLoc[1];
    $obsLoc[2] = (shift @_) - $tLoc[2];

    ## Compute range to the pca -- just a dot product (vector.pl)
    my $Range = dot(@uStar,@obsLoc);

    ## Now compute the dca
    ## First the 3-space point of closest approach minus the obsLoc
    my @sVec;
    $sVec[0] = ($Range * $uStar[0]) - $obsLoc[0];
    $sVec[1] = ($Range * $uStar[1]) - $obsLoc[1];
    $sVec[2] = ($Range * $uStar[2]) - $obsLoc[2];

    ## The dca is just the modulus of @s
    my $dcArg = &norm(@sVec);


    ## return the range and dca to caller
    return ($Range,$dcArg);
    
} ## end dca
