#! @PERL@
# -*- perl -*-
# @configure_input@

our $MODVERSION = '$Id: simbadInteractiveQuery.pl,v 1.23 2008/11/12 23:28:40 sgroom Exp $';

##########################################################################
# NASA Exoplanet Science Institute (NExScI) -- http://nexsci.caltech.edu #
#       Copyright (c) 1997-2008 California Institute of Technology       #
#                     U.S. Government Sponsorship                        #
# Under contract with the National Aeronautics and Space Administration  #
##########################################################################


use strict;
use warnings;
use FindBin qw($RealBin);
use lib "$RealBin/../lib/modules";		# location of getCal Perl modules
use gcConfig;
use miscString;
use Getopt::Long;
use Astro::SIMBAD::Query;
use astroCatalogue;
use gcSimbad3;
use miscArgs;
use colors;

my $cfg = gcConfig::config();
my $exitcode = TARGET_SUCCESS;

# boolean options
my $genUrl = 0; # print URL and exit
my $allData = 0; # dump full report
my $htmlFormat = 0; # dump full report in HTML (default is plain text)
my $commonNames = 0;
my $identifiers = 0;
my $getCoord = 0;
my $useStdin = 0;
my $listMode = 0;
my $useBrowser = 0;

# string options
my @filenames; # read search targets from file; empty for stdin
my ($hdNumber, $hipNumber, $saoNumber, $fk5Number);
my $kMag;
my $timeout = 30;
my $lineWidth = gcSimbad3::SIMBAD_ASCII_LINE_WIDTH;

sub parse_options {
    handleCommonOpts(0, "simbadShortHelp.txt", "simbadLongHelp.txt");
    my $result = GetOptions("allData|meas" => \$allData,
			    "browser" => \$useBrowser,
			    "commonNames" => \$commonNames,
			    "coord" => \$getCoord,
			    "filename=s" => \@filenames,
			    "FK5" => \$fk5Number,
			    "HD" => \$hdNumber,
			    "HIP" => \$hipNumber,
			    "html" => \$htmlFormat,
			    "identifiers|pseudonyms" => \$identifiers,
			    "Kmag" => \$kMag,
			    "lineWidth=i" => \$lineWidth,
			    "list" => \$listMode,
			    "SAO" => \$saoNumber,
			    "stdin" => \$useStdin,
			    "timeout=i" => \$timeout,
			    "url" => \$genUrl,
			    );
}

sub make_simbad_query {
    return prepSimbadQuery(Target => normalIdent(shift),
			   Error => ($listMode ? 1 : 0));
}

sub pop_browser {
    my $browser = $cfg->browser();
    if (! $browser)
    {
	print STDERR "simbadInteractiveQuery: no browser available, set GC_BROWSER to location of browser\n";
	return;
    }

    foreach my $target (@_) {
	my $query = make_simbad_query($target);
	my $url = $query->queryurl();
	my $cmd = $browser . " \"$url\" &";
	print STDERR "DEBUG: simbadInteractiveQuery: running command \"$cmd\"\n" if (gcConfig::debug());
	system $cmd;
    }
}

sub print_urls {
    foreach my $target (@_) {
	my $query = make_simbad_query($target);
	print $query->queryurl() . "\n";
    }
}

sub hit_simbad {
    foreach my $target (@_) {
	print STDERR "DEBUG: simbadInteractiveQuery: hit_simbad: target $target \n" if (gcConfig::debug());
	my $query = make_simbad_query($target);
	my $result;
	eval { $result = $query->querydb(); };
	print STDERR "DEBUG: simbadInteractiveQuery: query complete\n" if gcConfig::debug();
	my $ok = checkSimbadResult($result);
	print STDERR "DEBUG: simbadInteractiveQuery: query complete, ok=$ok\n" if gcConfig::debug();
	if ((!$ok) || (!$result->objects())) {
	    # The error could also be caused by Simbad being unavailable,
	    # but presently there's no way to distinguish Simbad/network
	    # problems from a lookup that didn't find anything. SG 1/2006
	    print "Designation $target is not present or uniquely identified in the Simbad database\n";
	    $exitcode = TARGET_NOT_FOUND;
	}
	else
	{
	    foreach my $object ($result->objects()) {
		if (&needTargetNameChange($target, $object->name())) {
		    print STDERR "DEBUG: simbadInteractiveQuery: Identifier change: $target -> " . $object->name() . "\n" if (gcConfig::debug());
		    $exitcode = TARGET_MISNAMED;		    
		}
		print_result($object, $target);
		if ($htmlFormat) {
		    print "\n";
		    print $query->{BUFFER};
		}
		elsif ($allData) {
		    print "\n";
		    print html2txt($query->{BUFFER}, $lineWidth);
		}
		print "\n";
	    }
	}
    }
}

sub print_result {
    my $object = shift;
    my $target = shift;
    printf "# Simbad Search %s: %s", $target, trimWhite($object->name());
    printf " -- %s  %s", trimWhite($object->long()), trimWhite($object->spec());
    if (isNumeric($object->vmag())) {
	if ($object->vmag()) {
	    printf " V=%g", $object->vmag();
	}
	if ($kMag) {
	    my $vmk = VminusK($object->spec());
	    if (defined $vmk)
	    {
		my $km = $object->vmag() - $vmk;
		printf " K=%g", $km;
	    }
	}
    }
    if ($commonNames) {
	for my $id (grep m/\*|NAME/, @{$object->ident()}) {
	    $id =~ s/^\s+|\s+$//g;
	    print " $id";
	}
    }

    if ($getCoord) {
	if (gcConfig::debug())
	{
	    my $frame = $object->frame();
	    print STDERR "DEBUG: simbadInteractiveQuery: print_result: frame " . $frame . "\n" if (gcConfig::debug());
	}
	printf "\ncoord %s", join(" ", $object->ra(), $object->dec());
	# display proper motion in arcsec/year
	my $pm = @{$object->pm}[1];
	# XXX THE DIVISION BELOW IS INCORRECT, NEEDS TO ACCOUNT FOR COS(DEC) - SG 12/18/06
	if ($pm) {
	    printf "  %4.3f %4.3f", @{$object->pm}[0]/1000, $pm/1000;
	}
    }

    my @idents;
    if ($identifiers) {
	@idents = @{$object->ident()};
    } else {
	my @catalogs;
	if ($hdNumber) { push @catalogs, "HD" }
	if ($hipNumber) { push @catalogs, "HIP" }
	if ($saoNumber) { push @catalogs, "SAO" }
	if ($fk5Number) { push @catalogs, "FK5" }
	if (@catalogs) {
	    @idents = identsByCatalogue(join('|', @catalogs), @{$object->ident()});
	}
    }
    foreach my $id (@idents) {
	print "\n$id ";
    }
}

# Main routine
parse_options();
my @targets = getArgs($useStdin, \@filenames);
if (! scalar @targets)
{
    helpFile(1,"simbadShortHelp.txt");
}

if ($useBrowser) {
    pop_browser(@targets);
} elsif ($genUrl) {
    print_urls(@targets);
} else {
    hit_simbad(@targets);
}
exit($exitcode);
