package dstBounds;
my $MODVERSION = '$Id: dstBounds.pm,v 1.7 2008/11/12 23:28:47 sgroom Exp $';

use strict;
use warnings;
require 5.000;

##########################################################################
# NASA Exoplanet Science Institute (NExScI) -- http://nexsci.caltech.edu #
#       Copyright (c) 1997-2008 California Institute of Technology       #
#                     U.S. Government Sponsorship                        #
# Under contract with the National Aeronautics and Space Administration  #
##########################################################################

# 3/07 SLG: updated with changes for US Energy Policy Act of 2005, effective 2007

use Time::JulianDay;

use Exporter();
my @ISA = qw(Exporter);
my @EXPORT = qw(dstBounds isDST);


## Package dstBounds -- A.B. after skycalc/John Thorstensen (thanks!)

## This package implements logic for Daylight Savings Time using a
## number of different local conventions.  These are keyed by the
## $useDST flag, and are as follows:

## $useDST == 0  -- Use not DST -- always standard time!!!

## $useDST == 1  -- Use USA convention:	(Updated 3/2007)
##                   Enter DST 2st Sun in March
##                   Exit DST 1st Sun in November

## $useDST == 2  -- Use Spanish convention:
##                   Enter DST last Sun in March 
##                   Exit DST last Sun in September

## $useDST == -1 -- Use Chilean convention (Southern Hemisphere):
##                   Enter DST 2nd Sun in October
##                   Exit DST 2nd Sun in March

## $useDST == -2 -- Use Australian convention (Southern Hemisphere):
##                   Enter DST last Sun in October
##                   Exit DST 1st Sun in March


## Note that we are using negative signs to key Southern Hemisphere
## useage where we enter DST late in the year, and then leave DST
## early in the subsequent year.




sub dstBounds
{
    my ($yr,$useDST) = @_;

    ## finds jd's at which daylight savings time begins and ends.  The
    ## parameter $useDST allows for a number of conventions, namely:
    ##  0 = don't use it at all (standard time all the time)
    ##  1 = use USA convention (1st Sun in April to last Sun in Oct after 1986;
    ##       last Sun in April before)
    ##  2 = use Spanish convention (for Canary Islands)
    ## -1 = use Chilean convention (CTIO).
    ## -2 = Australian convention (for AAT).
    ## Negative numbers denote sites in the southern hemisphere, where
    ## jdb and jde are beginning and end of STANDARD time for the
    ## year.  It's assumed that the time changes at 2AM local time; so
    ## when clock is set ahead, time jumps suddenly from 2 to 3, and
    ## when time is set back, the hour from 1 to 2 AM local time is
    ## repeated.  This could be changed in code if need be.

    if (($useDST == 1) || ($useDST == 0))
    {
	## USA Convention, and including no DST to be defensive 
	## Note that this ignores various wrinkles such as the
	## brief Nixon administration flirtation with year-round DST,
	## the extended DST of WW II, and so on. 

	my $trialHour = 2;	# changes occur at 2am
	my $trialMin = 0;
	my $trialSec = 0;

	my $jdEnterDST;
	my $jdExitDST;

	## Find DST start date
	if ($yr >= 2007)	## For year >= 2007 (Energy Policy Act of 2005)
	{
	    $jdEnterDST = &findJD(2, 0, 3, $yr);		# second Sunday in March
	}
	elsif ($yr >= 1986)	## For year 1986-2006
	{
	    $jdEnterDST = &findJD(1, 0, 4, $yr);		# first Sunday in April
	}
	else  ## Otherwise pre-1986
	{
	    $jdEnterDST = &findJD(-1, 0, 4, $yr);	# last Sunday in April
	}

	# Find DST End Date
	if ($yr >= 2007)	## For year >= 2007 (Energy Policy Act of 2005)
	{
	    $jdExitDST = &findJD(1, 0, 11, $yr);		# first Sunday in November
	}
	else  ## Otherwise pre-2007
	{
	    $jdExitDST = &findJD(-1, 0, 10, $yr);	# last Sunday in October
	}

	return ($jdEnterDST,$jdExitDST);
    } ## end $useDST == 0 || $useDST == 1

    elsif ($useDST == 2)  ## Spanish/Canary Islands
    {
	my $jdEnterDST = &findJD(-1, 0, 3, $yr);	# last Sunday in March
	my $jdExitDST = &findJD(-1, 0, 9, $yr);	# last Sunday in September
	return ($jdEnterDST,$jdExitDST);
    }  ## end $useDST == 2

    elsif ($useDST == -1)  ## Chilean for CTIO/Paranal
    {
	# southern hemisphere!
	my $jdEnterDST = &findJD(2, 0, 10, $yr);	# second Sunday in October
	my $jdExitDST = &findJD(2, 0, 3, $yr);	# second Sunday in March
	return ($jdEnterDST,$jdExitDST);
    }  ## end $useDST == -1

    elsif ($useDST == -2)  ## Australian for SUSI
    {
	my $jdEnterDST = &findJD(-1, 0, 10, $yr);	# last Sunday in October
	my $jdExitDST = &findJD(1, 0, 3, $yr);	# first Sunday in March
	return ($jdEnterDST,$jdExitDST);
    }  ## end $useDST == -2
}  ## end dstBounds


## isDST computes whether the input date is using DST
sub isDST
{
    my ($year,$month,$day,$useDST) = @_;

    ## If $useDST is zero we short-circut
    if ($useDST == 0)
    {
	return 0;
    } ## end always use standard time

    ## Compute the JD dst bounds for this DST convention
    my ($jdEnterDST,$jdExitDST) = &dstBounds($year,$useDST);

    ## Compute the current JD
    my $jd = &julian_day($year,$month,$day);

    ## Evaluate whether current JD is DST
    if ($useDST > 0)  ## Northern Hemisphere ($jdEnterDST < $jdExitDST)
    {
	## Here we simply need to check that the current JD is in the
	## bounds prescribed by the bound values
	if (($jd >= $jdEnterDST) && ($jd < $jdExitDST))
	{
	    return 1; ## We are in DST
	}
	else
	{
	    return 0; ## We are in standard time
	}
    } ## end $useDST > 0
    else  ## $useDST < 0   ($jdEnterDST > $jdExitDST)
    {
	## Here to be in DST we must either be greater than entry time
	## OR less than exit time

	if (($jd >= $jdEnterDST) || ($jd < $jdExitDST))
	{
	    return 1; ## We are in DST
	}
	else
	{
	    return 0; ## We are in standard time
	}
    }
}  ## end isDST

# find date (JD) corresponding to given criteria
# 	example "SECOND SUNDAY in MARCH for 2007"...
sub findJD
{
    my $which = shift;	# which one to find: 1=first, 2=second, and so forth. -1=last)
    my $dow = shift;	# day of week (0=sun, 1=mon, ... 6=sat)
    my $mo = shift;	# month of year(1=jan, 2=feb, ...)
    my $yr = shift;	# year

    my $trialDay;

    if ($which == -1)
    {
	# finding the last in a month: use number of days in that month
	# start and end and work backwards
	$trialDay = &nDays($yr,$mo);	# last day of given month and year
	while(&day_of_week(julian_day($yr,$mo,$trialDay)) != $dow)
	{
	    $trialDay--;
	}
    }
    else
    {
	# finding the nth in a month: start at earliest possible day in that month
	# step forwards until we hit desired day of week
	$trialDay = (($which-1) * 7) + 1; # first=>1, second=>8, third=>15, ...
	while(&day_of_week(julian_day($yr,$mo,$trialDay)) != $dow)
	{
	    $trialDay++;
	}
    }

    # julian_day returns days starting at noon, so we need to
    # back that up to days starting at midnight.
    my $jd = julian_day($yr,$mo,$trialDay) - 0.5;
    return $jd;
}

# Return number of days in given month and year.
# year is necessary to deal with pesky February.
sub nDays
{
    my $yr = shift;	# Gregorian year (e.g. 2007)
    my $mo = shift;	# month in year (1=jan 2=feb, 3=mar, ...)

    # days in each month
    my @daysInMonth = (31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);
    if ($mo != 2)
    {
	# not February: return number from list above
	return $daysInMonth[$mo-1];
    }
    else
    {
	# February: deal with leap years to decide
	if (($yr % 400) == 0)
	{
	    return 29;	# leap year
	}
	elsif (($yr % 100) == 0)
	{
	    return 28;	# not leap year
	}
	elsif (($yr % 4) == 0)
	{
	    return 29;	# leap year
	}
	else
	{
	    return 28;	# not leap year
	}
    }
}
