package pointing;
my $MODVERSION = '$Id: pointing.pm,v 1.10 2008/11/12 23:28:47 sgroom Exp $';

##########################################################################
# NASA Exoplanet Science Institute (NExScI) -- http://nexsci.caltech.edu #
#       Copyright (c) 1997-2008 California Institute of Technology       #
#                     U.S. Government Sponsorship                        #
# Under contract with the National Aeronautics and Space Administration  #
##########################################################################

#########################################################################
## pointing.pm
##
## This module contains routines for computations involving
## telescope/baseline pointing limits, and when a given target will be
## within pointing range.
## 
## Some of the routines here are rough and the calculations aren't
## always pretty or extremely precise, but we gotta start somewhere.
##
## Steve Groom, June 2006
#########################################################################

use strict;
use warnings;

require 5.000;

use Exporter();

our @ISA = qw(Exporter);
our @EXPORT = qw(hasLimits validateLimits computePointingLimits);

use gcConfig;
use angles;
use geometry;
use timingMisc;
use novasConstants;	# SolarToSidereal

#########################################################################
## hasLimits - return number of limit segments defined for baseline, 0 if none
#########################################################################
sub hasLimits
{
## declarations hasLimits  #####################################
    my $bPtr = shift @_;	## baseline object ptr
    my $bl = $$bPtr; 		## baseline object
## begin hasLimits  ############################################
    my $blName = $bl->{name};
    my $limits = $bl->{limits};
    if (!$limits)
    {
	print STDERR "DEBUG: hasLimits: no pointing limits defined for baseline $blName\n" if gcConfig::debug();
	return 0;	# no limits defined
    }
    my $nseg = scalar( @{ $limits });
    if (! $nseg)
    {
	# no limits defined, so nothing to check against
	print STDERR "DEBUG: hasLimits: no pointing limits defined for baseline $blName\n" if gcConfig::debug();
	return 0;	# no limits defined
    }
    return $nseg;
}

#########################################################################
## validateLimits - make sure telescope pointing range is well formed
## returns 0 if there is a problem with the limits definition for the
## given baseline. returns 1 if everything is OK.
#########################################################################
sub validateLimits
{
## declarations validateLimits  #####################################
    my $bPtr = shift @_;	## baseline object ptr
    my $bl = $$bPtr; 		## baseline object
## begin validateLimits  ############################################

    # See if this baseline has defined pointing limits, and if so,
    # that they are well-formed. 

    # Each map of limits is a map of az ranges, and the valid elevation
    # ranges for each az range. These maps are defined in interferometers.pm,
    # within the hash representing each location.
    # If no limits are defined for a baseline, then no pointing limit information
    # will be produced by computePointingLimits
    #
    # Each map lists ranges of azimuths, and the elevation limits
    # that apply for each azimuth range. The map is given as a list of
    # limit segments. Each segment is given as a beginning and ending azimuth,
    # and the min and max elevation within that azimuth range.
    # Here's an example:
    #	KI_K1K2_Limits => [
    #	    [   0.0,     5.3,   18.0,   88.9],  # az_begin, az_end, el_min, el_max (deg)
    #	    [   5.3,   146.2,   33.3,   88.9],
    #	    [ 146.2,   185.3,   18.0,   88.9],
    #	    [ 185.3,   332.8,   36.8,   88.9],
    #	    [ 332.8,   360.0,   18.0,   88.9]
    #	]
    # 
    # The validation checks performed here are:
    #   - pointing limits structure defined for the named baseline 
    #   - must have at least one limits segment
    #   - the first segment must start at az=0, and the last must end at 360
    #   - each segment must start where the previous one ended
    #   - every limits segment must contain four values

    my $blName = $bl->{name};
    my $limits = $bl->{limits};
    my $nseg = &hasLimits($bPtr);
    if (! $nseg)
    {
	print STDERR "DEBUG: validateLimits: no pointing limits defined for baseline $blName\n" if gcConfig::debug();
	return 1;	# ok, no limits defined
    }

    # do a quick check of defined limit segment
    # note the lowest of the mins and the highest of the max's 
    my $minaz = 360.0;
    my $maxaz = 0.0;
    my $lastaz = 0.0;	# must start at 0.0 and end at 360.0
    for (my $seg = 0; $seg < $nseg; $seg++)
    {
	my $nval = scalar(@{ $limits->[$seg] });
	if ($nval != 4)	# each entry must have 4 values: az1, az2, elMin, elMax
	{
	    print STDERR "validateLimits: malformed pointing limit segment for baseline $blName: wrong number of values in segment ($nval != 4)\n";
	    return 0;
	}
	
	my $segAzMin = $limits->[$seg][0];
	my $segAzMax = $limits->[$seg][1];
	my $segElMin = $limits->[$seg][2];
	my $segElMax = $limits->[$seg][3];
	if ($segAzMin >= $segAzMax)
	{
	    print STDERR "validateLimits: malformed pointing limit for baseline $blName: az out of order ($segAzMin >= $segAzMax)\n";
	    return 0;
	}
	if ($segAzMin != $lastaz)
	{
	    print STDERR "validateLimits: malformed pointing limit for baseline $blName: az not contiguous ($segAzMin != $lastaz)\n";
	    return 0;
	}
	if (($segAzMax < 0.0) || ($segAzMax > 360.0))
	{
	    print STDERR "validateLimits: malformed pointing limit for baseline $blName: az $segAzMax out of range 0.0-360.0\n";
	    return 0;
	}
	$lastaz = $segAzMax;
    }
    if ($lastaz != 360.0)	# make sure we end up at 360.0
    {
	print STDERR "validateLimits: malformed pointing limit for baseline $blName: az range not complete ($lastaz != 360.0)\n";
	return 0;
    }

    return 1;	# everything is okey dokey
} ## end validateLimits  ############################################


#########################################################################
## computePointingLimits -- compute telescope pointing limits for declination
#########################################################################
sub computePointingLimits
{
## declarations computePointingLimits  #####################################
    my $name = shift @_;  ## target designation
    my $dec = shift @_;  ## target declination (deg)
    my $lat = shift @_;  ## local latitude (deg)
    my $utTransit = shift @_; ## UT transit time (units?)
    my $bPtr = shift @_; ## baseline object ptr
    my $bl = $$bPtr; ## baseline object
    my $blName = $bl->{name};
    my $verbose = shift @_; ## verbose output

## begin computePointingLimits  ############################################

    my $dumpPoints = 0;	#XXX  debugging switch to control dumping of points to a file
    my @pList;	# list of pointing intervals to be returned

    # Use the baseline pointing limits to determine when the target will be within
    # pointing range.

    # See if this baseline has defined pointing limits. 
    my $limits = $bl->{limits};
    my $nseg = &hasLimits($bPtr);
    if (! $nseg)
    {
	# no limits defined, so nothing to check against
	print STDERR "DEBUG: computePointingLimits: no pointing limits defined for baseline $blName\n" if gcConfig::debug();
	return @pList;	# return the empty pointing interval list
    }

    # debugging: keep an array of all the [az,el] points we test and the accessibility
    # at that point, so they can be dumped to a file for examination
    my @testPoints = ();	# array of [az,el,state] for each point we test

    # Iterate over HA range, watching for transitions in and out of
    # telescope pointing limits
    my $initialized = 0;	# initialization flag, set to 1 after first position
    my $lastPFlag;	# pointing flag at last position
    my $lastHA;		# last checked HA
    my $lastAz;		# last checked azimuth
    my $lastEl;		# last checked elevation

    # check for accessibility at transit to make sure we flag problems observing around zenith
    my $transitha = 0.0;	# transit is always hour angle=0.0
    my $transitaz = azAngle($transitha,$dec,$lat);
    my $transitza = zenithAngle($transitha,$dec,$lat);
    my $transitel = 90.0 - $transitza;
    my $zFlag = isAccessible($transitaz, $transitel, $limits);
    push @testPoints, [$transitha, $dec, $transitaz, $transitel, $zFlag] if $dumpPoints;	# remember this point
    if (!$zFlag)
    {
	printf "# %s is out of pointing range at transit for %s\n",$name,$blName;
    }

    # Determine the HA range to test. Be careful with objects that
    # never rise or never set.
    my $haExtreme;	## HA extreme
    if (abs($dec - $lat) >= 90.0)
    {
	# object never rises above the ideal (0deg) horizon
	#printf "# %s is never within pointing range for %s\n",$name,$blName;
	# never in range, return empty pointing list
	return @pList;	# return the empty pointing interval list
    }
    elsif (abs($dec + $lat) >= 90.0)
    {
	# object never sets below the ideal horizon, check HA -180 to +180
	$haExtreme = 180.0;	# circumpolar so we check full HA range
    }
    else
    {
	$haExtreme = zaLimit($dec,$lat,90.0,1); # compute HA of object at ideal horizon (za=90)
    }

    # attempt to dynamically adjust our step size in HA to avoid stepping through
    # small intervals of inaccessibility, since we're stepping in HA but our
    # limits are expressed in az/el, and az changes more rapidly with HA as
    # elevation increases, becoming extreme near zenith.
    # (which is why az/el telescope mounts have trouble tracking through/near
    # zenith in the first place)
    my $haMaxStep = 1.0;	# largest HA step size (deg) 1deg = ~4 minutes time
    my $haMinStep = 0.004;	# smallest HA step size (deg) = ~1 second time
    my $haStep = $haMaxStep;	# HA step size - initially set to max, updated within loop
    my $haStart = undef;	# ha where a pointing interval starts
    my $transitionFound = 0;	# flag to note at least one transition found
    my $loopCount = 0;
    hastep: for (my $ha = -$haExtreme; $ha <= +$haExtreme; $ha += $haStep)
    {
	my $az = azAngle($ha,$dec,$lat);
	my $za = zenithAngle($ha,$dec,$lat);
	my $el = 90.0 - $za;
	$loopCount++;

	# Adjust HA step size to avoid stepping over and missing pointing obstructions
	# when at high elevations.  This very crude attempt doesn't capture the full effect,
	# (azimuth changes very rapidly when object transits near zenith),
	# but it gives roughly the desired behavior (use smaller HA steps closer
	# to zenith) -- SG 11/06
	my $cosEl = cos($el * DEG_TO_RAD);	# shrink step sizes as elevation increases - crude but effective
	$haStep = $haMaxStep * $cosEl;
	$haStep = $haMinStep if ($haStep < $haMinStep);	# apply mininum

	# DEBUGGING:
	if (gcConfig::debug() > 2)
	{
	    # print some debugging info to help assess the step size scaling
	    my $dAz=0;
	    my $dEl=0;
	    my $dAng=0;
	    if ($initialized)
	    {
		$dAz = abs($az - $lastAz);
		$dEl = abs($el - $lastEl);
		$dAng = sqrt($dAz*$dAz + $dEl*$dEl);
	    }
	    printf STDERR "DEBUG: computePointingLimits: testing ha=%7.2f dec=%7.2f lat=%7.2f az=%7.2f za=%7.2f el=%7.2f step=%6.4f dAz=%6.3f dEl=%6.3f dAng=%6.3f\n",
		$ha, $dec, $lat, $az, $za, $el, $haStep, $dAz, $dEl, $dAng;
	}

	# test az, el against telescope pointing limits
	my $pFlag = isAccessible($az, $el, $limits);
	push @testPoints, [$ha, $dec, $az, $el, $pFlag] if $dumpPoints;	# remember this point
	if (!$initialized)
	{
	    # first time through, just record state.
	    $lastPFlag = $pFlag;
	    $lastHA = $ha;
	    $lastAz = $az;
	    $lastEl = $el;
	    $initialized = 1;	# initialization done
	    if ($pFlag)
	    {
		printf "#>> %s pointing range START at ha=%8.3f, az=%8.3f, el=%8.3f, %s UT (initial) for %s\n",
		    $name,$ha,$az,$el, &timeString($utTransit + &siderealToSolar($ha),0),$blName if ($verbose>1);
		$haStart = $ha;		# note start of pointing range interval
	    }
	    next hastep;
	}

	# check for transitions
	if ($lastPFlag != $pFlag)
	{
	    $transitionFound = 1;	# there is a change in pointing accessibility
	    printf "#>> %s pointing transition between (ha=%8.3f az=%8.3f el=%8.3f time=%s UT) and (ha=%8.3f az=%8.3f el=%8.3f time=%s UT) for %s\n",
		    $name,$lastHA,$lastAz,$lastEl,&timeString($utTransit + $lastHA,0),
		    $ha,$az,$el,&timeString($utTransit + &siderealToSolar($ha),0),$blName if ($verbose > 1);
	    # Narrow the transition time by testing at intermediate hour angles.
	    # Do a binary search until we know the transition time to within
	    # one minute. This is much simpler than using the telescope limits to compute
	    # a more precise answer.
	    my $haTol = 0.5*(15.0 * novasConstants::SolarToSidereal/60.0);  # ~30 seconds time expressed as HA
	    my $ha1 = $lastHA;	# one side of the transition
	    my $ha2 = $ha;	# the other side of the transition
	    my $haEst = ($ha1 + $ha2)/2.0;	# split the difference
	    my $azEst = azAngle($haEst,$dec,$lat);
	    my $zaEst = zenithAngle($haEst,$dec,$lat);
	    my $elEst = 90.0 - $zaEst;
	    while (abs($ha2 - $ha1) > $haTol)
	    {
		printf "#>> %s testing pointing transition at ha=%8.3f az=%8.3f el=%8.3f time=%s UT for %s\n",
		    $name,$haEst,$azEst,$elEst, &timeString($utTransit + &siderealToSolar($haEst),0),$blName if ($verbose>1);
		my $newFlag = isAccessible($azEst, $elEst, $limits);
		push @testPoints, [$haEst, $dec, $azEst, $elEst, $newFlag] if $dumpPoints;	# remember this point
		if ($newFlag == $pFlag)
		{
		    $ha2 = $haEst;	# try earlier
		}
		else
		{
		    $ha1 = $haEst;	# try later
		}
		# update ha/az/el for the new guess
		$haEst = ($ha1 + $ha2)/2.0;	# split the difference
		$azEst = azAngle($haEst,$dec,$lat);
		$zaEst = zenithAngle($haEst,$dec,$lat);
		$elEst = 90.0 - $zaEst;
	    }
	    printf "#>> %s pointing transition refined as ha=%8.3f az=%8.3f el=%8.3f time=%s UT for %s\n",
		$name,$haEst,$azEst,$elEst, &timeString($utTransit + &siderealToSolar($haEst),0),$blName if ($verbose>1);

	    if (!$lastPFlag && $pFlag)
	    {
		## Case 1: transition into pointing range
		printf "#>> %s pointing range START at ha=%8.3f, az=%8.3f, el=%8.3f, %s UT for %s\n",
		    $name,$haEst,$azEst,$elEst, &timeString($utTransit + &siderealToSolar($haEst),0),$blName if ($verbose>1);
		$haStart = $haEst;		# note start of pointing range interval
	    } # end transition into range
	    else
	    {
		## Case 2: transition out of pointing range
		printf "#>> %s pointing range END   at ha=%8.3f, az=%8.3f, el=%8.3f, %s UT for %s\n",
		    $name,$haEst,$azEst,$elEst, &timeString($utTransit + &siderealToSolar($haEst),0),$blName if ($verbose>1);

		# report pointing range interval
		my $t1 = &siderealToSolar($haStart);
		my $t2 = &siderealToSolar($haEst);
		#printf "# %s is within pointing range from %s to %s UT for %s\n",
		#    $name,
		#     &timeString($utTransit + $t1),
		#     &timeString($utTransit + $t2), $blName;
		push @pList, [$t1, $t2];	# add pointing interval to the return list
	    } # end transition out of range
	}
	# record state
	$lastPFlag = $pFlag;
	$lastHA = $ha;
	$lastAz = $az;
	$lastEl = $el;
    } # end of HA step loop
    print STDERR "DEBUG: computePointingLimits: HA step loop count $loopCount\n" if gcConfig::debug();

    if (!$initialized)
    {
	# object is never in range
	#printf "# %s is never within pointing range for %s\n",$name,$blName;
	# don't add anything to the pointing interval return list
    }
    elsif ($lastPFlag)
    {
	printf "#>> %s pointing range END   at ha=%8.3f, az=%8.3f, el=%8.3f, %s UT (final) for %s\n",
	    $name,$lastHA,$lastAz,$lastEl, &timeString($utTransit + &siderealToSolar($lastHA),0),$blName if ($verbose>1);

	if (!$transitionFound)
	{
	    # we never saw a transition, and finished in range, so object is
	    # always within pointing range.
	    #printf "# %s is always within pointing range for %s\n", $name, $blName;
	    push @pList, [-$haExtreme, $haExtreme];	# add pointing interval to the return list
	}
	else
	{
	    # report pointing range interval
	    my $t1 = &siderealToSolar($haStart);
	    my $t2 = &siderealToSolar($lastHA);
	    # printf "# %s is within pointing range from %s to %s UT for %s\n",
	    #	$name,
	    #	 &timeString($utTransit + $t1),
	    #	 &timeString($utTransit + $t2), $blName;
	    push @pList, [$t1, $t2];	# add pointing interval to the return list
	}
    }

    ### debugging: dump the list of points we tested to the plot file
    if ($dumpPoints)
    {
	my $fname = "/tmp/timingplot." . $name . "." . $blName . ".dat";
	print STDERR "DEBUG: computePointingLimits: opening plot file $fname\n" if gcConfig::debug();
	open PLOTFILE, ">", $fname or die "computePointingLimits: can't open plot file $fname\n";
	print PLOTFILE "# $name $blName\n";
	print PLOTFILE "# ha dec az el accessible\n";
	for (my $i = 0; $i < scalar(@testPoints); $i++)
	{
	    print PLOTFILE $testPoints[$i][0], " ", $testPoints[$i][1], " ",
			   $testPoints[$i][2], " ", $testPoints[$i][3], " ",
			   $testPoints[$i][4], "\n";
	}
	close PLOTFILE;
	print STDERR "DEBUG: computePointingLimits: wrote plot file $fname\n" if gcConfig::debug();
    }
    return @pList;	# return list of pointing intervals
} ## end computePointingLimits  ############################################

#########################################################################
## isAccessible -- check az/el against telescope pointing limits
## returns 1 if accessible, 0 if not accessible
#########################################################################
sub isAccessible
{
## declarations isAccessible  ############################################
    my $az = shift @_;		# target azimuth (degrees)
    my $el = shift @_;		# target elevation (degrees)
    my $limits = shift @_;	# telescope pointing limits

## begin isAccessible  ###################################################
    my $nseg = scalar( @{ $limits });
    if (!$nseg)
    {
	#print STDERR "DEBUG: isAccessible: no pointing limits defined\n" if gcConfig::debug();
	# when no limits are defined, treat it as always accessible
	return 1;
    }

    # Test each limit segment. Be careful that we may be testing
    # at the boundary between two segments, so we need to check the
    # elev limits of every segment covering this az.
    # There must be at least one segment that matches this az,
    # but no matching segment that is out of elev limits.
    my $result = 1;	# assume accessible until proven otherwise
    my $azFound = 0;	# az not found until proven otherwise
    my $seg = 0;
    while ($result && ($seg < $nseg))
    {
	# check each defined limit segment
	my $nval = scalar(@{ $limits->[$seg] });
	my $segAzMin = $limits->[$seg][0];
	my $segAzMax = $limits->[$seg][1];
	my $segElMin = $limits->[$seg][2];
	my $segElMax = $limits->[$seg][3];
	#print STDERR "DEBUG: isAccessible: checking az=$az, el=$el against seg $seg: az [$segAzMin, $segAzMax], el [$segElMin, $segElMax]\n" if gcConfig::debug();

	if (($az >= $segAzMin) && ($az <= $segAzMax))
	{
	    $azFound = 1;
	    if (($el < $segElMin) || ($el > $segElMax))
	    {
		# within az range for this segment but out of el range
		#print STDERR "DEBUG: isAccessible: az=$az, el=$el OUT of seg $seg: az [$segAzMin, $segAzMax], el [$segElMin, $segElMax]\n" if gcConfig::debug();
		$result = 0;	# elevation not accessible at this azimuth
	    }
	    else
	    {	
		# within both az and el limits
		# so far so good, but we can't be certain until all
		# limit segments are tested.
		#print STDERR "DEBUG: isAccessible: az=$az, el=$el IN seg $seg: az [$segAzMin, $segAzMax], el [$segElMin, $segElMax]\n" if gcConfig::debug();
	    }
	}
	$seg++;
    }
    if (!$azFound)
    {
	print STDERR "ERROR: isAccessible: azimuth=$az not covered by any limit segment, check telescope elevation limits\n";
	return 0; # not found in any defined limit segment
    }
    if ($result)
    {
	print STDERR "DEBUG: isAccessible: az=$az, el=$el, returning IN\n" if (gcConfig::debug() > 2);
    }
    else
    {
	print STDERR "DEBUG: isAccessible: az=$az, el=$el, returning OUT\n" if (gcConfig::debug() > 2);
    }
    return $result;
} ## end isAccessible  ###################################################
