#! @PERL@
# -*- perl -*-
# @configure_input@

our $MODVERSION = '$Id: obsCalendar.pl,v 1.26 2008/11/12 23:28:46 sgroom Exp $';
my $MODREV = '$Revision: 1.26 $';

##########################################################################
# NASA Exoplanet Science Institute (NExScI) -- http://nexsci.caltech.edu #
#       Copyright (c) 1997-2008 California Institute of Technology       #
#                     U.S. Government Sponsorship                        #
# Under contract with the National Aeronautics and Space Administration  #
##########################################################################


### obsCalendar -- perl script to compute target calendar accessibility

# description: obsCalendar takes a GUI-readable list of objects from stdin,
# and it computes calendar accessabilities for each target

# Generic use model

# obsCalendar < GUI_list [> GUI_list_annotated_with_calendar_parameters]
# or
# obsCalendar GUI_list [> GUI_list_annotated_with_calendar_parameters]

use strict;  ## strict rules for variables
use warnings;
use FindBin qw($RealBin);
use lib "$RealBin/../lib/modules";		# location of getCal Perl modules
use coordinates;
use novasConstants;
use siderealTime;
use findScript;
use gcConfig;
use POSIX;
use angles;
use timingMisc;
use internalLine;
use interferometers;
use geometry;
use yearDay;

## Using new module for Julian day calcs (cleaner interface)
use Time::JulianDay;
use Time::DaysInMonth;
use dstBounds;
use miscMath;
use miscArgs;
use sunMoon;

my $cfg = gcConfig::config();
my @reqFields = qw/ designation raDeg decDeg tagROLE / ;

## Set run defaults
my ($readStdin,$noPreamble,$noEcho,$replace,$hourAngle);
my (@filenames);
my $preamble = 1;
my $location = $cfg->defaultLocation();
my $twilightAngle = $cfg->twilightAngle();
my $TOL = 0.3;
my $correctionGain = 0.6;
my $MAX_ITER = 12;

# Step 1.  Get current time information (year)

#   0    1    2     3     4    5     6     7     8
my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = gmtime(time);
$mon += 1; ## These months start at zero
$yday += 1; ## Adding 1 to conform to "date -u +%j" (Jan 1 = 001)
$year += 1900;  ## We know how the dates come out of gmtime...

## Adjust date formats
if ($yday < 10)
{
    $yday = join '',"0", $yday;
}

if ($yday < 100)
{
    $yday = join '',"0", $yday;
}


## Save this until after we've determined whether we are writing a
## preamble
my $revStr = "$MODREV";
$revStr =~ s/\$//g;	# strip the $'s from the revision string
$revStr =~ s/Revision: //g;
$revStr =~ s/ //g;
my $preambleText = "# Observing calendar (obsCalendar v$revStr)\n";


## Command-line options
use Getopt::Long;
handleCommonOpts(0, "obsCalendar.txt");
my $result = GetOptions("stdin" => \$readStdin,
			"filename=s" => \@filenames,
			"noPreamble" => \$noPreamble,
			"year=i" => \$year,
			"noEcho" => \$noEcho,
			"replace" => \$replace,
			"twilightAngle=f" => \$twilightAngle,
			"hourAngle=f" => \$hourAngle,
			"location=s" => \$location);

## Decide whether to output timing preamble
if ($noPreamble)
{
    $preamble = !($noPreamble);  ## No preamble
}
else
{
    print "$preambleText";  ## preamble -- dump what we have and continue
}

## year mods
if ($year < 90)
{
    $year += 100;
}

if ($year < 1900)
{
    $year += 1900;
}

print "#  for timings in $year UTC" if ($preamble);
print " (leap year)" if (($preamble) && is_leap($year));
print "\n" if ($preamble);


my $daysInYear = 365;  ## The number of days in this year
$daysInYear++ if (is_leap($year));  ## Leap year correction...


## Set/verify the geometry for the calculations (location, baselines)
my ($localLatitude,$localLongitude,$interferometer) = &setLocation($location);

if ($hourAngle && $preamble)
{
    print "# Using $hourAngle hour transit offsets in accessibilty calculations\n"
}


# Step 2.  Loop through input GUI list

my @inputStream;
if (($readStdin)
    || ((!(scalar @filenames)) && (!(scalar @ARGV)))) ## read from stdin
{
    @inputStream = <STDIN>;
}
elsif (scalar @filenames)  ## otherwise read from designated filenames
{
    foreach my $file (@filenames)
    {
	open (READ,$file) or die "Unable to open $file\n";
	push @inputStream , <READ>;
	close (READ);
    } ## end foreach
}
elsif (scalar @ARGV)
{
    foreach my $file (@ARGV)  ## Read all the remaing filenames
    {
	open (READ,$file) or die "Unable to open $file\n";
	my @tmp = <READ>;  ## force list context
	push @inputStream, @tmp;
	close (READ);
    }
}

if ($replace) ## strip-out old timing annotations as requested
{
    @inputStream = stripTiming(@inputStream);
}  ## end eliminate timings/calendar



## Pre-Compute jd and sun RA for Jan 1
my $jd1Jan = julian_day($year,1,1) - 0.5;
my ($sunRA1Jan,$sunDec1Jan) = &sunPos($jd1Jan);

## Local time/DST calcs
my @timeZone = @{ $$interferometer->{TimeZone} };
my $useDST = $timeZone[1];



## Big loop over each line in input stream
foreach my $line (@inputStream)
{

    if (!($line =~ m/^\#/)    ## if this isn't a comment line
	&& ($line =~ m/\S+/)  ## and the line apparently has data
	&& (($line =~ m/^\S+\s+\d+\s+\d+\s+\d+\.?\d*\s+\+?\-?\d+\s+\d+\s+\d+?/)   ## astrometry line match
	    || ($line =~ m/^\S+\s+\d+\s+\d+\s+\d+\.?\d*\s+\+?\-?\d+\s+\d+/))) ## astrometry line match
    {
        my $iLine = new internalLine(line => $line);

        ## TLE - 6/2005 check to see if all required fields are there
        if ( my $missingFields = $iLine->checkFields(@reqFields) )
        {
              die "ERROR: obsCalendar: Missing required fields $missingFields \n";
        }

        my $name = $iLine->extractField("designation");
        my $raD = $iLine->extractField("raDeg");
        my $decD = $iLine->extractField("decDeg");
        my $raString = &RAhms($raD);
        my $decString = &DECdms($decD);

        ##  TLE - 12/2004  also added boolean $isTrg for src type checks in next if block
        my $isTrg = (($iLine->extractField("tagROLE")) eq "TRG");


	if (($decD > 0) && ((90.0 - $decD) < $localLatitude))
	{
	    print "# $name $raString  $decString  is always available in the sky";
            print " target" if ($isTrg);
	    print "\n";
	}
	elsif (abs($decD - $localLatitude) > 90.0)
	{
	    print "# $name $raString  $decString  is never available in the sky";
            print " target" if ($isTrg);
	    print "\n";
	}
	elsif (($decD < 0) && ($decD + 90.0) < (- $localLatitude))
	{
	    print "# $name $raString  $decString  is always available in the sky";
            print " target" if ($isTrg);
	    print "\n";
	}
	else  ## normal computation
	{

	    ## Compute transit/sunrise day
	    ## Transit/sunrise happens when the sun lags the target RA by ~ 90 deg
	    ## First approximation
	    my $raTarget = $raD + 90.0;
	    if ($raTarget < $sunRA1Jan)
	    {
		$raTarget += 360.0;
	    }

	    my $ydayRise = int ($raTarget - $sunRA1Jan + 0.5);
	    my $jdRise = $jd1Jan + ($ydayRise - 1);

	    ## Now tune-up this root
	    my ($sunRA,$sunDec);
	    my $zaTarget = 90 + $twilightAngle;
	    my $iter = 0;
	    my $correction;
	    do
	    {
		$iter++;
		($sunRA,$sunDec) = sunPos($jdRise);
		my $haTarget
		    = POSIX::acos((cos($zaTarget * DEG_TO_RAD)
				   - sin($localLatitude * DEG_TO_RAD)
				   * sin($sunDec * DEG_TO_RAD))
				  / (cos($localLatitude * DEG_TO_RAD)
				     * cos($sunDec * DEG_TO_RAD)));
		$haTarget *= RAD_TO_DEG;
		$raTarget = $raD + $haTarget;
		$raTarget -= 360.0  if ($raTarget > 360.0);  ## fix wraps

		$correction = $raTarget - $sunRA;  ## differential correction
		$correction -= (&sign($correction) * 360.0)  ## correct any wraps
		    if (abs($correction) > 180.0);

		$ydayRise += $correctionGain * $correction;
		$ydayRise -= $daysInYear if ($ydayRise > $daysInYear);  ## More wrap logic
		$ydayRise += $daysInYear if ($ydayRise < 0);
		$jdRise = $jd1Jan + ($ydayRise - 1);
	    }
	    while ((abs($correction) > $TOL)
		   && ($iter < $MAX_ITER));

	    $ydayRise = int ($ydayRise + 0.5);  ## Round to nearest day
	    my $riseString = &dateString($year,$ydayRise);
	    $ydayRise = join '',"0", $ydayRise if ($ydayRise < 10);
	    $ydayRise = join '',"0", $ydayRise if ($ydayRise < 100);
	    printf "## Warning: answer suspect -- final residual %3.1f d\n", $correction if ($iter >= $MAX_ITER);
	    print "# $name $raString  $decString  is near transit at sunrise on $riseString ($year$ydayRise)";
            print " target" if ($isTrg);
	    print "\n";


	    if ($hourAngle)
	    {

		## Compute hour angle/sunset day
		## Transit/sunset happens when the sun leads the target RA by ~ 90 deg
		$raTarget = $raD + 90.0 - ($hourAngle * 15.0);
		if ($raTarget < $sunRA1Jan)
		{
		    $raTarget += 360.0;
		}

		$ydayRise = int ($raTarget - $sunRA1Jan + 0.5);
		$jdRise = $jd1Jan + ($ydayRise - 1.0);

		## Tune up the root
		$iter = 0;
		do
		{
		    $iter++;
		    ($sunRA,$sunDec) = sunPos($jdRise);
		    my $haTarget
			= POSIX::acos((cos($zaTarget * DEG_TO_RAD)
				       - sin($localLatitude * DEG_TO_RAD)
				       * sin($sunDec * DEG_TO_RAD))
				      / (cos($localLatitude * DEG_TO_RAD)
					 * cos($sunDec * DEG_TO_RAD)));
		    $haTarget *= RAD_TO_DEG;
		    $raTarget = $raD + $haTarget - ($hourAngle * 15.0);
		    $raTarget += 360.0  if ($raTarget < 0.0);  ## fix wraps
		    $raTarget -= 360.0  if ($raTarget > 360.0);  ## fix wraps

		    $correction = $raTarget - $sunRA;  ## differential correction
		    $correction -= (&sign($correction) * 360.0)  ## correct any wraps
			if (abs($correction) > 180.0);

		    $ydayRise += $correctionGain * $correction;
		    $ydayRise -= $daysInYear if ($ydayRise > $daysInYear);  ## More wrap logic
		    $ydayRise += $daysInYear if ($ydayRise < 0);
		    $jdRise = $jd1Jan + ($ydayRise - 1.0);

		}
		while ((abs($correction) > $TOL)
		       && ($iter < $MAX_ITER));

		$ydayRise = int ($ydayRise + 0.5);  ## Round to nearest day
		my $riseString = &dateString($year,$ydayRise);
		$ydayRise = join '',"0", $ydayRise if ($ydayRise < 10); 
		$ydayRise = join '',"0", $ydayRise if ($ydayRise < 100);

		printf "## Warning: answer suspect -- final residual %3.1f d\n", $correction if ($iter >= $MAX_ITER);
		print "# $name is $hourAngle hours pre-transit at sunrise on $riseString ($year$ydayRise)\n";

	    } ## end computing sunrise offset by $hourAngle


	    ## Compute transit/midnight day
	    ## Transit/midnight happens when the sun lags the target RA by 180 deg
	    ## First approximation
	    $raTarget = $raD + 180.0;
	    if ($raTarget < $sunRA1Jan)
	    {
		$raTarget += 360.0;
	    }

	    my $ydayMid = int ($raTarget - $sunRA1Jan + 0.5);
	    my $jdMid = $jd1Jan + ($ydayMid - 1);
	    my $lstMid;

	    ## Now tune-up this root
	    $iter = 0;
	    do
	    {
		$iter++;
		## Compute the LST at local midnight
		my $jdZeroUT = (int $jdMid) + 0.5;
	        my $lstZeroUT = &siderealTime::localSiderealTime($jdZeroUT,$localLongitude) * 15.0;
		my ($month,$mday,$rest) = split /\//, &dateString($year,$ydayMid), 3;
		my $midnightOffset = (- $timeZone[0]);
		if (($useDST) && (&dstBounds::isDST($year,$month,$mday,$useDST)))
		{
		    $isdst = 1;
		    $midnightOffset -= 1;  ## "Spring forward..."
		} ## checking Daylight Savings Time 
		$lstMid = $lstZeroUT
		    + ($midnightOffset * 15.0 * novasConstants::SolarToSidereal);
		$lstMid += 360.0 if ($lstMid < 0.0);
		$lstMid -= 360.0 if ($lstMid > 360.0);

		$correction = $raD - $lstMid;  ## differential correction
		$correction -= (&sign($correction) * 360.0)  ## correct any wraps
		    if (abs($correction) > 180.0);

		$ydayMid += $correctionGain * $correction;
		$ydayMid -= $daysInYear if ($ydayMid > $daysInYear);  ## More wrap logic
		$ydayMid += $daysInYear if ($ydayMid < 0);
		$jdMid = $jd1Jan + ($ydayMid - 1);

	    }
	    while ((abs($correction) > 1.0)
		   && ($iter < $MAX_ITER));
;

	    $ydayMid = int ($ydayMid + 0.5);  ## Round to nearest day
	    my $midString = &dateString($year,$ydayMid);
	    $ydayMid = join '',"0", $ydayMid if ($ydayMid < 10);
	    $ydayMid = join '',"0", $ydayMid if ($ydayMid < 100);

	    printf "## Warning: answer suspect -- final residual %3.1f d\n", $correction if ($iter >= $MAX_ITER);
	    print "# $name is near transit at midnight on $midString ($year$ydayMid)\n";


	    ## Compute transit/sunset day
	    ## Transit/sunset happens when the sun leads the target RA by ~ 90 deg
	    $raTarget = $raD - 90.0;
	    if ($raTarget < $sunRA1Jan)
	    {
		$raTarget += 360.0;
	    }

	    my $ydaySet = int ($raTarget - $sunRA1Jan + 0.5);
	    my $jdSet = $jd1Jan + ($ydaySet - 1.0);

	    ## Tune up the root
	    $iter = 0;
	    do
	    {
		$iter++;
		($sunRA,$sunDec) = sunPos($jdSet);
		my $haTarget
		    = POSIX::acos((cos($zaTarget * DEG_TO_RAD)
				   - sin($localLatitude * DEG_TO_RAD)
				   * sin($sunDec * DEG_TO_RAD))
				  / (cos($localLatitude * DEG_TO_RAD)
				     * cos($sunDec * DEG_TO_RAD)));
		$haTarget *= RAD_TO_DEG;
		$raTarget = $raD - $haTarget;
		$raTarget += 360.0 if ($raTarget < 0.0);  ## fix wraps

		$correction = $raTarget - $sunRA;  ## differential correction
		$correction -= (&sign($correction) * 360.0)  ## correct any wraps
		    if (abs($correction) > 180.0);

		$ydaySet += $correctionGain * $correction;
		$ydaySet -= $daysInYear if ($ydaySet > $daysInYear);  ## More wrap logic
		$ydaySet += $daysInYear if ($ydaySet < 0);
		$jdSet = $jd1Jan + ($ydaySet - 1.0);

	    }
	    while ((abs($correction) > $TOL)
		   && ($iter < $MAX_ITER));


	    $ydaySet = int ($ydaySet + 0.5);  ## Round to nearest day
	    my $setString = &dateString($year,$ydaySet);
	    $ydaySet = join '',"0", $ydaySet if ($ydaySet < 10); 
	    $ydaySet = join '',"0", $ydaySet if ($ydaySet < 100);

	    printf "## Warning: answer suspect -- final residual %3.1f d\n", $correction if ($iter >= $MAX_ITER);
	    print "# $name is near transit at sunset on $setString ($year$ydaySet)\n";


	    if ($hourAngle)
	    {

		## Compute hour angle/sunset day
		## Transit/sunset happens when the sun leads the target RA by ~ 90 deg
		$raTarget = $raD - 90.0 + ($hourAngle * 15.0);
		if ($raTarget < $sunRA1Jan)
		{
		    $raTarget += 360.0;
		}

		$ydaySet = int ($raTarget - $sunRA1Jan + 0.5);
		$jdSet = $jd1Jan + ($ydaySet - 1.0);

		## Tune up the root
		$iter = 0;
		do
		{
		    $iter++;
		    ($sunRA,$sunDec) = sunPos($jdSet);
		    my $haTarget
			= POSIX::acos((cos($zaTarget * DEG_TO_RAD)
				       - sin($localLatitude * DEG_TO_RAD)
				       * sin($sunDec * DEG_TO_RAD))
				      / (cos($localLatitude * DEG_TO_RAD)
					 * cos($sunDec * DEG_TO_RAD)));
		    $haTarget *= RAD_TO_DEG;
		    $raTarget = $raD - $haTarget + ($hourAngle * 15.0);
		    $raTarget += 360.0  if ($raTarget < 0.0);  ## fix wraps
		    $raTarget -= 360.0  if ($raTarget > 360.0);  ## fix wraps

		    $correction = $raTarget - $sunRA;  ## differential correction
		    $correction -= (&sign($correction) * 360.0)  ## correct any wraps
			if (abs($correction) > 180.0);

		    $ydaySet += $correctionGain * $correction;
		    $ydaySet -= $daysInYear if ($ydaySet > $daysInYear);  ## More wrap logic
		    $ydaySet += $daysInYear if ($ydaySet < 0);
		    $jdSet = $jd1Jan + ($ydaySet - 1.0);

		}
		while ((abs($correction) > $TOL)
		       && ($iter < $MAX_ITER));

		$ydaySet = int ($ydaySet + 0.5);  ## Round to nearest day
		my $setString = &dateString($year,$ydaySet);
		$ydaySet = join '',"0", $ydaySet if ($ydaySet < 10); 
		$ydaySet = join '',"0", $ydaySet if ($ydaySet < 100);

		printf "## Warning: answer suspect -- final residual %3.1f d\n", $correction if ($iter >= $MAX_ITER);
		print "# $name is $hourAngle hours post-transit at sunset on $setString ($year$ydaySet)\n";

	    } ## end computing sunset offset by $hourAngle

	}  ## end normal computation


    } ## end data line check

    if (!$noEcho)
    {
	print $line;
    }

} ## end foreach





exit(0); ## end obsCalendar
## end obsCalendar  #####################################################



#########################################################################
## setLocation -- define location for obsCalendar 
#########################################################################
sub setLocation
{
    ## setLocation subroutine handles location
    ## Mod 1/2001 for new interferometer module

    ## Input: location string
    ## Output: variables $localLongitude, $localLatitude, and
    ## @localBaseline loaded with their appropriate values

## declarations setLocation  ############################################
    ## Location logic
    my $location = shift @_;  ## Extract location call argument

## begin setLocation  ###################################################
    ## We could probably do some environment variable stuff here to
    ## allow users to define the default location for their use

    ## Verify specified location is supported
    my $interferometer;
    if (exists $interferometers::InterferometerList{$location})
    {
	## Define the interferometer we're working with
	$interferometer = $interferometers::InterferometerList{$location};
    }
    else ## unsupported location
    {
	print "Error -- location $location unsupported!\n";
	my @locations = sort keys %interferometers::InterferometerList;
	print " Supported locations (alphabetical order): @locations\n";
	exit(-1);
    } ## end interferometer definition/error trap


    ## Extract and verify location
    my $localLongitude = $$interferometer->{Longitude};
    my $longStr = &DECdms($localLongitude,0);
    $longStr =~ s/\s+/:/g;
    my $localLatitude = $$interferometer->{Latitude};
    my $latStr = &DECdms($localLatitude,0);
    $latStr =~ s/\s+/:/g;
    print "# Using $location Location (long: $longStr   lat: $latStr)\n" if ($preamble);

    return ($localLatitude,$localLongitude,$interferometer);


} ## end setLocation  ###################################################

#########################################################################
## dateString -- date string fun
#########################################################################
sub dateString
{

    my ($year,$day) = @_;

    my $month = 1;

    while ($day > days_in($year,$month))
    {
	$day -= days_in($year,$month);
	$month++;
    } ## end while

    return sprintf "%d/%d/%d", $month,$day,$year;

} ## end dateString  ####################################################
