#! @PERL@
# -*- perl -*-
# @configure_input@

our $MODVERSION = '$Id: timing.pl,v 1.38 2008/11/12 23:28:47 sgroom Exp $';
my $MODREV = '$Revision: 1.38 $';

##########################################################################
# NASA Exoplanet Science Institute (NExScI) -- http://nexsci.caltech.edu #
#       Copyright (c) 1997-2008 California Institute of Technology       #
#                     U.S. Government Sponsorship                        #
# Under contract with the National Aeronautics and Space Administration  #
##########################################################################


### timing -- perl script to compute target accessibility to interferometers

# description: timing takes a GUI-readable list of objects from stdin,
# and it computes the zenith angle and delay line accessibilities
# times for each target

# Generic use model

# timing < GUI_list [> GUI_list_annotated_with_timing_parameters]
# or
# timing GUI_list [> GUI_list_annotated_with_timing_parameters]

use strict;  ## Check syntax more carefully...
use warnings;
use FindBin qw($RealBin);
use lib "$RealBin/../lib/modules";		# location of getCal Perl modules

# standard modules:
use POSIX;
use Getopt::Long;
use Time::JulianDay;

# getCal modules:
use gcConfig;
use gcDebug;
use coordinates;
use novasConstants;
use siderealTime;
use findScript;
use dstBounds;
use angles;
use internalLine;
use interferometers;
use geometry;
use vector;
use yearDay;
use baseline;
use dstBounds;
use miscMath;
use miscArgs;
use miscString;
use pointing;
use timingMisc;
use sunMoon;
use interval;
use nullDepth;

my $cfg = gcConfig::config();	# initialize getCal settings
my @reqFields = qw/ designation raDeg decDeg tagROLE / ;

## max hour angle range is +/- 180.0 degrees (full sky)
my $maxHA = 180.0;

my $secondsPerDegree = 240.0;	# conversion constant: 360deg=24hr => 1deg=240.0seconds

# default values
my ($readStdin,$noPrecess,$noEcho,$replace,$airmassTarget);
my (@filenames,@baseline);

# pull defaults we care about from getCal config
my $location = $cfg->defaultLocation();		# location
my $defBaseline = $cfg->defaultBaseline();	# baseline
my $dOffset = $cfg->defaultBiasOffset();	# delay bias offset (LDL)
my $zaTarget = $cfg->zenithAngleLimit();	# zenith angle limit (deg from zenith)
my $twilightAngle = $cfg->twilightAngle();	# twilight angle (deg below horizon)
my $defNDwavelength = $cfg->nullDepthWavelength();	# wavelength for nulldepth
my $defDiam = $cfg->nullDepthTrgAngDiam();	# default ang diam for nulldepth

my $opt_delayLimit = undef;  ## defined in setLocation appropriate for interferometer
my $opt_delayMaxLimit = undef;  ## separate definition of maximum and minimum targets
my $opt_delayMinLimit = undef;
my $opt_table;			## option to display table of delay positions vs time
my $opt_tableStep = 300.0;	## time step size (seconds) to use for delay table
my $opt_NDstep = 300.0;	## time step size (seconds) to use for null depth table

# options related to null depth estimation
my $opt_nullDepth = 0;		## option to enable output of null depth estimates
my $opt_NDwavelength = $defNDwavelength;		## wavelength for null depth
my $opt_trgAngDiam = $defDiam;		## option to enable output of null depth estimates

my $preamble = 1;
my $precess = 1;
my $noPreamble = 0;
my $noTargets = 0;
my $verbose = 0;


# Step 1.  Get current time information, LST, JD, sunset/sunrise

#  0    1    2     3     4    5     6     7     8
my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = gmtime(time);
$mon += 1; ## These months start at zero
$yday += 1; ## Adding 1 to conform to "date -u +%j" (Jan 1 = 001)
$year += 1900;  ## We know how the dates come out of gmtime...

## Adjust date formats
if ($yday < 10)
{
    $yday = join '',"0", $yday;
}

if ($yday < 100)
{
    $yday = join '',"0", $yday;
}


## Save this until after we've determined whether we are writing a
## preamble
my $revStr = "$MODREV";
$revStr =~ s/\$//g;	# strip the $'s from the revision string
$revStr =~ s/Revision: //g;
$revStr =~ s/ //g;
my $preambleText = "# Timing summary (timing v$revStr)\n";


## Command-line options
handleCommonOpts(1, "timing.txt");
my $result = GetOptions("stdin" => \$readStdin,
			"filename=s" => \@filenames,
			"noPreamble" => \$noPreamble,
			"noTargets" => \$noTargets,
			"verbose" => sub { $verbose++; },
			"year=i" => \$year,
			"month=i" => \$mon,
			"day=i" => \$mday,
			"precess" => \$precess,
			"noPrecess" => \$noPrecess,
			"noEcho" => \$noEcho,
			"replace" => \$replace,
			"za|zenithAngle=f" => \$zaTarget,
			"airmass=f" => \$airmassTarget,
			"delayLimit=f" => \$opt_delayLimit,
			"delayMax=f" => \$opt_delayMaxLimit,
			"delayMin=f" => \$opt_delayMinLimit,
			"twilightAngle=f" => \$twilightAngle,
			"location=s" => \$location,
			"baseline=s" => \@baseline,
			"biasOffset=f" => \$dOffset,
			"offset=f" => \$dOffset,
			"LDL=f" => \$dOffset,
			"table" => \$opt_table,
			"tableStep=f" => \$opt_tableStep,
			"NDstep=f" => \$opt_NDstep,
			"NDwavelength" => \$opt_NDwavelength,
			"NDlambda" => \$opt_NDwavelength,
			"trgAngDiam=f" => \$opt_trgAngDiam,
			"nullDepth" => \$opt_nullDepth);

## If the user didn't specify a baseline, assume he wants the default
## baseline
if (!@baseline)
{
    $baseline[0] = $defBaseline;  ## default baseline
}

$baseline[0] = "none" if ($noTargets);  ## No baseline if no Targets



## Decide whether to output timing preamble
if ($noPreamble && !($noTargets))  ## No preamble and no targets doesn't make sense
{
    $preamble = !($noPreamble);  ## No preamble
}
else
{
    print "$preambleText";  ## preamble -- dump what we have and continue
}


if ($noPrecess)
{
    $precess = !($precess);
}


## Calculate yearday for the date we're actually servicing.  Adjust
## date formats for possible user input
$yday = yearDay($year,$mon,$mday);

## year mods
$year += 100 if ($year < 90);
$year += 1900 if ($year < 1900);

## yday mods
$yday = join '',"0", $yday if ($yday < 10);
$yday = join '',"0", $yday if ($yday < 100);

print "#  for timings on $mon/$mday/$year UTC, day $year$yday\n" if ($preamble) ;



## Set/verify the geometry for the calculations (location, baselines)
## and define the hash with baseline objects for the baselines we will
## be servicing
my ($localLongitude,$localLatitude,$interferometer,%bHash)
    = &setLocation($location,@baseline);

# check the pointing limits definition for each baseline to make sure
# it is valid - SG 6/06
my $blCount;	# number of baselines we're working with
foreach my $bl (sort keys %bHash)
{
    my $blOK = validateLimits(\$bHash{$bl});
    if (!$blOK)
    {
	#print STDERR "Malformed pointing limits for defined baseline " . $bHash{$bl}{name} . "\n";
	die "Malformed pointing limits for defined baseline " . $bHash{$bl}{name} . "\n";
    }
    $blCount++;
}

## Compute JD, LST, and local time at 0 hrs UT
my $myJD = julian_day($year,$mon,$mday) - 0.5;

print "# The JD at 0 hr UT is $myJD" if ($preamble);
print " (precessing coordinates to this date)" if (($precess) && ($preamble));
print "\n" if ($preamble);


my $LST_str = RAhms(&siderealTime::localSiderealTime($myJD,$localLongitude) * 15.0,0);
my $LST_ZeroUT = RAdeg($LST_str);
$LST_str =~ s/\s+/:/g;
print "# The LST at 0 hr UT is $LST_str\n" if ($preamble);

## Local time/DST calcs
my @timeZone = @{ $$interferometer->{TimeZone} };
my $Local_ZeroUT = $timeZone[0];
my $useDST = $timeZone[1];
$isdst = 0;
## Checking DST
if (($useDST) && (&dstBounds::isDST($year,$mon,$mday,$useDST)))
{
    $isdst = 1;
    $Local_ZeroUT += 1;  ## "Spring forward..."
} ## checking Daylight Savings Time 
    
if ($Local_ZeroUT < 0)  ## Check/fix wraps
{
    $Local_ZeroUT += 24;
}
my $Local_str = RAhms($Local_ZeroUT * 15.0,0);
$Local_str =~ s/\s/:/g;
print "# The Local Time at 0 hr UT is $Local_str" if ($preamble);
print " (Daylight Savings Time)" if ($preamble && $isdst);
print "\n" if ($preamble);



## Compute sunset/sunrise times
&sunTimings($myJD) if ($preamble);
## Compute moonrise/moonset times
&moonTimings($myJD) if ($preamble);


exit(0) if ($noTargets);  ## If noTargets is thrown, exit...


## Finally, report on zenith and delay limits
## Compute zenith/airmass constraints
if ($airmassTarget)  ## If we were given an airmass target
{
    $zaTarget = POSIX::acos(1.0 / $airmassTarget) * DEG_TO_RAD;
    $zaTarget = sprintf "%3.1f", $zaTarget;
}
else ## otherwise calculate resulting airmass constraint
{
    $airmassTarget = 1.0 / cos($zaTarget * DEG_TO_RAD);
    $airmassTarget = sprintf "%3.2f", $airmassTarget;
}
if ($preamble)
{
    print "# Using $zaTarget deg zenith angle ($airmassTarget airmass) constraint\n";

    foreach my $bl (sort keys %bHash)
    {
	my $dMinLimit = $bHash{$bl}{delayMinLimit};
	my $dMaxLimit = $bHash{$bl}{delayMaxLimit};

	print "# Using $dMinLimit to $dMaxLimit m delay interval";
	if ($dOffset)
	{
	    print " with $dOffset m bias offset";
	}
	print " on baseline $bl\n";
    }
}


### Finally starting the real work

# Step 2.  Loop through input GUI list
my @inputStream;
if (($readStdin)
    || ((! (scalar @filenames)) && (!(scalar @ARGV)))) ## read from stdin
{
    @inputStream = <STDIN>;
}
elsif (scalar @filenames)  ## otherwise read from designated filenames
{
    foreach my $file (@filenames)
    {
	open (READ,$file) or die "Unable to open $file\n";
	push @inputStream , <READ>;
	close (READ);
    } ## end foreach
}
elsif (scalar @ARGV)
{
    foreach my $file (@ARGV)  ## Read all the remaing filenames
    {
	open (READ,$file) or die "Unable to open $file\n";
	push @inputStream, <READ>;
	close (READ);
    } ## end foreach
}

if ($replace) ## strip-out old timing annotations as requested
{
    @inputStream = stripTiming(@inputStream);
}  ## end eliminate timings/calendar


## Big loop over each line in input stream
foreach my $line (@inputStream)
{
    if (!($line =~ m/^\#/)    ## if this isn't a comment line
	&& ($line =~ m/\S+/)  ## and the line apparently has data
	&& (($line =~ m/^\S+\s+\d+\s+\d+\s+\d+\.?\d*\s+\+?\-?\d+\s+\d+\s+\d+?/)   ## astrometry line match
	    || ($line =~ m/^\S+\s+\d+\s+\d+\s+\d+\.?\d*\s+\+?\-?\d+\s+\d+/))) ## astrometry line match
    {
        ##  TLE - 12/2004  use internalLine class 
        my $iLine = new internalLine(line => $line);

        ## TLE - 6/2005 check to see if all required fields are there
        if ( my $missingFields = $iLine->checkFields(@reqFields) ) 
        {
	      die "ERROR: timing: Missing required fields $missingFields, input line was\n$line\n";
        }

        my $name = $iLine->extractField("designation");
        my $raD = $iLine->extractField("raDeg");
        my $decD = $iLine->extractField("decDeg");

	if ($precess)
	{
	    ($raD,$decD) = &precession($raD,$decD,$myJD);
	}
	my $raString = &RAhms($raD);
	my $decString = &DECdms($decD);

	## if we're doing null depth, get ang diam information from object record
	my $angDiam = undef;
	if ($opt_nullDepth)
	{
	    # default ang diam
	    my $role = $iLine->extractField("tagROLE");
	    if ($role eq "TRG")
	    {
		$angDiam = $opt_trgAngDiam;	# targets have no tagDIAM, use user-supplied or default
	    }
	    else
	    {
		$angDiam = $iLine->extractField("tagDIAM");
		if (!defined $angDiam)
		{
		    $angDiam = $opt_trgAngDiam;	# cal has no tagDIAM, use user-supplied or default
		}
	    }
	}

	###print "$name $raString  $decString\n";

	# Compute the UT transit time, corrected for earth's rotation rate
	my $UT_Transit = &siderealToSolar(principalValue($raD - $LST_ZeroUT));
	my $UTstr = &timeString($UT_Transit);

	my $zaTransit = zenithAngle(0,$decD,$localLatitude);
	my $zaTransitStr = sprintf "%3.1f", $zaTransit;

	my $amTransit = 1.0 / cos($zaTransit * DEG_TO_RAD);
	$amTransit = sprintf "%3.2f", $amTransit;

	print "# $name $raString  $decString  transits at $UTstr UT, zenAng $zaTransitStr deg ($amTransit am)";
        if (($iLine->extractField("tagROLE")) eq "TRG")
	{
	    print " target";
	}
	print "\n";

	# Compute time interval for zenith angle limits
	if ($zaTransit < $zaTarget)
	{
	    print STDERR "DEBUG: timing: checking zenith angle range\n" if gcConfig::debug();
	    my $zaMax = zenithAngle(-180.0,$decD,$localLatitude);	# test max za for this object
	    if ($zaMax < $zaTarget)
	    {
		print "# $name is always within zenith angle range\n";
	    }
	    else
	    {
		my $ha1 = &zaLimit($decD,$localLatitude,$zaTarget,-1.0);	# HA of rising ZA
		my $ha2 = &zaLimit($decD,$localLatitude,$zaTarget,+1.0);	# HA of setting ZA
		my $inStr = &timeString($UT_Transit + &siderealToSolar($ha1));
		my $outStr = &timeString($UT_Transit + &siderealToSolar($ha2));
		print "# $name is within zenith angle range from $inStr to $outStr UT\n";
	    }
	} # end ZA computations
	else
	{
	    print "# $name is never within zenith angle range\n";
	} # end not ever in ZA range

	# Compute time interval where object is above the 0degree horizon.
	# This will be used to remove any extra delay intervals that occur
	# when the object is below the horizon and are therefore of no use,
	# they just clutter up the output.
	my $haHor;	# hour angle where object intersects horizon
	my @horizon;
	if (abs($decD - $localLatitude) >= 90.0)
	{
	    $haHor = 0.0;
	    # object never rises above the ideal (0deg) horizon
	    # do nothing, horizon will be empty list
	}
	elsif (abs($decD + $localLatitude) >= 90.0)
	{
	    $haHor = 180.0;	# circumpolar, always above horizon
	    push @horizon, [-$haHor, +$haHor];
	}
	else
	{
	    $haHor = zaLimit($decD,$localLatitude,90.0,1); # compute HA of object at ideal horizon (el=0, za=90)
	    push @horizon, [-$haHor, +$haHor];
	}

	## Compute time interval for delay and pointing limits
	# Initialize the composite delay&pointing range lists to "always"
	my @compositeDList = ([-$maxHA, +$maxHA]); # always in range, HA (degrees)
	my @compositePList = ([-$maxHA, +$maxHA]); # always in range, HA (degrees)

	my $foundPointing = 0;
	foreach my $bl (sort keys %bHash)
	{
	    ## compute and output delay limits for this baseline
	    printf "#>> checking delay limits for bl $bl\n" if ($verbose);
	    my @dList = &computeDelayLimits($name,$decD,$UT_Transit,\$bHash{$bl},$dOffset,$verbose);
	    # @dList now contains a list of delay intervals,
	    # stored as HA relative to transit, in units of degrees

	    # Testing: report the full set of delay intervals
	    #&reportIntervals("delayAll",$name,$bHash{$bl}{name},$UT_Transit,\@dList,$verbose);	# report delay

	    # SLG 6/28/07: ignore any delay intervals that occur when this object is below the horizon.
	    &intersectList(\@dList, \@horizon, \@dList);	# intersect with above-the-horizon interval

	    &reportIntervals("delay",$name,$bHash{$bl}{name},$UT_Transit,\@dList,$verbose);	# report delay

	    ## compute and output pointing limits for this baseline
	    my @pList;
	    my $blPointing = &hasLimits(\$bHash{$bl});	# check for pointing limits
	    if ($blPointing)	# if pointing limits are defined for this baseline
	    {
		printf "#>> checking pointing limits for bl $bl\n" if ($verbose);

		$foundPointing = 1;	# there are pointing limits on at least one baseline
		@pList = &computePointingLimits($name,$decD,$localLatitude,$UT_Transit,\$bHash{$bl},$verbose);
		# @pList now contains a list of pointing intervals,
		# stored as time relative to transit, in units of degrees
		&reportIntervals("pointing",$name,$bHash{$bl}{name},$UT_Transit,\@pList,$verbose);	# report pointing
	    }

	    ## update composite range lists by intersecting with this baseline's intervals
	    &intersectList(\@compositeDList, \@dList, \@compositeDList);
	    if ($blPointing)	# if pointing limits are defined for this baseline
	    {
		&intersectList(\@compositePList, \@pList, \@compositePList);
	    }

	    for (my $idx=0; $idx < scalar(@compositeDList); $idx++)
	    {
		printf "#>> updated composite delay interval with bl $bl: [%7.3f, %7.3f], UT [%s, %s]\n",
		    $compositeDList[$idx][0],$compositeDList[$idx][1],
		    &timeString($UT_Transit + &siderealToSolar($compositeDList[$idx][0]), 0),
		    &timeString($UT_Transit + &siderealToSolar($compositeDList[$idx][1]), 0)
		    if ($verbose);
	    }
	    for (my $idx=0; $idx < scalar(@compositePList); $idx++)
	    {
		printf "#>> updated composite pointing interval with bl $bl: [%7.3f, %7.3f], UT [%s, %s]\n",
		    $compositePList[$idx][0],$compositePList[$idx][1],
		    &timeString($UT_Transit + &siderealToSolar($compositePList[$idx][0]), 0),
		    &timeString($UT_Transit + &siderealToSolar($compositePList[$idx][1]), 0)
		    if ($verbose);
	    }

	} ## end foreach

	if ($blCount > 1)	# if more than one baseline, output results for the composite baseline
	{
	    my $compositeName = "Composite";
	    # output composite delay information
	    &reportIntervals("delay",$name,$compositeName,$UT_Transit,\@compositeDList,$verbose);	# report delay

	    # output composite pointing information
	    if ($foundPointing)	# pointing limits exist on at least one baseline
	    {
		&reportIntervals("pointing",$name,$compositeName,$UT_Transit,\@compositePList,$verbose);	# report pointing
	    } # end if $foundPointing
	} # end if $blCount>1

	# print table of computed delays for this object if selected and object rises above horizon
	if ($opt_table && $haHor)
	{
	    foreach my $bl (sort keys %bHash)
	    {
		&printDelayTable($name,$decD,$haHor,$UT_Transit,\$bHash{$bl},$dOffset,$verbose);
	    }
	}

	# print table of null depth estimates for this object if selected and object rises above horizon
	if ($opt_nullDepth && $haHor)
	{
	    foreach my $bl (sort keys %bHash)
	    {
		&printNDTable($name,$decD,$angDiam,$opt_NDwavelength,$haHor,$UT_Transit,\$bHash{$bl},$verbose);
		#&printNDTableHA($name,$decD,$angDiam,$opt_NDwavelength,$haHor,$UT_Transit,\$bHash{$bl},$verbose);
	    }
	}

    } # end if data line check

    if (!$noEcho)
    {
	print $line;
    } # echoing object line

} ## end big foreach $line loop


exit(0);
## end timing  ##########################################################





#########################################################################
## setLocation -- define location, baselines for timing
#########################################################################
sub setLocation
{
    ## setLocation subroutine handles location and baseline
    ## information for the interferometer determination.  A.B. 6/4/00
    ## Mod 1/2001 for new interferometer module
    ## Mod 2/2004 for new baseline module

    ## Input: location string
    ## Output: variables $localLongitude, $localLatitude
    ## Associative array %bHash is returned loaded with baseline
    ## objects for the baselines we are processing

## declarations setLocation  ############################################
    ## Location logic
    my $location = shift @_;  ## Extract location call argument
    my $interferometer;

## begin setLocation  ###################################################
    ## We could probably do some environment variable stuff here to
    ## allow users to define the default location for their use

    ## Verify specified location is supported
    if (exists $interferometers::InterferometerList{$location})
    {
	## Define the interferometer we're working with
	$interferometer = $interferometers::InterferometerList{$location};
    }
    else ## unsupported location
    {
	print "Error -- location $location unsupported!\n";
	my @locations = sort keys %interferometers::InterferometerList;
	print " Supported locations (alphabetical order): @locations\n";
	exit(-1);
    } ## end interferometer definition/error trap


    ## Extract and verify location
    my $localLongitude = $$interferometer->{Longitude};
    my $longStr = &DECdms($localLongitude,0);
    $longStr =~ s/\s+/:/g;
    my $localLatitude = $$interferometer->{Latitude};
    my $latStr = &DECdms($localLatitude,0);
    $latStr =~ s/\s+/:/g;
    print "# Using $location Location (long: $longStr   lat: $latStr)\n" if ($preamble);

    ## If not already set, define the delay limits appropriate to the location
    if (defined $opt_delayLimit) ## set extrema symmetrically around zero
    {
	$opt_delayMinLimit = - abs($opt_delayLimit);
	$opt_delayMaxLimit = + abs($opt_delayLimit);
    }

    ###############################
    ## Baseline definition logic ##
    ###############################
    my @bline = @_;  ## Extract baseline designation call argument
    my $bl;

    ## Check which baseline(s) have been specified. If none, we're
    ## already done. Otherwise, create and return a hash of the
    ## baseline objects for the selected baseline(s).

    if (grep /none|NONE|None/, @bline)
    {
	return ($localLongitude,$localLatitude,$interferometer);  ## returning no baselines
    }

    my %bHash;

    ## Check to see if the user has requested all the baselines at
    ## this interferometer
    if (grep /all|All|ALL/, @bline)
    {
       ## User has requested to process all baselines for this location
       ## Replace the baseline list with the individual baseline names
       my @baselines = sort @{ $$interferometer->{Baselines}};
       @bline = @baselines;
    }

    ## add each requested baseline to the hash
    foreach $bl (@bline)
    {
	## Verify specified baseline designation is supported
	if (defined($$interferometer->{$bl}) && @{ $$interferometer->{$bl}})
	{
	    # 3/07 - incorporate DelayOffset, indicate that the delay
	    # range center for a baseline is offset from 0 
	    my $limitName = $bl . "_Limits";	# name of pointing limits for this bl, if any
	    my $doffName = $bl . "_DelayOffset"; # name of delay offset for this bl, if any
	    my $doff = $$interferometer->{$doffName};
	    my $doffval = $doff || 0.0;

	    # delay range for this baseline - use site's range if no bl range specified.
	    my $dRangeName = $bl . "_DelayRange"; # name of delay range for this bl, if any
	    my $dRange = $$interferometer->{$dRangeName};	# check for bl delay range
	    if (defined $dRange)
	    {
		print STDERR "DEBUG: timing:setlocation: baseline $bl delay range $dRange\n" if gcConfig::debug();
	    }
	    else
	    {
		print STDERR "DEBUG: timing:setlocation: no delay range specified for baseline $bl\n" if gcConfig::debug();
		$dRange = $$interferometer->{DelayRange};	# use site range if no bl range
		if (!defined $dRange)
		{
		    print STDERR "WARNING: no site delay range found for location $location\n";
		}
	    }

	    # if user specified min/max delay limits, use those. Otherwise use bl or site delay limits.
	    my $dMinLimit;
	    my $dMaxLimit;
	    if (defined $opt_delayMinLimit)
	    {
		$dMinLimit = $opt_delayMinLimit;	# use specified min limit
	    }
	    else
	    {
		$dMinLimit = -$dRange;			# use bl/site min limit
	    }
	    if (defined $opt_delayMaxLimit)
	    {
		$dMaxLimit = $opt_delayMaxLimit;	# use specified max limit
	    }
	    else
	    {
		$dMaxLimit = +$dRange;			# use bl/site max limit
	    }
	    print STDERR "DEBUG: timing:setlocation: using delay min $dMinLimit max $dMaxLimit\n" if gcConfig::debug();

	    $bHash{$bl} = new baseline(name => $bl,
				      latitude => $$interferometer->{Latitude},
				      longitude => $$interferometer->{Longitude},
				      bVector => \@{ $$interferometer->{$bl} },
				      delayBias => @{ $$interferometer->{$bl} }[3] + $doffval,
				      limits =>  \@{ $$interferometer->{$limitName} },
				      delayMinLimit => $dMinLimit,
				      delayMaxLimit => $dMaxLimit);
	    print "# Using $bl Baseline (ENUBias: @{ $$interferometer->{$bl} })" if ($preamble);
	    print " (with ",$doffval,"m baseline delay offset)" if ($preamble && $doff);
	    print "\n" if ($preamble);
	}
	else ## $bl not supported
	{
	    print "Error -- baseline $bl unsupported for location $location!\n";
	    my @baselines = sort @{ $$interferometer->{Baselines}};
	    print " Valid baseline designations at $location: @baselines\n";
	    exit(-1);
	}
    } ## end foreach $bl

    ## return this baseline list to the calling routine
    return ($localLongitude,$localLatitude,$interferometer,%bHash);
} ## end setLocation  ###################################################


#########################################################################
## sunTimings -- compute sunset/sunrise times
#########################################################################
sub sunTimings
{
    ## sunTimings computes approximate sunset/sunrise times from input JD.

## declarations sunTimings  #############################################
    my $jd = shift @_;  ## extract JD argument

## begin sunTimings  ####################################################
    ## Compute sun RA/Dec for sunrise/sunset calculations
    my ($sunRAdeg,$sunDecdeg) = sunPos($jd);

    # Sun transit time (otherwise known coloquially as noon)
    my $sunUT_Transit = $sunRAdeg - $LST_ZeroUT;
    $sunUT_Transit = principalValue($sunUT_Transit);

    ## Set the ZA Target to 90 + $twilightAngle for sunup/sundown
    ## calculations
    my $zaSun = 90.0 + $twilightAngle;

    ## Sunrise
    my $sunRise = zaLimit($sunDecdeg,$localLatitude,$zaSun,-1.0);
    $sunRise += $sunUT_Transit;
    $sunRise = principalValue($sunRise);
    my $LSTsunRise = $LST_ZeroUT + ($sunRise * novasConstants::SolarToSidereal);
    $sunRise = RAhms($sunRise,-1); ## try without seconds
    $sunRise =~ s/\s+/:/g;
    $LSTsunRise = principalValue($LSTsunRise);
    $LSTsunRise = RAhms($LSTsunRise,-1);  ## try without seconds
    $LSTsunRise =~ s/\s+/:/g;

    ## Sunset
    my $sunSet = zaLimit($sunDecdeg,$localLatitude,$zaSun,+1.0);
    $sunSet += $sunUT_Transit;
    $sunSet = principalValue($sunSet);
    my $LSTsunSet = $LST_ZeroUT + ($sunSet * novasConstants::SolarToSidereal);
    $sunSet = RAhms($sunSet,-1);  ## try without seconds
    $sunSet =~ s/\s+/:/g;
    $LSTsunSet = principalValue($LSTsunSet);
    $LSTsunSet = RAhms($LSTsunSet,-1);  ## try without seconds
    $LSTsunSet =~ s/\s+/:/g;

    ## print-out the informational lines

    print "# Approximate UT sunset -- sunrise times: $sunSet -- $sunRise ($twilightAngle deg twilight)\n";
    print "# Approximate LST at sunset -- sunrise: $LSTsunSet -- $LSTsunRise\n";

} ## end sunTimings  ####################################################



#########################################################################
## moonTimings -- compute moon timing information
#########################################################################
sub moonTimings
{
    ## moonTimings computes approximate moonset/moonrise times from input JD.

## declarations moonTimings  ############################################
    my $jdZeroUT = shift @_;  ## extract JD call argument

## begin moonTimings  ###################################################
    my $TOL = 0.1;
    my $moonHalfSize = 0.25;
    my $MAX_ITER = 6;

    ## Compute moon RA/Dec for moonrise/moonset calculations
    ## First call made without topocentric data
    my ($moonRAdeg,$moonDecdeg) = moonPos($jdZeroUT);

    # Moon transit time
    my $moonUT_Transit = $moonRAdeg - $LST_ZeroUT;
    $moonUT_Transit = principalValue($moonUT_Transit);

    ## Set the ZA Target to 90 + $moonHalfSize
    my $zaMoon = 90.0 + $moonHalfSize;

    ## Moonrise

    ## This is a little more complicated than sun timings because the
    ## moon moves appreciably during the night, and the topocentric
    ## correction is appreciable (order 1 degree).  So we have an
    ## interative algorithm for determining the rise and set times

    ## Initialization
    my $moonRise = &zaLimit($moonDecdeg,$localLatitude,$zaMoon,-1.0);
    my $moonRiseOld;
    my $newTransit = $moonUT_Transit;  ## This will be our working transit time
    my $iter = 0;
    do 
    {
	$moonRiseOld = $moonRise;
	my $jdRise = $jdZeroUT + (($newTransit + $moonRise) / 360.0);
	my $sid = $newTransit + $moonRise;  ## location sidereal angle at rise
	my ($tmpRA,$tmpDec) = moonPos($jdRise, $sid, $localLatitude);
	$newTransit = principalValue($tmpRA - $LST_ZeroUT);
	$moonRise = zaLimit($tmpDec,$localLatitude,$zaMoon,-1.0);  ## re-compute the rise offset
	$iter++;
    }
    while ((abs($moonRise - $moonRiseOld) > $TOL)  ## iterate until succesive corrections get small
	   && ($iter < $MAX_ITER));

    if ($iter >= $MAX_ITER)  ## Average if we didn't see appropriate convergence
    {
	$moonRise += $moonRiseOld;
	$moonRise /= 2.0;
    }

    $moonRise += $newTransit;  ## This is the answer, format it...
    $moonRise = principalValue($moonRise);
    $moonRise = RAhms($moonRise,-1);  ## Try without seconds
    $moonRise =~ s/\s+/:/g;

    ## Moonset -- by same iterative algorithm
    my $moonSet = &zaLimit($moonDecdeg,$localLatitude,$zaMoon,+1.0);
    my $moonSetOld;
    $newTransit = $moonUT_Transit;
    $iter = 0;
    do 
    {
	$moonSetOld = $moonSet;
	my $jdSet = $jdZeroUT + (($newTransit + $moonSet) / 360.0);
	my $sid = $newTransit + $moonSet;
	my ($tmpRA,$tmpDec) = moonPos($jdSet, $sid, $localLatitude);
	$newTransit = principalValue($tmpRA - $LST_ZeroUT);
	$moonSet = zaLimit($tmpDec,$localLatitude,$zaMoon,+1.0);
	$iter++;
    }
    while ((abs($moonSet - $moonSetOld) > $TOL)
	   && ($iter < $MAX_ITER));

    if ($iter >= $MAX_ITER)  ## Average if we didn't see appropriate convergence
    {
	$moonSet += $moonSetOld;
	$moonSet /= 2.0;
    }

    $moonSet += $newTransit;
    $moonSet = principalValue($moonSet);
    $moonSet = RAhms($moonSet,-1);  ## try without seconds
    $moonSet =~ s/\s+/:/g;

    ## print-out the informational lines

    print "# Approximate UT moonrise -- moonset times: $moonRise -- $moonSet\n";


    ## Compute/print moon phase and illumination
    my $sid = $newTransit;
    my $jd = $jdZeroUT + ($newTransit / 360.0);
    my $phase = moonPhase($jd, $sid, $localLatitude);
    chomp $phase;
    my ($tmpRA,$tmpDec) = moonPos($jd, $sid, $localLatitude);
    $tmpRA = &RAhms($tmpRA,-1);
    $tmpDec = &DECdms($tmpDec,-1);
    $phase .= " (moon transit $tmpRA RA  $tmpDec Dec)";

    print "#  $phase\n";
} ## end moonTimings  ###################################################


#########################################################################
## computeDelayLimits -- compute delay limits for declination
##
## Compute and return a list of intervals when the target will be within
## delay range. Each returned interval is a start/end hour-angle pair.
## HA's are expressed in degrees, relative to transit.
#########################################################################
sub computeDelayLimits
{
## declarations computeDelayLimits  #####################################
    my $name = shift @_;  ## target designation
    my $targetDec = shift @_;  ## target declination (units of degrees)
    my $utTransit = shift @_; ## UT transit time (units?)
    my $bPtr = shift @_; ## baseline object ptr
    my $dOffset = shift @_; ## delay bias offset
    my $verbose = shift @_; ## verbose output flag

    my $baseline = $$bPtr; ## baseline object
    my $blName = $baseline->{name};
    my $dMinLimit = $baseline->{delayMinLimit};
    my $dMaxLimit = $baseline->{delayMaxLimit};

## begin computeDelayLimits  ############################################
    printf "#>> $blName delay limits: min $dMinLimit m, max $dMaxLimit m\n" if ($verbose);

    my @dList; # list of delay intervals we'll return.  These are [start,end] hour angle pairs (degrees)

    ## Start by computing the delay extremum parameters
    my ($haMin,$dMin,$haMax,$dMax)
	= $baseline->delayExtrema($targetDec);
    $dMin += $dOffset; ## Add in any LDL contribution
    $dMax += $dOffset; 

    printf "#>> $blName delay extrema parameters:\n" if ($verbose > 1);
    printf "#>>  min %4.1f m at %4.1f deg hour angle\n", $dMin, $haMin if ($verbose > 1);
    printf "#>>  max %4.1f m at %4.1f deg hour angle\n", $dMax, $haMax if ($verbose > 1);

    ## Start by approximating delay limit transitions with a
    ## linear model between $dMin and $dMax
    ## Working relative to transit
    my $slope = ($dMax - $dMin) / ($haMax - $haMin);
    my $intercept = $dMin - ($slope * $haMin);

    printf "#>>  computed slope %8.3f, intercept %4.1f\n",$slope,$intercept if ($verbose > 1);

    ## Now we solve for the delay intervals in five distinct cases

    ## Case 1: object is always in delay line range
    if (($dMin >= $dMinLimit)
	&& ($dMax <= $dMaxLimit))
    {
	#print "# $name is always within delay range for $blName\n";
	push @dList, [ -$maxHA, +$maxHA ];	# delay interval, relative time in degrees
    }

    ## Case 2: object is never in delay range
    elsif (($dMax < $dMinLimit) || ($dMin > $dMaxLimit))
    {
	#print "# $name is never within delay range for $blName\n";
	# don't insert anything into the list of delay intervals
    }
    
    ## Case 3: extrema straddle delay interval
    elsif (($dMin < $dMinLimit)
	   && ($dMax > $dMaxLimit))
    {

	print "#>> Case 3: extrema straddle delay interval\n" if ($verbose > 1);

	# Find first in-range period
	## Compute the delay limit transitions
	my $haInit = ($dMinLimit - $intercept) / $slope;
	my ($haMinTarget1,$entryMin1)
	    = &delayTarget($dMinLimit,$targetDec,\$baseline,$dOffset,
			   $haInit,$haMin,$haMax,-1);
	printf "#>>  haInit = %4.1f deg, haMinTarget1 = %4.1f deg, entryMin1 = %d\n",
	    $haInit, $haMinTarget1, $entryMin1 if ($verbose > 1);
	printf "#>> Delay minimum transition found at ha %4.1f deg\n",$haMinTarget1 if ($verbose > 1);

	$haInit = ($dMaxLimit - $intercept) / $slope;
	my ($haMaxTarget1,$entryMax1)
	    = &delayTarget($dMaxLimit,$targetDec,\$baseline,$dOffset,
			   $haInit,$haMin,$haMax,1);
	printf "#>>  haInit = %4.1f deg, haMaxTarget1 = %4.1f deg, entryMax1 = %d\n",
	    $haInit, $haMaxTarget1, $entryMax1 if ($verbose > 1);
	printf "#>> Delay maximum transition found at ha %4.1f deg\n",$haMaxTarget1 if ($verbose > 1);

	# Find second in-range period
	## Compute the delay limit transitions
	$haInit = (2.0 * $haMax) - $haMinTarget1;
	my ($haMinTarget2,$entryMin2)
	    = &delayTarget($dMinLimit,$targetDec,\$baseline,$dOffset,
			   $haInit,$haMin,$haMax,-1);
	printf "#>>  haInit = %4.1f deg, haMinTarget2 = %4.1f deg, entryMin2 = %d\n",
	    $haInit, $haMinTarget2, $entryMin2 if ($verbose > 1);
	printf "#>> Delay minimum transition found at ha %4.1f deg\n",$haMinTarget2 if ($verbose > 1);

	$haInit = (2.0 * $haMax) - $haMaxTarget1;
	my ($haMaxTarget2,$entryMax2)
	    = &delayTarget($dMaxLimit,$targetDec,\$baseline,$dOffset,
			   $haInit,$haMin,$haMax,1);
	printf "#>>  haInit = %4.1f deg, haMaxTarget2 = %4.1f deg, entryMax2 = %d\n",
	    $haInit, $haMaxTarget2, $entryMax2 if ($verbose > 1);
	printf "#>> Delay maximum transition found at ha %4.1f deg\n",$haMaxTarget2 if ($verbose > 1);

	# Report first in-range period
	my ($ha1, $ha2);
	if (($entryMin1 < 0) && ($entryMax1 > 0))
	{
	    ## the delay minimum transition is the delay range entry
	    $ha1 = $haMinTarget1;
	    $ha2 = $haMaxTarget1;
	}
	else
	{
	    ## the delay maximum transition is the delay range exit
	    $ha1 = $haMaxTarget1;
	    $ha2 = $haMinTarget1;
	} ## end delay 
	# within delay range from $ha1 to $ha2
	push @dList, [$ha1,$ha2];	# record delay interval

	# Report second in-range period
	if (($entryMin2 < 0) && ($entryMax2 > 0))
	{
	    ## the delay minimum transition is the delay range entry
	    $ha1 = $haMinTarget2;
	    $ha2 = $haMaxTarget2;
	}
	else
	{
	    ## the delay maximum transition is the delay range exit
	    $ha1 = $haMaxTarget2;
	    $ha2 = $haMinTarget2;
	} ## end delay 
	# within delay range from $ha1 to $ha2
	push @dList, [$ha1,$ha2];	# record delay interval
    } ## end case 3

    ## Case 4: delay minimum is in delay range
    elsif (($dMin > $dMinLimit)
	   && ($dMin < $dMaxLimit)
	   && ($dMax > $dMaxLimit))
    {

	print "#>> Case 4: delay minimum is in delay interval\n" if ($verbose > 1);

	## We will pivot about the extrema in the interval

	## In this instance the delay min is in interval

	## Compute the delay limit transitions
	my $haInit = ($dMaxLimit - $intercept) / $slope;
	my ($haTarget1,$entry1)
	    = &delayTarget($dMaxLimit,$targetDec,\$baseline,$dOffset,
			   $haInit,$haMin,$haMax,1);
	printf "#>>  haInit = %4.1f deg, haTarget1 = %4.1f deg, entry1 = %d\n",
	    $haInit, $haTarget1, $entry1 if ($verbose > 1);
	printf "#>> Delay maximum transition found at ha %4.1f deg\n",$haTarget1 if ($verbose > 1);

	$haInit = (2.0 * $haMin) - $haTarget1;
	my ($haTarget2,$entry2)
	    = &delayTarget($dMaxLimit,$targetDec,\$baseline,$dOffset,
			   $haInit,$haMin,$haMax,1);
	printf "#>>  haInit = %4.1f deg, haTarget2 = %4.1f deg, entry2 = %d\n",
	    $haInit, $haTarget2, $entry2 if ($verbose > 1);
	printf "#>> Delay maximum transition found at ha %4.1f deg\n",$haTarget2 if ($verbose > 1);

	## Now figure out the relative "parity" of these transition hour
	## angles and report
	my ($ha1, $ha2);
	if (($entry1 < 0) && ($entry2 > 0))
	{
	    ## the delay minimum transition is the delay range entry
	    $ha1 = $haTarget1;
	    $ha2 = $haTarget2;
	}
	else
	{
	    ## the delay maximum transition is the delay range exit
	    $ha1 = $haTarget2;
	    $ha2 = $haTarget1;
	} ## end delay 
	# within delay range from $ha1 to $ha2
	push @dList, [$ha1,$ha2];	# record delay interval
    } ## end case 4


    ## Case 5: delay maximum is in delay range
    elsif (($dMax > $dMinLimit)
	   && ($dMax < $dMaxLimit)
	   && ($dMin < $dMinLimit))
    {

	print "#>> Case 5: delay maximum is in delay interval\n" if ($verbose > 1);

	## We will pivot about the extrema in the interval
	## In this instance the delay max is in interval

	## Compute the delay limit transitions
	my $haInit = ($dMinLimit - $intercept) / $slope;
	my ($haTarget1,$entry1)
	    = &delayTarget($dMinLimit,$targetDec,\$baseline,$dOffset,
			   $haInit,$haMin,$haMax,-1);
	printf "#>> Delay minimum transition found at ha %4.1f deg\n",$haTarget1 if ($verbose > 1);

	$haInit = (2.0 * $haMax) - $haTarget1;
	my ($haTarget2,$entry2)
	    = &delayTarget($dMinLimit,$targetDec,\$baseline,$dOffset,
			   $haInit,$haMin,$haMax,-1);
	printf "#>> Delay minimum transition found at ha %4.1f deg\n",$haTarget2 if ($verbose > 1);


	## Now figure out the relative "parity" of these transition hour
	## angles and report
	my ($ha1, $ha2);
	if (($entry1 < 0) && ($entry2 > 0))
	{
	    ## the delay minimum transition is the delay range entry
	    $ha1 = $haTarget1;
	    $ha2 = $haTarget2;
	}
	else
	{
	    ## the delay maximum transition is the delay range exit
	    $ha1 = $haTarget2;
	    $ha2 = $haTarget1;
	} ## end delay 
	# within delay range from $ha1 to $ha2
	push @dList, [$ha1,$ha2];	# record delay interval

    } ## end case 5

    # finally, since the HA's computed above may fall outside [-180,+180],
    # we must clip all intervals to that range
    &wrapIntervals(\@dList);

    return @dList;	# return the list of delay intervals
} ## end computeDelayLimits  ############################################

#########################################################################
## reportIntervals -- output information about computed timing intervals
#########################################################################
sub reportIntervals
{
## declarations reportIntervals  ############################################
    my $type = shift;	# string to use (e.g. "delay","pointing") in labeling these intervals
    my $name = shift;
    my $blName = shift;
    my $utTransit = shift;
    my $iRef = shift;	# ref to list of intervals
    my $verbose = shift;
## begin reportIntervals  ############################################

    my $count = scalar (@$iRef);
    # handle "never within x range"
    if (!$count)
    {
	print "# $name is never within $type range for $blName\n";
	return;
    }
    # handle "always within x range"
    if (($count == 1) &&
	($iRef->[0][0] == -$maxHA) &&
	($iRef->[0][1] ==  $maxHA))
    {
	my $ha1 = $iRef->[0][0];
	my $ha2 = $iRef->[0][1];
	my $ut1 = $utTransit + &siderealToSolar($ha1);	# UT time (expressed in degrees)
	my $ut2 = $utTransit + &siderealToSolar($ha2);	# UT time (expressed in degrees)
	my $ut1sec = $ut1 * $secondsPerDegree; # convert UT time from degrees to seconds
	my $ut2sec = $ut2 * $secondsPerDegree; # convert UT time from degrees to seconds
	if ($verbose)
	{
	    printf "#>> $name $type interval for bl $blName: [%7.3f, %7.3f], UT [%s, %s] = UT [%.3fsec, %.3fsec]\n",
		$ha1, $ha2,
		&timeString($ut1,0),
		&timeString($ut2,0),
		$ut1sec,$ut2sec;
	}
	print "# $name is always within $type range for $blName\n";
	return;
    }
    # handle other cases
    for (my $i = 0; $i < $count; $i++)
    {
	my $ha1 = $iRef->[$i][0];
	my $ha2 = $iRef->[$i][1];
	my $ut1 = $utTransit + &siderealToSolar($ha1);	# UT time (expressed in degrees)
	my $ut2 = $utTransit + &siderealToSolar($ha2);	# UT time (expressed in degrees)
	my $ut1sec = $ut1 * $secondsPerDegree; # convert UT time from degrees to seconds
	my $ut2sec = $ut2 * $secondsPerDegree; # convert UT time from degrees to seconds
	if ($verbose)
	{
	    printf "#>> $name $type interval for bl $blName: [%7.3f, %7.3f], UT [%s, %s] = UT [%.3fsec, %.3fsec]\n",
		$ha1, $ha2,
		&timeString($ut1,0),
		&timeString($ut2,0),
		$ut1sec,$ut2sec;
	}
	printf "# $name is within $type range from %s to %s UT for $blName\n",
	    &timeString($ut1),&timeString($ut2);
    }
    return;
} ## end reportIntervals  ############################################

#########################################################################
## delayTarget -- tune-up delay target root by Newton-Raphson
#########################################################################
    sub delayTarget
{
## declarations delayTarget  ############################################
    my $delayTrg = shift @_;  ## delay target
    my $targetDec = shift @_;
    my $bPtr = shift @_; ## baseline object ptr
    my $baseline = $$bPtr;
    my $dOffset = shift @_;	## delay bias offset
    my $haInit = shift @_; ## initial hour-angle guess
    my $haMin = shift @_; ## Interval for search
    my $haMax = shift @_;
    my $minMax = shift @_; ## which interval extrema (-1 min, 1 max)


    my $MAXITER = 10; ## maximum iterations
    my $delayTol = 0.001;  ## convergence tolerance
    my $haStep = 2; ## derivative step
    my $ha = $haInit; ## working hour-angle

## begin delayTarget  ###################################################
    print STDERR "DEBUG: timing: delayTarget: haInit=$haInit haMin=$haMin haMax=$haMax minMax=$minMax\n" if gcConfig::debug();

    ## Check that target interval is oriented properly
    if ($haMin > $haMax)
    {
	print "#>>  Permuting Newton-Raphson search interval\n" if ($verbose > 1);
	my $tmp = $haMax;
	$haMax = $haMin;
	$haMin = $tmp;
    } ## end fix input interval


    my $iter = 0;
    my $delay = $baseline->delay($ha,$targetDec) + $dOffset;
    while (($iter < $MAXITER)
	   && (abs($delay - $delayTrg) > $delayTol))
    {

	## Newton-raphson loop
	## Compute the delay derivative at $ha
	my $dDelaydHA = ($baseline->delay($ha + $haStep,$targetDec)
			 - $baseline->delay($ha - $haStep,$targetDec))
	    / (2.0 * $haStep);

	my $newHA = $ha
	    + ($delayTrg - $delay) / $dDelaydHA;

	## Check for/correct divergence outside the target interval
	if ($newHA < $haMin)
	{
	    $newHA = ($ha + $haMin) / 2.0;
	}
	if ($newHA > $haMax)
	{
	    $newHA = ($ha + $haMax) / 2.0;
	}

	## Recompute delay
	my $newDelay = $baseline->delay($newHA,$targetDec) + $dOffset;

	if (abs($newDelay - $delayTrg) > abs($delay - $delayTrg))
	{
	    ## divergence -- partial step
	    $ha = ($ha + $newHA) / 2.0;
	    $delay = $baseline->delay($ha,$targetDec) + $dOffset;
	}
	else ## good looking convergence
	{
	    $ha = $newHA;
	    $delay = $newDelay;
	}

	$iter++; ## increment iteration counter

    } ## end while loop
    
    print "Error -- NR non-convergence\n" if ($iter >= $MAXITER);

    printf "#>>  Good N-R convergence, %.3g m residual in %d iterations\n",
      abs($delay - $delayTrg),$iter  if ($verbose > 1);

    ## Determine extry/exit of the interval
    my $dDelaydHA = ($baseline->delay($ha + $haStep,$targetDec)
		     - $baseline->delay($ha - $haStep,$targetDec))
	/ (2.0 * $haStep);

    ## $minMax contains extrema spec, -1 = minimum, +1 = maximum
    my $entry = &sign($dDelaydHA) * &sign($minMax);
    printf "#>>  Delay target is interval minimum, dDdHA is %.3f, entry/exit eval %d\n",
	$dDelaydHA,$entry if (($verbose > 1) && ($minMax < 0));
    printf "#>>  Delay target is interval maximum, dDdHA is %.3f, entry/exit eval %d\n",
	$dDelaydHA,$entry if (($verbose > 1) && ($minMax > 0));

    return ($ha,$entry);  ## return our answer
	

} ## end delayTarget  ###################################################


#########################################################################
## wrapIntervals - wrap intervals to fall entirely within principal range
#########################################################################
# Make sure the both values of each interval always fall within [-180,+180),
# splitting intervals into multiple pieces if they cross boundary +/-180 boundary
sub wrapIntervals
{
    my $listRef = shift;

    return if ((!defined $listRef) || (!scalar(@$listRef)));	# list is empty, just return.

    # check each interval in the list, folding it into the [-180,+180] interval,
    # and splitting it if it spans the -180/+180 boundary.
    my $count = scalar(@$listRef);
    my @wraplist;
    for (my $i = 0; $i < $count; $i++)
    {
	my $changed = 0;
	printf "#>> checking for wraps on interval [%7.3f, %7.3f]\n",
	    $listRef->[$i][0], $listRef->[$i][1] if ($verbose);

	# make sure second value follows first
	while ($listRef->[$i][1] < $listRef->[$i][0])
	{
	    $listRef->[$i][1] += 360.0;
	    $changed++;
	}

	# try to shift the interval into range without going too far
	while (($listRef->[$i][0] < -180.0) &&
	       ($listRef->[$i][1] < -180.0))
	{
	    $listRef->[$i][0] += 360.0;
	    $listRef->[$i][1] += 360.0;
	    $changed++;
	}
	while (($listRef->[$i][0] > +180.0) &&
	       ($listRef->[$i][1] > +180.0))
	{
	    $listRef->[$i][0] -= 360.0;
	    $listRef->[$i][1] -= 360.0;
	    $changed++;
	}

	if (($listRef->[$i][1] - $listRef->[$i][0]) > 360.0)
	{
	    # interval spans complete range(!), just clip it.
	    # always in range
	    $listRef->[$i][0] = -180.0;
	    $listRef->[$i][1] = +180.0;
	    $changed++;
	}
	elsif (($listRef->[$i][0] <  -180.0) &&
	       ($listRef->[$i][1] >= -180.0))
	{
	    # interval straddles -180, split it there
	    printf "#>> splitting interval [%7.3f, %7.3f] into [%7.3f, %7.3f] and [%7.3f, %7.3f]\n",
		$listRef->[$i][0], $listRef->[$i][1],
		-180.0, $listRef->[$i][1],
		$listRef->[$i][0] + 360.0, 180.0 if ($verbose);
	    push @wraplist, [$listRef->[$i][0] + 360.0, +180.0];
	    $listRef->[$i][0] = -180.0;		## clip start to -180
	    $changed++;
	} elsif (($listRef->[$i][0] <  +180.0) &&
	         ($listRef->[$i][1] > +180.0))
	{
	    # interval straddles +180, split it there
	    printf "#>> splitting interval [%7.3f, %7.3f] into [%7.3f, %7.3f] and [%7.3f, %7.3f]\n",
		$listRef->[$i][0], $listRef->[$i][1],
		$listRef->[$i][0], +180.0,
		-180.0, $listRef->[$i][1] - 360.0 if ($verbose);
	    push @wraplist, [-180.0, $listRef->[$i][1] - 360.0];
	    $listRef->[$i][1] = 180.0;		## clip end to -180
	    $changed++;
	}

	if ($changed)
	{
	    printf "#>> interval adjusted to [%7.3f, %7.3f]\n",
		$listRef->[$i][0], $listRef->[$i][1] if ($verbose);
	}
    }

    # append anything from the wraplist to the original list
    foreach my $win (@wraplist)
    {

	if (@$win[0] == -180.0)
	{
	    # add it to the end
	    printf "#>> appending split interval [%7.3f, %7.3f]\n",
		@$win[0],@$win[1] if ($verbose);
	    push @$listRef, $win;
	}
	else
	{
	    # insert this at the front
	    printf "#>> prepending split interval [%7.3f, %7.3f]\n",
		@$win[0],@$win[1] if ($verbose);
	    unshift @$listRef, $win;
	}
    }
    return;
}

#########################################################################
## printDelayTable -- diagnostic routine to output a table of delay vs UT
#########################################################################
    sub printDelayTable
{
## declarations printDelayTable  ############################################
    my $name = shift @_;	## target designation
    my $targetDec = shift @_;	## target declination (units of degrees)
    my $haRange = shift @_; 	## coverage angle (deg) for table (-x to +x)
    my $utTransit = shift @_;	## UT transit time (units?)
    my $bPtr = shift @_;	## baseline object ptr
    my $dOffset = shift @_;	## delay bias offset
    my $verbose = shift @_;	## verbose output flag

    my $baseline = $$bPtr; ## baseline object
    my $blName = $baseline->{name};
## begin printDelayTable  ###################################################

    my $utSecStep = $opt_tableStep;		# time step size in seconds
    my $utStep = $utSecStep / $secondsPerDegree; # convert to degrees
    my $utTransitStr = &timeString($utTransit,0);
    print "#>>\n";
    print "#>> Calculated delay for $name, baseline $blName, transit $utTransitStr UT\n";
    print "#>>\n";
    #print "#>> ut1=$ut1, ut2=$ut2, utTransit=$utTransit, utRange=$utRange\n";
    printf "#>>    %12s %12s %12s %12s %12s %12s\n",
	"HA (deg)",
	"UT",
	"UT (sec)",
	"Tot Delay (m)",
	"Bias (m)",
	"FDL (m)";

    my $utRange = &siderealToSolar($haRange);
    my $ut1 = &POSIX::floor($utTransit - $utRange);
    my $ut2 = &POSIX::ceil($utTransit + $utRange);

    for (my $ut = $ut1; $ut <= $ut2; $ut += $utStep)
    {
	my $utSec = $ut * $secondsPerDegree; # ut (solar) time in seconds
	my $ha = &solarToSidereal($ut - $utTransit);
	my $delay = $baseline->delay($ha,$targetDec);	# compute delay at this HA
	my $fdl = $delay + $dOffset;	# fdl is delay plus biasOffset
	my $utStr = &timeString($ut,0);	# time at this position
	printf "#>>>>  %12.3f %12s %12.3f %12.3f %12.3f %12.3f\n",
	    $ha, $utStr, $utSec, $delay, $dOffset, $fdl;
    }
    print "#>>\n";
} ## end printDelayTable  ###################################################


#########################################################################
## printNDTable -- output a table of null depth vs UT
#########################################################################
sub printNDTable
{
## declarations printNDTable  ############################################
    my $name = shift @_;	## target designation
    my $targetDec = shift @_;	## target declination (units of degrees)
    my $angDiam = shift @_;	## angular diameter
    my $ndWavelength = shift @_;## wavelength
    my $haRange = shift @_; 	## coverage angle (deg) for table (-x to +x)
    my $utTransit = shift @_;	## UT transit time (units?)
    my $bPtr = shift @_;	## baseline object ptr
    my $verbose = shift @_;	## verbose output flag

    my $baseline = $$bPtr; ## baseline object
    my $blName = $baseline->{name};
## begin printNDTable  ###################################################

    my $utSecStep = $opt_NDstep;		# time step size in seconds
    my $utStep = $utSecStep / $secondsPerDegree; # convert to degrees
    print "#>>\n";
    print "#>> Calculated null depth for $name, baseline $blName, angDiam $angDiam\n";
    print "#>>\n";
    printf "#>>    %12s %12s %12s %12s\n",
	"HA (deg)",
	"UT",
	"Proj.BL (m)",
	"nullDepth";

    my $NDest_tot = 0;
    my $NDest_min = undef;
    my $NDest_max = undef;
    my $n = 0;

    my $utRange = &siderealToSolar($haRange);
    my $ut1 = &POSIX::floor($utTransit - $utRange);
    my $ut2 = &POSIX::ceil($utTransit + $utRange);

    for (my $ut = $ut1; $ut <= $ut2; $ut += $utStep)
    {
	my $utStr = &timeString($ut,0);	# time at this position
	my $ha = &solarToSidereal($ut - $utTransit);

	my ($NDest, $projBL) = 
	    &nullDepthEstimate($ha, $angDiam, $targetDec, $ndWavelength, $bPtr);

	$NDest_tot += $NDest;
	$NDest_min = $NDest if ((!defined $NDest_min) || ($NDest < $NDest_min));
	$NDest_max = $NDest if ((!defined $NDest_max) || ($NDest > $NDest_max));
	$n++;

	printf "#>>>>  %12.3f %12s %12.3f %12.3f\n",
	    $ha, $utStr, $projBL, $NDest;
    }

    if ($n>0)
    {
	my $NDest_avg = $NDest_tot / $n;
	printf "#>>   Min ND = %3.2f, Max ND = %3.2f, Avg ND = %3.2f (# values = %d)\n",
	     $NDest_min, $NDest_max, $NDest_avg, $n;
    }
    print "#>>\n";
} ## end printNDTable  ###################################################


#########################################################################
## printNDTableHA -- output a table of null depth vs hour angle
#########################################################################
sub printNDTableHA
{
## declarations printNDTableHA  ############################################
    my $name = shift @_;	## target designation
    my $targetDec = shift @_;	## target declination (units of degrees)
    my $angDiam = shift @_;	## angular diameter
    my $ndWavelength = shift @_;## wavelength
    my $haRange = shift @_; 	## coverage angle (deg) for table (-x to +x)
    my $utTransit = shift @_;	## UT transit time (units?)
    my $bPtr = shift @_;	## baseline object ptr
    my $verbose = shift @_;	## verbose output flag

    my $baseline = $$bPtr; ## baseline object
    my $blName = $baseline->{name};
## begin printNDTableHA  ###################################################

    print "#>>\n";
    print "#>> Calculated null depth for $name, baseline $blName, angDiam $angDiam\n";
    print "#>>\n";
    printf "#>>    %12s %12s %12s %12s\n",
	"HA (deg)",
	"UT",
	"Proj.BL (m)",
	"nullDepth";

    my $NDest_tot = 0;
    my $NDest_min = undef;
    my $NDest_max = undef;
    my $n = 0;

    my $haStep = 5;		# HA step size in degrees
    my $ha1 = &POSIX::floor(-$haRange);
    my $ha2 = &POSIX::ceil(+$haRange);

    for (my $ha = $ha1; $ha <= $ha2; $ha += $haStep)
    {
	my $ut = &siderealToSolar($ha) + $utTransit;
	my $utStr = &timeString($ut,0);	# time at this position

	my ($NDest, $projBL) = 
	    &nullDepthEstimate($ha, $angDiam, $targetDec, $ndWavelength, $bPtr);

	$NDest_tot += $NDest;
	$NDest_min = $NDest if ((!defined $NDest_min) || ($NDest < $NDest_min));
	$NDest_max = $NDest if ((!defined $NDest_max) || ($NDest > $NDest_max));
	$n++;

	printf "#>>>>  %12.3f %12s %12.3f %12.3f\n",
	    $ha, $utStr, $projBL, $NDest;
    }

    if ($n>0)
    {
	my $NDest_avg = $NDest_tot / $n;
	printf "#>>   Min ND = %3.2f, Max ND = %3.2f, Avg ND = %3.2f (# values = %d)\n",
	     $NDest_min, $NDest_max, $NDest_avg, $n;
    }
    print "#>>\n";
} ## end printNDTable  ###################################################
